/*****************************************************************************/
/*      EQUINOX SuperSerial Technology (SST) Driver for Linux                */
/*                                                                           */
/*      Copyright (C) 1989-2002 Equinox Systems Inc.                         */
/*****************************************************************************/
                                                                               
#ifdef XENIX
#define inl ind
#define outl outd
#endif


#define PCI_M1_MAX_BUS         256  /* Maximum number of busses */
#define BUS_NUM_SHIFT          16
#define PCI_M2_MAX_BUS         0x100
#define M1_SLOT_MAX            32  /* Max number of slots per bus */
#define M1_SLOT_ID_SHIFT       11
#define M2_SLOT_MAX            16
#define M2_SLOT_ID_SHIFT       8
#define PCI_M1_ENABLE_BIT      0x80000000
#define PCI_NULL_ID            0xFFFFFFFF
#define PCI_NULL_BYTE          0xFF
#define PCI_M1                 0x01
#define PCI_M2                 0x02
#define PCI_CONFIG_ADDRESS_REG 0xCF8
#define PCI_CONFIG_SPACE_ENABLE_REG 0xCF8
#define PCI_CONFIG_DATA_REG    0xCFC
#define PCI_BASE_ADDR          0xC000
#define PCI_FORWARD_REG        0xCFA
#define PCI_M2_ENABLE_BIT      0x80

int eqx_pci_buspresent(struct pci_csh *csh);
unsigned int eqx_bios32_service_directory(void);

struct bios32 {
   union {
      unsigned char c[16];
      struct {
         unsigned int signature;
         unsigned int entry;
         unsigned char revision;
         unsigned char length;
         unsigned char checksum;
         unsigned char res[5];
      } v;
   } q;
} ;

unsigned int eqx_bios32_service_directory(void)
{
   struct bios32 *b32;
   unsigned char check_sum;
   char *vaddr;
   int i, len;
   unsigned int b32_sig = ('_' << 24) + ('2' << 16) + ('3' << 8) + '_';

#ifdef LINUX
	vaddr = (char *)0xe0000;
#else
   vaddr = (char *)mapphys((caddr_t)0xe0000, 0x20000);
#endif /* Not LINUX */
   for(b32 = (struct bios32 *)vaddr; (unsigned long)b32 < 
		   (unsigned long)vaddr + 0x20000; b32++) {
      if (b32->q.v.signature != b32_sig)
         continue;
      len = b32->q.v.length * sizeof(struct bios32);
      for(i = 0, check_sum = 0; i < len; i++)
         check_sum += b32->q.c[i];
      if (check_sum)
         continue;
      return(b32->q.v.entry);
   }
return(0);
}

int eqx_pci_buspresent(struct pci_csh *csh)
{
	int            i, 
	               j,
	               xx,
                       bus_num, 
                       slot_id, 
	               index = 0;
	unsigned short  config_space_id;
	unsigned char  cse_save;

	if (!csh) {
		return(0);
	}

	/* Mechanism 1 */
	for(i = 0; i < PCI_M1_MAX_BUS; i++) {
		bus_num = i << BUS_NUM_SHIFT;
		for(j = 0; j < M1_SLOT_MAX; j++) {
			slot_id = j << M1_SLOT_ID_SHIFT;
			/*
			 * Write to the CONFIG_ADDRESS register
			 * so we can read the CONFIG_DATA register
			 */
	                outl( PCI_M1_ENABLE_BIT | bus_num | slot_id,PCI_CONFIG_ADDRESS_REG);
			config_space_id = inw(PCI_CONFIG_DATA_REG);
			if (config_space_id == EQUINOX_PCI_ID) {
				for(xx = 0; xx < sizeof(struct pci_csh00); 
				    xx+=sizeof(int)) {
					outl( PCI_M1_ENABLE_BIT | bus_num |
					     slot_id | xx,PCI_CONFIG_ADDRESS_REG);
					csh->cs.vl[xx/sizeof(int)] = 
						inl(PCI_CONFIG_DATA_REG);

					/* Fix for cost reduced 4/8 and 4P/LP */
					/* Boards - Revision > 8. Check Rev ID*/
					/* And if appropriate, set MSC bit.   */
					/* ID > 8  (bit 3 set) is UNIQUE      */
					/* to these boards		      */
					
					if (xx == 2 * sizeof(int))
						if ( csh->cs.vl[2] && 0x08) {
							csh->cs.vl[1] |= 0x02;
							outl( PCI_M1_ENABLE_BIT 
								| bus_num 
								| slot_id 
								| xx/2 ,
								PCI_CONFIG_ADDRESS_REG);
							outl( csh->cs.vl[1], 
							   PCI_CONFIG_DATA_REG);
						}
					/* End Fix for cost reduced PCI boards*/
				}
				csh->bus = i;
				csh->slot = j;
				csh++;
				index++;
				if (index >= maxbrd) {
					return(maxbrd);
}
			}
		}
	}

	if (index) {
		return(index);
	} 
	
	/* Mechanism 2 */
	cse_save = inb(PCI_CONFIG_SPACE_ENABLE_REG);
	outb(PCI_M2_ENABLE_BIT,PCI_CONFIG_SPACE_ENABLE_REG );
	if (inb(PCI_CONFIG_SPACE_ENABLE_REG) != PCI_M2_ENABLE_BIT) {
		outb(cse_save,PCI_CONFIG_SPACE_ENABLE_REG );
		return(0);
	}
	for(bus_num = 0; bus_num < PCI_M2_MAX_BUS; bus_num++) {
		outb(bus_num,PCI_FORWARD_REG );
		for(i = 0; i < M2_SLOT_MAX; i++) {
			slot_id = i << M2_SLOT_ID_SHIFT;
			config_space_id = inw(PCI_BASE_ADDR | slot_id | 00);
			if (config_space_id == EQUINOX_PCI_ID) {
				for(xx = 0; xx < sizeof(struct pci_csh); 
				    xx+=sizeof(short)) {
					csh->cs.vs[xx/sizeof(short)] = 
						inw(PCI_BASE_ADDR | slot_id |
						    xx);

					/* Fix for cost reduced 4/8 and 4P/LP */
					/* Boards - Revision > 8. Check Rev ID*/
					/* And if appropriate, set MSC bit.   */
					/* ID > 8  (bit 3 set) is UNIQUE      */
					/* to these boards		      */
					
					if (xx == 4 * sizeof(short))
						if ( csh->cs.vs[4] && 0x08) {
							csh->cs.vs[2] |= 0x02;
							outw( csh->cs.vs[2], 
							   PCI_BASE_ADDR 
							   | slot_id | xx/2);
							  
						}
					/* End Fix for cost reduced boards */
				}
				csh->bus = bus_num;
				csh->slot = i;
				csh++;
				index++;
				if (index >= maxbrd) {
					return(maxbrd);
				}
			}
		}
	} 
	outb(cse_save,PCI_CONFIG_SPACE_ENABLE_REG );
	return(index);
}
