/*****************************************************************************/
/*      EQUINOX SuperSerial Technology (SST) Driver for Linux                */
/*                                                                           */
/*      Copyright (C) 1989-2002 Equinox Systems Inc.                         */
/*****************************************************************************/  

/* include file - eqn.h */

#ifndef uint_t
#define uint_t unsigned int
#endif

#define    mpa_global_s       icp_global_struct		/*Global regs*/ 
#define    mpa_input_s        cin_struct              	/*Input regs */
#define    mpa_output_s       cout_struct             	/*Output regs*/
#ifdef				RAMP_PRAGMAS		/*If Compile Time pragmas needed*/
#include	"rampprag.h"
#endif

#ifndef				RAMP_ICP_H_INCLUDED	/*If ramp_icp.h not previously included*/ /*R11*/
#include	"ramp_icp.h"
#endif

#ifndef				RAMP_H_INCLUDED		/*If ramp.h not previously included*/     /*R11*/
#include	"ramp.h"
#endif

/* 
	This file defines ioctl's private to this driver.  They may
	be called from a program based on the following code fragment::

	#include <stdio.h>
	#include <fcntl.h>
	#include <eqn.h>
	{
	struct strioctl strioc;
	int fd;

	  fd = open( "/dev/eqn", O_RDWR );
	  .
	  .
	  .
	  ioctl( fd, I_STR, &strioc );
	  .
	  .
	  .
	  close( fd );
	}

	Within the strioctl structure the following fields must be set:

	  ic_cmd - must be set to one of the following commands
	  ic_dp  - must be set to the address of a local data structure
	  ic_len - must be set to the length of the local data structure
	  ic_timout - INFTIM or integral number of seconds to wait.

	If a channel number or board number is required by the command,
	it must occupy the first "int" of the local data structure.
*/

#define EQNBRDSTATLEN   (('q'<<8)|1)    /* get board status block length */
#define EQNBRDSTATUS    (('q'<<8)|2)    /* get board status block */
#define EQNBRDSTATCLR   (('q'<<8)|3)    /* clear chnl statistics for board */

#define EQNCHSTATLEN    (('q'<<8)|16)   /* get chnl status block length */
#define EQNCHSTATUS     (('q'<<8)|17)   /* get chnl status block */
#define EQNCHBUFSTAT    (('q'<<8)|18)   /* get chnl buffer counts */
#define EQNCHSANE       (('q'<<8)|19)   /* channel sanity */
#define EQNCHIXON       (('q'<<8)|20)   /* disable stty -ixon */
#define EQNCHXIXON      (('q'<<8)|21)   /* enable stty -ixon */
#define EQNCHLOOPON     (('q'<<8)|22)   /* set chnl data loopback */
#define EQNCHLOOPOFF    (('q'<<8)|23)   /* clear chnl data loopback */
#define EQNCHRESUME     (('q'<<8)|24)   /* clear xoff-received state */
#define EQNCHLPSETUP    (('q'<<8)|25)   /* configure/enable slave printer */
#define EQNCHXLPSETUP   (('q'<<8)|26)   /* disable slave printer */
#define EQNCHMONSET     (('q'<<8)|32)	/* set chnl number for data monitor */
#define EQNCHMONCLR     (('q'<<8)|33)	/* clr chnl number for data monitor */
#define EQNISTCFG	(('q'<<8)|34)	/* get Installation Status Table */
#define EQNQUIETON	(('q'<<8)|35)	/* Turn quiet mode for ring config ON*/
#define EQNQUIETOFF	(('q'<<8)|36)	/* Turn quiet mode for ring config OFF*/
/* added during SSM */
#define EQNMOVEBRD      (('q'<<8)|37)   /* Move Isa brd to a new address*/
#define EQNCHRESET      (('q'<<8)|38)   /* Hard reset modem board */
#define EQNTRACEON      (('q'<<8)|39)   /* Turn RAMP verbose messages ON */
#define EQNTRACEOFF     (('q'<<8)|40)   /* Turn RAMP verbose messages OFF */

#define EQNLPCTRLLEN    8               /* slave printer on/off string len */


#if HINVDATA
#include <sys/serial.h>
#endif
#ifdef SCO 
#include <sys/tty.h>
#endif

struct lpslave 
{
        short lp_speed;                 /* printer speed in chars/sec 
                                           if negative, no esc processing */
        unsigned short lp_blksize;      /* printer packet size */
        char    lp_son[EQNLPCTRLLEN];   /* slave on string */
        char    lp_soff[EQNLPCTRLLEN];  /* slave off string */
};  /* lpslave */

/* For Compatibility reasons this structure and the one above should
   remain the same size and in the same order. */

#define SLAVELEN	8		/* slave printer on/off string length */
struct	megaslave {
	unsigned short sp_speed;	/* printer speed in chars/sec */
	unsigned short sp_blksize;	/* printer packet size */
	char	sp_son[SLAVELEN];	/* slave on */
	char	sp_soff[SLAVELEN];	/* slave off */
};

struct cst_struct 
{
  uint_t cst_in;
  uint_t cst_in_pferrs;			/* parity/framing rrors */
  uint_t cst_in_brks;
  uint_t cst_out;
};  /* cst_struct */

struct  eqnchstatus
{
	int ec_nbr;
	unsigned int c_iflag;		/* input mode flags */
	unsigned int c_oflag;		/* output mode flags */
	unsigned int c_cflag;		/* control mode flags */
	unsigned int c_lflag;		/* local mode flags */
	struct mpchan ec_mpc;
        union  input_regs_u ec_cin;
	union  output_regs_u ec_cout;
        struct mpa_struct ec_mpa;
        struct slot_struct ec_slot;
	struct megaslave ec_lpslave;
};  /* eqnchstatus */

struct  eqnbufstatus
{
  int ebf_nbr;
  unsigned ebf_incount;
  unsigned ebf_outcount;
  uchar_t ebf_inflow_state;   /* 1=on */
  uchar_t ebf_outflow_state;  /* 1=on */
  uchar_t ebf_insigs;   /* b3=ri b2=dsr b1=cts b0=dcd */
  uchar_t ebf_outsigs;  /* b1=rts b0=dtr */
};  /* eqnbufstatus */

#define BRDSTAT_STRLEN 82
struct  eqnbrdstatus 
{
	int brd_nbr;
	char gbl_version_str[BRDSTAT_STRLEN];
	char gbl_copyright_str[BRDSTAT_STRLEN];
	int gbl_neqnbrds;
	int gbl_neqnicps;
	struct mpdev brd_dev;
	struct cst_struct brd_cst[MAXBRDCHNL];
};  /* eqnbrdstatus */

/* added during SSM , new ioctl for brd move */
struct movebrd
{
  unsigned long addr;            /* addr to move to */
};

/* SSP2/4 */

#define SSP2_ASIC 0x88  /* Any ID less than this is not SSP2/4 */
#define SSP2_ID   0xF0  /* SSP2 ID */
#define SSP4_ID   0xF8  /* SSP4 ID */
#define SSP4_MASK 0xF8

#define IFNS_SSP	1
#define IFNS_SSP4	2
#define SSP4_PAGE_SEL	((int) 0x08)
#define SSP4_ADDR_LOW	((int) 0x09)
#define SSP4_ADDR_HIGH	((int) 0x0a)
