/*****************************************************************************/
/*      EQUINOX SuperSerial Technology (SST) Driver for Linux                */
/*                                                                           */
/*      Copyright (C) 1989-2002 Equinox Systems Inc.                         */
/*****************************************************************************/   

#define MAXSLOTS		8
#define MAXISAIOPORTS	16
#define MAXLMXMPC       16              /* max RS232/422 channels per lmx */

#define MAXCHNL_ANX	2

#define NSSPS		2		/* max SSP64s per adapter */
#define NSSP4S		4		/* max SSP4s per adapter */
#ifndef MAXBRDCHNL
#define MAXBRDCHNL	128		/* max channels per board */
#endif
#ifndef MAXICPCHNL
#define MAXICPCHNL	64
#endif
#ifndef NMEGAPORT
#define NMEGAPORT	8		/* maximum possible megaport boards */
#endif
#define MAXCHNL         (MAXBRDCHNL * NMEGAPORT) /* 1024 */

#define MPRAMPTIME0      ((HZ*40)/1000)	/* 40ms. - poll for eqn_ramp_admin() */
#define RAMP_FAST_TIME   ((HZ*20)/1000)	/* 20ms. - poll when doing call backs */
#define MPA_MAX_RETRY       7

#define REG_MODEM_INDEX          0
#define SET_LOOP_BACK_INDEX      1
#define CLR_LOOP_BACK_INDEX      2
#define START_BREAK_INDEX        3
#define STOP_BREAK_INDEX         4
#define INIT_MODEM_INDEX         5
#define MODIFY_SETTINGS_INDEX    6
#define HARD_RESET_INDEX         7
#define MAX_RAMP_INDEX           8


/* PCI defines */

#define EQUINOX_PCI_ID                 0x113f /* Vendor ID */
#define PCI_CLASS_COMM_CONTROLLER      0x07  /* Comm. Controller Class */
#define PCI_SUBCLASS_SERIAL_CONTROLLER 0x00  /* Serial Controller Sub-Class */
#define PCI_BASE_ADDR_MASK             0xFFFFFFF0 /* Mask off bits 0-3 */

/* Device IDs */
#define SST128_PCI_DEV_ID 0x10
#define SST64_PCI_DEV_ID  0x08
#define SST8F_PCI_DEV_ID  0x90
#define SST8P_PCI_DEV_ID  0x94
#define SST4_PCI_DEV_ID   0x88
#define SST2_PCI_DEV_ID   0xA8

/* PCI Register Offsets */

#define PCI_DEVICE_ID_REG    0x02  /* Device ID Register offset */
#define PCI_COMMAND_REG      0x04  /* Command Register offset */
#define PCI_REV_ID_REG       0x08  /* Revision Register offset */
#define PCI_CC_PROGRAM_REG   0x09  /* Program Interface Class Code Reg offset */
#define PCI_CC_SUBCLASS_REG  0x0A  /* Sub-Class Class Code Register offset */
#define PCI_CC_CLASS_REG     0x0B  /* Class Class Code Register offset */
#define PCI_HEADER_TYPE_REG  0x0E  /* Header Type Register offset */
#define PCI_BIST_REG         0x0F  /* Built In Self Test Register offset */
#define PCI_BASE_ADDRESS_REG 0x10  /* Base Register offset */

/* Memory Space Control bit (bit 1 @ offset 0x04) in I/O Configuration 
   Space's Command Reg. */
#define PCI_MSC 0x02 

/*
 * per-megaport board configuration and status
 */

struct lmx_struct
{
	int	lmx_active;
	int	lmx_numbr;
	int	lmx_id;
	int	lmx_speed;
	int	lmx_chan;
	int	lmx_type;		/* b0: 0 = local, 1 = bridge */
	int	lmx_rmt_active;
	int	lmx_rmt_id;		/* mux id */
	int	lmx_rmt_type;		/* mux type */
	int	lmx_wait;
	int	lmx_good_count;
	struct mpchan *lmx_mpc;		/* first channel mpc */
};

struct icp_struct
{
	uchar_t	icp_active;
	uchar_t	icp_rng_last;
	uchar_t	icp_rng_svcreq;
	uchar_t	icp_rng_state;
	int     icp_rng_wait;
	int	icp_rng_fail_count;
	int	icp_rng_good_count;
	paddr_t	icp_dram_start;
	paddr_t	icp_tags_start;
	paddr_t	icp_cmds_start;
	int	icp_dram_size;
	int	icp_tags_size;
	int	icp_cmd_size;
	mpaddr_t icp_regs_start;
	int	icp_minor_start;	/* start minor number for this icp */
	struct lmx_struct lmx[4];
	uchar_t	icp_poll_defer;         /* megapoll processing deferred */
	struct ssp_struct *sspcb;		/* Ramp control block */
	struct slot_struct *slot_ctrl_blk;	/* Ramp */
};

typedef struct icp_struct *icpaddr_t;

struct hwq_struct
{
  int hwq_size;
  uint_t hwq_bits;
  uint_t hwq_hiwat;
  uint_t hwq_lowat;
  uchar_t  hwq_rxwrap;
  uchar_t  hwq_txwrap;
  uchar_t  hwq_cmdsize;
  uint_t  hwq_minchar;
};  /* hwq_struct */

struct mpdev {			/* Adapter struct */
	uchar_t	mpd_alive;		/* board present */
	uchar_t mpd_mem_width;		/* memory access width */
	ushort	mpd_nchan;		/* number of channels used */
	ushort  mpd_id;			/* board type */
	uchar_t mpd_slot;		/* slot where the adapter was found */ 
	uchar_t	mpd_bus;		/* bus type machine */
	uchar_t mpd_nicps;		/* number of icps on board */
	uchar_t mpd_sspchan;		/* number of channels on ssp */
	uchar_t mpd_ccode;		/* PatH - Multimodem country code reg */
	uchar_t mpd_nwin;		/* size of win */
	ushort  mpd_io;			/* io port number */
	ushort	mpd_pgctrl;		/* io port for page control */
	struct	mpchan *mpd_mpc;	/* mpchan base address */
	paddr_t mpd_regs;		/* virtual address of icp regs. */
	paddr_t mpd_pmem;		/* physical memory address */
	int	mpd_addrsz;		/* physical memory size */
	paddr_t mpd_mem;		/* virtual address of on-board memory */
	paddr_t mpd_tags;		/* virtual addrs of tag memory dram */
	int	mpd_memsz;		/* buffer memory size */
	int	mpd_tagsz;
	int	mpd_minor;		/* start minor number for this board */
	struct hwq_struct *mpd_hwq;	/* queue param ptr */
	struct icp_struct icp[NSSP4S];
};

/*
 * software copy of icp circular queue
 */

typedef	struct {
	char *q_addr;			/* virtual base address */
	uint_t	q_ptr;			/* tail for rx; head for tx */
	ushort	q_begin;		/* start index */
	ushort	q_end;			/* end index */
	ushort	q_size;			/* queue size */
} queue_t;

/*
 * per-channel status information
 */

struct	mpchan {
	int	mpc_flags;
#define MPC_WSEC	0x1		/* wakeup second session */
#define MPC_MODEM	0x2		/* modem control lines in use */
#define MPC_DIALOUT	0x4		/* line used as dial-out */
#define MPC_SOFTCAR	0x8		/* disable modem control */
#define MPC_SLAVE	0x10		/* slave printer configured */
#define MPC_OPEN	0x20		/* channel open */
#define MPC_SPOPEN	0x40		/* slave printer opened */
#define MPC_BUSY	0x80		/* output active */
#define MPC_BLOCK	0x100		/* input flow controlled */
#define MPC_RXFLUSH	0x200		/* input queue flushed */
#define MPC_BREAKIN	0x400		/* received break start */
#define MPC_HIGHBAUD	0x800		/* use high speed table */
#define MPC_SCRN	0x1000		/* multi-screen */
#define MPC_DIALIN	0x2000		/* disconnect on carrier drop */
#define MPC_CTS		0x4000		/* control in used as CTSFLOW */
#define MPC_WBREAK	0x8000		/* wait for end spaceing line */
#define MPC_CHAN_LD	0x10000		/* LD port */
#define MPC_SANE_LD	0x20000		/* open with space, need recovery */
#define MPC_SCANCODE	0x40000		/* scan code mode */
#define MPC_LOOPBACK	0x80000		/* Port in Internal Loopback */
#define MPC_DSCOPER	0x100000	/* Port in Datascope Mode for reads */
#define MPC_DSCOPEW	0x200000	/* Port in Datascope Mode for writes*/
#define MPC_PFREEZE     0x400000        /* freeze in Poll */
#define MPC_DEFER       0x800000        /* Poll processing deferred */
	uchar_t	mpc_rxmask;		/* receive character mask */
	uchar_t	mpc_chan;		/* channel number on this device */
	uchar_t	mpc_wmode;		/* write mode (see below) */
	uchar_t	mpc_rmode;		/* read mode (see below) */
#define LINEDISCIPLINE	0		/* normal line discipline */
#define LD0SUBSET	1		/* use subset of ld 0 */
#define BLOCKCOPY	2		/* block copy */
	ushort	mpc_last_rcv_cnt;	/* previous icp_rxnext value */
	ushort	mpc_ptimer;		/* printer timer */
	ushort	mpc_btimer;		/* break on timer */
	uint_t	mpc_param;		/* overwrite parameters */
#define	IXONSET	1			/* force ixon */
#define	IXANYIG	2			/* ignore ixany setup */
#define	IOCTLFB	4			/* TCSETA -> TCSETAW */
#define	IOCTLLP	8			/* sync lp spoller */
#define	IOCTLLI	0x10			/* force input linediscipline */
#define	IOCTLLO	0x20			/* force output linediscipline */
#define IOCTCTS 0x40
#define IOCTRTS 0x80
#define IOCTLPC 0x100
#define IOCTLMC 0x200
#define IOCTLLB 0x400
#define IOCTXON 0x800
#define IOCTLCK 0x1000			/* force lock port settings */
#define IOMSCRN 0x8000
#define IOCTLHS	0x10000
	icpiaddr_t mpc_icpi;		/* icp input channel reg's ptr */
#define mpc_cin mpc_icpi
	icpoaddr_t mpc_icpo;		/* icp output channel reg's ptr */
#define mpc_cout mpc_icpo
	icpbaddr_t mpc_icpb;		/* pointer to freezed bank */
	icpaddr_t mpc_icp;		/* pointer to icp_struct */	
	ushort	mpc_cin_events;		/* pending input events */
	ushort	mpc_cout_events;	/* pending output events */
	ushort	mpc_cin_ena;		/* input events enabled */
	ushort	mpc_cout_ena;		/* output events enabled */
	ushort	mpc_count;		/* pending output */
	ushort	mpc_rcv_cnt;		/* last rcv count */
	ushort	mpc_tx_last_invent;	/* chars invented by icp in opost */
	ushort	mpc_cin_stat_mask;
	ushort	mpc_lastrxnext;
	ushort	mpc_rxbase;		/* base tail index */
	ushort	mpc_txbase;		/* base  index */
	queue_t	mpc_rxq;		/* receive queue */
	queue_t	mpc_txq;		/* transmit queue */
	struct	mpdev *mpc_mpd;		/* pointer to megaport board info */
	struct	tty_struct *mpc_tty;		/* pointer to tty structure */
	struct	tty_struct *mpc_sptty;		/* slave printer tty */
	struct	megaslave *mpc_slave;	/* slave printer control strings */
	uint_t mpc_tags;
	uint_t mpc_input;		/* counter total chars rcv */
	uint_t mpc_output;		/* counter total chars send */
	struct	buf *mpc_bp;		/* buffer for allocation heap */
	char *mpc_heap;			/* node area */
	naddr_t mpc_root;		/* pointer to top-level node */
	naddr_t mpc_ptr;		/* pointer to current node */
	uchar_t	mpc_rxbuf[KEYLEN];	/* holding area for input mapping */
	unsigned char *mpc_rxcp;	/* pointer into mpc_rxbuf */
#ifdef _INKERNEL
	v86_t	*mpc_v86;		/* pointer to VP/ix info */
#else
	void    *mpc_v86;
#endif
	uchar_t	mpc_inttype;		/* virtual device type */
	uchar_t	mpc_lcr;		/* i8250 line control register */
	uchar_t	mpc_start;		/* start output */
	uchar_t	mpc_stop;		/* stop output */
	struct  tty_struct *mpc_ptty;		/* pointer to port tty */
	struct  tty_struct *mpc_sctty;		/* pointer to multi screen tty */
	uchar_t mpc_brdno;		/* board number */
	uchar_t mpc_icpno;		/* icp number on the adapter */
	uchar_t	mpc_lmxno;		/* lmx number from the icp */
	uchar_t mpc_tail_end;
	ushort  mpc_last_esc;		/* last addr esc was detected */
	ushort  mpc_esc_timo;		/* actual esc time waited */
	uchar_t mpc_scrn_no;		/* active screen on this port */
	uchar_t mpc_scrn_count;		/* number screens open */
	struct  megascrn mpc_scrn;	/* multi screen control data */
	struct proc *mpc_selrd;		/* select read pointer */
	struct proc *mpc_selwr;		/* select write pointer */
	uchar_t mpc_selflags;		/* select flags */
	uchar_t mpc_rxpg;		/* select page rx buffer */
	uchar_t mpc_txpg;		/* select page tx buffer */
	uchar_t mpc_tgpg;		/* select page tags buffer */
	int	flags;			/* Linux specific serial flags */

#if	(LINUX_VERSION_CODE < 132096)
	/* kernels before 2.4 */
	struct	wait_queue *open_wait;
	struct	wait_queue *close_wait;
	struct	wait_queue *raw_wait;
#else
	/* 2.4 kernels and after */
	wait_queue_head_t open_wait;
	wait_queue_head_t close_wait;
	wait_queue_head_t raw_wait;
#endif

	int	refcount;
	struct termios *normaltermios;
	struct termios *callouttermios;
	pid_t	session;
	pid_t	pgrp;
	int 	openwaitcnt;
	int	close_delay;
	int	closing_wait;
	int	custom_divisor;
	int	baud_base;
	struct tq_struct	tqhangup;
	int mpc_block;
	int carr_state;
	unsigned char 		*xmit_buf;
	int			xmit_head;
	int			xmit_tail;
	int			xmit_cnt;
	unsigned int mpc_vtime;		/* extra VTIME for speed multiplier */
	unsigned int mpc_mpa_stat;	/* modem pool status word */
	/* MPA status falue */
#define MPA_SLOT_EMPTY			0x00000001
#define MPA_LMX_CHANGE			0x00000002
#define MPA_INITG_UART    		0x00000004
#define MPA_INIT_ERROR    		0x00000008
#define MPA_LOOP_ERROR    		0x00000010
#define MPA_BREAK_ERROR    		0x00000020
#define MPA_UART_CFG_DONE 		0x00000040
#define MPA_INITG_SET_LOOP_BACK		0x00000080
#define MPA_SET_LOOP_BACK_DONE		0x00000100
#define MPA_INITG_CLR_LOOP_BACK		0x00000200
#define MPA_SET_LOOP_BACK_ERROR		0x00000400
#define MPA_CLR_LOOP_BACK_ERROR		0x00000800
#define MPA_CALL_BACK_ERROR		0x00001000

#define MPA_INITG_START_BREAK		0x00002000
#define MPA_START_BREAK_DONE		0x00004000
#define MPA_INITG_STOP_BREAK		0x00008000
#define MPA_INITG_HARD_RESET		0x00010000
#define MPA_HARD_RESET_ERROR		0x00020000
#define MPA_INITG_INIT_MODEM		0x00040000
#define MPA_INITG_MODIFY_SETTINGS	0x00080000
#define MPA_INIT_MODEM_DONE		0x00100000
#define MPA_INIT_MODEM_ERROR		0x00200000

#if	(LINUX_VERSION_CODE < 132096)
	/* kernels before 2.4 */
	struct	wait_queue *mpc_mpa_slb; /* set loopback wait queue */
	struct	wait_queue *mpc_mpa_clb; /* clear loopback wait queue */
	struct	wait_queue *mpc_mpa_rst; /* reset modem wait queue */
	struct	wait_queue *mpc_mpa_call_back; /* writer's sleep queue */
#else
	/* 2.4 kernels and after */
	wait_queue_head_t mpc_mpa_slb; /* set loopback wait queue */
	wait_queue_head_t mpc_mpa_clb; /* clear loopback wait queue */
	wait_queue_head_t mpc_mpa_rst; /* reset modem wait queue */
	wait_queue_head_t mpc_mpa_call_back; /* writer's sleep queue */
#endif

	uchar_t mpc_mpa_init_retry;	/* retry count */
	uchar_t mpc_mpa_reset_retry;	/* reset count */
	uchar_t mpc_mpa_delay[MAX_RAMP_INDEX];
	ushort_t	mpc_mpa_cout_ctrl_sigs;	/* output control signals */
};
