/*****************************************************************************/
/*      EQUINOX SuperSerial Technology (SST) Driver for Linux                */
/*                                                                           */
/*      Copyright (C) 1989-2002 Equinox Systems Inc.                         */
/*****************************************************************************/   

/* include file - icp.h */

#define uchar_t  unsigned char

#define ushort_t unsigned short
#define uint_t   unsigned int

                /* cpu-owned registers - no indent  */
                /* icp-owned registers - one indent */
                /* filler bytes - full indent       */

/* icp global register structure - physically overlaps icp_out_struct */

struct icp_gbl_struct
{
  uchar_t               gicp_filler_0[24];
  uchar_t  gicp_bus_ctrl;               /* bus control */
  uchar_t       gicp_rev;               /* icp revision number */
  uchar_t               gicp_filler_1[2];
  uchar_t  gicp_initiate;               /* lmx control & icp enable */
  uchar_t  gicp_scan_spd;               /* lmx scan speeds */
  uchar_t  gicp_tmr_size;               /* interval timer scale preset */
  uchar_t  gicp_tmr_count;              /* interval timer count preset */
  uchar_t               gicp_filler_2[24];
  uchar_t  gicp_watchdog;               /* watchdog timer */
  uchar_t               gicp_filler_3[3];
  uchar_t       gicp_attn;              /* global status - poll for work */
  uchar_t       gicp_chnl;              /* current channel number */
  ushort_t      gicp_frame_ctr;         /* frame counter */
  uchar_t               gicp_filler_4[24];
  uchar_t       gicp_rcv_attn[8];       /* receive attention bits */
  uchar_t               gicp_filler_5[24];
  uchar_t       gicp_xmit_attn[8];      /* transmit attention bits */
};  /* icp_gbl_struct */

/* icp banked input register structure */

struct cin_bnk_struct
{
  ushort_t      bank_dma_ptr;           /* offset ptr to next input dma */
  uchar_t       bank_fifo_lvl;          /* number chars held in fifo */
  uchar_t       bank_tags_l;            /* input tags, low byte */
  ushort_t      bank_sigs;              /* channel input signals */
  uchar_t               bank_filler_0;
  uchar_t       bank_tags_h;            /* input tags, high byte */
  uchar_t	bank_fifo[8];		/* input 8 char fifo */
  ushort_t      bank_num_chars;         /* number of characters received */
  ushort_t      bank_events;            /* input events detected */
};  /* cin_bnk_struct */

/************************************************************
 *                                                          *
 *BANKED INPUT REGISTER DEFINITIONS                         *
 *                                                          *
 ************************************************************/

struct  ssp2_bank_s {
	unsigned short int	nxt_dma ; /*Next Input DMA offset*/
	unsigned char		fifo_lvl; /*COMMON- # of Char held in FIFO*/
	unsigned char		tags_l ;  /*COMMON - Input Tags, low byte */
	unsigned short int	signals ; /*Input signals */
	unsigned char		undef_6 ;
	unsigned char		tags_h ;  /*COMMON - Input Tags, high byte*/
	unsigned char		fifo_0 ;  /*COMMON - "Oldest" character not DMA'd */
	unsigned char		fifo_1 ;  /*COMMON */
	unsigned char		fifo_2 ;  /*COMMON */
	unsigned char		fifo_3 ;  /*COMMON */
	unsigned char		fifo_4 ;  /*COMMON */
	unsigned char		fifo_5 ;  /*COMMON */
	unsigned char		fifo_6 ;  /*COMMON */
	unsigned char		fifo_7 ;  /*COMMON */
	unsigned short int	num_chars ; /*Number of characters received*/
  	unsigned short int	events ;  /*COMMON - Input Events	*/

} ;	

/************************************************************
 *                                                          *
 * SSP & SSP2 COMMON BANKED REGISTERS                       *
 *                                                          *
 ***********************************************************/

struct  common_bank_s  {

	unsigned short int	unused_00 ; /* NOT COMMON */		
	unsigned char		fifo_lvl ;		
	unsigned char		tags_l ;		
	unsigned short int	unused_04 ; /* NOT COMMON */
	unsigned char		undef_6 ;		
	unsigned char		tags_h ;		
	unsigned char		fifo_0 ;		
	unsigned char		fifo_1 ;		
	unsigned char		fifo_2 ;	
	unsigned char		fifo_3 ;	
	unsigned char		fifo_4 ;
	unsigned char		fifo_5 ;
	unsigned char		fifo_6 ;
	unsigned char		fifo_7 ;
	unsigned short int	num_chars ;	
	unsigned short int	events ;	

};

/************************************************************
 *                                                          *
 * SSP2 INPUT REGISTER DEFINITION                           *
 *                                                          *
 ************************************************************/

struct  ssp2_input_s  {

	unsigned short int	char_ctrl ;/*COMMON - Input Character Control & Attributes*/
	unsigned char		locks ;	/*COMMON - Locks*/
	unsigned char		undef_03 ;
	unsigned short int	baud ;	/*COMMON - Input baud rate*/
	unsigned char		q_cntrl ; /*Input Queue Control */
	unsigned char		lit_nxt ; /*COMMON - Literal Next Character */
	unsigned char		xon_1 ; /*COMMON - XON-1 char */
	unsigned char		xon_2 ;	/*COMMON - XON-2 char (used if 2 char XON)*/
	unsigned char		xoff_1 ; /*COMMON - XOFF-1 char */
	unsigned char		xoff_2 ; /*COMMON - XOFF-2 char (used if 2 char XOFF)*/
	unsigned char		unused_0C [4] ;
	unsigned char		tmr_preset ; /*COMMON - Timer Preset*/
	unsigned char		tmr_count ; /*COMMON - Timer Counter*/
	unsigned short int	attn_ena ; /*COMMON - Attention Enable*/
	unsigned short int	over_load ; /*COMMON - Buffer Overload Level*/
	unsigned char		susp_off ; /*Stop/Suspend Output if "off" condition met*/
	unsigned char		susp_on ; /*Stop/Suspend Output if "on" condition met*/
	unsigned short int	min_char ; /*COMMON - Minimum character event threshold*/
	unsigned char		unused_1A [6] ;
	unsigned char		lookup_tbl [32] ; /*COMMON - Character Lookup Table*/
	unsigned short int	tail_a ;	/*COMMON - Tail Pointer A*/
	unsigned char		unused_42 [2] ;
	struct  ssp2_bank_s	bank_a ;	/*COMMON - Bank A */
	unsigned short int	intern_tdm_a ;	/*TDM*/
	unsigned char		unused_5A [2] ;
	unsigned short int	intern_baud_ctr ; /*Internal Baud Rate Counter	*/
	unsigned char		intern_char_asm ; /*Internal Character Assembly	*/
	unsigned char		intern_flags ;	  /*COMMON - Internal Flags */
	unsigned short int	tail_b ;	/*COMMON - Tail Pointer B*/
	unsigned char		unused_62 [2] ;
	struct  ssp2_bank_s	bank_b ;	/*COMMON - Bank B */
	unsigned short int	intern_tdm_b ;	/*TDM */
	unsigned char		unused_7A [2] ;
	unsigned char		intern_tmr_p ;	/*COMMON - Timer Scale */
	unsigned char		intern_tmr_c ;	/*COMMON - Timer Counter */
 	unsigned char		intern_prt_char ; /*COMMON - Partially Framed Character	*/
	unsigned char		in_band ;	/*COMMON - Inband Flow Control	*/
} ;

/************************************************************
 *                                                          *
 * SSP & SSP2 COMMON INPUT REGISTERS                        *
 *                                                          *
 ************************************************************/

struct  common_input_s  {				
	unsigned short int	char_ctrl ; /*COMMON - Input Character Control*/
	unsigned char		locks ;	    /*COMMON - Locks*/
	unsigned char		   undef_03 ;	
	unsigned short int	baud ;	       /*COMMON - Baud Rate*/
	unsigned char		   undef_06 ;
	unsigned char		lit_nxt ;	/*COMMON - Literal Next*/
	unsigned char		xon_1 ;		/*COMMON - XON-1 */
	unsigned char		xon_2 ;		/*COMMON - XON-2 */
	unsigned char		xoff_1 ;	/*COMMON - XOFF-1 */
	unsigned char		xoff_2 ;	/*COMMON - XOFF-2 */
	unsigned char		   unused_0C [4] ;	
	unsigned char		tmr_preset ;	/*COMMON - Timer Preset */
	unsigned char		tmr_count ;	/*COMMON - Timer Count */
	unsigned short int	attn_ena ;	/*COMMON - Attention Enable */
	unsigned short int	over_load ;	/*COMMON - Buffer Overload */
	unsigned char		   undef_16 ;	
	unsigned char		   undef_17 ;
	unsigned short int	min_char ;	/*COMMON - Minimum Character Event Threshold */
	unsigned char		   unused_1A [6] ;	
	unsigned char		lookup_tbl [32] ; /*COMMON - Lookup Table */
	unsigned short int	tail_a ;	/*COMMON - Tail A */
	unsigned char		   unused_42 [2] ;	
	struct  ssp2_bank_s	bank_a ;	/*COMMON - Bank A */
	unsigned char		   undef_58 [8] ;
	unsigned short int	tail_b ;	    /*Common - Tail A */
	unsigned char		   unused_62 [2] ;	
	struct  ssp2_bank_s	bank_b ;	    /*COMMON - Bank B */
	unsigned char		   undef_78 [2] ;	
	unsigned char		   unused_7A [2] ;
	unsigned char		intern_tmr_p ;	     /*COMMON - Internal Timer Scale */
	unsigned char		intern_tmr_c ;	     /*COMMON - Internal Timer Count */
	unsigned char		intern_prt_char ;    /*COMMON - Partial Character */
	unsigned char		in_band ;	     /*COMMON - Inband Flow Control */
};


/************************************************************
 *                                                          *
 * OUTPUT QUEUE REGISTER DEFINITIONS                        *
 *                                                          *
 ************************************************************/

struct  ssp2_queue_s {
	unsigned char		undef_00 [4] ;
	unsigned short int	data_ptr ;      /*Queue Data Pointer*/
	unsigned char		undef_06 ;
	unsigned char		buff_type_sz ;	 /*Buffer type & Size */
	unsigned short int	data_count ;	 /*Output Data Count  */
	unsigned char		low_threshold ;	 /*Low Water Threshold	*/
	unsigned char		send_data_state ; /*Send Data State   */
} ;


 
/************************************************************
 *                                                          *
 *OUTPUT SESSION REGISTER DEFINITIONS                       *
 *                                                          *
 ************************************************************/

struct  ssp2_session_s {
	unsigned char		col ;	/*COMMON - Session column counter  */
	unsigned char		expnd_ena ;  /*COMMON - OPOST Expansion Enable	*/
	unsigned short int	expnd_ctr ;   /*Expansion Character Counter  */

};


/************************************************************
 *                                                          *
 * SSP2 OUTPUT REGISTER DEFINITIONS                         *
 *                                                          *
 ************************************************************/

struct  ssp2_output_s  {

	struct  ssp2_queue_s	q0 ; /*COMMON - Queue 0 */
	unsigned short int	intern_tdm_a ;	/*Output TDM 0 */
	unsigned short int	intern_tdm_b ;	/*Output TDM 1	*/
	unsigned char		fifo_0 ;	/*COMMON */
	unsigned char		fifo_1 ;	/*COMMON */
	unsigned char		fifo_2 ;	/*COMMON */
	unsigned char		fifo_3 ;	/*COMMON */
	struct  ssp2_session_s	ses_a ;		/*COMMON - Session A */
	unsigned short int	frame_ctr ;	/*Frame Counter	*/
	unsigned char		undef_1A [6] ;
	unsigned char		intern_flow ;/*COMMON - Internal Flow Cintrol State	*/
	unsigned char		intern_opost ;  /*Internal OPOST State	*/
	unsigned char		intern_fifo_ptr ; /*COMMON - Internal FIFO Pointers */
	unsigned char		intern_tog ;	/*Internal Saved Toggles */
	unsigned short int	intern_tmr ;	/*COMMON - Output Timer Counter	*/
	unsigned char		undef_26 [2] ;
	unsigned short int	intern_tmr_g ;	/*Global Timer Counter	*/
	unsigned short int	events_a ;	/*Event register A	*/
	unsigned short int	intern_status ;	/*Status		*/
	unsigned short int	events_b ;	/*Events Register B 	*/
	unsigned short int	nxt_dma ;	/*Next DMA Address	*/
	unsigned char		unused_0x31 ;
	unsigned char		intern_dma_sv ;	/*Saved DMA Status	*/
	struct  ssp2_session_s	ses_b ;		/*COMMON - Session B	*/
	unsigned char		undef_38 [8] ;
	struct  ssp2_queue_s	q1 ;		/*COMMON - Queue 1 */
	unsigned short int	intern_baud_ctr;/*COMMON-Internal Baud Rate Counter	*/
	unsigned short int	intern_state;/*COMMON-Internal Output Character State */
	unsigned char		fifo_4 ;	/*COMMON */
	unsigned char		fifo_5 ;	/*COMMON */
	unsigned char		fifo_6 ;	/*COMMON */
	unsigned char		fifo_7 ;	/*COMMON */
	struct  ssp2_session_s	ses_c ;		/*COMMON - Session C */
	unsigned char		undef_58 [8] ;
	unsigned short int	baud ;		/*COMMON - Baud Rate */
	unsigned char		char_fmt ;	/*COMMON - Character Format */
	unsigned char		flow_config ;	/*XON/XOFF Control */
	unsigned short int	cntrl_sigs ;	/*Control Signals */
	unsigned char		loop_backs ;	/*Loop Backs	*/
	unsigned char		unused_0x67 ;
	unsigned char		glb_tmr_preset ; /*Global Timer scale preset	*/
	unsigned char		glb_tmr_count ;	/*Global Timer Down Counter preset */
	unsigned short int	attn_ena ;	/*Attention Enable 	*/
	unsigned char		xoff_1 ;	/*COMMON - XOFF-1 Character */
	unsigned char		xoff_2 ;	/*COMMON - XOFF-1 Character */
	unsigned char		xon_1 ;		/*COMMON - XON-1 Character */
	unsigned char		xon_2 ;		/*COMMON - XON-2 Character */
	unsigned char		tmr_preset ;	/*COMMON - Timer Scale Preset	*/
	unsigned char		tmr_count ;	/*COMMON - Timer Down Counter	*/
	unsigned char		request ;	/*Request	*/
	unsigned char		locks ;		/*Locks		*/
	struct  ssp2_session_s	ses_d ;		/*COMMON - Session D */
	unsigned char		undef_78 [8] ;

} ;	

/************************************************************
 *                                                          *
 * SSP & SSP2 COMMON OUTPUT REGISTERS                       *
 *                                                          *
 ************************************************************/

struct  common_output_s  {
	unsigned char		usused_00 [sizeof(struct  ssp2_queue_s)] ;
	unsigned char		usused_0C [4] ;		
	unsigned char		fifo_0 ;	/*COMMON - FIFO 0 Character */
	unsigned char		fifo_1 ;	/*COMMON - FIFO 1 Character */
	unsigned char		fifo_2 ;	/*COMMON - FIFO 2 Character */
	unsigned char		fifo_3 ;	/*COMMON - FIFO 3 Characte */
	struct  ssp2_session_s	ses_a ;		/*COMMON - Session A	*/
	unsigned char		unused_18 [2] ;
	unsigned char		undef_1A  [6] ;
	unsigned char		intern_flow ;	/*COMMON - Internal Flow Cintrol State	*/
	unsigned char		unused_21 ;	
	unsigned char		intern_fifo_ptr ;/*COMMON - Internal FIFO Pointers*/
	unsigned char		unused_23 ;	
	unsigned short int	intern_tmr ;	/*COMMON - Output Timer Counter	*/
	unsigned char		undef_26 [2] ;
	unsigned char		unused_28 [12] ;
	struct  ssp2_session_s	ses_b ;		/*COMMON - Session B	*/
	unsigned char		undef_38 [8] ;		
	struct  ssp2_queue_s	q1 ;		/*COMMON - Queue 1 */
	unsigned short int	intern_baud_ctr ;/*COMMON - Internal Baud Rate Counter */
	unsigned short int	intern_state ;	/*COMMON - Internal Output Character State */
	unsigned char		fifo_4 ;	/*COMMON - FIFO 4 Character */
	unsigned char		fifo_5 ;	/*COMMON - FIFO 5 Character */
	unsigned char		fifo_6 ;	/*COMMON - FIFO 6 Character */
	unsigned char		fifo_7 ;	/*COMMON - FIFO 7 Character */
	struct  ssp2_session_s	ses_c ;		/*COMMON - Session C	*/
	unsigned char		undef_58 [8] ;	
	unsigned short int	baud ;		/*COMMON - Baud Rate */
	unsigned char		char_fmt ;	/*COMMON - Character Format*/
	unsigned char		unused_63 [3] ;	
	unsigned char		loop_backs ;	/*COMMON - Loop Back bits (0 & 1)*/
	unsigned char		unused_67 [5] ;	
	unsigned char		xoff_1 ;	/*COMMON - XOFF-1 Character */
	unsigned char		xoff_2 ;	/*COMMON - XOFF-1 Character */
	unsigned char		xon_1 ;		/*COMMON - XON-1 Character */
	unsigned char		xon_2 ;		/*COMMON - XON-2 Character */
	unsigned char		tmr_preset ;	/*COMMON - Timer Scale Preset */
	unsigned char		tmr_count ;	/*COMMON - Timer Down Counter*/
	unsigned char		unused_72 [2] ;	
	struct  ssp2_session_s	ses_d ;		/*COMMON - Session D	*/
	unsigned char		undef_78 [8] ;

} ;

/************************************************************
 *                                                          *
 * SSP2 GLOBAL REGISTER DEFINITIONS                         *
 *                                                          *
 ***********************************************************/

struct  ssp2_global_s  {

	unsigned char	bus_cntrl ;	/*Global Control register*/
	unsigned char	rev ;		/*Revision Level Number	*/
	unsigned char	on_line ;	/*On-line	*/
	unsigned char	unused_3 ;
	unsigned char	chan_ctr ;	/*Active Channel Number*/
	unsigned char	unused_5 ;
	unsigned char	chan_attn ;	/*Channel Attention bits*/
	unsigned char	tmr_evnt ;	/*Timer Event Bits	*/
	unsigned char	unused_8 [16] ;
    unsigned char	filler3;
	unsigned char	type ;		/*COMMON	*/
	unsigned char	unused_1A [0x3C - 0x1A] ;
	unsigned char	attn_pend ;	/*Channel Attention and/or   */
                                        /*  Timer Events Outstanding */
};

struct  common_global_s  {

	unsigned char	unused_8 [0x19 - 0x00] ;
	unsigned char	type ;	/*SSP Type bit	*/
	unsigned char		unused_1A [0x3D - 0x1A] ;

} ;

/************************************************************/
/*                                                          */
/* UNION OF GLOBAL REGISTER STRUCTURES                      */
/*                                                          */
/************************************************************/

union	global_regs_u	/*Declare Union of SSP & SSP2 Global registers	*/
{
  struct  icp_gbl_struct	ssp ;   /*SSP Global registers structure */
  struct  ssp2_global_s 	ssp2 ;	/*SSP2 Global registers structure */
  struct  common_global_s	com ;	/*SSP/SSP2 Common Global regs struct */

} ;

/************************************************************
 *                                                          * 
 * SSP2 ADDRESS SPACE DEFINITION                            *
 * Place here to calculate sizes of registers               *
 ************************************************************/

struct  ssp2_addr_space_s  {
	struct  ssp2_input_s   	in_chan  [4] ;	/*Input Channel area */
	struct  ssp2_output_s	out_chan [4] ;	/*Output Channel area */
	union   global_regs_u  	global ;	/*Global area */
        unsigned char empty_0[0x1000 - 0x400 - sizeof(union global_regs_u)];
	unsigned char		in_buf  [0x1000] ;	/*Input buffer area */
	unsigned char		out_buf [0x1000] ;	/*Output buffer area */
	unsigned char		tags    [0x0400] ;	/*Input tag area */
	unsigned char empty_1[0x4000 - 0x3000 - 0x400] ;
} ;	

/************************************************************
 *                                                          *
 *UNION OF BANKED REGISTER STRUCTURES                       *
 *                                                          *
 ************************************************************/

union	bank_regs_u	/*Declare Union of SSP & SSP2 Banked registers	*/
{
  struct  cin_bnk_struct	ssp ;  /*SSP Banked registers structure	*/
  struct  ssp2_bank_s		ssp2 ; /*SSP2 Banked registers structure*/
  struct  common_bank_s		com ;  /*SSP/SSP2 Common Banked regs struct*/

} ;


/* icp input register structure */

struct icp_in_struct
{
  ushort_t cin_char_ctrl;               /* input char control & attributes */
  uchar_t  cin_locks;                   /* input locks */
  uchar_t  cin_dma_base;                /* input dma base - bits b23 - b16 */
  ushort_t cin_baud;                    /* input baud */
  uchar_t  cin_q_ctrl;                  /* input queue control reg */
  uchar_t  cin_lit_nxt_char;            /* literal next char */
  uchar_t  cin_xon_1;                   /* xon-1 character */
  uchar_t  cin_xon_2;                   /* xon-2 character */
  uchar_t  cin_xoff_1;                  /* xoff-1 character */
  uchar_t  cin_xoff_2;                  /* xoff-2 character */
  uint_t                cin_filler_0;  
  uchar_t  cin_tmr_preset_size;         /* character timer scale preset */
  uchar_t  cin_tmr_preset_count;        /* character tick count preset */
  ushort_t cin_attn_ena;                /* input attention mask */
  ushort_t cin_overload_lvl;            /* input queue/buffer overload level */
  uchar_t  cin_susp_output_lmx;         /* enable channel output suspension */
  uchar_t  cin_susp_output_sig;         /* enable channel output suspension */
  ushort_t cin_min_char_lvl;            /* minimum character level */
  ushort_t              cin_filler_1[3];
  uchar_t  cin_lookup_tbl[32];          /* char lookup attn bits */
  ushort_t cin_tail_ptr_a;              /* tail pointer a */
  ushort_t              cin_filler_2;
  union		bank_regs_u cin_bank_a;
  ushort_t      cin_tdm_early;          /* input tdm early */
  ushort_t              cin_filler_3;
  ushort_t      cin_baud_ctr;           /* autobaud ctr & ctr bit sample */
  uchar_t       cin_input_state;        /* input state */
  uchar_t       cin_int_flags;          /* self-clearing flags */
  ushort_t cin_tail_ptr_b;              /* tail pointer b */
  ushort_t              cin_filler_4;
  union		bank_regs_u cin_bank_b;
  ushort_t      cin_tdm_late;           /* input tdm late */
  ushort_t              cin_filler_5;
  uchar_t       cin_tmr_size;           /* character timer scale */
  uchar_t       cin_char_tmr_remain;    /* (VTIME) number of remaining ticks */
  uchar_t       cin_partial_char;       /* partial incoming character */
  uchar_t       cin_inband_flow_ctrl;   /* inband flow control */
};   /* icp_in_struct */

/* icp output queue structure -
     this structure is "owned" by cpu until a command is given to
     execute commands (after which it is owned by icp */

struct cout_que_struct
{
  ushort_t      q_cmd_ptr;              /* queue cmd ptr - lower 16 bits 
                                           must be left-shifted by 2 when
                                           written and right-shifted by 2
                                           when read */
  uchar_t       q_cmd_base;             /* queue cmd base - bits 23..16 */
  uchar_t       q_cmd_save;             /* queue command save register */
  ushort_t      q_data_ptr;             /* queue data ptr - low 16 bits */
  uchar_t       q_data_base;            /* queue data base - bits 23..16 */
  uchar_t       q_size;                 /* circular queue size/wrap reg(type) */
  ushort_t      q_data_count;           /* queue data count register */
  uchar_t       q_block_count;          /* queue block count register */
  uchar_t       q_out_state;            /* queue output state register */
};   /* cout_que_struct */

/************************************************************
 *                                                          *
 * UNION OF QUEUE REGISTER STRUCTURES                       *
 *                                                          *
 ************************************************************/

union	queue_regs_u	/*Declare Union of SSP & SSP2 Queue registers	*/
{
	struct  cout_que_struct	ssp ;	/*SSP Queue registers structure	*/
	struct  ssp2_queue_s		ssp2 ;	/*SSP2 Queue registers structure */
} ;

/* icp output register structure */

struct  icp_out_struct
{
  union		queue_regs_u cout_q0;   /* queue zero */
  ushort_t      cout_tdm_0;             /* output tdm 0 */
  ushort_t      cout_tdm_1;             /* output tdm 1 */
  uint_t        cout_tx_fifo_0;         /* next four chars to tx */
  uchar_t       cout_ses_col_a;         /* session a column counter */
  uchar_t  cout_ses_ctrl_a;             /* session a control register */
  ushort_t	cout_ses_invent_a;	/* session a count chars invented */
  uchar_t               cout_filler_0[8];
  uchar_t	cout_int_flow_ctrl;     /* cpu flow control register */
  uchar_t       cout_int_opost;         /* internal opost & flow control */
  uchar_t       cout_int_fifo_ptr;      /* internal fifo pointers */
  uchar_t       cout_int_save_togl;     /* internal request and state mgmt */
  ushort_t      cout_timer;             /* (24h) output timer */
  ushort_t      cout_flow_count;        /* output flow control counter */
  ushort_t      cout_cmd_dly_timer;     /* command delay timer */
  ushort_t      cout_events_a;          /* (2ah) output events register a */
  ushort_t      cout_status;            /* (2ch) output status register */
  ushort_t      cout_events_b;          /* (2eh) output events register b */

                                        /* these dma ptrs are copied from 
                                           current cout_que_struct */
  ushort_t      cout_dma_ptr;           /* dma address ptr - low 16 bits */
  uchar_t       cout_dma_base;          /* dma base - upper 8 bits */

  uchar_t       cout_dma_stat_save;     /* dma status save */
  uchar_t       cout_ses_col_b;         /* session b column counter */
  uchar_t  cout_ses_ctrl_b;             /* session b control register */
  ushort_t	cout_ses_invent_b;	/* session b count chars invented */
  uchar_t               cout_filler_1[8];
  union		queue_regs_u cout_q1;   /* queue one */
  ushort_t      cout_int_baud_ctr;      /* internal baud counter */
  ushort_t      cout_int_state;         /* output state mach state */
  uint_t        cout_tx_fifo_1;         /* later four chars to tx */
  uchar_t       cout_ses_col_c;         /* session c column counter */
  uchar_t  cout_ses_ctrl_c;             /* session c control register */
  ushort_t	cout_ses_invent_c;	/* session c count chars invented */
  uchar_t               cout_filler_2[8];
  ushort_t cout_baud;             	/* output baud rate register */
  uchar_t  cout_char_fmt;               /* output character format register */
  uchar_t  cout_flow_cfg;               /* cpu flow control register */
  ushort_t cout_ctrl_sigs;              /* output control signals */
  uchar_t  cout_lmx_cmd;            	/* lmx command register */
  uchar_t       cout_mux_ctrl;          /* mux control */
  ushort_t cout_flow_timeout;           /* flow control timeout */
  ushort_t cout_attn_ena;               /* (6ah) output attention mask */
  uchar_t  cout_xoff_1;                 /* xoff-1 character */
  uchar_t  cout_xoff_2;                 /* xoff-2 character */
  uchar_t  cout_xon_1;                  /* xon-1 character */
  uchar_t  cout_xon_2;                  /* xon-2 character */
  uchar_t  cout_tmr_size;               /* (70h) timer scale register */
  uchar_t  cout_tmr_count;              /* (71h) timer register */
  uchar_t  cout_cpu_req;                /* (72h) cpu request register */
  uchar_t  cout_lock_ctrl;              /* (73h) output lock and control reg */
  uchar_t       cout_ses_col_d;         /* session d column counter */
  uchar_t  cout_ses_ctrl_d;             /* session d control register */
  ushort_t	cout_ses_invent_d;	/* session d count chars invented */
  uchar_t               cout_filler_3[8];
};  /* icp_out_struct */

/************************************************************
 *                                                          *
 * UNION OF INPUT REGISTER STRUCTURES	                    *
 *                                                          *
 ************************************************************/

union	input_regs_u		/*Declare Union of SSP & SSP2 Input registers */
{
  struct  icp_in_struct	ssp ;	/*SSP Input registers structure	*/
  struct  ssp2_input_s	ssp2 ;	/*SSP2 Input registers structure */
  struct  common_input_s com ;	/*SSP & SSP2 Common Input registers structure	*/

} ;

/************************************************************
 *                                                          *
 * UNION OF OUTPUT REGISTER STRUCTURES                      *
 *                                                          *
 ************************************************************/

union	output_regs_u	/*Declare Union of SSP & SSP2 Output registers	*/
{
  struct  icp_out_struct	ssp ;	/*SSP Output registers structure*/
  struct  ssp2_output_s		ssp2 ;	/*SSP2 Output registers structure*/
  struct  common_output_s	com ;	/*SSP & SSP2 Common Output registers structure	*/

} ;

