/*****************************************************************************/
/*      EQUINOX SuperSerial Technology (SST) Driver for Linux                */
/*                                                                           */
/*      Copyright (C) 1989-2002 Equinox Systems Inc.                         */
/*****************************************************************************/   

/*
 * ioctl's private to this driver
 */
#ifndef uchar_t
#define uchar_t unsigned char
#endif
#ifndef uint_t
#define uint_t unsigned int
#endif
#ifndef ulong_t
#define ulong_t unsigned long
#endif
#ifndef ushort
#define ushort unsigned short
#endif

#define ISA_BUS   1
#define EISA_BUS  2
#define MCA_BUS   3
#define PCI_BUS   4

#define BUS_ISA   ISA_BUS
#define BUS_EISA  EISA_BUS
#define BUS_MCA   MCA_BUS
#define BUS_PCI   PCI_BUS

#define HWQ4SIZE        4096		/* size of queue - each chnl has two */
#define HWQ4BITS        0x0fff    	/* 12 bit mask for 4096 */
#define HWQ4RXWRAP      0x04		/* 4096 */
#define HWQ4TXWRAP      0x0c		/* 4096, circular */
#define HWQ4HIWAT       HWQ4SIZE * 7 / 8
#define HWQ4LOWAT       HWQ4SIZE / 8
#define HWQ4CMDSIZE     0x04    	/* 1024 = 4096/4 */
#define HWQ4MINCHAR	HWQ4SIZE / 2

#define HWQ2SIZE        2048		/* size of queue - each chnl has two */
#define HWQ2BITS        0x07ff    	/* 12 bit mask for 2048 */
#define HWQ2RXWRAP      0x03		/* 2048 */
#define HWQ2TXWRAP      0x0b		/* 2048, circular */
#define HWQ2HIWAT       HWQ2SIZE * 3 / 4
#define HWQ2LOWAT       HWQ2SIZE / 4
#define HWQ2CMDSIZE     0x03    	/* 512 = 2048/4 */
#define HWQ2MINCHAR	HWQ2SIZE / 2

/* for SSM */
#define SSM             1
#define HWQ1SIZE        1024            /* size of queue - each chnl has two */
#define HWQ1BITS        0x03ff          /* 12 bit mask for 1024 */
#define HWQ1RXWRAP      0x02            /* 1024 */
#define HWQ1TXWRAP      0x0a            /* 1024, circular */
#define HWQ1HIWAT       960             /* 1024 - 64 */
#define HWQ1LOWAT       HWQ1SIZE / 4
#define HWQ1CMDSIZE     0x03            /* 512 = 2048/4 */
#define HWQ1MINCHAR	64

#define HWQ_5SIZE        512           /* size of queue - each chnl has two */
#define HWQ_5BITS        0x01ff          /* 12 bit mask for 512 */
#define HWQ_5RXWRAP      0x01            /* 512 */
#define HWQ_5TXWRAP      0x09            /* 512, circular */
#define HWQ_5HIWAT       448            /* 512- 64 */
#define HWQ_5LOWAT       HWQ_5SIZE / 4
#define HWQ_5CMDSIZE     0x02            /* 256 = 2048/8 */
#define HWQ_5MINCHAR	 32
/* end SSM changes   */

/* begin RS422 changes */
#define RS422   1
#define LMX_8E_422         0x06           /* ss8-e 422 */
#define LMX_PM16_422       0x07           /* rm16-rj 422 */
/* end RS 422 changes */

#define HWREGSLEN	0x4000		/* 16k for every icp */

#define MEGASLAVE	(('M'<<8)|81)	/* set slave printer on/off strings */
#define BINDTOKEY	(('M'<<8)|82)	/* map functions keys */
#define GETBINDINGS	(('M'<<8)|83)	/* list bindings */
#define FLUSHBINDINGS	(('M'<<8)|84)	/* flush bindings */
#define STATUS1		(('M'<<8)|85)	/* get port status */
#define MEGASTTY	(('M'<<8)|86)	/* overwrite stty setup */
#define MEGARESUME	(('M'<<8)|87)	/* resume output */
#define MSCREEN		(('M'<<8)|88)	/* set multi screen parameters */
#define MEGAMEM		(('M'<<8)|89)	/* display memory */
#define STATUS2		(('M'<<8)|90)	/* get port settings */
#define STATUS3		(('M'<<8)|91)	/* get port number */
#define MPEQULZ		(('M'<<8)|92)	/* equalization */
#define MEGADTR		(('M'<<8)|93)	/* toggle dtr */
#define SSTCFG		(('M'<<8)|94)	/* configure sst devices */

struct mpmem {
	int memstart;
	int memlen;
	int memdev;
	char *memptr;
};

#define KEYLEN		8
#define VALUELEN	128
struct	megamap {
	char	key[KEYLEN];
	char	value[VALUELEN];
};

#define SCREENLEN	8
#define HOTKEYLEN	2	
struct  megascrn {
	short   esc_time;		/* time in 80ms inc wait for esc */
	char    scrn1[SCREENLEN];	/* goto screen 0 esc string */
	char    scrn2[SCREENLEN];	/* goto screen 1 esc string */
	uchar_t hotkey[HOTKEYLEN];	/* hotkey to select active screen */
};

#ifndef READ_COLLISION
#define READ_COLLISION 2
#define WRITE_COLLISION 4
#endif

typedef struct node *naddr_t;

struct	node {
	ushort	n_type;			/* see defines below */
#define NODE_MAP	0
#define NODE_STR	1
	short	n_len;			/* length of n_value */
	uchar_t	n_name[KEYLEN];		/* node name */
	naddr_t	n_left, n_right;	/* pointer to left and right subtrees */
	uchar_t	n_value[1];		/* length varies */
};


struct 	disp {
	int	mp_xstate;
	int	mp_state;
	int	mp_iflag;
	int	mp_oflag;
	int	mp_lflag;
	int	mp_cflag;
	int	mp_flags;
	int mp_dev;
	uchar_t mp_rawqcc;
	uchar_t mp_outqcc;
	uchar_t mp_xoutqcc;
	uchar_t mp_delct;
	uint_t	mp_param;
	uint_t	mp_input;
	uint_t	mp_output;

	uchar_t	mp_tx_base;
	uchar_t mp_rx_base;
	uchar_t mp_rx_fmt;
	uchar_t mp_tx_fmt;
	ushort	mp_tx_q_ptr;
	ushort	mp_tx_q_cnt;
	uchar_t	mp_tx_q_blk;
	uchar_t mp_tx_q_state;
	ushort	mp_tx_csr;
	ushort	mp_txbaud;
	uchar_t	mp_tx_flow_cfg;
	uchar_t	mp_tx_cpu_reg;
	ushort	mp_tx_count;
	ushort	mp_rx_next;
	ushort	mp_rx_tail;
	ushort	mp_rxbaud;
	ushort	mp_cin_ctrl;
	ushort	mp_cout_ctrl;
	ushort	mp_rx_csr;
	ushort	mp_cie;
	ushort	mp_cis;
	ushort	mp_rxtdm;
	ushort	mp_txtdm;
	uchar_t	mp_susp_lmx;
	uchar_t	mp_susp_sig;
	ushort	mp_rx_rcv_cnt;
	uchar_t	mp_vmin;
	uchar_t	mp_vtime;
	uchar_t	mp_fifo_cnt;
	uchar_t mp_rx_locks;
	ushort	mp_cieo;
	ushort	mp_tx_events;
	uchar_t	mp_int_flgs;
	uchar_t mp_rxpg;
	uchar_t mp_txpg;
	uchar_t	mp_ldisp;
	uchar_t	mp_rmode;
	uchar_t	mp_wmode;
	uchar_t mp_start;
	uchar_t mp_stop;
	uchar_t mp_esctim;
	uchar_t mp_htky1;
	uchar_t mp_htky2;
	uchar_t mp_bus;
	uchar_t mp_brdno;
	uchar_t mp_icpno;
	uchar_t mp_lmxno;
	uchar_t mp_chan;
	uchar_t mp_lmx_cmd;
	uchar_t mp_g_attn;
	uchar_t mp_g_init;
	int	mp_g_rx_attn0;
	int     mp_mpa_stat;
};

struct ModemStatus{
	int mp_dev;
	uint_t mpstatus;
};

struct SttyStatus{
	int mp_dev;
	uint_t mpc_param;
};

