/*****************************************************************************/
/*      EQUINOX SuperSerial Technology (SST) Driver for Linux                */
/*                                                                           */
/*      Copyright (C) 1989-2002 Equinox Systems Inc.                         */
/*****************************************************************************/   

typedef struct megaport *mpaddr_t;
typedef union  input_regs_u *icpiaddr_t;
typedef union  output_regs_u *icpoaddr_t;
typedef union  global_regs_u *icpgaddr_t;
typedef union  bank_regs_u *icpbaddr_t;
typedef union  queue_regs_u *icpqaddr_t;
typedef struct sess *icpsaddr_t;
typedef struct sessions *icpssaddr_t;

struct ccb_struct *ccb;

struct megaport {				/* 16KB */
	union  input_regs_u mp_icpi[64];		/* 8KB */
	union  output_regs_u mp_icpo[64];		/* 8KB */
};

/* examples using new icp struct */
/*
mpd->mpd_mp = (mpaddr_t) sptalloc......(alloc 16kb)

mpc->mpc_icpi = (icpiaddr_t)&mpd->mpd_mp->mp_icpi[chan];
mpc->mpc_icpo = (icpoaddr_t)&mpd->mpd_mp->mp_icpo[chan];

mpd->mpd_icpg = (icpgaddr_t)&mpd->mpd_mp->mp_icpo[0];

register icpiaddr_t icpi;
register icpbaddr_t icpb;

icpi = mpc->ssp.mpc_icpi;

icpb = (rx_csr & BANK_B_ACT) ? &icpi->ssp.cin_bank_b : &icpi->ssp.cin_bank_a;

*/

/* session control struct */

struct sess {
        char    fill_0[20];
        uchar_t col_cnt;                /* column counter */
        uchar_t sess_cntr;              /* session control reg */        
        ushort_t sess_char_cnt;         /* opost expension chars */   
        char    fill_1[8];
};

struct sessions {
        struct sess cout_sess[256];
};
                
/* defines gicp_bus_ctrl */

#define CPU_BUS_8	0
#define CPU_BUS_16	1
#define CPU_BUS_32	2
#define DRAM_24		0
#define DRAM_40		4
#define STERNG		8
#define SWAPNG		0x10
#define DMARQ_2		0x20
#define GTIMER_EN	0x40

/* defines gicp_initiate */

#define ICP_2		1
#define RNG_CLK_ON	2
#define ICP_PRAM_WR	4
#define DMA_EN		8

/* defines gicp_scan_spd */

#define A_16_PORTS	0
#define A_8_PORTS	1
#define A_4_PORTS	2
#define A_2_PORTS	3

#define B_16_PORTS	0
#define B_8_PORTS	4
#define B_4_PORTS	8
#define B_2_PORTS	0xc

#define C_16_PORTS	0
#define C_8_PORTS	0x10
#define C_4_PORTS	0x20
#define C_2_PORTS	0x30

#define D_16_PORTS	0
#define D_8_PORTS	0x40
#define D_4_PORTS	0x80
#define D_2_PORTS	0xc0

/* gicp_tmr_size res 520.8 us when set to 0xff and 133.3 msec when set to 00 
   (256 - Desired tick size / 520.8 us) + 1 */

/* gicp_tmr_count (256 - Desired time / tick size) + 1; 
   520.8 usec <= Timer Interval <= 34.13 seconds */

/* gicp_watchdog */

#define WDOG_DIS	0
#define WDOG_1_SEC	0x20
#define WDOG_2_SEC	0x40
#define WDOG_4_SEC	0x60
#define WDOG_8_SEC	0x80
#define WDOG_16_SEC	0xa0
#define WDOG_32_SEC	0xc0

/* gicp_attn */

#define GATTN_RX	1
#define GATTN_TX	2
#define RNG_FAIL	4
#define WDOG_EXP	8

/* current chan */

#define CHAN_NUM	0xf
#define LMX_NUM		0x30

/* cin_bank_signals */

#define CIN_BREAK	1
#define CIN_DCD         2
#define CIN_CTS         4
#define CIN_DSR         8
#define CIN_RI          0x10
#define LMX_ONLN        0x20    
#define LMX_RMT         0x40    /* if 0 then local lmx or bridge, else mux */
#define LMX_PRESENT	0x80    /* channel's LMX present */
#define ICP_OUT_2	0x100
#define LMX_OUT_2	0x200
#define AMI_LNK_ON      0x400   /* if a bridge link to a mux */
#define XPRNCY_PEND	0x800   /* waiting for link with mux */
#define LMX_LCK		0x1000  /* ring locked */
#define AMI_CNFG	0x2000  /* for bridge and mux is 1, for panel is 0 */

/* if LMX_ONLN is 1 then we can use all other lmx bits for info */

/* cin_bank_events - status of events that changed */

#define EV_AUTO_BAUD	1
#define EV_DCD_CNG	2
#define EV_CTS_CNG	4
#define EV_DSR_CNG	8
#define EV_RI_CNG	0x10
#define EV_BREAK_CNG	0x20
#define EV_LMX_CNG	0x40
#define EV_PAR_ERR	0x80
#define EV_FRM_ERR	0x100
#define EV_CHAR_LOOKUP	0x200
#define EV_VTIME	0x400
#define EV_VMIN		0x800
#define EV_OVERRUN	0x1000
#define EV_DMA_FAIL	0x2000
#define EV_REG_UPDT	0x4000
#define CHAN_ATTN_SET	0x8000
 
/* cin_char_cntrl */

#define CS_5		0		/* not valid if one of four top baud */
#define CS_6		1
#define CS_7		2
#define CS_8		3
#define PARITY_ON	4
#define PAR_ODD		0
#define PAR_EVEN	8
#define PAR_MARK	0x10
#define PAR_SPACE	0x18
#define IGN_BAD_CHAR	0x20
#define IGN_BRK_NULL	0x40
#define EN_CHAR_LOOKUP	0x80
#define NO_CMP_ERR	0x100
#define EN_LITNXT	0x200
#define EN_IXANY	0x400
#define EN_DBL_FLW	0x800
#define EN_DNS_FLW	0x1000
#define EN_ISTRIP	0x2000
#define EN_XOFF		0x4000
#define EN_XON		0x8000
  
/* input locks */

#define DIS_BANK_A	1
#define DIS_BANK_B	2
#define DIS_IN_STAT	4
#define DIS_IBAND_FLW	8
#define DIS_VTIME_WR	0x10
#define DIS_AUTOBAUD	0x20
#define DIS_SLF_CLRFLG	0x40
#define DIS_DMA_WR	0x80
#define SEL_BANK_A	2
#define SEL_BANK_B	1
#define SEL_NO_BANK	3

/* input queue control */

#define Q_SIZE_256	0
#define Q_SIZE_512	1
#define Q_SIZE_1K	2
#define Q_SIZE_2K	3
#define Q_SIZE_4K	4
#define Q_SIZE_8K	5	
#define Q_SIZE_16K	6
#define Q_SIZE_32K	7
#define Q_SIZE_64K	8

#define TAIL_PTR_B	0x10		/* set tail ptr b wr/rd */
#define EN_IXOFF_SVC	0x20
#define ST_IN_TMR       0x40            /* toggle input char timer */
#define CIN_TMR_MODE	0x80

/* cin_attn_ena */

#define ENA_AUTO_BUAD	1
#define ENA_DCD_CNG	2
#define ENA_CTS_CNG	4
#define ENA_DSR_CNG	8
#define ENA_RI_CNG	0x10
#define ENA_BREAK_CNG	0x20
#define ENA_LMX_CNG	0x40
#define ENA_PAR_ERR	0x80
#define ENA_FRM_ERR	0x100
#define ENA_CHAR_LOOKUP	0x200
#define ENA_VTIME	0x400
#define ENA_VMIN	0x800
#define ENA_IXOFF_SVS	0x1000
#define ENA_DMA_FAIL	0x2000
#define ENA_REG_UPDT	0x4000
#define EN_REG_UPDT_EV	0x8000

/* input signal flow control - cin_susp_output_lmx */

#define LMX_NOT_CONN	1
#define DCD_OFF		2
#define CTS_OFF		4
#define DSR_OFF		8
#define RI_OFF		0x10
#define MUX_NOT_CONN	0x20
#define LMX_OFF_LINE	0x40
#define CHAN_DAT_LNBK	0x80            /* chan data line non broken */

/* input signal flow control - cin_susp_output_sig */

#define DCD_ON		2
#define CTS_ON		4
#define DSR_ON		8
#define RI_ON		0x10
#define CHAN_DAT_LBRK	0x80            /* chan data line broken */

/* self clearing input flags - read only by cpu */

#define GOD_FRM_RCV	1
#define XON_1_RCV	2
#define XOFF_1_RCV	4
#define PRV_CH_LNXT	8
#define NO_CMP_ON_ERR	0x10
#define DAT_OUT_SUSP	0x20
#define IN_BUF_OVFL	0x40
#define BANK_B_ACT	0x80

/* output defines */

/* data queue/buffer type & size */

#define QSZ_256         0
#define QSZ_512         1
#define QSZ_1K          2
#define QSZ_2K          3
#define QSZ_4K          4
#define QSZ_8K          5
#define QSZ_16K         6
#define QSZ_32K         7
#define EN_CIRC_Q       8
#define EN_TX_LOW       0x10
#define EN_TX_EMPTY     0x20

/* queue output state */

#define	CMDQ_64		0
#define CMDQ_128	1
#define CMDQ_256	2
#define CMDQ_512	3
#define CMDQ_1K		4
#define CMDQ_2K		5
#define CMDQ_4K		6
#define CMDQ_8K		7
#define CMDQ_HALT	8
#define CMDQ_SND	0x10
#define CMDQ_EXEC	0x20
#define CMDQ_WAIT       0x40
#define CMDQ_CONT_SND   0x80            /* cpu write use lock 6 */

/* session control register */

#define SCR_EN		1
#define SCR_TABX	2
#define SCR_NLCR	4
#define SCR_CRNL	8
#define SCR_NOCR	0x10
#define SCR_NLRET	0x20

/* output event register */

#define EV_CPU_REQ_DN	1
#define EV_Q0_MRKR	2
#define EV_Q0_HLT_CMD	4
#define EV_Q1_MRKR	8
#define EV_Q1_HLT_CMD	0x10
#define EV_FLC_TMO	0x20
#define EV_CMD_PRC_INH  0x40
#define EV_OUT_TMR_EXP	0x80
#define EV_ILG_REQ	0x100
#define EV_TX_EMPTY_Q0  0x200
#define EV_TX_LOW_Q0    0x400
#define EV_TX_EMPTY_Q1  0x800
#define EV_TX_LOW_Q1    0x1000
#define EV_REG_UPDT	0x4000
#define EV_ATTN_SET	0x8000

/* output status register */

#define TXSR_Q1_ACT	1
#define TXSR_SND_BRK	2
#define TXSR_UNBRK_SEQ	4
#define TXSR_OPOST_INH	8
#define TXSR_EV_B_ACT	0x10
#define TXSR_CMDPRC_INH	0x20
#define TXSR_SND_HALT   0x40            /* Transistion to halt from send data */

/* output char control */

#define TX_CS5		0
#define TX_CS6		1
#define TX_CS7		2
#define TX_CS8		3
#define TX_PARENB	4
#define TX_PAR_ODD	0
#define TX_PAR_EVEN	8
#define TX_PAR_MARK	0x10
#define TX_PAR_SPACE	0x18
#define TX_2STPB	0x20

/* xon/xoff config */

#define TX_XOFF_RDN	1
#define TX_XON_XOFF_EN	2
#define TX_XON_DBL	4
#define TX_SND_XON      8
#define TX_TGL_XON_XOFF 0x10
#define	TX_BREAK_ON	0x20
#define TX_XTRA_DMA     0x40
#define TX_SUSP         0x80

/* output control signals */

#define TX_DTR		1
#define TX_RTS		2
#define TX_CNT_2	4
#define TX_CNT_3	8
#define TX_HFC_DTR	0x10
#define TX_HFC_RTS	0x20
#define TX_HFC_2	0x40
#define TX_HFC_3	0x80

#define TX_LED		0x100
#define TX_SND_CTRL_TG	0x200
#define TX_TRGT_LMX_MUX	0x8000

/* lmx command register */

#define TX_INTR_LB	1
#define TX_XTRN_LB	2
#define TX_LMX_ONLN	8
#define TX_BRDG_XPRNT	0x10
#define TX_LCK_LMX	0x20
#define TX_LMX_CMD_EN	0x40
#define TX_TRGT_MUX	0x80

/* output attention enable */

#define ENA_CPU_REQ_DN	1
#define ENA_Q0_MRKR	2
#define ENA_Q0_HLT_CMD	4
#define ENA_Q1_MRKR	8
#define ENA_Q1_HLT_CMD	0x10
#define ENA_FLC_TMO	0x20
#define ENA_CMD_PRC_INH 0x40
#define ENA_OUT_TMR_EXP	0x80
#define ENA_ILG_REQ	0x100
#define ENA_TX_EMPTY_Q0 0x200
#define ENA_TX_LOW_Q0   0x400
#define ENA_TX_EMPTY_Q1 0x800
#define ENA_TX_LOW_Q1   0x1000
#define ENA_REG_UPDT	0x4000
#define EN_REG_UPDT_EV	0x8000

/* cpu request register */

#define CPU_HLT_REQ	1
#define CPU_START_REQ	2
#define CPU_SND_REQ	4
#define CPU_SWTCH_Q	8
#define CPU_TMR_TGL     0x10

/* output lock register */

#define LCK_EVT_A	1
#define LCK_EVT_B	2
#define LCK_FLW_TMR	4
#define LCK_MISC_WR	8
#define LCK_OUT_TMR	0x10
#define LCK_STATUS	0x20
#define LCK_Q_ACT	0x40
#define DIS_CMD_PROC	0x80

/* session control - opost */

#define MOPOST           1
#define MTABX            2
#define MONLCR           4
#define MOCRNL           8
#define MONOCR           0x10
#define MONLRET          0x20

#define NCHARS  blk_sz
#define LMASK   ((output_tags_sz -1) << 2)
#define UMASK   (((short)output_tags << 2) & ~LMASK)
#define ZMASK   (UMASK|LMASK)

/* globals defines */

#define gicp_bus	icpg->ssp.gicp_bus_ctrl
#define Gicp_rev	icpg->ssp.gicp_rev	/* COMMON */
#define Gicp_initiate	icpg->ssp.gicp_initiate
#define Gicp_scan_spd	icpg->ssp.gicp_scan_spd
#define Gicp_tmr_size	icpg->ssp.gicp_tmr_size
#define Gicp_tmr_count	icpg->ssp.gicp_tmr_count
#define Gicp_watchdog	icpg->ssp.gicp_watchdog
#define i_gicp_attn	icpg->ssp.gicp_attn
#define Gicp_chnl	icpg->ssp.gicp_chnl
#define Gicp_fram_ctr	icpg->ssp.gicp_frame_ctr
#define gicp_rx_attn	icpg->ssp.gicp_rcv_attn
#define gicp_tx_attn	icpg->ssp.gicp_xmit_attn


/* input struct defines */

#define rx_base		icpi->ssp.cin_dma_base
#define rx_baud		icpi->ssp.cin_baud	/* COMMON */
#define rx_locks	icpi->ssp.cin_locks	/* COMMON */
#define rx_xon		icpi->ssp.cin_xon_1	/* COMMON */
#define rx_xoff		icpi->ssp.cin_xoff_1	/* COMMON */
#define rx_param	icpi->ssp.cin_char_ctrl	/* COMMON */
#define rx_q_ctrl	icpi->ssp.cin_q_ctrl	/* COMMON */
#define rx_vtime_scale	icpi->ssp.cin_tmr_preset_size	/* COMMON */
#define rx_vtime	icpi->ssp.cin_tmr_preset_count	/* COMMON */
#define rx_int_vtime_scale icpi->ssp.cin_tmr_size	/* COMMON */
#define rx_int_vtime	icpi->ssp.cin_char_tmr_remain	/* COMMON */
#define rx_cie		icpi->ssp.cin_attn_ena	/* COMMON */
#define rx_hiwat	icpi->ssp.cin_overload_lvl	/* COMMON */
#define rx_suspo_lmx	icpi->ssp.cin_susp_output_lmx	/* COMMON */
#define rx_suspo_sig	icpi->ssp.cin_susp_output_sig	/* COMMON */
#define rx_vmin		icpi->ssp.cin_min_char_lvl	/* COMMON */
#define rx_lokup_tbl	icpi->ssp.cin_lookup_tbl	/* COMMON */
#define rx_csr		icpi->ssp.cin_int_flags	/* COMMON */
#define rx_ixoff_rcv	icpi->ssp.cin_inband_flow_ctrl	/* COMMON */
#define rx_tdm_early	icpi->ssp.cin_tdm_early

/* bank defines - needs to load bank ptr with current bank address */
#define rx_next		icpb->ssp.bank_dma_ptr
#define rx_fifo_cnt	icpb->ssp.bank_fifo_lvl
#define rx_ctrl_sig	icpb->ssp.bank_sigs
#define rx_tags_l	icpb->ssp.bank_tags_l
#define rx_tags_h	icpb->ssp.bank_tags_h
#define rx_rcv_cnt	icpb->ssp.bank_num_chars
#define rx_events	icpb->ssp.bank_events
#define rx_fifo		icpb->ssp.bank_fifo

/* output defines */
/************************************************************************/
/* output queue */
/*
icpg = (icpgaddr_t)ccb->ccb_output_regs;
icpo = (icpoaddr_t)ccb->ccb_output_regs;
*/
#define tx_csr		icpo->ssp.cout_status
#define tx_ses_col	icpo->ssp.cout_ses_col_a
#define tx_baud		icpo->ssp.cout_baud
#define tx_baud_ctr     icpo->ssp.cout_int_baud_ctr
#define tx_char_fmt	icpo->ssp.cout_char_fmt
#define tx_flow_config	icpo->ssp.cout_flow_cfg
#define tx_ctrl_sig	icpo->ssp.cout_ctrl_sigs
#define tx_lmx_cmd	icpo->ssp.cout_lmx_cmd
#define tx_cie		icpo->ssp.cout_attn_ena
#define tx_xoff		icpo->ssp.cout_xoff_1
#define tx_xon		icpo->ssp.cout_xon_1
#define tx_cpu_reg	icpo->ssp.cout_cpu_req
#define tx_svd_togls	icpo->ssp.cout_int_save_togl
#define tx_lck_ctrl	icpo->ssp.cout_lock_ctrl
#define tx_timer	icpo->ssp.cout_timer
#define tx_dma_ptr	icpo->ssp.cout_dma_ptr
#define tx_dma_base	icpo->ssp.cout_dma_base
#define tx_dma_stat     icpo->ssp.cout_dma_stat_save
#define tx_susp_timeo	icpo->ssp.cout_flow_timeout

/*
icpq = (tx_csr & TXSR_Q1_ACT) ? &icpo->ssp.cout_cout_q1 : &icpo->ssp.cout_q0;
*/
#define Q_cmd_ptr	icpq->ssp.q_cmd_ptr
#define Q_cmd_base	icpq->ssp.q_cmd_base
#define Q_data_ptr	icpq->ssp.q_data_ptr
#define Q_data_base	icpq->ssp.q_data_base
#define q_type          icpq->ssp.q_size
#define Q_data_count	icpq->ssp.q_data_count
#define Q_block_count	icpq->ssp.q_block_count
#define Q_out_state	icpq->ssp.q_out_state


#define get_tail() \
{ \
	if(icpi->ssp.cin_q_ctrl & TAIL_PTR_B) \
	  return (icpi->ssp.cin_tail_ptr_b); \
	else \
	  return (icpi->ssp.cin_tail_ptr_a); \
}


#define rx_tail(val) \
{ \
        if (icpi->ssp.cin_q_ctrl & TAIL_PTR_B) \
           icpi->ssp.cin_tail_ptr_a = (val); \
	else  \
	   icpi->ssp.cin_tail_ptr_b = (val); \
       	rx_q_ctrl ^= TAIL_PTR_B; \
}           


#define FREEZ_BANK(mpc) \
{ \
	ushort_t cie = CHAN_ATTN_SET | rx_cie; \
	int frztimeo = 0; \
unsigned  char lcks = 0; \
	rx_cie = 0; \
    if( (rx_locks & 1) == 1) { /* Bank A is locked */\
       icpb = &icpi->ssp.cin_bank_b; \
	lcks = 0x80; \
     }  \
     else  \
       icpb = &icpi->ssp.cin_bank_a; \
     if (!(icpb->ssp.bank_events & EV_REG_UPDT)) { \
        while ((icpi->ssp.cin_int_flags & 0x80) != lcks) \
           if (++frztimeo > 8000) break; \
     }  \
     mpc->mpc_icpb = icpb; \
     rx_locks ^= 3; /* Flip to the other bank */\
	cur_chnl_sync(mpc); \
	 mpc->mpc_cin_events |= rx_events; \
	 rx_events = 0; \
	 rx_cie = cie; \
} 


#define TX_EVENTS(x, mpc) \
{ \
	volatile uchar_t csr = tx_csr; \
	volatile ushort oie = tx_cie; \
       	tx_cie = 0; \
        if(csr & TXSR_EV_B_ACT) { \
             	 tx_lck_ctrl ^= 3; \
		cur_chnl_sync(mpc); \
             (x) |= icpo->ssp.cout_events_b; \
             icpo->ssp.cout_events_b = 0; \
        } else { \
        	 tx_lck_ctrl ^= 3; \
		cur_chnl_sync(mpc); \
             (x) |= icpo->ssp.cout_events_a; \
             icpo->ssp.cout_events_a = 0; \
        } \
        if((x) & EV_TX_EMPTY_Q0) \
        	oie &= ~ENA_TX_EMPTY_Q0;\
        if((x) & EV_TX_LOW_Q0) \
        	oie &= ~ENA_TX_LOW_Q0;\
	tx_cie = oie; \
}
	
#define nothing 0
/*
             if((tx_lck_ctrl & 3) == 1) { \
             if((tx_lck_ctrl & 3) == 2) { \
        if((x) & EV_TX_EMPTY_Q0) oie &= ~ENA_TX_EMPTY_Q0; \
        if((x) & EV_TX_LOW_Q0) oie &= ~ENA_TX_LOW_Q0; \
*/
        /*if(((x) & EV_TX_EMPTY_Q0) && (qcount))\
		x &= ~EV_TX_EMPTY_Q0; \
	else \
        	oie &= ~ENA_TX_EMPTY_Q0;\
        if(((x) & EV_TX_LOW_Q0) && (qcount >=\
		icpq->ssp.q_block_count * 64))\
		x &= ~EV_TX_LOW_Q0; \
	else \
        	oie &= ~ENA_TX_LOW_Q0;\*/
