/*****************************************************************************/
/*      EQUINOX SuperSerial Technology (SST) Driver for Linux                */
/*                                                                           */
/*      Copyright (C) 1989-2002 Equinox Systems Inc.                         */
/*****************************************************************************/
                                                                               
/*#title RAMP ADMIN/DIAG FACILITIES (RAMPADMN.C) E Q U I N O X   C O N F I D E N T I A L#*/
/********************************************************************************
 *                                                                              *
 *              REMOTE ACCESS MODEM POOL(RAMP) ADMIN/DIAG FACILITIES	        *
 *              ----------------------------------------------------        	*
 *                                                                              *
 * PURPOSE: To provide RAMP Adminsitrative/Diagnostic facilities and Diag/Admin	*
 *	    Poll Function as described in "Remote Access Modem Pool (RAMP) 	*
 *	    Software Services" documentation.					*
 *                                                                              *
 * NOTE: - Software was originally developed using:                             *
 *           o WATCOM 32 bit compiler                                           *
 *           o Phar Lap 386|linker                                              *
 *           o Debugged/tested in Phar Lap DOS Extender environment             *
 *	 - "MPA" is used throughout source and is the same as "RAMP"		*
 *	 - All page references refer to Plug and Play Specification Version 1.0	*
 *	   dated 5/28/93)							*
 *                                                                              *
 *                                                                              *
 * FILE:   RAMPADMN.C                                                           *
 * AUTHOR: BILL KRAMER                                                          *
 * REV:    06 (update "MPA_ADMIN_DIAG" in Header file, MPA.H, if change rev)    *
 * DATE:   10/02/97                                                             *
 *                                                                              *
 *                      E Q U I N O X   C O N F I D E N T I A L            	*
 *                      =======================================            	*
 *                                                                              *
 ********************************************************************************/
/*#page*/
/********************************************************************************
 *                                                                              *
 *                               REVISION HISTORY                               *
 *                               ================                               *
 *                                                                              *
 * REV	  DATE				                     COMMENTS           *
 * ===  ========        ======================================================= *
 *  06	09/30/97	Added conditional compilations to accommodate BSDI 	*
 *			build environment.  Following symbol, if defined, 	*
 *			indicates BSDI compilation:				*
 *				RAMP_BSDI  = BSDI compile			*
 *			NOTE: No changes were necessary for LINUX compilations	*
 *  05  09/09/96	Modified Plug and Play probe sequence to do 144 Reads	*
 *			required to isolate some newer PnP boards 		*
 *			(e.g. 33.6 Zoom)					*
 *  04  06/10/96	Added code to Admin/Poll function that saves slots' 	*
 *			Power State and save in	Slot Control Block.		*
 *  03	05/23/96	Modified function declarations to accomodate UNIX 	*
 *		        compiliers						*
 *  02	05/20/96	Modified PnP Probe functions so that a non-PnP board 	*
 *			is not Probed thus assuring when board registered it is	*
 *			recognized as a non-PnP board.  Also modified, PnP Probe*
 *			function 56 to recognize PnP Board Presence and		*
 *			therefore not Probe again.				*
 *  01	05/10/96	Initial release						*
 *                                                                              *
 ********************************************************************************/





/*#stitle HEADER FILE DECLARATIONS#											*/
/*#page*/
/*********************************************************
 * HEADER FILE DECLARATIONS                              *
 *********************************************************/

/*SSP Structure names (used to point to Global, Input, and Output registers)            */
/*NOTE: Macros below must be defined prior to inclusion of RAMP.H.  The macros below are*/
/*	necessary since various development environments utilized different structure	*/
/*	definitions for SSP Global, Input and output registers.  The definitions below	*/
/*	refer to structures defined in ramp_icp.h					*/
#define    mpa_global_s       icp_global_struct       /*Global regs                     */
#define    mpa_input_s        cin_struct              /*Input regs                      */
#define    mpa_output_s       cout_struct             /*Output regs                     */


#ifdef				RAMP_PRAGMAS		/*If Compile Time pragmas needed*/
#include	"rampprag.h"
#endif

#ifndef				RAMP_ICP_H_INCLUDED	/*If ramp_icp.h not previously included*/ /*R11*/
#include	"ramp_icp.h"
#endif

#ifndef				RAMP_H_INCLUDED		/*If ramp.h not previously included*/     /*R11*/
#include	"ramp.h"
#endif





/*#stitle STRUCTURE DEFINITIONS/DEFINES/LITERALS DECLACARATIONS#*/
/*#page*/
/************************************************************************
 * Structure definitions & defines/literals                             *
 ************************************************************************/

#define	SSPCB_LIST_MAX	((unsigned int)32)	/*Max. number of SSPCBs	to srch	*/
						/* in SSPCB List - circular 	*/
						/*  (infinite) chain limit	*/

							



/************************
 *    MACROS/LITERALS	*
 ************************/











/*#stitle PUBLIC DECLACARATIONS#*/
/*#page*/
/************************************************************************************************
 * Public Declarations i.e. Items DEFINED in this compilation Referenced in other Compilations	 *
 ************************************************************************************************/


/****************
 *  Data Areas	*
 ****************/




/***********************
 *  Service Functions  *
 ***********************/

/*NOTE: See MPA.H	*/










/*#stitle EXTERNAL DECLACARATIONS#*/
/*#page*/
/********************************************************
 * DATA EXTERNAL to this compilation                    *
 ********************************************************/

/*Data areas defined in RAMPSRVC.C							*/
extern struct	admin_diag_hdr_struct	admin_diag_hdr ; /*Administrative & Diagnostic	*/
							 /*Services Header area		*/

/*#page*/
/********************************************************
 * Functions EXTERNAL to this compilation		*
 ********************************************************/

/*Functions defined in RAMPSRVC.C						*/
extern unsigned char	inc_tail (
#ifndef 				NO_FN_PROTO_PARMS
				  struct cin_struct FARm   *
#endif
				  ) ;

extern unsigned char	mpa_fn_access_sspcb_lst (
#ifndef 				NO_FN_PROTO_PARMS
						 unsigned long int FARm *
#endif
						) ;

extern	void		mpa_fn_relse_sspcb_lst (
#ifndef 				NO_FN_PROTO_PARMS
						 unsigned long int
#endif
						) ;







/*#stitle FORWARD REFERENCE DECLACARATIONS#*/
/*#page*/
/*********************************************************************************************************************
 * Forward Reference Declarations i.e. Functions that are INTERNAL to this compilation but are forward referenced    *
 *********************************************************************************************************************/

/*Plug and Play Probing Functions								*/
static void pnp_probe_start (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_02 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_03 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_04 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_05 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_06 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_07 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_08 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_09 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_10 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_11 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_12 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_13 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_14 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_15 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_16 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_17 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_18 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_19 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_20 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_21 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_22 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_23 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_24 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_24a (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			     ) ;

static void pnp_probe_fn_24b (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			     ) ;

static void pnp_probe_fn_24c (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			     ) ;

static void pnp_probe_fn_24d (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			     ) ;

static void pnp_probe_fn_25 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_26 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_27 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;
/*NOTE: Functions 28 & 29 deleted, see note at end of Function # 27		*/

static void pnp_probe_fn_30 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_31 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_32 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_33 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_34 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_35 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_36 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_37 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_38 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_39 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_40 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_41 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_42 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_43 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_44 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_45 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_46 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_47 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_48 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_49 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_50 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_51 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_52 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_53 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_54 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_55 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_56 (
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void pnp_probe_fn_57 (				/*R2*/
#ifndef 				NO_FN_PROTO_PARMS
			     struct ssp_struct   FARm *,
      			     struct slot_struct  FARm *,
      			     struct mpa_global_s FARm *,
			     struct mpa_input_s  FARm *,
      			     struct mpa_output_s FARm *
#endif
			    ) ;

static void	pnp_clean_up_ssp (
#ifndef 				NO_FN_PROTO_PARMS
				  struct slot_struct FARm *    slot,
			          unsigned char	               type
#endif
				 ) ;

/*Internal/Local Functions									*/
static unsigned char	pnp_frame_delay_chk (
#ifndef 						NO_FN_PROTO_PARMS
					     unsigned short int,
		  			     struct icp_global_struct FARm *,
					     struct slot_struct FARm       *
#endif
					    ) ;











/*#stitle GLOBAL (STATIC) DATA AREA#*/
/*#page*/
/******************************************************************************************
 * Global Data Area i.e. Variables declared in Data Segment but local to this compilation *
 ******************************************************************************************/

/*PnP Initiation Key Sequence												*/
static unsigned char pnp_init_seq [] = {0x00, 0x00,
					0x6A, 0xB5, 0xDA, 0xED, 0xF6, 0xFB, 0x7D, 0xBE,
					0xDF, 0x6F, 0x37, 0x1B, 0x0D, 0x86, 0xC3, 0x61,
					0xB0, 0x58, 0x2C, 0x16, 0x8B, 0x45, 0xA2, 0xD1,
					0xE8, 0x74, 0x3A, 0x9D, 0xCE, 0xE7, 0x73, 0x39
				       }  ;







/*#stitle ADMIN/DIAG SERVICE - ENABLE/DISABLE PLUG & PLAY PROBE (mpa_srvc_pnp_probe_cntrl)#*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *                        ENABLE/DISABLE PLUG & PLAY PROBE		       *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
extern unsigned short int	mpa_srvc_pnp_probe_cntrl (struct ssp_struct FARm *  sspcb,
	                               	          	  unsigned char		    probe_ena)
#else
extern unsigned short int	mpa_srvc_pnp_probe_cntrl (sspcb, probe_ena)

struct ssp_struct FARm *  sspcb ;
unsigned char		  probe_ena;
#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

/*Variables used to access and search SSPCB List					*/
unsigned long int	svd_int_msk ;		/*Int Msk prior to Dsabl		*/
struct ssp_struct FARm	*sspcb_srch_ptr ;	/*Ptr to SSPCB in List			*/
unsigned int		silent_death_ctr ;	/*Silent death counter			*/

/*Variables used to access an SSPCB's MPA(s) and SLOTCBs				*/
unsigned int		mpa_index ;		/*MPA Array member			*/
unsigned int		num_slots ;		/*# Slots assoc'd w/MPA Array Member	*/
struct slot_struct FARm	*slot ;			/*MPA's/Eng's associated Slot Member	*/

/*General Working Variables								*/
unsigned int		i ;				/*Loop counter			*/




/************************
 *   START OF CODE	*
 ************************/

/*Validate caller's parameters								*/
if ( (sspcb            == (struct ssp_struct FARm *)0) || /*Does caller's SSP		*/
     (sspcb->in_use    != 0xFF)     ||			  /* Control Block		*/
     (sspcb->rev       != MPA_REV)  ||			  /*  appear sane?		*/
     (sspcb->signature != sspcb) )
	return (ERR_PNP_PROBE_SSPCB_BAD) ;		/*If no, exit			*/

  /*Set up to search for caller's SSPCB in Administrative & Diagnostic Service		*/
  /*Header's Chain by first gaining access to SSPCB list.  Gaining access to List	*/
  /*prevents this function from searching chain while chain being modified (via		*/
  /*Register SSP Service).  Also when List is acquired, Diag/Admin Poll function	*/
  /*and other Services are held off accessing SSPCB list until this function		*/
  /*completes										*/
while ( (mpa_fn_access_sspcb_lst ((unsigned long int FARm *)&svd_int_msk)) != 0xFF ) {} /*Spin till access granted*/

  /*Access to SSPCB list granted, SSPCB List is locked, Search for caller's SSPCB 	*/
for (i = SSPCB_LIST_MAX, sspcb_srch_ptr = admin_diag_hdr.sspcb_link; 
     (i != 0 )					     &&	 /*Srch SSPCB List for caller's	*/
     (sspcb_srch_ptr != (struct ssp_struct FARm *)0) &&	 /*SSPCB		 	*/
     (sspcb_srch_ptr != sspcb) ;			 /*NOTE: i = circular chain 	*/
     --i, sspcb_srch_ptr = sspcb_srch_ptr->sspcb_link ); /*	     safey check	*/
if (sspcb_srch_ptr != sspcb)				/*Caller's SSPCB in lst?	*/
{							/*If no, return error		*/
	mpa_fn_relse_sspcb_lst (svd_int_msk) ;		/*Release SSCP List		*/
	return (ERR_PNP_PROBE_SSPCB_BAD) ;		/*If no, exit			*/
}

  /*Caller's SSPCB found/valid and SSPCB List is Locked, set up & handle enable/disable	*/
  /*request										*/
if (probe_ena == 0xFF)					/*Enabling PnP Probing?		*/
{							/*If yes			*/
    /*Caller's Enabling SSPCB's PnP Probing, Go initialize each MPA's SLOTCB PnP Probe	*/
    /*state										*/
	if (sspcb->pnp_probe_ena != 0x00)		/*Probing already ENA?		*/
	{						/*If yes, return error		*/
		mpa_fn_relse_sspcb_lst (svd_int_msk) ;	/*Release SSCP List		*/
		return (ERR_PNP_PROBE_REDUND) ;
	}
    /*For caller's SSP assure each Modem Pool Slot's PnP area is initialized		*/
	for ( mpa_index = 0; mpa_index < ENGS_MPA; ++mpa_index )
	{						/*Loop through SSP's MPAs or Engs*/
		if (sspcb->mpa[mpa_index].base_chan == 0xFF ) /*Does MPA/ENG exist?	 */
			continue ;			      /*If no, chk nxt Ring postn*/
           /*MPA or Engine exists on Ring at mpa_index, assure PnP area initialized	 */
		num_slots = 				/*Get "current" Eng's # Slots	 */
		  ((sspcb->mpa[mpa_index].mpa_num_engs) == 1)  ? 16 :
		  (((sspcb->mpa[mpa_index].mpa_num_engs) == 2) ?  8 : 4 ) ;
		for (slot = sspcb->mpa[mpa_index].eng_head; /*Loop to init MPA's/Eng's	 */
		     num_slots != 0 ; 			    /* Slots PnP Probe Area	 */
		     --num_slots, ++slot )
		{
			slot->pnp_probe_state = 0x0000;	/*PnP Probe state = not init'd	 */
#ifndef 			NO_FN_PROTO_PARMS 		/*Compile if Protypes OK */
			slot->pnp_probe_funct = (void (*)(	/*PnP Probe Fn	 	 */
	      			     struct ssp_struct   FARm *,
	      			     struct slot_struct  FARm *,
	      			     struct mpa_global_s FARm *,
				     struct mpa_input_s  FARm *,
	      			     struct mpa_output_s FARm *))0 ;
#else								/*Compile if NO Protypes */
			slot->pnp_probe_funct = (void (*)())0 ;
#endif
#ifndef 				NO_FN_PROTO_PARMS 	/*Compile if Protypes OK */
			slot->pnp_clean_up   = (void (*)(struct slot_struct  FARm *,
	      			     			 unsigned char              ))0 ;
#else								/*Compile if NO Protypes */
			slot->pnp_clean_up   = (void (*)())0 ;
#endif
		}
	}
   /*All MPA (or Engine) PnP areas that are associated with caller's SSP have	*/
   /*been initialized, set up so that Administrative & Diagnostic Poll function	*/
   /*is notified that SSP's PnP Probing has been enabled			*/
	if ( admin_diag_hdr.pnp_probe != (unsigned short int)0xFF ) /*1st time	*/
	{							   /* PnP enabld*/
		admin_diag_hdr.pnp_probe     = 0xFF ;	/*Indicate at least 1 	*/
							/*SSP PnP Probe enabled	*/
		admin_diag_hdr.pnp_sspcb     = sspcb ;	/*Set SSPCB Probe ptr	*/
		admin_diag_hdr.pnp_eng_index = 0x00 ; 	/*Pt Probe to 1st 	*/
							/* possible MPA or Eng	*/
	}
	sspcb->pnp_probe_ena     = 0xFF ;		/*Allow SSP PnP Probing	*/
							/* on caller's SSP	*/
}						/*End of PnP Probe enable	*/
else
{
     /*Caller Disabling SSP's PnP Probing, Go Clean Up any outstanding PnP	*/
     /*Probes									*/
	if (sspcb->pnp_probe_ena == 0x00)		/*Probing alrdy DSA?	*/
	{						/*If yes, return error	*/
		mpa_fn_relse_sspcb_lst (svd_int_msk) ;	/*Release SSCP List	*/
		return (ERR_PNP_PROBE_REDUND) ;
	}
	for ( mpa_index = 0; mpa_index < ENGS_MPA; ++mpa_index )
	{						/*Loop through SSP's MPAs or Engs*/
		if (sspcb->mpa[mpa_index].base_chan == 0xFF ) /*Does MPA/ENG exist?	 */
			continue ;			      /*If no, chk nxt Ring postn*/
           /*MPA or Engine exists on Ring at mpa_index, cancel any outstandng PnP Probing*/
		num_slots = 				/*Get "current" Eng's # Slots	 */
		  ((sspcb->mpa[mpa_index].mpa_num_engs) == 1) ? 16 :
		  (((sspcb->mpa[mpa_index].mpa_num_engs) == 2) ?  8 : 4 ) ;
		for (slot = sspcb->mpa[mpa_index].eng_head; /*Loop to cancel MPA's/Eng's */
		     num_slots != 0 ;			    /* Slot PnP Probing		 */
		     --num_slots, ++slot )
		{
			if ( (slot->slot_state      == 0x00)   && /*Is slot empty &	 */
/*R10*/			     (slot->pnp_probe_state != 0x0000) && /*PnP Probe = active?  */
#ifndef 				NO_FN_PROTO_PARMS	  /*Compile if Protypes OK*/
     			     (slot->pnp_clean_up    != (void (*)(struct slot_struct FARm *, unsigned char))0) )
#else								  /*Compile if NO Protypes*/
     			     (slot->pnp_clean_up    != (void (*)())0) )
#endif
			{				   	/*If yes, cancel probe	 */
				(*slot->pnp_clean_up)(slot, (unsigned char)0xFF) ;
			}
			slot->pnp_probe_state = 0x0000 ;	/*Probe state=not init'd*/
		}
	}
	sspcb->pnp_probe_ena = 0x00 ;				/*Dsa PnP Probing on SSP */

     /*SSP's outstanding PnP Probes have been canceled, determine if any SSP still has 	 */
     /*PnP Probing enabled. If not, clear Administrative & Diagnostic Services Header's	 */
     /*PnP area such that Administrative & Diagnostic function is notified that SSP's 	 */
     /*PnP Probing is disabled							 	 */
	for ( silent_death_ctr = 0x200, sspcb_srch_ptr = admin_diag_hdr.sspcb_link ;
	     (sspcb_srch_ptr != (struct ssp_struct FARm *)0) &&	
	     (sspcb_srch_ptr->pnp_probe_ena == 0x00)	     &&
	     (silent_death_ctr != 0) 			     ;
     	     sspcb_srch_ptr = sspcb_srch_ptr->sspcb_link,
	     --silent_death_ctr				     ) ;
	if ( sspcb_srch_ptr == (struct ssp_struct FARm *)0 )	/*Any SSP have PnP enbled*/
	{							/*If no			 */
		admin_diag_hdr.pnp_probe     = 0x00 ;		/*Indicate no PnP Probing*/
		admin_diag_hdr.pnp_sspcb     = (struct ssp_struct FARm *)0 ;
		admin_diag_hdr.pnp_eng_index = 0x00 ;
	}
}						/*End of PnP Probe disable		 */

  /*Caller's SSP's PnP Probing Enable/Disable request complete, Unlock SSPCB List	 */
  /*and return to caller with good status						 */
mpa_fn_relse_sspcb_lst (svd_int_msk) ;		/*Unlock SSPCB List			 */
return (0x00) ;

}						/*End of EAN/DSA PnP Probing Fn	*/







/*#stitle ADMIN/DIAG SERVICE - ADMINISTRATIVE/DIAGNOSTIC POLL (mpa_srvc_diag_poll)#*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *                        ADMINSTRATIVE/DIAGNOSTIC POLL			       *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
extern unsigned short int	mpa_srvc_diag_poll (void) 
#else
extern unsigned short int	mpa_srvc_diag_poll () 
#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

/*Plug and Play Probing variables							*/
unsigned long int	svd_int_msk ;		/*Int Msk prior to Dsabl		*/
struct ssp_struct FARm	*sspcb_srch_ptr ;	/*Ptr to SSP to Probe for PnP		*/
unsigned int		srch_index ;		/*Index to SSP's MPA/Eng. to Probe	*/
unsigned char		mpa_found ;		/*Flag indicatg if found an MPA to Probe*/
						/*  0x00 = Did not find MPA to probe	*/
						/*  0xFF = Found MPA/Eng to probe	*/
struct ssp_struct FARm	*probe_sspcb ;		/*Ptr to SSP to Probe for PnP		*/
unsigned int		probe_index ;		/*Index to SSP's MPA/Eng. to Probe	*/
unsigned int		num_slots_probe ;	/*Number of Slots to Probe		*/
struct slot_struct FARm	*slot ;			/*Pointer to Slot to Probe		*/

/*SSP Channel/Slot Variables								*/
#ifndef					RAMP_BSDI		/*R6*/
struct mpa_global_s FARm *  gregs ;		/*SSP's Global registers		*/
struct mpa_input_s  FARm *  iregs ;		/*Slot's Input Registers		*/
struct mpa_output_s FARm *  oregs ;		/*Slot's Output Regsiters		*/
struct cin_bank_struct FARm *actv_bank ;	/*Slot's active bank pointer		*/ /*R2*/
#else								/*BSDI Compile		  */
volatile struct mpa_global_s FARm *  gregs ;	/*SSP's Global registers		*/
volatile struct mpa_input_s  FARm *  iregs ;	/*Slot's Input Registers		*/
volatile struct mpa_output_s FARm *  oregs ;	/*Slot's Output Regsiters		*/
volatile struct cin_bank_struct FARm *actv_bank; /*Slot's active bank pointer		*/ /*R2*/
#endif								/*End RAMP_BSDI Conditional*/

/*Variables use to MAP/UNMAP SSP Registers into memory					*/
void FARm		*unmap_parm ;		/*Parameter used by UNMAP function	*/

/*Variables used in accessing SSP registers						*/
unsigned short int	ssp_ushort_var ;	/*Used to modify 16 bit	SSP registers	*/

/*General Working Variables								*/
unsigned int		infinite_chk ;		/*Loop counter to guard against		*/
						/*infinite/circular SSPCB chain		*/




/************************
 *   START OF CODE	*
 ************************/

/*Assure that System Functions have been defined						*/
if ( admin_diag_hdr.init_flag != 0xFFFF )	/*Have Fn addrs been spec'd?			*/
	return (ERR_DIAG_POLL_NO_FNS) ;		/*If no, error					*/

/*Set up and determine if need to (or can) perform Plug and Play Probing			*/
if ( admin_diag_hdr.pnp_probe != 0xFF )		/*Is PnP Probing enabled on at least one SSP	*/
	goto  admin_diag_pnp_poll_cmplt ;	/*If no, continue other Poll activities		*/
if ( mpa_fn_access_sspcb_lst ((unsigned long int FARm *)&svd_int_msk) == 0x00 ) /*Did gain access to SSPCB list? 		*/
	goto  admin_diag_pnp_poll_cmplt ;	      /*If no, cont Poll activities  		*/

/*Some SSP's PnP Probe is enabled and have gained access (and locked) SSPCB List search for	*/
/*SSPCB and MPA to Probe									*/
infinite_chk    = SSPCB_LIST_MAX ;		/*Init infinite chain safety counter		*/
mpa_found 	= 0x00 ;			/*Set MPA found flag = not found		*/
sspcb_srch_ptr  = admin_diag_hdr.pnp_sspcb ;	/*Get ptr to SSPCB to search			*/
srch_index      = admin_diag_hdr.pnp_eng_index;	/*Get SSPCB's MPA array index			*/
do
{
	probe_sspcb = sspcb_srch_ptr ;		/*Anticapte finding SSPCB and MPA/ENG to Probe	*/
	probe_index = srch_index ;
	if ( ++srch_index >= ENGS_MPA )		/*Pt to nxt SSP/MPA to Probe			*/
	{
		srch_index = 0 ;
		sspcb_srch_ptr = (sspcb_srch_ptr->sspcb_link != (struct ssp_struct FARm *)0) ?
			          sspcb_srch_ptr->sspcb_link : admin_diag_hdr.sspcb_link ;
	}
	if ( (probe_sspcb->pnp_probe_ena == 0xFF) 	       && /*Is PnP Probing enabled	*/
	     (probe_sspcb->mpa[probe_index].base_chan != 0xFF)  ) /* and does MPA exist?	*/
	{							  /*If found MPA to probe	*/
		mpa_found = 0xFF ;				  /*Indicate found MPA		*/
		break ;
	}
} while ( ( (sspcb_srch_ptr != admin_diag_hdr.pnp_sspcb)   ||     /*End of srch for MPA to Probe*/
	    (srch_index     != admin_diag_hdr.pnp_eng_index) ) &&
	  (--infinite_chk != 0x00)			   ) ;


  /*Search for SSP and MPA/Eng to Probe complete, determine if found an MPA/Eng to search	*/
  /*NOTE: mpa_found      = Flag indicating if able to identify SSP/MPA to Probe for Plug & Play	*/
  /*				0x00 = NO SSP/MPA found						*/
  /*				0xFF = identified SSP/MPA to Probe for Plug and Play boards	*/
  /*	sspcb_srch_ptr = Pointer to next SSP to probe when Poll function called again		*/
  /*	srch_index     = Index (in SSP's MPA array) to nxt MPA to Probe when Poll called again	*/
  /*	probe_sspcb    = Pointer to SSP to Probe for PnP					*/
  /*	probe_index    = Index (in SSP's MPA array) to MPA to Probe for PnP boards		*/
if ( mpa_found != 0xFF )				/*Did find an MPA/Eng to Probe?		*/
	goto  admin_diag_pnp_no_probe ;			/*If no, continue Admin/Diag Poll  	*/
admin_diag_hdr.pnp_sspcb     = sspcb_srch_ptr ;		/*Sv SSPCB search ptr for nxt Poll	*/
admin_diag_hdr.pnp_eng_index = srch_index ;		/*Sv SSPCB's MPA array index	  	*/


  /*Found MPA/Engine to perform Plug and Play Probing on, if appropriate MAP SSP Registers	*/
  /*NOTE: probe_sspcb = Pointer to SSP to Probe for PnP						*/
  /*	  probe_index = Index (in SSP's MPA array) to MPA to Probe for PnP boards		*/
#ifndef 	NO_FN_PROTO_PARMS 			/*Compile if Protypes OK		*/
if ( probe_sspcb->map_fn != (void FARm *(*)(void FARm *))0 ) /*Are SSP registers paged?		*/
#else							/*Compile if NO Protypes		*/
if ( probe_sspcb->map_fn != (void FARm *(*)())0 )	     /*Are SSP registers paged?		*/
#endif
{							/*If SSP register page, go map in regs	*/
	unmap_parm = (*probe_sspcb->map_fn)(probe_sspcb->map_parm) ; /*Go MAP in SSP registers	*/
}

  /*If required, SSP MAPPed into memory, loop through all slots performing next "state machine"	*/
  /*function											*/
gregs 		= probe_sspcb->global ;			/*Point to SSP's Global regsiters	*/
num_slots_probe = ((probe_sspcb->mpa[probe_index].mpa_num_engs) == 1) ? 16 :
		 (((probe_sspcb->mpa[probe_index].mpa_num_engs) == 2) ?  8 : 4 ) ;

for (slot = probe_sspcb->mpa[probe_index].eng_head;	/*Loop thru all slot's PnP Probe Functs	*/
     num_slots_probe != 0 ;
     --num_slots_probe, ++slot )
{
	iregs     = slot->input ;			/*Set ptr to channel's Input registers	*/
/*R2*/	actv_bank = ( (iregs->cin_locks & 0x01) == 0x00 ) 		  ? /*Is Bank A active?	*/
		      (struct cin_bank_struct FARm *)&(iregs->cin_bank_a) : /*If yes	  	*/
		      (struct cin_bank_struct FARm *)&(iregs->cin_bank_b) ; /*If no	  	*/
	oregs 	  = slot->output ;			/*Set ptr to channel's Output registers	*/
/*R4*/	slot->power_state = ( ((actv_bank->cin_bank_signals) & 0x0800) == 0x0000 )  ?
		(unsigned short int)0x0000 : (unsigned short int)0x00FF 	    ;
	if ( slot->slot_state != 0x00 )			/*Is Slot empty?			*/
		continue ;				/*If no, chk PnP Probing on next slot	*/
	if ( slot->pnp_probe_state == 0x0000 )		/*Has Slot been init'd for PnP Probing?	*/
	{						/*If no, initialize PnP state Function	*/

/*R2*/	   /*First time Slot is being Probed for Plug and Play Board - chk if board exists and	*/
	   /*if so assume it is a non-PnP board that has yet to be registered.  Therefore do not*/
	   /*initiate PnP Probe.								*/
/*R2*/		if ( (actv_bank->cin_bank_signals & 0x0001) == 0 ) /*Is Brd present?		*/
			continue ;				   /*If yes, assume unreg'd	*/
								   /*unreg'd non-PNP board	*/
	   /*No board present in Slot - save state of SSP channel registers and initialize 	*/
	   /*registers so can start PnP Probe and isuue I/O to board etc.			*/
		slot->pnp_sv_cout_flow_config = oregs->cout_flow_config ;
		slot->pnp_sv_cout_cntrl_sig   = oregs->cout_cntrl_sig ;
		slot->pnp_sv_cout_xon_1       = oregs->cout_xon_1 ;
#ifndef 			NO_FN_PROTO_PARMS			/*Compile if Protypes OK*/
		slot->pnp_clean_up            = (void (*)(struct slot_struct FARm *, unsigned char))&pnp_clean_up_ssp ;
#else									/*Compile if NO Protypes*/
		slot->pnp_clean_up            = (void (*)())pnp_clean_up_ssp ;
#endif
		  /*	*** Transmint & Receive char attributes (8,N,1)				*/
		ssp_ushort_var  = (iregs->cin_char_cntrl & 0xFFE0) ;	/*Set up SSP Input to 	*/
		ssp_ushort_var |= 0x03 ;				/*recv 8 bits, no parity*/
		iregs->cin_char_cntrl = ssp_ushort_var ;
		oregs->cout_char_fmt = 0x03 ;				/*SSP output = 8, none	*/
		  /*	*** Transmint & Receive Baud Rate = Top rate				*/
		iregs->cin_baud = 0x7FFF ;				/*Input baud rate = TOP	*/
		oregs->cout_baud_rate = 0x7FFF ;			/*Output baud rate = TOP*/
		while ( 1 == 1 )					/*Loop to reset SSP 	*/
		{							/* internal baud ctrs	*/
			while ( gregs->gicp_chan == slot->chan_num ) {}	/*  assures baud setting*/
			iregs->cin_baud_ctr &= 0xC000 ;			/*   takes affect imedly*/
			oregs->cout_intnl_baud_ctr = 0 ;
			while ( gregs->gicp_chan == slot->chan_num ) {}	
			if ( ((iregs->cin_baud_ctr & 0x3FFF) == 0) &&
		              (oregs->cout_intnl_baud_ctr    == 0) )
					break ;
		}
		  /*	*** Set Xtra DMA (since running at TOP baud rate) & Send XON Char	*/
		oregs->cout_flow_config = (oregs->cout_flow_config & 0x10) | 0x40 | 0x08 ;
		  /*	*** Flow Control, MPA-to-SSP = "MUX not Connected"			*/
		  /*	                  SSP-to-MPA = Output Cntrl sig 3, 			*/
		iregs->cin_susp_output_lmx |= 0x20 ;			/*MUX not Connected	*/
		iregs->cin_q_cntrl |= 0x20 ;				/*Set Input Flow Control*/
		  /*	*** Clear internal/external Loop backs					*/
		oregs->cout_lmx_command &= 0xFC ;			/*Clear loop backs	*/
		  /*    *** Set up to call 1st PnP Probe function				*/
		slot->pnp_probe_funct = pnp_probe_start ;
		slot->pnp_probe_state = 0x0080 ;		/*Indicate PnP Probing initiated*/
	}

   /*Continue Slot's PnP Probe by calling PnP Probe function					*/
	(*slot->pnp_probe_funct)(				/*Call Slot's PnP Probe Function*/
				 probe_sspcb,			/*  - Probe Slot's SSPCB	*/
				 slot,				/*  - Probe Slot's SLOTCB	*/
				 gregs,				/*  - Probe Slot's Global regs	*/
				 iregs,				/*  - Probe Slot's Input regs	*/
				 oregs				/*  - Probe Slot's Output regs	*/
				) ;
}								/*End of for loop Probing MPA	*/
								/* Slot's for PnP boards	*/

/*Plug and Play Probing complete, UNMAP SSP if neccessary					*/
#ifndef NO_FN_PROTO_PARMS 				/*Compile if Protypes OK		*/
if ( probe_sspcb->map_fn != (void FARm *(*)(void FARm *))0 ) /*Are SSP registers paged?		*/
#else							/*Compile if NO Protypes		*/
if ( probe_sspcb->map_fn != (void FARm *(*)())0 )	     /*Are SSP registers paged?		*/
#endif
{							/*If SSP register page, go map in regs	*/
	(*probe_sspcb->unmap_fn)(unmap_parm) ; 		/*Go UNMAP in SSP registers		*/
}


/*Unlock sspcb list and perform next Administrative/Diagnostic Poll function			*/
admin_diag_pnp_no_probe:
mpa_fn_relse_sspcb_lst (svd_int_msk) ;			/*Unlock SSPCB List			*/

/*Plug and Play Probing complete, perform next Admin/Diag activity				*/
admin_diag_pnp_poll_cmplt:				/*Pt when PnP Polling complete		*/

/*No more Administrative/Diagnostic activities to perform, return to caller with good status	*/
return (0x00) ;

}						/*End of mpa_srvc_diag_poll Funct*/







/*#stitle ADMIN/DIAG SERVICE - PnP PROBE FUNCTIONS#*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *              INITIAL PnP PROBE FUNCTION - ISSUE HARD RESET TO SLOT	       *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_start (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
     			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_start (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb       = sspcb ;
iregs       = iregs ;

/*Set slot's/channel's control signals to indicate to MPA to Hard Reset Modem Board (if exists)				*/
/*Note: Hard Reset causes any fully configured PnP board to reset and enter "Wait for Key" state thus MPA no longer	*/
/*	recognizes board.  Also, Hard Reset clears AUX Reg's PnP bit and Read Toggle bit.  As result of PnP bit	being	*/
/*	cleared Modem Pool will recognize a non-Plug and Play board.  Hence Hard Reset provides a "window" in Plug and	*/
/*	Play probe cycle where non-plug and play boards can be recognized.						*/
oregs->cout_cntrl_sig = 0x1CC ;				/*Assert ISA Reset to Reset Modem Board (if exists)		*/
slot->pnp_frame_ctr   = gregs->gicp_frame_ctr ;		/*Snap shot for timing						*/
slot->read_toggle     = 0x00 ;				/*UART I/O Read toggle						*/

/*Exit function & delay 8 frames thus allowing time for MPA to "see" Modem Reset					*/
slot->pnp_probe_funct = pnp_probe_fn_02 ;
return ;

}	/*END of PnP Hard Reset Modem - 1st function, pnp_probe_start							*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *      2nd PnP PROBE FN - HARD RESET COMPLETE, INITIATE WAIT FOR KEY	       *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_02 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_02 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;
oregs	   = oregs ;

/*Delay 8 frames - allow time for MPA to "see" Hard Reset								*/
if ( pnp_frame_delay_chk ( (unsigned short int)0x08, gregs, slot ) == 0 ) /*Has 8 frames elapsed?			*/
	return ;							  /*If no, continue probe			*/

/*Assure delay of at least 2 millisecond to allow time for PnP board to initialize (see Plug and Play spec. Ver 1.0,	*/
/*5/28/93, p. 19).  If PnP Board present it is now in "Wait for Key" State						*/
oregs->cout_cntrl_sig   = (slot->pnp_sv_cout_cntrl_sig & 0x0237) | 0x0004 ; /*Restore control signals			*/
slot->pnp_frame_ctr     = gregs->gicp_frame_ctr ;			    /*Snap shot for timing			*/
slot->pnp_probe_funct   = pnp_probe_fn_03 ;
return ;

}	/*END of PnP - 2nd function, pnp_probe_fn_02									*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *      	 3rd PnP PROBE FN - Plug and PLAY BOARD RESET DELAY	       *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_03 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_03 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Delay 16 frames thus assuring at least 2 millisecond delay (allow time for PnP board to init after Hard Reset		*/
if ( pnp_frame_delay_chk ( (unsigned short int)0x10, gregs, slot ) == 0 ) /*Has 10 frames elapsed?			*/
	return ;							  /*If no, continue probe			*/

  /*Have delay long enough for PnP board to Reset. Note: If PnP board present, Hard Reset places PnP board in "Wait for	*/
  /*Key" state (see Plug and Play spec. Version 1.0, 5/28/93, p. 15).  In order to place PnP Board into "Sleep" State,	*/
  /*need to send Initiation Key, 34 OUTs to PnP "Address Port" @ I/O addr 0x279 (see PnP spec. pages 13, 18, and 53)	*/
  /*The steps required by the Modem Pool to do this are as follows (note: each step is a seperate function)		*/
  /*    1. In order to address MPA Slot "AUX register", Set chan's control signals to 0x199				*/
  /*    2. Set PnP I/O bit in Aux register by xmit'g 0x20								*/
  /*    3. In order to address PnP "Address Port" (i.e. I/O addr 0x279), Set chan's control signals to 0x111.		*/
  /*    4. Sending Initiation Key (34 Output bytes to PnP Address Port)							*/
oregs->cout_cntrl_sig = 0x199 ;						/*Addr MPA's AUX reg				*/
slot->pnp_frame_ctr   = gregs->gicp_frame_ctr ;				/*Snap shot for timing				*/
slot->pnp_probe_funct = pnp_probe_fn_04 ;
return ;

}	/*End pnp_probe_fn_03												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 *      	 4th PnP PROBE FN - Wait till AUX Reg addressed		       	*
 *                                                                             	*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_04 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_04 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Delay 4 frames - allow time for MPA to see AUX Reg being addressed							*/
if ( pnp_frame_delay_chk ( (unsigned short int)0x04, gregs, slot ) == 0 ) /*Has 4 frames elapsed?			*/
	return ;							  /*If no, continue probe			*/

/*Have delayed long enough for MPA to "see" control signal setting indicating that output be directed to MPA's AUX Reg,	*/
/*write to AUX register setting PnP I/O bit										*/
oregs->cout_xon_1 	  = 0x20 ;					/*Set AUX reg PnP I/O bit			*/
oregs->cout_flow_config  ^= 0x10 ;					/*Initiate xmit'g char				*/
slot->pnp_frame_ctr	  = gregs->gicp_frame_ctr ;			/*Snap shot for timing				*/
slot->pnp_probe_funct     = pnp_probe_fn_05 ;
return ;

}	/*End pnp_probe_fn_04												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 *      	 5th PnP PROBE FN - Wait till AUX Reg Receives PnP bit		*
 *                                                                             	*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_05 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_05 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Wait for AUX Reg byte (with PnP bit set) to be sent									*/
if ( ((oregs->cout_flow_config & 0x10) != (oregs->cout_intnl_flow_ctrl & 0x10)) ||
     ((oregs->cout_intnl_flow_ctrl & 0x08) == 0x08) )			/*Is AUX reg char xmit done?			*/
{									/*If no, check deadman				*/
	if ( pnp_frame_delay_chk (DEADMAN_DELAY, gregs, slot) == 0x00 )	/*Has deadman time expired?			*/
	{								/*If no, wait & check again later		*/
		return ;
	}								/*If deadman expired, Modem Pool not functioning*/
	pnp_clean_up_ssp (slot, (unsigned char)0xFF) ;			/*Go clean up so can start over			*/
	return ;
}

/*MPA's AUX Reg PnP bit set, start sending Initiation Key by first addressing PnP's Address Port (0x279)		*/
oregs->cout_cntrl_sig = 0x111 ;						/*Pt MPA to PnP's Address port (0x279)		*/
slot->pnp_frame_ctr   = gregs->gicp_frame_ctr ;				/*Snap shot for timing				*/
slot->pnp_probe_funct = pnp_probe_fn_06 ;
return ;

}	/*End pnp_probe_fn_05												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 *           6th PnP PROBE FN - Wait till PnP Address Port addressed		*
 *                                                                             	*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_06 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_06 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Delay 4 frames - allow time for MPA to see PnP Address Port being addressed						*/
if ( pnp_frame_delay_chk ( (unsigned short int)0x04, gregs, slot ) == 0 ) /*Has 4 frames elapsed?			*/
	return ;							  /*If no, continue probe			*/

/*Have delayed long enough for MPA to "see" control signal setting indicating that output be directed to PnP's Address	*/
/*Port, Start sending PnP Initiation Key (see Plug and Play spec. Version 1.0, 5/28/93, p. 18 and 53)			*/
slot->pnp_probe_parm      = 0x00 ;					/*Initiate parm to strt of PnP Init. Key sequnce*/
oregs->cout_xon_1 	  = pnp_init_seq [slot->pnp_probe_parm++] ;	/*Set AUX reg PnP I/O bit			*/
oregs->cout_flow_config  ^= 0x10 ;					/*Initiate xmit'g char				*/
slot->pnp_frame_ctr       = gregs->gicp_frame_ctr ;			/*Snap shot for timing				*/
slot->pnp_probe_funct     = pnp_probe_fn_07 ;
return ;

}	/*End pnp_probe_fn_06												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 *   7th PnP PROBE FN - Send PnP Initiation Key Sequence to PnP Address Port	*
 *                                                                             	*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_07 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_07 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Wait for a byte of PnP Initiation Key to be sent (then send next byte)						*/
if ( ((oregs->cout_flow_config & 0x10) != (oregs->cout_intnl_flow_ctrl & 0x10)) ||
     ((oregs->cout_intnl_flow_ctrl & 0x08) == 0x08) )			/*Has PnP Initiation Key byte been sent?	*/
{									/*If no, check deadman				*/
	if ( pnp_frame_delay_chk (DEADMAN_DELAY, gregs, slot) == 0x00 )	/*Has deadman time expired?			*/
	{								/*If no, wait & check again later		*/
		return ;
	}								/*If deadman expired, Modem Pool not functioning*/
	pnp_clean_up_ssp (slot, (unsigned char)0xFF) ;			/*Go clean up so can start over			*/
	return ;
}

/*PnP Initiation Key Sequence byte has been sent, check if sent entire byte sequence sent and if not send next byte	*/
if ( slot->pnp_probe_parm < sizeof(pnp_init_seq) )			/*Have completed PnP Initiation Sequence?	*/
{									/*If no, send next byte in sequence		*/
	oregs->cout_xon_1 = pnp_init_seq [slot->pnp_probe_parm++] ;		/*Set AUX reg PnP I/O bit		*/
	oregs->cout_flow_config  ^= 0x10 ;					/*Initiate xmit'g char			*/
	slot->pnp_frame_ctr = gregs->gicp_frame_ctr ;				/*Snap shot for timing			*/
	return ;
}

/*PnP Initiation Sequence sent, If PnP Board present it is now in "Sleep" State, (see Plug and Play spec. Ver 1.0, 	*/
/*5/28/93, p. 18).  Now, clear AUX Register's PnP bit thus providing Modem Pool time to see if a non-PnP Modem Board	*/
/* has been inserted into slot												*/
oregs->cout_cntrl_sig = 0x199 ;						/*Addr MPA's AUX reg				*/
slot->pnp_frame_ctr   = gregs->gicp_frame_ctr ;				/*Snap shot for timing				*/
slot->pnp_probe_funct = pnp_probe_fn_08 ;
return ;

}	/*End pnp_probe_fn_07												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 *    8th PnP PROBE FN - Wait till AUX Reg addressed (clear PnP Port access)	*
 *                                                                             	*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_08 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_08 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Delay 4 frames - allow time for MPA to see AUX Reg being addressed							*/
if ( pnp_frame_delay_chk ( (unsigned short int)0x04, gregs, slot ) == 0 ) /*Has 4 frames elapsed?			*/
	return ;							  /*If no, continue probe			*/

/*Have delayed long enough for MPA to "see" control signal setting indicating that output be directed to MPA's AUX Reg,	*/
/*write to AUX register clearing PnP I/O bit										*/
oregs->cout_xon_1	  = 0x00 ;					/*Clear AUX reg PnP I/O bit			*/
oregs->cout_flow_config  ^= 0x10 ;					/*Initiate xmit'g char				*/
slot->pnp_frame_ctr       = gregs->gicp_frame_ctr ;			/*Snap shot for timing				*/
slot->pnp_probe_funct 	  = pnp_probe_fn_09 ;
return ;

}	/*End pnp_probe_fn_08												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 *     		9th PnP PROBE FN - Wait till AUX Reg Receives PnP bit clear	*
 *                                                                             	*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_09 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_09 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Wait for AUX Reg byte (with PnP bit cleared) to be sent								*/
if ( ((oregs->cout_flow_config & 0x10) != (oregs->cout_intnl_flow_ctrl & 0x10)) ||
     ((oregs->cout_intnl_flow_ctrl & 0x08) == 0x08) )			/*Is AUX reg char xmit done?			*/
{									/*If no, check deadman				*/
	if ( pnp_frame_delay_chk (DEADMAN_DELAY, gregs, slot) == 0x00 )	/*Has deadman time expired?			*/
	{								/*If no, wait & check again later		*/
		return ;
	}								/*If deadman expired, Modem Pool not functioning*/
	pnp_clean_up_ssp (slot, (unsigned char)0xFF) ;			/*Go clean up so can start over			*/
	return ;
}

/*MPA's AUX Reg PnP bit cleared, delay thus allowing Modem Pool time to recognize if a non-PnP Modem is present		*/
oregs->cout_cntrl_sig = (slot->pnp_sv_cout_cntrl_sig & 0x0233) ; /*R5*/	/*Set Control Signals back to default		*/
									/*Note: MPA Input Flow Control bit off thus	*/
									/*	stopping MPA from sending any input 	*/
									/*	chars.  This is done here since this is	*/
									/*	last time UART control signals are 	*/
									/*	changed prior to readg Serial Identifier*/
									/*	required to isolate board.  See fns	*/
									/*	starting at pnp_probe_fn_24		*/
slot->pnp_frame_ctr   = gregs->gicp_frame_ctr ;				/*Snap shot so can delay			*/
slot->pnp_probe_funct = pnp_probe_fn_10 ;
return ;

}	/*End pnp_probe_fn_09												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 *  10th PnP PROBE FN - Delay to allow MPA time to recognize a non-PnP Modem	*
 *                                                                             	*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_10 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_10 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Delay 20 frames - allow MPA time to recognize a non-PnP Modem								*/
if ( pnp_frame_delay_chk ( (unsigned short int)20, gregs, slot ) == 0 )   /*Has 20 frames elapsed?			*/
	return ;							  /*If no, continue probe			*/

/*Have given enough of a window for Modem Pool to recognize a non-PnP Modem, set up so can address PnP ports (again) by	*/
/*setting Modem Pool AUX register's PnP bit										*/
oregs->cout_cntrl_sig = 0x199 ;						/*Addr MPA's AUX reg				*/
slot->pnp_frame_ctr   = gregs->gicp_frame_ctr ;				/*Snap shot for timing				*/
slot->pnp_probe_funct = pnp_probe_fn_11 ;
return ;

}	/*End pnp_probe_fn_10												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 *      	 11th PnP PROBE FN - Wait till AUX Reg addressed		*
 *                                                                             	*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_11 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_11 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Delay 4 frames - allow time for MPA to see AUX Reg being addressed							*/
if ( pnp_frame_delay_chk ( (unsigned short int)0x04, gregs, slot ) == 0 ) /*Has 4 frames elapsed?			*/
	return ;							  /*If no, continue probe			*/

/*Have delayed long enough for MPA to "see" control signal setting indicating that output be directed to MPA's AUX Reg,	*/
/*write to AUX register setting PnP I/O bit										*/
oregs->cout_xon_1 = 0x20 ;						/*Set AUX reg PnP I/O bit			*/
oregs->cout_flow_config  ^= 0x10 ;					/*Initiate xmit'g char				*/
slot->pnp_frame_ctr       = gregs->gicp_frame_ctr ;			/*Snap shot for timing				*/
slot->pnp_probe_funct     = pnp_probe_fn_12 ;
return ;

}	/*End pnp_probe_fn_11												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 *      	 12th PnP PROBE FN - Wait till AUX Reg Receives PnP bit		*
 *                                                                             	*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_12 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_12 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Wait for AUX Reg byte (with PnP bit set) to be sent									*/
if ( ((oregs->cout_flow_config & 0x10) != (oregs->cout_intnl_flow_ctrl & 0x10)) ||
     ((oregs->cout_intnl_flow_ctrl & 0x08) == 0x08) )			/*Is AUX reg char xmit done?			*/
{									/*If no, check deadman				*/
	if ( pnp_frame_delay_chk (DEADMAN_DELAY, gregs, slot) == 0x00 )	/*Has deadman time expired?			*/
	{								/*If no, wait & check again later		*/
		return ;
	}								/*If deadman expired, Modem Pool not functioning*/
	pnp_clean_up_ssp (slot, (unsigned char)0xFF) ;			/*Go clean up so can start over			*/
	return ;
}

/*MPA's AUX Reg PnP bit set and if PnP Board present it is in "Sleep" State, Reset PnP Card Select Number (CSN) by	*/
/*writing 0x04 to PnP Configuration Control reg (PnP reg @ 0x02). See Plug and Play spec. Version 1.0, 5/28/93, p. 45 )	*/
oregs->cout_cntrl_sig = 0x111 ;						/*Pt MPA to PnP's Address port (0x279)		*/
slot->pnp_frame_ctr   = gregs->gicp_frame_ctr ;				/*Snap shot for timing				*/
slot->pnp_probe_funct = pnp_probe_fn_13 ;
return ;

}	/*End pnp_probe_fn_12												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 *           13th PnP PROBE FN - Wait till PnP Address Port addressed		*
 *                                                                             	*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_13 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_13 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Delay 4 frames - allow time for MPA to see PnP Address Port being addressed						*/
if ( pnp_frame_delay_chk ( (unsigned short int)0x04, gregs, slot ) == 0 ) /*Has 4 frames elapsed?			*/
	return ;							  /*If no, continue probe			*/

/*Have delayed long enough for MPA to "see" control signal setting indicating that output be directed to PnP's Address	*/
/*Port, now point to PnP Control Register 0x02 (Config Control) by writing 0x02 to PnP Address Port			*/
oregs->cout_xon_1 	  = 0x02 ;					/*Send PnP Control Register # to PnP Addr Port	*/
oregs->cout_flow_config  ^= 0x10 ;					/*Initiate xmit'g char				*/
slot->pnp_frame_ctr       = gregs->gicp_frame_ctr ;			/*Snap shot for timing				*/
slot->pnp_probe_funct     = pnp_probe_fn_14 ;
return ;

}	/*End pnp_probe_fn_13												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 *  14th PnP PROBE FN - Set up to send Reset CSN to PnP Control Register 0x02	*
 *                                                                             	*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_14 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_14 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Wait till PnP Config Control register # been sent to PnP Address Port							*/
if ( ((oregs->cout_flow_config & 0x10) != (oregs->cout_intnl_flow_ctrl & 0x10)) ||
     ((oregs->cout_intnl_flow_ctrl & 0x08) == 0x08) )			/*Has PnP Config Control reg # been sent?	*/
{									/*If no, check deadman				*/
	if ( pnp_frame_delay_chk (DEADMAN_DELAY, gregs, slot) == 0x00 )	/*Has deadman time expired?			*/
	{								/*If no, wait & check again later		*/
		return ;
	}								/*If deadman expired, Modem Pool not functioning*/
	pnp_clean_up_ssp (slot, (unsigned char)0xFF) ;			/*Go clean up so can start over			*/
	return ;
}

/*PnP Address register now points to PnP Control Register 0x02, now set control signals to address PnP's Write Data Port*/
oregs->cout_cntrl_sig = 0x155 ;						/*Pt MPA to PnP's Write Data Port (0xA79)	*/
slot->pnp_frame_ctr   = gregs->gicp_frame_ctr ;				/*Snap shot for timing				*/
slot->pnp_probe_funct = pnp_probe_fn_15 ;
return ;

}	/*End pnp_probe_fn_14												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 *           15th PnP PROBE FN - Wait till PnP Write Data Port addressed	*
 *                                                                             	*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_15 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_15 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Delay 4 frames - allow time for MPA to see PnP Write Data Port being addressed					*/
if ( pnp_frame_delay_chk ( (unsigned short int)0x04, gregs, slot ) == 0 ) /*Has 4 frames elapsed?			*/
	return ;							  /*If no, continue probe			*/

/*Have delayed long enough for MPA to "see" control signal setting indicating that output be directed to PnP's Write	*/
/*Data Port, by writing a 0x04 to Write Data Port - PnP's Configuration Control register bit 2 is set thus causing PnP	*/
/*Board to zero its Card Select Number (CSN)										*/
oregs->cout_xon_1 	  = 0x04 ;					/*Send PnP Reset CSN bit/command		*/
oregs->cout_flow_config  ^= 0x10 ;					/*Initiate xmit'g char				*/
slot->pnp_frame_ctr       = gregs->gicp_frame_ctr ;			/*Snap shot for timing				*/
slot->pnp_probe_funct     = pnp_probe_fn_16 ;
return ;

}	/*End pnp_probe_fn_15												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 * 16th PnP PROBE FN - Set up to Send Wake command via PnP Control Register 0x03*
 *                                                                             	*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_16 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_16 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Wait for Reset CSN bit/command to be sent (i.e. 0x04 sent to PnP Control Register 0x02)				*/
if ( ((oregs->cout_flow_config & 0x10) != (oregs->cout_intnl_flow_ctrl & 0x10)) ||
     ((oregs->cout_intnl_flow_ctrl & 0x08) == 0x08) )			/*Has Reset CSN Command bit been sent?		*/
{									/*If no, check deadman				*/
	if ( pnp_frame_delay_chk (DEADMAN_DELAY, gregs, slot) == 0x00 )	/*Has deadman time expired?			*/
	{								/*If no, wait & check again later		*/
		return ;
	}								/*If deadman expired, Modem Pool not functioning*/
	pnp_clean_up_ssp (slot, (unsigned char)0xFF) ;			/*Go clean up so can start over			*/
	return ;
}

  /*PnP Board's CSN now reset (CSN = 0) and Board still in "Sleep" sate, set up to place board in "Isolation" state by	*/
  /*writing a zero (0) to PnP Control register 0x03 (Wake register).  See Plug and Play spec. Version 1.0, 5/28/93,	*/
  /*pages 13, 15 & 46).  The steps required by the Modem Pool to do this are as follows (note: each step is a seperate	*/
  /* function):														*/
  /*    1. Set chan's control signals to 0x111 (address PnP Address Port)	 					*/
  /*    2. Output 0x03, writes 0x03 to PnP Address Port (indicates to PnP board that write_data addresses Wake register)*/
  /*    3. Set chan's control signals to 0x155 (address PnP Write Data Port)	 					*/
  /*    4. Output 0x00, writes 0x00 to PnP Wake register (causes PnP board to enter isolation state)			*/
oregs->cout_cntrl_sig = 0x111 ;						/*Pt MPA to PnP's Address port (0x279)		*/
slot->pnp_frame_ctr   = gregs->gicp_frame_ctr ;				/*Snap shot for timing				*/
slot->pnp_probe_funct = pnp_probe_fn_17 ;
return ;

}	/*End pnp_probe_fn_16												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 *           17th PnP PROBE FN - Wait till PnP Address Port addressed		*
 *                                                                             	*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_17 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_17 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Delay 4 frames - allow time for MPA to see PnP Address Port being addressed						*/
if ( pnp_frame_delay_chk ( (unsigned short int)0x04, gregs, slot ) == 0 ) /*Has 4 frames elapsed?			*/
	return ;							  /*If no, continue probe			*/

/*Have delayed long enough for MPA to "see" control signal setting indicating that output be directed to PnP's Address	*/
/*Port, now point to PnP Control Register 0x03 (Wake) by writing 0x03 to PnP Address Port				*/
oregs->cout_xon_1 	  = 0x03 ;					/*Send PnP Control Register # to PnP Addr Port	*/
oregs->cout_flow_config  ^= 0x10 ;					/*Initiate xmit'g char				*/
slot->pnp_frame_ctr       = gregs->gicp_frame_ctr ;			/*Snap shot for timing				*/
slot->pnp_probe_funct     = pnp_probe_fn_18 ;
return ;

}	/*End pnp_probe_fn_17												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 *  		18th PnP PROBE FN - Set up to address PnP Write Data Port 	*
 *                                                                             	*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_18 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_18 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Wait till PnP Config Control register # been sent to PnP Address Port							*/
if ( ((oregs->cout_flow_config & 0x10) != (oregs->cout_intnl_flow_ctrl & 0x10)) ||
     ((oregs->cout_intnl_flow_ctrl & 0x08) == 0x08) )			/*Has PnP Config Control reg # been sent?	*/
{									/*If no, check deadman				*/
	if ( pnp_frame_delay_chk (DEADMAN_DELAY, gregs, slot) == 0x00 )	/*Has deadman time expired?			*/
	{								/*If no, wait & check again later		*/
		return ;
	}								/*If deadman expired, Modem Pool not functioning*/
	pnp_clean_up_ssp (slot, (unsigned char)0xFF) ;			/*Go clean up so can start over			*/
	return ;
}

/*PnP Address register now points to PnP Control Register 0x03 (WAKE), now set control signals to address PnP's Write	*/
/*Data Port														*/
oregs->cout_cntrl_sig = 0x155 ;						/*Pt MPA to PnP's Write Data Port (0xA79)	*/
slot->pnp_frame_ctr   = gregs->gicp_frame_ctr ;				/*Snap shot for timing				*/
slot->pnp_probe_funct = pnp_probe_fn_19 ;
return ;

}	/*End pnp_probe_fn_18												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 *           19th PnP PROBE FN - Wait till PnP Write Data Port addressed and	*
 *				 then send 0 (Wake) to PnP Control Reg 0x03	*
 *				 thus causing PnP board to enter Isolation State*
 *                                                                             	*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_19 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_19 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Delay 4 frames - allow time for MPA to see PnP Write Data Port being addressed					*/
if ( pnp_frame_delay_chk ( (unsigned short int)0x04, gregs, slot ) == 0 ) /*Has 4 frames elapsed?			*/
	return ;							  /*If no, continue probe			*/

/*Have delayed long enough for MPA to "see" control signal setting indicating that output be directed to PnP's Write	*/
/*Data Port, by writing a 0x00 to Write Data Port - PnP's Wake register is cleared thus causing PnP Board to enter	*/
/*Isolation state (see PnP spec. pages 13, 18, and 46)									*/
oregs->cout_xon_1 	  = 0x00 ;					/*Send PnP Reset CSN bit/command		*/
oregs->cout_flow_config  ^= 0x10 ;					/*Initiate xmit'g char				*/
slot->pnp_frame_ctr       = gregs->gicp_frame_ctr ;			/*Snap shot for timing				*/
slot->pnp_probe_funct     = pnp_probe_fn_20 ;
return ;

}	/*End pnp_probe_fn_19												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 *    20th PnP PROBE FN - Wait till data sent to PnP Wake register & Board in 	*
 *		          Isolation state					*
 *                                                                             	*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_20 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_20 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Wait for data to be sent to Wake register (i.e. 0x00 sent to PnP Control Register 0x03)				*/
if ( ((oregs->cout_flow_config & 0x10) != (oregs->cout_intnl_flow_ctrl & 0x10)) ||
     ((oregs->cout_intnl_flow_ctrl & 0x08) == 0x08) )			/*Has Reset CSN Command bit been sent?		*/
{									/*If no, check deadman				*/
	if ( pnp_frame_delay_chk (DEADMAN_DELAY, gregs, slot) == 0x00 )	/*Has deadman time expired?			*/
	{								/*If no, wait & check again later		*/
		return ;
	}								/*If deadman expired, Modem Pool not functioning*/
	pnp_clean_up_ssp (slot, (unsigned char)0xFF) ;			/*Go clean up so can start over			*/
	return ;
}

  /*PnP Board's now in Isolation State, set board's Read Data Port to 0x27B (see PnP spec. p. 15 & 45). The steps	*/
  /*required by the Modem Pool to do this are as follows (note: each step is a seperate function)			*/
  /*    1. Set chan's control signals to 0x111 (address PnP Address Port)	 					*/
  /*    2. Output 0x00, wrt 0x00 to PnP Address Port (indicates to PnP board that write_data addresses Set RD Data Port)*/
  /*    3. Set chan's control signals to 0x155 (address PnP Write Data Port)	 					*/
  /*    4. Output 0x9E, sets PnP Read Data Port to 0x27B (0x9E = 0x27b >> 2)						*/
oregs->cout_cntrl_sig = 0x111 ;						/*Pt MPA to PnP's Address port (0x279)		*/
slot->pnp_frame_ctr   = gregs->gicp_frame_ctr ;				/*Snap shot for timing				*/
slot->pnp_probe_funct = pnp_probe_fn_21 ;
return ;

}	/*End pnp_probe_fn_20												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 *           21st PnP PROBE FN - Wait till PnP Address Port addressed		*
 *                                                                             	*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_21 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_21 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Delay 4 frames - allow time for MPA to see PnP Address Port being addressed						*/
if ( pnp_frame_delay_chk ( (unsigned short int)0x04, gregs, slot ) == 0 ) /*Has 4 frames elapsed?			*/
	return ;							  /*If no, continue probe			*/

/*Have delayed long enough for MPA to "see" control signal setting indicating that output be directed to PnP's Address	*/
/*Port, now point to PnP Control Register 0x00 (Read Data Port) by writing 0x00 to PnP Address Port			*/
oregs->cout_xon_1 	  = 0x00 ;					/*Send PnP Control Register # to PnP Addr Port	*/
oregs->cout_flow_config  ^= 0x10 ;					/*Initiate xmit'g char				*/
slot->pnp_frame_ctr       = gregs->gicp_frame_ctr ;			/*Snap shot for timing				*/
slot->pnp_probe_funct     = pnp_probe_fn_22 ;
return ;

}	/*End pnp_probe_fn_21												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 *  22nd PnP PROBE FN - Set up to send PnP Read Data Port address to PnP 	*
 *			Control Register 0x00					*
 *                                                                             	*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_22 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_22 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Wait till PnP Config Control register # been sent to PnP Address Port							*/
if ( ((oregs->cout_flow_config & 0x10) != (oregs->cout_intnl_flow_ctrl & 0x10)) ||
     ((oregs->cout_intnl_flow_ctrl & 0x08) == 0x08) )			/*Has PnP Config Control reg # been sent?	*/
{									/*If no, check deadman				*/
	if ( pnp_frame_delay_chk (DEADMAN_DELAY, gregs, slot) == 0x00 )	/*Has deadman time expired?			*/
	{								/*If no, wait & check again later		*/
		return ;
	}								/*If deadman expired, Modem Pool not functioning*/
	pnp_clean_up_ssp (slot, (unsigned char)0xFF) ;			/*Go clean up so can start over			*/
	return ;
}

/*PnP Address register now points to PnP Control Register 0x00, now set control signals to address PnP's Write Data Port*/
oregs->cout_cntrl_sig = 0x155 ;						/*Pt MPA to PnP's Write Data Port (0xA79)	*/
slot->pnp_frame_ctr   = gregs->gicp_frame_ctr ;				/*Snap shot for timing				*/
slot->pnp_probe_funct = pnp_probe_fn_23 ;
return ;

}	/*End pnp_probe_fn_22												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 *           23rd PnP PROBE FN - Wait till PnP Write Data Port addressed	*
 *                                                                             	*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_23 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_23 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Delay 4 frames - allow time for MPA to see PnP Write Data Port being addressed					*/
if ( pnp_frame_delay_chk ( (unsigned short int)0x04, gregs, slot ) == 0 ) /*Has 4 frames elapsed?			*/
	return ;							  /*If no, continue probe			*/

/*Have delayed long enough for MPA to "see" control signal setting indicating that output be directed to PnP's Write	*/
/*Data Port, write a 0x9E to Write Data Port - thus causing PnP to set its Read Data Port to 0x27B (see spec. page 7,	*/
/*15, and 45)														*/
oregs->cout_xon_1 	  = (0x27B >> 2) ;				/*Send PnP Read Data Port address		*/
oregs->cout_flow_config  ^= 0x10 ;					/*Initiate xmit'g char				*/
slot->pnp_frame_ctr       = gregs->gicp_frame_ctr ;			/*Snap shot for timing				*/
slot->pnp_probe_funct     = pnp_probe_fn_24 ;
return ;

}	/*End pnp_probe_fn_23												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 * 24th PnP PROBE FN - Set up to place PnP board into Configuration state	*
 *                                                                             	*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_24 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_24 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Wait for PnP Read Data Port Address to be sent (i.e. 0x9E sent to PnP Control Register 0x00)				*/
if ( ((oregs->cout_flow_config & 0x10) != (oregs->cout_intnl_flow_ctrl & 0x10)) ||
     ((oregs->cout_intnl_flow_ctrl & 0x08) == 0x08) )			/*Has Reset CSN Command bit been sent?		*/
{									/*If no, check deadman				*/
	if ( pnp_frame_delay_chk (DEADMAN_DELAY, gregs, slot) == 0x00 )	/*Has deadman time expired?			*/
	{								/*If no, wait & check again later		*/
		return ;
	}								/*If deadman expired, Modem Pool not functioning*/
	pnp_clean_up_ssp (slot, (unsigned char)0xFF) ;			/*Go clean up so can start over			*/
	return ;
}

/*R5*/
/*PnP Board's Read Data Port address is now set to 0x27B, Read Serial Identifier from Data Port - requires 144 - 8 bit	*/
/*reads of Data Port to isolate card as per PnP spec., see pages 9, 19 & 25 (Zoom 33.6 SVD Faxmodem, Model 2802 	*/
/*requires this).													*/
/*NOTE: Since Reading PnP Read Data port causes a byte to be input'd and when a byte is input'd SSP determines line is	*/
/*	  no longer in "Break" state thusly clearing the Break status bit.  This, in turn, would cause a Driver to	*/
/*	  falsely determine that a Modem board is present.  Therefore, MPA was flowed off in function pnp_probe_fn_09	*/
/*The steps required by the Modem Pool to read the Serial identifier are as follows (note: each step is a seperate 	*/
/*function):														*/
/*    1. Set chan's control signals to 0x111 (indicates to MPA to address PnP Address Port, 0x279)	 		*/
/*    2. Xmit 0x01 to PnP Address port (indicates to PnP board that a read operation addresses Serial Isolation 	*/
/*	 register, see p. 25 & 45)											*/
/*    3. In order to address "AUX register", Set chan's control signals to 0x199					*/
/*    4. In order to read PnP I/O port 0x27B, Xmit 0x23 to AUX register (toggling bit 3)				*/
/*R5*/
oregs->cout_cntrl_sig = 0x111 ;						/*Set Cntrl Sigs to address PnP port 0x279	*/
slot->pnp_frame_ctr   = gregs->gicp_frame_ctr ;				/*Snap shot for timing				*/
slot->pnp_probe_funct = pnp_probe_fn_24a ;
return ;

}	/*End pnp_probe_fn_24												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 * 24a PnP PROBE FN - Wait for MPA to see Output Control Signals indicating to 	*
 *		      to address (direct output to) PnP Address Port (@0x279)	*
 *                                                                             	*
 ********************************************************************************/

/*R5*/
#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_24a (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_24a (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Delay 4 frames - allow time for MPA to see Control Signals								*/
if ( pnp_frame_delay_chk ( (unsigned short int)0x04, gregs, slot ) == 0 ) /*Has 4 frames elapsed?			*/
	return ;							  /*If no, continue probe			*/

/*Have delayed long enough for MPA to "see" control signal setting indicating that output be directed to PnP's Address	*/
/*Port, now point to PnP Serial Isolation Register by writing 0x01 to PnP Address Port					*/
oregs->cout_xon_1 	  = 0x01 ;					/*Send PnP Control Register # to PnP Addr Port	*/
oregs->cout_flow_config  ^= 0x10 ;					/*Initiate xmit'g char				*/
slot->pnp_frame_ctr       = gregs->gicp_frame_ctr ;			/*Snap shot for timing				*/
slot->pnp_probe_funct     = pnp_probe_fn_24b ;
return ;

}	/*End pnp_probe_fn_24a												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 *  22nd PnP PROBE FN - Set up to send PnP Read Data Port address to PnP 	*
 *			Control Register 0x00					*
 *                                                                             	*
 ********************************************************************************/

/*R5*/
#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_24b (struct ssp_struct   FARm *  sspcb,	/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_24b (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Wait till PnP Config Control register # been sent to PnP Address Port							*/
if ( ((oregs->cout_flow_config & 0x10) != (oregs->cout_intnl_flow_ctrl & 0x10)) ||
     ((oregs->cout_intnl_flow_ctrl & 0x08) == 0x08) )			/*Has PnP Config Control reg # been sent?	*/
{									/*If no, check deadman				*/
	if ( pnp_frame_delay_chk (DEADMAN_DELAY, gregs, slot) == 0x00 )	/*Has deadman time expired?			*/
	{								/*If no, wait & check again later		*/
		return ;
	}								/*If deadman expired, Modem Pool not functioning*/
	pnp_clean_up_ssp (slot, (unsigned char)0xFF) ;			/*Go clean up so can start over			*/
	return ;
}

/*PnP Address register now points to PnP Control Register 0x01 (Serial Isolation register), set Control Signals to	*/
/*address AUX register (so can initiate reads of PnP port 0x27B and thus receive Serial Identifier bits.		*/
oregs->cout_cntrl_sig = 0x199 ;						/*Addr MPA's AUX reg				*/
slot->pnp_frame_ctr   = gregs->gicp_frame_ctr ;				/*Snap shot for timing				*/
slot->pnp_probe_funct = pnp_probe_fn_24c ;
return ;

}	/*End pnp_probe_fn_24b												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 *      	 24c PnP PROBE FN - Wait till AUX Reg addressed			*
 *                                                                             	*
 ********************************************************************************/

/*R5*/
#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_24c (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_24c (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Delay 4 frames - allow time for MPA to see AUX Reg being addressed							*/
if ( pnp_frame_delay_chk ( (unsigned short int)0x04, gregs, slot ) == 0 ) /*Has 4 frames elapsed?			*/
	return ;							  /*If no, continue probe			*/

/*Have delayed long enough for MPA to "see" control signal setting indicating that output be directed to MPA's AUX Reg,	*/
/*write to AUX register toggling "read toggle" thus causing MPA to read a byte from PnP Data Port (gets a bit of board's*/
/*Serial Identifier).													*/
slot->pnp_probe_parm    = 0x00 ;					/*Initiate parm to count Serial Identifier Reads*/
slot->read_toggle       ^= 0x08 ;					/*Save I/O Read toggle state			*/
oregs->cout_xon_1       = 0x23 | slot->read_toggle ;			/*Set up & send AUX Reg value =			*/
oregs->cout_flow_config ^= 0x10 ;					/* Read PnP I/O Port 0x27B			*/
slot->pnp_frame_ctr     = gregs->gicp_frame_ctr ;			/*Snap shot for timing				*/
slot->pnp_probe_funct   = pnp_probe_fn_24d ;
return ;

}	/*End pnp_probe_fn_24c												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 *   24d PnP PROBE FN - Continue/complete Read of Serial Identifier bits from	*
 *			PnP Data Port (0x27B)					*
 *                                                                             	*
 ********************************************************************************/

/*R5*/
#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_24d (struct ssp_struct   FARm *  sspcb,	/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_24d (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Wait for write of AUX Reg byte to complete.  Note: Byte value written indicates to MPA to Read a byte from PnP Data 	*/
/*Port (0x27B).  Note: Byte received is part of board's (if it exists) Serial Identifier and actually represents only	*/
/*1 bit of Identifier													*/
if ( ((oregs->cout_flow_config & 0x10) != (oregs->cout_intnl_flow_ctrl & 0x10)) ||
     ((oregs->cout_intnl_flow_ctrl & 0x08) == 0x08) )			/*Has PnP Aux Byte been sent?			*/
{									/*If no, check deadman				*/
	if ( pnp_frame_delay_chk (DEADMAN_DELAY, gregs, slot) == 0x00 )	/*Has deadman time expired?			*/
	{								/*If no, wait & check again later		*/
		return ;
	}								/*If deadman expired, Modem Pool not functioning*/
	pnp_clean_up_ssp (slot, (unsigned char)0xFF) ;			/*Go clean up so can start over			*/
	return ;
}

/*Read a byte of board's (if it exists) Serial Identifier, check if done reading a bytes (bits)				*/
if ( ++slot->pnp_probe_parm < 72*2 )					/*Have completed reading Serial Identifier bits?*/
{									/*If no, read next byte				*/
	slot->read_toggle       ^= 0x08 ;				/*Save I/O Read toggle state			*/
	oregs->cout_xon_1       = 0x23 | slot->read_toggle ;		/*Set up & send AUX Reg value =			*/
	oregs->cout_flow_config ^= 0x10 ;				/* Read PnP I/O Port 0x27B			*/
	slot->pnp_frame_ctr     = gregs->gicp_frame_ctr ;		/*Snap shot for timing				*/
	return ;
}

/*Completed reading PnP board's Serial Identifier (assuming board exists), board (if exists) now isolated, set up to	*/
/*place board into Configuration state by writing 1 to PnP Card Number Select register (reg 0x06).  See Plug and Play	*/
/*spec. pgs 13, 18 & 46).  The steps required by the Modem Pool to do this are as follows (note: each step is a 	*/
/*seperate function):													*/
/*    1. Set chan's control signals to 0x111 (address PnP Address Port)	 						*/
/*    2. Output 0x06, writes 0x06 to PnP Address Port (indicates to PnP board that write_data addresses Card Select	*/
/*	 Number register)												*/
/*    3. Set chan's control signals to 0x155 (address PnP Write Data Port)	 					*/
/*    4. Output 0x01, writes 0x01 to PnP Card Select Number (causes PnP board to enter Configuration state)		*/
oregs->cout_cntrl_sig = 0x111 ;						/*Pt MPA to PnP's Address port (0x279)		*/
slot->pnp_frame_ctr   = gregs->gicp_frame_ctr ;				/*Snap shot for timing				*/
slot->pnp_probe_funct = pnp_probe_fn_25 ;
return ;

}	/*End pnp_probe_fn_24c												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 *           25th PnP PROBE FN - Wait till PnP Address Port addressed		*
 *                                                                             	*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_25 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_25 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Delay 4 frames - allow time for MPA to see PnP Address Port being addressed						*/
if ( pnp_frame_delay_chk ( (unsigned short int)0x04, gregs, slot ) == 0 ) /*Has 4 frames elapsed?			*/
	return ;							  /*If no, continue probe			*/

/*Have delayed long enough for MPA to "see" control signal setting indicating that output be directed to PnP's Address	*/
/*Port, now point to PnP Control Register 0x06 (Read Data Port) by writing 0x06 to PnP Address Port			*/
oregs->cout_xon_1 	  = 0x06 ;					/*Send PnP Control Register # to PnP Addr Port	*/
oregs->cout_flow_config  ^= 0x10 ;					/*Initiate xmit'g char				*/
slot->pnp_frame_ctr       = gregs->gicp_frame_ctr ;			/*Snap shot for timing				*/
slot->pnp_probe_funct     = pnp_probe_fn_26 ;
return ;

}	/*End pnp_probe_fn_25												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 *  26th PnP PROBE FN - Set up to send PnP Card Select Number Port address to 	*
 *			PnP Control Register 0x06				*
 *                                                                             	*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_26 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_26 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Wait till PnP Config Control register # been sent to PnP Address Port							*/
if ( ((oregs->cout_flow_config & 0x10) != (oregs->cout_intnl_flow_ctrl & 0x10)) ||
     ((oregs->cout_intnl_flow_ctrl & 0x08) == 0x08) )			/*Has PnP Config Control reg # been sent?	*/
{									/*If no, check deadman				*/
	if ( pnp_frame_delay_chk (DEADMAN_DELAY, gregs, slot) == 0x00 )	/*Has deadman time expired?			*/
	{								/*If no, wait & check again later		*/
		return ;
	}								/*If deadman expired, Modem Pool not functioning*/
	pnp_clean_up_ssp (slot, (unsigned char)0xFF) ;			/*Go clean up so can start over			*/
	return ;
}

/*PnP Address register now points to PnP Card Select Number Register 0x06, now set control signals to address PnP's	*/
/*Write Data Port													*/
oregs->cout_cntrl_sig = 0x155 ;						/*Pt MPA to PnP's Write Data Port (0xA79)	*/
slot->pnp_frame_ctr   = gregs->gicp_frame_ctr ;				/*Snap shot for timing				*/
slot->pnp_probe_funct = pnp_probe_fn_27 ;
return ;

}	/*End pnp_probe_fn_26												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 *           27th PnP PROBE FN - Wait till PnP Write Data Port addressed and	*
 *				 write board's CSN				*
 *                                                                             	*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_27 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_27 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Delay 4 frames - allow time for MPA to see PnP Write Data Port being addressed					*/
if ( pnp_frame_delay_chk ( (unsigned short int)0x04, gregs, slot ) == 0 ) /*Has 4 frames elapsed?			*/
	return ;							  /*If no, continue probe			*/

/*Have delayed long enough for MPA to "see" control signal setting indicating that output be directed to PnP's Write	*/
/*Data Port, write a 0x01 to Write Data Port - thus causing PnP to set its Card Select Number and enter Configuration	*/
/*state (see spec. page 7, 16, 18, and 46) from Isolation state								*/
oregs->cout_xon_1 	  = 0x01 ;					/*Send PnP CSN Number				*/
oregs->cout_flow_config  ^= 0x10 ;					/*Initiate xmit'g char				*/
slot->pnp_frame_ctr       = gregs->gicp_frame_ctr ;			/*Snap shot for timing				*/
slot->pnp_probe_funct     = pnp_probe_fn_30 ;
return ;

}	/*End pnp_probe_fn_27												*/
/*NOTE: Functions 28 & 29 were deleted.  It was originally intended that these functions would read attempt to read CSN	*/
/*	and if CSN not read then PnP Probe would start all over.  If CSN successfully read then Probing would continue	*/
/*	at function 30.  Functions 28 & 29 can not be done since reading CSN (if PnP board present) causes a byte to be	*/
/*	input'd.  When a byte input'd SSP determines line no longer in "Break" state and hence clears Break status bit.	*/
/*	This, in turn, causes Driver to falsely determine the presence of a board.					*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 *      	 30th PnP PROBE FN - Wait till PnP CSN sent and start to assign	*
 *				     I/O address				*
 *                                                                             	*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_30 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_30 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Wait for PnP CSN to be sent												*/
if ( ((oregs->cout_flow_config & 0x10) != (oregs->cout_intnl_flow_ctrl & 0x10)) ||
     ((oregs->cout_intnl_flow_ctrl & 0x08) == 0x08) )			/*Is CSN xmit done?				*/
{									/*If no, check deadman				*/
	if ( pnp_frame_delay_chk (DEADMAN_DELAY, gregs, slot) == 0x00 )	/*Has deadman time expired?			*/
	{								/*If no, wait & check again later		*/
		return ;
	}								/*If deadman expired, Modem Pool not functioning*/
	pnp_clean_up_ssp (slot, (unsigned char)0xFF) ;			/*Go clean up so can start over			*/
	return ;
}

  /*CSN number assigned (if board exists) and board is in configuration state.  Set up and assign to board a base I/O	*/
  /*address of 0x3F8 (COM1) and an Interrupt (IRQ4 - COM1).  Once assigned activate PnP board.  The steps required by	*/
  /*Modem Pool are as follows (note: each step is a seperate function):							*/
  /*    1. Set chan's control signals to 0x111 (address PnP Address Port)	 					*/
  /*    2. Output 0x60, writes 0x60 to PnP Address Port (indicates to PnP board that write_data addresses I/O Port Base	*/
  /*	 address bits 15:08).												*/
  /*    3. Set chan's control signals to 0x155 (address PnP Write Data Port)	 					*/
  /*    4. Output 0x03, writes 0x03 (high byte of COM1 I/O port addr, 0x3F8) to board's I/O Port address reg (high byte)*/
  /*    5. Set chan's control signals to 0x111 (address PnP Address Port)	 					*/
  /*    6. Output 0x61, writes 0x61 to PnP Address Port (indicates to PnP board that write_data addresses I/O Base Port	*/
  /*	 address register low order byte, bits 07:00).									*/
  /*    7. Set chan's control signals to 0x155 (address PnP Write Data Port)	 					*/
  /*    8. Output 0xF8, writes 0xF8 (low byte of COM1 I/O port addr, 0x3F8) to board's I/O Port address reg (low byte)	*/
  /*    9. Set chan's control signals to 0x111 (address PnP Address Port)	 					*/
  /*   10. Output 0x70, writes 0x70 to PnP Address Port (indicates to PnP board that write_data addresses Interrupt	*/
  /*	 Request Level Select 0 register).										*/
  /*   11. Set chan's control signals to 0x155 (address PnP Write Data Port)	 					*/
  /*   12. Output 0x04, writes 0x04 (COM1 Interrupt) to board's Interrupt Request Level Select				*/
  /*   13. Set chan's control signals to 0x111 (address PnP Address Port)	 					*/
  /*   14. Output 0x71, writes 0x71 to PnP Address Port (indicates to PnP board that write_data addresses Interrupt	*/
  /*	 Request Type Select 0 register).										*/
  /*   15. Set chan's control signals to 0x155 (address PnP Write Data Port)	 					*/
  /*   16. Output 0x03, writes 0x03 (High - Level triggered Interrupt) to board's Interrupt Request Type Select		*/
oregs->cout_cntrl_sig = 0x111 ;						/*Pt MPA to PnP's Address port (0x279)		*/
slot->pnp_frame_ctr   = gregs->gicp_frame_ctr ;				/*Snap shot for timing				*/
slot->pnp_probe_funct = pnp_probe_fn_31 ;
return ;

}	/*End pnp_probe_fn_30												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 *           31th PnP PROBE FN - Wait till PnP Address Port addressed		*
 *                                                                             	*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_31 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_31 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Delay 4 frames - allow time for MPA to see PnP Address Port being addressed						*/
if ( pnp_frame_delay_chk ( (unsigned short int)0x04, gregs, slot ) == 0 ) /*Has 4 frames elapsed?			*/
	return ;							  /*If no, continue probe			*/

/*Have delayed long enough for MPA to "see" control signal setting indicating that output be directed to PnP's Address	*/
/*Port, now point to PnP Control Register 0x60 (I/O Port Base addr bits 15:08) by writing 0x60 to PnP Address Port	*/
oregs->cout_xon_1 	  = 0x60 ;					/*Send I/O Port Addr bit 15:08 to PnP Addr Port	*/
oregs->cout_flow_config  ^= 0x10 ;					/*Initiate xmit'g char				*/
slot->pnp_frame_ctr       = gregs->gicp_frame_ctr ;			/*Snap shot for timing				*/
slot->pnp_probe_funct     = pnp_probe_fn_32 ;
return ;

}	/*End pnp_probe_fn_31												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 *  	   32th PnP PROBE FN - Set up to send I/O Port Base addr bits 15:08	*
 *                                                                             	*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_32 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_32 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Wait till PnP I/O Port Base addr bits 15:08 register # has been sent to PnP Address Port				*/
if ( ((oregs->cout_flow_config & 0x10) != (oregs->cout_intnl_flow_ctrl & 0x10)) ||
     ((oregs->cout_intnl_flow_ctrl & 0x08) == 0x08) )			/*Has PnP I/O Port bits been sent?		*/
{									/*If no, check deadman				*/
	if ( pnp_frame_delay_chk (DEADMAN_DELAY, gregs, slot) == 0x00 )	/*Has deadman time expired?			*/
	{								/*If no, wait & check again later		*/
		return ;
	}								/*If deadman expired, Modem Pool not functioning*/
	pnp_clean_up_ssp (slot, (unsigned char)0xFF) ;			/*Go clean up so can start over			*/
	return ;
}

/*PnP Address register now points to PnP I/O Port Base Address bits 15:08, now set control signals to address PnP's	*/
/*Write Data Port													*/
oregs->cout_cntrl_sig = 0x155 ;						/*Pt MPA to PnP's Write Data Port (0xA79)	*/
slot->pnp_frame_ctr   = gregs->gicp_frame_ctr ;				/*Snap shot for timing				*/
slot->pnp_probe_funct = pnp_probe_fn_33 ;
return ;

}	/*End pnp_probe_fn_32												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 *           33th PnP PROBE FN - Wait till PnP Write Data Port addressed	*
 *                                                                             	*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_33 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_33 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Delay 4 frames - allow time for MPA to see PnP Write Data Port being addressed					*/
if ( pnp_frame_delay_chk ( (unsigned short int)0x04, gregs, slot ) == 0 ) /*Has 4 frames elapsed?			*/
	return ;							  /*If no, continue probe			*/

/*Have delayed long enough for MPA to "see" control signal setting indicating that output be directed to PnP's Write	*/
/*Data Port, write a 0x03 to Write Data Port - thus causing PnP board to set its ISA I/O Port Base addr bits (15:08) to	*/
/*0x03 (high byte of COM I/O port address 0x3F8), see page 50.								*/
oregs->cout_xon_1 	  = 0x03 ;					/*Send high I/O Port address bits		*/
oregs->cout_flow_config  ^= 0x10 ;					/*Initiate xmit'g char				*/
slot->pnp_frame_ctr       = gregs->gicp_frame_ctr ;			/*Snap shot for timing				*/
slot->pnp_probe_funct     = pnp_probe_fn_34 ;
return ;

}	/*End pnp_probe_fn_33												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 * 34th PnP PROBE FN - start setting up ISA I/O Port Base Address bit 07:00 by	*
 *		       first pointing to PnP Address Port			*
 *                                                                             	*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_34 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_34 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Wait for PnP CSN to be sent												*/
if ( ((oregs->cout_flow_config & 0x10) != (oregs->cout_intnl_flow_ctrl & 0x10)) ||
     ((oregs->cout_intnl_flow_ctrl & 0x08) == 0x08) )			/*Have I/O Ports bits been sent?		*/
{									/*If no, check deadman				*/
	if ( pnp_frame_delay_chk (DEADMAN_DELAY, gregs, slot) == 0x00 )	/*Has deadman time expired?			*/
	{								/*If no, wait & check again later		*/
		return ;
	}								/*If deadman expired, Modem Pool not functioning*/
	pnp_clean_up_ssp (slot, (unsigned char)0xFF) ;			/*Go clean up so can start over			*/
	return ;
}

  /*PnP Board's ISA I/O Port Base Address bits 15:08 = 0x03, point to PnP address port so set write address = 0x61, PnP	*/
  /*I/O Port Base Address bits 07:00											*/
oregs->cout_cntrl_sig = 0x111 ;						/*Pt MPA to PnP's Address port (0x279)		*/
slot->pnp_frame_ctr   = gregs->gicp_frame_ctr ;				/*Snap shot for timing				*/
slot->pnp_probe_funct = pnp_probe_fn_35 ;
return ;

}	/*End pnp_probe_fn_34												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 *           35th PnP PROBE FN - Wait till PnP Address Port addressed		*
 *                                                                             	*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_35 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_35 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Delay 4 frames - allow time for MPA to see PnP Address Port being addressed						*/
if ( pnp_frame_delay_chk ( (unsigned short int)0x04, gregs, slot ) == 0 ) /*Has 4 frames elapsed?			*/
	return ;							  /*If no, continue probe			*/

/*Have delayed long enough for MPA to "see" control signal setting indicating that output be directed to PnP's Address	*/
/*Port, now point to PnP Control Register 0x61 (I/O Port Base addr bits 07:00) by writing 0x61 to PnP Address Port	*/
oregs->cout_xon_1 	  = 0x61 ;					/*Send I/O Port Addr bit 07:00 to PnP Addr Port	*/
oregs->cout_flow_config  ^= 0x10 ;					/*Initiate xmit'g char				*/
slot->pnp_frame_ctr       = gregs->gicp_frame_ctr ;			/*Snap shot for timing				*/
slot->pnp_probe_funct     = pnp_probe_fn_36 ;
return ;

}	/*End pnp_probe_fn_35												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 *  	   36th PnP PROBE FN - Set up to send I/O Port Base addr bits 07:00	*
 *                                                                             	*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_36 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_36 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Wait till PnP I/O Port Base Addr register # been sent to PnP Address Port						*/
if ( ((oregs->cout_flow_config & 0x10) != (oregs->cout_intnl_flow_ctrl & 0x10)) ||
     ((oregs->cout_intnl_flow_ctrl & 0x08) == 0x08) )			/*Has register # been sent?			*/
{									/*If no, check deadman				*/
	if ( pnp_frame_delay_chk (DEADMAN_DELAY, gregs, slot) == 0x00 )	/*Has deadman time expired?			*/
	{								/*If no, wait & check again later		*/
		return ;
	}								/*If deadman expired, Modem Pool not functioning*/
	pnp_clean_up_ssp (slot, (unsigned char)0xFF) ;			/*Go clean up so can start over			*/
	return ;
}

/*PnP Address register now points to PnP I/O Port Base Address bits 07:00, now set control signals to address PnP's	*/
/*Write Data Port													*/
oregs->cout_cntrl_sig = 0x155 ;						/*Pt MPA to PnP's Write Data Port (0xA79)	*/
slot->pnp_frame_ctr   = gregs->gicp_frame_ctr ;				/*Snap shot for timing				*/
slot->pnp_probe_funct = pnp_probe_fn_37 ;
return ;

}	/*End pnp_probe_fn_36												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 *           37th PnP PROBE FN - Wait till PnP Write Data Port addressed	*
 *                                                                             	*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_37 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_37 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Delay 4 frames - allow time for MPA to see PnP Write Data Port being addressed					*/
if ( pnp_frame_delay_chk ( (unsigned short int)0x04, gregs, slot ) == 0 ) /*Has 4 frames elapsed?			*/
	return ;							  /*If no, continue probe			*/

/*Have delayed long enough for MPA to "see" control signal setting indicating that output be directed to PnP's Write	*/
/*Data Port, write a 0xF8 to Write Data Port - thus causing PnP board to set its ISA I/O Port Base addr bits (07:00) to	*/
/*0xF8 (low byte of COM I/O port address 0x3F8), see page 50.								*/
oregs->cout_xon_1 	  = 0xF8 ;					/*Send high I/O Port address bits		*/
oregs->cout_flow_config  ^= 0x10 ;					/*Initiate xmit'g char				*/
slot->pnp_frame_ctr       = gregs->gicp_frame_ctr ;			/*Snap shot for timing				*/
slot->pnp_probe_funct     = pnp_probe_fn_38 ;
return ;

}	/*End pnp_probe_fn_37												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 * 38th PnP PROBE FN - start setting up ISA Interrupt number by	first pointing	*
 *		       to PnP Address Port					*
 *                                                                             	*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_38 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_38 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Wait for PnP ISA I/O Port Base addr bits (07:00) to be sent								*/
if ( ((oregs->cout_flow_config & 0x10) != (oregs->cout_intnl_flow_ctrl & 0x10)) ||
     ((oregs->cout_intnl_flow_ctrl & 0x08) == 0x08) )			/*Have I/O Ports bits been sent?		*/
{									/*If no, check deadman				*/
	if ( pnp_frame_delay_chk (DEADMAN_DELAY, gregs, slot) == 0x00 )	/*Has deadman time expired?			*/
	{								/*If no, wait & check again later		*/
		return ;
	}								/*If deadman expired, Modem Pool not functioning*/
	pnp_clean_up_ssp (slot, (unsigned char)0xFF) ;			/*Go clean up so can start over			*/
	return ;
}

  /*PnP Board's ISA I/O Port Base Address bits 07:00 = 0xF8, point to PnP address port so set write address = 0x70, PnP	*/
  /*Interrupt Request Level (see page 51)										*/
oregs->cout_cntrl_sig = 0x111 ;						/*Pt MPA to PnP's Address port (0x279)		*/
slot->pnp_frame_ctr   = gregs->gicp_frame_ctr ;				/*Snap shot for timing				*/
slot->pnp_probe_funct = pnp_probe_fn_39 ;
return ;

}	/*End pnp_probe_fn_38												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 *           39th PnP PROBE FN - Wait till PnP Address Port addressed		*
 *                                                                             	*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_39 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_39 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Delay 4 frames - allow time for MPA to see PnP Address Port being addressed						*/
if ( pnp_frame_delay_chk ( (unsigned short int)0x04, gregs, slot ) == 0 ) /*Has 4 frames elapsed?			*/
	return ;							  /*If no, continue probe			*/

/*Have delayed long enough for MPA to "see" control signal setting indicating that output be directed to PnP's Address	*/
/*Port, now point to PnP Control Register 0x70 (Interrupt Request Level) by writing 0x70 to PnP Address Port		*/
oregs->cout_xon_1 	  = 0x70 ;					/*Send Interrupt Request Addr to PnP Addr Port	*/
oregs->cout_flow_config  ^= 0x10 ;					/*Initiate xmit'g char				*/
slot->pnp_frame_ctr       = gregs->gicp_frame_ctr ;			/*Snap shot for timing				*/
slot->pnp_probe_funct     = pnp_probe_fn_40 ;
return ;

}	/*End pnp_probe_fn_39												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 *  	   40th PnP PROBE FN - Set up to send Interrupt Request Level 		*
 *                                                                             	*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_40 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_40 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Wait till PnP Interrupt Request Level register # been sent to PnP Address Port					*/
if ( ((oregs->cout_flow_config & 0x10) != (oregs->cout_intnl_flow_ctrl & 0x10)) ||
     ((oregs->cout_intnl_flow_ctrl & 0x08) == 0x08) )			/*Has PnP I/O Port bits been sent?		*/
{									/*If no, check deadman				*/
	if ( pnp_frame_delay_chk (DEADMAN_DELAY, gregs, slot) == 0x00 )	/*Has deadman time expired?			*/
	{								/*If no, wait & check again later		*/
		return ;
	}								/*If deadman expired, Modem Pool not functioning*/
	pnp_clean_up_ssp (slot, (unsigned char)0xFF) ;			/*Go clean up so can start over			*/
	return ;
}

/*PnP Address register now points to PnP Interrupt Request Level register, now set control signals to address PnP's	*/
/*Write Data Port													*/
oregs->cout_cntrl_sig = 0x155 ;						/*Pt MPA to PnP's Write Data Port (0xA79)	*/
slot->pnp_frame_ctr   = gregs->gicp_frame_ctr ;				/*Snap shot for timing				*/
slot->pnp_probe_funct = pnp_probe_fn_41 ;
return ;

}	/*End pnp_probe_fn_40												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 *           41st PnP PROBE FN - Wait till PnP Write Data Port addressed	*
 *                                                                             	*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_41 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_41 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Delay 4 frames - allow time for MPA to see PnP Write Data Port being addressed					*/
if ( pnp_frame_delay_chk ( (unsigned short int)0x04, gregs, slot ) == 0 ) /*Has 4 frames elapsed?			*/
	return ;							  /*If no, continue probe			*/

/*Have delayed long enough for MPA to "see" control signal setting indicating that output be directed to PnP's Write	*/
/*Data Port, wrt 0x04 to Write Data Port - thus causing PnP board to set its ISA Interrupt # to COM1's Interrupt (0x04)	*/
oregs->cout_xon_1 	  = 0x04 ;					/*Send Interrupt # (see p. 51)			*/
oregs->cout_flow_config  ^= 0x10 ;					/*Initiate xmit'g char				*/
slot->pnp_frame_ctr       = gregs->gicp_frame_ctr ;			/*Snap shot for timing				*/
slot->pnp_probe_funct     = pnp_probe_fn_42 ;
return ;

}	/*End pnp_probe_fn_41												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 * 42th PnP PROBE FN - start setting up ISA Interrupt Request Type by first 	*
 *		       pointing to PnP Address Port				*
 *                                                                             	*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_42 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_42 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Wait for Interrupt Number to be sent											*/
if ( ((oregs->cout_flow_config & 0x10) != (oregs->cout_intnl_flow_ctrl & 0x10)) ||
     ((oregs->cout_intnl_flow_ctrl & 0x08) == 0x08) )			/*Has Interrupt Number been sent?		*/
{									/*If no, check deadman				*/
	if ( pnp_frame_delay_chk (DEADMAN_DELAY, gregs, slot) == 0x00 )	/*Has deadman time expired?			*/
	{								/*If no, wait & check again later		*/
		return ;
	}								/*If deadman expired, Modem Pool not functioning*/
	pnp_clean_up_ssp (slot, (unsigned char)0xFF) ;			/*Go clean up so can start over			*/
	return ;
}
  /*PnP Board's ISA Interrupt Request Level (number) has been sent, point to PnP address port so set write address =	*/
  /*0x70, PnP Interrupt Request Type (see page 51)									*/
oregs->cout_cntrl_sig = 0x111 ;						/*Pt MPA to PnP's Address port (0x279)		*/
slot->pnp_frame_ctr   = gregs->gicp_frame_ctr ;				/*Snap shot for timing				*/
slot->pnp_probe_funct = pnp_probe_fn_43 ;
return ;

}	/*End pnp_probe_fn_42												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 *           43rd PnP PROBE FN - Wait till PnP Address Port addressed		*
 *                                                                             	*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_43 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_43 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Delay 4 frames - allow time for MPA to see PnP Address Port being addressed						*/
if ( pnp_frame_delay_chk ( (unsigned short int)0x04, gregs, slot ) == 0 ) /*Has 4 frames elapsed?			*/
	return ;							  /*If no, continue probe			*/

/*Have delayed long enough for MPA to "see" control signal setting indicating that output be directed to PnP's Address	*/
/*Port, now point to PnP Control Register 0x71 (Interrupt Request Type) by writing 0x71 to PnP Address Port		*/
oregs->cout_xon_1 	  = 0x71 ;					/*Send Interrupt Req Type Addr to PnP Addr Port	*/
oregs->cout_flow_config  ^= 0x10 ;					/*Initiate xmit'g char				*/
slot->pnp_frame_ctr       = gregs->gicp_frame_ctr ;			/*Snap shot for timing				*/
slot->pnp_probe_funct     = pnp_probe_fn_44 ;
return ;

}	/*End pnp_probe_fn_43												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 *  	   44th PnP PROBE FN - Set up to send Interrupt Request Type 		*
 *                                                                             	*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_44 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_44 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Wait till PnP Interrupt Request Type register # been sent to PnP Address Port						*/
if ( ((oregs->cout_flow_config & 0x10) != (oregs->cout_intnl_flow_ctrl & 0x10)) ||
     ((oregs->cout_intnl_flow_ctrl & 0x08) == 0x08) )			/*Has PnP Interrupt Request type bits been sent?*/
{									/*If no, check deadman				*/
	if ( pnp_frame_delay_chk (DEADMAN_DELAY, gregs, slot) == 0x00 )	/*Has deadman time expired?			*/
	{								/*If no, wait & check again later		*/
		return ;
	}								/*If deadman expired, Modem Pool not functioning*/
	pnp_clean_up_ssp (slot, (unsigned char)0xFF) ;			/*Go clean up so can start over			*/
	return ;
}

/*PnP Address register now points to PnP Interrupt Request type register, now set control signals to address PnP's	*/
/*Write Data Port													*/
oregs->cout_cntrl_sig = 0x155 ;						/*Pt MPA to PnP's Write Data Port (0xA79)	*/
slot->pnp_frame_ctr   = gregs->gicp_frame_ctr ;				/*Snap shot for timing				*/
slot->pnp_probe_funct = pnp_probe_fn_45 ;
return ;

}	/*End pnp_probe_fn_44												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 *           45st PnP PROBE FN - Wait till PnP Write Data Port addressed	*
 *                                                                             	*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_45 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_45 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Delay 4 frames - allow time for MPA to see PnP Write Data Port being addressed					*/
if ( pnp_frame_delay_chk ( (unsigned short int)0x04, gregs, slot ) == 0 ) /*Has 4 frames elapsed?			*/
	return ;							  /*If no, continue probe			*/

/*Have delayed long enough for MPA to "see" control signal setting indicating that output be directed to PnP's Write	*/
/*Data Port, wrt 0x03 to Write Data Port - thus causing PnP board to set its ISA Interrupt type to Interrupt 		*/
/*level = high and Interrupt type = level (see page 51)									*/
oregs->cout_xon_1 	  = 0x03 ;					/*Send Interrupt Request Type (see p. 51)	*/
oregs->cout_flow_config  ^= 0x10 ;					/*Initiate xmit'g char				*/
slot->pnp_frame_ctr       = gregs->gicp_frame_ctr ;			/*Snap shot for timing				*/
slot->pnp_probe_funct     = pnp_probe_fn_46 ;
return ;

}	/*End pnp_probe_fn_45												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 * 		     46th PnP PROBE FN - Set up to activate PnP board		*
 *                                                                             	*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_46 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_46 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Wait for Interrupt Type to be sent											*/
if ( ((oregs->cout_flow_config & 0x10) != (oregs->cout_intnl_flow_ctrl & 0x10)) ||
     ((oregs->cout_intnl_flow_ctrl & 0x08) == 0x08) )			/*Has Interrupt type been sent?			*/
{									/*If no, check deadman				*/
	if ( pnp_frame_delay_chk (DEADMAN_DELAY, gregs, slot) == 0x00 )	/*Has deadman time expired?			*/
	{								/*If no, wait & check again later		*/
		return ;
	}								/*If deadman expired, Modem Pool not functioning*/
	pnp_clean_up_ssp (slot, (unsigned char)0xFF) ;			/*Go clean up so can start over			*/
	return ;
}
  /*PnP board has been configured -  I/O Port Address = 0x3F8, Interrupt number = IRQ4 and Interrupt = high, level 	*/
  /*triggered.  Now activate board and set up so MPA can recognize board's presence.  The steps required by the Modem 	*/
  /*Pool are as follows (each step is a seperate function):								*/
  /*    1. Set chan's control signals to 0x111 (address PnP Address Port)	 					*/
  /*    2. Output 0x30, writes 0x30 to PnP Address Port (indicates to PnP board that write_data addresses Activate Reg)	*/
  /*    3. Set chan's control signals to 0x155 (address PnP Write Data Port)	 					*/
  /*    4. Output 0x01, writes 0x01 to board's Activate register							*/
  /*    5. Set chan's control signals to 0x111 (address PnP Address Port)	 					*/
  /*    6. Output 0x02, writes 0x02 to PnP Address Port (indicates to PnP board that write_data addresses Configuration	*/
  /*	 Control register)												*/
  /*    7. Set chan's control signals to 0x155 (address PnP Write Data Port)	 					*/
  /*    8. Output 0x02, writes 0x02 to board's Config. Ctrl reg indicating to board to enter "Wait for Key" state 	*/
  /*    9. Clear MPA AUX reg by issuing "Flush" so PnP I/O is disabled hence enabling MPA to "see" newly configured 	*/
  /*	 PnP board.  Also, Clear slot's read toggle (slot->read_toggle == 0x00) and set slot's PnP probe state 		*/
  /*	 (slot->pnp_probe_state = 0xFF) to indicate Probe completed successfully					*/
oregs->cout_cntrl_sig = 0x111 ;						/*Pt MPA to PnP's Address port (0x279)		*/
slot->pnp_frame_ctr   = gregs->gicp_frame_ctr ;				/*Snap shot for timing				*/
slot->pnp_probe_funct = pnp_probe_fn_47 ;
return ;

}	/*End pnp_probe_fn_46												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 *           47th PnP PROBE FN - Wait till PnP Address Port addressed		*
 *                                                                             	*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_47 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_47 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Delay 4 frames - allow time for MPA to see PnP Address Port being addressed						*/
if ( pnp_frame_delay_chk ( (unsigned short int)0x04, gregs, slot ) == 0 ) /*Has 4 frames elapsed?			*/
	return ;							  /*If no, continue probe			*/

/*Have delayed long enough for MPA to "see" control signal setting indicating that output be directed to PnP's Address	*/
/*Port, now point to PnP Control Register 0x30 (Activate register) by writing 0x30 to PnP Address Port			*/
oregs->cout_xon_1 	  = 0x30 ;					/*Send Activate Reg Addr to PnP Addr Port	*/
oregs->cout_flow_config  ^= 0x10 ;					/*Initiate xmit'g char				*/
slot->pnp_frame_ctr       = gregs->gicp_frame_ctr ;			/*Snap shot for timing				*/
slot->pnp_probe_funct     = pnp_probe_fn_48 ;
return ;

}	/*End pnp_probe_fn_47												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 *  	   48th PnP PROBE FN - Set up to Activate PnP board 			*
 *                                                                             	*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_48 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_48 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Wait till Activate register # been sent to PnP Address Port								*/
if ( ((oregs->cout_flow_config & 0x10) != (oregs->cout_intnl_flow_ctrl & 0x10)) ||
     ((oregs->cout_intnl_flow_ctrl & 0x08) == 0x08) )			/*Has PnP Activate reg # bits been sent?	*/
{									/*If no, check deadman				*/
	if ( pnp_frame_delay_chk (DEADMAN_DELAY, gregs, slot) == 0x00 )	/*Has deadman time expired?			*/
	{								/*If no, wait & check again later		*/
		return ;
	}								/*If deadman expired, Modem Pool not functioning*/
	pnp_clean_up_ssp (slot, (unsigned char)0xFF) ;			/*Go clean up so can start over			*/
	return ;
}

/*PnP Address register now points to PnP Activate register, now set control signals to address PnP's Write Data Port	*/
oregs->cout_cntrl_sig = 0x155 ;						/*Pt MPA to PnP's Write Data Port (0xA79)	*/
slot->pnp_frame_ctr   = gregs->gicp_frame_ctr ;				/*Snap shot for timing				*/
slot->pnp_probe_funct = pnp_probe_fn_49 ;
return ;

}	/*End pnp_probe_fn_48												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 *           49th PnP PROBE FN - Wait till PnP Write Data Port addressed	*
 *                                                                             	*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_49 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_49 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Delay 4 frames - allow time for MPA to see PnP Write Data Port being addressed					*/
if ( pnp_frame_delay_chk ( (unsigned short int)0x04, gregs, slot ) == 0 ) /*Has 4 frames elapsed?			*/
	return ;							  /*If no, continue probe			*/

/*Have delayed long enough for MPA to "see" control signal setting indicating that output be directed to PnP's Write	*/
/*Data Port, wrt 0x01 to Write Data Port - thus causing PnP board to be activated (see page 47)				*/
oregs->cout_xon_1 	  = 0x01 ;					/*Send Activate					*/
oregs->cout_flow_config  ^= 0x10 ;					/*Initiate xmit'g char				*/
slot->pnp_frame_ctr       = gregs->gicp_frame_ctr ;			/*Snap shot for timing				*/
slot->pnp_probe_funct     = pnp_probe_fn_50 ;
return ;

}	/*End pnp_probe_fn_49												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 * 	50th PnP PROBE FN - Set up to Place PnP back to "Wait for Key" state	*
 *                                                                             	*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_50 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_50 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Wait for Activate byte to be sent											*/
if ( ((oregs->cout_flow_config & 0x10) != (oregs->cout_intnl_flow_ctrl & 0x10)) ||
     ((oregs->cout_intnl_flow_ctrl & 0x08) == 0x08) )			/*Has Activate been sent?			*/
{									/*If no, check deadman				*/
	if ( pnp_frame_delay_chk (DEADMAN_DELAY, gregs, slot) == 0x00 )	/*Has deadman time expired?			*/
	{								/*If no, wait & check again later		*/
		return ;
	}								/*If deadman expired, Modem Pool not functioning*/
	pnp_clean_up_ssp (slot, (unsigned char)0xFF) ;			/*Go clean up so can start over			*/
	return ;
}
oregs->cout_cntrl_sig = 0x111 ;						/*Pt MPA to PnP's Address port (0x279)		*/
slot->pnp_frame_ctr   = gregs->gicp_frame_ctr ;				/*Snap shot for timing				*/
slot->pnp_probe_funct = pnp_probe_fn_51 ;
return ;

}	/*End pnp_probe_fn_50												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 *           51st PnP PROBE FN - Wait till PnP Address Port addressed		*
 *                                                                             	*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_51 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_51 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Delay 4 frames - allow time for MPA to see PnP Address Port being addressed						*/
if ( pnp_frame_delay_chk ( (unsigned short int)0x04, gregs, slot ) == 0 ) /*Has 4 frames elapsed?			*/
	return ;							  /*If no, continue probe			*/

/*Have delayed long enough for MPA to "see" control signal setting indicating that output be directed to PnP's Address	*/
/*Port, now point to PnP Control Register 0x02 (Configuration Control register) by writing 0x02 to PnP Address Port	*/
oregs->cout_xon_1 	  = 0x02 ;					/*Send Cnfg Cntrl Reg Addr to PnP Addr Port	*/
oregs->cout_flow_config  ^= 0x10 ;					/*Initiate xmit'g char				*/
slot->pnp_frame_ctr       = gregs->gicp_frame_ctr ;			/*Snap shot for timing				*/
slot->pnp_probe_funct     = pnp_probe_fn_52 ;
return ;

}	/*End pnp_probe_fn_51												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 *  	 52nd PnP PROBE FN - Set up to place board in "Wait for Key" state	*
 *                                                                             	*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_52 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_52 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Wait till Config. Control register # been sent to PnP Address Port							*/
if ( ((oregs->cout_flow_config & 0x10) != (oregs->cout_intnl_flow_ctrl & 0x10)) ||
     ((oregs->cout_intnl_flow_ctrl & 0x08) == 0x08) )			/*Has Config. Cntrl reg # bits been sent?	*/
{									/*If no, check deadman				*/
	if ( pnp_frame_delay_chk (DEADMAN_DELAY, gregs, slot) == 0x00 )	/*Has deadman time expired?			*/
	{								/*If no, wait & check again later		*/
		return ;
	}								/*If deadman expired, Modem Pool not functioning*/
	pnp_clean_up_ssp (slot, (unsigned char)0xFF) ;			/*Go clean up so can start over			*/
	return ;
}

/*PnP Address register now points to PnP Configuration Control register, now set control signals to address PnP's Write */
/*Data Port														*/
oregs->cout_cntrl_sig = 0x155 ;						/*Pt MPA to PnP's Write Data Port (0xA79)	*/
slot->pnp_frame_ctr   = gregs->gicp_frame_ctr ;				/*Snap shot for timing				*/
slot->pnp_probe_funct = pnp_probe_fn_53 ;
return ;

}	/*End pnp_probe_fn_52												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 *           53rd PnP PROBE FN - Wait till PnP Write Data Port addressed	*
 *                                                                             	*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_53 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_53 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Delay 4 frames - allow time for MPA to see PnP Write Data Port being addressed					*/
if ( pnp_frame_delay_chk ( (unsigned short int)0x04, gregs, slot ) == 0 ) /*Has 4 frames elapsed?			*/
	return ;							  /*If no, continue probe			*/

/*Have delayed long enough for MPA to "see" control signal setting indicating that output be directed to PnP's Write	*/
/*Data Port, wrt 0x02 to Write Data Port - thus causing PnP board to enter "Wait for Key" state				*/
oregs->cout_xon_1 	  = 0x02 ;					/*Send "Wait for Key" state bit			*/
oregs->cout_flow_config  ^= 0x10 ;					/*Initiate xmit'g char				*/
slot->pnp_frame_ctr       = gregs->gicp_frame_ctr ;			/*Snap shot for timing				*/
slot->pnp_probe_funct     = pnp_probe_fn_54 ;
return ;

}	/*End pnp_probe_fn_53												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 * 	54th PnP PROBE FN - Set up to Place PnP back to "Wait for Key" state	*
 *                                                                             	*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_54 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_54 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb      = sspcb ;
iregs	   = iregs ;

/*Wait for "Wait for Key" state bit to be sent										*/
if ( ((oregs->cout_flow_config & 0x10) != (oregs->cout_intnl_flow_ctrl & 0x10)) ||
     ((oregs->cout_intnl_flow_ctrl & 0x08) == 0x08) )			/*Has "Wait for Key" state been sent?		*/
{									/*If no, check deadman				*/
	if ( pnp_frame_delay_chk (DEADMAN_DELAY, gregs, slot) == 0x00 )	/*Has deadman time expired?			*/
	{								/*If no, wait & check again later		*/
		return ;
	}								/*If deadman expired, Modem Pool not functioning*/
	pnp_clean_up_ssp (slot, (unsigned char)0xFF) ;			/*Go clean up so can start over			*/
	return ;
}

  /*PnP Board (if exists) is finally configured and initialized.  Modem Pool still in Plug and Play state (i.e. AUX 	*/
  /*reg's PnP bit is set) thus Modem Pool does not check for UART.  Therefore board has yet to be recognized.  Clear	*/
  /*Modem Pool's AUX Reg PnP bit by issuing "Flush".  When Flush issued, Modem Pool starts probing slot for UART and 	*/
  /*since PnP board (if exists) is configured Modem Pool should recognize board and lower slot's break bit.  Driver	*/
  /*should then recognize that Break bit has been lowered and then call "Register Modem Board" Service/function.	*/
  /*Note: 1. Since Modem Pool recognizes PnP board as soon as "Flush" is issued, "Flush" control signals can be set when*/
  /*	     Driver recognizes PnP board and calls "Register Modem" service/function.					*/
  /*	  2. If no PnP Board present then all above logic was executed for naught.  Consequently, functions below delay	*/
  /*	     approximately one minute and then PnP Probe restarts							*/
slot->pnp_probe_state = 0xFF ;						/*Indicate Slot's board has been PnP Configured	*/
slot->read_toggle     = 0x00 ;						/*Set Slot's Read Toogle since Flush clears bit	*/
oregs->cout_cntrl_sig = 0x188 ;						/*Issue Slot Flush to clear Modem Pool's PnP bit*/
slot->pnp_frame_ctr   = gregs->gicp_frame_ctr ;				/*Snap shot for timing				*/
slot->pnp_probe_funct = pnp_probe_fn_55 ;
return ;

}	/*End pnp_probe_fn_54												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 *           55th PnP PROBE FN - Wait for Flush Control Bits to be sent		*
 *                                                                             	*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_55 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_55 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

#ifndef					RAMP_BSDI				/*R6*/
struct cin_bank_struct FARm		*actv_bank ;	/*Ptr to active bank						*/ /*R2*/
#else										/*BSDI Compile				*/
volatile struct cin_bank_struct FARm	*actv_bank ;	/*Ptr to active bank						*/ /*R2*/
#endif										/*End RAMP_BSDI Conditional		*/



/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb = sspcb ;
oregs = oregs ;

/*Delay 8 frames - allow time for MPA Flush (note: Modem Pool may recognize PnP board while Control signals are set to	*/
/*"Flush")														*/
if ( pnp_frame_delay_chk ( (unsigned short int)0x08, gregs, slot ) == 0 ) /*Has 8 frames elapsed?			*/
	return ;							  /*If no, continue probe			*/

  /*Have delayed long enough for MPA to "see" control signal setting indicating to reset Slot.  Consequently, PnP board */
  /*(if exists) has been seen by Modem Pool (i.e. Modem Pool has cleared Break bit).  If not happened already, Driver	*/
  /*should soon recognize board.  Set up and check if PnP board present (i.e. Break bit = 0).  If board present, point	*/
  /*to function that "spins" on Break = 0 (waiting for Driver to recognize PnP board).  If board not present,		*/
  /* (i.e. Break bit = 1), point to function that delays 5 seconds then restart PnP Porbe.				*/
  /*Note: If Driver did recognize PnP board and called "Register Modem Board" then Slot's state changes such that PnP	*/
  /*	  Probing (and function calls) is stopped.  Hence control may never "reach" this function or the next function.	*/
  /*	  On the other hand if Driver did not recognize PnP board instantly, then control is returned here.		*/
pnp_clean_up_ssp (slot, (unsigned char)0x00) ;				/*Go clean up control signals			*/
actv_bank = ( (iregs->cin_locks & 0x01) == 0x00 ) 		  ? 	/*Is Bank A active?				*/
	      (struct cin_bank_struct FARm *)&(iregs->cin_bank_a) : 	/*If yes	  				*/
	      (struct cin_bank_struct FARm *)&(iregs->cin_bank_b) ; 	/*If no	  					*/
if ( (actv_bank->cin_bank_signals & 0x0001) == 0 )		    	/*Is PnP Brd presnt (but not regstered by Drvr)?*/
{									/*If yes, go spin waiting for Driver recognition*/
	slot->pnp_probe_funct = pnp_probe_fn_56 ;			/*Point to "spin" function			*/
	return ;
}

  /*Break bit = 1 indicating no PnP board present, point to function that delays 5 seconds prior to restarting PnP Probe*/
slot->pnp_probe_parm  = 0x00 ;						/*Init PnP Board recognition ctr		*/
slot->pnp_probe_state = 0x008F ;					/*Assure slot in initial PnP state indicating no*/
									/* PnP board present in slot			*/
slot->pnp_probe_funct = pnp_probe_fn_57 ;				/*Point to delay function			*/
slot->pnp_frame_ctr   = gregs->gicp_frame_ctr ;				/*Snap shot for timing				*/
return ;

}	/*End pnp_probe_fn_55												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 * 56th PnP PROBE FN - PnP Board present - "spin" waiting for Driver to 	*
 *		       recognize board						*
 *                                                                             	*
 ********************************************************************************/
/*R2*/
#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_56 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_56 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

#ifndef					RAMP_BSDI				/*R6*/
struct cin_bank_struct FARm		*actv_bank ;	/*Ptr to active bank						*/ /*R2*/
#else										/*BSDI Compile				*/
volatile struct cin_bank_struct FARm	*actv_bank ;	/*Ptr to active bank						*/ /*R2*/
#endif										/*End RAMP_BSDI Conditional		*/



/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb = sspcb ;
gregs = gregs ;
oregs = oregs ;

/*Check if PnP board still present											*/
actv_bank = ( (iregs->cin_locks & 0x01) == 0x00 ) 		  ? /*Is Bank A active?					*/
	      (struct cin_bank_struct FARm *)&(iregs->cin_bank_a) : /*If yes	  					*/
	      (struct cin_bank_struct FARm *)&(iregs->cin_bank_b) ; /*If no	  					*/
if ( (actv_bank->cin_bank_signals & 0x0001) == 0 )		    /*Is PnP Brd present (but not registered by Driver)?*/
	return ;						    /*If yes, spin till Driver recognizes board		*/
	
/*PnP board no longer present - start PnP Probe over									*/
slot->pnp_probe_state = 0x0000 ;					/*Assure slot in initial PnP state indicating no*/
									/* PnP board present in slot			*/
return ;

}	/*End pnp_probe_fn_56												*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 * 57th PnP PROBE FN - PnP Probe complete and PnP board not present, delay 5 	*
 *		       seconds prior to restarting Probe.  Note: delay provides	*
 *		       window where Modem Pool's PnP I/O bit not set thus 	*
 *		       providing window for Modem Pool to recognize non-PnP 	*
 *		       board insertion.						*
 *                                                                             	*
 ********************************************************************************/
/*R2*/
#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void pnp_probe_fn_57 (struct ssp_struct   FARm *  sspcb,		/*SSP probing being performed on		*/
      			     struct slot_struct  FARm *  slot,		/*MPA Slot probing being performed on		*/
      			     struct mpa_global_s FARm *  gregs,		/*SSP's Global registers			*/
			     struct mpa_input_s  FARm *  iregs,		/*Slot's Input Registers			*/
      			     struct mpa_output_s FARm *  oregs		/*Slot's Output Regsiters			*/
			    )
#else
static void pnp_probe_fn_57 (sspcb, slot, gregs, iregs, oregs)

struct ssp_struct   FARm *  sspcb;					/*SSP probing being performed on		*/
struct slot_struct  FARm *  slot;					/*MPA Slot probing being performed on		*/

#ifndef					RAMP_BSDI				/*R6*/
struct mpa_global_s FARm *  gregs ;					/*SSP's Global registers			*/
struct mpa_input_s  FARm *  iregs ;					/*Slot's Input Registers			*/
struct mpa_output_s FARm *  oregs ;					/*Slot's Output Regsiters			*/
#else										/*BSDI Compile				*/
volatile struct mpa_global_s FARm *  gregs ;				/*SSP's Global registers			*/
volatile struct mpa_input_s  FARm *  iregs ;				/*Slot's Input Registers			*/
volatile struct mpa_output_s FARm *  oregs ;				/*Slot's Output Regsiters			*/
#endif										/*End RAMP_BSDI Conditional		*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)						*/
sspcb = sspcb ;
iregs = iregs ;
oregs = oregs ;

/*Delay 2000 frames (~ .5+ seconds) then increment PnP Board recognition delay counter					*/
if ( pnp_frame_delay_chk ( (unsigned short int)2000, gregs, slot ) == 0 ) /*Has ~ 1/2 second elapsed?			*/
	return ;							  /*If no, continue delay			*/

/*Have delayed for ~ .5+ seconds, increment PnP Board recognition delay counter and check if time for Driver to 	*/
/*recognize PnP board has expired											*/
if ( (++slot->pnp_probe_parm) < (5*2) )					/*Has ~5 secs passed & PnP brd not recognized?	*/
{									/*If no, continue delay				*/
	slot->pnp_frame_ctr = gregs->gicp_frame_ctr ;			/*Snap shot for timing				*/
	return ;
}	

/*~5 seconds have expired and PnP Board yet to be registered, assume Modem Pool unable to recognize PnP board as a	*/
/*UART/modem board.  Restart PnP Probe over!										*/
slot->pnp_probe_state = 0x0000 ;					/*Set PnP Probe state = not initialized		*/
return ;

}	/*End pnp_probe_fn_57												*/







/*#stitle ADMIN/DIAG SERVICE - PnP PROBE FUNCTIONS, CLEAN UP SSP REGS#*/
/*#page*/
/****************************************************************************************
 *                                                                             		*
 *     	 ADMIN/DIAG SERVICE - PnP PROBE FUNCTIONS, CLEAN UP SSP REGS (pnp_clean_up_ssp)	*
 *                                                                     			*
 * PURPOSE: To restore state of SSP registers when Plug and Play Probing aborted or 	*
 *	   completed									*
 *											*
 * NOTES:										*
 *                                                                     			*
 * CALL: void  pnp_clean_up_ssp  (struct slot_struct FARm *    slot,			*
 *			          unsigned char		       type)			*
 *                                                                     			*
 *			where:	slot = Slot Control Block pointer			*
 *				type = Clean up type:					*
 *					o 0x00 = PnP Probe cmplt - board found		*
 *					o 0xFF = PnP Probe abort			*
 *                                                                     			*
 *                                                                     			*
 * RETURN: None										*
 *                                                                     			*
 ****************************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void	pnp_clean_up_ssp (struct slot_struct FARm *    slot,
			          unsigned char	               type)
#else
static void	pnp_clean_up_ssp (slot, type)

struct slot_struct FARm *    slot;
unsigned char	             type;
#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

/*SSP variables relating to MARB's/Request's specific SLOT (Channel)							*/
#ifndef					RAMP_BSDI				/*R6*/
struct mpa_input_s  FARm *  	iregs ;			/*Slot's Input Registers					*/
struct cin_bank_struct FARm	*actv_bank ;		/*Slot's active bank pointer					*/ /*R2*/
struct mpa_output_s FARm *  	oregs ;			/*Slot's Output Regsiters					*/
#else										/*BSDI Compile				*/
volatile struct mpa_input_s  FARm *  	iregs ;		/*Slot's Input Registers					*/
volatile struct cin_bank_struct FARm	*actv_bank ;	/*Slot's active bank pointer					*/ /*R2*/
volatile struct mpa_output_s FARm *  	oregs ;		/*Slot's Output Regsiters					*/
#endif										/*End RAMP_BSDI Conditional		*/

unsigned int			num_chars ;		/*Number of Input Characters to Flush				*/



/************************
 *   START OF CODE	*
 ************************/

/*Restore saved SSP registers												*/
iregs = slot->input ;							/*Slot's input regs ptr				*/
actv_bank = ( (iregs->cin_locks & 0x01) == 0x00 ) 		  ? 	/*Is Bank A active?				*/
	      (struct cin_bank_struct FARm *)&(iregs->cin_bank_a) : 	/*If yes	  				*/
	      (struct cin_bank_struct FARm *)&(iregs->cin_bank_b) ; 	/*If no	  					*/
oregs = slot->output ;							/*Slot's output regs ptr			*/

/*Assure Input Buffer empty i.e. assure slot's/channel's number of characters = 0					*/
for (num_chars = actv_bank->cin_bank_num_chars; num_chars != 0; --num_chars) /*If number of Input Chars != 0, update	*/
	inc_tail (iregs) ;						     /*Tail Pointer so number characters = 0	*/

/*Restore Driver's SSP Registers											*/
oregs->cout_flow_config = (slot->pnp_sv_cout_flow_config & 0x2F) |
                          (oregs->cout_flow_config & 0x10) | 0x40 ;
oregs->cout_cntrl_sig   = (slot->pnp_sv_cout_cntrl_sig & 0x0237) | 0x0004 ;
oregs->cout_xon_1	= slot->pnp_sv_cout_xon_1 ;
#ifndef 			NO_FN_PROTO_PARMS	/*Compile if Protypes OK*/
slot->pnp_clean_up      = (void (*)(struct slot_struct FARm *, unsigned char))0 ;
#else							/*Compile if NO Protypes*/
slot->pnp_clean_up      = (void (*)())0 ;
#endif
if ( type != 0x00 )					/*Clean up due to abort?					*/
	slot->pnp_probe_state   = 0x0000 ;		/*If yes, assure slot in initial PnP state			*/
return ;

}					/*End of pnp_clean_up_ssp							*/








/*#stitle INTERNAL/LOCAL FN - PLUG & PLAY FRAME DELAY CHECK (pnp_frame_delay_chk)#*/
/*#page*/
/************************************************************************************************
 *                                                                             			*
 *               	PLUG & PLAY FRAME DELAY CHECK						*
 *                                                                     				*
 * PURPOSE: To determine if caller's number of SSP Frames have elapsed.				*
 *												*
 * NOTES:											*
 *                                                                     				*
 * CALL: unsigned char  pnp_frame_delay_chk  (unsigned short int              frame_delay,	*
 *		  			      struct icp_global_struct FARm   *gregs,		*
 *					      struct slot_struct FARm         *slot)		*
 *                                                                     				*
 *			where:	frame_delay = number of elapsed frames				*
 *				gregs  	    = SSP Global reg pointer				*
 *				slot        = Slot Control Block pointer			*
 *                                                                     				*
 *                                                                     				*
 * RETURN: Number of Elapsed Frames exceeded flag:						*
 *		0x00 = Elapsed # Frames NOT exceeded						*
 *		0xFF = Elasped # Frames exceeded						*
 *                                                                     				*
 ************************************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static unsigned char	pnp_frame_delay_chk  (unsigned short int              frame_delay,
	  			  	      struct icp_global_struct FARm   *gregs,
				  	      struct slot_struct FARm         *slot
				 	     )
#else
static unsigned char	pnp_frame_delay_chk  (frame_delay, gregs, slot )

unsigned short int              frame_delay;

#ifndef						RAMP_BSDI			/*R6*/
struct icp_global_struct FARm   	*gregs;
#else										/*BSDI Compile				*/
volatile struct icp_global_struct FARm   *gregs;
#endif										/*End RAMP_BSDI Conditional		*/

struct slot_struct FARm         *slot;
#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

/*SSP variables relating to MARB's/Request's specific SLOT (Channel)							*/
unsigned short int	cur_frame_ct ;			/*Current SSP Frame Ctr						*/



/************************
 *   START OF CODE	*
 ************************/

cur_frame_ct = gregs->gicp_frame_ctr ;			/*Get SSP current Frame Counter					*/
if ( cur_frame_ct < slot->pnp_frame_ctr )		/*Did PnP Frame Conter wrap?					*/
	slot->pnp_frame_ctr = 0 ;			/*If yes, adjust						*/
if ( (unsigned short int)(cur_frame_ct - slot->pnp_frame_ctr) < frame_delay )
	return ( (unsigned char)0 ) ;			/*If delay time has not elapsed					*/
return ( (unsigned char)0xFF ) ;			/*If delay time has elapsed					*/

}							/*End of pnp_frame_delay_chk function				*/





/*END*/

