/*****************************************************************************/
/*      EQUINOX SuperSerial Technology (SST) Driver for Linux                */
/*                                                                           */
/*      Copyright (C) 1989-2002 Equinox Systems Inc.                         */
/*****************************************************************************/
                                                                               
/*#title RAMP SOFTWARE SERVICES (RAMPSRVC.C) E Q U I N O X   C O N F I D E N T I A L#*/
/********************************************************************************
 *                                                                              *
 *                  REMOTE ACCESS MODEM POOL(RAMP) SOFTWARE SERVICES            *
 *                  ------------------------------------------------            *
 *                                                                              *
 * PURPOSE: To provide a single set of functions for all Drivers that manages   *
 *         RAMP(s).  This single source makes the details of handling RAMP 	*
 *	   hardware transparent to Driver software.  The software is     	*
 *         written such that it can be migrated to any environment or OS.       *
 *                                                                              *
 * NOTE: - Software was originally developed using:                             *
 *           o WATCOM 32 bit compiler                                           *
 *           o Phar Lap 386|linker                                              *
 *           o Debugged/tested in Phar Lap DOS Extender environment             *
 *	 - "MPA" is used throughout source and is the same as "RAMP"		*
 *                                                                              *
 *                                                                              *
 * FILE:   RAMPSRVC.C                                                           *
 * AUTHOR: BILL KRAMER                                                          *
 * REV:    15 (update "MPA_REV" in Header file, RAMP.H, if change rev here)     *
 * DATE:   01/15/98                                                             *
 *                                                                              *
 *                      E Q U I N O X   C O N F I D E N T I A L            	*
 *                      =======================================            	*
 *                                                                              *
 ********************************************************************************/
/*#page*/
/********************************************************************************
 *                                                                              *
 *                               REVISION HISTORY                               *
 *                               ================                               *
 *                                                                              *
 * REV	  DATE				                     COMMENTS           *
 * ===  ========        ======================================================= *
 *  15	01/15/98	1. Added functions to Register Modem to enable 16550 	*
 *			   FIFO							*
 *			2. Added functions to Flush Input/Output Service to 	*
 *			   flush 16550 UART FIFO				*
 *			3. Added functions to Hard Reset to enable 16550 FIFO	*
 *  14	09/30/97	1. Added conditional compilations to accommodate BSDI &	*
 *			   Linux build environments.  Following symbols, if 	*
 *			   defined, indicate respective compilations:		*
 *				RAMP_LINUX = Linux compile			*
 *				RAMP_BSDI  = BSDI compile			*
 *			   NOTE: RAMP_LINUX & RAMP_BSDI are assumed mutually 	*
 *				 exclusive					*
 *  13	09/12/97	1. Increased delay from 4 frames to 0x2000 in "Register	*
 *			   Modem Service" function, mpa_srvc_reg_modem_1_, to 	*
 *			   accomodate "slow" initializing modems.  This problem	*
 *			   was first identified in May '97 and was patched by	*
 *			   Driver developers (see e-mail of 6/3/97 "Modem Pool	*
 *			   & Slow Modems interim resolution")			*
 *  12  06/10/96	1. Added code to determine slots' Power State and save	*
 *			   in Slot Control Block.				*
 *  11	05/23/96	1. Modified function declarations to accomodate UNIX 	*
 *		           compiliers						*
 *  10	05/20/96	1. See MPA_SRVC.DOC for list of changes			*
 *  09	05/10/96	1. See MPA_SRVC.DOC for list of changes			*
 *  08  03/05/96	1. Fixed warning messages when compiled under "streams"	*
 *  07	02/27/96	1. Fix timing problem in "mpa_srvc_reg_modem" init'g	*
 *			   Input/Output baud rate when rate was set to 0	*
 *  06  02/13/96	1. Add new service "mpa_srvc_modify_settings"		*
 *  05  01/19/96	1. Add new service "mpa_srvc_id_functs"			*
 *			2. Updated function "mpa_srvc_reg_ssp" as follows:	*
 *				A. Check if Sys Functions have been established	*
 *				B. Add addresses of Map/Unmap functions to SSPCB*
 *			3. Added new service - "Hard Reset Modem Board"		*
 *			4. Added stub for new service - "Admin/Diag Poll"	*
 *                                                                              *
 * 04A  01/05/95	1. Fixed bug in "async_req_hndlr" to initialize slotcb	*
 *			   such that a "waited" request appears outstanding	*
 *                                                                              *
 *  04  12/21/95	1. Added, setting "internal loop back" control signal in*
 *			   Register Modem Pool Service thus allowing USRobotics	*
 *			   14.4 Sportster Modem to function.			*
 *			2. Added Frame delay to Set Internal & Clear Internal 	*
 *			   Loop Services.  USRobotics 14.4 Sportster needs delay*
 *			   in order to Loop Back data successfully.		*
 *			3. Modified casting of some statements to satisfy UNIX 	*
 *			   compiler						*
 *                                                                              *
 *  03  11/22/95	Added Loop Back Services				*
 *                                                                              *
 *  02	11/08/95        Version used to perform preliminary RAMP hardware	*
 *			validation						*
 *                                                                              *
 *  01  09/07/95	Partially debugged (achival) version with		*
 *			mpa_srvc_reg_modem subfunctions 14 & 15 in tact.	*
 *                                                                              *
 ********************************************************************************/





/*#stitle HEADER FILE DECLARATIONS#											*/
/*#page*/
/*********************************************************
 * HEADER FILE DECLARATIONS                              *
 *********************************************************/

/*SSP Structure names (used to point to Global, Input, and Output registers)            */
/*NOTE: Macros below must be defined prior to inclusion of RAMP.H.  The macros below are*/
/*	necessary since various development environments utilized different structure	*/
/*	definitions for SSP Global, Input and output registers.  The definitions below	*/
/*	refer to structures defined in ramp_icp.h					*/
#define    mpa_global_s       icp_global_struct		/*Global regs                   */
#define    mpa_input_s        cin_struct              	/*Input regs                    */
#define    mpa_output_s       cout_struct             	/*Output regs                   */


#ifdef				RAMP_PRAGMAS		/*If Compile Time pragmas needed*/
#include	"rampprag.h"
#endif

#ifndef				RAMP_ICP_H_INCLUDED	/*If ramp_icp.h not previously included*/ /*R11*/
#include	"ramp_icp.h"
#endif

#ifndef				RAMP_H_INCLUDED		/*If ramp.h not previously included*/     /*R11*/
#include	"ramp.h"
#endif





/*#stitle STRUCTURE DEFINITIONS/DEFINES/LITERALS DECLACARATIONS#*/
/*#page*/
/************************************************************************
 * Structure definitions & defines/literals                             *
 ************************************************************************/





/************************
 *    MACROS/LITERALS	*
 ************************/











/*#stitle PUBLIC DECLACARATIONS#*/
/*#page*/
/************************************************************************************************
 * Public Declarations i.e. Items DEFINED in this compilation Referenced in other Compilations	 *
 ************************************************************************************************/


/****************
 *  Data Areas	*
 ****************/

/****************************************
 *     Data referenced in RAMPADMN.C	*
 ****************************************/

#ifndef				RAMP_LINUX		/*R14*/

#ifndef				RAMP_BSDI		/*R14*/
struct	admin_diag_hdr_struct	admin_diag_hdr = {{0}};	/*Administrative & Diagnostic	*/
#else							/*BSDI Compile			*/
struct  admin_diag_hdr_struct   admin_diag_hdr = {0};
#endif							/*End RAMP_BSDI Conditional	*/

#else							/*LINUX Compile			*/
struct  admin_diag_hdr_struct   admin_diag_hdr = {0,{0,0,0,0,0},0,0,0,0,0,0};
#endif							/*End RAMP_LINUX Conditional	   */



/***********************
 *  Service Functions  *
 ***********************/

/*NOTE: See MPA.H	*/



/****************************************
 *     Functions called in RAMPADMN.C	*
 ****************************************/

unsigned char	inc_tail (
#ifndef 				NO_FN_PROTO_PARMS	/*R11*/

#ifndef					RAMP_BSDI			/*R14*/
			  	   struct cin_struct FARm   *
#else									/*BSDI Compile			*/
			  volatile struct cin_struct FARm   *
#endif									/*End RAMP_BSDI Conditional	*/

#endif									/*End NO_FN_PROTO_PARMS conditional*/
			  ) ;

unsigned char	mpa_fn_access_sspcb_lst (
#ifndef 				NO_FN_PROTO_PARMS	/*R11*/
					 unsigned long int FARm *
#endif
					) ;

void		mpa_fn_relse_sspcb_lst (
#ifndef 				NO_FN_PROTO_PARMS	/*R11*/
					 unsigned long int
#endif
				       ) ;




/*#stitle EXTERNAL DECLACARATIONS#*/
/*#page*/
/********************************************************
 * DATA EXTERNAL to this compilation                    *
 ********************************************************/

/*#page*/
/********************************************************
 * Functions EXTERNAL to this compilation		*
 ********************************************************/








/*#stitle FORWARD REFERENCE DECLACARATIONS#*/
/*#page*/
/*********************************************************************************************************************
 * Forward Reference Declarations i.e. Functions that are INTERNAL to this compilation but are forward referenced    *
 *********************************************************************************************************************/

/*Register Modem Board "call back" Functions							*/
static void  mpa_srvc_reg_modem_1  ( 
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else									/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				   ) ;

static void  mpa_srvc_reg_modem_1_ (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				   ) ;

static void  mpa_srvc_reg_modem_1_x (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				    ) ;

static void  mpa_srvc_reg_modem_1A (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				   ) ;

static void  mpa_srvc_reg_modem_1B (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				   ) ;


static void  mpa_srvc_reg_modem_1C (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				   ) ;

static void  mpa_srvc_reg_modem_1D (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				   ) ;

static void  mpa_srvc_reg_modem_1E (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				   ) ;

static void  mpa_srvc_reg_modem_1F (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				   ) ;

static void  mpa_srvc_reg_modem_1G (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				   ) ;

static void  mpa_srvc_reg_modem_1H (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				   ) ;

static void  mpa_srvc_reg_modem_1I (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				   ) ;

static void  mpa_srvc_reg_modem_2  (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				   ) ;

static void  mpa_srvc_reg_modem_3  (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				   ) ;

static void  mpa_srvc_reg_modem_4  (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				   ) ;

static void  mpa_srvc_reg_modem_5  (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				   ) ;

static void  mpa_srvc_reg_modem_5a  (					/*R15*/
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				   ) ;


static void  mpa_srvc_reg_modem_5b  (					/*R15*/
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				   ) ;

static void  mpa_srvc_reg_modem_6  (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				   ) ;

static void  mpa_srvc_reg_modem_7  (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				   ) ;

static void  mpa_srvc_reg_modem_8  (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				   ) ;

static void  mpa_srvc_reg_modem_9  (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				   ) ;

static void  mpa_srvc_reg_modem_10 (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				   ) ;

static void  mpa_srvc_reg_modem_11 (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				   ) ;

static void  mpa_srvc_reg_modem_12 (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				   ) ;

static void  mpa_srvc_reg_modem_13 (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				   ) ;

static void  mpa_srvc_reg_modem_16 (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				   ) ;

static void  mpa_srvc_reg_modem_17 (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				   ) ;

static void  mpa_srvc_reg_modem_18 (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				   ) ;

static void  mpa_srvc_reg_modem_19 (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				   ) ;

static void  mpa_srvc_reg_modem_20 (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				   ) ;

static void	mpa_srvc_reg_modem_clean_up (
#ifndef 				NO_FN_PROTO_PARMS
					     struct slot_struct FARm *,
					     unsigned char
#endif
			  		    ) ;

static void	mpa_srvc_reg_modem_cln_rd (					   /*R9*/
#ifndef 				NO_FN_PROTO_PARMS
					     struct slot_struct FARm *,
					     unsigned char
#endif
			  		  ) ;


/*Deregister Modem Board "call back" Functions							*/
static void  mpa_srvc_dereg_modem_1 (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				   	 ) ;


/*Slot Status "call back" Functions								*/
static void  mpa_srvc_slot_status_1 ( 
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				    ) ;


/*Initialize (open) UART/Modem "call back" Functions						*/
static void  mpa_srvc_init_modem_1 (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				   ) ;

static void  mpa_srvc_init_modem_1A  (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				   ) ;

static void  mpa_srvc_init_modem_2 (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				   ) ;

static void  mpa_srvc_init_modem_3 (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				   ) ;

static void  mpa_srvc_init_modem_4 (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				   ) ;

static void  mpa_srvc_init_modem_5 (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				   ) ;

static void  mpa_srvc_init_modem_6 (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				   ) ;

static void  mpa_srvc_init_modem_7 (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				   ) ;


static void  mpa_srvc_init_modem_8 (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				   ) ;

static void  mpa_srvc_init_modem_9 (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
		      		   ) ;

static void  mpa_srvc_init_modem_10 (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				    ) ;

static void  mpa_srvc_init_modem_11 (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				    ) ;

static void  mpa_srvc_init_modem_12 (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				    ) ;

static void  mpa_srvc_init_modem_13 (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				    ) ;

static void	mpa_srvc_init_modem_clean_up (
#ifndef 						NO_FN_PROTO_PARMS
					      struct slot_struct FARm *,
					      unsigned char
#endif
			  		     ) ;

/*Modify UART/Modem Port Settings "call back" Functions						*/
static void  mpa_srvc_mod_sets_1 (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				 ) ;

static void  mpa_srvc_mod_sets_2 (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				 ) ;

static void  mpa_srvc_mod_sets_3 (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				 ) ;

static void  mpa_srvc_mod_sets_4 (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				 ) ;

static void  mpa_srvc_mod_sets_5 (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				 ) ;

static void  mpa_srvc_mod_sets_6 (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				 ) ;

static void  mpa_srvc_mod_sets_7 (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				 ) ;

static void  mpa_srvc_mod_sets_8 (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				 ) ;

static void  mpa_srvc_mod_sets_9 (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
		      		 ) ;

static void  mpa_srvc_mod_sets_10 (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				  ) ;

static void  mpa_srvc_mod_sets_11 (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				  ) ;

static void  mpa_srvc_mod_sets_12 (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				  ) ;

static void	mpa_srvc_mod_sets_clean_up (
#ifndef 				NO_FN_PROTO_PARMS
					    struct slot_struct FARm *,
				            unsigned char
#endif
			  		   ) ;

/*Set Internal Loop Back "call back" Functions							*/
static void  mpa_srvc_set_loop_1 (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
			         ) ;

static void  mpa_srvc_set_loop_2 (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				 ) ;

static void	mpa_srvc_set_loop_clean_up (
#ifndef 				NO_FN_PROTO_PARMS
					    struct slot_struct FARm *,
				            unsigned char
#endif
			  		   ) ;

/*Clear Internal Loop Back "call back" Functions						*/
static void  mpa_srvc_clr_loop_1 (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
			         ) ;

static void  mpa_srvc_clr_loop_2 (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
			         ) ;

static void	mpa_srvc_clr_loop_clean_up (
#ifndef 				NO_FN_PROTO_PARMS
					    struct slot_struct FARm *,
				            unsigned char
#endif
			  		   ) ;

/*Close UART/Modem "call back" Functions							*/
static void  mpa_srvc_close_modem_1 (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				    ) ;

/*Flush MPA Slot Input/Output Buffers "call back" Functions					*/
static void  mpa_srvc_flush_1  (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
			       ) ;

static void  mpa_srvc_flush_2  (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
			       ) ;

static void  mpa_srvc_flush_3  (				/*R15*/
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
			       ) ;


static void  mpa_srvc_flush_4  (				/*R15*/
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
			       ) ;

static void	mpa_srvc_flush_clean_up (
#ifndef 				NO_FN_PROTO_PARMS
					 struct slot_struct     FARm *,
					 unsigned char
#endif
		  		        ) ;

/*Hard Reset Modem Board "call back" Functions							*/
static void  mpa_srvc_hard_rst_1  (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
		                  ) ;

static void  mpa_srvc_hard_rst_2  (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
			          ) ;

static void  mpa_srvc_hard_rst_3  (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
			          ) ;

static void  mpa_srvc_hard_rst_4  (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
			          ) ;

static void  mpa_srvc_hard_rst_5  (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
			          ) ;

static void  mpa_srvc_hard_rst_6  (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
			          ) ;

static void  mpa_srvc_hard_rst_6a  (				/*R15*/
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
			          ) ;


static void  mpa_srvc_hard_rst_6b  (				/*R15*/
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
			          ) ;

static void  mpa_srvc_hard_rst_7  (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
			          ) ;

static void  mpa_srvc_hard_rst_8  (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
			          ) ;

static void  mpa_srvc_hard_rst_9  (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
			          ) ;

static void  mpa_srvc_hard_rst_10 (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
			          ) ;

static void  mpa_srvc_hard_rst_11 (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
			          ) ;

static void  mpa_srvc_hard_rst_12 (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
			          ) ;

static void  mpa_srvc_hard_rst_13 (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
			          ) ;

static void  mpa_srvc_hard_rst_14 (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
			          ) ;

static void  mpa_srvc_hard_rst_15 (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
			          ) ;

static void  mpa_srvc_hard_rst_cmplt (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
			             ) ;

static void	mpa_srvc_hard_rst_clean_up (
#ifndef 				NO_FN_PROTO_PARMS
					    struct slot_struct     FARm *,
					    unsigned char
#endif
		  		           ) ;

/*Input Error Handler "call back" Functions							*/
static void  mpa_srvc_error_1 	(
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				) ;

/*Start Break "call back" Functions								*/
static void  mpa_srvc_start_break_1   (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				) ;

/*Common Break "call back" Functions								*/
static void  mpa_srvc_break_2   (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				) ;

static void  mpa_srvc_break_3   (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				) ;

static void	mpa_srvc_break_clean_up (
#ifndef 				NO_FN_PROTO_PARMS
					 struct slot_struct FARm *,
					 unsigned char
#endif
		  		        ) ;

/*Stop Break "call back" Functions								*/
static void  mpa_srvc_stop_break_1   (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				) ;

/*MPA<->UART Hardware Flow Control "call back" Functions					*/
static void  mpa_srvc_mpa_flow_1   (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				   ) ;

static void  mpa_srvc_mpa_flow_2   (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				   ) ;

static void  mpa_srvc_mpa_flow_3   (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
				   ) ;

static void	mpa_srvc_mpa_flow_clean_up (
#ifndef 				NO_FN_PROTO_PARMS
					    struct slot_struct  FARm *,
					    unsigned char
#endif
		  		           ) ;

/*Read UART Register "call back" Functions							*/
static void  mpa_srvc_rd_uart_1 (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
			        ) ;

static void  mpa_srvc_rd_uart_2 (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
			        ) ;

static void  mpa_srvc_rd_uart_3 (
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
	      			     struct icp_global_struct   FARm *,
				     struct cin_struct 		FARm *,
	      			     struct cout_struct 	FARm *
#else								/*BSDI Compile*/
				     unsigned long,
				     struct marb_struct         FARm *,	
	      			     struct ssp_struct          FARm *,
	      			     struct slot_struct         FARm *,
			    volatile struct icp_global_struct   FARm *,
			    volatile struct cin_struct          FARm *,
			    volatile struct cout_struct         FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
			        ) ;

static void	mpa_srvc_rd_uart_clean_up (
#ifndef 				NO_FN_PROTO_PARMS
					   struct slot_struct     FARm *,
					   unsigned char
#endif
		  		          ) ;

static void	mpa_srvc_rd_uart_cln_rd (
#ifndef 				NO_FN_PROTO_PARMS
					   struct slot_struct     FARm *,
					   unsigned char
#endif
		  		          ) ;
/*#page*/
/*Internal/Local Functions									*/
static unsigned char	val_ssp (				 /*Validate SSP Data Area ptr	*/
#ifndef 				NO_FN_PROTO_PARMS
				 struct ssp_struct FARm *
#endif
				) ;

static unsigned char	val_marb (				/*Validate MARB			*/
#ifndef 				NO_FN_PROTO_PARMS
				   struct marb_struct FARm *
#endif
				 ) ;

static unsigned short 	async_req_hndlr (			 	  /*Async Req Handler	*/
#ifndef 				NO_FN_PROTO_PARMS
					 struct marb_struct FARm *
#endif
					) ;

static unsigned char	frame_delay_chk (				/*Frame Delay Check	*/
#ifndef 				NO_FN_PROTO_PARMS

#ifndef					RAMP_BSDI			/*R14*/
					 unsigned short int,
	  			 	 struct icp_global_struct FARm *,
				 	 struct slot_struct 	  FARm *
#else								/*BSDI Compile*/
					 unsigned short int,
				volatile struct icp_global_struct FARm *,
				volatile struct slot_struct 	  FARm *
#endif									/*End RAMP_BSDI Conditional	   */

#endif									/*End NO_FN_PROTO_PARMS conditional*/
					) ;












/*#stitle GLOBAL (STATIC) DATA AREA#*/
/*#page*/
/******************************************************************************************
 * Global Data Area i.e. Variables declared in Data Segment but local to this compilation *
 ******************************************************************************************/






/*#stitle ADAPTER SERVICE - IDENTIFY SYSTEM FUNCTIONS (mpa_srvc_id_functs)#*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *                          IDENTIFY SYSTEM FUNCTIONS			       *
 *                                                                             *
 *******************************************************************************/
/*R5*/
#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
extern	unsigned short int	mpa_srvc_id_functs (struct sys_functs_struct FARm * import_blk) 
#else
extern	unsigned short int	mpa_srvc_id_functs (import_blk)

struct sys_functs_struct FARm * import_blk ;
#endif


{

/************************
 *     LOCAL DATA	*
 ************************/

unsigned long int	i ;			/*Index/counter used to move	*/
						/*Import Block to Admin/diag hdr*/


/************************
 *   START OF CODE	*
 ************************/

/*Assure call not redundant							*/
if ( admin_diag_hdr.init_flag != 0x00 )		/*Fn addrs been prev spec'd?	*/
	return (ERR_ID_FN_REDUNT) ;		/*If yes, redundant call	*/

/*Assure all Function pointers are valid and coherent i.e.			*/
/*  1. Interrupt Function pointers not 0					*/
/*  2. Lock Function pointer either all 0 OR all not 0				*/
#ifndef 	NO_FN_PROTO_PARMS 			/*Compile if Protypes OK*/
if ( (import_blk->blk_ints_fn  == (unsigned long int(*)(void))0) ||
     (import_blk->rstr_ints_fn == (void(*)(unsigned long int))0)  )
	return (ERR_ID_FN_ADDR_BAD) ;		/*If Interrupt Fn Ptrs bad	*/
if ( import_blk->init_lock_fn   == (void *(*)(unsigned long int FARm *))0 )
{
	if ( (import_blk->attmpt_lock_fn != (unsigned char(*)(void FARm *))0) ||
     	     (import_blk->unlock_fn	 != (void(*)(void FARm *))0) )
		return (ERR_ID_FN_ADDR_BAD) ;	/*If Lock Fn ptrs bad		*/
}
else						/*If int_lock function defined	*/
{
	if ( (import_blk->attmpt_lock_fn == (unsigned char(*)(void FARm *))0) ||
     	     (import_blk->unlock_fn	 == (void(*)(void FARm *))0) )
		return (ERR_ID_FN_ADDR_BAD) ;	/*If Lock Fn ptrs bad		*/
}
#else							/*Compile if NO Protypes*/
if ( (import_blk->blk_ints_fn  == (unsigned long int(*)())0) ||
     (import_blk->rstr_ints_fn == (void(*)())0)               )
	return (ERR_ID_FN_ADDR_BAD) ;		/*If Interrupt Fn ptrs bad	*/
if ( import_blk->init_lock_fn   == (void *(*)())0 )
{
	if ( (import_blk->attmpt_lock_fn != (unsigned char(*)())0) ||
     	     (import_blk->unlock_fn	 != (void(*)())0) 	    )
		return (ERR_ID_FN_ADDR_BAD) ;	/*If Lock Fn ptrs bad		*/
}
else						/*If int_lock function defined	*/
{
	if ( (import_blk->attmpt_lock_fn == (unsigned char(*)())0) ||
     	     (import_blk->unlock_fn	 == (void(*)())0) 	    )
		return (ERR_ID_FN_ADDR_BAD) ;	/*If Lock Fn ptrs bad		*/
}
#endif

/*Caller's request is valid, Initialize Admin/Diag. Services Header		 */
admin_diag_hdr.init_flag = 0xFFFF ;				/*Indicate init'd*/
for ( i = 0; i < sizeof (struct sys_functs_struct); ++i )	/*Set up & move	 */
	*(((unsigned char *)&(admin_diag_hdr.os_functs))+i) = 	/* System Fn ptrs*/
				*(((unsigned char *)import_blk)+i) ;
admin_diag_hdr.sspcb_lst_lck_hndl = (void FARm *)0 ;		 /*Init Lock hndl*/
admin_diag_hdr.sspcb_lock	  = (unsigned long int)0 ;	 /*Init prmtve lk*/
admin_diag_hdr.sspcb_link	  = (struct ssp_struct FARm *)0; /*Init SSPCB hdr*/

/*If system supports/requires LOCKing for muti-processor synchronization, 	*/
/*initialize SSPCB List's LOCK							*/ /*R9*/
#ifndef NO_FN_PROTO_PARMS 				/*Compile if Protypes OK*/
if ( admin_diag_hdr.os_functs.init_lock_fn != (void *(*)(unsigned long int FARm *))0 )
#else							/*Compile if NO Protypes*/
if ( admin_diag_hdr.os_functs.init_lock_fn != (void *(*)())0 )			   /*R11*/
#endif
	admin_diag_hdr.sspcb_lst_lck_hndl =		/*System supports Lockng*/
	     (*admin_diag_hdr.os_functs.init_lock_fn)(&admin_diag_hdr.sspcb_lock) ;

/*Return to caller with good status						*/
return (0x00) ;

}						/*End of mpa_srvc_id_functs Function*/







/*#stitle ADAPTER SERVICE - REGISTER SSP (mpa_srvc_reg_ssp)#*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *                          REGISTER SSP                                       *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
extern unsigned short int	mpa_srvc_reg_ssp (struct ssp_struct        FARm	*  sspcb,
                                	          struct icp_global_struct FARm	*  greg,
						  void 			   FARm *  (* map_fn)( 
									       	       void FARm * map_parm
									   		     ),
						  void			   FARm * map_parm,
						  void			           (* unmap_fn)(
									       	       void FARm * unmap_parm
									                       )
						 )
#else
extern unsigned short int	mpa_srvc_reg_ssp (sspcb, greg, map_fn, map_parm, unmap_fn)

struct ssp_struct        FARm	*  sspcb;
#ifndef						RAMP_BSDI	/*R14*/
struct icp_global_struct FARm	*  greg;
#else								/*BSDI Compile*/
volatile  struct icp_global_struct FARm	*  greg;
#endif								/*End RAMP_BSDI Conditional	   */
void 			 FARm   * (* map_fn)( );
void			 FARm   * map_parm;
void			          (* unmap_fn)( );
#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

/*Variables used to manage List of SSPCBs					*/
unsigned long int	svd_int_msk ;			/*Int Msk prior to Dsabl*/
struct ssp_struct FARm	*sspcb_srch_ptr ;		/*Ptr to SSPCB in List	*/
struct ssp_struct FARm	*current_hd ;			/*Ptr to SSPCB at Lst HD*/

/*General Working Variables							*/
unsigned long int	silent_death_ctr ;		/*Silent death counter	*/
unsigned short int	frame_ctr ;			/*SSP Frame Counter	*/
unsigned short int	i ;				/*Loop counter		*/


/************************
 *   START OF CODE	*
 ************************/

/*Validate caller's parameters							*/
if ( admin_diag_hdr.init_flag != 0xFFFF )	/*Have Fn addrs been spec'd?	*/ /*R5*/
	return (ERR_REG_SSP_NO_FNS) ;		/*If no, error			*/
#ifndef NO_FN_PROTO_PARMS 				/*Compile if Protypes OK*/
if ( map_fn == (void FARm *(*)(void FARm *))0 )	/*Are function addrs consistent?*/
{
	if ( unmap_fn != (void (*)(void FARm *))0 )				   /*R8*/
		return (ERR_REG_SSP_FN_ADR_BAD) ;	  /*If no		*/
}
else
{
	if ( unmap_fn == (void (*)(void FARm *))0 )				   /*R8*/
		return (ERR_REG_SSP_FN_ADR_BAD) ;
}
#else							/*Compile if NO Protypes*/
if ( map_fn == (void FARm *(*)())0 )		/*Are function addrs consistent?*/
{
	if ( unmap_fn != (void (*)())0 )					   /*R8*/
		return (ERR_REG_SSP_FN_ADR_BAD) ;	  /*If no		*/
}
else
{
	if ( unmap_fn == (void (*)())0 )					   /*R8*/
		return (ERR_REG_SSP_FN_ADR_BAD) ;
}
#endif
for (silent_death_ctr = 0xFFFFF, frame_ctr = greg->gicp_frame_ctr; /*Set up & chk*/
     (frame_ctr == greg->gicp_frame_ctr) && 			   /*if Global	 */
     (silent_death_ctr != 0) ;					   /*reg ptr sane*/
     --silent_death_ctr) ;
if (silent_death_ctr == 0)					/*Is Ptr OK?	*/
	return (ERR_REG_SSP_GREG) ;				/*If no, err ret*/
if ( (sspcb->signature == sspcb) || (sspcb->in_use == 0xFF) )
	return (ERR_REG_SSP_SSP_AREA) ;				/*SSP ptr bad	*/

/*Gain access to SSPCB list thus preventing Service functions and Diag/Admin 	*/
/*Poll function from accessing SSPCB list while determining if caller's SSPCB 	*/
/*already in list and while SSPCB being added to list				*/
while ( (mpa_fn_access_sspcb_lst ((unsigned long int FARm *)&svd_int_msk)) != 0xFF ) {}	/*Spin till	*/ /*R9*/
							      	/*access granted*/
for ( silent_death_ctr = 0x200, sspcb_srch_ptr = admin_diag_hdr.sspcb_link ;	   /*R9*/
     (sspcb_srch_ptr   != (struct ssp_struct FARm *)0) &&
     (silent_death_ctr != 0) 			       ;
     sspcb_srch_ptr = sspcb_srch_ptr->sspcb_link,
     --silent_death_ctr				      )	/*Chk if caller's SSPCB	*/
{							/*already in list	*/
	if ( sspcb == sspcb_srch_ptr )			/*SSPCB in list ?	*/
	{						/*If yes, error		*/
		mpa_fn_relse_sspcb_lst (svd_int_msk) ;
		return (ERR_REG_SSP_SSP_AREA) ;		/*SSP ptr bad		*/
	}
}										   /*R9*/

/*Caller's SSPCB valid, Initialize caller's SSPCB				*/
/*NOTE: SSPCB List Locked							*/
for (i = 0; i < sizeof(struct ssp_struct); ++i )
	*(((unsigned char *)sspcb)+i) = 0 ;
sspcb->in_use     = 0xFF ;					/*In Use flag	*/
sspcb->rev        = MPA_REV ;					/*Software rev	*/
sspcb->admin_rev  = MPA_ADMIN_DIAG ;				/*Admin rev	*/  /*R9*/
sspcb->signature  = sspcb ;					/*Signature	*/
sspcb->map_fn     = map_fn ;					/*Sv MAP Fn addr*/
sspcb->map_parm   = map_parm ;					/*Sv MAP FN parm*/
sspcb->unmap_fn   = unmap_fn ;					/*Sv UNMAP addr	*/
sspcb->global     = greg ;					/*SSP Global reg*/
for (i = 0; i < ENGS_MPA; ++i)
	sspcb->mpa[i].base_chan = 0xFF ;			/*Eng. base chan*/

/*Set up and add SSPCB to Administrative & Diagnostic Service Header's Chain of	*/
/*Registered SSPCBs 								*/
/*NOTE: SSPCB List Locked							*/
current_hd 		  = admin_diag_hdr.sspcb_link ;	/*Addr of SSPCB @ Lst HD  */
admin_diag_hdr.sspcb_link = sspcb ;			/*Caller's SSPCB to Lst HD*/
sspcb->sspcb_link 	  = current_hd ;		/*Chain list to new SSPCB */

/*Release SSPCB List and return to caller					  */
mpa_fn_relse_sspcb_lst (svd_int_msk) ;						    /*R9*/
return (0x00) ;

}						/*End of mpa_srvc_reg_ssp Function*/









/*#stitle ADAPTER SERVICE - REGISTER MPA (mpa_srvc_reg_mpa)#*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *                          REGISTER MPA                                       *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
extern unsigned short int	mpa_srvc_reg_mpa (struct ssp_struct FARm *  sspcb,
                                	  	  unsigned char		    chan_num,
					  	  struct slot_struct FARm   slot[],
					  	  struct cin_struct FARm *  ireg)
#else

extern unsigned short int	mpa_srvc_reg_mpa (sspcb, chan_num, slot, ireg)

struct ssp_struct FARm *  sspcb ;
unsigned char		  chan_num;
struct slot_struct FARm   slot[];
#ifndef					RAMP_BSDI		/*R14*/
struct cin_struct FARm *  ireg;
#else								/*BSDI Compile		  */
volatile struct cin_struct FARm *  ireg;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

/*Variables used in accessing SSP registers					*/
unsigned short int		lmx_id ;		/*LMX ID frm off-lne TDM*/

/*MPA hardware variables							*/
unsigned char			mpa_id ;		/*MPA ID		*/
unsigned char			num_engs ;		/*MPA's # Engs (1,2,4)	*/
unsigned char			sp_mul_msk ;		/*SSP's LMX Scan sp mask*/
unsigned char			sp_mul_set ;		/*SSP's LMX Scan sp setg*/
unsigned int			slots_per_eng ;		/*# Slots per MPA Eng	*/
unsigned char			rev ;			/*Modem Pool's rhdwr REV*/ /*R12*/

/*Variables used to gain access to SSPCB List					*/
unsigned long int		svd_int_msk ;		/*Int Msk prior to Dsabl*/

/*SSPCB (SSP Control Block), MPACB (MPA Control Block) and SLOTCB (Slot 	*/
/*Control Block) variables							*/
unsigned int			mpa_base_index ;	/*1st MPA's MPACB index	*/
							/*or # 1st MPA on Ring	*/
unsigned int			mpa_index ;		/*Working MPA Array indx*/
unsigned char			eng_num ;		/*MPA current Engine #	*/
struct slot_struct FARm		*slot_member ;		/*Eng's Slot Member	*/
#ifndef					RAMP_BSDI		/*R14*/
struct cin_struct  FARm		*slot_iregs ;		/*Slot's SSP Input reg	*/
struct cout_struct FARm		*slot_oregs ;		/*Slot's SSP Output reg	*/
#else								/*BSDI Compile		  */
volatile struct cin_struct  FARm *slot_iregs ;		/*Slot's SSP Input reg	*/
volatile struct cout_struct FARm *slot_oregs ;		/*Slot's SSP Output reg	*/
#endif								/*End RAMP_BSDI Conditional*/
struct mpa_struct  FARm		*actv_eng;		/*MPA member be'g init'd*/

/*General working variables							*/
unsigned int			i ;			/*Loop counter		*/
unsigned int			j ;			/*Loop counter		*/



/************************
 *   START OF CODE	*
 ************************/

/*Validate caller's parameters							*/
if ( (sspcb->in_use    != 0xFF)     ||			/*Does caller's SSP	*/
     (sspcb->rev       != MPA_REV)  ||			/* Control Block apper	*/
     (sspcb->signature != sspcb) )			/*  sane?		*/
	return (ERR_REG_MPA_SSP_AREA) ;			/*If no, exit		*/
if ( (chan_num & 0xCF) != 0 )				/*Is caller's chan # OK?*/
	return (ERR_REG_MPA_CHAN_NUM) ;			/*If no, exit		*/
for ( i = 0; i < SLOT_ARRAY_SZ; ++i)			/*Is caller's Slot array*/
	if (slot[i].in_use == 0xFF)			/* in use?		*/
		return (ERR_REG_MPA_SLOT_ARRY) ;	/*If yes, exit		*/

/*Set up and get MPA's number of engines					*/
lmx_id = ireg[1].cin_tdm_early ;			/*Get off-line TDM w/ID	*/
if ( (lmx_id & 0xC400) != 0xC000 )			/*Is TDM sane?		*/
	return (ERR_REG_MPA_IN_REG) ;			/*If no,caller's ptr bad*/
mpa_id = (unsigned char)((unsigned int)(lmx_id & 0x01FE) >> 1); /*Develop MPA ID*/
if ( (mpa_id != 0x08) &&				/*Is MPA ID valid?	*/
     (mpa_id != 0x09) &&
     (mpa_id != 0x0B) )
	return (ERR_REG_MPA_ID_BAD) ;			/*If MPA ID bad		*/
num_engs = ((mpa_id & 3) == 0) ? 1 : 			/*Develop # MPA engs 	*/
	   ( ((mpa_id & 3) == 1) ? 2 : 4) ;

/*Set up and validate that MPA & its engines have not been previously registered*/
mpa_base_index = chan_num >> 4 ;			/*Index to 1st MPACB	*/
for (mpa_index = mpa_base_index, i = 0;			/*Loop to check if SSP's*/ 
     i < num_engs; 					/* MPA & Engines already*/
     ++mpa_index, ++i)					/*  registered		*/
{
	if ( mpa_index >= ENGS_MPA )			/*Did exceed MPA/Eng Array*/
		return (ERR_REG_MPA_ENG_OFL) ;		/*If yes, config error	*/
	if ( sspcb->mpa[mpa_index].base_chan != 0xFF )	/*Is engine alrdy def'd?*/
		return (ERR_REG_MPA_ENG_ALLOC) ;	/*If yes, return w/error*/
}

/*Caller's parms & MPA Hardware are OK, initialize SSP's speed multiplier based	*/
/*on MPA's starting position in Ring and MPA's number of Engines		*/
switch (mpa_base_index)					/*Position of 1st MPA	*/
{							/* channel on Ring:	*/
	case 0:						/*MPA @ LMX 0		*/
		switch (num_engs)
		{
			case 1:				/*SP X, 1 eng @ LMX 0	*/
				sp_mul_msk = 0xFC ;
				sp_mul_set = 0x00 ;
				break ;
			case 2:				/*SP X, 2 engs @ LMX 0	*/
				sp_mul_msk = 0xF0 ;
				sp_mul_set = 0x05 ;
				break ;
			case 4:				/*SP X, 4 engs @ LMX 0	*/
				sp_mul_msk = 0x00 ;
				sp_mul_set = 0xAA ;
				break ;
			default:
				return (ERR_REG_MPA_ENG_OFL) ;
		}
		break ;
	case 1:						/*MPA @ LMX 1		*/
		switch (num_engs)
		{
			case 1:				/*SP X, 1 eng @ LMX 1	*/
				sp_mul_msk = 0xF3 ;
				sp_mul_set = 0x00 ;
				break ;
			case 2:				/*SP X, 2 engs @ LMX 1	*/
				sp_mul_msk = 0xC3 ;
				sp_mul_set = 0x14 ;
				break ;
			default:
				return (ERR_REG_MPA_ENG_OFL) ;
		}
		break ;
	case 2:						/*MPA @ LMX 2		*/
		switch (num_engs)
		{
			case 1:				/*SP X, 1 eng @ LMX 2	*/
				sp_mul_msk = 0xCF ;
				sp_mul_set = 0x00 ;
				break ;
			case 2:				/*SP X, 2 engs @ LMX 2	*/
				sp_mul_msk = 0x0F ;
				sp_mul_set = 0x50 ;
				break ;
			default:
				return (ERR_REG_MPA_ENG_OFL) ;
		}
		break ;
	case 3:						/*MPA @ LMX 3		*/
		switch (num_engs)
		{
			case 1:				/*SP X, 1 eng @ LMX 3	*/
				sp_mul_msk = 0x3F ;
				sp_mul_set = 0x00 ;
				break ;
			default:
				return (ERR_REG_MPA_ENG_OFL) ;
		}
		break ;
	default:					/*Should not occur, but	*/
		return (ERR_REG_MPA_CHAN_NUM) ;		/* just in case		*/
}
sspcb->sv_sp_mul_msk = sp_mul_msk ;			/*Sv for sanity chk'g	*/
sspcb->sv_sp_mul_set = sp_mul_set ;			/*Sv for sanity chk'g	*/
sspcb->global->gicp_scan_spd &= sp_mul_msk ;		/*Clr MPA Eng's SP X bits*/
sspcb->global->gicp_scan_spd |= sp_mul_set ;		/*Set MPA Eng's SP X bits*/

/*Background caller's Slot array area to 0					*/
for (i = 0; i < (sizeof(struct slot_struct)*SLOT_ARRAY_SZ); ++i )
	((unsigned char *)slot)[i] = 0 ;

/*Prepare to initialize each engine's MPA Control Block				*/
slots_per_eng = SLOT_ARRAY_SZ/num_engs ;		/*# slots per MPA Eng.	*/
eng_num       = 0 ;					/*Initialize Eng. #	*/
slot_member   = slot ;					/*Eng's 1st Slot member	*/
slot_iregs    = ireg ;					/*Eng's 1st Slot inp reg*/
slot_oregs    = (struct cout_struct FARm *)(((unsigned char FARm *)ireg) + 
		((sizeof(struct cin_struct)*64))) ;
rev	      = (ireg->cin_tdm_early >> 1) & 0x0F ;	/*Get Modem Pool's Rev	*/ /*R12*/

/*For caller's SSPCB, prepare to initialize each eng's MPA Control Block (MPACB)*/
/*but first gain access to SSPCB List thus Blocking Admin/Diag Poll function  	*/
/*from traversing and potentually accessing a partially initialized MPACB	*/ /*R9*/
while ( (mpa_fn_access_sspcb_lst ((unsigned long int FARm *)&svd_int_msk)) != 0xFF ) {} /*Spin till access*/
for (mpa_index = mpa_base_index, i = 0;			/*Loop to init. SSP's	*/
     i < num_engs; 					/* MPA/Eng and Slot	*/
     ++mpa_index,					/*  array		*/
     ++eng_num,
     ++i)
{ 
     /*Initialize SSP's MPACB							*/
	actv_eng = &(sspcb->mpa[mpa_index]) ;		/*Point to MPA/Eng memb	*/
	actv_eng->base_chan      = chan_num ;		/*MPA's Ring Position 	*/
	actv_eng->mpa_num_engs   = num_engs ;		/*MPA's Number of Engs.	*/
	actv_eng->input_base     = ireg ;		/*MPA's 1st Input reg	*/
	actv_eng->slot_array_hd  = slot ;		/*Hd of MPA's Slot Array*/
	actv_eng->mpa_hrdwre_rev = rev ;		/*Save MPA's Hrdwre REV	*/ /*R12*/
	actv_eng->eng_num        = eng_num ;		/*MPA array member Eng #*/
	actv_eng->eng_head       = slot_member ;	/*MPA Eng's 1st Slot mem*/

     /*Initialize MPA (Engine's) Slot Members AND each slot's Internal Loop Back*/
     /*Control signal (SSP Output Control Signal register) thus stopping "flood"*/
     /*of UART Interrupts received by RAMP.  This is necessary since some 	*/
     /*Modem's interrupt line is "floating" until Slot is initialized.		*/
	for (j = 0; j < slots_per_eng;			/*Loop to init Eng's	*/
	     *(unsigned long int FARm *)&slot_member += sizeof(struct slot_struct), ++j)
	{						/* Slot members		*/
		slot_member->in_use    = 0xFF ;		/*Set Mem in use flag	*/
		slot_member->alloc     = 0xFF ;		/*Indicate member init'd*/
		slot_member->signature = slot_member ;	/*Member's signature	*/
		slot_member->input     = slot_iregs+j ;	/*Slot's SSP Input regs	*/
		slot_member->output    = slot_oregs+j ;	/*Slot's SSP Output regs*/
/*R9*/		slot_member->global    = sspcb->global;	/*Slot's Global Reg base*/
		slot_member->chan_num  = chan_num+(i * 0x10)+j; /*Slot's Chan #	*/
/*R9*/		slot_member->slot_state      = 0x00 ;	/*PnP slot state = empty*/
/*R9*/		slot_member->pnp_probe_state = 0x0000 ;	/*PnP Probe state = 	*/
							/* Probing not init'd	*/
	     /*Initialize Slot's Internal Loop Back Control Signals		*/
/*R4*/		(slot_oregs+j)->cout_cntrl_sig = ((slot_oregs+j)->cout_cntrl_sig & 0x02BF) | 0x008C ;
	}						/*End of FOR loop init'g*/
							/*an Eng's Slot member	*/
	*(unsigned long FARm *)&slot_iregs  += 		/*Nxt Eng(LMX) Input regs*/
		(0x10 * sizeof(struct cin_struct)) ;
	*(unsigned long FARm *)&slot_oregs  += 		/*Nxt Eng(LMX) Output regs*/
		(0x10 * sizeof(struct cout_struct)) ;
}							/*End of FOR loop init'g*/
							/*SSP MPA/Eng member(s)	*/

/*SSP MPACBs and corresponding Slot members initialized, release SSPCB List and */
/*return to caller with good status						*/
mpa_fn_relse_sspcb_lst (svd_int_msk) ;						  /*R9*/
return (0x00) ;

}					/*END of mpa_srvc_reg_mpa function	*/







/*#stitle ADAPTER SERVICE - DEREGISTER MPA (mpa_srvc_dereg_mpa)#*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *                          DE-REGISTER MPA                                    *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
extern unsigned short int	mpa_srvc_dereg_mpa (struct ssp_struct FARm *       sspcb,
                                	  	  unsigned char		           chan_num,
					  	  struct slot_struct FARm * FARm * slot_ret_parm)
#else
extern unsigned short int	mpa_srvc_dereg_mpa (sspcb, chan_num, slot_ret_parm)

struct ssp_struct FARm *       	  sspcb;
unsigned char		       	  chan_num;
struct slot_struct FARm * FARm *  slot_ret_parm ;
#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

/*SSPCB (SSP Control Block), MPACB (MPA Control Block) and SLOTCB (Slot 	*/
/*Control Block) variables							*/
unsigned int		mpa_index ;			/*MPA Array member	*/
struct slot_struct FARm	*slot_member ;			/*MPA's Slot Member	*/

/*MPA hardware variables							*/
unsigned int		num_engs ;			/*MPA's # Engs (1,2,4)	*/

/*Variables used to gain access to SSPCB List					*/
unsigned long int	svd_int_msk ;			/*Int Msk prior to Dsabl*/

/*Variables used to clean up SLOTCB						*/
unsigned int			slots_per_eng ;		/*# slots per eng	*/
unsigned char			eng_slot_num_base ;	/*Multi-eng Slot # base	*/
unsigned char			eng_slot_num_offset ;	/*Multi-eng Slot # offst*/
struct marb_struct FARm *	can_marb ;		/*Canceled MARB addr	*/

/*General working variables							*/
unsigned char			status ;		/*Funct. ret status	*/
unsigned short int		short_status ;		/*Funct. ret status	*/
unsigned int			i ;			/*Loop counter		*/



/************************
 *   START OF CODE	*
 ************************/

/*Validate caller's parameters							*/
if ( (status = val_ssp (sspcb)) != 0x00 )		/*Is caller's parm OK?	*/
	return (ERR_DREG_MPA_SSP_AREA) ;		/*If no, exit		*/
if ( (chan_num & 0xCF) != 0 )				/*Is caller's chan # OK?*/
	return (ERR_DREG_MPA_CHAN_NUM) ;		/*If no, exit		*/
mpa_index   = chan_num >> 4 ;				/*Index to 1st MPACB	*/
if ( sspcb->mpa[mpa_index].base_chan == 0xFF )		/*Chan # pt to valid MPA*/
	return (ERR_DREG_MPA_BAD_ENG) ;			/*If no, exit		*/
if ( sspcb->mpa[mpa_index].eng_num != 0x00 )		/*Chan # pt to 1st eng	*/
	return (ERR_DREG_MPA_BAD_ENG) ;			/*If no, exit		*/

/*Init variables so can free SSP MPA Array members and cooresponding Slot area	*/
slot_member   = sspcb->mpa[mpa_index].slot_array_hd;	/*MPA's Slot Array base	*/
num_engs      = sspcb->mpa[mpa_index].mpa_num_engs ;	/*MPA # Engs.		*/
slots_per_eng =  (num_engs == 1) ? 16 :			/*Get # slots per engine*/ /*R9*/
		((num_engs == 2) ?  8 : 4 ) ;

/*Prior to "cleaning up" each MPA Slot, gain access to SSPCB List thus Blocking */
/*Admin/Diag Poll function from traversing and potentually accessing a SLOTCB	*/
/*that is in process of being cleaned up					*/ /*R9*/
while ( (mpa_fn_access_sspcb_lst ((unsigned long int FARm *)&svd_int_msk)) != 0xFF ) {} /*Spin till access granted*/
for ( eng_slot_num_base = 0, eng_slot_num_offset = 0, i = 0;
      i < SLOT_ARRAY_SZ;
      ++eng_slot_num_offset, ++i)
{
	if ( eng_slot_num_offset >= slots_per_eng )	/*If cleaning up next	*/
	{						/* eng's Slot's		*/
		eng_slot_num_base   += 0x10 ;		/*Get nxt eng's 1st slot*/
		eng_slot_num_offset  = 0 ;		/* number		*/
	}
	if (slot_member[i].slot_state != 0x00)		/*Is Slot Empty?	*/
	{						 /*If no, go cancel any	*/
		short_status = mpa_srvc_can_marb (sspcb, /* outstanding MARB	*/
				 	  (unsigned char)(chan_num+eng_slot_num_base+eng_slot_num_offset),
					  (struct marb_struct FARm * FARm *)&can_marb) ;
	}
	else						/*Empty Slot, go cancel	*/
	{						/* PnP Probing		*/
		if ( (sspcb->pnp_probe_ena == 0xFF) 		&&
/*R10*/		     (slot_member[i].pnp_probe_state != 0x0000) &&
#ifndef 				NO_FN_PROTO_PARMS	  /*Compile if Protypes OK*/
     		     (slot_member[i].pnp_clean_up    != (void (*)(struct slot_struct FARm *, unsigned char))0) )
#else								  /*Compile if NO Protypes*/
     		     ((slot_member[i].pnp_clean_up   != (void (*)())0)) )
#endif
		{					/*If PnP Probe active	*/
			(*slot_member[i].pnp_clean_up)(&slot_member[i], (unsigned char)0xFF) ;
		}
	}
	slot_member[i].in_use          = 0x00 ;		/*Free Slot member	*/
	slot_member[i].alloc           = 0x00 ;		/*Elm. uninit'd		*/
	slot_member[i].slot_state      = 0x00 ;		/*Indicate no Modem prsnt*/
	slot_member[i].init_state      = 0x00 ;		/*MODEM/UART Closed	*/
	slot_member[i].pnp_probe_state = 0x0000 ;	/*PnP not initialized	*/
}

/*Set up & free all Engines associated with caller's MPA			*/
for (i = 0; i < num_engs; ++mpa_index, ++i)		/*Loop to free MPA Engs	*/
	sspcb->mpa[mpa_index].base_chan = 0x0FF ;	/*Indicate MPA (Eng) not*/
							/* on Ring		*/

/*Caller's MPA is de-registered, release SSPCB List and return pointer to freed	*/
/*Slot area									*/
mpa_fn_relse_sspcb_lst (svd_int_msk) ;						 /*R9*/
*slot_ret_parm = slot_member ;
return (0x00) ;

}					/*END of mpa_srvc_dereg_mpa function	*/







/*#stitle SLOT SERVICE - REGISTER MODEM BOARD (mpa_srvc_reg_modem)#*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *                  REGISTER MODEM BOARD - SERVICE ENTRY POINT                 *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
extern unsigned short int	mpa_srvc_reg_modem (struct marb_struct FARm  *marb)
#else
extern unsigned short int	mpa_srvc_reg_modem (marb)

struct marb_struct FARm  *marb;
#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

/*General working variables							*/
unsigned char		status ;			/*Funct. ret status	*/



/************************
 *   START OF CODE	*
 ************************/

status = val_marb (marb) ;				/*Validate caller's MARB*/
if ( status != 0 )					/*Is MARB OK to use?	*/
	return (ERR_REG_MODEM_MARB) ;			/*If no, return error	*/
if ( marb->in_use == 0 )				/*Initial MARB Request?	*/
{							/*If yes, init Fn ptr	*/
	marb->cont_funct = mpa_srvc_reg_modem_1 ;	/*Init MARB's FN ptr	*/
}
else							/*"Call Back" MARB,	*/
{							/*Validate SRVC active	*/
	if ( marb->actv_srvc != MARB_ACTV_SRVC_REG_MODEM ) /*Valid "Call Back"?	*/
	{						/*If no, exit		*/
		marb->srvc_status = ERR_REG_MODEM_CALL_BK ;
		marb->srvc_state  = 0x00 ;
		return ((unsigned short int)(((unsigned short int)(marb->srvc_state)<<8)+
                                               marb->srvc_status) ) ;
	}
}
return ( async_req_hndlr(marb) ) ;			/*Go handle request	*/

}			/*END of mpa_srvc_reg_modem Service Entry function	*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *             REGISTER MODEM BOARD - 1st FUNCTION, INIT SLOT SSP REGS.	       *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_reg_modem_1  ( unsigned long		        caller_parm,
 				     struct marb_struct FARm *  	marb,
	      			     struct ssp_struct FARm *   	sspcb,
	      			     struct slot_struct FARm *  	slot,
	      			     struct icp_global_struct FARm * 	gregs,
				     struct cin_struct FARm *	  	iregs,
	      			     struct cout_struct FARm *	  	oregs
				   )
#else
static void  mpa_srvc_reg_modem_1  (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;
#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

/*Variables used to determine MPA Slot Power States				    */ 	/*R12*/
#ifndef					RAMP_BSDI		/*R14*/
struct cin_struct  FARm *	mpa_1st_iregs ;		/*MPA's 1st Slot Input regs */
#else								/*BSDI Compile		  */
volatile struct cin_struct FARm * mpa_1st_iregs ;	/*MPA's 1st Slot Input regs */
#endif								/*End RAMP_BSDI Conditional*/
struct slot_struct FARm *  	mpa_1st_slot ;		/*MPA's 1st SLOTCB	    */
unsigned char			slot_pwr_state ;	/*MPA SLOT PWR State	    */

/*Variables used in accessing SSP registers					*/
#ifndef					RAMP_BSDI		/*R14*/
struct cin_bank_struct FARm *	actv_bank ;		/*Ptr to active bank	*/
#else								/*BSDI Compile		  */
volatile struct cin_bank_struct FARm *	actv_bank ;	/*Ptr to active bank	*/
#endif								/*End RAMP_BSDI Conditional*/
unsigned short int		ssp_ushort_var ;	/*Used to modify 16 bit	*/
							/*SSP registers		*/
unsigned char			ssp_uchar_var ;		/*Used to modify 8 bit	*/
							/*SSP registers		*/

/*Variables used to gain access to SSPCB List					*/
unsigned long int		svd_int_msk ;		/*Int Msk prior to Dsabl*/

/*General working variables							*/
unsigned int			i ;			/*Loop counter		*/
unsigned int			j ;			/*Loop counter		*/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;

/*For request's MPA, update the Power Switch state for each Slot Member	     			*/ 	/*R12*/
mpa_1st_iregs = sspcb->mpa[(marb->slot_chan >> 4)].input_base ;   /*Pt to MPA's base Input regs	*/
mpa_1st_slot  = sspcb->mpa[(marb->slot_chan >> 4)].slot_array_hd; /*Pt to 1st SLOT to update	*/
for ( i = 0; i < 16; i = i + 4 )				  /*Update MPA's SLOT PWR States*/
{
	actv_bank = ( ((mpa_1st_iregs+i)->cin_locks & 0x01) == 0x00 )		?
      		(struct cin_bank_struct FARm *)&((mpa_1st_iregs+i)->cin_bank_a) :
      		(struct cin_bank_struct FARm *)&((mpa_1st_iregs+i)->cin_bank_b) ;
	slot_pwr_state = ( ((actv_bank->cin_bank_signals) & 0x0800) == 0x0000)  ?
		(unsigned short int)0x0000 : (unsigned short int)0x00FF 	;
	for ( j = 0; j < 4; ++j )				  /*Loop to update SLOT PWR State*/
		(mpa_1st_slot+i+j)->power_state = slot_pwr_state ;
}													/*R12*/

/*Complete Request's validation							*/
if ( slot->req_outstnd != 0x00 )			/*Is a Req Outstand'g?	*/
{							/*If yes, return error	*/
	marb->srvc_status = ERR_REG_MODEM_REQ_OUT ;	/*Indicate error	*/
	return ;					/*Exit			*/
}
if ( slot->slot_state != 0x00 )				/*Is Modem already reg?	*/
{							/*If yes, error		*/
	marb->srvc_status = ERR_REG_MODEM_REG ;		/*Indicate error	*/
	return ;					/*Exit			*/
}
actv_bank = ( (iregs->cin_locks & 0x01) == 0x00 ) 		  ? /*Bank A actv?*/
	      (struct cin_bank_struct FARm *)&(iregs->cin_bank_a) : /*If yes	  */
	      (struct cin_bank_struct FARm *)&(iregs->cin_bank_b) ; /*If no	  */
if ( (actv_bank->cin_bank_num_chars) != 0 )		/*Is Input Queue empty?	*/
{							/*If no, error		*/
	marb->srvc_status = ERR_REG_MODEM_IN_Q;		/*Indicate error	*/
	return ;					/*Exit			*/
}

/*Caller's request valid, gain access to SSPCB List thus blocking Admin/Diag Poll*/  /*R9*/
/*Function from traversing and potentually accessing a SLOTCB that is in process */
/*of being initialized.								*/
while ( (mpa_fn_access_sspcb_lst ((unsigned long int FARm *)&svd_int_msk)) != 0xFF ) {} /*Spin till access granted*/
slot->type = (slot->pnp_probe_state != 0xFF) ? 0x00 : 0x01  ; /*Indicate Board type	*/
if ( (sspcb->pnp_probe_ena  == 0xFF)   &&		      /*Is Slot PnP Probing	*/
     (slot->pnp_probe_state != 0x0000) &&		      /*  active? 	        */
#ifndef 			NO_FN_PROTO_PARMS	      /*Compile if Protypes OK*/
     (slot->pnp_clean_up    != (void (*)(struct slot_struct FARm *, unsigned char))0) )
#else							/*Compile if NO Protypes*/
     (slot->pnp_clean_up    != (void (*)())0) )
#endif
{							      /*If yes, cancel Probing  */
	(*slot->pnp_clean_up)(slot, (unsigned char)0xFF) ;
}

/*Save SSP registers that will be used to Register Modem Board			*/
slot->sv_cout_flow_config = oregs->cout_flow_config ;
slot->sv_cout_cntrl_sig   = oregs->cout_cntrl_sig ;
slot->sv_cout_xon_1       = oregs->cout_xon_1 ;

/*Establish "clean up" function in case	De-register Modem request handled 	*/
/*asynchronously								*/
#ifndef 				NO_FN_PROTO_PARMS /*Compile if Protypes OK*/
slot->clean_up            = (void(*)(struct slot_struct FARm *, unsigned char))mpa_srvc_reg_modem_clean_up ;
#else							  /*Compile if NO Protypes*/
slot->clean_up            = (void(*)())mpa_srvc_reg_modem_clean_up ;
#endif

/*Initialize MARB's Slot Control Block						*/
slot->slot_state      = 0x80 ;				/*Modem be'g registered	*/
slot->io_base         = 0x00 ;				/*I/O base = invalid	*/
slot->init_state      = 0x00 ;				/*MODEM/UART closed	*/
slot->read_toggle     = 0x00 ;				/*UART I/O Read toggle	*/
slot->flow_setting    = marb->sup_uchar0 ;		/*Save flow setting	*/
slot->num_bits	      = marb->sup_uchar1 ;		/*Save # bits setting	*/
slot->stop_bits	      = marb->sup_uchar2 ;		/*Save # stop bits	*/
slot->parity   	      = marb->sup_uchar3 ;		/*Save parity setting	*/
slot->baud	      = marb->sup_ushort ;		/*Save baud setting	*/
slot->break_state     = 0x00 ;				/*Indicate Break = off	*/
slot->loop_back       = 0x00 ;				/*Loop Back state = OFF	*/
slot->pnp_probe_state = 0x0000 ;			/*PnP not initialized	*/
mpa_fn_relse_sspcb_lst (svd_int_msk) ;			/*Release SSPCB List	*/ /*R9*/

if ( marb->scratch_area == (unsigned char FARm *)0 )	/*Is scratch area spec'd*/
	for ( i = 0; i < SLOT_SCRATCH_SZ; ++i )		/*If no, clear (0) slot	*/
		slot->rscratch[i] = (unsigned char)0x00; /* area		*/
else
	for ( i = 0; i < SLOT_SCRATCH_SZ; ++i )		/*If yes, copy area	*/
		slot->rscratch[i] = marb->scratch_area[i] ;
for ( i = 0; i < SLOT_SCRATCH_SZ; ++i )			/*Clear scratch spec'd	*/
	slot->oscratch[i] = (unsigned char)0x00 ;	/*during Init/Open Req	*/

/*Initialize Slot's SSP channel registers:					*/
/*	*** Transmint & Receive char attributes (8,N,1)				*/
ssp_ushort_var  = (iregs->cin_char_cntrl & 0xFFE0) ;	/*Set up SSP Input to 	*/
ssp_ushort_var |= 0x03 ;				/*recv 8 bits, no parity*/
iregs->cin_char_cntrl = ssp_ushort_var ;
oregs->cout_char_fmt = 0x03 ;				/*SSP output = 8, none	*/

/*	*** Transmint & Receive Baud Rate = Top rate				*/
iregs->cin_baud = 0x7FFF ;				/*Input baud rate = TOP	*/
oregs->cout_baud_rate = 0x7FFF ;			/*Output baud rate = TOP*/
while ( 1 == 1 )					/*Loop to reset SSP 	*/ /*R7*/
{							/* internal baud ctrs	*/
	while ( gregs->gicp_chan == slot->chan_num ) {}	/*  assures baud setting*/
	iregs->cin_baud_ctr &= 0xC000 ;			/*   takes affect imedly*/
	oregs->cout_intnl_baud_ctr = 0 ;
	while ( gregs->gicp_chan == slot->chan_num ) {}	
	if ( ((iregs->cin_baud_ctr & 0x3FFF) == 0) &&
	       (oregs->cout_intnl_baud_ctr   == 0) )
			break ;
}

/*	*** Set Xtra DMA (since running at TOP baud rate) & Send XON Char	*/
ssp_uchar_var  = (oregs->cout_flow_config & 0x10) | 0x40 | 0x08 ;
oregs->cout_flow_config = ssp_uchar_var ;

/*	*** Flow Control, MPA-to-SSP = "MUX not Connected"			*/
/*	                  SSP-to-MPA = Output Cntrl sig 3, 			*/
iregs->cin_susp_output_lmx |= 0x20 ;			/*MUX not Connected	*/
iregs->cin_q_cntrl |= 0x20 ;				/*Set Input Flow Control*/

/*	*** Clear internal/external Loop backs					*/
oregs->cout_lmx_command &= 0xFC ;			/*Clear loop backs	*/

/*Change Control Signals so UART gets properly initialize			*/
oregs->cout_cntrl_sig = 0x77 ;				/*Set UART's cntrl sigs */
slot->frame_ctr  = gregs->gicp_frame_ctr ;		/*Snap shot for timing	*/
marb->actv_srvc  = MARB_ACTV_SRVC_REG_MODEM ;		/*Indicate outstnd'g req*/
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
marb->cont_funct = mpa_srvc_reg_modem_1_ ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/
}		/*End of REG MODEM BOARD - 1 Function, mpa_srvc_reg_modem_1_	*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *    REGISTER MODEM BOARD - 1_ FUNCTION, CHANGE CONTROL SIGNALS (Forcing MPA  *
 *					  to Write/initialize MCR)	       *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_reg_modem_1_ ( unsigned long		        caller_parm,
				     struct marb_struct FARm *  	marb,
	      			     struct ssp_struct FARm *   	sspcb,
	      			     struct slot_struct FARm *  	slot,
	      			     struct icp_global_struct FARm * 	gregs,
				     struct cin_struct FARm *	  	iregs,
	      			     struct cout_struct FARm *	  	oregs
				   )
#else
static void  mpa_srvc_reg_modem_1_ (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;
#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/
#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
iregs       = iregs ;
sspcb       = sspcb ;

/*Start of function's logic							*/
if ( slot->slot_state != 0x80 )				/*Is Brd still present?	*/
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_REG_MODEM_REMOVED ;	/*Set completion status	*/
	return ;
}
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
if ( frame_delay_chk ( (unsigned short int)0x2000, gregs, slot ) == 0 )	 /*R13*/

	return ;					/*4 frames not elapsed	*/

/*Have delayed long enough for MPA to "see" 1st control signal setting, change	*/
/*output Control Signals thus forcing MPA to write to UART MCR (thus initialize	*/
/*MCR register).  Note: assumes	board is a Modem board				*/
oregs->cout_cntrl_sig = 0x44 ;				/*Set UART's cntrl sigs */
slot->frame_ctr = gregs->gicp_frame_ctr ;		/*Snap shot for timing	*/
marb->cont_funct = mpa_srvc_reg_modem_1_x ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/
}		/*End of REG MODEM BOARD - 1_ Function, mpa_srvc_reg_modem_1_	*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *    REGISTER MODEM BOARD - 1_x FUNCTION, RESET MPA  			       *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_reg_modem_1_x ( unsigned long		        caller_parm,
				     struct marb_struct FARm *  	marb,
	      			     struct ssp_struct FARm *   	sspcb,
	      			     struct slot_struct FARm *  	slot,
	      			     struct icp_global_struct FARm * 	gregs,
				     struct cin_struct FARm *	  	iregs,
	      			     struct cout_struct FARm *	  	oregs
				    )
#else
static void  mpa_srvc_reg_modem_1_x (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)
unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;
#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/
#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
iregs       = iregs ;
sspcb       = sspcb ;

/*Start of function's logic							*/
if ( slot->slot_state != 0x80 )				/*Is Brd still present?	*/
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_REG_MODEM_REMOVED ;	/*Set completion status	*/
	return ;
}
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
if ( frame_delay_chk ( (unsigned short int)0x04, gregs, slot ) == 0 )
	return ;					/*4 frames not elapsed	*/

/*Have delayed long enough for MPA to "see" control signal setting - MPA has	*/
/*initialized UART's MCR (assuming Modem board), issue MPA RESET		*/
oregs->cout_cntrl_sig = 0x188 ;				/*Issue Slot RESET to 	*/
slot->read_toggle     = 0x00 ;				/*assure state of MPA's	*/
							/*Read Toggle		*/
slot->frame_ctr = gregs->gicp_frame_ctr ;		/*Snap shot for timing	*/
marb->cont_funct = mpa_srvc_reg_modem_1A ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/
}		/*End of REG MODEM BOARD - 1_x Function, mpa_srvc_reg_modem_1_x	*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *    REGISTER MODEM BOARD - 1A FUNCTION, ADDRESS UART's SPR REGISTER 	       *
 *					  (Determine if Board has UART)	       *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_reg_modem_1A ( unsigned long		        caller_parm,
				     struct marb_struct FARm *  	marb,
	      			     struct ssp_struct FARm *   	sspcb,
	      			     struct slot_struct FARm *  	slot,
	      			     struct icp_global_struct FARm * 	gregs,
				     struct cin_struct FARm *	  	iregs,
	      			     struct cout_struct FARm *	  	oregs
				   )
#else
static void  mpa_srvc_reg_modem_1A (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
iregs       = iregs ;
sspcb       = sspcb ;

/*Start of function's logic							*/
if ( slot->slot_state != 0x80 )				/*Is Brd still present?	*/
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_REG_MODEM_REMOVED ;	/*Set completion status	*/
	return ;
}
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
if ( frame_delay_chk ( (unsigned short int)0x08, gregs, slot ) == 0 )
	return ;					/*8 frames not elapsed	*/

/*Have delayed long enough for MPA to "see" control signal setting indicating	*/
/*that Slot should be reset, set up to address SPR register so can write to it	*/
oregs->cout_cntrl_sig = 0x177 ;				/*Addr UART's SPR	*/
slot->frame_ctr = gregs->gicp_frame_ctr ;		/*Snap shot for timing	*/
marb->cont_funct = mpa_srvc_reg_modem_1B ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/
}		/*End of REG MODEM BOARD - 1A Function, mpa_srvc_reg_modem_1A	*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *    REGISTER MODEM BOARD - 1B FUNCTION, ATTEMPT TO WRITE TO UART's SPR SO    *
 *			           	  CAN ATTEMPT TO READ BACK & DETERMINE *
 *					  IF BOARD HAS UART	               *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_reg_modem_1B ( unsigned long		        caller_parm,
				     struct marb_struct FARm *  	marb,
	      			     struct ssp_struct FARm *   	sspcb,
	      			     struct slot_struct FARm *  	slot,
	      			     struct icp_global_struct FARm * 	gregs,
				     struct cin_struct FARm *	  	iregs,
	      			     struct cout_struct FARm *	  	oregs
				   )
#else
static void  mpa_srvc_reg_modem_1B (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
iregs       = iregs ;
sspcb       = sspcb ;

/*Start of function's logic							*/
if ( slot->slot_state != 0x80 )				/*Is Brd still present?	*/
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_REG_MODEM_REMOVED ;	/*Set completion status	*/
	return ;
}
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
if ( frame_delay_chk ( (unsigned short int)0x04, gregs, slot ) == 0 )
	return ;					/*4 frames not elapsed	*/

/*Have delayed long enough for MPA to "see" control signal setting indicating	*/
/*that output be directed to UART's Scratchpad Register (SPR), write pattern	*/
/*so can read back & confirm SRP functional and thus board has UART		*/
oregs->cout_xon_1         = 0x55 ;  			/*SPR = 0x55 ????	*/
oregs->cout_flow_config  ^= 0x10 ;			/*Initiate xmit'g char	*/

/*Set up & wait till SPR output sent						*/
slot->frame_ctr = gregs->gicp_frame_ctr ;		/*Snap shot for timing	*/
marb->cont_funct = mpa_srvc_reg_modem_1C ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*End of REG MODEM BOARD - 1B Function, mpa_srvc_reg_modem_1B		*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *    REGISTER MODEM BOARD - 1C FUNCTION, SET UP TO READ SPR BY ADDRESSING     *
 *					  MPA's AUX Reg		       	       *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_reg_modem_1C  ( unsigned long		        caller_parm,
				     struct marb_struct FARm *  	marb,
	      			     struct ssp_struct FARm *   	sspcb,
	      			     struct slot_struct FARm *  	slot,
	      			     struct icp_global_struct FARm * 	gregs,
				     struct cin_struct FARm *	  	iregs,
	      			     struct cout_struct FARm *	  	oregs
				   )
#else
static void  mpa_srvc_reg_modem_1C  (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
sspcb       = sspcb ;
iregs       = iregs ;

/*Start of function's logic							*/
if ( slot->slot_state != 0x80 )				/*Is Brd still present?	*/
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_REG_MODEM_REMOVED ;	/*Set completion status	*/
	return ;
}
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
if ( ((oregs->cout_flow_config & 0x10) != (oregs->cout_intnl_flow_ctrl & 0x10)) ||
     ((oregs->cout_intnl_flow_ctrl & 0x08) == 0x08) )	/*SPR char xmit done	*/
{							/*If no, check deadman	*/
	if ( frame_delay_chk (DEADMAN_DELAY, gregs, slot) == 0x00 )
	{						/*If deadman not expired*/
		return ;
	}
	(*slot->clean_up)(slot, (unsigned char)0xFF) ;
	marb->srvc_status = ERR_REG_MODEM_DEADMAN ; 	/*Set cmpltn status	*/
	marb->srvc_state = 0x00 ;			/*Set "complete" state	*/
	return ;
}

/*Modem's SPR written to, Set up to Read SPR by address MPA's AUX reg		*/
oregs->cout_cntrl_sig = 0x199 ;				/*Addr MPA's AUX reg	*/
slot->frame_ctr = gregs->gicp_frame_ctr ;		/*Snap shot for timing	*/

/*Exit function & delay 4 frames thus allowing time for MPA to "see" AUX Reg 	*/
/*control signal setting							*/
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
marb->cont_funct = mpa_srvc_reg_modem_1D ;
							/*Flow Cntrl Function	*/
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}		/*End of REG MODEM BOARD - 1C Function, mpa_srvc_reg_modem_1C	*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *    REGISTER MODEM BOARD - 1D FUNCTION, WRITE TO MPA's AUX REG INDICATING TO *
 *					  MPA TO READ AND (XMIT) UART's SPR REG*
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_reg_modem_1D ( unsigned long		        caller_parm,
				     struct marb_struct FARm *  	marb,
	      			     struct ssp_struct FARm *   	sspcb,
	      			     struct slot_struct FARm *  	slot,
	      			     struct icp_global_struct FARm * 	gregs,
				     struct cin_struct FARm *	  	iregs,
	      			     struct cout_struct FARm *	  	oregs
				   )
#else
static void  mpa_srvc_reg_modem_1D (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
sspcb       = sspcb ;
iregs       = iregs ;

/*Start of function's logic							*/
if ( slot->slot_state != 0x80 )				/*Is Brd still present?	*/
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_REG_MODEM_REMOVED ;	/*Set completion status	*/
	return ;
}
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
if ( frame_delay_chk ( (unsigned short int)0x04, gregs, slot ) == 0 )
	return ;					/*4 frames not elapsed	*/

/*Have delayed long enough for MPA to "see" control signal setting indicating	*/
/*that output be directed to MPA's AUX Register (SPR).  Write pattern indicating*/
/*to MPA to read UART's SPR.  Note: Change clean up function so if abort during	*/
/*read then outstanding read is cleaned up i.e. read character accounted for.	*/
#ifndef 				NO_FN_PROTO_PARMS /*Compile if Protypes OK*/
slot->clean_up            = (void(*)(struct slot_struct FARm *, unsigned char))mpa_srvc_reg_modem_cln_rd ;
#else							  /*Compile if NO Protypes*/
slot->clean_up            = (void(*)())mpa_srvc_reg_modem_cln_rd ;		   /*R9*/
#endif
oregs->cout_xon_1         = 0x0F ;  			/*AUG REG byte = RD SPR	*/
oregs->cout_flow_config  ^= 0x10 ;			/*Initiate xmit'g char	*/
slot->read_toggle         = 0x08 ;			/*Sv I/O Rd Toggle state*/

/*Set up & wait till AUX REG output sent					*/
slot->frame_ctr = gregs->gicp_frame_ctr ;		/*Snap shot for timing	*/
marb->cont_funct = mpa_srvc_reg_modem_1E ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*End of REG MODEM BOARD - 1D Function, mpa_srvc_reg_modem_1D		*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *    REGISTER MODEM BOARD - 1E FUNCTION, WAIT FOR SPR BYTE TO BE RECIEVED &   *
 *					  VALIDATE BYTE			       *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_reg_modem_1E  ( unsigned long		        caller_parm,
				     struct marb_struct FARm *  	marb,
	      			     struct ssp_struct FARm *   	sspcb,
	      			     struct slot_struct FARm *  	slot,
	      			     struct icp_global_struct FARm * 	gregs,
				     struct cin_struct FARm *	  	iregs,
	      			     struct cout_struct FARm *	  	oregs
				   )
#else
static void  mpa_srvc_reg_modem_1E  (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

/*SSP input area variables									   */
#ifndef						RAMP_BSDI		/*R14*/
struct cin_bank_struct FARm		*actv_bank ;		/*Ptr to active bank		   */
#else									/*BSDI Compile		   */
volatile struct cin_bank_struct FARm	*actv_bank ;		/*Ptr to active bank		   */
#endif									/*End RAMP_BSDI Conditional*/



/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
sspcb       = sspcb ;

/*Start of function's logic							*/
if ( slot->slot_state != 0x80 )				/*Is Brd still present?	*/
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_REG_MODEM_REMOVED ;	/*Set completion status	*/
	return ;
}
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
if ( ((oregs->cout_flow_config & 0x10) != (oregs->cout_intnl_flow_ctrl & 0x10)) ||
     ((oregs->cout_intnl_flow_ctrl & 0x08) == 0x08) )	/*AUX reg char xmit done*/
{							/*If no, check deadman	*/
	if ( frame_delay_chk (DEADMAN_DELAY, gregs, slot) == 0x00 )
	{						/*If deadman not expired*/
		return ;
	}
	(*slot->clean_up)(slot, (unsigned char)0xFF) ;
	slot->slot_state  = 0xFE ;			/*Set slot's state = 	*/
							/*unknown board		*/
	marb->srvc_status = ERR_REG_MODEM_DEADMAN ; 	/*Set cmpltn status	*/
	marb->srvc_state  = 0x00 ;			/*Set "complete" state	*/
	return ;
}

/*MPA's AUX Reg READ UART's SPR request sent, wait for SPR byte to be received	*/
actv_bank = ( (iregs->cin_locks & 0x01) == 0x00 ) 		  ? /*Bank A actv?*/
	      (struct cin_bank_struct FARm *)&(iregs->cin_bank_a) : /*If yes	  */
	      (struct cin_bank_struct FARm *)&(iregs->cin_bank_b) ; /*If no	  */
if ( (actv_bank->cin_bank_num_chars) != 1 )		/*Have recv'd SPR byte?	*/
{							/*If no, check deadman	*/
	if ( frame_delay_chk ( (DEADMAN_DELAY+5), gregs, slot) == 0x00 )
	{						/*If deadman not expired*/
		return ;
	}
	(*slot->clean_up)(slot, (unsigned char)0xFF) ;
	slot->slot_state  = 0xFE ;			/*Set slot's state = 	*/
							/*unknown board		*/
	marb->srvc_status = ERR_REG_MODEM_INPUT_TO ; 	/*Set cmpltn status	*/
	marb->srvc_state  = 0x00 ;			/*Set "complete" state	*/
	return ;
}
#ifndef 				NO_FN_PROTO_PARMS /*Compile if Protypes OK*/
slot->clean_up            = (void(*)(struct slot_struct FARm *, unsigned char))mpa_srvc_reg_modem_clean_up ;
#else							  /*Compile if NO Protypes*/
slot->clean_up            = (void(*)())mpa_srvc_reg_modem_clean_up ;		   /*R9*/
#endif
if ( (inc_tail (iregs)) != 0x00 )			/*Was tail updated OK?	*/
{							/*If no, error		*/
	(*slot->clean_up)(slot, (unsigned char)0xFF) ;
	marb->srvc_status = ERR_REG_MODEM_BAD_SSP ; 	/*Set cmpltn status	*/
	slot->slot_state  = 0xFE ;			/*Set slot's state = 	*/
							/*unknown board		*/
	marb->srvc_state  = 0x00 ;			/*Set "complete" state	*/
	return ;
}
if ( *((unsigned char *)(&actv_bank->cin_bank_chars_0) +		/*Rd OK?*/
               ((actv_bank->cin_bank_fifo_lvl - 1) & 0x03) ) != 0x55 )	
{									/*If no	*/
	(*slot->clean_up)(slot, (unsigned char)0xFF) ;
	marb->srvc_status = ERR_REG_MODEM_NO_UART ; 	/*Set cmpltn status	*/
	slot->slot_state  = 0xFE ;			/*Set slot's state = 	*/
							/*unknown board		*/
	marb->srvc_state  = 0x00 ;			/*Set "complete" state	*/
	return ;
}

/*Board appears to have a UART - set up to write different pattern into SPR	*/
oregs->cout_cntrl_sig = 0x177 ;				/*Addr UART's SPR	*/
slot->frame_ctr = gregs->gicp_frame_ctr ;		/*Snap shot for timing	*/
marb->cont_funct = mpa_srvc_reg_modem_1F ;
							/*Flow Cntrl Function	*/
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}		/*End of REG MODEM BOARD - 1E Function, mpa_srvc_reg_modem_1E	*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *    REGISTER MODEM BOARD - 1F FUNCTION, ATTEMPT TO WRITE 2nd CHAR TO UART's  *
 *					  SPR SO CAN ATTEMPT TO READ BACK &    *
 *					  DETERMINE IF BOARD HAS UART	       *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_reg_modem_1F ( unsigned long		        caller_parm,
				     struct marb_struct FARm *  	marb,
	      			     struct ssp_struct FARm *   	sspcb,
	      			     struct slot_struct FARm *  	slot,
	      			     struct icp_global_struct FARm * 	gregs,
				     struct cin_struct FARm *	  	iregs,
	      			     struct cout_struct FARm *	  	oregs
				   )
#else
static void  mpa_srvc_reg_modem_1F (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
sspcb       = sspcb ;
iregs       = iregs ;

/*Start of function's logic							*/
if ( slot->slot_state != 0x80 )				/*Is Brd still present?	*/
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_REG_MODEM_REMOVED ;	/*Set completion status	*/
	return ;
}
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
if ( frame_delay_chk ( (unsigned short int)0x04, gregs, slot ) == 0 )
	return ;					/*4 frames not elapsed	*/

/*Have delayed long enough for MPA to "see" control signal setting indicating	*/
/*that output be directed to UART's Scratchpad Register (SPR), write pattern	*/
/*so can read back & confirm SRP functional and thus board has UART		*/
oregs->cout_xon_1         = 0xAA ;  			/*SPR = 0xAA ????	*/
oregs->cout_flow_config  ^= 0x10 ;			/*Initiate xmit'g char	*/

/*Set up & wait till SPR output sent						*/
slot->frame_ctr = gregs->gicp_frame_ctr ;		/*Snap shot for timing	*/
marb->cont_funct = mpa_srvc_reg_modem_1G ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*End of REG MODEM BOARD - 1F Function, mpa_srvc_reg_modem_1F		*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *    REGISTER MODEM BOARD - 1G FUNCTION, SET UP TO READ SPR BY ADDRESSING     *
 *					  MPA's AUX Reg		       	       *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_reg_modem_1G  ( unsigned long		        caller_parm,
				     struct marb_struct FARm *  	marb,
	      			     struct ssp_struct FARm *   	sspcb,
	      			     struct slot_struct FARm *  	slot,
	      			     struct icp_global_struct FARm * 	gregs,
				     struct cin_struct FARm *	  	iregs,
	      			     struct cout_struct FARm *	  	oregs
				   )
#else
static void  mpa_srvc_reg_modem_1G  (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
sspcb       = sspcb ;
iregs       = iregs ;

/*Start of function's logic							*/
if ( slot->slot_state != 0x80 )				/*Is Brd still present?	*/
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_REG_MODEM_REMOVED ;	/*Set completion status	*/
	return ;
}
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
if ( ((oregs->cout_flow_config & 0x10) != (oregs->cout_intnl_flow_ctrl & 0x10)) ||
     ((oregs->cout_intnl_flow_ctrl & 0x08) == 0x08) )	/*SPR char xmit done	*/
{							/*If no, check deadman	*/
	if ( frame_delay_chk (DEADMAN_DELAY, gregs, slot) == 0x00 )
	{						/*If deadman not expired*/
		return ;
	}
	(*slot->clean_up)(slot, (unsigned char)0xFF) ;
	marb->srvc_status = ERR_REG_MODEM_DEADMAN ; 	/*Set cmpltn status	*/
	marb->srvc_state = 0x00 ;			/*Set "complete" state	*/
	return ;
}

/*Modem's SPR written to, Set up to Read SPR by address MPA's AUX reg		*/
oregs->cout_cntrl_sig = 0x199 ;				/*Addr MPA's AUX reg	*/
slot->frame_ctr = gregs->gicp_frame_ctr ;		/*Snap shot for timing	*/

/*Exit function & delay 4 frames thus allowing time for MPA to "see" AUX Reg 	*/
/*control signal setting							*/
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
marb->cont_funct = mpa_srvc_reg_modem_1H ;
							/*Flow Cntrl Function	*/
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}		/*End of REG MODEM BOARD - 1G Function, mpa_srvc_reg_modem_1G	*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *    REGISTER MODEM BOARD - 1H FUNCTION, WRITE TO MPA's AUX REG INDICATING TO *
 *					  MPA TO READ AND (XMIT) UART's SPR REG*
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_reg_modem_1H ( unsigned long		        caller_parm,
				     struct marb_struct FARm *  	marb,
	      			     struct ssp_struct FARm *   	sspcb,
	      			     struct slot_struct FARm *  	slot,
	      			     struct icp_global_struct FARm * 	gregs,
				     struct cin_struct FARm *	  	iregs,
	      			     struct cout_struct FARm *	  	oregs
				   )
#else
static void  mpa_srvc_reg_modem_1H (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
sspcb       = sspcb ;
iregs       = iregs ;

/*Start of function's logic							*/
if ( slot->slot_state != 0x80 )				/*Is Brd still present?	*/
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_REG_MODEM_REMOVED ;	/*Set completion status	*/
	return ;
}
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
if ( frame_delay_chk ( (unsigned short int)0x04, gregs, slot ) == 0 )
	return ;					/*4 frames not elapsed	*/

/*Have delayed long enough for MPA to "see" control signal setting indicating	*/
/*that output be directed to MPA's AUX Register (SPR).  Write pattern indicating*/
/*to MPA to read UART's SPR.  Note: Change clean up function so if abort during	*/
/*read then outstanding read is cleaned up i.e. read character accounted for.	*/
#ifndef 				NO_FN_PROTO_PARMS /*Compile if Protypes OK*/
slot->clean_up            = (void(*)(struct slot_struct FARm *, unsigned char))mpa_srvc_reg_modem_cln_rd ;
#else							  /*Compile if NO Protypes*/
slot->clean_up            = (void(*)())mpa_srvc_reg_modem_cln_rd ;		   /*R9*/
#endif
oregs->cout_xon_1         = 0x07 ;  			/*AUG REG byte = RD SPR	*/
oregs->cout_flow_config  ^= 0x10 ;			/*Initiate xmit'g char	*/
slot->read_toggle         = 0x00 ;			/*Sv I/O Rd Toggle state*/

/*Set up & wait till AUX REG output sent					*/
slot->frame_ctr = gregs->gicp_frame_ctr ;		/*Snap shot for timing	*/
marb->cont_funct = mpa_srvc_reg_modem_1I ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*End of REG MODEM BOARD - 1H Function, mpa_srvc_reg_modem_1H		*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *    REGISTER MODEM BOARD - 1I FUNCTION, WAIT FOR SPR BYTE TO BE RECIEVED &   *
 *					  VALIDATE BYTE			       *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_reg_modem_1I  ( unsigned long		        caller_parm,
				     struct marb_struct FARm *  	marb,
	      			     struct ssp_struct FARm *   	sspcb,
	      			     struct slot_struct FARm *  	slot,
	      			     struct icp_global_struct FARm * 	gregs,
				     struct cin_struct FARm *	  	iregs,
	      			     struct cout_struct FARm *	  	oregs
				   )
#else
static void  mpa_srvc_reg_modem_1I  (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

/*SSP input area variables									   */
#ifndef					RAMP_BSDI		/*R14*/
struct cin_bank_struct FARm		*actv_bank ;		/*Ptr to active bank		   */
#else									/*BSDI Compile		   */
volatile struct cin_bank_struct FARm	*actv_bank ;		/*Ptr to active bank	 	   */
#endif									/*End RAMP_BSDI Conditional*/



/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
sspcb       = sspcb ;

/*Start of function's logic							*/
if ( slot->slot_state != 0x80 )				/*Is Brd still present?	*/
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_REG_MODEM_REMOVED ;	/*Set completion status	*/
	return ;
}
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
if ( ((oregs->cout_flow_config & 0x10) != (oregs->cout_intnl_flow_ctrl & 0x10)) ||
     ((oregs->cout_intnl_flow_ctrl & 0x08) == 0x08) )	/*AUX reg char xmit done*/
{							/*If no, check deadman	*/
	if ( frame_delay_chk (DEADMAN_DELAY, gregs, slot) == 0x00 )
	{						/*If deadman not expired*/
		return ;
	}
	(*slot->clean_up)(slot, (unsigned char)0xFF) ;
	slot->slot_state  = 0xFE ;			/*Set slot's state = 	*/
							/*unknown board		*/
	marb->srvc_status = ERR_REG_MODEM_DEADMAN ; 	/*Set cmpltn status	*/
	marb->srvc_state = 0x00 ;			/*Set "complete" state	*/
	return ;
}

/*MPA's AUX Reg READ UART's SPR request sent, wait for SPR byte to be received	*/
actv_bank = ( (iregs->cin_locks & 0x01) == 0x00 ) 		  ? /*Bank A actv?*/
	      (struct cin_bank_struct FARm *)&(iregs->cin_bank_a) : /*If yes	  */
	      (struct cin_bank_struct FARm *)&(iregs->cin_bank_b) ; /*If no	  */
if ( (actv_bank->cin_bank_num_chars) != 1 )		/*Have recv'd SPR byte?	*/
{							/*If no, check deadman	*/
	if ( frame_delay_chk ( (DEADMAN_DELAY+5), gregs, slot) == 0x00 )
	{						/*If deadman not expired*/
		return ;
	}
	(*slot->clean_up)(slot, (unsigned char)0xFF) ;
	slot->slot_state  = 0xFE ;			/*Set slot's state = 	*/
							/*unknown board		*/
	marb->srvc_status = ERR_REG_MODEM_INPUT_TO ; 	/*Set cmpltn status	*/
	marb->srvc_state  = 0x00 ;			/*Set "complete" state	*/
	return ;
}
#ifndef 				NO_FN_PROTO_PARMS /*Compile if Protypes OK*/
slot->clean_up            = (void(*)(struct slot_struct FARm *, unsigned char))mpa_srvc_reg_modem_clean_up ;
#else							  /*Compile if NO Protypes*/
slot->clean_up            = (void(*)())mpa_srvc_reg_modem_clean_up ;		   /*R9*/
#endif
if ( (inc_tail (iregs)) != 0x00 )			/*Was tail updated OK?	*/
{							/*If no, error		*/
	(*slot->clean_up)(slot, (unsigned char)0xFF) ;
	marb->srvc_status = ERR_REG_MODEM_BAD_SSP ; 	/*Set cmpltn status	*/
	slot->slot_state  = 0xFE ;			/*Set slot's state = 	*/
							/*unknown board		*/
	marb->srvc_state  = 0x00 ;			/*Set "complete" state	*/
	return ;
}
if ( *((unsigned char *)(&actv_bank->cin_bank_chars_0) +		/*Rd OK?*/
               ((actv_bank->cin_bank_fifo_lvl - 1) & 0x03) ) != 0xAA )	
{									/*If no	*/
	(*slot->clean_up)(slot, (unsigned char)0xFF) ;
	marb->srvc_status = ERR_REG_MODEM_NO_UART ; 	/*Set cmpltn status	*/
	slot->slot_state  = 0xFE ;			/*Set slot's state = 	*/
							/*unknown board		*/
	marb->srvc_state  = 0x00 ;			/*Set "complete" state	*/
	return ;
}

/*Board appears to have a UART - set up to address UART's IER			*/
oregs->cout_cntrl_sig = 0x111 ;				/*Addr UART's IER	*/
slot->frame_ctr = gregs->gicp_frame_ctr ;		/*Snap shot for timing	*/
marb->cont_funct = mpa_srvc_reg_modem_2 ;
							/*Flow Cntrl Function	*/
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}		/*End of REG MODEM BOARD - 1I Function, mpa_srvc_reg_modem_1I	*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *    REGISTER MODEM BOARD - 2nd FUNCTION, INITIALIZE (WRITE TO) UART's IER    *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_reg_modem_2  ( unsigned long		        caller_parm,
				     struct marb_struct FARm *  	marb,
	      			     struct ssp_struct FARm *   	sspcb,
	      			     struct slot_struct FARm *  	slot,
	      			     struct icp_global_struct FARm * 	gregs,
				     struct cin_struct FARm *	  	iregs,
	      			     struct cout_struct FARm *	  	oregs
				   )
#else
static void  mpa_srvc_reg_modem_2  (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
sspcb       = sspcb ;
iregs       = iregs ;

/*Start of function's logic							*/
if ( slot->slot_state != 0x80 )				/*Is Brd still present?	*/
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_REG_MODEM_REMOVED ;	/*Set completion status	*/
	return ;
}
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
if ( frame_delay_chk ( (unsigned short int)0x04, gregs, slot ) == 0 )
	return ;					/*4 frames not elapsed	*/

/*Have delayed long enough for MPA to "see" control signal setting indicating	*/
/*that output be directed to UART's Interrupt Enable Register (IER)		*/
oregs->cout_xon_1         = 0x08 ;			/*IER=Modem Stat int ena*/
oregs->cout_flow_config  ^= 0x10 ;			/*Initiate xmit'g char	*/

/*Set up & wait till IER output sent						*/
slot->frame_ctr = gregs->gicp_frame_ctr ;		/*Snap shot for timing	*/
marb->cont_funct = mpa_srvc_reg_modem_3 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*End of REG MODEM BOARD - 2nd Function, mpa_srvc_reg_modem_2		*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *    REGISTER MODEM BOARD - 3rd FUNCTION, SET UP MPA SO OUTPUT GOES TO SLOT'S *
 *                                         AUX REG - MPA-to-UART FLOW CONTROL  *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_reg_modem_3  ( unsigned long		        caller_parm,
				     struct marb_struct FARm *  	marb,
	      			     struct ssp_struct FARm *   	sspcb,
	      			     struct slot_struct FARm *  	slot,
	      			     struct icp_global_struct FARm * 	gregs,
				     struct cin_struct FARm *	  	iregs,
	      			     struct cout_struct FARm *	  	oregs
				   )
#else
static void  mpa_srvc_reg_modem_3  (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
sspcb       = sspcb ;
iregs       = iregs ;

/*Start of function's logic							*/
if ( slot->slot_state != 0x80 )				/*Is Brd still present?	*/
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_REG_MODEM_REMOVED ;	/*Set completion status	*/
	return ;
}
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
if ( ((oregs->cout_flow_config & 0x10) != (oregs->cout_intnl_flow_ctrl & 0x10)) ||
     ((oregs->cout_intnl_flow_ctrl & 0x08) == 0x08) )	/*IER Reg xmit cmplt	*/
{							/*If no, check deadman	*/
	if ( frame_delay_chk (DEADMAN_DELAY, gregs, slot) == 0x00 )
	{						/*If deadman not expired*/
		return ;
	}
	(*slot->clean_up)(slot, (unsigned char)0xFF) ;
	marb->srvc_status = ERR_REG_MODEM_DEADMAN ; 	/*Set cmpltn status	*/
	marb->srvc_state = 0x00 ;			/*Set "complete" state	*/
	return ;
}

/*Setting UART's IER register complete.  Set up so output sent to MPA's AUX reg	*/
/*MPA-to-UART Flow Control							*/
oregs->cout_cntrl_sig = 0x1AA ;				/*Send control sigs to	*/
							/*MPA indicat'g to direct*/
							/*output to MPA-to-UART	*/
							/*Flow Cntrl		*/
slot->frame_ctr = gregs->gicp_frame_ctr ;		/*Snap shot for timing	*/
marb->cont_funct = mpa_srvc_reg_modem_4 ;		/*Disable MPA-to-UART	*/
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*End of REG MODEM BOARD - 3rd FUNCTION, mpa_srvc_reg_modem_3		*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *    REGISTER MODEM BOARD - 4th FUNCTION, INITIALIZE MPA-to-UART FLOW CONTROL *
 *                                                                             *
 * NOTE: Function also called by "mpa_srvc_init_modem_2,		       *
 *				 "mpa_srvc_mpa_flow_2" and 		       *
 *				 "mpa_srvc_mod_sets_2			       *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_reg_modem_4  ( unsigned long		        caller_parm,
				     struct marb_struct FARm *  	marb,
	      			     struct ssp_struct FARm *   	sspcb,
	      			     struct slot_struct FARm *  	slot,
	      			     struct icp_global_struct FARm * 	gregs,
				     struct cin_struct FARm *	  	iregs,
	      			     struct cout_struct FARm *	  	oregs
				   )
#else
static void  mpa_srvc_reg_modem_4  (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
sspcb       = sspcb ;
iregs       = iregs ;

/*Start of function's logic							*/
if ( ( (caller_parm == 0) && (slot->slot_state != 0x80) ) || /*Is Brd still present?*/
     ( (caller_parm != 0) && (slot->slot_state != 0xFF) ) )
{							     /*If no, modem pulled  */
	marb->srvc_status = ERR_REG_MODEM_REMOVED ;	/*Set completion status	*/
	return ;
}
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
if ( frame_delay_chk ( (unsigned short int)0x04, gregs, slot ) == 0 )
	return ;					/*4 frames not elapsed	*/

/*Have delayed long enough for MPA to "see" control signal setting indicating	*/
/*that output be directed to MPA AUX register - MPA-to-UART Flow Control setting*/
oregs->cout_xon_1         = slot->flow_setting ;	/*AUX Reg flow ctrl	*/
oregs->cout_flow_config  ^= 0x10 ;			/*Initiate xmit'g char	*/

/*Set up & wait till MPA AUX Reg Flow Control setting (character) sent		*/
slot->frame_ctr = gregs->gicp_frame_ctr ;		/*Snap shot for timing	*/
marb->cont_funct = mpa_srvc_reg_modem_5 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*End of REG MODEM BOARD - 4th FUNCTION, mpa_srvc_reg_modem_4		*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *    REGISTER MODEM BOARD - 5th FUNCTION, ADDRESS UART's LCR or FCR REGISTER  *
 *                                                                             *
 * NOTE: Function also called by "mpa_srvc_init_modem_3" and		       *
 * 				 "mpa_srvc_mod_sets_3"			       *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_reg_modem_5  ( unsigned long		        caller_parm,
				     struct marb_struct FARm *  	marb,
	      			     struct ssp_struct FARm *   	sspcb,
	      			     struct slot_struct FARm *  	slot,
	      			     struct icp_global_struct FARm * 	gregs,
				     struct cin_struct FARm *	  	iregs,
	      			     struct cout_struct FARm *	  	oregs
				   )
#else
static void  mpa_srvc_reg_modem_5  (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
sspcb       = sspcb ;
iregs       = iregs ;

/*Start of function's logic							*/
if ( ( (caller_parm == 0) && (slot->slot_state != 0x80) ) || /*Is Brd still present?*/
     ( (caller_parm != 0) && (slot->slot_state != 0xFF) ) )
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_REG_MODEM_REMOVED ;	/*Set completion status	*/
	return ;
}
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
if ( ((oregs->cout_flow_config & 0x10) != (oregs->cout_intnl_flow_ctrl & 0x10)) ||
     ((oregs->cout_intnl_flow_ctrl & 0x08) == 0x08) )	/*AUX Reg char xmit done*/
{							/*If no, check deadman	*/
	if ( frame_delay_chk (DEADMAN_DELAY, gregs, slot) == 0x00 )
	{						/*If deadman not expired*/
		return ;
	}
	(*slot->clean_up)(slot, (unsigned char)0xFF) ;
	marb->srvc_status = ERR_REG_MODEM_DEADMAN ; 	/*Set cmpltn status	*/
	marb->srvc_state = 0x00 ;			/*Set "complete" state	*/
	return ;
}

/*MPA-to-UART Flow Control established, set output control signals to address	*/
/*UART's FIFO Control Register (FCB) if Registering Modem or Line Control 	*/
/*Register (LCR) if some other Service using this function			*/
if ( caller_parm == 0 )				/*R15*/	/*Performing Reg Modem?	*/
{							/*If yes, init. FCB reg	*/
	oregs->cout_cntrl_sig = 0x122 ;		/*R15*/	/*Addr UART's FCR	*/
	marb->cont_funct = mpa_srvc_reg_modem_5a ; /*R15*/
}							/*Function being called	*/
else							/* by some Service other*/
{							/*  then Reg Modem	*/
	oregs->cout_cntrl_sig = 0x133 ;			/*Addr UART's LCR	*/
	marb->cont_funct = mpa_srvc_reg_modem_6 ;
}
slot->frame_ctr = gregs->gicp_frame_ctr ;		/*Snap shot for timing	*/
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*End of REG MODEM BOARD - 5th Function, mpa_srvc_reg_modem_5		*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *    REGISTER MODEM BOARD - 5a FUNCTION, INITIALIZE UART's FCR, FIFO Control  *
 *			     Register (if it exists)			       *
 *                                                                             *
 * NOTE: Function also called by "mpa_srvc_hard_rst_6a"			       *
 *                                                                             *
 *******************************************************************************/
/*R15*/

#ifndef 		NO_FN_PROTO_PARMS
static void  mpa_srvc_reg_modem_5a  ( unsigned long		        caller_parm,
				     struct marb_struct FARm *  	marb,
	      			     struct ssp_struct FARm *   	sspcb,
	      			     struct slot_struct FARm *  	slot,
	      			     struct icp_global_struct FARm * 	gregs,
				     struct cin_struct FARm *	  	iregs,
	      			     struct cout_struct FARm *	  	oregs
				   )
#else
static void  mpa_srvc_reg_modem_5a  (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
sspcb       = sspcb ;
iregs       = iregs ;

/*Start of function's logic							*/
if ( ( (caller_parm == 0) && (slot->slot_state != 0x80) ) || /*Is Brd still present?*/
     ( (caller_parm != 0) && (slot->slot_state != 0xFF) ) )
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_REG_MODEM_REMOVED ;	/*Set completion status	*/
	return ;
}
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
if ( frame_delay_chk ( (unsigned short int)0x04, gregs, slot ) == 0 )
	return ;					/*4 frames not elapsed	*/

/*Have delayed long enough for MPA to "see" control signal setting indicating	*/
/*that output be directed to UART's FIFO Control Register (FCR)			*/
oregs->cout_xon_1 = 0x07 ;				/*Enable 16550 FIFO	*/
							/* (assumes 16550)	*/
oregs->cout_flow_config  ^= 0x10 ;			/*Initiate xmit'g char	*/

/*Set up & wait till FCR output sent						*/
slot->frame_ctr = gregs->gicp_frame_ctr ;		/*Snap shot for timing	*/
marb->cont_funct = mpa_srvc_reg_modem_5b ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*End of REG MODEM BOARD - 5a Function, mpa_srvc_reg_modem_5a		*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *  REGISTER MODEM BOARD - 5b FUNCTION, ADDRESS UART's LCR REGISTER 	       *
 *                                                                             *
 * NOTE: Function also called by "mpa_srvc_hard_rst_8"			       *
 *                                                                             *
 *******************************************************************************/
/*R15*/

#ifndef 		NO_FN_PROTO_PARMS
static void  mpa_srvc_reg_modem_5b  ( unsigned long		        caller_parm,
				     struct marb_struct FARm *  	marb,
	      			     struct ssp_struct FARm *   	sspcb,
	      			     struct slot_struct FARm *  	slot,
	      			     struct icp_global_struct FARm * 	gregs,
				     struct cin_struct FARm *	  	iregs,
	      			     struct cout_struct FARm *	  	oregs
				   )
#else
static void  mpa_srvc_reg_modem_5b  (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
sspcb       = sspcb ;
iregs       = iregs ;

/*Start of function's logic							*/
if ( ( (caller_parm == 0) && (slot->slot_state != 0x80) ) || /*Is Brd still present?*/
     ( (caller_parm != 0) && (slot->slot_state != 0xFF) ) )
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_REG_MODEM_REMOVED ;	/*Set completion status	*/
	return ;
}
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
if ( ((oregs->cout_flow_config & 0x10) != (oregs->cout_intnl_flow_ctrl & 0x10)) ||
     ((oregs->cout_intnl_flow_ctrl & 0x08) == 0x08) )	/*FCR char xmit done	*/
{							/*If no, check deadman	*/
	if ( frame_delay_chk (DEADMAN_DELAY, gregs, slot) == 0x00 )
	{						/*If deadman not expired*/
		return ;
	}
	(*slot->clean_up)(slot, (unsigned char)0xFF) ;
	marb->srvc_status = ERR_REG_MODEM_DEADMAN ; 	/*Set cmpltn status	*/
	marb->srvc_state = 0x00 ;			/*Set "complete" state	*/
	return ;
}

/*Modem's FCR, FIFO Control Register, written to (if it exists), set output 	*/
/*control signals to address UART's Line Control Register (LCR)			*/
oregs->cout_cntrl_sig = 0x133 ;				/*Addr UART's LCR	*/
slot->frame_ctr = gregs->gicp_frame_ctr ;		/*Snap shot for timing	*/
marb->cont_funct = mpa_srvc_reg_modem_6 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*End of REG MODEM BOARD - 5b Function, mpa_srvc_reg_modem_5b		*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *    REGISTER MODEM BOARD - 6th FUNCTION, INITIALIZE UART's LCR REGISTER      *
 *                                                                             *
 * NOTE: Function also called by "mpa_srvc_init_modem_4" 		       *
 *				 "mpa_srvc_hard_rst_7" and		       *
 * 				 "mpa_srvc_mod_sets_4"			       *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_reg_modem_6  ( unsigned long		        caller_parm,
				     struct marb_struct FARm *  	marb,
	      			     struct ssp_struct FARm *   	sspcb,
	      			     struct slot_struct FARm *  	slot,
	      			     struct icp_global_struct FARm * 	gregs,
				     struct cin_struct FARm *	  	iregs,
	      			     struct cout_struct FARm *	  	oregs
				   )
#else
static void  mpa_srvc_reg_modem_6  (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
sspcb       = sspcb ;
iregs       = iregs ;

/*Start of function's logic							*/
if ( ( (caller_parm == 0) && (slot->slot_state != 0x80) ) || /*Is Brd still present?*/
     ( (caller_parm != 0) && (slot->slot_state != 0xFF) ) )
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_REG_MODEM_REMOVED ;	/*Set completion status	*/
	return ;
}
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
if ( frame_delay_chk ( (unsigned short int)0x04, gregs, slot ) == 0 )
	return ;					/*4 frames not elapsed	*/

/*Have delayed long enough for MPA to "see" control signal setting indicating	*/
/*that output be directed to UART's Line Control Register (LCR)			*/
oregs->cout_xon_1         = 0x80   	      |		/*LCR=Divsr Latch ena +	*/
			    slot->num_bits    |		/*    char size +	*/
			    slot->stop_bits   |		/*    stop bits +	*/
			    slot->parity      |		/*    parity    +	*/
                            slot->break_state ;		/*    break state	*/
oregs->cout_flow_config  ^= 0x10 ;			/*Initiate xmit'g char	*/

/*Set up & wait till LCR output sent						*/
slot->frame_ctr = gregs->gicp_frame_ctr ;		/*Snap shot for timing	*/
marb->cont_funct = mpa_srvc_reg_modem_7 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*End of REG MODEM BOARD - 6th Function, mpa_srvc_reg_modem_6		*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *  REGISTER MODEM BOARD - 7th FUNCTION, ADDRESS UART's DIVISOR LATCH REGISTER *
 *			(Least Significant byte) DLL			       *
 *                                                                             *
 * NOTE: Function also called by "mpa_srvc_init_modem_5" and		       *
 *				 "mpa_srvc_hard_rst_8"			       *
 *				 "mpa_srvc_mod_sets_5"			       *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_reg_modem_7  ( unsigned long		        caller_parm,
				     struct marb_struct FARm *  	marb,
	      			     struct ssp_struct FARm *   	sspcb,
	      			     struct slot_struct FARm *  	slot,
	      			     struct icp_global_struct FARm * 	gregs,
				     struct cin_struct FARm *	  	iregs,
	      			     struct cout_struct FARm *	  	oregs
				   )
#else
static void  mpa_srvc_reg_modem_7  (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
sspcb       = sspcb ;
iregs       = iregs ;

/*Start of function's logic							*/
if ( ( (caller_parm == 0) && (slot->slot_state != 0x80) ) || /*Is Brd still present?*/
     ( (caller_parm != 0) && (slot->slot_state != 0xFF) ) )
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_REG_MODEM_REMOVED ;	/*Set completion status	*/
	return ;
}
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
if ( ((oregs->cout_flow_config & 0x10) != (oregs->cout_intnl_flow_ctrl & 0x10)) ||
     ((oregs->cout_intnl_flow_ctrl & 0x08) == 0x08) )	/*LCR char xmit done	*/
{							/*If no, check deadman	*/
	if ( frame_delay_chk (DEADMAN_DELAY, gregs, slot) == 0x00 )
	{						/*If deadman not expired*/
		return ;
	}
	(*slot->clean_up)(slot, (unsigned char)0xFF) ;
	marb->srvc_status = ERR_REG_MODEM_DEADMAN ; 	/*Set cmpltn status	*/
	marb->srvc_state = 0x00 ;			/*Set "complete" state	*/
	return ;
}

/*Modem's LCR written to, set output control signals to address UART's Divisor 	*/
/*Latch, least significant byte (DLL)						*/
oregs->cout_cntrl_sig = 0x100 ;				/*Addr UART's DLL	*/
slot->frame_ctr = gregs->gicp_frame_ctr ;		/*Snap shot for timing	*/
marb->cont_funct = mpa_srvc_reg_modem_8 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*End of REG MODEM BOARD - 7th Function, mpa_srvc_reg_modem_7		*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *    REGISTER MODEM BOARD - 8th FUNCTION, INITIALIZE UART's DLL REGISTER      *
 *                                                                             *
 * NOTE: Function also called by "mpa_srvc_init_modem_6",		       *
 *				 "mpa_srvc_hard_rst_9" and		       *
 *				 "mpa_srvc_mod_sets_6" 			       *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_reg_modem_8  ( unsigned long		        caller_parm,
				     struct marb_struct FARm *  	marb,
	      			     struct ssp_struct FARm *   	sspcb,
	      			     struct slot_struct FARm *  	slot,
	      			     struct icp_global_struct FARm * 	gregs,
				     struct cin_struct FARm *	  	iregs,
	      			     struct cout_struct FARm *	  	oregs
				   )
#else
static void  mpa_srvc_reg_modem_8  (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
sspcb       = sspcb ;
iregs       = iregs ;

/*Start of function's logic							*/
if ( ( (caller_parm == 0) && (slot->slot_state != 0x80) ) || /*Is Brd still present?*/
     ( (caller_parm != 0) && (slot->slot_state != 0xFF) ) )
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_REG_MODEM_REMOVED ;	/*Set completion status	*/
	return ;
}
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
if ( frame_delay_chk ( (unsigned short int)0x04, gregs, slot ) == 0 )
	return ;					/*4 frames not elapsed	*/

/*Have delayed long enough for MPA to "see" control signal setting indicating	*/
/*that output be directed to UART's Divisor Latch least significant byte (DLL)	*/
oregs->cout_xon_1         = (unsigned char)(slot->baud & 0xFF) ;
oregs->cout_flow_config  ^= 0x10 ;			/*Initiate xmit'g char	*/

/*Set up & wait till DLL output sent						*/
slot->frame_ctr = gregs->gicp_frame_ctr ;		/*Snap shot for timing	*/
marb->cont_funct = mpa_srvc_reg_modem_9 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*End of REG MODEM BOARD - 8th Function, mpa_srvc_reg_modem_8		*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *  REGISTER MODEM BOARD - 9th FUNCTION, ADDRESS UART's DIVISOR LATCH REGISTER *
 *			(Most Significant byte) DLM			       *
 *                                                                             *
 * NOTE: Function also called by "mpa_srvc_init_modem_7",		       *
 *				 "mpa_srvc_hard_rst_10" and		       *
 *				 "mpa_srvc_mod_sets_7"			       *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_reg_modem_9  ( unsigned long		        caller_parm,
				     struct marb_struct FARm *  	marb,
	      			     struct ssp_struct FARm *   	sspcb,
	      			     struct slot_struct FARm *  	slot,
	      			     struct icp_global_struct FARm * 	gregs,
				     struct cin_struct FARm *	  	iregs,
	      			     struct cout_struct FARm *	  	oregs
				   )
#else
static void  mpa_srvc_reg_modem_9  (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
sspcb       = sspcb ;
iregs       = iregs ;

/*Start of function's logic							*/
if ( ( (caller_parm == 0) && (slot->slot_state != 0x80) ) || /*Is Brd still present?*/
     ( (caller_parm != 0) && (slot->slot_state != 0xFF) ) )
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_REG_MODEM_REMOVED ;	/*Set completion status	*/
	return ;
}
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
if ( ((oregs->cout_flow_config & 0x10) != (oregs->cout_intnl_flow_ctrl & 0x10)) ||
     ((oregs->cout_intnl_flow_ctrl & 0x08) == 0x08) )	/*DLL char xmit done	*/
{							/*If no, check deadman	*/
	if ( frame_delay_chk (DEADMAN_DELAY, gregs, slot) == 0x00 )
	{						/*If deadman not expired*/
		return ;
	}
	(*slot->clean_up)(slot, (unsigned char)0xFF) ;
	marb->srvc_status = ERR_REG_MODEM_DEADMAN ; 	/*Set cmpltn status	*/
	marb->srvc_state = 0x00 ;			/*Set "complete" state	*/
	return ;
}

/*Modem's DLL written to, set output control signals to address UART's Divisor 	*/
/*Latch, most significant byte (DLM)						*/
oregs->cout_cntrl_sig = 0x111 ;				/*Addr UART's DLL	*/
slot->frame_ctr = gregs->gicp_frame_ctr ;		/*Snap shot for timing	*/
marb->cont_funct = mpa_srvc_reg_modem_10 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*End of REG MODEM BOARD - 9th Function, mpa_srvc_reg_modem_9		*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *    REGISTER MODEM BOARD - 10th FUNCTION, INITIALIZE UART's DLM REGISTER     *
 *                                                                             *
 * NOTE: Function also called by "mpa_srvc_init_modem_8" 		       *
 *				 "mpa_srvc_hard_rst_11" and		       *
 *				 "mpa_srvc_mod_sets_8"			       *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_reg_modem_10 ( unsigned long		        caller_parm,
				     struct marb_struct FARm *  	marb,
	      			     struct ssp_struct FARm *   	sspcb,
	      			     struct slot_struct FARm *  	slot,
	      			     struct icp_global_struct FARm * 	gregs,
				     struct cin_struct FARm *	  	iregs,
	      			     struct cout_struct FARm *	  	oregs
				   )
#else
static void  mpa_srvc_reg_modem_10 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
sspcb       = sspcb ;
iregs       = iregs ;

/*Start of function's logic							*/
if ( ( (caller_parm == 0) && (slot->slot_state != 0x80) ) || /*Is Brd still present?*/
     ( (caller_parm != 0) && (slot->slot_state != 0xFF) ) )
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_REG_MODEM_REMOVED ;	/*Set completion status	*/
	return ;
}
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
if ( frame_delay_chk ( (unsigned short int)0x04, gregs, slot ) == 0 )
	return ;					/*4 frames not elapsed	*/

/*Have delayed long enough for MPA to "see" control signal setting indicating	*/
/*that output be directed to UART's Divisor Latch most significant byte (DLM)	*/
oregs->cout_xon_1         = (unsigned char)( (slot->baud >> 8) & 0xFF ) ;
oregs->cout_flow_config  ^= 0x10 ;			/*Initiate xmit'g char	*/

/*Set up & wait till DLM output sent						*/
slot->frame_ctr = gregs->gicp_frame_ctr ;		/*Snap shot for timing	*/
marb->cont_funct = mpa_srvc_reg_modem_11 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*End of REG MODEM BOARD - 10th Function, mpa_srvc_reg_modem_10		*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *    REGISTER MODEM BOARD - 11th FUNCTION, ADDRESS UART's LCR REGISTER	       *
 *                                                                             *
 * NOTE: Function also called by "mpa_srvc_init_modem_9",		       *
 *				 "mpa_srvc_hard_rst_12",		       *
 * 				 "mpa_srvc_start_break_1"  		       *
 * 				 "mpa_srvc_stop_break_1" and		       *
 * 				 "mpa_srvc_mod_sets_9"			       *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_reg_modem_11  ( unsigned long		        caller_parm,
				     struct marb_struct FARm *  	marb,
	      			     struct ssp_struct FARm *   	sspcb,
	      			     struct slot_struct FARm *  	slot,
	      			     struct icp_global_struct FARm * 	gregs,
				     struct cin_struct FARm *	  	iregs,
	      			     struct cout_struct FARm *	  	oregs
				   )
#else
static void  mpa_srvc_reg_modem_11  (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
sspcb       = sspcb ;
iregs       = iregs ;

/*Start of function's logic							*/
if ( ( (caller_parm == 0) && (slot->slot_state != 0x80) ) || /*Is Brd still present?*/
     ( (caller_parm != 0) && (slot->slot_state != 0xFF) ) )
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_REG_MODEM_REMOVED ;	/*Set completion status	*/
	return ;
}
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
if ( ((oregs->cout_flow_config & 0x10) != (oregs->cout_intnl_flow_ctrl & 0x10)) ||
     ((oregs->cout_intnl_flow_ctrl & 0x08) == 0x08) )	/*DLM char xmit done	*/
{							/*If no, check deadman	*/
	if ( frame_delay_chk (DEADMAN_DELAY, gregs, slot) == 0x00 )
	{						/*If deadman not expired*/
		return ;
	}
	(*slot->clean_up)(slot, (unsigned char)0xFF) ;
	marb->srvc_status = ERR_REG_MODEM_DEADMAN ; 	/*Set cmpltn status	*/
	marb->srvc_state = 0x00 ;			/*Set "complete" state	*/
	return ;
}

/*Modem's DLM written to, clear LCR's divisor latch enable bit			*/
oregs->cout_cntrl_sig = 0x133 ;				/*Addr UART's LCR	*/
slot->frame_ctr = gregs->gicp_frame_ctr ;		/*Snap shot for timing	*/
marb->cont_funct = mpa_srvc_reg_modem_12 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*End of REG MODEM BOARD - 11th Function, mpa_srvc_reg_modem_11		*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *    REGISTER MODEM BOARD - 12th FUNCTION, CLEAR UART'S LCR REGISTER DIVISOR  *
 *                                          LATCH ENABLE BIT		       *
 *                                                                             *
 * NOTE: Function also called by "mpa_srvc_init_modem_10",		       *
 *				 "mpa_srvc_hard_rst_13", 		       *
 * 				 "mpa_srvc_break_2", and		       *
 *				 "mpa_srvc_mod_sets_10			       *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_reg_modem_12 ( unsigned long		        caller_parm,
				     struct marb_struct FARm *  	marb,
	      			     struct ssp_struct FARm *   	sspcb,
	      			     struct slot_struct FARm *  	slot,
	      			     struct icp_global_struct FARm * 	gregs,
				     struct cin_struct FARm *	  	iregs,
	      			     struct cout_struct FARm *	  	oregs
				   )
#else
static void  mpa_srvc_reg_modem_12 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
sspcb       = sspcb ;
iregs       = iregs ;

/*Start of function's logic							*/
if ( ( (caller_parm == 0) && (slot->slot_state != 0x80) ) || /*Is Brd still present?*/
     ( (caller_parm != 0) && (slot->slot_state != 0xFF) ) )
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_REG_MODEM_REMOVED ;	/*Set completion status	*/
	return ;
}
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
if ( frame_delay_chk ( (unsigned short int)0x04, gregs, slot ) == 0 )
	return ;					/*4 frames not elapsed	*/

/*Have delayed long enough for MPA to "see" control signal setting indicating	*/
/*that output be directed to UART's Line Control Register (LCR)			*/
oregs->cout_xon_1         = 0x00 	      | 	/*LCR=Divsr Latch dsa +	*/
			    slot->num_bits    |		/*    char size +	*/
			    slot->stop_bits   |		/*    stop bits +	*/
			    slot->parity      |		/*    parity    +	*/
                            slot->break_state ;		/*    break state	*/
oregs->cout_flow_config  ^= 0x10 ;			/*Initiate xmit'g char	*/

/*Set up & wait till LCR output sent						*/
slot->frame_ctr = gregs->gicp_frame_ctr ;		/*Snap shot for timing	*/
marb->cont_funct = mpa_srvc_reg_modem_13 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*End of REG MODEM BOARD - 12th Function, mpa_srvc_reg_modem_12		*/

/*#page*/
/*******************************************************************************
 *                                                                             *
 *    REGISTER MODEM BOARD - 13th FUNCTION, ADDRESS UART's SPR REGISTER	       *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_reg_modem_13  ( unsigned long		        caller_parm,
				     struct marb_struct FARm *  	marb,
	      			     struct ssp_struct FARm *   	sspcb,
	      			     struct slot_struct FARm *  	slot,
	      			     struct icp_global_struct FARm * 	gregs,
				     struct cin_struct FARm *	  	iregs,
	      			     struct cout_struct FARm *	  	oregs
				   )
#else
static void  mpa_srvc_reg_modem_13  (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
sspcb       = sspcb ;
iregs       = iregs ;

/*Start of function's logic							*/
if ( slot->slot_state != 0x80 )				/*Is Brd still present?	*/
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_REG_MODEM_REMOVED ;	/*Set completion status	*/
	return ;
}
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
if ( ((oregs->cout_flow_config & 0x10) != (oregs->cout_intnl_flow_ctrl & 0x10)) ||
     ((oregs->cout_intnl_flow_ctrl & 0x08) == 0x08) )	/*LCR char xmit done	*/
{							/*If no, check deadman	*/
	if ( frame_delay_chk (DEADMAN_DELAY, gregs, slot) == 0x00 )
	{						/*If deadman not expired*/
		return ;
	}
	(*slot->clean_up)(slot, (unsigned char)0xFF) ;
	marb->srvc_status = ERR_REG_MODEM_DEADMAN ; 	/*Set cmpltn status	*/
	marb->srvc_state = 0x00 ;			/*Set "complete" state	*/
	return ;
}

/*Modem's LCR written to, set output control signals to address	UART's  	*/
/*Scratchpad Register (SPR)							*/
oregs->cout_cntrl_sig = 0x177 ;				/*Addr UART's SPR	*/
slot->frame_ctr = gregs->gicp_frame_ctr ;		/*Snap shot for timing	*/
marb->cont_funct = mpa_srvc_reg_modem_16 ;		/*NOTE: Functns 14 & 15	*/
							/*	deleted. 	*/
							/*	Originaly init'd*/
							/*	MCR		*/
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*End of REG MODEM BOARD - 13th Function, mpa_srvc_reg_modem_13		*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *    REGISTER MODEM BOARD - 16th FUNCTION, INITIALIZE UART's SPR REGISTER     *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_reg_modem_16 ( unsigned long		        caller_parm,
				     struct marb_struct FARm *  	marb,
	      			     struct ssp_struct FARm *   	sspcb,
	      			     struct slot_struct FARm *  	slot,
	      			     struct icp_global_struct FARm * 	gregs,
				     struct cin_struct FARm *	  	iregs,
	      			     struct cout_struct FARm *	  	oregs
				   )
#else
static void  mpa_srvc_reg_modem_16 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
sspcb       = sspcb ;
iregs       = iregs ;

/*Start of function's logic							*/
if ( slot->slot_state != 0x80 )				/*Is Brd still present?	*/
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_REG_MODEM_REMOVED ;	/*Set completion status	*/
	return ;
}
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
if ( frame_delay_chk ( (unsigned short int)0x04, gregs, slot ) == 0 )
	return ;					/*4 frames not elapsed	*/

/*Have delayed long enough for MPA to "see" control signal setting indicating	*/
/*that output be directed to UART's Scratchpad Register (SPR)			*/
oregs->cout_xon_1         = 0x00 ;  			/*SPR = 0		*/
oregs->cout_flow_config  ^= 0x10 ;			/*Initiate xmit'g char	*/

/*Set up & wait till SPR output sent						*/
slot->frame_ctr = gregs->gicp_frame_ctr ;		/*Snap shot for timing	*/
marb->cont_funct = mpa_srvc_reg_modem_17 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*End of REG MODEM BOARD - 16th Function, mpa_srvc_reg_modem_16		*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *    REGISTER MODEM BOARD - 17th FUNCTION, RESET SLOT			       *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_reg_modem_17  ( unsigned long		        caller_parm,
				     struct marb_struct FARm *  	marb,
	      			     struct ssp_struct FARm *   	sspcb,
	      			     struct slot_struct FARm *  	slot,
	      			     struct icp_global_struct FARm * 	gregs,
				     struct cin_struct FARm *	  	iregs,
	      			     struct cout_struct FARm *	  	oregs
				   )
#else
static void  mpa_srvc_reg_modem_17  (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
sspcb       = sspcb ;
iregs       = iregs ;

/*Start of function's logic							*/
if ( slot->slot_state != 0x80 )				/*Is Brd still present?	*/
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_REG_MODEM_REMOVED ;	/*Set completion status	*/
	return ;
}
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
if ( ((oregs->cout_flow_config & 0x10) != (oregs->cout_intnl_flow_ctrl & 0x10)) ||
     ((oregs->cout_intnl_flow_ctrl & 0x08) == 0x08) )	/*SPR char xmit done	*/
{							/*If no, check deadman	*/
	if ( frame_delay_chk (DEADMAN_DELAY, gregs, slot) == 0x00 )
	{						/*If deadman not expired*/
		return ;
	}
	(*slot->clean_up)(slot, (unsigned char)0xFF) ;
	marb->srvc_status = ERR_REG_MODEM_DEADMAN ; 	/*Set cmpltn status	*/
	marb->srvc_state = 0x00 ;			/*Set "complete" state	*/
	return ;
}

/*Modem's SPR written to, Reset SLOT						*/
oregs->cout_cntrl_sig = 0x188 ;				/*MPA's Slot Reset	*/
slot->read_toggle     = 0x00 ;				/*UART I/O Read toggle	*/
slot->frame_ctr  = gregs->gicp_frame_ctr ;		/*Snap shot for timing	*/
marb->cont_funct = mpa_srvc_reg_modem_18 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*End of REG MODEM BOARD - 17th Function, mpa_srvc_reg_modem_17		*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *    REGISTER MODEM BOARD - 18th FUNCTION, SAVE MODEM/UART's BASE I/O ADDR &  *
 *					    BEGIN TO ESTABLISH SSP OUTPUT      *
 *					    CONTROL SIGNAL SETTINGS	       *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_reg_modem_18 ( unsigned long		        caller_parm,
				     struct marb_struct FARm *  	marb,
	      			     struct ssp_struct FARm *   	sspcb,
	      			     struct slot_struct FARm *  	slot,
	      			     struct icp_global_struct FARm * 	gregs,
				     struct cin_struct FARm *	  	iregs,
	      			     struct cout_struct FARm *	  	oregs
				   )
#else
static void  mpa_srvc_reg_modem_18 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

unsigned short int	raw_reset_tdm ;			/*Reset TDM		*/



/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
sspcb       = sspcb ;

/*Start of function's logic							*/
if ( slot->slot_state != 0x80 )				/*Is Brd still present?	*/
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_REG_MODEM_REMOVED ;	/*Set completion status	*/
	return ;
}
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
if ( frame_delay_chk ( (unsigned short int)0x08, gregs, slot ) == 0 )
	return ;					/*8 frames not elapsed	*/

/*Have delayed long enough for MPA to "see" control signal setting indicating	*/
/*that Slot should be reset, get base I/O port address from TDM			*/
raw_reset_tdm  = iregs->cin_tdm_early ;			/*Get raw TDM		*/
slot->io_base  = (unsigned short int)(0x02E8 |		/*Develop I/O port addr	*/
		 ((((unsigned int)raw_reset_tdm) & 0x200) >> 1)   |
		 ((((unsigned int)raw_reset_tdm) & 0x100) >> 4) ) ;

/*Set up & establish inverse of Requestor's output control signals in order for	*/
/*MPA to "see" change in control signals which in turn causes MPA to write to	*/
/*UART's MCR. NOTE: "Toggling" control signals assures UART MCR control signals	*/
/*coherent with Requestor's SSP control signal settings				*/
oregs->cout_cntrl_sig = ((slot->sv_cout_cntrl_sig & 0x0237) | 0x0004) ^ 0x0003 ;
slot->frame_ctr = gregs->gicp_frame_ctr ;		/*Snap shot for timing	*/
marb->cont_funct = mpa_srvc_reg_modem_19 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*End of REG MODEM BOARD - 18th Function, mpa_srvc_reg_modem_18		*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *    REGISTER MODEM BOARD - 19th FUNCTION, SET SSP OUTPUT CONTROL SIGNAL HENCE*
 *					    SINCE CONTROL SIGNALS CHANGED, UART*
 *					    MCR IS UPDATED		       *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_reg_modem_19 ( unsigned long		        caller_parm,
				     struct marb_struct FARm *  	marb,
	      			     struct ssp_struct FARm *   	sspcb,
	      			     struct slot_struct FARm *  	slot,
	      			     struct icp_global_struct FARm * 	gregs,
				     struct cin_struct FARm *	  	iregs,
	      			     struct cout_struct FARm *	  	oregs
				   )
#else
static void  mpa_srvc_reg_modem_19 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
sspcb       = sspcb ;
iregs       = iregs ;

/*Start of function's logic							*/
if ( slot->slot_state != 0x80 )				/*Is Brd still present?	*/
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_REG_MODEM_REMOVED ;	/*Set completion status	*/
	return ;
}
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
if ( frame_delay_chk ( (unsigned short int)0x04, gregs, slot ) == 0 )
	return ;					/*4 frames not elapsed	*/

/*Have delayed long enough for MPA to "see" inverse of actual control signals	*/
/*hence MCR register's control signals may have been refreshed (if MPA saved 	*/
/*control signals different).  Restore requestor's control signals - this 	*/
/*assures MPA updates MCR to reflect requestor's control signal setting		*/
oregs->cout_cntrl_sig = (slot->sv_cout_cntrl_sig & 0x0237) | 0x0004 ;
slot->frame_ctr = gregs->gicp_frame_ctr ;		/*Snap shot for timing	*/
marb->cont_funct = mpa_srvc_reg_modem_20 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*End of REG MODEM BOARD - 19th Function, mpa_srvc_reg_modem_19		*/

/*#page*/
/*******************************************************************************
 *                                                                             *
 *    	      REGISTER MODEM BOARD - 20th FUNCTION, REQUEST COMPLETE           *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_reg_modem_20 ( unsigned long		        caller_parm,
				     struct marb_struct FARm *  	marb,
	      			     struct ssp_struct FARm *   	sspcb,
	      			     struct slot_struct FARm *  	slot,
	      			     struct icp_global_struct FARm * 	gregs,
				     struct cin_struct FARm *	  	iregs,
	      			     struct cout_struct FARm *	  	oregs
				   )
#else
static void  mpa_srvc_reg_modem_20 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
sspcb       = sspcb ;
iregs       = iregs ;
oregs       = oregs ;

/*Start of function's logic							*/
if ( slot->slot_state != 0x80 )				/*Is Brd still present?	*/
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_REG_MODEM_REMOVED ;	/*Set completion status	*/
	return ;
}
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
if ( frame_delay_chk ( (unsigned short int)0x04, gregs, slot ) == 0 )
	return ;					/*4 frames not elapsed	*/

/*Have delayed long enough for MPA to "see" control signals, complete request	*/
slot->slot_state = 0xFF ;				/*Indicate Modem presnt	*/
slot->init_state = 0xFF ;				/*MODEM/UART Openned	*/
mpa_srvc_reg_modem_clean_up (slot, (unsigned char)0x00 ) ; /*Restore SSP regs	*/
marb->srvc_status = 0x00 ;				/*Set completion status	*/
marb->srvc_state  = 0x00 ;				/*Set Srvc State = Done	*/
return ;						/*Exit, Request Complete*/

}	/*End of REG MODEM BOARD - 20th Function, mpa_srvc_reg_modem_20		*/





/*#stitle SLOT SERVICE - REGISTER MODEM BOARD, LOCAL FUNCTION#*/
/*#page*/
/****************************************************************************************
 *                                                                             		*
 * REGISTER MODEM BOARD, LOCAL FUNCTION - Clean Up SSP (mpa_srvc_reg_modem_clean_up)	*
 *                                                                     			*
 * PURPOSE: To restore state of SSP registers when service has been completed (either 	*
 *	   due to error or successful completion					*
 *											*
 * NOTES:										*
 *                                                                     			*
 * CALL: void  mpa_srvc_reg_modem_clean_up  (struct slot_struct FARm    *slot,		*
 *					     unsigned char		type)		*
 *                                                                     			*
 *			where:	slot = Slot Control Block pointer			*
 *				type = Clean up type:					*
 *					o 0x00 = Normal i.e. Reg Modem request cmplt	*
 *					o 0xFF = Abnormal i.e. Reg Modem request	*
 *						 aborted due to some failure or Board	*
 *						 removed				*
 *                                                                     			*
 *                                                                     			*
 * RETURN: None										*
 *                                                                     			*
 ****************************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R3*/
static void	mpa_srvc_reg_modem_clean_up (struct slot_struct FARm   	*slot,
					     unsigned char		type)
#else
static void	mpa_srvc_reg_modem_clean_up (slot, type)

struct slot_struct FARm	*slot;
unsigned char		type;
#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

/*SSP variables relating to MARB's/Request's specific SLOT (Channel)			   */
#ifndef					RAMP_BSDI		/*R14*/
struct cout_struct FARm *	  	oregs ;		/*Slot's SSP Output regs	   */
#else								/*BSDI Compile		   */
volatile struct cout_struct FARm *	oregs ;		/*Slot's SSP Output regs	   */
#endif								/*End RAMP_BSDI Conditional*/



/************************
 *   START OF CODE	*
 ************************/

oregs = slot->output ;					/*Slot's output regs ptr*/

/*Delay if actually canceling/aborting an outstanding Register Modem Board MARB	*/
/*that is in process of Restting Slot. This allows time for MPA to handle Reset	*/
if ((type == 0xFF) 					&&
    (slot->actv_marb->srvc_status == MARB_OUT_CANCELED) &&
    (oregs->cout_cntrl_sig == 0x188)  			)
     /*If abort'g MARB, delay to give MPA chance to reset			*/
	while ( frame_delay_chk ( (unsigned short int)0x08, 	
				  slot->actv_marb->sspcb->global, slot)	== 0 )
	{ }

/*Restore saved SSP registers							*/
oregs->cout_flow_config = (slot->sv_cout_flow_config & 0x2F) |  /*Rstr out sigs */
                          (oregs->cout_flow_config & 0x10) | 0x40 ;
oregs->cout_cntrl_sig   = (slot->sv_cout_cntrl_sig & 0x0237) | 0x0004 ;
oregs->cout_xon_1	= slot->sv_cout_xon_1 ;		/*Restore XON char	*/
#ifndef 				NO_FN_PROTO_PARMS /*Compile if Protypes OK*/
slot->clean_up = (void (*)(struct slot_struct FARm *, unsigned char))0 ;
#else							  /*Compile if NO Protypes*/
slot->clean_up = (void (*)())0 ;
#endif
return ;

}					/*End of mpa_srvc_reg_modem_clean_up	*/
/*#page*/
/****************************************************************************************
 *                                                                             		*
 * 	REGISTER MODEM BOARD, LOCAL FUNCTION - Outstanding Read Clean Up SSP 		*
 *				(mpa_srvc_reg_modem_cln_rd)				*
 *                                                                     			*
 * PURPOSE: To restore state of the SSP when UART Read outstanding has been aborted.	*
 *											*
 * NOTES:										*
 *                                                                     			*
 * CALL: void  mpa_srvc_reg_modem_cln_rd    (struct slot_struct FARm    *slot,		*
 *					     unsigned char		type)		*
 *                                                                     			*
 *			where:	slot = Slot Control Block pointer			*
 *				type = Clean up type:					*
 *					o 0x00 = Normal i.e. Reg Modem request cmplt	*
 *					o 0xFF = Abnormal i.e. Reg Modem request	*
 *						 aborted due to some failure or Board	*
 *						 removed				*
 *                                                                     			*
 *                                                                     			*
 * RETURN: None										*
 *                                                                     			*
 ****************************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void	mpa_srvc_reg_modem_cln_rd   (struct slot_struct FARm   	*slot,	   /*R9*/
					     unsigned char		type)
#else
static void	mpa_srvc_reg_modem_cln_rd   (slot, type)

struct slot_struct FARm	*slot;
unsigned char		type;
#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

/*SSP variables									*/
#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * 	gregs ;		/*Slot's Global regs	*/
struct cin_struct FARm *	  	iregs ;		/*Slot's Input regs	*/
struct cin_bank_struct FARm *		actv_bank ;	/*Slot's active Bank	*/
struct cout_struct FARm *	  	oregs ;		/*Slot's Output regs	*/
#else								/*BSDI Compile	*/
volatile struct icp_global_struct FARm * gregs ;	/*Slot's Global regs	*/
volatile struct cin_struct FARm *	 iregs ;	/*Slot's Input regs	*/
volatile struct cin_bank_struct FARm *	 actv_bank ;	/*Slot's active Bank	*/
volatile struct cout_struct FARm *	 oregs ;	/*Slot's Output regs	*/
#endif								/*End RAMP_BSDI Conditional*/

/*Misc variables								*/
unsigned char				status ;	/*Returned status	*/



/************************
 *   START OF CODE	*
 ************************/

/*Set Slot's SSP pointer's							*/
gregs	  = slot->global ;				/*Slot's global regs ptr*/
iregs	  = slot->input ;				/*Slot's input regs ptr	*/
oregs	  = slot->output ;				/*Slot's output regs ptr*/
actv_bank = ( (iregs->cin_locks & 0x01) == 0x00 ) 		  ? /*Bank A actv?*/
	      (struct cin_bank_struct FARm *)&(iregs->cin_bank_a) : /*If yes	  */
	      (struct cin_bank_struct FARm *)&(iregs->cin_bank_b) ; /*If no	  */

/*Assure MPA AUX reg character sent (Read UART toggle)				*/
while ( ((oregs->cout_flow_config & 0x10) != (oregs->cout_intnl_flow_ctrl & 0x10)) ||
        ((oregs->cout_intnl_flow_ctrl & 0x08) == 0x08) ) /*AUX reg char xmit dne*/
{							 /*If no, check deadman	*/
	if ( frame_delay_chk (DEADMAN_DELAY, gregs, slot) == 0xFF )
		break ;					/*If deadman expired	*/
}

/*MPA's AUX Reg READ UART's SPR request sent, wait for SPR byte to be received	*/
while ( (actv_bank->cin_bank_num_chars) != 1 )		/*Have recv'd SPR byte?	*/
{							/*If no, check deadman	*/
	if ( frame_delay_chk ( (DEADMAN_DELAY+5), gregs, slot) == 0xFF )
		break ;					/*If deadman expired	*/
}

/*May have read (and received) UART's char, if so update tail pointer to reflect*/
/*receipt of character								*/
if ( actv_bank->cin_bank_num_chars == 1 )		/*Was UART char recv'd?	*/
	status = inc_tail ( iregs ) ;			/*If yes, update tail	*/

/*Have cleaned up outstanding read, now restore saved SSP registers by calling	*/
/*"standard" clean up routine.							*/
mpa_srvc_reg_modem_clean_up ( slot, type ) ;		/*Restore sv'd SSP regs	*/
return ;

}					/*End of mpa_srvc_reg_modem_cln_rd	*/







/*#stitle SLOT SERVICE - DEREGISTER MODEM BOARD (mpa_srvc_dereg_modem)#*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *                  DEREGISTER MODEM BOARD - SERVICE ENTRY POINT               *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
extern unsigned short int	mpa_srvc_dereg_modem (struct marb_struct FARm  *marb)
#else
extern unsigned short int	mpa_srvc_dereg_modem (marb)

struct marb_struct FARm  *marb;
#endif
{

/************************
 *     LOCAL DATA	*
 ************************/

/*General working variables							*/
unsigned char		status ;			/*Funct. ret status	*/



/************************
 *   START OF CODE	*
 ************************/

status = val_marb (marb) ;				/*Validate caller's MARB*/
if ( status != 0 )					/*Is MARB OK to use?	*/
	return (ERR_DEREG_MODEM_MARB) ;			/*If no, return error	*/
if ( (marb->in_use != 0) || (marb->req_type != 0x00) )	/*Init MARB Req & Waitd?*/
	return (ERR_DEREG_MODEM_MARB) ;			/*If no, return error	*/
marb->cont_funct = mpa_srvc_dereg_modem_1 ;		/*Init MARB's FN ptr	*/
return ( async_req_hndlr(marb) ) ;			/*Go handle request	*/

}			/*END of mpa_srvc_dereg_modem Service Entry function	*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *  DEREGISTER MODEM BOARD - 1st (and only) Function, Clean Up SLOTCB Function *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_dereg_modem_1 ( unsigned long		        caller_parm,
				      struct marb_struct FARm *  	marb,
	      			      struct ssp_struct FARm *   	sspcb,
	      			      struct slot_struct FARm *  	slot,
	      			      struct icp_global_struct FARm * 	gregs,
				      struct cin_struct FARm *	  	iregs,
	      			      struct cout_struct FARm *	  	oregs
				    )
#else
static void  mpa_srvc_dereg_modem_1 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

/*Variables used to determine MPA Slot Power States				    */ 	/*R12*/

#ifndef					RAMP_BSDI		/*R14*/
struct cin_struct  FARm *	   mpa_1st_iregs ;	/*MPA's 1st Slot Input regs 	   */
struct cin_bank_struct FARm *	   actv_bank ;		/*Ptr to active bank	    	   */
#else								/*BSDI Compile		   */
volatile struct cin_struct  FARm *      mpa_1st_iregs;	/*MPA's 1st Slot Input regs 	   */
volatile struct cin_bank_struct FARm *	actv_bank ;	/*Ptr to active bank	    	   */
#endif								/*End RAMP_BSDI Conditional*/

struct slot_struct FARm *  	mpa_1st_slot ;		/*MPA's 1st SLOTCB	    */
unsigned char			slot_pwr_state ;	/*MPA SLOT PWR State	    */

/*Variables used to Deregister Modem Board					 */
unsigned short int		status ;		/*Status from Cancel MARB*/
struct marb_struct FARm	*	can_marb_ptr ;		/*Ptr to canceled MARB	 */
unsigned long int		svd_int_msk ;		/*Int Msk prior to Dsabl */

/*General working variables							*/	/*R12*/
unsigned int			i ;			/*Loop counter		*/
unsigned int			j ;			/*Loop counter		*/



/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
gregs       = gregs ;
iregs       = iregs ;
oregs       = oregs ;

/*For request's MPA, update the Power Switch state for each Slot Member	     			*/ 	/*R12*/
mpa_1st_iregs = sspcb->mpa[(marb->slot_chan >> 4)].input_base ;   /*Pt to MPA's base Input regs	*/
mpa_1st_slot  = sspcb->mpa[(marb->slot_chan >> 4)].slot_array_hd; /*Pt to 1st SLOT to update	*/
for ( i = 0; i < 16; i = i + 4 )				  /*Update MPA's SLOT PWR States*/
{
	actv_bank = ( ((mpa_1st_iregs+i)->cin_locks & 0x01) == 0x00 )		?
      		(struct cin_bank_struct FARm *)&((mpa_1st_iregs+i)->cin_bank_a) :
      		(struct cin_bank_struct FARm *)&((mpa_1st_iregs+i)->cin_bank_b) ;
	slot_pwr_state = ( ((actv_bank->cin_bank_signals) & 0x0800) == 0x0000)  ?
		(unsigned short int)0x0000 : (unsigned short int)0x00FF 	;
	for ( j = 0; j < 4; ++j )				  /*Loop to update SLOT PWR State*/
		(mpa_1st_slot+i+j)->power_state = slot_pwr_state ;
}													/*R12*/

/*Complete Request's validation							*/
if ( slot->slot_state == 0x00 )				/*Is Slot alrdy empty?	*/
{							/*If yes, error		*/
	marb->srvc_status = ERR_DEREG_MODEM_NOT ;	/*Indicate error	*/
	return ;					/*Exit			*/
}
if ( slot->req_outstnd != 0x00 ) 			/*Is a Req Outstand'g 	*/
{							/*If yes, go attempt to	*/
							/*cancel & clean up	*/
	status = mpa_srvc_can_marb (sspcb, 
				    marb->slot_chan, 
				    (struct marb_struct FARm * FARm *)&can_marb_ptr) ;
}

/*Prior to marking MPA Slot empty, and thus making it available for PnP 	*/
/*probing, gain access to SSPCB List thus Blocking Admin/Diag Poll function	*/
/*from potentually accessing a SLOTCB that is in process of being cleaned up	*/ /*R9*/
while ( (mpa_fn_access_sspcb_lst ((unsigned long int FARm *)&svd_int_msk)) != 0xFF ) {} /*Spin till access granted*/
slot->slot_state      = 0x00 ;				/*Indicate no Modem prsnt*/
slot->init_state      = 0x00 ;				/*MODEM/UART Closed	*/
slot->pnp_probe_state = 0x0000 ;			/*PnP not initialized	*/
marb->srvc_status     = 0x00 ;				/*Set completion status	*/
mpa_fn_relse_sspcb_lst (svd_int_msk) ;						    /*R9*/
return ;						/*Exit, Request Complete*/

}	/*End of DEREGISTER MODEM BOARD - 1st Function, mpa_srvc_dereg_modem_1	*/








/*#stitle SLOT SERVICE - SLOT STATUS (mpa_srvc_slot_status)#*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *                  	SLOT STATUS - SERVICE ENTRY POINT                      *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
extern unsigned short int   mpa_srvc_slot_status (struct marb_struct FARm  *marb,
						  struct slot_struct FARm * FARm * slot_ret_parm)
#else
extern unsigned short int   mpa_srvc_slot_status (marb, slot_ret_parm)

struct marb_struct FARm  *marb;
struct slot_struct FARm * FARm * slot_ret_parm;
#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

/*General working variables							*/
unsigned char		status ;			/*Funct. ret status	*/
unsigned char FARm *	sv_scratch_ptr ;		/*Requestor's scrtch ptr*/
unsigned short int	ret_status ;			/*Completion status	*/



/************************
 *   START OF CODE	*
 ************************/

status = val_marb (marb) ;				/*Validate caller's MARB*/
if ( status != 0 )					/*Is MARB OK to use?	*/
	return (ERR_SLOT_STATUS_MARB) ;			/*If no, return error	*/
if ( (marb->in_use != 0) || (marb->req_type != 0x00) )	/*Init MARB Req & Waitd?*/
	return (ERR_SLOT_STATUS_MARB) ;			/*If no, return error	*/
sv_scratch_ptr     = marb->scratch_area ;		/*Sv scratch ptr	*/
marb->scratch_area = (unsigned char FARm *)slot_ret_parm ; /*Pass ret parm ptr	*/
marb->cont_funct   = mpa_srvc_slot_status_1 ;		/*Init MARB's FN ptr	*/
ret_status         = async_req_hndlr(marb) ;		/*Go handle request	*/
marb->scratch_area = sv_scratch_ptr ;			/*Restore scratch ptr	*/
return (ret_status) ;					/*Request complete, exit*/
}			/*END of mpa_srvc_slot_status Service Entry function	*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *	      SLOT STATUS - 1st (and only) Function, Return Slot pointer       *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_slot_status_1  ( unsigned long		        caller_parm,
				       struct marb_struct FARm *  	marb,
	      			       struct ssp_struct FARm *   	sspcb,
	      			       struct slot_struct FARm *  	slot,
	      			       struct icp_global_struct FARm * 	gregs,
				       struct cin_struct FARm *	  	iregs,
	      			       struct cout_struct FARm *  	oregs
				    )
#else
static void  mpa_srvc_slot_status_1  (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
sspcb       = sspcb ;
gregs       = gregs ;
iregs       = iregs ;
oregs       = oregs ;

/*Start of function's logic							*/
*((struct slot_struct FARm *  FARm *)marb->scratch_area) = slot ; /*Ret SLOTCB ptr*/
marb->srvc_status = 0x00 ;				/*Set completion status	*/
return ;						/*Exit, Request Complete*/

}	/*End of SLOT STATUS - 1st Function, mpa_srvc_slot_status_1		*/









/*#stitle SLOT SERVICE - INITIALIZE (OPEN) UART/MODEM (mpa_srvc_init_modem)#*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *              INITIALIZE (OPEN) UART/MODEM - SERVICE ENTRY POINT             *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
extern unsigned short int	mpa_srvc_init_modem (struct marb_struct FARm  *marb)
#else
extern unsigned short int	mpa_srvc_init_modem (marb)

struct marb_struct FARm  *marb;
#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

/*General working variables							*/
unsigned char		status ;			/*Funct. ret status	*/



/************************
 *   START OF CODE	*
 ************************/

status = val_marb (marb) ;				/*Validate caller's MARB*/
if ( status != 0 )					/*Is MARB OK to use?	*/
	return (ERR_INIT_MODEM_MARB) ;			/*If no, return error	*/
if ( marb->in_use == 0 )				/*Initial MARB Request?	*/
{							/*If yes, init Fn ptr	*/
	marb->gate = 0x00 ;				/*Set 1st call flag/gate*/
	marb->cont_funct = mpa_srvc_init_modem_1 ;	/*Init MARB's FN ptr	*/
}
else							/*"Call Back" MARB,	*/
{							/*Validate SRVC active	*/
	if ( marb->actv_srvc != MARB_ACTV_SRVC_INIT_MODEM ) /*Valid "Call Back"?*/
	{						/*If no, exit		*/
		marb->srvc_status = ERR_INIT_MODEM_CALL_BK ;
		marb->srvc_state  = 0x00 ;
		return ((unsigned short int)(((unsigned short int)(marb->srvc_state)<<8)+
                                               marb->srvc_status) ) ;
	}
}
return ( async_req_hndlr(marb) ) ;			/*Go handle request	*/


}			/*END of mpa_srvc_reg_modem Service Entry function	*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 * INITIALIZE (OPEN) UART/MODEM BOARD - 1st FUNCTION, MPA-to-UART FLOW CONTROL *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_init_modem_1 ( unsigned long		        caller_parm,
				     struct marb_struct FARm *  	marb,
	      			     struct ssp_struct FARm *   	sspcb,
	      			     struct slot_struct FARm *  	slot,
	      			     struct icp_global_struct FARm * 	gregs,
				     struct cin_struct FARm *	  	iregs,
	      			     struct cout_struct FARm *	  	oregs
				   )
#else
static void  mpa_srvc_init_modem_1 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

/*General working variables							*/
unsigned int		i ;				/*Loop counter		*/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
sspcb       = sspcb ;
iregs       = iregs ;
oregs       = oregs ;

/*Start of function's logic							*/
if ( (marb->gate == 0x00) && (slot->req_outstnd != 0x00) ) /*Req Outstand'g?	*/
{							/*If yes, return error	*/
	marb->srvc_status = ERR_INIT_MODEM_REQ_OUT ;	/*Indicate error	*/
	return ;					/*Exit			*/
}
if ( slot->slot_state != 0xFF )				/*Is Modem registered?	*/
{							/*If no, error		*/
	marb->srvc_status = ERR_INIT_MODEM_NOT_REG ;	/*Indicate error	*/
	return ;					/*Exit			*/
}

/*Caller's request valid, if 1st call initialize MARB's Slot Control Block	*/
if ( marb->gate == 0x00 )				/*1st time Fn called?	*/
{							/*If yes		*/
	slot->flow_setting = marb->sup_uchar0 ;		/*Save flow setting	*/
	slot->num_bits	   = marb->sup_uchar1 ;		/*Save # bits setting	*/
	slot->stop_bits	   = marb->sup_uchar2 ;		/*Save # stop bits	*/
	slot->parity   	   = marb->sup_uchar3 ;		/*Save parity setting	*/
	slot->baud	   = marb->sup_ushort ;		/*Save baud setting	*/

   /*Determine how to initialize SLOTCB's Open scratch area			*/
	if ( marb->scratch_area == (unsigned char FARm *)0 ) /*Is scratch area spec'd*/
		for ( i = 0; i < SLOT_SCRATCH_SZ; ++i )	     /*If no, 0 slot area    */
			slot->oscratch[i] = (unsigned char)0x00 ;
	else
		for ( i = 0; i < SLOT_SCRATCH_SZ; ++i )	     /*If yes, copy area*/
			slot->oscratch[i] = marb->scratch_area[i] ;

   /*Save SSP registers that will be used to initialized Slot & Modem Board's UART	*/
   /*and establish "clean up" function in case De-register Modem request handled 	*/
   /*asynchronously								*/
	slot->sv_cout_flow_config = oregs->cout_flow_config ;
	slot->sv_cout_cntrl_sig   = oregs->cout_cntrl_sig ;
	slot->sv_cout_xon_1       = oregs->cout_xon_1 ;
#ifndef 				NO_FN_PROTO_PARMS /*Compile if Protypes OK*/
	slot->clean_up   = (void (*)(struct slot_struct FARm *, unsigned char))mpa_srvc_init_modem_clean_up ;
#else							  /*Compile if NO Protypes*/
	slot->clean_up   = (void (*)())mpa_srvc_init_modem_clean_up ;
#endif
	slot->frame_ctr  = gregs->gicp_frame_ctr ;	/*Snap shot for timing	*/
	marb->actv_srvc  = MARB_ACTV_SRVC_INIT_MODEM ;	/*Indicate outstnd'g req*/
	marb->gate = 0xFF ;				/*Set for repeat call	*/
	oregs->cout_flow_config = (oregs->cout_flow_config & 0x10) | 0x40 | 0x08 ;
}

/*Determine if Slot already openned.  If not openned (i.e. Closed), set up and	*/
/*place MPA into Reset state thus flushing internal input/output buffers, else	*/
/*set up to set MPA-to-UART Flow Control					*/
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
if ( slot->init_state != 0xFF )				/*Is Slot closed?	*/
{							/*If yes, init'g closed	*/
							/*slot, go issue reset	*/
	oregs->cout_cntrl_sig = 0x188 ;			/*MPA's Slot Reset	*/
	slot->read_toggle     = 0x00 ;			/*UART I/O Read toggle	*/
	slot->frame_ctr  = gregs->gicp_frame_ctr ;	/*Snap shot for timing	*/
	marb->cont_funct = mpa_srvc_init_modem_1A ;
	return ;					/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/
}

/*Set up so output sent to MPA's AUX reg - MPA-to-UART Flow Control		*/
oregs->cout_cntrl_sig = 0x1AA ;				/*Send control sigs to	*/
							/*MPA indicat'g to direct*/
							/*output to AUX reg,MPA-*/
							/*to-UART Flow Cntrl	*/

/*Exit function & delay 4 frames thus allowing time for MPA to "see" AUX Reg 	*/
/*control signal setting							*/
marb->cont_funct = mpa_srvc_init_modem_2 ;
							/*Flow Cntrl Function	*/
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}		/*End of INIT MODEM BOARD - 1st Function, mpa_srvc_init_modem_1	*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *  	       INITIALIZE MODEM BOARD - 1A FUNCTION, RESET COMPLETE	       *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_init_modem_1A ( unsigned long		        caller_parm,
				     struct marb_struct FARm *  	marb,
	      			     struct ssp_struct FARm *   	sspcb,
	      			     struct slot_struct FARm *  	slot,
	      			     struct icp_global_struct FARm * 	gregs,
				     struct cin_struct FARm *	  	iregs,
	      			     struct cout_struct FARm *	  	oregs
				   )
#else
static void  mpa_srvc_init_modem_1A (caller_parm, marb, sspcb, slot, gregs, iregs, oregs )

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
sspcb       = sspcb ;
iregs       = iregs ;

/*Start of function's logic							*/
if ( slot->slot_state != 0xFF )				/*Is Brd still present?	*/
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_INIT_MODEM_REMOVED ;	/*Set completion status	*/
	return ;
}
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
if ( frame_delay_chk ( (unsigned short int)0x08, gregs, slot ) == 0 )
	return ;					/*8 frames not elapsed	*/

/*Have delayed long enough for MPA to "see" control signal setting indicating	*/
/*that Slot should be reset, set up so output sent to MPA's AUX reg - 		*/
/*MPA-to-UART Flow Control							*/
oregs->cout_cntrl_sig = 0x1AA ;				/*Send control sigs to	*/
							/*MPA indicat'g to direct*/
							/*output to AUX reg,MPA-*/
							/*to-UART Flow Cntrl	*/
marb->cont_funct = mpa_srvc_init_modem_2 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/
}		/*End of INIT MODEM BOARD - 1A Function, mpa_srvc_init_modem_1A	*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 * INITIALIZE MODEM BOARD - 2nd FUNCTION, INITIALIZE MPA-to-UART FLOW CONTROL  *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_init_modem_2 ( unsigned long		        caller_parm,
				     struct marb_struct FARm *  	marb,
	      			     struct ssp_struct FARm *   	sspcb,
	      			     struct slot_struct FARm *  	slot,
	      			     struct icp_global_struct FARm * 	gregs,
				     struct cin_struct FARm *	  	iregs,
	      			     struct cout_struct FARm *	  	oregs
				   )
#else
static void  mpa_srvc_init_modem_2 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs )

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

#ifndef				RAMP_LINUX		/*R14*/
void		(FARm *sv_cont_funct)() ;		/*Saved addr of Register*/
							/*Modem request's "link"*/
							/*function		*/
#else
void            (FARm *sv_cont_funct)(
				      unsigned long int        caller_parm,
				      struct marb_struct       FARm * ,
				      struct ssp_struct        FARm * ,
				      struct slot_struct       FARm * ,
				      struct mpa_global_s      FARm * ,
				      struct mpa_input_s       FARm * ,
				      struct mpa_output_s      FARm * 
				     ) ;
#endif


/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;

/*Start of function's logic							*/
sv_cont_funct = marb->cont_funct ;			/*Save addr of "link" fn*/
mpa_srvc_reg_modem_4 ( (unsigned long)MARB_ACTV_SRVC_INIT_MODEM,
                       marb,  sspcb, slot,		/*Go send requestor's 	*/
		       gregs, iregs, oregs ) ;		/*MPA-to-UART Flow set'g*/
if ( (marb->srvc_state == 0x00)		   ||		/*Did error occur? or	*/
     (sv_cont_funct    == marb->cont_funct) )		/*Wait'g for frame delay*/
	return ;					/*If yes		*/

/*Set up & wait till MPA AUX Reg Flow Control setting (character) sent then 	*/
/*address LCR									*/
marb->cont_funct = mpa_srvc_init_modem_3 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*End of INIT MODEM BOARD - 2nd FUNCTION, mpa_srvc_init_modem_2		*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 * 	INITIALIZE MODEM BOARD - 3rd FUNCTION, ADDRESS UART'S LCR REGISTER     *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_init_modem_3 ( unsigned long		        caller_parm,
				     struct marb_struct FARm *  	marb,
	      			     struct ssp_struct FARm *   	sspcb,
	      			     struct slot_struct FARm *  	slot,
	      			     struct icp_global_struct FARm * 	gregs,
				     struct cin_struct FARm *	  	iregs,
	      			     struct cout_struct FARm *	  	oregs
				   )
#else
static void  mpa_srvc_init_modem_3 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs )

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

#ifndef				RAMP_LINUX		/*R14*/
void		(FARm *sv_cont_funct)() ;		/*Saved addr of Register*/
							/*Modem request's "link"*/
							/*function		*/
#else
void            (FARm *sv_cont_funct)(
				      unsigned long int        caller_parm,
				      struct marb_struct       FARm * ,
				      struct ssp_struct        FARm * ,
				      struct slot_struct       FARm * ,
				      struct mpa_global_s      FARm * ,
				      struct mpa_input_s       FARm * ,
				      struct mpa_output_s      FARm * 
				     ) ;
#endif


/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;

/*Start of function's logic							*/
sv_cont_funct = marb->cont_funct ;			/*Save addr of "link" fn*/
mpa_srvc_reg_modem_5 ( (unsigned long)MARB_ACTV_SRVC_INIT_MODEM,
                       marb,  sspcb, slot,		/*Go set control sig's	*/
		       gregs, iregs, oregs ) ;		/*to addr LCR reg	*/
if ( (marb->srvc_state == 0x00)		   ||		/*Did error occur? or	*/
     (sv_cont_funct    == marb->cont_funct) )		/*Wait'g for frame delay*/
	return ;					/*If yes		*/

/*Set up & initialize LCR							*/
marb->cont_funct = mpa_srvc_init_modem_4 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*End of INIT MODEM BOARD - 3rd FUNCTION, mpa_srvc_init_modem_3		*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *    INITIALIZE MODEM BOARD - 4th FUNCTION, INITIALIZE UART'S LCR REGISTER    *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_init_modem_4 ( unsigned long		        caller_parm,
				     struct marb_struct FARm *  	marb,
	      			     struct ssp_struct FARm *   	sspcb,
	      			     struct slot_struct FARm *  	slot,
	      			     struct icp_global_struct FARm * 	gregs,
				     struct cin_struct FARm *	  	iregs,
	      			     struct cout_struct FARm *	  	oregs
				   )
#else
static void  mpa_srvc_init_modem_4 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs )

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

#ifndef				RAMP_LINUX		/*R14*/
void		(FARm *sv_cont_funct)() ;		/*Saved addr of Register*/
							/*Modem request's "link"*/
							/*function		*/
#else
void            (FARm *sv_cont_funct)(
				      unsigned long int        caller_parm,
				      struct marb_struct       FARm * ,
				      struct ssp_struct        FARm * ,
				      struct slot_struct       FARm * ,
				      struct mpa_global_s      FARm * ,
				      struct mpa_input_s       FARm * ,
				      struct mpa_output_s      FARm * 
				     ) ;
#endif


/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;

/*Start of function's logic							*/
sv_cont_funct = marb->cont_funct ;			/*Save addr of "link" fn*/
mpa_srvc_reg_modem_6 ( (unsigned long)MARB_ACTV_SRVC_INIT_MODEM,
                       marb,  sspcb, slot,		/*Go init LCR		*/
		       gregs, iregs, oregs ) ;
if ( (marb->srvc_state == 0x00)		   ||		/*Did error occur? or	*/
     (sv_cont_funct    == marb->cont_funct) )		/*Wait'g for frame delay*/
	return ;					/*If yes		*/

/*Set up & wait till LCR Char sent then address DLL				*/
marb->cont_funct = mpa_srvc_init_modem_5 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*End of INIT MODEM BOARD - 4th FUNCTION, mpa_srvc_init_modem_4		*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *    INITIALIZE MODEM BOARD - 5th FUNCTION, ADDRESS UART's DIVISOR LATCH      *
 *			REGISTER (Least Significant byte) DLL		       *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_init_modem_5 ( unsigned long		        caller_parm,
				     struct marb_struct FARm *  	marb,
	      			     struct ssp_struct FARm *   	sspcb,
	      			     struct slot_struct FARm *  	slot,
	      			     struct icp_global_struct FARm * 	gregs,
				     struct cin_struct FARm *	  	iregs,
	      			     struct cout_struct FARm *	  	oregs
				   )
#else
static void  mpa_srvc_init_modem_5 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs )

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

#ifndef				RAMP_LINUX		/*R14*/
void		(FARm *sv_cont_funct)() ;		/*Saved addr of Register*/
							/*Modem request's "link"*/
							/*function		*/
#else
void            (FARm *sv_cont_funct)(
				      unsigned long int        caller_parm,
				      struct marb_struct       FARm * ,
				      struct ssp_struct        FARm * ,
				      struct slot_struct       FARm * ,
				      struct mpa_global_s      FARm * ,
				      struct mpa_input_s       FARm * ,
				      struct mpa_output_s      FARm * 
				     ) ;
#endif


/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;

/*Start of function's logic							*/
sv_cont_funct = marb->cont_funct ;			/*Save addr of "link" fn*/
mpa_srvc_reg_modem_7 ( (unsigned long)MARB_ACTV_SRVC_INIT_MODEM,
                       marb,  sspcb, slot,		/*Go set control sig's	*/
		       gregs, iregs, oregs ) ;		/*to addr DLL reg	*/
if ( (marb->srvc_state == 0x00)		   ||		/*Did error occur? or	*/
     (sv_cont_funct    == marb->cont_funct) )		/*Wait'g for frame delay*/
	return ;					/*If yes		*/

/*Set up & initialize DLL							*/
marb->cont_funct = mpa_srvc_init_modem_6 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*End of INIT MODEM BOARD - 5th FUNCTION, mpa_srvc_init_modem_5		*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *    INITIALIZE MODEM BOARD - 6th FUNCTION, INITIALIZE UART'S DLL REGISTER    *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_init_modem_6 ( unsigned long		        caller_parm,
				     struct marb_struct FARm *  	marb,
	      			     struct ssp_struct FARm *   	sspcb,
	      			     struct slot_struct FARm *  	slot,
	      			     struct icp_global_struct FARm * 	gregs,
				     struct cin_struct FARm *	  	iregs,
	      			     struct cout_struct FARm *	  	oregs
				   )
#else
static void  mpa_srvc_init_modem_6 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs )

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

#ifndef				RAMP_LINUX		/*R14*/
void		(FARm *sv_cont_funct)() ;		/*Saved addr of Register*/
							/*Modem request's "link"*/
							/*function		*/
#else
void            (FARm *sv_cont_funct)(
				      unsigned long int        caller_parm,
				      struct marb_struct       FARm * ,
				      struct ssp_struct        FARm * ,
				      struct slot_struct       FARm * ,
				      struct mpa_global_s      FARm * ,
				      struct mpa_input_s       FARm * ,
				      struct mpa_output_s      FARm * 
				     ) ;
#endif


/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;

/*Start of function's logic							*/
sv_cont_funct = marb->cont_funct ;			/*Save addr of "link" fn*/
mpa_srvc_reg_modem_8 ( (unsigned long)MARB_ACTV_SRVC_INIT_MODEM,
                       marb,  sspcb, slot,		/*Go init DLL		*/
		       gregs, iregs, oregs ) ;
if ( (marb->srvc_state == 0x00)		   ||		/*Did error occur? or	*/
     (sv_cont_funct    == marb->cont_funct) )		/*Wait'g for frame delay*/
	return ;					/*If yes		*/

/*Set up & wait till DLL Char sent then address DLM				*/
marb->cont_funct = mpa_srvc_init_modem_7 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*End of INIT MODEM BOARD - 6th FUNCTION, mpa_srvc_init_modem_6		*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *    INITIALIZE MODEM BOARD - 7th FUNCTION, ADDRESS UART's DIVISOR LATCH      *
 *			REGISTER (Most Significant byte) DLM		       *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_init_modem_7 ( unsigned long		        caller_parm,
				     struct marb_struct FARm *  	marb,
	      			     struct ssp_struct FARm *   	sspcb,
	      			     struct slot_struct FARm *  	slot,
	      			     struct icp_global_struct FARm * 	gregs,
				     struct cin_struct FARm *	  	iregs,
	      			     struct cout_struct FARm *	  	oregs
				   )
#else
static void  mpa_srvc_init_modem_7 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs )

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

#ifndef				RAMP_LINUX		/*R14*/
void		(FARm *sv_cont_funct)() ;		/*Saved addr of Register*/
							/*Modem request's "link"*/
							/*function		*/
#else
void            (FARm *sv_cont_funct)(
				      unsigned long int        caller_parm,
				      struct marb_struct       FARm * ,
				      struct ssp_struct        FARm * ,
				      struct slot_struct       FARm * ,
				      struct mpa_global_s      FARm * ,
				      struct mpa_input_s       FARm * ,
				      struct mpa_output_s      FARm * 
				     ) ;
#endif


/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;

/*Start of function's logic							*/
sv_cont_funct = marb->cont_funct ;			/*Save addr of "link" fn*/
mpa_srvc_reg_modem_9 ( (unsigned long)MARB_ACTV_SRVC_INIT_MODEM,
                       marb,  sspcb, slot,		/*Go set control sig's	*/
		       gregs, iregs, oregs ) ;		/*to addr DLM reg	*/
if ( (marb->srvc_state == 0x00)		   ||		/*Did error occur? or	*/
     (sv_cont_funct    == marb->cont_funct) )		/*Wait'g for frame delay*/
	return ;					/*If yes		*/

/*Set up & initialize DLM							*/
marb->cont_funct = mpa_srvc_init_modem_8 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*End of INIT MODEM BOARD - 7th FUNCTION, mpa_srvc_init_modem_7		*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *    INITIALIZE MODEM BOARD - 8th FUNCTION, INITIALIZE UART'S DLM REGISTER    *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_init_modem_8 ( unsigned long		        caller_parm,
				     struct marb_struct FARm *  	marb,
	      			     struct ssp_struct FARm *   	sspcb,
	      			     struct slot_struct FARm *  	slot,
	      			     struct icp_global_struct FARm * 	gregs,
				     struct cin_struct FARm *	  	iregs,
	      			     struct cout_struct FARm *	  	oregs
				   )
#else
static void  mpa_srvc_init_modem_8 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs )

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

#ifndef				RAMP_LINUX		/*R14*/
void		(FARm *sv_cont_funct)() ;		/*Saved addr of Register*/
							/*Modem request's "link"*/
							/*function		*/
#else
void            (FARm *sv_cont_funct)(
				      unsigned long int        caller_parm,
				      struct marb_struct       FARm * ,
				      struct ssp_struct        FARm * ,
				      struct slot_struct       FARm * ,
				      struct mpa_global_s      FARm * ,
				      struct mpa_input_s       FARm * ,
				      struct mpa_output_s      FARm * 
				     ) ;
#endif


/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;

/*Start of function's logic							*/
sv_cont_funct = marb->cont_funct ;			/*Save addr of "link" fn*/
mpa_srvc_reg_modem_10 ( (unsigned long)MARB_ACTV_SRVC_INIT_MODEM,
                        marb,  sspcb, slot,		/*Go init DLM		*/
		        gregs, iregs, oregs ) ;
if ( (marb->srvc_state == 0x00)		   ||		/*Did error occur? or	*/
     (sv_cont_funct    == marb->cont_funct) )		/*Wait'g for frame delay*/
	return ;					/*If yes		*/

/*Set up & wait till DLM Char sent then address LCR (to clr divisor latch ena)	*/
marb->cont_funct = mpa_srvc_init_modem_9 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*End of INIT MODEM BOARD - 8th FUNCTION, mpa_srvc_init_modem_8		*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 * 	INITIALIZE MODEM BOARD - 9th FUNCTION, ADDRESS UART'S LCR REGISTER     *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_init_modem_9 ( unsigned long		        caller_parm,
				     struct marb_struct FARm *  	marb,
	      			     struct ssp_struct FARm *   	sspcb,
	      			     struct slot_struct FARm *  	slot,
	      			     struct icp_global_struct FARm * 	gregs,
				     struct cin_struct FARm *	  	iregs,
	      			     struct cout_struct FARm *	  	oregs
				   )
#else
static void  mpa_srvc_init_modem_9 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs )

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

#ifndef				RAMP_LINUX		/*R14*/
void		(FARm *sv_cont_funct)() ;		/*Saved addr of Register*/
							/*Modem request's "link"*/
							/*function		*/
#else
void            (FARm *sv_cont_funct)(
				      unsigned long int        caller_parm,
				      struct marb_struct       FARm * ,
				      struct ssp_struct        FARm * ,
				      struct slot_struct       FARm * ,
				      struct mpa_global_s      FARm * ,
				      struct mpa_input_s       FARm * ,
				      struct mpa_output_s      FARm * 
				     ) ;
#endif


/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;

/*Start of function's logic							*/
sv_cont_funct = marb->cont_funct ;			/*Save addr of "link" fn*/
mpa_srvc_reg_modem_11 ( (unsigned long)MARB_ACTV_SRVC_INIT_MODEM,
                        marb,  sspcb, slot,		/*Go set control sig's	*/
		        gregs, iregs, oregs ) ;		/*to addr LCR reg	*/
if ( (marb->srvc_state == 0x00)		   ||		/*Did error occur? or	*/
     (sv_cont_funct    == marb->cont_funct) )		/*Wait'g for frame delay*/
	return ;					/*If yes		*/

/*Set up & initialize LCR							*/
marb->cont_funct = mpa_srvc_init_modem_10 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*End of INIT MODEM BOARD - 9th FUNCTION, mpa_srvc_init_modem_9		*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *  INITIALIZE MODEM BOARD - 10th FUNCTION, CLEAR UART'S LCR REGISTER DIVISOR  *
 *                                          LATCH ENABLE BIT		       *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_init_modem_10 ( unsigned long		        caller_parm,
				      struct marb_struct FARm *  	marb,
	      			      struct ssp_struct FARm *   	sspcb,
	      			      struct slot_struct FARm *  	slot,
	      			      struct icp_global_struct FARm * 	gregs,
				      struct cin_struct FARm *	  	iregs,
	      			      struct cout_struct FARm *	  	oregs
				    )
#else
static void  mpa_srvc_init_modem_10 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs )

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

#ifndef				RAMP_LINUX		/*R14*/
void		(FARm *sv_cont_funct)() ;		/*Saved addr of Register*/
							/*Modem request's "link"*/
							/*function		*/
#else
void            (FARm *sv_cont_funct)(
				      unsigned long int        caller_parm,
				      struct marb_struct       FARm * ,
				      struct ssp_struct        FARm * ,
				      struct slot_struct       FARm * ,
				      struct mpa_global_s      FARm * ,
				      struct mpa_input_s       FARm * ,
				      struct mpa_output_s      FARm * 
				     ) ;
#endif


/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
sspcb       = sspcb ;

/*Start of function's logic							*/
sv_cont_funct = marb->cont_funct ;			/*Save addr of "link" fn*/
mpa_srvc_reg_modem_12 ( (unsigned long)MARB_ACTV_SRVC_INIT_MODEM,
                        marb,  sspcb, slot,		/*Go init LCR		*/
		        gregs, iregs, oregs ) ;
if ( (marb->srvc_state == 0x00)		   ||		/*Did error occur? or	*/
     (sv_cont_funct    == marb->cont_funct) )		/*Wait'g for frame delay*/
	return ;					/*If yes		*/

/*Set up & wait till LCR Char sent then set slot's/chan's control signals for	*/
/*normal operation								*/
marb->cont_funct = mpa_srvc_init_modem_11 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*End of INIT MODEM BOARD - 10th FUNCTION, mpa_srvc_init_modem_10	*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *  INITIALIZE MODEM BOARD - 11th FUNCTION, SET CONTROL SIGNALS FOR NORMAL USE *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_init_modem_11 ( unsigned long		        caller_parm,
				      struct marb_struct FARm *  	marb,
	      			      struct ssp_struct FARm *   	sspcb,
	      			      struct slot_struct FARm *  	slot,
	      			      struct icp_global_struct FARm * 	gregs,
				      struct cin_struct FARm *	  	iregs,
	      			      struct cout_struct FARm *	  	oregs
				    )
#else
static void  mpa_srvc_init_modem_11 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs )

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/



/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
sspcb       = sspcb ;
iregs       = iregs ;

/*Start of function's logic							*/
if ( slot->slot_state != 0xFF )				/*Is Brd still present?	*/
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_INIT_MODEM_REMOVED ;	/*Set completion status	*/
	return ;
}
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
if ( ((oregs->cout_flow_config & 0x10) != (oregs->cout_intnl_flow_ctrl & 0x10)) ||
     ((oregs->cout_intnl_flow_ctrl & 0x08) == 0x08) )	/*LCR char xmit done	*/
{							/*If no, check deadman	*/
	if ( frame_delay_chk (DEADMAN_DELAY, gregs, slot) == 0x00 )
	{						/*If deadman not expired*/
		return ;
	}
	(*slot->clean_up)(slot, (unsigned char)0xFF) ;
	marb->srvc_status = ERR_INIT_MODEM_DEADMAN ; 	/*Set cmpltn status	*/
	marb->srvc_state = 0x00 ;			/*Set "complete" state	*/
	return ;
}

/*Modem's LCR written to, determine if Slot already openned.  If not openned,	*/
/*set up & place MPA into Reset state thus flushing internal input/output	*/
/*buffers, else complete request by setting control signals			*/
if ( slot->init_state != 0xFF )				/*Is Slot closed?	*/
{							/*If yes, init'g closed	*/
							/*slot, go issue reset	*/
	oregs->cout_cntrl_sig = 0x188 ;			/*MPA's Slot Reset	*/
	slot->read_toggle     = 0x00 ;			/*UART I/O Read toggle	*/
	slot->frame_ctr  = gregs->gicp_frame_ctr ;	/*Snap shot for timing	*/
	marb->cont_funct = mpa_srvc_init_modem_12 ;	/*Pt to reset cmplt fn	*/
	return ;					/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/
}

/*Slot already openned (do not issue reset), set Slot's control signals for 	*/
/*normal operation								*/
oregs->cout_cntrl_sig = (slot->sv_cout_cntrl_sig & 0x02BB) | 0x0004 ;
slot->frame_ctr = gregs->gicp_frame_ctr ;		/*Snap shot for timing	*/
marb->cont_funct = mpa_srvc_init_modem_13 ;		/*Point to req cmplt fn	*/
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/
}	/*End of INIT MODEM BOARD - 11th FUNCTION, mpa_srvc_init_modem_11	*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *  	       INITIALIZE MODEM BOARD - 12th FUNCTION, RESET COMPLETE	       *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_init_modem_12 ( unsigned long		        caller_parm,
				     struct marb_struct FARm *  	marb,
	      			     struct ssp_struct FARm *   	sspcb,
	      			     struct slot_struct FARm *  	slot,
	      			     struct icp_global_struct FARm * 	gregs,
				     struct cin_struct FARm *	  	iregs,
	      			     struct cout_struct FARm *	  	oregs
				   )
#else
static void  mpa_srvc_init_modem_12 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs )

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
sspcb       = sspcb ;
iregs       = iregs ;

/*Start of function's logic							*/
if ( slot->slot_state != 0xFF )				/*Is Brd still present?	*/
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_INIT_MODEM_REMOVED ;	/*Set completion status	*/
	return ;
}
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
if ( frame_delay_chk ( (unsigned short int)0x08, gregs, slot ) == 0 )
	return ;					/*8 frames not elapsed	*/

/*Have delayed long enough for MPA to "see" control signal setting indicating	*/
/*that Slot should be reset, set Slot's control signals for normal operation	*/
oregs->cout_cntrl_sig = (slot->sv_cout_cntrl_sig & 0x02BB) | 0x0004 ;
slot->frame_ctr = gregs->gicp_frame_ctr ;		/*Snap shot for timing	*/
marb->cont_funct = mpa_srvc_init_modem_13 ;		/*Point to req cmplt fn	*/
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*End of INIT MODEM BOARD - 12th Function, mpa_srvc_reg_modem_12	*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *  	       INITIALIZE MODEM BOARD - 13th FUNCTION, REQUEST COMPLETE	       *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_init_modem_13 ( unsigned long		        caller_parm,
				      struct marb_struct FARm *  	marb,
	      			      struct ssp_struct FARm *   	sspcb,
	      			      struct slot_struct FARm *  	slot,
	      			      struct icp_global_struct FARm * 	gregs,
				      struct cin_struct FARm *	  	iregs,
	      			      struct cout_struct FARm *	  	oregs
				    )
#else
static void  mpa_srvc_init_modem_13 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs )

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/



/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
sspcb       = sspcb ;
iregs       = iregs ;
oregs       = oregs ;

/*Start of function's logic							*/
if ( slot->slot_state != 0xFF )				/*Is Brd still present?	*/
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_INIT_MODEM_REMOVED ;	/*Set completion status	*/
	return ;
}
if ( frame_delay_chk ( (unsigned short int)0x04, gregs, slot ) == 0 )
{
	marb->srvc_state = 0xFF ;			/*Set "call back" state	*/
	return ;					/*4 frames not elapsed	*/
}

/*Have delayed long enough for MPA to "see" control signals, complete request	*/
slot->init_state = 0xFF ;				/*MODEM/UART Openned	*/
mpa_srvc_init_modem_clean_up (slot, (unsigned char)0x00 ) ; /*Restore SSP regs	*/
marb->srvc_status = 0x00 ;				/*Set completion status	*/
return ;						/*Exit, Request Complete*/

}	/*End of INIT MODEM BOARD - 13th FUNCTION, mpa_srvc_init_modem_13	*/








/*#stitle SLOT SERVICE - INITIALIZE MODEM BOARD, LOCAL FUNCTION#*/
/*#page*/
/****************************************************************************************
 *                                                                             		*
 * INITIALIZE MODEM BOARD, LOCAL FUNCTION - Clean Up SSP (mpa_srvc_init_modem_clean_up)	*
 *                                                                     			*
 * PURPOSE: To restore state of SSP registers when service has been completed (either 	*
 *	   due to error or successful completion					*
 *											*
 * NOTES:										*
 *                                                                     			*
 * CALL: void  mpa_srvc_init_modem_clean_up  (struct slot_struct FARm  * slot,		*
 *					      unsigned char		 type)		*
 *                                                                     			*
 *			where:	slot = Slot Control Block pointer			*
 *				type = Clean up type:					*
 *					o 0x00 = Normal i.e. Reg Modem request cmplt	*
 *					o 0xFF = Abnormal i.e. Reg Modem request	*
 *						 aborted due to some failure or Board	*
 *						 removed				*
 *                                                                     			*
 *                                                                     			*
 * RETURN: None										*
 *                                                                     			*
 ****************************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void	mpa_srvc_init_modem_clean_up (struct slot_struct FARm *  slot,
					      unsigned char		 type)
#else
static void	mpa_srvc_init_modem_clean_up (slot, type)

struct slot_struct FARm *  slot;
unsigned char		   type;
#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

/*SSP variables relating to MARB's/Request's specific SLOT (Channel)		*/
#ifndef					RAMP_BSDI		/*R14*/
struct cout_struct FARm *	  	oregs ;		/*Slot's SSP Output regs	   */
#else								/*BSDI Compile		   */
volatile struct cout_struct FARm *	oregs ;		/*Slot's SSP Output regs	   */
#endif								/*End RAMP_BSDI Conditional*/



/************************
 *   START OF CODE	*
 ************************/

oregs = slot->output ;					/*Slot's output regs ptr*/

/*Delay if actually canceling/aborting an outstanding Register Modem Board MARB	*/
/*that is in process of Restting Slot. This allows time for MPA to handle Reset	*/
if ((type == 0xFF) 					&&
    (slot->actv_marb->srvc_status == MARB_OUT_CANCELED) &&
    (oregs->cout_cntrl_sig == 0x188)  			)
     /*If abort'g MARB, delay to give MPA chance to reset			*/
	while ( frame_delay_chk ( (unsigned short int)0x08, 	
				  slot->actv_marb->sspcb->global, slot)	== 0 )
	{ }

/*Restore saved SSP registers							*/
oregs->cout_flow_config = (slot->sv_cout_flow_config & 0x2F) |  /*Rstr out sigs */
                          (oregs->cout_flow_config & 0x10) | 0x40 ;
oregs->cout_cntrl_sig   = (slot->sv_cout_cntrl_sig & 0x02BB) | 0x0004 ;
oregs->cout_xon_1	= slot->sv_cout_xon_1 ;		/*Restore XON char	*/
#ifndef 				NO_FN_PROTO_PARMS /*Compile if Protypes OK*/
slot->clean_up          = (void (*)(struct slot_struct FARm *, unsigned char))0 ;
#else							  /*Compile if NO Protypes*/
slot->clean_up          = (void (*)())0 ;
#endif
return ;

}					/*End of mpa_srvc_init_modem_clean_up	*/








/*#stitle SLOT SERVICE - MODIFY UART/MODEM SETTINGS (mpa_srvc_modify_settings)#*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *              MODIFY UART/MODEM SETTINGS - SERVICE ENTRY POINT               *
 *                                                                             *
 *******************************************************************************/
/*R6*/
#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
extern unsigned short int	mpa_srvc_modify_settings (struct marb_struct FARm  *marb)
#else
extern unsigned short int	mpa_srvc_modify_settings (marb)

struct marb_struct FARm  *marb;
#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

unsigned char		status ;			/*Funct. ret status	*/



/************************
 *   START OF CODE	*
 ************************/

status = val_marb (marb) ;				/*Validate caller's MARB*/
if ( status != 0 )					/*Is MARB OK to use?	*/
	return (ERR_MOD_MODEM_MARB) ;			/*If no, return error	*/
if ( marb->in_use == 0 )				/*Initial MARB Request?	*/
{							/*If yes, init Fn ptr	*/
	marb->cont_funct = mpa_srvc_mod_sets_1 ;	/*Init MARB's FN ptr	*/
}
else							/*"Call Back" MARB,	*/
{							/*Validate SRVC active	*/
	if ( marb->actv_srvc != MARB_ACTV_SRVC_MOD_SETS) /*Valid "Call Back"?	*/
	{						 /*If no, exit		*/
		marb->srvc_status = ERR_MOD_MODEM_CALL_BK ;
		marb->srvc_state  = 0x00 ;
		return ((unsigned short int)(((unsigned short int)(marb->srvc_state)<<8)+
                                               marb->srvc_status) ) ;
	}
}
return ( async_req_hndlr(marb) ) ;			/*Go handle request	*/


}			/*END of mpa_srvc_modify_settings Service Entry function*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *     MODIFY UART/MODEM SETTINGS - 1st FUNCTION, CHECK IF ANY SETTINGS CHANGE *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_mod_sets_1  ( unsigned long			caller_parm,
				    struct marb_struct FARm *  		marb,
	      			    struct ssp_struct FARm *   		sspcb,
	      			    struct slot_struct FARm *  		slot,
	      			    struct icp_global_struct FARm * 	gregs,
				    struct cin_struct FARm *	  	iregs,
	      			    struct cout_struct FARm *	  	oregs
				  )
#else
static void  mpa_srvc_mod_sets_1 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif
{

/************************
 *     LOCAL DATA	*
 ************************/

unsigned int		i ;				/*Loop counter		*/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
sspcb       = sspcb ;
iregs       = iregs ;
oregs       = oregs ;

/*Start of function's logic							*/
if ( slot->req_outstnd != 0x00 ) 			/*Req Outstand'g?	*/
{							/*If yes, return error	*/
	marb->srvc_status = ERR_MOD_MODEM_REQ_OUT ;	/*Indicate error	*/
	return ;					/*Exit			*/
}
if ( slot->slot_state != 0xFF )				/*Is Modem registered?	*/
{							/*If no, error		*/
	marb->srvc_status = ERR_MOD_MODEM_NOT_REG ;	/*Indicate error	*/
	return ;					/*Exit			*/
}
if ( slot->init_state != 0xFF )				/*Is Slot init'd/opn'd?	*/
{							/*If no, error		*/
	marb->srvc_status = ERR_MOD_MODEM_NOPN ;	/*Indicate error	*/
	return ;					/*Exit			*/
}

/*Caller's request valid, set up to modify UART's setting			*/
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
marb->actv_srvc  = MARB_ACTV_SRVC_MOD_SETS ;		/*Indicate outstnd'g req*/

/*Save SSP registers that will be used to modify UART's settings		*/
slot->sv_cout_flow_config = oregs->cout_flow_config ;
slot->sv_cout_cntrl_sig   = oregs->cout_cntrl_sig ;
slot->sv_cout_xon_1       = oregs->cout_xon_1 ;
#ifndef 				NO_FN_PROTO_PARMS /*Compile if Protypes OK*/
slot->clean_up   = (void (*)(struct slot_struct FARm *, unsigned char))mpa_srvc_mod_sets_clean_up ;
#else							  /*Compile if NO Protypes*/
slot->clean_up   = (void (*)())mpa_srvc_mod_sets_clean_up ;
#endif
oregs->cout_flow_config = (oregs->cout_flow_config & 0x10) | 0x40 | 0x08 ;

/*Check if should update caller's Open Scratch Area in SLOTCB			*/
if ( marb->scratch_area != (unsigned char FARm *)0 ) 	/*Is scratch area spec'd*/
	for ( i = 0; i < SLOT_SCRATCH_SZ; ++i )	     	/*If yes, copy area	*/
		slot->oscratch[i] = marb->scratch_area[i] ;

/*Set up and determine if Flow Control Settings are changing			*/
if ( marb->sup_uchar0 != slot->flow_setting )		/*Is Flow Cntrl chng'g?	*/
{							/*If yes		*/
	slot->flow_setting = marb->sup_uchar0 ;		/*Save new flow setting	*/
	oregs->cout_cntrl_sig = 0x1AA ;			/*Send control sigs to	*/
							/*MPA indicat'g to direct*/
							/*output to AUX reg,MPA-*/
							/*to-UART Flow Cntrl	*/
	slot->frame_ctr  = gregs->gicp_frame_ctr ;	/*Snap shot for timing	*/
	marb->cont_funct = mpa_srvc_mod_sets_2 ;	/*Flow Cntrl Function	*/
	return ;					/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/
}

/*Set up and determine if Baud Rate setting is changing				*/
if ( marb->sup_ushort != slot->baud )			/*Is Baud Rate chng'g?	*/
{							/*If yes		*/
	slot->baud      = marb->sup_ushort ;		/*Save new Baud Rate	*/
	slot->num_bits	= marb->sup_uchar1 ;		/*Assure LCR bits as per*/
	slot->stop_bits	= marb->sup_uchar2 ;		/* Request since LCR is	*/
	slot->parity   	= marb->sup_uchar3 ;		/*  Writtn to when chnge*/
							/*   Baud Rate		*/
	oregs->cout_cntrl_sig = 0x133 ;			/*Addr UART's LCR	*/
	slot->frame_ctr  = gregs->gicp_frame_ctr ;	/*Snap shot for timing	*/
	marb->cont_funct = mpa_srvc_mod_sets_4 ;
	return ;					/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/
}

/*Set up and determine if Line Control Register settings are changing		*/
if ( (marb->sup_uchar1 != slot->num_bits)  ||		/*Is char size chng'g?	*/
     (marb->sup_uchar2 != slot->stop_bits) ||		/*Is # stop bits chng'g?*/
     (marb->sup_uchar3 != slot->parity)	   )		/*Is parity changing?	*/
{							/*If yes		*/
	slot->num_bits	= marb->sup_uchar1 ;		/*Save # bits setting	*/
	slot->stop_bits	= marb->sup_uchar2 ;		/*Save # stop bits	*/
	slot->parity   	= marb->sup_uchar3 ;		/*Save parity setting	*/
	oregs->cout_cntrl_sig = 0x133 ;			/*Addr UART's LCR	*/
	slot->frame_ctr       = gregs->gicp_frame_ctr;	/*Snap shot for timing	*/
	marb->cont_funct = mpa_srvc_mod_sets_10 ;
	return ;					/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/
}

/*No Modem Settings are being changed, return to caller with successful		*/
/*completion status								*/
mpa_srvc_mod_sets_clean_up (slot, (unsigned char)0x00 ) ; /*Restore SSP regs	*/
marb->srvc_state  = 0x00 ;				/*Set Srvc State = Done	*/
marb->srvc_status = 0x00 ;				/*Set completion status	*/
marb->actv_srvc   = 0x00 ;				/*Set no req outstndg	*/
return ;						/*Exit, Request Complete*/

}		/*End of MODIFY UART/MODEM SETTINGS - 1st Fn, mpa_srvc_mod_sets_1*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 * MODIFY UART/MODEM SETTINGS - 2nd FUNCTION, INIT MPA-to-UART FLOW CONTROL    *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_mod_sets_2 ( unsigned long		        caller_parm,
				   struct marb_struct FARm *  		marb,
	      			   struct ssp_struct FARm *   		sspcb,
	      			   struct slot_struct FARm *  		slot,
	      			   struct icp_global_struct FARm * 	gregs,
				   struct cin_struct FARm *	  	iregs,
	      			   struct cout_struct FARm *	  	oregs
				 )
#else
static void  mpa_srvc_mod_sets_2 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs )

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

#ifndef				RAMP_LINUX		/*R14*/
void		(FARm *sv_cont_funct)() ;		/*Saved addr of Register*/
							/*Modem request's "link"*/
							/*function		*/
#else
void            (FARm *sv_cont_funct)(
				      unsigned long int        caller_parm,
				      struct marb_struct       FARm * ,
				      struct ssp_struct        FARm * ,
				      struct slot_struct       FARm * ,
				      struct mpa_global_s      FARm * ,
				      struct mpa_input_s       FARm * ,
				      struct mpa_output_s      FARm * 
				     ) ;
#endif


/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;

/*Start of function's logic							*/
sv_cont_funct = marb->cont_funct ;			/*Save addr of "link" fn*/
mpa_srvc_reg_modem_4 ( (unsigned long)MARB_ACTV_SRVC_MOD_SETS,
                       marb,  sspcb, slot,		/*Go send requestor's 	*/
		       gregs, iregs, oregs ) ;		/*MPA-to-UART Flow set'g*/
if ( (marb->srvc_state == 0x00)		   ||		/*Did error occur? or	*/
     (sv_cont_funct    == marb->cont_funct) )		/*Wait'g for frame delay*/
	return ;					/*If yes		*/

/*Set up & wait till MPA AUX Reg Flow Control setting (character) sent then 	*/
/*address LCR									*/
marb->cont_funct = mpa_srvc_mod_sets_3 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*End of MODIFY UART/MODEM SETTINGS - 2nd FUNCTION, mpa_srvc_mod_sets_2	*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 * MODIFY UART/MODEM SETTINGS - 3rd FUNCTION, CHK FOR BAUD RATE or LCR CHANGES *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_mod_sets_3 ( unsigned long		        caller_parm,
				   struct marb_struct FARm *  		marb,
	      			   struct ssp_struct FARm *   		sspcb,
	      			   struct slot_struct FARm *  		slot,
	      			   struct icp_global_struct FARm * 	gregs,
				   struct cin_struct FARm *	  	iregs,
	      			   struct cout_struct FARm *	  	oregs
				 )
#else
static void  mpa_srvc_mod_sets_3 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

#ifndef				RAMP_LINUX		/*R14*/
void		(FARm *sv_cont_funct)() ;		/*Saved addr of Register*/
							/*Modem request's "link"*/
							/*function		*/
#else
void            (FARm *sv_cont_funct)(
				      unsigned long int        caller_parm,
				      struct marb_struct       FARm * ,
				      struct ssp_struct        FARm * ,
				      struct slot_struct       FARm * ,
				      struct mpa_global_s      FARm * ,
				      struct mpa_input_s       FARm * ,
				      struct mpa_output_s      FARm * 
				     ) ;
#endif


/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;

/*Start of function's logic							*/
sv_cont_funct = marb->cont_funct ;			/*Save addr of "link" fn*/
mpa_srvc_reg_modem_5 ( (unsigned long)MARB_ACTV_SRVC_MOD_SETS,
                       marb,  sspcb, slot,		/*Go set control sig's	*/
		       gregs, iregs, oregs ) ;		/*to addr LCR reg	*/
if ( (marb->srvc_state == 0x00)		   ||		/*Did error occur? or	*/
     (sv_cont_funct    == marb->cont_funct) )		/*Wait'g for frame delay*/
	return ;					/*If yes		*/

/*MPA-to-UART Flow Control updated, set up & determine if Baud Rate and/or LCR	*/
/*(char size, parity, stop bits) is changing					*/
if ( marb->sup_ushort != slot->baud )			/*Is Baud Rate chng'g?	*/
{							/*If yes		*/
	slot->baud      = marb->sup_ushort ;		/*Save new Baud Rate	*/
	slot->num_bits	= marb->sup_uchar1 ;		/*Assure LCR bits as per*/
	slot->stop_bits	= marb->sup_uchar2 ;		/* Request since LCR is	*/
	slot->parity   	= marb->sup_uchar3 ;		/*  Writtn to when chnge*/
							/*   Baud Rate		*/
	marb->cont_funct = mpa_srvc_mod_sets_4 ;	/*Set up to wrt to LCR	*/
							/*so can change Divisors*/
	return ;					/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/
}
if ( (marb->sup_uchar1 != slot->num_bits)  ||		/*Is char size chng'g?	*/
     (marb->sup_uchar2 != slot->stop_bits) ||		/*Is # stop bits chng'g?*/
     (marb->sup_uchar3 != slot->parity)	   )		/*Is parity changing?	*/
{							/*If yes		*/
	slot->num_bits	 = marb->sup_uchar1 ;		/*Save # bits setting	*/
	slot->stop_bits	 = marb->sup_uchar2 ;		/*Save # stop bits	*/
	slot->parity   	 = marb->sup_uchar3 ;		/*Save parity setting	*/
	marb->cont_funct = mpa_srvc_mod_sets_10 ;	/*Set up to wrt to LCR	*/
	return ;					/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/
}

/*Neither Baud Rate or LCR (char size, parity, stop bits) is being changed, set	*/
/*up to complete service/function by restoring Control Signals			*/
oregs->cout_cntrl_sig = (slot->sv_cout_cntrl_sig & 0x02BB) | 0x0004 ;
slot->frame_ctr = gregs->gicp_frame_ctr ;		/*Snap shot for timing	*/
marb->cont_funct = mpa_srvc_mod_sets_12 ;		/*Point to req cmplt fn	*/
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*End of MODIFY UART/MODEM SETTINGS - 3rd FUNCTION, mpa_srvc_mod_sets_3	*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 * MODIFY UART/MODEM SETTINGS - 4th FUNCTION, INITIALIZE UART'S LCR REGISTER SO*
 *					      CAN CHANGE BAUD RATE DIVISORS    *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_mod_sets_4 ( unsigned long			caller_parm,
				   struct marb_struct FARm *  		marb,
	      			   struct ssp_struct FARm *   		sspcb,
	      			   struct slot_struct FARm *  		slot,
	      			   struct icp_global_struct FARm * 	gregs,
				   struct cin_struct FARm *	  	iregs,
	      			   struct cout_struct FARm *	  	oregs
			         )
#else
static void  mpa_srvc_mod_sets_4 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long			caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

#ifndef				RAMP_LINUX		/*R14*/
void		(FARm *sv_cont_funct)() ;		/*Saved addr of Register*/
							/*Modem request's "link"*/
							/*function		*/
#else
void            (FARm *sv_cont_funct)(
				      unsigned long int        caller_parm,
				      struct marb_struct       FARm * ,
				      struct ssp_struct        FARm * ,
				      struct slot_struct       FARm * ,
				      struct mpa_global_s      FARm * ,
				      struct mpa_input_s       FARm * ,
				      struct mpa_output_s      FARm * 
				     ) ;
#endif


/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;

/*Start of function's logic							*/
sv_cont_funct = marb->cont_funct ;			/*Save addr of "link" fn*/
mpa_srvc_reg_modem_6 ( (unsigned long)MARB_ACTV_SRVC_MOD_SETS,
                       marb,  sspcb, slot,		/*Go init LCR		*/
		       gregs, iregs, oregs ) ;
if ( (marb->srvc_state == 0x00)		   ||		/*Did error occur? or	*/
     (sv_cont_funct    == marb->cont_funct) )		/*Wait'g for frame delay*/
	return ;					/*If yes		*/

/*Set up & wait till LCR Char sent then address DLL				*/
marb->cont_funct = mpa_srvc_mod_sets_5 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*End of MODIFY UART/MODEM SETTINGS - 4th FUNCTION, mpa_srvc_mod_sets_4	*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 * MODIFY UART/MODEM SETTINGS - 5th FUNCTION, ADDRESS UART's DIVISOR LATCH     *
 *			                      REG (Least Significant byte) DLL *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_mod_sets_5 ( unsigned long			caller_parm,
				   struct marb_struct FARm *  		marb,
	      			   struct ssp_struct FARm *   		sspcb,
	      			   struct slot_struct FARm *  		slot,
	      			   struct icp_global_struct FARm * 	gregs,
				   struct cin_struct FARm *	  	iregs,
	      			   struct cout_struct FARm *	  	oregs
				 )
#else
static void  mpa_srvc_mod_sets_5 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long			caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

#ifndef				RAMP_LINUX		/*R14*/
void		(FARm *sv_cont_funct)() ;		/*Saved addr of Register*/
							/*Modem request's "link"*/
							/*function		*/
#else
void            (FARm *sv_cont_funct)(
				      unsigned long int        caller_parm,
				      struct marb_struct       FARm * ,
				      struct ssp_struct        FARm * ,
				      struct slot_struct       FARm * ,
				      struct mpa_global_s      FARm * ,
				      struct mpa_input_s       FARm * ,
				      struct mpa_output_s      FARm * 
				     ) ;
#endif


/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;

/*Start of function's logic							*/
sv_cont_funct = marb->cont_funct ;			/*Save addr of "link" fn*/
mpa_srvc_reg_modem_7 ( (unsigned long)MARB_ACTV_SRVC_MOD_SETS,
                       marb,  sspcb, slot,		/*Go set control sig's	*/
		       gregs, iregs, oregs ) ;		/*to addr DLL reg	*/
if ( (marb->srvc_state == 0x00)		   ||		/*Did error occur? or	*/
     (sv_cont_funct    == marb->cont_funct) )		/*Wait'g for frame delay*/
	return ;					/*If yes		*/

/*Set up & initialize DLL							*/
marb->cont_funct = mpa_srvc_mod_sets_6 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*End of MODIFY UART/MODEM SETTINGS - 5th FUNCTION, mpa_srvc_mod_sets_5	*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 * MODIFY UART/MODEM SETTINGS - 6th FUNCTION, INITIALIZE UART'S DLL REGISTER   *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_mod_sets_6 ( unsigned long			caller_parm,
				   struct marb_struct FARm *  		marb,
	      			   struct ssp_struct FARm *   		sspcb,
	      			   struct slot_struct FARm *  		slot,
	      			   struct icp_global_struct FARm * 	gregs,
				   struct cin_struct FARm *	  	iregs,
	      			   struct cout_struct FARm *	  	oregs
				 )
#else
static void  mpa_srvc_mod_sets_6 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long			caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

#ifndef				RAMP_LINUX		/*R14*/
void		(FARm *sv_cont_funct)() ;		/*Saved addr of Register*/
							/*Modem request's "link"*/
							/*function		*/
#else
void            (FARm *sv_cont_funct)(
				      unsigned long int        caller_parm,
				      struct marb_struct       FARm * ,
				      struct ssp_struct        FARm * ,
				      struct slot_struct       FARm * ,
				      struct mpa_global_s      FARm * ,
				      struct mpa_input_s       FARm * ,
				      struct mpa_output_s      FARm * 
				     ) ;
#endif


/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;

/*Start of function's logic							*/
sv_cont_funct = marb->cont_funct ;			/*Save addr of "link" fn*/
mpa_srvc_reg_modem_8 ( (unsigned long)MARB_ACTV_SRVC_MOD_SETS,
                       marb,  sspcb, slot,		/*Go init DLL		*/
		       gregs, iregs, oregs ) ;
if ( (marb->srvc_state == 0x00)		   ||		/*Did error occur? or	*/
     (sv_cont_funct    == marb->cont_funct) )		/*Wait'g for frame delay*/
	return ;					/*If yes		*/

/*Set up & wait till DLL Char sent then address DLM				*/
marb->cont_funct = mpa_srvc_mod_sets_7 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*End of MODIFY UART/MODEM SETTINGS - 6th FUNCTION, mpa_srvc_mod_sets_6	*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 * MODIFY UART/MODEM SETTINGS - 7th FUNCTION, ADDRESS UART's DIVISOR LATCH     *
 *			                      REG (Most Significant byte) DLM  *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_mod_sets_7 ( unsigned long			caller_parm,
				   struct marb_struct FARm *  		marb,
	      			   struct ssp_struct FARm *   		sspcb,
	      			   struct slot_struct FARm *  		slot,
	      			   struct icp_global_struct FARm * 	gregs,
				   struct cin_struct FARm *	  	iregs,
	      			   struct cout_struct FARm *	  	oregs
				 )
#else
static void  mpa_srvc_mod_sets_7 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long			caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

#ifndef				RAMP_LINUX		/*R14*/
void		(FARm *sv_cont_funct)() ;		/*Saved addr of Register*/
							/*Modem request's "link"*/
							/*function		*/
#else
void            (FARm *sv_cont_funct)(
				      unsigned long int        caller_parm,
				      struct marb_struct       FARm * ,
				      struct ssp_struct        FARm * ,
				      struct slot_struct       FARm * ,
				      struct mpa_global_s      FARm * ,
				      struct mpa_input_s       FARm * ,
				      struct mpa_output_s      FARm * 
				     ) ;
#endif


/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;

/*Start of function's logic							*/
sv_cont_funct = marb->cont_funct ;			/*Save addr of "link" fn*/
mpa_srvc_reg_modem_9 ( (unsigned long)MARB_ACTV_SRVC_MOD_SETS,
                       marb,  sspcb, slot,		/*Go set control sig's	*/
		       gregs, iregs, oregs ) ;		/*to addr DLM reg	*/
if ( (marb->srvc_state == 0x00)		   ||		/*Did error occur? or	*/
     (sv_cont_funct    == marb->cont_funct) )		/*Wait'g for frame delay*/
	return ;					/*If yes		*/

/*Set up & initialize DLM							*/
marb->cont_funct = mpa_srvc_mod_sets_8 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*End of MODIFY UART/MODEM SETTINGS - 7th FUNCTION, mpa_srvc_mod_sets_7 */
/*#page*/
/*******************************************************************************
 *                                                                             *
 * MODIFY UART/MODEM SETTINGS - 8th FUNCTION, INITIALIZE UART'S DLM REGISTER   *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_mod_sets_8 ( unsigned long			caller_parm,
				   struct marb_struct FARm *  		marb,
	      			   struct ssp_struct FARm *   		sspcb,
	      			   struct slot_struct FARm *  		slot,
	      			   struct icp_global_struct FARm * 	gregs,
				   struct cin_struct FARm *	  	iregs,
	      			   struct cout_struct FARm *	  	oregs
				 )
#else
static void  mpa_srvc_mod_sets_8 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long			caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

#ifndef				RAMP_LINUX		/*R14*/
void		(FARm *sv_cont_funct)() ;		/*Saved addr of Register*/
							/*Modem request's "link"*/
							/*function		*/
#else
void            (FARm *sv_cont_funct)(
				      unsigned long int        caller_parm,
				      struct marb_struct       FARm * ,
				      struct ssp_struct        FARm * ,
				      struct slot_struct       FARm * ,
				      struct mpa_global_s      FARm * ,
				      struct mpa_input_s       FARm * ,
				      struct mpa_output_s      FARm * 
				     ) ;
#endif


/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;

/*Start of function's logic							*/
sv_cont_funct = marb->cont_funct ;			/*Save addr of "link" fn*/
mpa_srvc_reg_modem_10 ( (unsigned long)MARB_ACTV_SRVC_MOD_SETS,
                        marb,  sspcb, slot,		/*Go init DLM		*/
		        gregs, iregs, oregs ) ;
if ( (marb->srvc_state == 0x00)		   ||		/*Did error occur? or	*/
     (sv_cont_funct    == marb->cont_funct) )		/*Wait'g for frame delay*/
	return ;					/*If yes		*/

/*Set up & wait till DLM Char sent then address LCR (to clr divisor latch ena)	*/
marb->cont_funct = mpa_srvc_mod_sets_9 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*End of MODIFY UART/MODEM SETTINGS - 8th FUNCTION, mpa_srvc_mod_sets_8 */
/*#page*/
/*******************************************************************************
 *                                                                             *
 * MODIFY UART/MODEM SETTINGS - 9th FUNCTION, ADDRESS UART'S LCR REGISTER      *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_mod_sets_9 ( unsigned long		        caller_parm,
				   struct marb_struct FARm *  		marb,
	      			   struct ssp_struct FARm *   		sspcb,
	      			   struct slot_struct FARm *  		slot,
	      			   struct icp_global_struct FARm * 	gregs,
				   struct cin_struct FARm *	  	iregs,
	      			   struct cout_struct FARm *	  	oregs
				 )
#else
static void  mpa_srvc_mod_sets_9 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

#ifndef				RAMP_LINUX		/*R14*/
void		(FARm *sv_cont_funct)() ;		/*Saved addr of Register*/
							/*Modem request's "link"*/
							/*function		*/
#else
void            (FARm *sv_cont_funct)(
				      unsigned long int        caller_parm,
				      struct marb_struct       FARm * ,
				      struct ssp_struct        FARm * ,
				      struct slot_struct       FARm * ,
				      struct mpa_global_s      FARm * ,
				      struct mpa_input_s       FARm * ,
				      struct mpa_output_s      FARm * 
				     ) ;
#endif


/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;

/*Start of function's logic							*/
sv_cont_funct = marb->cont_funct ;			/*Save addr of "link" fn*/
mpa_srvc_reg_modem_11 ( (unsigned long)MARB_ACTV_SRVC_MOD_SETS,
                        marb,  sspcb, slot,		/*Go set control sig's	*/
		        gregs, iregs, oregs ) ;		/*to addr LCR reg	*/
if ( (marb->srvc_state == 0x00)		   ||		/*Did error occur? or	*/
     (sv_cont_funct    == marb->cont_funct) )		/*Wait'g for frame delay*/
	return ;					/*If yes		*/

/*Set up & initialize LCR							*/
marb->cont_funct = mpa_srvc_mod_sets_10 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*End of MODIFY UART/MODEM SETTINGS - 9th FUNCTION, mpa_srvc_mod_sets_9	*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *MODIFY UART/MODEM SETTINGS - 10th FUNCTION, CLEAR UART'S LCR REGISTER DIVISOR*
 *                                            LATCH ENABLE BIT		       *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_mod_sets_10 ( unsigned long		        caller_parm,
				    struct marb_struct FARm *  		marb,
	      			    struct ssp_struct FARm *   		sspcb,
	      			    struct slot_struct FARm *  		slot,
	      			    struct icp_global_struct FARm * 	gregs,
				    struct cin_struct FARm *	  	iregs,
	      			    struct cout_struct FARm *	  	oregs
				  )
#else
static void  mpa_srvc_mod_sets_10 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

#ifndef				RAMP_LINUX		/*R14*/
void		(FARm *sv_cont_funct)() ;		/*Saved addr of Register*/
							/*Modem request's "link"*/
							/*function		*/
#else
void            (FARm *sv_cont_funct)(
				      unsigned long int        caller_parm,
				      struct marb_struct       FARm * ,
				      struct ssp_struct        FARm * ,
				      struct slot_struct       FARm * ,
				      struct mpa_global_s      FARm * ,
				      struct mpa_input_s       FARm * ,
				      struct mpa_output_s      FARm * 
				     ) ;
#endif


/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
sspcb       = sspcb ;

/*Start of function's logic							*/
sv_cont_funct = marb->cont_funct ;			/*Save addr of "link" fn*/
mpa_srvc_reg_modem_12 ( (unsigned long)MARB_ACTV_SRVC_MOD_SETS,
                        marb,  sspcb, slot,		/*Go init LCR		*/
		        gregs, iregs, oregs ) ;
if ( (marb->srvc_state == 0x00)		   ||		/*Did error occur? or	*/
     (sv_cont_funct    == marb->cont_funct) )		/*Wait'g for frame delay*/
	return ;					/*If yes		*/

/*Set up & wait till LCR Char sent then set slot's/chan's control signals for	*/
/*normal operation								*/
marb->cont_funct = mpa_srvc_mod_sets_11 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}   /*End of MODIFY UART/MODEM SETTINGS - 10th FUNCTION, mpa_srvc_mod_sets_10	*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *MODIFY UART/MODEM SETTINGS - 11th FUNCTION, SET CONTROL SIGNALS FOR NORMAL   *
 *					      OPERATION			       *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_mod_sets_11 ( unsigned long		        caller_parm,
				    struct marb_struct FARm *  		marb,
	      			    struct ssp_struct FARm *   		sspcb,
	      			    struct slot_struct FARm *  		slot,
	      			    struct icp_global_struct FARm * 	gregs,
				    struct cin_struct FARm *	  	iregs,
	      			    struct cout_struct FARm *	  	oregs
			          )
#else
static void  mpa_srvc_mod_sets_11 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/



/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
sspcb       = sspcb ;
iregs       = iregs ;

/*Start of function's logic							*/
if ( slot->slot_state != 0xFF )				/*Is Brd still present?	*/
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_MOD_MODEM_REMOVED ;	/*Set completion status	*/
	return ;
}
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
if ( ((oregs->cout_flow_config & 0x10) != (oregs->cout_intnl_flow_ctrl & 0x10)) ||
     ((oregs->cout_intnl_flow_ctrl & 0x08) == 0x08) )	/*LCR char xmit done	*/
{							/*If no, check deadman	*/
	if ( frame_delay_chk (DEADMAN_DELAY, gregs, slot) == 0x00 )
	{						/*If deadman not expired*/
		return ;
	}
	mpa_srvc_mod_sets_clean_up (slot, (unsigned char)0xFF ) ;
	marb->srvc_status = ERR_MOD_MODEM_DEADMAN ; 	/*Set cmpltn status	*/
	marb->srvc_state = 0x00 ;			/*Set "complete" state	*/
	return ;
}

/*Modem's LCR written to, Restore Contol Signals for normal use			*/
oregs->cout_cntrl_sig = (slot->sv_cout_cntrl_sig & 0x02BB) | 0x0004 ;
slot->frame_ctr = gregs->gicp_frame_ctr ;		/*Snap shot for timing	*/
marb->cont_funct = mpa_srvc_mod_sets_12 ;		/*Point to req cmplt fn	*/
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/
}   /*End of MODIFY UART/MODEM SETTINGS - 11th FUNCTION, mpa_srvc_mod_sets_11	*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *           MODIFY UART/MODEM SETTINGS - 12th FUNCTION, REQUEST COMPLETE      *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_mod_sets_12 ( unsigned long		        caller_parm,
				    struct marb_struct FARm *  		marb,
	      			    struct ssp_struct FARm *   		sspcb,
	      			    struct slot_struct FARm *  		slot,
	      			    struct icp_global_struct FARm * 	gregs,
				    struct cin_struct FARm *	  	iregs,
	      			    struct cout_struct FARm *	  	oregs
			          )
#else
static void  mpa_srvc_mod_sets_12 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/



/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
sspcb       = sspcb ;
iregs       = iregs ;
oregs       = oregs ;

/*Start of function's logic							*/
if ( slot->slot_state != 0xFF )				/*Is Brd still present?	*/
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_MOD_MODEM_REMOVED ;	/*Set completion status	*/
	return ;
}
if ( frame_delay_chk ( (unsigned short int)0x04, gregs, slot ) == 0 )
{
	marb->srvc_state = 0xFF ;			/*Set "call back" state	*/
	return ;					/*4 frames not elapsed	*/
}

/*Have delayed long enough for MPA to "see" control signals, complete request	*/
mpa_srvc_mod_sets_clean_up (slot, (unsigned char)0x00 ) ; /*Restore SSP regs	*/
marb->srvc_status = 0x00 ;				/*Set completion status	*/
return ;						/*Exit, Request Complete*/

}   /*End of MODIFY UART/MODEM SETTINGS - 12th FUNCTION, mpa_srvc_mod_sets_12	*/








/*#stitle SLOT SERVICE - MODIFY UART/MODEM SETTINGS, LOCAL FUNCTION#*/
/*#page*/
/****************************************************************************************
 *                                                                             		*
 *MODIFY UART/MODEM SETTINGS, LOCAL FUNCTION - Clean Up SSP (mpa_srvc_mod_sets_clean_up)*
 *                                                                     			*
 * PURPOSE: To restore state of SSP registers when service has been completed (either 	*
 *	   due to error or successful completion					*
 *											*
 * NOTES:										*
 *                                                                     			*
 * CALL: void  mpa_srvc_mod_sets_clean_up  (struct slot_struct FARm  *	slot,		*
 *					    unsigned char		type)		*
 *                                                                     			*
 *			where:	slot = Slot Control Block pointer			*
 *				type = Clean up type:					*
 *					o 0x00 = Normal i.e. Reg Modem request cmplt	*
 *					o 0xFF = Abnormal i.e. Reg Modem request	*
 *						 aborted due to some failure or Board	*
 *						 removed				*
 *                                                                     			*
 *                                                                     			*
 * RETURN: None										*
 *                                                                     			*
 ****************************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void	mpa_srvc_mod_sets_clean_up (struct slot_struct FARm *  slot,
					    unsigned char	       type)
#else
static void	mpa_srvc_mod_sets_clean_up (slot, type)

struct slot_struct FARm *  slot;
unsigned char	       	   type;
#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

#ifndef					RAMP_BSDI		/*R14*/
struct cout_struct FARm *	  	oregs ;		/*Slot's SSP Output regs	   */
#else								/*BSDI Compile		   */
volatile struct cout_struct FARm *	oregs ;		/*Slot's SSP Output regs	   */
#endif								/*End RAMP_BSDI Conditional*/



/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
type = type ;

/*Restore saved SSP registers							*/
oregs = slot->output ;					/*Slot's output regs ptr*/
oregs->cout_flow_config = (slot->sv_cout_flow_config & 0x2F) |  /*Rstr out sigs */
                          (oregs->cout_flow_config & 0x10) | 0x40 ;
oregs->cout_cntrl_sig   = (slot->sv_cout_cntrl_sig & 0x02BB) | 0x0004 ;
oregs->cout_xon_1	= slot->sv_cout_xon_1 ;		/*Restore XON char	*/
#ifndef 				NO_FN_PROTO_PARMS /*Compile if Protypes OK*/
slot->clean_up          = (void (*)(struct slot_struct FARm *, unsigned char))0 ;
#else							  /*Compile if NO Protypes*/
slot->clean_up          = (void (*)())0 ;
#endif
return ;

}					/*End of mpa_srvc_mod_sets_clean_up	*/






/*#stitle SLOT SERVICE - SET INTERNAL LOOP BACK (mpa_srvc_set_loop_back)#*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *                  SET INTERNAL LOOP BACK - SERVICE ENTRY POINT               *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
extern unsigned short int   mpa_srvc_set_loop_back (struct marb_struct FARm  *marb)
#else
extern unsigned short int   mpa_srvc_set_loop_back (marb)

struct marb_struct FARm  *marb;
#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

/*General working variables							*/
unsigned char		status ;			/*Funct. ret status	*/



/************************
 *   START OF CODE	*
 ************************/

status = val_marb (marb) ;				/*Validate caller's MARB*/
if ( status != 0 )					/*Is MARB OK to use?	*/
	return (ERR_SET_LOOP_MARB) ;			/*If no, return error	*/
if ( marb->in_use == 0 )				/*Initial MARB Request?	*/
{							/*If yes, init Fn ptr	*/
	marb->cont_funct = mpa_srvc_set_loop_1 ;	/*Init MARB's FN ptr	*/
}
else							/*"Call Back" MARB,	*/
{							/*Validate SRVC active	*/
	if (marb->actv_srvc != MARB_ACTV_SRVC_SET_LOOP) /*Valid "Call Back"? 	*/
	{						/*If no, exit		*/
		marb->srvc_status = ERR_SET_LOOP_CALL_BK ;
		marb->srvc_state  = 0x00 ;
		return ((unsigned short int)(((unsigned short int)(marb->srvc_state)<<8)+
                                               marb->srvc_status) ) ;
	}
}
return ( async_req_hndlr(marb) ) ;			/*Go handle request	*/

}			/*END of mpa_srvc_set_loop_back Service Entry function	*/

/*#page*/
/*******************************************************************************
 *                                                                             *
 *	      	SET INTERNAL LOOP BACK - 1st Function, Set Loop Back Control   *
 *						      Signal bits	       *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_set_loop_1 ( unsigned long		   caller_parm,
			           struct marb_struct FARm *  	   marb,
	      		           struct ssp_struct FARm *   	   sspcb,
	      		           struct slot_struct FARm *  	   slot,
	      		           struct icp_global_struct FARm * gregs,
			           struct cin_struct FARm *	   iregs,
	      		           struct cout_struct FARm *  	   oregs
			         )
#else
static void  mpa_srvc_set_loop_1 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

caller_parm = caller_parm ;
sspcb       = sspcb ;
iregs       = iregs ;

/*Start of function's logic							*/
if ( slot->req_outstnd != 0x00 )			/*Is a Req Outstand'g?	*/
{							/*If yes, return error	*/
	marb->srvc_status = ERR_SET_LOOP_REQ_OUT ;	/*Indicate error	*/
	return ;					/*Exit			*/
}
if ( slot->slot_state != 0xFF )				/*Is Modem registered?	*/
{							/*If no, error		*/
	marb->srvc_status = ERR_SET_LOOP_NOT_REG ;	/*Indicate error	*/
	return ;					/*Exit			*/
}

/*Caller's request valid, set Internal Loop Back Control Signal bits		*/
oregs->cout_cntrl_sig = (oregs->cout_cntrl_sig & 0x02BF) | 0x008C; /*Set loop bk*/

/*Exit function & delay 0x1800 frames thus allowing time for MPA to "see" Internal*/
/*Loop Back control signal setting and allow time for non-UART based Modems to	*/
/*get set up (long delay required by Boca Modem)				*/
slot->frame_ctr  = gregs->gicp_frame_ctr ;		/*Snap shot for timing	*/
#ifndef 				NO_FN_PROTO_PARMS /*Compile if Protypes OK*/
slot->clean_up   = (void (*)(struct slot_struct FARm *, unsigned char))mpa_srvc_set_loop_clean_up ;
#else							  /*Compile if NO Protypes*/
slot->clean_up   = (void (*)())mpa_srvc_set_loop_clean_up ;
#endif
marb->actv_srvc  = MARB_ACTV_SRVC_SET_LOOP ;		/*Indicate outstnd'g req*/
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
marb->cont_funct = mpa_srvc_set_loop_2 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/
}	/*End of Set Internal Loop Back  - 1st Function, mpa_srvc_set_loop_1	*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *	      	SET INTERNAL LOOP BACK - 2nd Function, Delay		       *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_set_loop_2 ( unsigned long		   caller_parm,
				   struct marb_struct FARm *  	   marb,
	      			   struct ssp_struct FARm *   	   sspcb,
	      			   struct slot_struct FARm *  	   slot,
	      			   struct icp_global_struct FARm * gregs,
				   struct cin_struct FARm *	   iregs,
	      			   struct cout_struct FARm *	   oregs
				 )
#else
static void  mpa_srvc_set_loop_2 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		   	caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/



/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
sspcb       = sspcb ;
oregs       = oregs ;
iregs       = iregs ;

/*Start of function's logic							*/
if ( slot->slot_state != 0xFF )				/*Is Brd still present?	*/
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_SET_LOOP_REMOVED ;	/*Set completion status	*/
	return ;
}
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
if ( frame_delay_chk ( (unsigned short int)0x1800, gregs, slot ) == 0 )
	return ;					/*1800 frames not elapsed*/

/*Have delayed long enough for MPA to "see" control signal setting indicating	*/
/*to set MCR's Internal Loop Back bit, complete request				*/
slot->loop_back = 0xFF ;				/*Loop Back state = ON	*/
mpa_srvc_set_loop_clean_up (slot, (unsigned char)0x00 ); /*Restore SSP regs	*/
marb->srvc_status = 0x00 ;				/*Set completion status	*/
marb->srvc_state  = 0x00 ;				/*Set completion state	*/
return ;						/*Exit, Request Cmplt	*/
}	/*End of Set Internal Loop Back  - 2nd Function, mpa_srvc_set_loop_back_2*/









/*#stitle SLOT SERVICE - SET INTERNAL LOOP BACK, LOCAL FUNCTION#*/
/*#page*/
/****************************************************************************************
 *                                                                             		*
 *         SET INTERNAL LOOP BACK - Clean Up SSP (mpa_srvc_set_loop_clean_up)		*
 *                                                                     			*
 * PURPOSE: To restore state of SSP registers when service has been CANCELED		*
 *											*
 * NOTES: 										*
 *                                                                     			*
 * CALL: void  mpa_srvc_set_loop_clean_up (struct slot_struct FARm *    slot,		*
 *			     	           unsigned char		type)		*
 *                                                                     			*
 *			where:	slot = Slot Control Block pointer			*
 *				type = Clean up type:					*
 *					o 0x00 = Normal					*
 *					o 0xFF = Abnormal, aborted due to some failure	*
 *						 or Board removed			*
 *                                                                     			*
 *                                                                     			*
 * RETURN: None										*
 *                                                                     			*
 ****************************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void	mpa_srvc_set_loop_clean_up (struct slot_struct FARm *    slot,
					    unsigned char		 type)
#else
static void	mpa_srvc_set_loop_clean_up (slot, type)

struct slot_struct FARm *  slot;
unsigned char	       	   type;
#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clear Internal Loop Back Control Signals if abnormal "clean up"		*/
if (type != 0x00)					/*Abnormal call?	*/
{							/*If yes, clear loop back*/
	slot->output->cout_cntrl_sig = (slot->output->cout_cntrl_sig & 0x0237) | 0x0004 ;
}
#ifndef 				NO_FN_PROTO_PARMS /*Compile if Protypes OK*/
slot->clean_up = (void (*)(struct slot_struct FARm *, unsigned char))0 ;
#else							  /*Compile if NO Protypes*/
slot->clean_up = (void (*)())0 ;
#endif
return ;

}					/*End of mpa_srvc_set_loop_clean_up	*/









/*#stitle SLOT SERVICE - CLEAR INTERNAL LOOP BACK (mpa_srvc_clr_loop_back)#*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *                  CLEAR INTERNAL LOOP BACK - SERVICE ENTRY POINT             *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
extern unsigned short int   mpa_srvc_clr_loop_back (struct marb_struct FARm  *marb)
#else
extern unsigned short int   mpa_srvc_clr_loop_back (marb)

struct marb_struct FARm  *marb;
#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

/*General working variables							*/
unsigned char		status ;			/*Funct. ret status	*/



/************************
 *   START OF CODE	*
 ************************/

status = val_marb (marb) ;				/*Validate caller's MARB*/
if ( status != 0 )					/*Is MARB OK to use?	*/
	return (ERR_CLR_LOOP_MARB) ;			/*If no, return error	*/
if ( marb->in_use == 0 )				/*Initial MARB Request?	*/
{							/*If yes, init Fn ptr	*/
	marb->cont_funct = mpa_srvc_clr_loop_1 ;	/*Init MARB's FN ptr	*/
}
else							/*"Call Back" MARB,	*/
{							/*Validate SRVC active	*/
	if (marb->actv_srvc != MARB_ACTV_SRVC_CLR_LOOP) /*Valid "Call Back"? 	*/
	{						/*If no, exit		*/
		marb->srvc_status = ERR_CLR_LOOP_CALL_BK ;
		marb->srvc_state  = 0x00 ;
		return ((unsigned short int)(((unsigned short int)(marb->srvc_state)<<8)+
                                               marb->srvc_status) ) ;
	}
}
return ( async_req_hndlr(marb) ) ;			/*Go handle request	*/

}			/*END of mpa_srvc_clr_loop_back Service Entry function	*/

/*#page*/
/*******************************************************************************
 *                                                                             *
 *	   CLEAR INTERNAL LOOP BACK - 1st Function, Clear Loop Back Control    *
 *						      Signal bits	       *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_clr_loop_1 ( unsigned long		   caller_parm,
			           struct marb_struct FARm *  	   marb,
	      		           struct ssp_struct FARm *   	   sspcb,
	      		           struct slot_struct FARm *  	   slot,
	      		           struct icp_global_struct FARm * gregs,
			           struct cin_struct FARm *	   iregs,
	      		           struct cout_struct FARm *  	   oregs
			         )
#else
static void  mpa_srvc_clr_loop_1 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif
{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

caller_parm = caller_parm ;
sspcb       = sspcb ;
iregs       = iregs ;

/*Start of function's logic							*/
if ( slot->req_outstnd != 0x00 )			/*Is a Req Outstand'g?	*/
{							/*If yes, return error	*/
	marb->srvc_status = ERR_CLR_LOOP_REQ_OUT ;	/*Indicate error	*/
	return ;					/*Exit			*/
}
if ( slot->slot_state != 0xFF )				/*Is Modem registered?	*/
{							/*If no, error		*/
	marb->srvc_status = ERR_CLR_LOOP_NOT_REG ;	/*Indicate error	*/
	return ;					/*Exit			*/
}

/*Caller's request valid, clear Internal Loop Back Control Signal bits		*/
oregs->cout_cntrl_sig = (oregs->cout_cntrl_sig & 0x0237) | 0x0004; /*Clr loop bk*/

/*Exit function & delay 256 frames thus allowing time for MPA to "see" Internal */
/*Loop Back control signal setting and allow time for non-UART based Modems to	*/
/*get set up (long delay required by Boca Modem)				*/
slot->frame_ctr  = gregs->gicp_frame_ctr ;		/*Snap shot for timing	*/
#ifndef 				NO_FN_PROTO_PARMS /*Compile if Protypes OK*/
slot->clean_up   = (void (*)(struct slot_struct FARm *, unsigned char))mpa_srvc_clr_loop_clean_up ;
#else						/*Compile if NO Protypes*/
slot->clean_up   = (void (*)())mpa_srvc_clr_loop_clean_up ;
#endif
marb->actv_srvc  = MARB_ACTV_SRVC_CLR_LOOP ;		/*Indicate outstnd'g req*/
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
marb->cont_funct = mpa_srvc_clr_loop_2 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/
}	/*End of Clear Internal Loop Back  - 1st Function, mpa_srvc_clr_loop_1	*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *	      	CLEAR INTERNAL LOOP BACK - 2nd Function, Delay		       *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_clr_loop_2 ( unsigned long		   caller_parm,
				   struct marb_struct FARm *  	   marb,
	      			   struct ssp_struct FARm *   	   sspcb,
	      			   struct slot_struct FARm *  	   slot,
	      			   struct icp_global_struct FARm * gregs,
				   struct cin_struct FARm *	   iregs,
	      			   struct cout_struct FARm *	   oregs
				 )
#else
static void  mpa_srvc_clr_loop_2 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		   	caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/



/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
sspcb       = sspcb ;
oregs       = oregs ;
iregs       = iregs ;

/*Start of function's logic							*/
if ( slot->slot_state != 0xFF )				/*Is Brd still present?	*/
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_CLR_LOOP_REMOVED ;	/*Set completion status	*/
	return ;
}
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
if ( frame_delay_chk ( (unsigned short int)0x1800, gregs, slot ) == 0 )
	return ;					/*1800 frames not elapsed*/

/*Have delayed long enough for MPA to "see" control signal setting indicating	*/
/*to clear MCR's Internal Loop Back bit, complete request			*/
slot->loop_back = 0x00 ;				/*Loop Back state = OFF	*/
mpa_srvc_clr_loop_clean_up (slot, (unsigned char)0x00 ); /*Restore SSP regs	*/
marb->srvc_status = 0x00 ;				/*Set completion status	*/
marb->srvc_state  = 0x00 ;				/*Set completion state	*/
return ;						/*Exit, Request Cmplt	*/
} /*End of Clear Internal Loop Back  - 2nd Function, mpa_srvc_clr_loop_back_2	*/









/*#stitle EXCEPTION SERVICE - CLEAR INTERNAL LOOP BACK, LOCAL FUNCTION#*/
/*#page*/
/****************************************************************************************
 *                                                                             		*
 *         CLEAR INTERNAL LOOP BACK - Clean Up SSP (mpa_srvc_clr_loop_clean_up)		*
 *                                                                     			*
 * PURPOSE: To restore state of SSP registers when service has been CANCELED		*
 *											*
 * NOTES: 										*
 *                                                                     			*
 * CALL: void  mpa_srvc_clr_loop_clean_up (struct slot_struct FARm *    slot,		*
 *			     	           unsigned char		type)		*
 *                                                                     			*
 *			where:	slot = Slot Control Block pointer			*
 *				type = Clean up type:					*
 *					o 0x00 = Normal					*
 *					o 0xFF = Abnormal, aborted due to some failure	*
 *						 or Board removed			*
 *                                                                     			*
 *                                                                     			*
 * RETURN: None										*
 *                                                                     			*
 ****************************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void	mpa_srvc_clr_loop_clean_up (struct slot_struct FARm *    slot,
					    unsigned char		 type)
#else
static void	mpa_srvc_clr_loop_clean_up (slot, type)

struct slot_struct FARm *  slot;
unsigned char	       	   type;
#endif
{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
type = type ;

/*Start of actual "clean up"							*/
#ifndef 				NO_FN_PROTO_PARMS /*Compile if Protypes OK*/
slot->clean_up = (void (*)(struct slot_struct FARm *, unsigned char))0 ;
#else							  /*Compile if NO Protypes*/
slot->clean_up = (void (*)())0 ;
#endif
return ;

}					/*End of mpa_srvc_clr_loop_clean_up	*/








/*#stitle SLOT SERVICE - CLOSE MODEM BOARD (mpa_srvc_close_modem)#*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *                       CLOSE MODEM BOARD - SERVICE ENTRY POINT               *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
extern unsigned short int   mpa_srvc_close_modem (struct marb_struct FARm *  marb)
#else
extern unsigned short int   mpa_srvc_close_modem (marb)

struct marb_struct FARm  *marb;
#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

/*General working variables							*/
unsigned char		status ;			/*Funct. ret status	*/



/************************
 *   START OF CODE	*
 ************************/

status = val_marb (marb) ;				/*Validate caller's MARB*/
if ( status != 0 )					/*Is MARB OK to use?	*/
	return (ERR_CLOSE_MODEM_MARB) ;			/*If no, return error	*/
if ( (marb->in_use != 0) || (marb->req_type != 0x00) )	/*Init MARB Req & Waitd?*/
	return (ERR_CLOSE_MODEM_MARB) ;			/*If no, return error	*/
marb->cont_funct = mpa_srvc_close_modem_1 ;		/*Init MARB's FN ptr	*/
return ( async_req_hndlr(marb) ) ;			/*Go handle request	*/

}			/*END of mpa_srvc_close_modem Service Entry function	*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *      CLOSE MODEM BOARD - 1st (and only) Function, Clean Up SLOTCB Function  *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_close_modem_1 ( unsigned long		        caller_parm,
				      struct marb_struct FARm *  	marb,
	      			      struct ssp_struct FARm *   	sspcb,
	      			      struct slot_struct FARm *  	slot,
	      			      struct icp_global_struct FARm * 	gregs,
				      struct cin_struct FARm *	  	iregs,
	      			      struct cout_struct FARm *	  	oregs
				    )
#else
static void  mpa_srvc_close_modem_1 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
sspcb       = sspcb ;
gregs       = gregs ;
iregs       = iregs ;

/*Start of function's logic							*/
if ( slot->req_outstnd != 0x00 )			/*Is a Req Outstand'g?	*/
{							/*If yes, return error	*/
	marb->srvc_status = ERR_CLOSE_MODEM_REQ_OUT ;	/*Indicate error	*/
	return ;					/*Exit			*/
}
if ( slot->slot_state != 0xFF )				/*Is Slot occupied?	*/
{							/*If no, error		*/
	marb->srvc_status = ERR_CLOSE_MODEM_NOT ;	/*Indicate error	*/
	return ;					/*Exit			*/
}
if ( slot->init_state != 0xFF )				/*Is Slot init'd/opn'd?	*/
{							/*If no, error		*/
	marb->srvc_status = ERR_CLOSE_MODEM_NOPN ;	/*Indicate error	*/
	return ;					/*Exit			*/
}
slot->init_state = 0x00 ;				/*MODEM/UART Closed	*/
marb->srvc_status = 0x00 ;				/*Set completion status	*/
oregs->cout_cntrl_sig = (oregs->cout_cntrl_sig & 0x0237) | 0x0004; /*Kill loop bk*/
return ;						/*Exit, Request Complete*/

}	/*End of CLOSE MODEM BOARD - 1st Function, mpa_srvc_close_modem_1	*/









/*#stitle SLOT SERVICE - CANCEL SLOT'S OUTSTANDING MARB (mpa_srvc_can_marb)#*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *              CANCEL SLOT'S OUTSTANDING MARB - SERVICE ENTRY POINT           *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
extern unsigned short int   mpa_srvc_can_marb (struct ssp_struct  FARm *        sspcb,
                               	  	       unsigned char                    slot_chan,
					       struct marb_struct FARm * FARm * ret_can_marb)
#else
extern unsigned short int   mpa_srvc_can_marb (sspcb, slot_chan, ret_can_marb)

struct ssp_struct  FARm *        sspcb;
unsigned char                    slot_chan;
struct marb_struct FARm * FARm * ret_can_marb;
#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

/*MPACB (MPA Control Block) and SLOTCB (Slot Control Block) variables		*/
unsigned int		 	mpa_index ;		/*MARB's MPA/Eng index	*/
struct slot_struct FARm  	*slot ;			/*MARB's SLOT Cntrl Blk	*/

/*Variables used to cancel MARB							*/
struct marb_struct FARm  	*outstnd_marb ;		/*Slot's Outstand'g MARB*/



/************************
 *   START OF CODE	*
 ************************/

*ret_can_marb = (struct marb_struct FARm *)0xFFFFFFFF ;	/*Ret SLOTCB ptr	*/
if ( val_ssp (sspcb) != 0x00 )				/*Is caller's parm OK?	*/
	return (ERR_CAN_SSPCB) ;			/*If no, exit		*/
mpa_index = slot_chan >> 4 ;				/*Index to MPA/Eng	*/
if ( sspcb->mpa[mpa_index].base_chan != 		/*Is caller's Chan OK?	*/
    ((mpa_index - (sspcb->mpa[mpa_index].eng_num)) << 4) ) 
	return (ERR_CAN_MPACB) ;			/*If no, error		*/
slot = (struct slot_struct FARm  *)&((sspcb->mpa[mpa_index].eng_head)[(slot_chan & 0x0f)]) ;

if ( (slot->in_use     != 0xFF)		||		/*Is Caller's Slot valid*/
     (slot->alloc      != 0xFF)		||
     (slot->chan_num   != slot_chan)	||
     (slot->slot_state == 0x00)		||
     (slot->signature->in_use != 0xFF)  )
	return (ERR_CAN_SLOTCB) ;				/*If no, error		*/

/*Caller's request valid, set up and determine if Slot has outstanding MARB	*/
*ret_can_marb = (struct marb_struct FARm *)0x00000000 ;	/*Indicate request valid*/
if ( slot->req_outstnd != 0 )				/*Is MARB oustnd'g?	*/
{							/*If yes		*/
	outstnd_marb = slot->actv_marb ;		/*Get ptr to Slot's MARB*/
	if ( (outstnd_marb != (struct marb_struct FARm *)0)   && /*Is Slot's 	*/
	     (outstnd_marb->in_use      != 0) 		      && /*oustand MARB	*/
	     (outstnd_marb->actv_srvc   == slot->req_outstnd) && /*pointer OK?	*/
	     (outstnd_marb->srvc_status == 0x00) 	      &&
	     (outstnd_marb->srvc_state  == 0xFF) 	      &&
	     (outstnd_marb->signature   == outstnd_marb)      &&
	     (outstnd_marb->cont_funct  == slot->cont_funct)  )
	{						        /*If yes	*/
		outstnd_marb->in_use     = 0x00 ; 	/*Indicate avail MARB	*/
		outstnd_marb->actv_srvc  = 0x00 ; 	/*Set no req outstndg	*/
		outstnd_marb->srvc_status = MARB_OUT_CANCELED ;
		outstnd_marb->srvc_state  = 0x00 ;
		outstnd_marb->signature  = (struct marb_struct FARm *)0 ;
#ifndef 				NO_FN_PROTO_PARMS /*Compile if Protypes OK*/	/*R11*/
		outstnd_marb->cont_funct = (void (*)(unsigned long,
					  	     struct marb_struct  FARm *,
	      			     	  	     struct ssp_struct   FARm *,
	      			          	     struct slot_struct  FARm *,
	      			          	     struct mpa_global_s FARm *,
				          	     struct mpa_input_s  FARm *,
	      			          	     struct mpa_output_s FARm *))0 ;
#else							  /*Compile if NO Protypes*/
		outstnd_marb->cont_funct = (void (*)())0 ;
#endif
		*ret_can_marb = outstnd_marb ;
	    /*Call outstanding MARB Service Clean Up function			*/
#ifndef 				NO_FN_PROTO_PARMS /*Compile if Protypes OK*/
		if ( slot->clean_up != (void (*)(struct slot_struct FARm *, unsigned char))0)
			(*slot->clean_up)(slot, (unsigned char)0xFF); /*If yes	*/
#else							  /*Compile if NO Protypes*/
		if ( slot->clean_up != (void (*)())0)
			(*slot->clean_up)(slot, (unsigned char)0xFF); /*If yes	*/
#endif
	}
}
slot->req_outstnd = 0x00 ;				/*Slot has no REQ Outstd*/
slot->actv_marb   = (struct marb_struct FARm *)0 ; 	/*No actv MARB		*/
#ifndef 				NO_FN_PROTO_PARMS /*Compile if Protypes OK*/	/*R11*/
slot->cont_funct  = (void (*)(unsigned long,		/*No "call back"	*/
		  	      struct marb_struct  FARm *,
	      		      struct ssp_struct   FARm *,
	      		      struct slot_struct  FARm *,
	      		      struct mpa_global_s FARm *,
			      struct mpa_input_s  FARm *,
	      		      struct mpa_output_s FARm *))0 ;
slot->clean_up = (void (*)(struct slot_struct FARm *, unsigned char))0 ;
#else							  /*Compile if NO Protypes*/
slot->cont_funct  = (void (*)())0 ;
slot->clean_up = (void (*)())0 ;
#endif
return (0x00) ;						/*Exit, Request Complete*/

}	/*End of CANCEL SLOT'S OUTSTANDING MARB - mpa_srvc_can_marb		*/









/*#stitle SLOT SERVICE - FLUSH MPA SLOT INPUT/OUTPUT BUFFERS (mpa_srvc_flush)#*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *          FLUSH MPA SLOT INPUT/OUTPUT BUFFERS - SERVICE ENTRY POINT          *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
extern unsigned short int    mpa_srvc_flush (struct marb_struct FARm  *marb)
#else
extern unsigned short int    mpa_srvc_flush (marb)

struct marb_struct FARm  *marb;
#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

/*General working variables							*/
unsigned char		status ;			/*Funct. ret status	*/



/************************
 *   START OF CODE	*
 ************************/

status = val_marb (marb) ;				/*Validate caller's MARB*/
if ( status != 0 )					/*Is MARB OK to use?	*/
	return (ERR_MPA_FLUSH_MARB) ;			/*If no, return error	*/
if ( marb->in_use == 0 )				/*Initial MARB Request?	*/
{							/*If yes, init Fn ptr	*/
	marb->cont_funct = mpa_srvc_flush_1 ;		/*Init MARB's FN ptr	*/
}
else							/*"Call Back" MARB,	*/
{							/*Validate SRVC active	*/
	if ( marb->actv_srvc != MARB_ACTV_SRVC_FLUSH ) 	/*Valid "Call Back"?  	*/
	{						/*If no, exit		*/
		marb->srvc_status = ERR_MPA_FLUSH_CALL_BK ;
		marb->srvc_state  = 0x00 ;
		return ((unsigned short int)(((unsigned short int)(marb->srvc_state)<<8)+
                                               marb->srvc_status) ) ;
	}
}
return ( async_req_hndlr(marb) ) ;			/*Go handle request	*/


}			/*END of mpa_srvc_flush Service Entry function		*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *     FLUSH MPA SLOT INPUT/OUTPUT BUFFERS - 1st FUNCTION, ISSUE SLOT RESET    *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_flush_1     ( unsigned long		        caller_parm,
				    struct marb_struct FARm *  		marb,
	      			    struct ssp_struct FARm *   		sspcb,
	      			    struct slot_struct FARm *  		slot,
	      			    struct icp_global_struct FARm * 	gregs,
				    struct cin_struct FARm *	  	iregs,
	      			    struct cout_struct FARm *	  	oregs
				  )
#else
static void  mpa_srvc_flush_1 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/





/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
sspcb       = sspcb ;
iregs       = iregs ;

/*Start of function's logic							*/
if ( slot->req_outstnd != 0x00 )			/*Is a Req Outstand'g?	*/
{							/*If yes, return error	*/
	marb->srvc_status = ERR_MPA_FLUSH_REQ_OUT ;	/*Indicate error	*/
	return ;					/*Exit			*/
}
if ( slot->slot_state != 0xFF )				/*Is Modem registered?	*/
{							/*If no, error		*/
	marb->srvc_status = ERR_MPA_FLUSH_NOT_REG ;	/*Indicate error	*/
	return ;					/*Exit			*/
}
if ( slot->init_state != 0xFF )				/*Is Slot init'd/opn'd?	*/
{							/*If no, error		*/
	marb->srvc_status = ERR_MPA_FLUSH_NOPN ;	/*Indicate error	*/
	return ;					/*Exit			*/
}

/*Caller's request valid, save SSP registers that will be used to Flush Slot's	*/
/*Input/Output buffers								*/
slot->sv_cout_cntrl_sig   = oregs->cout_cntrl_sig ;
#ifndef 				NO_FN_PROTO_PARMS /*Compile if Protypes OK*/
slot->clean_up            = (void (*)(struct slot_struct FARm *, unsigned char))mpa_srvc_flush_clean_up ;
#else							  /*Compile if NO Protypes*/
slot->clean_up            = (void (*)())mpa_srvc_flush_clean_up ;
#endif

/*Set slot's/channel's control signals to indicate to MPA to Flush (reset)	*/
/*Input/Output buffers								*/
oregs->cout_cntrl_sig = 0x188 ;				/*MPA's Slot Reset	*/
slot->read_toggle     = 0x00 ;				/*UART I/O Read toggle	*/
slot->frame_ctr = gregs->gicp_frame_ctr ;		/*Snap shot for timing	*/

/*Exit function & delay 8 frames thus allowing time for MPA to "see" RESET	*/
marb->actv_srvc  = MARB_ACTV_SRVC_FLUSH ;		/*Indicate outstnd'g req*/
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
marb->cont_funct = mpa_srvc_flush_2 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*END of Flush MPA Input/Output buffers - 1st function, mpa_srvc_flush_1*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *     FLUSH MPA SLOT INPUT/OUTPUT BUFFERS - 2nd FUNCTION, Address UART's FCR  *
 *					     FIFO Control Register	       *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_flush_2   ( unsigned long		        	caller_parm,
				  struct marb_struct FARm *  		marb,
	      			  struct ssp_struct FARm *   		sspcb,
	      			  struct slot_struct FARm *  		slot,
	      			  struct icp_global_struct FARm * 	gregs,
				  struct cin_struct FARm *	  	iregs,
	      			  struct cout_struct FARm *	  	oregs
				)
#else
static void  mpa_srvc_flush_2   (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/



/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
sspcb       = sspcb ;
iregs       = iregs ;
oregs       = oregs ;

/*Start of function's logic							*/
if ( slot->slot_state != 0xFF )				/*Board still present?	*/
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_MPA_FLUSH_REMOVED ;	/*Set completion status	*/
	return ;
}
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
if ( frame_delay_chk ( (unsigned short int)0x08, gregs, slot ) == 0 )
	return ;					/*4 frames not elapsed	*/

/*Have delayed long enough for MPA to "see" RESET, point to UART's FIFO Control	*/
/*Register (FCR) assuming that it exists					*/
oregs->cout_cntrl_sig = 0x122 ;			/*R15*/	/*Addr UART's FCR	*/
marb->cont_funct = mpa_srvc_flush_3 ; 		/*R15*/
slot->frame_ctr = gregs->gicp_frame_ctr ;		/*Snap shot for timing	*/
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*END of Flush MPA Input/Output buffers - 2nd function, mpa_srvc_flush_2*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *     FLUSH MPA SLOT INPUT/OUTPUT BUFFERS - 3rd FUNCTION, Flush UART's FIFO,  *
 *					     assuming it exists		       *
 *                                                                             *
 *******************************************************************************/
/*R15*/

#ifndef 		NO_FN_PROTO_PARMS
static void  mpa_srvc_flush_3   ( unsigned long		        	caller_parm,
				  struct marb_struct FARm *  		marb,
	      			  struct ssp_struct FARm *   		sspcb,
	      			  struct slot_struct FARm *  		slot,
	      			  struct icp_global_struct FARm * 	gregs,
				  struct cin_struct FARm *	  	iregs,
	      			  struct cout_struct FARm *	  	oregs
				)
#else
static void  mpa_srvc_flush_3   (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/



/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
sspcb       = sspcb ;
iregs       = iregs ;
oregs       = oregs ;

/*Start of function's logic							*/
if ( slot->slot_state != 0xFF )				/*Board still present?	*/
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_MPA_FLUSH_REMOVED ;	/*Set completion status	*/
	return ;
}
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
if ( frame_delay_chk ( (unsigned short int)0x04, gregs, slot ) == 0 )
	return ;					/*4 frames not elapsed	*/

/*Have delayed long enough for MPA to "see" control signal setting indicating	*/
/*that output be directed to UART's FIFO Control Register (FCR)			*/
oregs->cout_xon_1 = 0x07 ;				/*Enable 16550 FIFO	*/
							/* (assumes 16550)	*/
oregs->cout_flow_config  ^= 0x10 ;			/*Initiate xmit'g char	*/

/*Set up & wait till FCR output sent						*/
slot->frame_ctr = gregs->gicp_frame_ctr ;		/*Snap shot for timing	*/
marb->cont_funct = mpa_srvc_flush_4 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*END of Flush MPA Input/Output buffers - 3rd function, mpa_srvc_flush_3*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *     FLUSH MPA SLOT INPUT/OUTPUT BUFFERS - 4th FUNCTION, Request Complete    *
 *                                                                             *
 *******************************************************************************/
/*R15*/

#ifndef 		NO_FN_PROTO_PARMS
static void  mpa_srvc_flush_4   ( unsigned long		        	caller_parm,
				  struct marb_struct FARm *  		marb,
	      			  struct ssp_struct FARm *   		sspcb,
	      			  struct slot_struct FARm *  		slot,
	      			  struct icp_global_struct FARm * 	gregs,
				  struct cin_struct FARm *	  	iregs,
	      			  struct cout_struct FARm *	  	oregs
				)
#else
static void  mpa_srvc_flush_4   (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
sspcb       = sspcb ;
iregs       = iregs ;
oregs       = oregs ;

/*Start of function's logic							*/
if ( slot->slot_state != 0xFF )				/*Board still present?	*/
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_MPA_FLUSH_REMOVED ;	/*Set completion status	*/
	return ;
}
if ( ((oregs->cout_flow_config & 0x10) != (oregs->cout_intnl_flow_ctrl & 0x10)) ||
     ((oregs->cout_intnl_flow_ctrl & 0x08) == 0x08) )	/*FCR char xmit done	*/
{							/*If no, check deadman	*/
	if ( frame_delay_chk (DEADMAN_DELAY, gregs, slot) == 0x00 )
	{						/*If deadman not expired*/
		marb->srvc_state = 0xFF ;		/*Set "call back" state	*/
		return ;
	}
	(*slot->clean_up)(slot, (unsigned char)0xFF) ;
	marb->srvc_status = ERR_REG_MODEM_DEADMAN ; 	/*Set cmpltn status	*/
	return ;
}

/*Modem's FCR, FIFO Control Register, written to (if it exists) and hence UART	*/
/*FIFO flushed, Complete Request						*/
mpa_srvc_flush_clean_up  (slot, (unsigned char)0x00 ) ; /*Restore SSP regs	*/
marb->srvc_status = 0x00 ;				/*Set completion status	*/
return ;						/*Exit, Request Cmplt	*/

}	/*END of Flush MPA Input/Output buffers - 4th function, mpa_srvc_flush_4*/









/*#stitle SLOT SERVICE - FLUSH MPA SLOT INPUT/OUTPUT BUFFERS, LOCAL FUNCTION#*/
/*#page*/
/****************************************************************************************
 *                                                                             		*
 *     FLUSH MPA SLOT INPUT/OUTPUT BUFFERS - Clean Up SSP (mpa_srvc_flush_clean_up)	*
 *                                                                     			*
 * PURPOSE: To restore state of SSP registers when service has been completed (either 	*
 *	   due to error or successful completion)					*
 *											*
 * NOTES:										*
 *                                                                     			*
 * CALL: void  mpa_srvc_flush_clean_up  (struct slot_struct FARm *    slot,		*
 *			     	         unsigned char		      type)		*
 *                                                                     			*
 *			where:	slot = Slot Control Block pointer			*
 *				type = Clean up type:					*
 *					o 0x00 = Normal i.e. Reg Modem request cmplt	*
 *					o 0xFF = Abnormal i.e. Reg Modem request	*
 *						 aborted due to some failure or Board	*
 *						 removed				*
 *                                                                     			*
 *                                                                     			*
 * RETURN: None										*
 *                                                                     			*
 ****************************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void	mpa_srvc_flush_clean_up (struct slot_struct FARm *    slot,
					 unsigned char		      type)
#else
static void	mpa_srvc_flush_clean_up (slot, type)

struct slot_struct FARm *  slot;
unsigned char	       	   type;
#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

/*SSP variables relating to MARB's/Request's specific SLOT (Channel)		*/
#ifndef					RAMP_BSDI		/*R14*/
struct cout_struct FARm *	  	oregs ;		/*Slot's SSP Output regs	   */
#else								/*BSDI Compile		   */
volatile struct cout_struct FARm *	oregs ;		/*Slot's SSP Output regs	   */
#endif								/*End RAMP_BSDI Conditional*/



/************************
 *   START OF CODE	*
 ************************/

/*Delay if actually canceling/aborting an outstanding Flush MARB, thus allowing	*/
/*for MPA to handle Reset							*/
if ( (type == 0xFF) && (slot->actv_marb->srvc_status == MARB_OUT_CANCELED) )
     /*If abort'g MARB, delay to give MPA chance to reset			*/
	while ( frame_delay_chk ( (unsigned short int)0x08, 	
				  slot->actv_marb->sspcb->global, slot)	== 0 )
	{ }

/*Restore saved SSP registers							*/
oregs = slot->output ;					/*Slot's output regs ptr*/
oregs->cout_cntrl_sig   = (slot->sv_cout_cntrl_sig & 0x02BB) | 0x0004 ;
#ifndef 				NO_FN_PROTO_PARMS /*Compile if Protypes OK*/
slot->clean_up          = (void (*)(struct slot_struct FARm *, unsigned char))0 ;
#else							  /*Compile if NO Protypes*/
slot->clean_up          = (void (*)())0 ;
#endif
return ;

}					/*End of mpa_srvc_flush_clean_up	*/









/*#stitle SLOT SERVICE - HARD RESET MODEM BOARD (mpa_srvc_hard_reset)#*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *          		HARD RESET MODEM BOARD - SERVICE ENTRY POINT           *
 *                                                                             *
 *******************************************************************************/
/*R5*/
#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
extern unsigned short int   mpa_srvc_hard_reset (struct marb_struct FARm  *marb)
#else
extern unsigned short int   mpa_srvc_hard_reset (marb)

struct marb_struct FARm  *marb;
#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

/*General working variables							*/
unsigned char		status ;			/*Funct. ret status	*/



/************************
 *   START OF CODE	*
 ************************/

status = val_marb (marb) ;				/*Validate caller's MARB*/
if ( status != 0 )					/*Is MARB OK to use?	*/
	return (ERR_HRD_RST_MARB) ;			/*If no, return error	*/
if ( marb->in_use == 0 )				/*Initial MARB Request?	*/
{							/*If yes, init Fn ptr	*/
	marb->cont_funct = mpa_srvc_hard_rst_1 ;	/*Init MARB's FN ptr	*/
}
else							/*"Call Back" MARB,	*/
{							/*Validate SRVC active	*/
	if ( marb->actv_srvc != MARB_ACTV_SRVC_HARD_RESET ) /*Valid "Call Back"?*/
	{						/*If no, exit		*/
		marb->srvc_status = ERR_HRD_RST_CALL_BK ;
		marb->srvc_state  = 0x00 ;
		return ((unsigned short int)(((unsigned short int)(marb->srvc_state)<<8)+
                                               marb->srvc_status) ) ;
	}
}
return ( async_req_hndlr(marb) ) ;			/*Go handle request	*/


}			/*END of mpa_srvc_hard_reset Service Entry function	*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 * 	HARD RESET MODEM BOARD - 1st FUNCTION, ISSUE HARD MODEM BOARD RESET    *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_hard_rst_1  ( unsigned long		        caller_parm,
				    struct marb_struct FARm *  		marb,
	      			    struct ssp_struct FARm *   		sspcb,
	      			    struct slot_struct FARm *  		slot,
	      			    struct icp_global_struct FARm * 	gregs,
				    struct cin_struct FARm *	  	iregs,
	      			    struct cout_struct FARm *	  	oregs
				  )
#else
static void  mpa_srvc_hard_rst_1 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/





/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
sspcb       = sspcb ;
iregs       = iregs ;

/*Start of function's logic							*/
if ( slot->req_outstnd != 0x00 )			/*Is a Req Outstand'g?	*/
{							/*If yes, return error	*/
	marb->srvc_status = ERR_HRD_RST_REQ_OUT ;	/*Indicate error	*/
	return ;					/*Exit			*/
}
if ( slot->slot_state != 0xFF )				/*Is Modem registered?	*/
{							/*If no, error		*/
	marb->srvc_status = ERR_HRD_RST_NOT_REG ;	/*Indicate error	*/
	return ;					/*Exit			*/
}
if ( slot->init_state != 0xFF )				/*Is Slot init'd/opn'd?	*/
{							/*If no, error		*/
	marb->srvc_status = ERR_HRD_RST_NOPN ;		/*Indicate error	*/
	return ;					/*Exit			*/
}

/*Caller's request valid, save SSP registers that will be used to Reset Modem	*/
slot->sv_cout_flow_config = oregs->cout_flow_config ;
slot->sv_cout_cntrl_sig   = oregs->cout_cntrl_sig ;
slot->sv_cout_xon_1       = oregs->cout_xon_1 ;
#ifndef 				NO_FN_PROTO_PARMS /*Compile if Protypes OK*/
slot->clean_up            = (void (*)(struct slot_struct FARm *, unsigned char))mpa_srvc_hard_rst_clean_up ;
#else							  /*Compile if NO Protypes*/
slot->clean_up            = (void (*)())mpa_srvc_hard_rst_clean_up ;
#endif
oregs->cout_flow_config = (oregs->cout_flow_config & 0x10) | 0x40 | 0x08 ;

/*Set slot's/channel's control signals to indicate to MPA to Hard Reset Modem Board*/
oregs->cout_cntrl_sig = 0x1CC ;				/*Issue Reset Modem	*/
slot->frame_ctr       = gregs->gicp_frame_ctr ;		/*Snap shot for timing	*/
slot->read_toggle     = 0x00 ;				/*UART I/O Read toggle	*/
slot->break_state     = 0x00 ;				/*Indicate Break = OFF	*/
slot->loop_back	      = 0x00 ;				/*Indicate Loop BK = OFF*/

/*Exit function & delay 8 frames thus allowing time for MPA to "see" Modem Reset*/
marb->actv_srvc  = MARB_ACTV_SRVC_HARD_RESET ;		/*Indicate outstnd'g req*/
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
marb->cont_funct = mpa_srvc_hard_rst_2 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*END of Hard Reset Modem - 1st function, mpa_srvc_hard_rst_1		*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 * 	HARD RESET MODEM BOARD - 2nd FUNCTION, ADDRESS UART's IER Register     *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_hard_rst_2 ( unsigned long		        caller_parm,
				   struct marb_struct FARm *  		marb,
	      			   struct ssp_struct FARm *   		sspcb,
	      			   struct slot_struct FARm *  		slot,
	      			   struct icp_global_struct FARm * 	gregs,
				   struct cin_struct FARm *	  	iregs,
	      			   struct cout_struct FARm *	  	oregs
				 )
#else
static void  mpa_srvc_hard_rst_2 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/



/************************
 *   START OF CODE	*
 ************************/

caller_parm = caller_parm ;
sspcb       = sspcb ;
iregs       = iregs ;

/*Start of function's logic							*/
if ( slot->slot_state != 0xFF )				/*Board still present?	*/
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_HRD_RST_REMOVED ;	/*Set completion status	*/
	return ;
}
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
if ( frame_delay_chk ( (unsigned short int)0x08, gregs, slot ) == 0 )
	return ;					/*8 frames not elapsed	*/

/*Have delayed long enough for MPA to "see" Modem Reset, set up control	signals	*/
/*to address UART's IER								*/
if ( slot->type == 0x01 )				   /*PnP brd be'g Reset?*/ /*R9*/
{							   /*If yes, do't reinit*/
	marb->cont_funct      = mpa_srvc_hard_rst_cmplt;  /*brd since Hard Rst	*/
	return ;					   /*places PnP brd in	*/
}							   /*Wait for Key state	*/

	
oregs->cout_cntrl_sig = 0x111 ;				/*Addr UART's IER	*/
slot->frame_ctr       = gregs->gicp_frame_ctr ;		/*Snap shot for timing	*/
marb->cont_funct      = mpa_srvc_hard_rst_3 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*END of Hard Reset Modem - 2nd function, mpa_srvc_hard_rst_2		*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 * 	HARD RESET MODEM BOARD - 3rd FUNCTION, INITIALIZE (WRITE TO) UART's IER*
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_hard_rst_3 ( unsigned long		        caller_parm,
				   struct marb_struct FARm *  		marb,
	      			   struct ssp_struct FARm *   		sspcb,
	      			   struct slot_struct FARm *  		slot,
	      			   struct icp_global_struct FARm * 	gregs,
				   struct cin_struct FARm *	  	iregs,
	      			   struct cout_struct FARm *	  	oregs
				 )
#else
static void  mpa_srvc_hard_rst_3 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
sspcb       = sspcb ;
iregs       = iregs ;

/*Start of function's logic							*/
if ( slot->slot_state != 0xFF )				/*Board still present?	*/
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_HRD_RST_REMOVED ;	/*Set completion status	*/
	return ;
}
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
if ( frame_delay_chk ( (unsigned short int)0x100, gregs, slot ) == 0 )
	return ;					/*256 frames not elapsed*/

/*Have delayed long enough to allow non-UART based Modems to initialize and also*/
/*allowed time for MPA to "see" control signal setting indicating that output be*/
/*directed to UART's Interrupt Enable Register (IER)				*/
oregs->cout_xon_1         = 0x08 ;			/*IER=Modem Stat int ena*/
oregs->cout_flow_config  ^= 0x10 ;			/*Initiate xmit'g char	*/

/*Set up & wait till IER output sent						*/
slot->frame_ctr  = gregs->gicp_frame_ctr ;		/*Snap shot for timing	*/
marb->cont_funct = mpa_srvc_hard_rst_4 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*END of Hard Reset Modem - 3rd function, mpa_srvc_hard_rst_3		*/

/*#page*/
/*******************************************************************************
 *                                                                             *
 * 		HARD RESET MODEM BOARD - 4th FUNCTION, TOGGLE CONTROL SIGNALS  *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_hard_rst_4 ( unsigned long		        caller_parm,
				   struct marb_struct FARm *  		marb,
	      			   struct ssp_struct FARm *   		sspcb,
	      			   struct slot_struct FARm *  		slot,
	      			   struct icp_global_struct FARm * 	gregs,
				   struct cin_struct FARm *	  	iregs,
	      			   struct cout_struct FARm *	  	oregs
				 )
#else
static void  mpa_srvc_hard_rst_4 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/



/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
sspcb       = sspcb ;
iregs       = iregs ;

/*Start of function's logic							*/
if ( slot->slot_state != 0xFF )				/*Board still present?	*/
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_HRD_RST_REMOVED ;	/*Set completion status	*/
	return ;
}
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
if ( ((oregs->cout_flow_config & 0x10) != (oregs->cout_intnl_flow_ctrl & 0x10)) ||
     ((oregs->cout_intnl_flow_ctrl & 0x08) == 0x08) )	/*IER char xmit done	*/
{							/*If no, check deadman	*/
	if ( frame_delay_chk (DEADMAN_DELAY, gregs, slot) == 0x00 )
	{						/*If deadman not expired*/
		return ;
	}
	mpa_srvc_hard_rst_clean_up (slot, (unsigned char)0x00 ) ; /*Restore SSP regs	*/
	marb->srvc_status = ERR_HRD_RST_DEADMAN ; 	/*Set cmpltn status	*/
	marb->srvc_state = 0x00 ;			/*Set "complete" state	*/
	return ;
}

/*Modem's IER written to, set up to toggle control signals thus causing MCR to	*/
/* be updated									*/
oregs->cout_cntrl_sig = ((slot->sv_cout_cntrl_sig & 0x0237) | 0x0004) ^ 0x33 ;
slot->frame_ctr       = gregs->gicp_frame_ctr ;		/*Snap shot for timing	*/
marb->cont_funct      = mpa_srvc_hard_rst_5 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*END of Hard Reset Modem - 4th function, mpa_srvc_hard_rst_4		*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 * 		HARD RESET MODEM BOARD - 5th FUNCTION, TOGGLE CONTROL SIGNALS  *
 *						       (forcing MPA to wrt/init*
 *							UART's/Modem's MCR)    *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_hard_rst_5 ( unsigned long		        caller_parm,
				   struct marb_struct FARm *  		marb,
	      			   struct ssp_struct FARm *   		sspcb,
	      			   struct slot_struct FARm *  		slot,
	      			   struct icp_global_struct FARm * 	gregs,
				   struct cin_struct FARm *	  	iregs,
	      			   struct cout_struct FARm *	  	oregs
				 )
#else
static void  mpa_srvc_hard_rst_5 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/



/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
sspcb       = sspcb ;
iregs       = iregs ;

/*Start of function's logic							*/
if ( slot->slot_state != 0xFF )				/*Board still present?	*/
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_HRD_RST_REMOVED ;	/*Set completion status	*/
	return ;
}
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
if ( frame_delay_chk ( (unsigned short int)0x04, gregs, slot ) == 0 )
	return ;					/*4 frames not elapsed	*/

/*Have delayed long enough for MPA to inverse of control signals, restore	*/
/*Control Toggle signals to assure MCR updated					*/
oregs->cout_cntrl_sig = ((slot->sv_cout_cntrl_sig & 0x0237) | 0x0004) ;
slot->frame_ctr       = gregs->gicp_frame_ctr ;		/*Snap shot for timing	*/
marb->cont_funct      = mpa_srvc_hard_rst_6 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*END of Hard Reset Modem - 5th function, mpa_srvc_hard_rst_5		*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 * 	HARD RESET MODEM BOARD - 6th FUNCTION, ADDRESS UART'S LCR REGISTER     *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_hard_rst_6 ( unsigned long		        caller_parm,
				   struct marb_struct FARm *  		marb,
	      			   struct ssp_struct FARm *   		sspcb,
	      			   struct slot_struct FARm *  		slot,
	      			   struct icp_global_struct FARm * 	gregs,
				   struct cin_struct FARm *	  	iregs,
	      			   struct cout_struct FARm *	  	oregs
				 )
#else
static void  mpa_srvc_hard_rst_6 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/



/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
sspcb       = sspcb ;
iregs       = iregs ;

/*Start of function's logic							*/
if ( slot->slot_state != 0xFF )				/*Board still present?	*/
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_HRD_RST_REMOVED ;	/*Set completion status	*/
	return ;
}
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
if ( frame_delay_chk ( (unsigned short int)0x04, gregs, slot ) == 0 )
	return ;					/*4 frames not elapsed	*/

/*Have delayed long enough for MPA to inverse of control signals, set output 	*/
/*contol signals to address UART's Line Control Register (LCR)			*/
oregs->cout_cntrl_sig = 0x122 ;			/*R15*/	/*Addr UART's FCB 	*/ 
slot->frame_ctr = gregs->gicp_frame_ctr ;		/*Snap shot for timing	*/
marb->cont_funct = mpa_srvc_hard_rst_6a ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*END of Hard Reset Modem - 6th function, mpa_srvc_hard_rst_6		*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 * 	HARD RESET MODEM BOARD - 6a FUNCTION, INITIALIZE UART'S FCR REGISTER   *
 *                                                                             *
 *******************************************************************************/
/*R15*/
#ifndef 		NO_FN_PROTO_PARMS
static void  mpa_srvc_hard_rst_6a ( unsigned long		        caller_parm,
				   struct marb_struct FARm *  		marb,
	      			   struct ssp_struct FARm *   		sspcb,
	      			   struct slot_struct FARm *  		slot,
	      			   struct icp_global_struct FARm * 	gregs,
				   struct cin_struct FARm *	  	iregs,
	      			   struct cout_struct FARm *	  	oregs
				 )
#else
static void  mpa_srvc_hard_rst_6a (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

#ifndef				RAMP_LINUX
void		(FARm *sv_cont_funct)() ;		/*Saved addr of Register*/
							/*Modem request's "link"*/
							/*function		*/
#else
void            (FARm *sv_cont_funct)(
				      unsigned long int        caller_parm,
				      struct marb_struct       FARm * ,
				      struct ssp_struct        FARm * ,
				      struct slot_struct       FARm * ,
				      struct mpa_global_s      FARm * ,
				      struct mpa_input_s       FARm * ,
				      struct mpa_output_s      FARm * 
				     ) ;
#endif


/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;

/*Start of function's logic							*/
sv_cont_funct = marb->cont_funct ;			/*Save addr of "link" fn*/
mpa_srvc_reg_modem_5a ( (unsigned long)MARB_ACTV_SRVC_HARD_RESET,
                       marb,  sspcb, slot,		/*Go init FCR		*/
		       gregs, iregs, oregs ) ;
if ( (marb->srvc_state == 0x00)		   ||		/*Did error occur? or	*/
     (sv_cont_funct    == marb->cont_funct) )		/*Wait'g for frame delay*/
	return ;					/*If yes		*/

/*Set up & wait till FCR Char sent then address LCB				*/
marb->cont_funct = mpa_srvc_hard_rst_6b ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*END of Hard Reset Modem - 6a function, mpa_srvc_hard_rst_6a		*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 * 	HARD RESET MODEM BOARD - 6b FUNCTION, ADDRESS LCR REGISTER   	       *
 *                                                                             *
 *******************************************************************************/
/*R15*/
#ifndef 		NO_FN_PROTO_PARMS
static void  mpa_srvc_hard_rst_6b ( unsigned long		        caller_parm,
				   struct marb_struct FARm *  		marb,
	      			   struct ssp_struct FARm *   		sspcb,
	      			   struct slot_struct FARm *  		slot,
	      			   struct icp_global_struct FARm * 	gregs,
				   struct cin_struct FARm *	  	iregs,
	      			   struct cout_struct FARm *	  	oregs
				 )
#else
static void  mpa_srvc_hard_rst_6b (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

#ifndef				RAMP_LINUX
void		(FARm *sv_cont_funct)() ;		/*Saved addr of Register*/
							/*Modem request's "link"*/
							/*function		*/
#else
void            (FARm *sv_cont_funct)(
				      unsigned long int        caller_parm,
				      struct marb_struct       FARm * ,
				      struct ssp_struct        FARm * ,
				      struct slot_struct       FARm * ,
				      struct mpa_global_s      FARm * ,
				      struct mpa_input_s       FARm * ,
				      struct mpa_output_s      FARm * 
				     ) ;
#endif


/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;

/*Start of function's logic							*/
sv_cont_funct = marb->cont_funct ;			/*Save addr of "link" fn*/
mpa_srvc_reg_modem_5b ( (unsigned long)MARB_ACTV_SRVC_HARD_RESET,
                       marb,  sspcb, slot,		/*Go init FCR		*/
		       gregs, iregs, oregs ) ;
if ( (marb->srvc_state == 0x00)		   ||		/*Did error occur? or	*/
     (sv_cont_funct    == marb->cont_funct) )		/*Wait'g for frame delay*/
	return ;					/*If yes		*/

/*Set up & wait till LCR Char sent then address LCR				*/
marb->cont_funct = mpa_srvc_hard_rst_7 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*END of Hard Reset Modem - 6b function, mpa_srvc_hard_rst_6b		*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 * 	HARD RESET MODEM BOARD - 7th FUNCTION, INITIALIZE UART'S LCR REGISTER  *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_hard_rst_7 ( unsigned long		        caller_parm,
				   struct marb_struct FARm *  		marb,
	      			   struct ssp_struct FARm *   		sspcb,
	      			   struct slot_struct FARm *  		slot,
	      			   struct icp_global_struct FARm * 	gregs,
				   struct cin_struct FARm *	  	iregs,
	      			   struct cout_struct FARm *	  	oregs
				 )
#else
static void  mpa_srvc_hard_rst_7 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

#ifndef				RAMP_LINUX		/*R14*/
void		(FARm *sv_cont_funct)() ;		/*Saved addr of Register*/
							/*Modem request's "link"*/
							/*function		*/
#else
void            (FARm *sv_cont_funct)(
				      unsigned long int        caller_parm,
				      struct marb_struct       FARm * ,
				      struct ssp_struct        FARm * ,
				      struct slot_struct       FARm * ,
				      struct mpa_global_s      FARm * ,
				      struct mpa_input_s       FARm * ,
				      struct mpa_output_s      FARm * 
				     ) ;
#endif


/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;

/*Start of function's logic							*/
sv_cont_funct = marb->cont_funct ;			/*Save addr of "link" fn*/
mpa_srvc_reg_modem_6 ( (unsigned long)MARB_ACTV_SRVC_HARD_RESET,
                       marb,  sspcb, slot,		/*Go init LCR		*/
		       gregs, iregs, oregs ) ;
if ( (marb->srvc_state == 0x00)		   ||		/*Did error occur? or	*/
     (sv_cont_funct    == marb->cont_funct) )		/*Wait'g for frame delay*/
	return ;					/*If yes		*/

/*Set up & wait till LCR Char sent then address DLL				*/
marb->cont_funct = mpa_srvc_hard_rst_8 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*END of Hard Reset Modem - 7th function, mpa_srvc_hard_rst_7		*/
/*#page*/
/********************************************************************************
 *                                                                              *
 * 	HARD RESET MODEM BOARD - 8th FUNCTION, ADDRESS UART's DLL, DIVISOR LATCH*
 *				 	       REGISTER (Least Significant byte)*
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_hard_rst_8 ( unsigned long		        caller_parm,
				   struct marb_struct FARm *  		marb,
	      			   struct ssp_struct FARm *   		sspcb,
	      			   struct slot_struct FARm *  		slot,
	      			   struct icp_global_struct FARm * 	gregs,
				   struct cin_struct FARm *	  	iregs,
	      			   struct cout_struct FARm *	  	oregs
				 )
#else
static void  mpa_srvc_hard_rst_8 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

#ifndef				RAMP_LINUX		/*R14*/
void		(FARm *sv_cont_funct)() ;		/*Saved addr of Register*/
							/*Modem request's "link"*/
							/*function		*/
#else
void            (FARm *sv_cont_funct)(
				      unsigned long int        caller_parm,
				      struct marb_struct       FARm * ,
				      struct ssp_struct        FARm * ,
				      struct slot_struct       FARm * ,
				      struct mpa_global_s      FARm * ,
				      struct mpa_input_s       FARm * ,
				      struct mpa_output_s      FARm * 
				     ) ;
#endif


/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;

/*Start of function's logic							*/
sv_cont_funct = marb->cont_funct ;			/*Save addr of "link" fn*/
mpa_srvc_reg_modem_7 ( (unsigned long)MARB_ACTV_SRVC_HARD_RESET,
                       marb,  sspcb, slot,		/*Go set control sig's	*/
		       gregs, iregs, oregs ) ;		/*to addr DLL reg	*/
if ( (marb->srvc_state == 0x00)		   ||		/*Did error occur? or	*/
     (sv_cont_funct    == marb->cont_funct) )		/*Wait'g for frame delay*/
	return ;					/*If yes		*/

/*Set up & initialize DLL							*/
marb->cont_funct = mpa_srvc_hard_rst_9 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*END of Hard Reset Modem - 8th function, mpa_srvc_hard_rst_8		*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 * 	HARD RESET MODEM BOARD - 9th FUNCTION, INITIALIZE UART'S DLL REGISTER  *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_hard_rst_9 ( unsigned long		        caller_parm,
				   struct marb_struct FARm *  		marb,
	      			   struct ssp_struct FARm *   		sspcb,
	      			   struct slot_struct FARm *  		slot,
	      			   struct icp_global_struct FARm * 	gregs,
				   struct cin_struct FARm *	  	iregs,
	      			   struct cout_struct FARm *	  	oregs
				 )
#else
static void  mpa_srvc_hard_rst_9 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

#ifndef				RAMP_LINUX		/*R14*/
void		(FARm *sv_cont_funct)() ;		/*Saved addr of Register*/
							/*Modem request's "link"*/
							/*function		*/
#else
void            (FARm *sv_cont_funct)(
				      unsigned long int        caller_parm,
				      struct marb_struct       FARm * ,
				      struct ssp_struct        FARm * ,
				      struct slot_struct       FARm * ,
				      struct mpa_global_s      FARm * ,
				      struct mpa_input_s       FARm * ,
				      struct mpa_output_s      FARm * 
				     ) ;
#endif


/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;

/*Start of function's logic							*/
sv_cont_funct = marb->cont_funct ;			/*Save addr of "link" fn*/
mpa_srvc_reg_modem_8 ( (unsigned long)MARB_ACTV_SRVC_HARD_RESET,
                       marb,  sspcb, slot,		/*Go init DLL		*/
		       gregs, iregs, oregs ) ;
if ( (marb->srvc_state == 0x00)		   ||		/*Did error occur? or	*/
     (sv_cont_funct    == marb->cont_funct) )		/*Wait'g for frame delay*/
	return ;					/*If yes		*/

/*Set up & wait till DLL Char sent then address DLM				*/
marb->cont_funct = mpa_srvc_hard_rst_10 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*END of Hard Reset Modem - 9th function, mpa_srvc_hard_rst_9		*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *    HARD RESET MODEM BOARD - 10th FUNCTION, ADDRESS UART's DIVISOR LATCH     *
 *			REGISTER (Most Significant byte) DLM		       *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_hard_rst_10 ( unsigned long		        caller_parm,
				   struct marb_struct FARm *  		marb,
	      			   struct ssp_struct FARm *   		sspcb,
	      			   struct slot_struct FARm *  		slot,
	      			   struct icp_global_struct FARm * 	gregs,
				   struct cin_struct FARm *	  	iregs,
	      			   struct cout_struct FARm *	  	oregs
				 )
#else
static void  mpa_srvc_hard_rst_10 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

#ifndef				RAMP_LINUX		/*R14*/
void		(FARm *sv_cont_funct)() ;		/*Saved addr of Register*/
							/*Modem request's "link"*/
							/*function		*/
#else
void            (FARm *sv_cont_funct)(
				      unsigned long int        caller_parm,
				      struct marb_struct       FARm * ,
				      struct ssp_struct        FARm * ,
				      struct slot_struct       FARm * ,
				      struct mpa_global_s      FARm * ,
				      struct mpa_input_s       FARm * ,
				      struct mpa_output_s      FARm * 
				     ) ;
#endif


/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;

/*Start of function's logic							*/
sv_cont_funct = marb->cont_funct ;			/*Save addr of "link" fn*/
mpa_srvc_reg_modem_9 ( (unsigned long)MARB_ACTV_SRVC_HARD_RESET,
                       marb,  sspcb, slot,		/*Go set control sig's	*/
		       gregs, iregs, oregs ) ;		/*to addr DLM reg	*/
if ( (marb->srvc_state == 0x00)		   ||		/*Did error occur? or	*/
     (sv_cont_funct    == marb->cont_funct) )		/*Wait'g for frame delay*/
	return ;					/*If yes		*/

/*Set up & initialize DLM							*/
marb->cont_funct = mpa_srvc_hard_rst_11 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*END of Hard Reset Modem - 10th function, mpa_srvc_hard_rst_10		*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *    HARD RESET MODEM BOARD - 11th FUNCTION, INITIALIZE UART'S DLM REGISTER   *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_hard_rst_11 ( unsigned long		        caller_parm,
				   struct marb_struct FARm *  		marb,
	      			   struct ssp_struct FARm *   		sspcb,
	      			   struct slot_struct FARm *  		slot,
	      			   struct icp_global_struct FARm * 	gregs,
				   struct cin_struct FARm *	  	iregs,
	      			   struct cout_struct FARm *	  	oregs
				 )
#else
static void  mpa_srvc_hard_rst_11 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

#ifndef				RAMP_LINUX		/*R14*/
void		(FARm *sv_cont_funct)() ;		/*Saved addr of Register*/
							/*Modem request's "link"*/
							/*function		*/
#else
void            (FARm *sv_cont_funct)(
				      unsigned long int        caller_parm,
				      struct marb_struct       FARm * ,
				      struct ssp_struct        FARm * ,
				      struct slot_struct       FARm * ,
				      struct mpa_global_s      FARm * ,
				      struct mpa_input_s       FARm * ,
				      struct mpa_output_s      FARm * 
				     ) ;
#endif


/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;

/*Start of function's logic							*/
sv_cont_funct = marb->cont_funct ;			/*Save addr of "link" fn*/
mpa_srvc_reg_modem_10 ( (unsigned long)MARB_ACTV_SRVC_HARD_RESET,
                        marb,  sspcb, slot,		/*Go init DLM		*/
		        gregs, iregs, oregs ) ;
if ( (marb->srvc_state == 0x00)		   ||		/*Did error occur? or	*/
     (sv_cont_funct    == marb->cont_funct) )		/*Wait'g for frame delay*/
	return ;					/*If yes		*/

/*Set up & wait till DLM Char sent then address LCR (to clr divisor latch ena)	*/
marb->cont_funct = mpa_srvc_hard_rst_12 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*END of Hard Reset Modem - 11th function, mpa_srvc_hard_rst_11		*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *    HARD RESET MODEM BOARD - 12th FUNCTION, ADDRESS UART'S LCR REGISTER      *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_hard_rst_12 ( unsigned long		        caller_parm,
				   struct marb_struct FARm *  		marb,
	      			   struct ssp_struct FARm *   		sspcb,
	      			   struct slot_struct FARm *  		slot,
	      			   struct icp_global_struct FARm * 	gregs,
				   struct cin_struct FARm *	  	iregs,
	      			   struct cout_struct FARm *	  	oregs
				 )
#else
static void  mpa_srvc_hard_rst_12 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

#ifndef				RAMP_LINUX		/*R14*/
void		(FARm *sv_cont_funct)() ;		/*Saved addr of Register*/
							/*Modem request's "link"*/
							/*function		*/
#else
void            (FARm *sv_cont_funct)(
				      unsigned long int        caller_parm,
				      struct marb_struct       FARm * ,
				      struct ssp_struct        FARm * ,
				      struct slot_struct       FARm * ,
				      struct mpa_global_s      FARm * ,
				      struct mpa_input_s       FARm * ,
				      struct mpa_output_s      FARm * 
				     ) ;
#endif


/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;

/*Start of function's logic							*/
sv_cont_funct = marb->cont_funct ;			/*Save addr of "link" fn*/
mpa_srvc_reg_modem_11 ( (unsigned long)MARB_ACTV_SRVC_HARD_RESET,
                        marb,  sspcb, slot,		/*Go set control sig's	*/
		        gregs, iregs, oregs ) ;		/*to addr LCR reg	*/
if ( (marb->srvc_state == 0x00)		   ||		/*Did error occur? or	*/
     (sv_cont_funct    == marb->cont_funct) )		/*Wait'g for frame delay*/
	return ;					/*If yes		*/

/*Set up & initialize LCR							*/
marb->cont_funct = mpa_srvc_hard_rst_13 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*END of Hard Reset Modem - 12th function, mpa_srvc_hard_rst_12		*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *  HARD RESET MODEM BOARD - 13th FUNCTION, CLEAR UART'S LCR REGISTER DIVISOR  *
 *                                          LATCH ENABLE BIT		       *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_hard_rst_13 ( unsigned long		        caller_parm,
				   struct marb_struct FARm *  		marb,
	      			   struct ssp_struct FARm *   		sspcb,
	      			   struct slot_struct FARm *  		slot,
	      			   struct icp_global_struct FARm * 	gregs,
				   struct cin_struct FARm *	  	iregs,
	      			   struct cout_struct FARm *	  	oregs
				 )
#else
static void  mpa_srvc_hard_rst_13 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

#ifndef				RAMP_LINUX		/*R14*/
void		(FARm *sv_cont_funct)() ;		/*Saved addr of Register*/
							/*Modem request's "link"*/
							/*function		*/
#else
void            (FARm *sv_cont_funct)(
				      unsigned long int        caller_parm,
				      struct marb_struct       FARm * ,
				      struct ssp_struct        FARm * ,
				      struct slot_struct       FARm * ,
				      struct mpa_global_s      FARm * ,
				      struct mpa_input_s       FARm * ,
				      struct mpa_output_s      FARm * 
				     ) ;
#endif


/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
sspcb       = sspcb ;

/*Start of function's logic							*/
sv_cont_funct = marb->cont_funct ;			/*Save addr of "link" fn*/
mpa_srvc_reg_modem_12 ( (unsigned long)MARB_ACTV_SRVC_HARD_RESET,
                        marb,  sspcb, slot,		/*Go init LCR		*/
		        gregs, iregs, oregs ) ;
if ( (marb->srvc_state == 0x00)		   ||		/*Did error occur? or	*/
     (sv_cont_funct    == marb->cont_funct) )		/*Wait'g for frame delay*/
	return ;					/*If yes		*/

/*Set up & wait till LCR Char sent then set slot's/chan's control signals for	*/
/*normal operation								*/
marb->cont_funct = mpa_srvc_hard_rst_14 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*END of Hard Reset Modem - 13th function, mpa_srvc_hard_rst_13		*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *  		HARD RESET MODEM BOARD - 14th FUNCTION, FLUSH MPA FIFOs	       *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_hard_rst_14 ( unsigned long		        caller_parm,
				   struct marb_struct FARm *  		marb,
	      			   struct ssp_struct FARm *   		sspcb,
	      			   struct slot_struct FARm *  		slot,
	      			   struct icp_global_struct FARm * 	gregs,
				   struct cin_struct FARm *	  	iregs,
	      			   struct cout_struct FARm *	  	oregs
				 )
#else
static void  mpa_srvc_hard_rst_14 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/



/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
sspcb       = sspcb ;
iregs       = iregs ;

/*Start of function's logic							*/
if ( slot->slot_state != 0xFF )				/*Is Brd still present?	*/
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_HRD_RST_REMOVED ;	/*Set completion status	*/
	return ;
}
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
if ( ((oregs->cout_flow_config & 0x10) != (oregs->cout_intnl_flow_ctrl & 0x10)) ||
     ((oregs->cout_intnl_flow_ctrl & 0x08) == 0x08) )	/*LCR char xmit done	*/
{							/*If no, check deadman	*/
	if ( frame_delay_chk (DEADMAN_DELAY, gregs, slot) == 0x00 )
	{						/*If deadman not expired*/
		return ;
	}
	mpa_srvc_hard_rst_clean_up (slot, (unsigned char)0x00 ) ; /*Restore SSP regs	*/
	marb->srvc_status = ERR_HRD_RST_DEADMAN ; 	/*Set cmpltn status	*/
	marb->srvc_state = 0x00 ;			/*Set "complete" state	*/
	return ;
}

/*Modem's LCR written to, place MPA into Reset (FLUSH) state thus flushing	*/
/*internal input/output buffers							*/
oregs->cout_cntrl_sig = 0x188 ;				/*MPA's Slot Reset	*/
slot->read_toggle     = 0x00 ;				/*UART I/O Read toggle	*/
slot->frame_ctr  = gregs->gicp_frame_ctr ;		/*Snap shot for timing	*/
marb->cont_funct = mpa_srvc_hard_rst_15 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*END of Hard Reset Modem - 14th function, mpa_srvc_hard_rst_14		*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *  		HARD RESET MODEM BOARD - 15th FUNCTION, SET CONTROL SIGNALS    *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_hard_rst_15 ( unsigned long		        caller_parm,
				   struct marb_struct FARm *  		marb,
	      			   struct ssp_struct FARm *   		sspcb,
	      			   struct slot_struct FARm *  		slot,
	      			   struct icp_global_struct FARm * 	gregs,
				   struct cin_struct FARm *	  	iregs,
	      			   struct cout_struct FARm *	  	oregs
				 )
#else
static void  mpa_srvc_hard_rst_15 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/




/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
sspcb       = sspcb ;
iregs       = iregs ;

/*Start of function's logic							*/
if ( slot->slot_state != 0xFF )				/*Is Brd still present?	*/
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_HRD_RST_REMOVED ;	/*Set completion status	*/
	return ;
}
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
if ( frame_delay_chk ( (unsigned short int)0x08, gregs, slot ) == 0 )
	return ;					/*8 frames not elapsed	*/

/*Have delayed long enough for MPA to "see" control signal setting indicating	*/
/*that Slot should be reset, set Slot's control signals for normal operation	*/
oregs->cout_cntrl_sig = (slot->sv_cout_cntrl_sig & 0x0237) | 0x0004 ;
slot->frame_ctr = gregs->gicp_frame_ctr ;		/*Snap shot for timing	*/
marb->cont_funct = mpa_srvc_hard_rst_cmplt ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*END of Hard Reset Modem - 15th function, mpa_srvc_hard_rst_15		*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *  	      HARD RESET MODEM BOARD - 16th FUNCTION, REQUEST COMPLETE	       *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_hard_rst_cmplt ( unsigned long		        caller_parm, /*R9*/
				       struct marb_struct FARm *  	marb,
	      			       struct ssp_struct FARm *   	sspcb,
	      			       struct slot_struct FARm *  	slot,
	      			       struct icp_global_struct FARm * 	gregs,
				       struct cin_struct FARm *	  	iregs,
	      			       struct cout_struct FARm *	oregs
				     )
#else
static void  mpa_srvc_hard_rst_cmplt (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/



/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
sspcb       = sspcb ;
iregs       = iregs ;
oregs       = oregs ;

/*Start of function's logic							*/
if ( slot->slot_state != 0xFF )				/*Is Brd still present?	*/
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_HRD_RST_REMOVED ;	/*Set completion status	*/
	return ;
}
if ( frame_delay_chk ( (unsigned short int)0x04, gregs, slot ) == 0 )
{
	marb->srvc_state = 0xFF ;			/*Set "call back" state	*/
	return ;					/*4 frames not elapsed	*/
}

/*Have delayed long enough for MPA to "see" control signals, complete request	*/
mpa_srvc_hard_rst_clean_up (slot, (unsigned char)0x00 ) ; /*Restore SSP regs	*/
marb->srvc_status = 0x00 ;				/*Set completion status	*/
return ;						/*Exit, Request Complete*/

}	/*END of Hard Reset Modem - 16th function, mpa_srvc_hard_rst_cmplt		*/






/*#stitle SLOT SERVICE - HARD RESET MODEM BOARD, LOCAL FUNCTION#*/
/*#page*/
/****************************************************************************************
 *                                                                             		*
 *     	     HARD RESET MODEM BOARD - Clean Up SSP (mpa_srvc_hard_rst_clean_up)		*
 *                                                                     			*
 * PURPOSE: To restore state of SSP registers when service has been completed (either 	*
 *	   due to error or successful completion)					*
 *											*
 * NOTES:										*
 *                                                                     			*
 * CALL: void  mpa_srvc_hard_rst_clean_up (struct slot_struct FARm *    slot,		*
 *			     	           unsigned char		type)		*
 *                                                                     			*
 *			where:	slot = Slot Control Block pointer			*
 *				type = Clean up type:					*
 *					o 0x00 = Normal i.e. Reg Modem request cmplt	*
 *					o 0xFF = Abnormal i.e. Reg Modem request	*
 *						 aborted due to some failure or Board	*
 *						 removed				*
 *                                                                     			*
 *                                                                     			*
 * RETURN: None										*
 *                                                                     			*
 ****************************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void	mpa_srvc_hard_rst_clean_up (struct slot_struct FARm *    slot,
					    unsigned char	         type)
#else
static void	mpa_srvc_hard_rst_clean_up (slot, type)

struct slot_struct FARm *  slot;
unsigned char	       	   type;
#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

/*SSP variables relating to MARB's/Request's specific SLOT (Channel)		*/
#ifndef					RAMP_BSDI		/*R14*/
struct cout_struct FARm *	  	oregs ;		/*Slot's SSP Output regs	   */
#else								/*BSDI Compile		   */
volatile struct cout_struct FARm *	oregs ;		/*Slot's SSP Output regs	   */
#endif								/*End RAMP_BSDI Conditional*/



/************************
 *   START OF CODE	*
 ************************/

oregs = slot->output ;					/*Slot's output regs ptr*/

/*Delay if actually canceling/aborting an outstanding Hard Reset MARB, thus 	*/
/*allowing for MPA to handle Reset						*/
if ( (type == 0xFF) && (slot->actv_marb->srvc_status == MARB_OUT_CANCELED) )
{
	if ( (oregs->cout_cntrl_sig == 0x188) ||	/*State of MPA = Flush	*/
	     (oregs->cout_cntrl_sig == 0x1CC) )		/*   OR Hard Reset	*/
	{						/*If yes		*/
	     /*If abort'g MARB, delay to give MPA chance to FLUSH or Hard Reset	*/
		while ( frame_delay_chk ( (unsigned short int)0x08,
					  slot->actv_marb->sspcb->global, slot)	== 0 )
		{ }
	}
}

/*Restore saved SSP registers							*/
oregs->cout_flow_config = (slot->sv_cout_flow_config & 0x2F) |  /*Rstr out sigs */
                          (oregs->cout_flow_config & 0x10) | 0x40 ;
oregs->cout_cntrl_sig   = (slot->sv_cout_cntrl_sig & 0x0237) | 0x0004 ;
oregs->cout_xon_1	= slot->sv_cout_xon_1 ;		/*Restore XON char	*/
#ifndef 				NO_FN_PROTO_PARMS /*Compile if Protypes OK*/
slot->clean_up          = (void (*)(struct slot_struct FARm *, unsigned char))0 ;
#else							  /*Compile if NO Protypes*/
slot->clean_up          = (void (*)())0 ;
#endif
return ;

}					/*End of mpa_srvc_hard_rst_clean_up	*/







/*#stitle EXCEPTION SERVICE - INPUT ERROR (mpa_srvc_input_error)#*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *                           INPUT ERROR - SERVICE ENTRY POINT                 *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
extern unsigned short int	mpa_srvc_input_error (struct marb_struct FARm *  marb,
						      unsigned char FARm         error_descrpt[] )
#else
extern unsigned short int	mpa_srvc_input_error (marb, error_descrpt)

struct marb_struct FARm *  marb;
unsigned char FARm         error_descrpt[] ;

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

/*General working variables							*/
unsigned char		status ;			/*Funct. ret status	*/
unsigned char		sv_sup_uchar3 ;			/*Saved MARB sup_uchar3	*/
unsigned short int	ret_status ;			/*Completion status	*/



/************************
 *   START OF CODE	*
 ************************/

status = val_marb (marb) ;				/*Validate caller's MARB*/
if ( status != 0 )					/*Is MARB OK to use?	*/
	return (ERR_SLOT_ERROR_MARB) ;			/*If no, return error	*/
if ( (marb->in_use != 0) || (marb->req_type != 0x00) )	/*Init MARB Req & Waitd?*/
	return (ERR_SLOT_ERROR_MARB) ;			/*If no, return error	*/

/*MARB OK, attempt to determine error						*/
sv_sup_uchar3 = marb->sup_uchar3 ;			/*Save 			*/
marb->cont_funct = mpa_srvc_error_1 ;			/*Init MARB's FN ptr	*/
ret_status       = async_req_hndlr(marb) ;		/*Go handle request	*/
if ( ret_status == 0 )					/*Is Request OK?	*/
{							/*If yes, set ret parms	*/
	error_descrpt[0] = marb->sup_uchar3 ;		/*Error type		*/
	error_descrpt[1] = marb->sup_uchar0 ;		/*Errored character	*/
}
marb->sup_uchar3 = sv_sup_uchar3 ;			/*Restore		*/
return (ret_status) ;					/*Request complete, exit*/
}			/*END of mpa_srvc_input_error Service Entry function	*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *    INPUT ERROR HANDLING - 1st (and only) Function, Evaluate Error String    *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_error_1 ( unsigned long		        caller_parm,
				struct marb_struct FARm *  	marb,
	      			struct ssp_struct FARm *   	sspcb,
	      			struct slot_struct FARm *  	slot,
	      			struct icp_global_struct FARm * gregs,
				struct cin_struct FARm *	iregs,
	      			struct cout_struct FARm *	oregs
				)
#else
static void  mpa_srvc_error_1 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		   	caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

unsigned char		lsr ;				/*Byte describing error	*/
							/*UART's LSR register	*/

unsigned char		i ;				/*Counter used to determ*/
							/*which LSR bit set	*/


/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
sspcb       = sspcb ;
gregs       = gregs ;
iregs       = iregs ;
oregs       = oregs ;

/*Start of function's logic							*/
if ( slot->slot_state != 0xFF )				/*Is Slot occupied?	*/
{							/*If no, error		*/
	marb->srvc_status = ERR_SLOT_ERROR_NOT ;	/*Indicate error	*/
	return ;					/*Exit			*/
}
if ( slot->init_state != 0xFF )				/*Is Slot init'd/opn'd?	*/
{							/*If no, error		*/
	marb->srvc_status = ERR_SLOT_ERROR_NOPN ;	/*Indicate error	*/
	return ;					/*Exit			*/
}

/*Slot Openned, evaluate requestor's string					*/
lsr = marb->sup_uchar1 ;				/*Get LSR reg		*/
for ( i = 4; i != 0; --i )				/*Determ which error bit*/
{							/*is set		*/
	if ( (lsr & 0x20) == 0x20 )			/*Is error bit set?	*/
		break ;					/*If yes		*/
	lsr <<= 1 ;					/*Shift next error bit	*/
}
if ( i == 0 )						/*Is an error bit set?	*/
{							/*If no, error		*/
	marb->srvc_status = ERR_SLOT_ERROR_STRNG ;	/*Indicate error	*/
	return ;					/*Exit			*/
}
marb->sup_uchar3 = i - 1 ;				/*Pass error type back	*/
marb->srvc_status = 0x00 ;				/*Set completion status	*/
return ;						/*Exit, Request Complete*/
}	/*End of INPUT ERROR HANDLER - 1st Function, mpa_srvc_error_1		*/











/*#stitle EXCEPTION SERVICE - START BREAK (mpa_srvc_start_break)#*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *                           START BREAK - SERVICE ENTRY POINT                 *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
extern unsigned short int   mpa_srvc_start_break (struct marb_struct FARm *  marb)
#else
extern unsigned short int   mpa_srvc_start_break (marb)

struct marb_struct FARm  *marb;
#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

/*General working variables							*/
unsigned char		status ;			/*Funct. ret status	*/



/************************
 *   START OF CODE	*
 ************************/

status = val_marb (marb) ;				/*Validate caller's MARB*/
if ( status != 0 )					/*Is MARB OK to use?	*/
	return (ERR_START_BREAK_MARB) ;			/*If no, return error	*/
if ( marb->in_use == 0 )				/*Initial MARB Request?	*/
{							/*If yes, init Fn ptr	*/
	marb->gate = 0x00 ;				/*Set 1st call flag/gate*/
	marb->cont_funct = mpa_srvc_start_break_1 ;	/*Init MARB's FN ptr	*/
}
else							/*"Call Back" MARB,	*/
{							/*Validate SRVC active	*/
	if ( marb->actv_srvc != MARB_ACTV_SRVC_START_BREAK ) /*Valid "Call Back"?*/
	{						/*If no, exit		*/
		marb->srvc_status = ERR_START_BREAK_CALL_BK ;
		marb->srvc_state  = 0x00 ;
		return ((unsigned short int)(((unsigned short int)(marb->srvc_state)<<8)+
                                               marb->srvc_status) ) ;
	}
}
return ( async_req_hndlr(marb) ) ;			/*Go handle request	*/


}			/*END of mpa_srvc_start_break Service Entry function	*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *    	        START BREAK - 1st Function, ADDRESS UART'S LCR REGISTER	       *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_start_break_1  ( unsigned long		        caller_parm,
				       struct marb_struct FARm *  	marb,
	      			       struct ssp_struct FARm *   	sspcb,
	      			       struct slot_struct FARm *  	slot,
	      			       struct icp_global_struct FARm *  gregs,
				       struct cin_struct FARm *		iregs,
	      			       struct cout_struct FARm *	oregs
				     )
#else
static void  mpa_srvc_start_break_1 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

#ifndef				RAMP_LINUX		/*R14*/
void		(FARm *sv_cont_funct)() ;		/*Saved addr of Register*/
							/*Modem request's "link"*/
							/*function		*/
#else
void            (FARm *sv_cont_funct)(
				      unsigned long int        caller_parm,
				      struct marb_struct       FARm * ,
				      struct ssp_struct        FARm * ,
				      struct slot_struct       FARm * ,
				      struct mpa_global_s      FARm * ,
				      struct mpa_input_s       FARm * ,
				      struct mpa_output_s      FARm * 
				     ) ;
#endif



/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;

/*Start of function's logic							*/
if ( (marb->gate == 0x00) && (slot->req_outstnd != 0x00) ) /*Req Outstand'g?	*/
{							/*If yes, return error	*/
	marb->srvc_status = ERR_START_BREAK_REQ_OUT ;	/*Indicate error	*/
	return ;					/*Exit			*/
}
if ( slot->slot_state != 0xFF )				/*Is Modem registered?	*/
{							/*If no, error		*/
	marb->srvc_status = ERR_START_BREAK_NOT_REG ;	/*Indicate error	*/
	return ;					/*Exit			*/
}
if ( slot->init_state != 0xFF )				/*Is Slot init'd/opn'd?	*/
{							/*If no, error		*/
	marb->srvc_status = ERR_START_BREAK_NOPN ;	/*Indicate error	*/
	return ;					/*Exit			*/
}

/*Slot Openned, if 1st call to function save SSP registers that will be used to	*/
/*access LCR register and establish "clean up" function in case De-register	*/
/*Modem request handled asynchronously						*/
if ( marb->gate == 0x00 )				/*1st time Fn called?	*/
{							/*If yes		*/
	slot->sv_cout_flow_config = oregs->cout_flow_config ;
	slot->sv_cout_cntrl_sig   = oregs->cout_cntrl_sig ;
	slot->sv_cout_xon_1       = oregs->cout_xon_1 ;
#ifndef 				NO_FN_PROTO_PARMS /*Compile if Protypes OK*/
	slot->clean_up   = (void (*)(struct slot_struct FARm *, unsigned char))mpa_srvc_break_clean_up ;
#else						  	  /*Compile if NO Protypes*/
	slot->clean_up   = (void (*)())mpa_srvc_break_clean_up ;
#endif
	slot->frame_ctr  = gregs->gicp_frame_ctr ;	/*Snap shot for timing	*/
	marb->actv_srvc  = MARB_ACTV_SRVC_START_BREAK ;	/*Indicate outstnd'g req*/
	marb->gate = 0xFF ;				/*Set for repeat call	*/
	oregs->cout_flow_config = (oregs->cout_flow_config & 0x10) | 0x40 | 0x08 ;
}

/*Set up to address LCR register so can turn BREAK ON				*/
sv_cont_funct = marb->cont_funct ;			/*Save addr of "link" fn*/
mpa_srvc_reg_modem_11 ( (unsigned long)MARB_ACTV_SRVC_START_BREAK,
                       marb,  sspcb, slot,		/*Go set control sig's	*/
		       gregs, iregs, oregs ) ;		/*to addr LCR reg	*/
if ( (marb->srvc_state == 0x00)		   ||		/*Did error occur? or	*/
     (sv_cont_funct    == marb->cont_funct) )		/*Wait'g for frame delay*/
	return ;					/*If yes		*/

/*Set up & initialize LCR							*/
marb->sup_uchar2 = 0x40 ;				/*Indicate SET BREAK	*/
marb->cont_funct = mpa_srvc_break_2 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/


}		/*End of START BREAK - 1st Function, mpa_srvc_start_break_1	*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *   	     BREAK Handling - 2nd Function, INITIALIZE UART'S LCR REGISTER     *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_break_2 ( unsigned long		        caller_parm,
				struct marb_struct FARm *  	marb,
	      			struct ssp_struct FARm *   	sspcb,
	      			struct slot_struct FARm *  	slot,
	      			struct icp_global_struct FARm * gregs,
				struct cin_struct FARm *	iregs,
	      			struct cout_struct FARm *	oregs
			       )
#else
static void  mpa_srvc_break_2 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		   	caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;
#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

#ifndef				RAMP_LINUX		/*R14*/
void		(FARm *sv_cont_funct)() ;		/*Saved addr of Register*/
							/*Modem request's "link"*/
							/*function		*/
#else
void            (FARm *sv_cont_funct)(
				      unsigned long int        caller_parm,
				      struct marb_struct       FARm * ,
				      struct ssp_struct        FARm * ,
				      struct slot_struct       FARm * ,
				      struct mpa_global_s      FARm * ,
				      struct mpa_input_s       FARm * ,
				      struct mpa_output_s      FARm * 
				     ) ;
#endif

unsigned char	sv_break_state ;			/*Slot's svd BREAK State*/


/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;

/*Start of function's logic							*/
sv_break_state    = slot->break_state ;			/*Sv in case need restre*/
slot->break_state = marb->sup_uchar2 ;			/*Set BREAK State	*/
sv_cont_funct = marb->cont_funct ;			/*Save addr of "link" fn*/
mpa_srvc_reg_modem_12 ( (unsigned long)MARB_ACTV_SRVC_START_BREAK,
                       marb,  sspcb, slot,		/*Go init LCR		*/
		       gregs, iregs, oregs ) ;
if ( (marb->srvc_state == 0x00)		   ||		/*Did error occur? or	*/
     (sv_cont_funct    == marb->cont_funct) )		/*Wait'g for frame delay*/
{							/*If yes		*/
	slot->break_state = sv_break_state ;		/*Restore BREAK State	*/
	return ;					/*If yes		*/
}

/*Set up & wait till LCR Char sent						*/
marb->cont_funct = mpa_srvc_break_3 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*End of START Handling - 2nd FUNCTION, mpa_srvc_break_2		*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *   	          BREAK Handling - 3rd Function, REQUEST COMPLETE	       *
 *                                                                             *
 *******************************************************************************/

#ifndef			NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_break_3  ( unsigned long		        caller_parm,
				 struct marb_struct FARm *  	marb,
	      			 struct ssp_struct FARm *   	sspcb,
	      			 struct slot_struct FARm *  	slot,
	      			 struct icp_global_struct FARm * gregs,
				 struct cin_struct FARm *	 iregs,
	      			 struct cout_struct FARm *	 oregs
				)
#else
static void  mpa_srvc_break_3 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		   	caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/



/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
sspcb       = sspcb ;
iregs       = iregs ;

/*Start of function's logic							*/
if ( slot->slot_state != 0xFF )				/*Is Brd still present?	*/
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_START_BREAK_REMOVED ;	/*Set completion status	*/
	return ;
}
if ( ((oregs->cout_flow_config & 0x10) != (oregs->cout_intnl_flow_ctrl & 0x10)) ||
     ((oregs->cout_intnl_flow_ctrl & 0x08) == 0x08) )	/*LCR char xmit done	*/
{							/*If no, check deadman	*/
	if ( frame_delay_chk (DEADMAN_DELAY, gregs, slot) == 0x00 )
	{						/*If deadman not expired*/
		marb->srvc_state = 0xFF ;		/*Set "call back" state	*/
		return ;
	}
	(*slot->clean_up)(slot, (unsigned char)0xFF) ;
	marb->srvc_status = ERR_START_BREAK_DEADMAN ; 	/*Set cmpltn status	*/
	return ;
}

/*Modem's LCR written to, complete request					*/
(*slot->clean_up)(slot, (unsigned char)0x00) ;		/*Restore SSP regs	*/
marb->srvc_status = 0x00 ;				/*Set completion status	*/
return ;						/*Exit, Request Complete*/

}	/*End of BREAK Handling - 3rd FUNCTION, mpa_srvc_break_3		*/








/*#stitle EXCEPTION SERVICE - BREAK Handling, LOCAL FUNCTION#*/
/*#page*/
/****************************************************************************************
 *                                                                             		*
 *       BREAK Handling, LOCAL FUNCTION - Clean Up SSP (mpa_srvc_break_clean_up)	*
 *                                                                     			*
 * PURPOSE: To restore state of SSP registers when service has been completed (either 	*
 *	   due to error or successful completion)					*
 *											*
 * NOTES: - Function is used by both Start & Stop BREAK					*
 *                                                                     			*
 * CALL: void  mpa_srvc_break_clean_up  (struct slot_struct FARm *  slot,		*
 *					 unsigned char		    type)		*
 *                                                                     			*
 *			where:	slot = Slot Control Block pointer			*
 *				type = Clean up type:					*
 *					o 0x00 = Normal i.e. Reg Modem request cmplt	*
 *					o 0xFF = Abnormal i.e. Reg Modem request	*
 *						 aborted due to some failure or Board	*
 *						 removed				*
 *                                                                     			*
 *                                                                     			*
 * RETURN: None										*
 *                                                                     			*
 ****************************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void	mpa_srvc_break_clean_up (struct slot_struct FARm *  slot,
					 unsigned char		    type)
#else
static void	mpa_srvc_break_clean_up (slot, type)

struct slot_struct FARm *  slot;
unsigned char	       	   type;
#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

/*SSP variables relating to MARB's/Request's specific SLOT (Channel)		*/
#ifndef					RAMP_BSDI		/*R14*/
struct cout_struct FARm *	  	oregs ;		/*Slot's SSP Output regs	   */
#else								/*BSDI Compile		   */
volatile struct cout_struct FARm *	oregs ;		/*Slot's SSP Output regs	   */
#endif								/*End RAMP_BSDI Conditional*/



/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
type = type ;

/*Restore saved SSP registers							*/
oregs = slot->output ;					/*Slot's output regs ptr*/
oregs->cout_flow_config = (slot->sv_cout_flow_config & 0x2F) |  /*Rstr out sigs */
                          (oregs->cout_flow_config & 0x10) | 0x40 ;
oregs->cout_cntrl_sig   = (slot->sv_cout_cntrl_sig & 0x02BB) | 0x0004 ;
oregs->cout_xon_1	= slot->sv_cout_xon_1 ;		/*Restore XON char	*/
#ifndef 				NO_FN_PROTO_PARMS /*Compile if Protypes OK*/
slot->clean_up          = (void (*)(struct slot_struct FARm *, unsigned char))0 ;
#else							  /*Compile if NO Protypes*/
slot->clean_up          = (void (*)())0 ;
#endif
return ;

}					/*End mpa_srvc_break_clean_up		*/











/*#stitle EXCEPTION SERVICE - STOP BREAK (mpa_srvc_stop_break)#*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *                            STOP BREAK - SERVICE ENTRY POINT                 *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
extern unsigned short int   mpa_srvc_stop_break (struct marb_struct FARm *  marb)
#else
extern unsigned short int   mpa_srvc_stop_break (marb)

struct marb_struct FARm  *marb;
#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

/*General working variables							*/
unsigned char		status ;			/*Funct. ret status	*/



/************************
 *   START OF CODE	*
 ************************/

status = val_marb (marb) ;				/*Validate caller's MARB*/
if ( status != 0 )					/*Is MARB OK to use?	*/
	return (ERR_STOP_BREAK_MARB) ;			/*If no, return error	*/
if ( marb->in_use == 0 )				/*Initial MARB Request?	*/
{							/*If yes, init Fn ptr	*/
	marb->gate = 0x00 ;				/*Set 1st call flag	*/
	marb->cont_funct = mpa_srvc_stop_break_1 ;	/*Init MARB's FN ptr	*/
}
else							/*"Call Back" MARB,	*/
{							/*Validate SRVC active	*/
	if ( marb->actv_srvc != MARB_ACTV_SRVC_STOP_BREAK ) /*Valid "Call Back"?*/
	{						/*If no, exit		*/
		marb->srvc_status = ERR_STOP_BREAK_CALL_BK ;
		marb->srvc_state  = 0x00 ;
		return ((unsigned short int)(((unsigned short int)(marb->srvc_state)<<8)+
                                               marb->srvc_status) ) ;
	}
}
return ( async_req_hndlr(marb) ) ;			/*Go handle request	*/


}			/*END of mpa_srvc_stop_break Service Entry function	*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *    	        STOP BREAK - 1st Function, ADDRESS UART'S LCR REGISTER	       *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_stop_break_1  ( unsigned long		        caller_parm,
				      struct marb_struct FARm *  	marb,
	      			      struct ssp_struct FARm *   	sspcb,
	      			      struct slot_struct FARm *  	slot,
	      			      struct icp_global_struct FARm *   gregs,
				      struct cin_struct FARm *		iregs,
	      			      struct cout_struct FARm *		oregs
				     )
#else
static void  mpa_srvc_stop_break_1 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

#ifndef				RAMP_LINUX		/*R14*/
void		(FARm *sv_cont_funct)() ;		/*Saved addr of Register*/
							/*Modem request's "link"*/
							/*function		*/
#else
void            (FARm *sv_cont_funct)(
				      unsigned long int        caller_parm,
				      struct marb_struct       FARm * ,
				      struct ssp_struct        FARm * ,
				      struct slot_struct       FARm * ,
				      struct mpa_global_s      FARm * ,
				      struct mpa_input_s       FARm * ,
				      struct mpa_output_s      FARm * 
				     ) ;
#endif



/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;

/*Start of function's logic							*/
if ( (marb->gate == 0x00) && (slot->req_outstnd != 0x00) )  /*Req Outstand'g?	*/
{							/*If yes, return error	*/
	marb->srvc_status = ERR_STOP_BREAK_REQ_OUT ;	/*Indicate error	*/
	return ;					/*Exit			*/
}
if ( slot->slot_state != 0xFF )				/*Is Modem registered?	*/
{							/*If no, error		*/
	marb->srvc_status = ERR_STOP_BREAK_NOT_REG ;	/*Indicate error	*/
	return ;					/*Exit			*/
}
if ( slot->init_state != 0xFF )				/*Is Slot init'd/opn'd?	*/
{							/*If no, error		*/
	marb->srvc_status = ERR_STOP_BREAK_NOPN ;	/*Indicate error	*/
	return ;					/*Exit			*/
}

/*Slot Openned, if 1st call to function save SSP registers that will be used to	*/
/*access LCR register and establish "clean up" function in case De-register	*/
/*Modem request handled asynchronously						*/
if ( marb->gate == 0x00 )				/*1st time Fn called?	*/
{							/*If yes		*/
	slot->sv_cout_flow_config = oregs->cout_flow_config ;
	slot->sv_cout_cntrl_sig   = oregs->cout_cntrl_sig ;
	slot->sv_cout_xon_1       = oregs->cout_xon_1 ;
#ifndef 				NO_FN_PROTO_PARMS /*Compile if Protypes OK*/
	slot->clean_up            = (void (*)(struct slot_struct FARm *, unsigned char))mpa_srvc_break_clean_up ;
#else							  /*Compile if NO Protypes*/
	slot->clean_up            = (void (*)())mpa_srvc_break_clean_up ;
#endif
	slot->frame_ctr  = gregs->gicp_frame_ctr ;	/*Snap shot for timing	*/
	marb->actv_srvc  = MARB_ACTV_SRVC_STOP_BREAK ;	/*Indicate outstnd'g req*/
	marb->gate = 0xFF ;				/*Set for repeat call	*/
	oregs->cout_flow_config = (oregs->cout_flow_config & 0x10) | 0x40 | 0x08 ;
}

/*Set up to address LCR register so can turn BREAK ON				*/
sv_cont_funct = marb->cont_funct ;			/*Save addr of "link" fn*/
mpa_srvc_reg_modem_11 ( (unsigned long)MARB_ACTV_SRVC_STOP_BREAK,
                       marb,  sspcb, slot,		/*Go set control sig's	*/
		       gregs, iregs, oregs ) ;		/*to addr LCR reg	*/
if ( (marb->srvc_state == 0x00)		   ||		/*Did error occur? or	*/
     (sv_cont_funct    == marb->cont_funct) )		/*Wait'g for frame delay*/
	return ;					/*If yes		*/

/*Set up & initialize LCR							*/
marb->sup_uchar2 = 0x00 ;				/*Indicate CLEAR BREAK	*/
marb->cont_funct = mpa_srvc_break_2 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/


}		/*End of STOP BREAK - 1st Function, mpa_srvc_stop_break_1	*/









/*#stitle EXCEPTION SERVICE - MPA<->UART HRDWRE FLOW CONTROL (mpa_srvc_mpa_flow)#*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *              MPA<->UART HARDWARE FLOW CONTROL - SERVICE ENTRY POINT          *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
extern unsigned short int   mpa_srvc_mpa_flow (struct marb_struct FARm  *marb)
#else
extern unsigned short int   mpa_srvc_mpa_flow (marb)

struct marb_struct FARm  *marb;
#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

/*General working variables							*/
unsigned char		status ;			/*Funct. ret status	*/



/************************
 *   START OF CODE	*
 ************************/

status = val_marb (marb) ;				/*Validate caller's MARB*/
if ( status != 0 )					/*Is MARB OK to use?	*/
	return (ERR_MPA_FLOW_MARB) ;			/*If no, return error	*/
if ( marb->in_use == 0 )				/*Initial MARB Request?	*/
{							/*If yes, init Fn ptr	*/
	marb->gate = 0x00 ;				/*Set 1st call flag/gate*/
	marb->cont_funct = mpa_srvc_mpa_flow_1 ;	/*Init MARB's FN ptr	*/
}
else							/*"Call Back" MARB,	*/
{							/*Validate SRVC active	*/
	if ( marb->actv_srvc != MARB_ACTV_SRVC_MPA_FLOW ) /*Valid "Call Back"?  */
	{						/*If no, exit		*/
		marb->srvc_status = ERR_MPA_FLOW_CALL_BK ;
		marb->srvc_state  = 0x00 ;
		return ((unsigned short int)(((unsigned short int)(marb->srvc_state)<<8)+
                                               marb->srvc_status) ) ;
	}
}
return ( async_req_hndlr(marb) ) ;			/*Go handle request	*/


}			/*END of mpa_srvc_mpa_flow Service Entry function	*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *      MPA<->UART HARDWARE FLOW CONTROL - 1st FUNCTION, ADDRESS MPA AUX Reg   *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_mpa_flow_1  ( unsigned long		        caller_parm,
				    struct marb_struct FARm *  		marb,
	      			    struct ssp_struct FARm *   		sspcb,
	      			    struct slot_struct FARm *  		slot,
	      			    struct icp_global_struct FARm * 	gregs,
				    struct cin_struct FARm *	  	iregs,
	      			    struct cout_struct FARm *	  	oregs
				  )
#else
static void  mpa_srvc_mpa_flow_1 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/





/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
sspcb       = sspcb ;
iregs       = iregs ;

/*Start of function's logic							*/
if ( (marb->gate == 0x00) && (slot->req_outstnd != 0x00) ) /*Req Outstand'g?	*/
{							/*If yes, return error	*/
	marb->srvc_status = ERR_MPA_FLOW_REQ_OUT ;	/*Indicate error	*/
	return ;					/*Exit			*/
}
if ( slot->slot_state != 0xFF )				/*Is Modem registered?	*/
{							/*If no, error		*/
	marb->srvc_status = ERR_MPA_FLOW_NOT_REG ;	/*Indicate error	*/
	return ;					/*Exit			*/
}
if ( slot->init_state != 0xFF )				/*Is Slot init'd/opn'd?	*/
{							/*If no, error		*/
	marb->srvc_status = ERR_MPA_FLOW_NOPN ;		/*Indicate error	*/
	return ;					/*Exit			*/
}
if ( (marb->sup_uchar0 & 0xC0) != 0 )			/*Is Flow parm sane?	*/
{							/*If no, error		*/
	marb->srvc_status = ERR_MPA_FLOW_BAD_PARM ;	/*Indicate error	*/
	return ;					/*Exit			*/
}
	
/*Caller's request valid, if 1st call initialize MARB's Slot Control Block	*/
if ( marb->gate == 0x00 )				/*1st time Fn called?	*/
{
	slot->flow_setting = marb->sup_uchar0 ;		/*Save flow setting	*/

   /*Save SSP registers that will be used to establish flow control and establish	*/
   /*"clean up" function in case De-register Modem request handled asynchronously	*/
	slot->sv_cout_flow_config = oregs->cout_flow_config ;
	slot->sv_cout_cntrl_sig   = oregs->cout_cntrl_sig ;
	slot->sv_cout_xon_1       = oregs->cout_xon_1 ;
#ifndef 				NO_FN_PROTO_PARMS /*Compile if Protypes OK*/
	slot->clean_up            = (void (*)(struct slot_struct FARm *, unsigned char))mpa_srvc_mpa_flow_clean_up ;
#else							  /*Compile if NO Protypes*/
	slot->clean_up            = (void (*)())mpa_srvc_mpa_flow_clean_up ;
#endif
	slot->frame_ctr  = gregs->gicp_frame_ctr ;	/*Snap shot for timing	*/
	marb->actv_srvc  = MARB_ACTV_SRVC_MPA_FLOW ;	/*Indicate outstnd'g req*/
	marb->gate = 0xFF ;				/*Set for repeat call	*/
	oregs->cout_flow_config = (oregs->cout_flow_config & 0x10) | 0x40 | 0x08 ;
}

/*Set up so output sent to MPA's AUX reg - MPA-to-UART Flow Control		*/
oregs->cout_cntrl_sig = 0x1AA ;				/*Send control sigs to	*/
							/*MPA indicat'g to direct*/
							/*output to MPA-to-UART	*/
							/*Flow Cntrl		*/

/*Exit function & delay 4 frames thus allowing time for MPA to "see" AUX Reg 	*/
/*control signal setting							*/
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
marb->cont_funct = mpa_srvc_mpa_flow_2 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*End of MPA<->UART FLOW CNTRL - 1st Function, mpa_srvc_mpa_flow_1	*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *  MPA<->UART HARDWARE FLOW CONTROL - 2nd FUNCTION, INITIALIZE FLOW CONTROL   *
 *                                                                             *
 *******************************************************************************/

#ifndef			NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_mpa_flow_2   ( unsigned long		        caller_parm,
				     struct marb_struct FARm *  	marb,
	      			     struct ssp_struct FARm *   	sspcb,
	      			     struct slot_struct FARm *  	slot,
	      			     struct icp_global_struct FARm * 	gregs,
				     struct cin_struct FARm *	  	iregs,
	      			     struct cout_struct FARm *	  	oregs
				   )
#else
static void  mpa_srvc_mpa_flow_2 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

#ifndef				RAMP_LINUX		/*R14*/
void		(FARm *sv_cont_funct)() ;		/*Saved addr of Register*/
							/*Modem request's "link"*/
							/*function		*/
#else
void            (FARm *sv_cont_funct)(
				      unsigned long int        caller_parm,
				      struct marb_struct       FARm * ,
				      struct ssp_struct        FARm * ,
				      struct slot_struct       FARm * ,
				      struct mpa_global_s      FARm * ,
				      struct mpa_input_s       FARm * ,
				      struct mpa_output_s      FARm * 
				     ) ;
#endif


/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;

/*Start of function's logic							*/
sv_cont_funct = marb->cont_funct ;			/*Save addr of "link" fn*/
mpa_srvc_reg_modem_4 ( (unsigned long)MARB_ACTV_SRVC_MPA_FLOW,
                       marb,  sspcb, slot,		/*Go send requestor's 	*/
		       gregs, iregs, oregs ) ;		/*MPA-to-UART Flow set'g*/
if ( (marb->srvc_state == 0x00)		   ||		/*Did error occur? or	*/
     (sv_cont_funct    == marb->cont_funct) )		/*Wait'g for frame delay*/
	return ;					/*If yes		*/

/*Set up & wait till MPA AUX Reg Flow Control setting (character) sent		*/
marb->cont_funct = mpa_srvc_mpa_flow_3 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}	/*End of MPA<->UART FLOW CNTRL - 2nd Function, mpa_srvc_mpa_flow_2	*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *  	  MPA<->UART HARDWARE FLOW CONTROL - 3rd FUNCTION, REQUEST COMPLETE    *
 *                                                                             *
 *******************************************************************************/

#ifndef			NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_mpa_flow_3   ( unsigned long		        caller_parm,
				     struct marb_struct FARm *  	marb,
	      			     struct ssp_struct FARm *   	sspcb,
	      			     struct slot_struct FARm *  	slot,
	      			     struct icp_global_struct FARm * 	gregs,
				     struct cin_struct FARm *	  	iregs,
	      			     struct cout_struct FARm *	  	oregs
				   )
#else
static void  mpa_srvc_mpa_flow_3 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/



/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
sspcb       = sspcb ;
iregs       = iregs ;

/*Start of function's logic							*/
if ( slot->slot_state != 0xFF )				/*Board still present?	*/
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_MPA_FLOW_REMOVED ;	/*Set completion status	*/
	return ;
}

if ( ((oregs->cout_flow_config & 0x10) != (oregs->cout_intnl_flow_ctrl & 0x10)) ||
     ((oregs->cout_intnl_flow_ctrl & 0x08) == 0x08) )	/*AUX Reg char xmit done*/
{							/*If no, check deadman	*/
	if ( frame_delay_chk (DEADMAN_DELAY, gregs, slot) == 0x00 )
	{						/*If deadman not expired*/
		marb->srvc_state = 0xFF ;		/*Set "call back" state	*/
		return ;
	}
	(*slot->clean_up)(slot, (unsigned char)0xFF) ;
	marb->srvc_status = ERR_MPA_FLOW_DEADMAN ; 	/*Set cmpltn status	*/
	return ;
}

/*MPA-to-UART Flow Control established, complete request			*/
mpa_srvc_mpa_flow_clean_up  (slot, (unsigned char)0x00 ) ; /*Restore SSP regs	*/
marb->srvc_status = 0x00 ;				/*Set completion status	*/
return ;						/*Exit, Request Cmplt	*/

}	/*End of MPA<->UART FLOW CNTRL - 3rd Function, mpa_srvc_mpa_flow_3	*/










/*#stitle EXCEPTION SERVICE - MPA<->UART HRDWRE FLOW CONTROL, LOCAL FUNCTION#*/
/*#page*/
/****************************************************************************************
 *                                                                             		*
 *	 MPA<->UART HRDWRE FLOW CONTROL - Clean Up SSP (mpa_srvc_mpa_flow_clean_up)	*
 *                                                                     			*
 * PURPOSE: To restore state of SSP registers when service has been completed (either 	*
 *	   due to error or successful completion)					*
 *											*
 * NOTES:										*
 *                                                                     			*
 * CALL: void  mpa_srvc_mpa_flow_clean_up  (struct slot_struct FARm *    slot,		*
 *					      unsigned char		 type)		*
 *                                                                     			*
 *			where:	slot = Slot Control Block pointer			*
 *				type = Clean up type:					*
 *					o 0x00 = Normal i.e. Reg Modem request cmplt	*
 *					o 0xFF = Abnormal i.e. Reg Modem request	*
 *						 aborted due to some failure or Board	*
 *						 removed				*
 *                                                                     			*
 *                                                                     			*
 * RETURN: None										*
 *                                                                     			*
 ****************************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void	mpa_srvc_mpa_flow_clean_up (struct slot_struct FARm *    slot,
					      unsigned char		 type)
#else
static void	mpa_srvc_mpa_flow_clean_up (slot, type)

struct slot_struct FARm *  slot;
unsigned char	       	   type;
#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

/*SSP variables relating to MARB's/Request's specific SLOT (Channel)		*/
#ifndef					RAMP_BSDI		/*R14*/
struct cout_struct FARm *	  	oregs ;		/*Slot's SSP Output regs	   */
#else								/*BSDI Compile		   */
volatile struct cout_struct FARm *	oregs ;		/*Slot's SSP Output regs	   */
#endif								/*End RAMP_BSDI Conditional*/



/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
type = type ;

/*Restore saved SSP registers							*/
oregs = slot->output ;					/*Slot's output regs ptr*/
oregs->cout_flow_config = (slot->sv_cout_flow_config & 0x2F) |  /*Rstr out sigs */
                          (oregs->cout_flow_config & 0x10) | 0x40 ;
oregs->cout_cntrl_sig   = (slot->sv_cout_cntrl_sig & 0x02BB) | 0x0004 ;
oregs->cout_xon_1	= slot->sv_cout_xon_1 ;		/*Restore XON char	*/
#ifndef 				NO_FN_PROTO_PARMS /*Compile if Protypes OK*/
slot->clean_up          = (void (*)(struct slot_struct FARm *, unsigned char))0 ;
#else							  /*Compile if NO Protypes*/
slot->clean_up          = (void (*)())0 ;
#endif
return ;

}					/*End of mpa_srvc_mpa_flow_clean_up	*/








/*#stitle EXCEPTION SERVICE - READ UART REGISTER (mpa_srvc_read_uart)#*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *          		READ UART REGISTER - SERVICE ENTRY POINT               *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
extern unsigned short int   mpa_srvc_read_uart (struct marb_struct FARm  *marb)
#else
extern unsigned short int   mpa_srvc_read_uart (marb)

struct marb_struct FARm  *marb;
#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

/*General working variables							*/
unsigned char		status ;			/*Funct. ret status	*/



/************************
 *   START OF CODE	*
 ************************/

status = val_marb (marb) ;				/*Validate caller's MARB*/
if ( status != 0 )					/*Is MARB OK to use?	*/
	return (ERR_RD_UART_MARB) ;			/*If no, return error	*/
if ( marb->in_use == 0 )				/*Initial MARB Request?	*/
{							/*If yes, init Fn ptr	*/
	marb->cont_funct = mpa_srvc_rd_uart_1 ;	/*Init MARB's FN ptr	*/
}
else							/*"Call Back" MARB,	*/
{							/*Validate SRVC active	*/
	if ( marb->actv_srvc != MARB_ACTV_SRVC_READ_UART ) /*Valid "Call Back"? */
	{						/*If no, exit		*/
		marb->srvc_status = ERR_RD_UART_CALL_BK ;
		marb->srvc_state  = 0x00 ;
		return ((unsigned short int)(((unsigned short int)(marb->srvc_state)<<8)+
                                               marb->srvc_status) ) ;
	}
}
return ( async_req_hndlr(marb) ) ;			/*Go handle request	*/


}			/*END of mpa_srvc_read_uart Service Entry function	*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *             READ UART REGISTER - 1st FUNCTION, SET TO READ UART REGISTER BY *
 *						 ADDRESSING MPA's AUX Reg      *
 *                                                                             *
 *******************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_rd_uart_1   ( unsigned long		        caller_parm,
				    struct marb_struct FARm *  		marb,
	      			    struct ssp_struct FARm *   		sspcb,
	      			    struct slot_struct FARm *  		slot,
	      			    struct icp_global_struct FARm * 	gregs,
				    struct cin_struct FARm *	  	iregs,
	      			    struct cout_struct FARm *	  	oregs
				  )
#else
static void  mpa_srvc_rd_uart_1 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/





/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
sspcb       = sspcb ;
iregs       = iregs ;

/*Start of function's logic							*/
if ( slot->req_outstnd != 0x00 )			/*Is a Req Outstand'g?	*/
{							/*If yes, return error	*/
	marb->srvc_status = ERR_RD_UART_REQ_OUT ;	/*Indicate error	*/
	return ;					/*Exit			*/
}
if ( slot->slot_state != 0xFF )				/*Is Modem registered?	*/
{							/*If no, error		*/
	marb->srvc_status = ERR_RD_UART_NOT_REG ;	/*Indicate error	*/
	return ;					/*Exit			*/
}
if ( slot->init_state != 0xFF )				/*Is Slot init'd/opn'd?	*/
{							/*If no, error		*/
	marb->srvc_status = ERR_RD_UART_NOPN ;		/*Indicate error	*/
	return ;					/*Exit			*/
}
/*Caller's request valid, save SSP registers that will be used to Read Slot's	*/
/*UART register									*/
slot->sv_cout_flow_config = oregs->cout_flow_config ;
slot->sv_cout_cntrl_sig   = oregs->cout_cntrl_sig ;
slot->sv_cout_xon_1       = oregs->cout_xon_1 ;
#ifndef 				NO_FN_PROTO_PARMS /*Compile if Protypes OK*/
slot->clean_up            = (void (*)(struct slot_struct FARm *, unsigned char))mpa_srvc_rd_uart_clean_up ;
#else							  /*Compile if NO Protypes*/
slot->clean_up            = (void (*)())mpa_srvc_rd_uart_clean_up ;
#endif

/*Set slot's/channel's control signals to address AUX (UART Read) reg.		*/
oregs->cout_cntrl_sig = 0x199 ;				/*Addr MPA's AUX reg	*/
slot->frame_ctr = gregs->gicp_frame_ctr ;		/*Snap shot for timing	*/
oregs->cout_flow_config |= 0x08 ;			/*Assure xmit XON char	*/

/*Exit function & delay 4 frames thus allowing time for MPA to "see" AUX Reg 	*/
/*control signal setting							*/
marb->actv_srvc  = MARB_ACTV_SRVC_READ_UART ;		/*Indicate outstnd'g req*/
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
marb->cont_funct = mpa_srvc_rd_uart_2 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}		/*End of READ UART REG - 1st Function, mpa_srvc_rd_uart_1	*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *             READ UART REGISTER - 2nd FUNCTION, WRITE TO MPA's AUX REG       *
 *					  INDICATING TO MPA TO READ AND (XMIT) *
 *					  REQUESTOR's UART REG		       *
 *                                                                             *
 *******************************************************************************/

#ifndef			NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_rd_uart_2    ( unsigned long		        caller_parm,
				     struct marb_struct FARm *  	marb,
	      			     struct ssp_struct FARm *   	sspcb,
	      			     struct slot_struct FARm *  	slot,
	      			     struct icp_global_struct FARm * 	gregs,
				     struct cin_struct FARm *	  	iregs,
	      			     struct cout_struct FARm *	  	oregs
				   )
#else
static void  mpa_srvc_rd_uart_2 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

/*SSP input area variables							*/
#ifndef					RAMP_BSDI		/*R14*/
struct cin_bank_struct FARm		*actv_bank ;	/*Ptr to active bank		   */
#else								/*BSDI Compile		   */
volatile struct cin_bank_struct FARm	*actv_bank ;	/*Ptr to active bank		   */
#endif								/*End RAMP_BSDI Conditional*/


/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
sspcb       = sspcb ;

/*Start of function's logic							*/
if ( slot->slot_state != 0xFF )				/*Is Brd still present?	*/
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_RD_UART_REMOVED ;	/*Set completion status	*/
	return ;
}
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
if ( frame_delay_chk ( (unsigned short int)0x04, gregs, slot ) == 0 )
	return ;					/*4 frames not elapsed	*/

/*Have delayed long enough for MPA to "see" control signal setting indicating	*/
/*that output be directed to MPA's AUX Register (SPR), save slot's "number of 	*/
/*input characters" so can determine when UART's byte has been received		*/
actv_bank = ( (iregs->cin_locks & 0x01) == 0x00 ) 		  ? /*Bank A actv?*/
	      (struct cin_bank_struct FARm *)&(iregs->cin_bank_a) : /*If yes	*/
	      (struct cin_bank_struct FARm *)&(iregs->cin_bank_b) ; /*If no	*/
marb->sup_ushort = actv_bank->cin_bank_num_chars ;		    /*Save	*/

/*Initiate read of requestor's UART register by writing pattern to AUX register	*/
/*indicating to MPA to read Requestor's UART register.  Note: Change clean up	*/
/*function so if abort during read then outstanding read is cleaned up i.e. read*/
/*character accounted for.							*/
#ifndef 				NO_FN_PROTO_PARMS /*Compile if Protypes OK*/
slot->clean_up            = (void(*)(struct slot_struct FARm *, unsigned char))mpa_srvc_rd_uart_cln_rd ;
#else							  /*Compile if NO Protypes*/
slot->clean_up            = (void(*)())mpa_srvc_rd_uart_cln_rd ;		   /*R9*/
#endif
slot->read_toggle        ^= 0x08 ;			/*Sv I/O Rd Toggle state*/
oregs->cout_xon_1         = marb->sup_uchar0 | 		/*AUX REG byte = RD UART*/
			    slot->read_toggle ; 
oregs->cout_flow_config  ^= 0x10 ;			/*Initiate xmit'g char	*/

/*Set up & wait till AUX REG output sent					*/
slot->frame_ctr = gregs->gicp_frame_ctr ;		/*Snap shot for timing	*/
marb->cont_funct = mpa_srvc_rd_uart_3 ;
return ;						/*Exit, thus allowing	*/
							/*non-waited request to	*/
							/*"work mix"		*/

}		/*End of READ UART REG - 2nd Function, mpa_srvc_rd_uart_2	*/
/*#page*/
/*******************************************************************************
 *                                                                             *
 *             READ UART REGISTER - 3rd FUNCTION, WAIT FOR UART REGISTER BYTE  *
 *					 	  TO BE RECIEVED	       *
 *                                                                             *
 *******************************************************************************/

#ifndef			NO_FN_PROTO_PARMS	/*R11*/
static void  mpa_srvc_rd_uart_3    ( unsigned long		        caller_parm,
				     struct marb_struct FARm *  	marb,
	      			     struct ssp_struct FARm *   	sspcb,
	      			     struct slot_struct FARm *  	slot,
	      			     struct icp_global_struct FARm * 	gregs,
				     struct cin_struct FARm *	  	iregs,
	      			     struct cout_struct FARm *	  	oregs
				   )
#else
static void  mpa_srvc_rd_uart_3 (caller_parm, marb, sspcb, slot, gregs, iregs, oregs)

unsigned long		        caller_parm;
struct marb_struct FARm *  	marb;
struct ssp_struct FARm *   	sspcb;
struct slot_struct FARm *  	slot;

#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * gregs;
struct cin_struct FARm *	iregs;
struct cout_struct FARm *	oregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs;
volatile struct cin_struct FARm *	iregs;
volatile struct cout_struct FARm *	oregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

/*SSP input area variables							*/
#ifndef					RAMP_BSDI		/*R14*/
struct cin_bank_struct FARm		*actv_bank ;	/*Ptr to active bank	 	   */
#else								/*BSDI Compile		   */
volatile struct cin_bank_struct FARm	*actv_bank ;	/*Ptr to active bank		   */
#endif								/*End RAMP_BSDI Conditional*/



/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
caller_parm = caller_parm ;
sspcb       = sspcb ;

/*Start of function's logic							*/
if ( slot->slot_state != 0xFF )				/*Is Brd still present?	*/
{							/*If no, modem pulled	*/
	marb->srvc_status = ERR_RD_UART_REMOVED ;	/*Set completion status	*/
	return ;
}
marb->srvc_state = 0xFF ;				/*Set "call back" state	*/
if ( ((oregs->cout_flow_config & 0x10) != (oregs->cout_intnl_flow_ctrl & 0x10)) ||
     ((oregs->cout_intnl_flow_ctrl & 0x08) == 0x08) )	/*AUX reg char xmit done*/
{							/*If no, check deadman	*/
	if ( frame_delay_chk (DEADMAN_DELAY, gregs, slot) == 0x00 )
	{						/*If deadman not expired*/
		return ;
	}
	mpa_srvc_rd_uart_clean_up (slot, (unsigned char)0xFF) ;			   /*R9*/
	marb->srvc_status = ERR_RD_UART_DEADMAN ; 	/*Set cmpltn status	*/
	marb->srvc_state = 0x00 ;			/*Set "complete" state	*/
	return ;
}

/*MPA's AUX Reg Requestor's READ UART Register request sent, wait for UART 	*/
/*Register's byte to be received						*/
actv_bank = ( (iregs->cin_locks & 0x01) == 0x00 ) 		  ? /*Bank A actv?*/
	      (struct cin_bank_struct FARm *)&(iregs->cin_bank_a) : /*If yes	  */
	      (struct cin_bank_struct FARm *)&(iregs->cin_bank_b) ; /*If no	  */
if ( actv_bank->cin_bank_num_chars == marb->sup_ushort ) /*Have recv'd UART byte? */
{							 /*If no, check deadman	  */
	if ( frame_delay_chk ( (DEADMAN_DELAY+5), gregs, slot) == 0x00 )
	{						/*If deadman not expired*/
		return ;
	}
	mpa_srvc_rd_uart_clean_up (slot, (unsigned char)0xFF) ;			   /*R9*/
	marb->srvc_status = ERR_RD_UART_INPUT_TO ; 	/*Set cmpltn status	*/
	marb->srvc_state  = 0x00 ;			/*Set "complete" state	*/
	return ;
}

/*UART Register byte received, complete request					*/
mpa_srvc_rd_uart_clean_up (slot, (unsigned char)0x00 ); /*Restore SSP regs	*/
marb->srvc_status = 0x00 ;				/*Set completion status	*/
marb->srvc_state  = 0x00 ;				/*Set completion state	*/
return ;						/*Exit, Request Cmplt	*/
}		/*End of READ UART REG - 3rd Function, mpa_srvc_rd_uart_3	*/









/*#stitle EXCEPTION SERVICE - READ UART REGISTER, LOCAL FUNCTION#*/
/*#page*/
/****************************************************************************************
 *                                                                             		*
 *     		READ UART REGISTER - Clean Up SSP (mpa_srvc_rd_uart_clean_up)		*
 *                                                                     			*
 * PURPOSE: To restore state of SSP registers when service has been completed (either 	*
 *	   due to error or successful completion)					*
 *											*
 * NOTES:										*
 *                                                                     			*
 * CALL: void  mpa_srvc_rd_uart_clean_up (struct slot_struct FARm *    slot,		*
 *			     	          unsigned char		       type)		*
 *                                                                     			*
 *			where:	slot = Slot Control Block pointer			*
 *				type = Clean up type:					*
 *					o 0x00 = Normal i.e. Reg Modem request cmplt	*
 *					o 0xFF = Abnormal i.e. Reg Modem request	*
 *						 aborted due to some failure or Board	*
 *						 removed				*
 *                                                                     			*
 *                                                                     			*
 * RETURN: None										*
 *                                                                     			*
 ****************************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void	mpa_srvc_rd_uart_clean_up (struct slot_struct FARm *    slot,
					   unsigned char		type)
#else
static void	mpa_srvc_rd_uart_clean_up (slot, type)

struct slot_struct FARm *  slot;
unsigned char	       	   type;
#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

/*SSP variables relating to MARB's/Request's specific SLOT (Channel)		*/
#ifndef					RAMP_BSDI		/*R14*/
struct cout_struct FARm *	  	oregs ;		/*Slot's SSP Output regs	   */
#else								/*BSDI Compile		   */
volatile struct cout_struct FARm *	oregs ;		/*Slot's SSP Output regs	   */
#endif								/*End RAMP_BSDI Conditional*/



/************************
 *   START OF CODE	*
 ************************/

/*Clean up compiler warnings - unused variables (note: should compile as NOPs)	*/
type = type ;

/*Start of function's logic							*/
/*Restore saved SSP registers							*/
oregs = slot->output ;					/*Slot's output regs ptr*/
oregs->cout_flow_config = (slot->sv_cout_flow_config & 0x2F) |  /*Rstr out sigs */
                          (oregs->cout_flow_config & 0x10) | 0x40 ;
oregs->cout_cntrl_sig   = (slot->sv_cout_cntrl_sig & 0x02BB) | 0x0004 ;
oregs->cout_xon_1	= slot->sv_cout_xon_1 ;		/*Restore XON char	*/
#ifndef 				NO_FN_PROTO_PARMS /*Compile if Protypes OK*/
slot->clean_up          = (void (*)(struct slot_struct FARm *, unsigned char))0 ;
#else							  /*Compile if NO Protypes*/
slot->clean_up          = (void (*)())0 ;
#endif
return ;

}					/*End of mpa_srvc_rd_uart_clean_up	*/
/*#page*/
/****************************************************************************************
 *                                                                             		*
 *    READ UART REGISTER - Outstanding Read Clean Up SSP (mpa_srvc_rd_uart_cln_rd)	*
 *                                                                     			*
 * PURPOSE: To restore state of the SSP when UART Read outstanding has been aborted.	*
 *											*
 * NOTES:										*
 *                                                                     			*
 * CALL: void  mpa_srvc_reg_modem_cln_rd    (struct slot_struct FARm    *slot,		*
 *					     unsigned char		type)		*
 *                                                                     			*
 *			where:	slot = Slot Control Block pointer			*
 *				type = Clean up type:					*
 *					o 0x00 = Normal i.e. Reg Modem request cmplt	*
 *					o 0xFF = Abnormal i.e. Reg Modem request	*
 *						 aborted due to some failure or Board	*
 *						 removed				*
 *                                                                     			*
 *                                                                     			*
 * RETURN: None										*
 *                                                                     			*
 ****************************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static void	mpa_srvc_rd_uart_cln_rd  (struct slot_struct FARm   	*slot,	   /*R9*/
					  unsigned char			type)

#else
static void	mpa_srvc_rd_uart_cln_rd (slot, type)

struct slot_struct FARm *  slot;
unsigned char	       	   type;
#endif
{

/************************
 *     LOCAL DATA	*
 ************************/

/*SSP variables									*/
#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm * 	gregs ;		/*Slot's Global regs	*/
struct cin_struct FARm *	  	iregs ;		/*Slot's Input regs	*/
struct cin_bank_struct FARm *		actv_bank ;	/*Slot's active Bank	*/
struct cout_struct FARm *	  	oregs ;		/*Slot's Output regs	*/
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm * gregs ;	/*Slot's Global regs	*/
volatile struct cin_struct FARm *	 iregs ;	/*Slot's Input regs	*/
volatile struct cin_bank_struct FARm *	 actv_bank ;	/*Slot's active Bank	*/
volatile struct cout_struct FARm *	 oregs ;	/*Slot's Output regs	*/
#endif								/*End RAMP_BSDI Conditional*/

/*Variables use to delay for UART character to be received			*/
unsigned short int			init_frame ; 	/*Initial Frame ctr	*/
unsigned short int			cur_frame_ct ;	/*Frame ctr to calc delta*/

/************************
 *   START OF CODE	*
 ************************/

/*Set Slot's SSP pointer's							*/
gregs	  = slot->global ;				/*Slot's global regs ptr*/
iregs	  = slot->input ;				/*Slot's input regs ptr	*/
oregs	  = slot->output ;				/*Slot's output regs ptr*/
actv_bank = ( (iregs->cin_locks & 0x01) == 0x00 ) 		  ? /*Bank A actv?*/
	      (struct cin_bank_struct FARm *)&(iregs->cin_bank_a) : /*If yes	  */
	      (struct cin_bank_struct FARm *)&(iregs->cin_bank_b) ; /*If no	  */

/*Assure MPA AUX reg character sent (Read UART toggle)				*/
while ( ((oregs->cout_flow_config & 0x10) != (oregs->cout_intnl_flow_ctrl & 0x10)) ||
        ((oregs->cout_intnl_flow_ctrl & 0x08) == 0x08) ) /*AUX reg char xmit dne*/
{							 /*If no, check deadman	*/
	if ( frame_delay_chk (DEADMAN_DELAY, gregs, slot) == 0xFF )
		break ;					/*If deadman expired	*/
}

/*MPA's AUX Reg RD UART Reg req sent, delay to allow time for byte to be received*/
init_frame = gregs->gicp_frame_ctr ;			/*Snap shot for delay	*/
while ( 1 == 1 )					/*Delay - allow time for*/
{							/*UART char to be recv'd*/
	cur_frame_ct = gregs->gicp_frame_ctr ;		/*SSP current Frame CTR	*/
	if ( cur_frame_ct < init_frame )		/*Did Frame Ct wrap?	*/
		init_frame = 0 ;  			/*If yes, adjust	*/
	if ( (unsigned short int)(cur_frame_ct - init_frame) >= 6 )
		break ;
}

/*Have cleaned up outstanding read, now restore saved SSP registers by calling	*/
/*"standard" clean up routine.							*/
mpa_srvc_rd_uart_clean_up ( slot, type ) ;		/*Restore sv'd SSP regs	*/
return ;

}					/*End of mpa_srvc_rd_uart_cln_rd	*/





/*#stitle INTERNAL/LOCAL FN - VALIDATE SSP DATA AREA PTR (val_ssp)#*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 *               	VALIDATE SSP DATA AREA PTR				*
 *                                                                     		*
 * PURPOSE: To validate SSP Data Area pointer					*
 *										*
 * NOTES:									*
 *                                                                     		*
 * CALL: unsigned char	val_ssp	(struct ssp_struct	*sspcb)			*
 *                                                                     		*
 *			where: sspcb = SSP Data Area pointer to validate	*
 *                                                                     		*
 *                                                                     		*
 * RETURN: Status (unsigned char):						*
 *		o 0x00 = Pointer valid, OK to use				*
 *		o 0xF0 = SSPCB's SSP Global registers not accessable		*
 *		o 0xFF = Pointer invalid, do not use				*
 *                                                                     		*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static unsigned char	val_ssp (struct ssp_struct FARm	*sspcb)
#else
static unsigned char	val_ssp (sspcb)

struct ssp_struct FARm	*sspcb;
#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

#ifndef					RAMP_BSDI		/*R14*/
struct mpa_global_s FARm       		*global_regs ;        	/*SSP's Global reg         */
#else								/*BSDI Compile		   */
volatile struct mpa_global_s FARm       *global_regs ;        	/*SSP's Global reg         */
#endif								/*End RAMP_BSDI Conditional*/



/************************
 *   START OF CODE	*
 ************************/

if ( (sspcb->in_use    != 0xFF)     ||
     (sspcb->rev       != MPA_REV)  ||
     (sspcb->signature != sspcb) )
	return ( (unsigned char)0xFF ) ;

/*SSP Control Block appears sane, validate that SSP Registers appear "mapped"	*/
global_regs = sspcb->global ;				/*SSPCB's SSP Globl regs*/
if (                                                    /*Does it appear that 	*/
      ((global_regs->gicp_attn & 0xF0) != 0 ) ||	/* SSPCB SSP reg area is*/
      ((global_regs->gicp_chan & 0xC0) != 0 ) ||	/*  mapped in?		*/
     (((global_regs->gicp_scan_spd) & ~sspcb->sv_sp_mul_msk) != sspcb->sv_sp_mul_set) )
	return ( (unsigned char)0xF0 ) ;		/*If no, error		*/

return ( (unsigned char)0x00 ) ;

}					/*END of val_ssp function		*/

     








/*#stitle INTERNAL/LOCAL FN - VALIDATE MARB (val_marb)#*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 *               	       VALIDATE MARB					*
 *                                                                     		*
 * PURPOSE: To validate MPA Async Service Request Block (MARB)			*
 *										*
 * NOTES:									*
 *                                                                     		*
 * CALL: unsigned char	val_marb (struct marb_struct FARm	*marb)		*
 *                                                                     		*
 *			where: marb = MARB pointer to validate			*
 *                                                                     		*
 *                                                                     		*
 * RETURN: Status (unsigned char):						*
 *		o 0x00 = MARB valid (OK to use)					*
 *		o 0xFF = MARB parameter bad (do not use)			*
 *                                                                     		*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static unsigned char	val_marb (struct marb_struct FARm	*marb)
#else
static unsigned char	val_marb (marb)

struct marb_struct FARm	*marb;
#endif

{

/************************
 *     LOCAL DATA	*
 ************************/



/************************
 *   START OF CODE	*
 ************************/

if ( marb->in_use == 0 )				/*Initial MARB call	*/
{							/*If yes		*/
	if ( (marb->signature  != (struct marb_struct FARm *)0) 	    ||
#ifndef 				NO_FN_PROTO_PARMS /*Compile if Protypes OK*/	/*R11*/
	     (marb->cont_funct != (void (*)(unsigned long,
					    struct marb_struct  FARm *,
	      			     	    struct ssp_struct   FARm *,
	      			            struct slot_struct  FARm *,
	      			            struct mpa_global_s FARm *,
				            struct mpa_input_s  FARm *,
	      			            struct mpa_output_s FARm *))0) )
#else							  /*Compile if NO Protypes*/
	     (marb->cont_funct != (void (*)())0) )
#endif
		return ((unsigned char)0xFF) ;		/*If MARB bad, ret error*/
}
else							/*MARB being used for	*/
{							/*"call back"		*/
	if ( (marb->signature  != marb) ||
#ifndef 				NO_FN_PROTO_PARMS /*Compile if Protypes OK*/	/*R11*/
	     (marb->cont_funct == (void (*)(unsigned long,
					    struct marb_struct  FARm *,
	      			     	    struct ssp_struct   FARm *,
	      			            struct slot_struct  FARm *,
	      			            struct mpa_global_s FARm *,
				            struct mpa_input_s  FARm *,
	      			            struct mpa_output_s FARm *))0) )
#else							  /*Compile if NO Protypes*/
	     (marb->cont_funct == (void (*)())0) )
#endif
		return ((unsigned char)0xFF) ;		/*If MARB bad, ret error*/
}

/*MARB valid, OK to use, return with good status				*/
return ((unsigned char)0x00) ;

}					/*END of val_marb function		*/



/*#stitle INTERNAL/LOCAL FN - ASYNC SERVICE REQUEST HANDLER (async_req_hndlr)#*/
/*#page*/
/********************************************************************************
 *                                                                             	*
 *               	   ASYNC SERVICE REQUEST HANDLER 			*
 *                                                                     		*
 * PURPOSE: To validate a requestor's MPA Async Request Block (MARB) and if the	*
 *	   MARB is validate "link" to the MARB's Handler function.		*
 *										*
 * NOTES:									*
 *                                                                     		*
 * CALL: unsigned short int   async_req_hndlr  (struct marb_struct FARm	*marb)	*
 *                                                                     		*
 *			where: marb = MARB pointer to validate			*
 *                                                                     		*
 *                                                                     		*
 * RETURN: Service Request State & Status					*
 *                                                                     		*
 ********************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static unsigned short int   async_req_hndlr  (struct marb_struct FARm  *marb)
#else
static unsigned short int   async_req_hndlr  (marb)

struct marb_struct FARm  *marb;
#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

/*SSPCB (SSP Control Block), MPACB (MPA Control Block) and SLOTCB (Slot 	*/
/*Control Block) variables							*/
struct ssp_struct FARm   	*sspcb ;		/*MARB's SSCB pointer	*/
unsigned int		 	mpa_index ;		/*MARB's MPA/Eng index	*/
struct slot_struct FARm  	*slot ;			/*MARB's SLOT Cntrl Blk	*/

/*SSP variables relating to MARB's/Request's specific SLOT (Channel)		*/
#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm  	*gregs ;		/*Request's Global regs	*/
struct cin_struct FARm	       	*iregs ;		/*Request's Input regs	*/
struct cout_struct FARm	       	*oregs ;		/*Request's Output regs	*/
#else								/*BSDI Compile		   */
volatile struct icp_global_struct FARm  *gregs ;		/*Request's Global regs	   */
volatile struct cin_struct FARm	       	*iregs ;		/*Request's Input regs	   */
volatile struct cout_struct FARm	*oregs ;		/*Request's Output regs	   */
#endif								/*End RAMP_BSDI Conditional*/

/*General working variables							*/
unsigned char		status ;			/*Funct. ret status	*/


/************************
 *   START OF CODE	*
 ************************/

marb->srvc_state = 0x00 ;				/*Set MARB's State=done	*/
sspcb = marb->sspcb ;					/*Get MARB's SSPCB's	*/
status = val_ssp ( sspcb ) ;				/*Validate MARB's SSPCB	*/
if ( status != 00 )					/*Is MARB's SSPCB OK?	*/
{							/*If no, exit with error*/
  /*MARB's SSPCB is invalid (not in correct state), since MARB's SSPCB not in 	*/
  /*good shape clean up MARB and return error status				*/
	marb->srvc_status = MARB_STATUS_SSPCB ;		/*MARB's status = bad	*/
	goto COMPLETE_EXIT ;
}
/*MARB's SSPCB valid, set up & get ptr to Request's (MARB's) Slot Control Block	*/
/*and validate SLOTCB ok							*/
mpa_index = marb->slot_chan >> 4 ;			/*Index to Req's MPA/Eng*/
if ( sspcb->mpa[mpa_index].base_chan != 
    ((mpa_index - (sspcb->mpa[mpa_index].eng_num)) << 4) ) /*Is Marb's MPA OK?	*/
{							   /*If no, error	*/
	marb->srvc_status = MARB_STATUS_MPACB ;		/*MARB does not pt to	*/
	goto COMPLETE_EXIT ;				/* valid MPA, error	*/
}
slot = (struct slot_struct FARm  *)&((sspcb->mpa[mpa_index].eng_head)[(marb->slot_chan & 0x0f)]) ;
if ( (slot->in_use   != 0xFF) 		 || 		/*Is MARB's Slot valid	*/
     (slot->alloc    != 0xFF)		 ||
     (slot->chan_num != marb->slot_chan) ||
     (slot->signature->in_use != 0xFF) )
{							/*If no, error		*/
	marb->srvc_status = MARB_STATUS_SLOTCB ;	/*MARB's does not pt to	*/
	goto COMPLETE_EXIT ;				/*valid MPA, error	*/
}

/*MARB's SLOT Control Block ok, point to request's (MARB's) SSP registers	*/
gregs = sspcb->global ;					/*Req's SSP Global regs	*/
iregs = slot->input ;					/*Req's SSP Input regs	*/
oregs = slot->output ;					/*Req's SSP Output regs	*/

/*Determine if 1st request or subsequent "call back" request and validate MARB	*/
if ( marb->in_use == 0 )				/*Is 1st request ?	*/
{							/*If yes		*/
	if ( slot->req_outstnd == 0 )			/*Is a req oustnd?	*/
#ifndef 				NO_FN_PROTO_PARMS /*Compile if Protypes OK*/
		slot->clean_up = (void (*)(struct slot_struct FARm *, unsigned char))0 ;
#else							  /*Compile if NO Protypes*/
		slot->clean_up = (void (*)())0 ;
#endif
}
else							/*"Call back" MARB,	*/
{							/*validate request OK	*/
	if ( (slot->req_outstnd == 0x00) ||		/*Does MARB's SLOT	*/
	     (slot->actv_marb   != marb) ||		/* indicate Request	*/
	     (slot->cont_funct  != marb->cont_funct) ||	/*  outstanding and	*/
	     (marb->signature->in_use != 0xFF) )	/*   is MARB OK?	*/
	{						/*SLOT/MARB not set up	*/
							/*  to continue/call bk	*/
		marb->srvc_status = MARB_STATUS_CALL_BK ;
		goto COMPLETE_EXIT ;
	}
}

/*MARB and SLOT etc. valid, call MARB's function and if Request broken into 	*/
/*multiple "call back" functions continue calling if "waited" MARB		*/
do
{
	marb->srvc_state = 0x00 ;			/*Pre-init state = done	*/
	(*marb->cont_funct)(				/*Call MARB's function:	*/
	      (unsigned long)                   0,	/*  - Calling Fn's parm */
	      (struct marb_struct FARm *)  	marb,	/*  - MARB		*/
	      (struct ssp_struct FARm *)   	sspcb,	/*  - SSP Control Block	*/
	      (struct slot_struct FARm *)  	slot,	/*  - SLOT Control Block*/
	      (struct icp_global_struct FARm *) gregs,	/*  - SSP Global Regs	*/
	      (struct cin_struct FARm *)	iregs,	/*  - Slot's Input regs	*/
	      (struct cout_struct FARm *)	oregs	/*  - Slot's Output regs*/
	     ) ;
	if ( marb->srvc_state != 0 )			/*Is MARB completed?	*/
	{						/*If no			*/
		slot->req_outstnd = marb->actv_srvc ;	/*Set MARB's SLOT = 	*/
		slot->actv_marb   = marb ;		/* Request outstanding	*/
	}
}
while ( (marb->srvc_state != 0) && (marb->req_type == 0x00) ) ;

/*Return to to caller, Request is either complete or "call back" required	*/
if ( marb->srvc_state == 0x00 )				/*Is Request complete?	*/
{							/*If yes		*/
	if ( (slot->req_outstnd != 0x00) &&		/*Has outstanding Req 	*/
	     (slot->actv_marb   == marb)  )		/*been completed?	*/
							/*Note: A request that	*/
							/*	was issued while*/
							/*	a req outstndg	*/
							/*	won't take path	*/
	{						/*If yes, clean up	*/
		slot->req_outstnd = 0x00 ;		/*Slot has no REQ Outstd*/
		slot->actv_marb   = (struct marb_struct FARm *)0 ; /* No actv MARB*/
#ifndef 				NO_FN_PROTO_PARMS /*Compile if Protypes OK*/	/*R11*/
		slot->cont_funct  = (void (*)(unsigned long,	   /*No "call back"*/
					      struct marb_struct  FARm *,
		      			      struct ssp_struct   FARm *,
		      			      struct slot_struct  FARm *,
		      			      struct mpa_global_s FARm *,
					      struct mpa_input_s  FARm *,
		      			      struct mpa_output_s FARm *))0 ;
		slot->clean_up = (void (*)(struct slot_struct FARm *, unsigned char))0 ;
#else							  /*Compile if NO Protypes*/
		slot->cont_funct  = (void (*)())0 ;					/*R11*/
		slot->clean_up = (void (*)())0 ;
#endif
	}

  /*Common exit point when Async Request is complete due to success or error.	*/
  /*Clean up MARB so it can be used for a new request.				*/
  /*NOTE: Assumes that MARB's Service Status (srvc_status) has been set		*/
COMPLETE_EXIT:
	marb->in_use     = 0x00 ;			/*Indicate avail MARB	*/
	marb->actv_srvc  = 0x00 ;			/*Set no req outstndg	*/
	marb->signature  = (struct marb_struct FARm *)0; /*Assure MARB invalid	*/
#ifndef 				NO_FN_PROTO_PARMS /*Compile if Protypes OK*/	/*R11*/
	marb->cont_funct = (void (*)(unsigned long,
				     struct marb_struct  FARm *,
	      			     struct ssp_struct   FARm *,
	      			     struct slot_struct  FARm *,
	      			     struct mpa_global_s FARm *,
				     struct mpa_input_s  FARm *,
	      			     struct mpa_output_s FARm *))0 ;
#else							  /*Compile if NO Protypes*/
	marb->cont_funct = (void (*)())0 ;
#endif
	return ((unsigned short int)(((unsigned short int)(marb->srvc_state)<<8)+
                                       marb->srvc_status) ) ;
}


/*Service not complete & caller's request is (marb->req_type == 0xFF), set up	*/
/*for "call back" 								*/
slot->cont_funct  = marb->cont_funct ;			/*Set up nxt Srvc Fn	*/
marb->in_use      = 0xFF ;				/*Indicate MARB "in_use"*/
marb->signature   = marb ; 				/*Assure MARB valid	*/
marb->srvc_status = 0x00 ;				/*Indicate "call back"	*/
marb->srvc_state  = 0xFF ;				/* required		*/
return ( (unsigned short int)0xFF00 ) ;			/*Return "call back"	*/
							/* completion status	*/

}					/*END of async_req_hndlr function	*/








/*#stitle INTERNAL/LOCAL FN - FRAME DELAY CHECK (frame_delay_chk)#*/
/*#page*/
/****************************************************************************************
 *                                                                             		*
 *               	   FRAME DELAY CHECK						*
 *                                                                     			*
 * PURPOSE: To determine if caller's number of SSP Frames have elapsed.			*
 *											*
 * NOTES:										*
 *                                                                     			*
 * CALL: unsigned char  frame_delay_chk  (unsigned short int              frame_delay,	*
 *		  			  struct icp_global_struct FARm   *gregs,	*
 *					  struct slot_struct FARm         *slot)	*
 *                                                                     			*
 *			where:	frame_delay = number of elapsed frames			*
 *				gregs  	    = SSP Global reg pointer			*
 *				slot        = Slot Control Block pointer		*
 *                                                                     			*
 *                                                                     			*
 * RETURN: Number of Elapsed Frames exceeded flag:					*
 *		0x00 = Elapsed # Frames NOT exceeded					*
 *		0xFF = Elasped # Frames exceeded					*
 *                                                                     			*
 ****************************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
static unsigned char	frame_delay_chk  (unsigned short int              frame_delay,
	  			  	  struct icp_global_struct FARm   *gregs,
				  	  struct slot_struct FARm         *slot
					 )
#else
static unsigned char	frame_delay_chk  (frame_delay, gregs, slot)

unsigned short int              frame_delay;
#ifndef					RAMP_BSDI		/*R14*/
struct icp_global_struct FARm   *gregs;
#else								/*BSDI Compile		  */
volatile struct icp_global_struct FARm   *gregs;
#endif								/*End RAMP_BSDI Conditional*/
struct slot_struct FARm         *slot;
#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

/*SSP variables relating to MARB's/Request's specific SLOT (Channel)		*/
unsigned short int	cur_frame_ct ;			/*Current SSP Frame Ctr	*/



/************************
 *   START OF CODE	*
 ************************/

cur_frame_ct = gregs->gicp_frame_ctr ;			/*SSP current Frame CTR	*/
if ( cur_frame_ct < slot->frame_ctr )			/*Did Frame Ct wrap?	*/
	slot->frame_ctr = 0 ;  				/*If yes, adjust	*/
if ( (unsigned short int)(cur_frame_ct - slot->frame_ctr) < frame_delay )
	return ( (unsigned char)0 ) ;			/*If time has no elapsed*/
return ( (unsigned char)0xFF ) ;			/*If frame ct exceeded	*/

}					/*End of frame_delay_chk function	*/








/*#stitle INTERNAL/LOCAL FN - INCREMENT TAIL POINTER (inc_tail)#*/
/*#page*/
/****************************************************************************************
 *                                                                             		*
 *               	   INCREMENT TAIL POINTER					*
 *                                                                     			*
 * PURPOSE: To update increment slot's/channel's tail pointer thus acknowledging 	*
 *	   receipt of an input character						*
 *											*
 * NOTES:										*
 *                                                                     			*
 * CALL: unsigned char   inc_tail (struct cin_struct FARm *  iregs)			*
 *                                                                     			*
 * RETURN: Function Status:								*
 *		0x00 = Successful							*
 *		0xFF = Failure, invalid buffer size (CIN_Q_CNTRL @ 0x06, Q size > 8)	*
 *                                                                     			*
 ****************************************************************************************/

/************************
 *	GLOBAL DATA	*
 ************************/

/*Global table of masks used to MOD INC Slot's Tail Pointer			*/
static unsigned short int    mask_tbl[] = {		/*Table of masks indexed*/
							/*by Input Queue Size:	*/
					    0x00FF,	/*  Q size = 0 (256)	*/
					    0x01FF,	/*  Q size = 1 (512)	*/
					    0x03FF,	/*  Q size = 2 ( 1K)	*/
					    0x07FF,	/*  Q size = 3 ( 2K)	*/
					    0x0FFF,	/*  Q size = 4 ( 4K)	*/
					    0x1FFF,	/*  Q size = 5 ( 8K)	*/
					    0x3FFF,	/*  Q size = 6 (16K)	*/
					    0x7FFF,	/*  Q size = 7 (32K)	*/
					    0xFFFF	/*  Q size = 8 (64K)	*/
					  } ;


/*#page*/
/************************
 *  FUNCTION ENTRANCE	*
 ************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
unsigned char	inc_tail (struct cin_struct FARm *  iregs)
#else
unsigned char	inc_tail (iregs)

#ifndef					RAMP_BSDI		/*R14*/
struct cin_struct FARm *  iregs;
#else								/*BSDI Compile		  */
volatile struct cin_struct FARm *  iregs;
#endif								/*End RAMP_BSDI Conditional*/

#endif

{

/************************
 *     LOCAL DATA	*
 ************************/

unsigned char			q_cntrl ;		/*Copy of CIN_Q_CNTRL	*/
unsigned int			mask_index ;		/*Q size - index used to*/
							/*develop masks below	*/
unsigned short int		q_displ_mask ;		/*Mask used to MOD INC	*/
							/*Tail pointer		*/
unsigned short int		actv_tail_value ;	/*Active tail's value	*/

#ifndef					RAMP_BSDI		/*R14*/
unsigned short int FARm 	*non_actv_tail_ptr ;	/*Ptr to non-actv tail		   */
#else								/*BSDI Compile		   */
volatile unsigned short int FARm *non_actv_tail_ptr ;	/*Ptr to non-actv tail		   */
#endif								/*End RAMP_BSDI Conditional*/

unsigned short int		tail_displ ;		/*Tail Offset	 	*/


/************************
 *   START OF CODE	*
 ************************/

q_cntrl = iregs->cin_q_cntrl ;				/*Get Slot's Q Cntrl reg*/

/*Develop masks used to do MOD INC of tail pointer				*/
mask_index = q_cntrl & 0x0F ;				/*Mask table index	*/
if ( mask_index > 8 )					/*Is Q buffer size OK?	*/
	return ( (unsigned char)0xFF ) ;		/*If no			*/
q_displ_mask = mask_tbl[mask_index] ;			/*Get mask from tbl	*/

/*Determine active tail pointer & get its value					*/
if ( (q_cntrl & 0x10) == 0 )				/*Is tail A active?	*/
{							/*If yes		*/
	actv_tail_value   = iregs->cin_tail_ptr_a ;	/*Get actv tail value	*/
	non_actv_tail_ptr = &(iregs->cin_tail_ptr_b) ;	/*Pt to non-actv tail	*/
}
else							/*If tail B active	*/
{
	actv_tail_value   = iregs->cin_tail_ptr_b ;	/*Get actv tail value	*/
	non_actv_tail_ptr = &(iregs->cin_tail_ptr_a) ;	/*Pt to non-actv tail	*/
}

/*Upate tail pointer by performing MOD INC					*/
tail_displ          = (actv_tail_value + 1) & q_displ_mask  ;	/*Offset + 1	*/
actv_tail_value     = (actv_tail_value & (~q_displ_mask)) | tail_displ ;
*non_actv_tail_ptr  = actv_tail_value ;
iregs->cin_q_cntrl ^= 0x10 ;				/*Change tails		*/
return ( 0 ) ;
}						/*End of INC Tail function	*/








/*#stitle INTERNAL/LOCAL FN - GAIN ACCESS TO SSPCB LIST (mpa_fn_access_sspcb_lst)#*/
/*#page*/
/****************************************************************************************
 *                                                                             		*
 *               	   GAIN ACCESS TO SSPCB LIST					*
 *                                                                     			*
 * PURPOSE: To gain exclusive access to list of Registered SSPCBs by Blocking 		*
 *	   pre-emption and (if appropriate) LOCKING SSPCB List.				*
 *											*
 * NOTES:										*
 *                                                                     			*
 * CALL: unsigned char   mpa_fn_access_sspcb_lst (unsigned long int FARm  *pre_empt_msk)*
 *                                                                     			*
 * INPUT:  pre_empt_msk = pointer for return parm, state of pre-emption (interrupt mask)*
 *			  prior to Blocking pre-emption					*
 *											*
 * RETURN: Function Status:								*
 *		0x00 = Failure, Access to SSPCB NOT granted (pre_empt_msk = invalid)	*
 *		0xFF = Access to SSPCB granted (pre_empt_msk returned)			*
 *                                                                     			*
 ****************************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
unsigned char   mpa_fn_access_sspcb_lst (unsigned long int FARm *pre_empt_msk)
#else
unsigned char   mpa_fn_access_sspcb_lst (pre_empt_msk)

unsigned long int FARm *pre_empt_msk;
#endif

{

/************************
 *     LOCAL DATA	*
 ************************/


/************************
 *   START OF CODE	*
 ************************/

*pre_empt_msk = (*admin_diag_hdr.os_functs.blk_ints_fn)(); /*Block pre-emption	 */
#ifndef NO_FN_PROTO_PARMS 				/*Compile if Protypes OK */
if ( admin_diag_hdr.os_functs.init_lock_fn == (void FARm *(*)(unsigned long int FARm *))0 )
#else							/*Compile if NO Protypes */
if ( admin_diag_hdr.os_functs.init_lock_fn == (void FARm *(*)())0 )
#endif
{						/*System doesn't support LOCKing */
	return ((unsigned char) 0xFF) ;		/* pre-emption blocked, access to*/
}						/*  SSPCB List granted		 */

/*Pre-emption blocked and system supports LOCKNG, attempt to LOCK SSPCB List	*/
if ( ((*admin_diag_hdr.os_functs.attmpt_lock_fn)(admin_diag_hdr.sspcb_lst_lck_hndl)) == (unsigned char)0xFF )
		return ((unsigned char) 0xFF) ;	/*If Lock granted, grant access	*/

/*Unable to LOCK, restore pre-emption (interrupt) mask and indicated to caller	 */
/*that access to SSPCB List not granted						 */
(*admin_diag_hdr.os_functs.rstr_ints_fn)(*pre_empt_msk) ;
return ((unsigned char) 0x00) ;

}






/*#stitle INTERNAL/LOCAL FN - RELEASE ACCESS TO SSPCB LIST (mpa_fn_relse_sspcb_lst)#*/
/*#page*/
/****************************************************************************************
 *                                                                             		*
 *               	   RELEASE ACCESS TO SSPCB LIST					*
 *                                                                     			*
 * PURPOSE: To rlease exclusive access to list of Registered SSPCBs by clearing (if 	*
 *	   appropriate) SSPCB List LOCK and Clearing Pre-emption Block			*
 *											*
 * NOTES:										*
 *                                                                     			*
 * CALL: void	mpa_fn_relse_sspcb_lst (unsigned long int   pre_empt_msk)		*
 *                                                                     			*
 * INPUT:  pre_empt_msk = state of pre-emption (interrupt mask) prior to pre-emption 	*
 *			  being Blocked (parm returned by mpa_fn_access_sspcb_lst)	*
 *											*
 * RETURN: None										*
 *                                                                     			*
 ****************************************************************************************/

#ifndef 		NO_FN_PROTO_PARMS	/*R11*/
void		mpa_fn_relse_sspcb_lst (unsigned long int   pre_empt_msk) 
#else
void		mpa_fn_relse_sspcb_lst (pre_empt_msk)

unsigned long int   pre_empt_msk;
#endif

{

/************************
 *     LOCAL DATA	*
 ************************/


/************************
 *   START OF CODE	*
 ************************/

#ifndef NO_FN_PROTO_PARMS 				/*Compile if Protypes OK*/
if ( admin_diag_hdr.os_functs.init_lock_fn != (void FARm *(*)(unsigned long int FARm *))0 )
#else							/*Compile if NO Protypes*/
if ( admin_diag_hdr.os_functs.init_lock_fn != (void FARm *(*)())0 )
#endif
	(*admin_diag_hdr.os_functs.unlock_fn)(admin_diag_hdr.sspcb_lst_lck_hndl) ;
(*admin_diag_hdr.os_functs.rstr_ints_fn)(pre_empt_msk) ; /*Clear pre-emption blk*/
return ;

}



/*END*/

