/*****************************************************************************/
/*      EQUINOX SuperSerial Technology (SST) Driver for Linux                */
/*                                                                           */
/*      Copyright (C) 1989-2002 Equinox Systems Inc.                         */
/*****************************************************************************/
                                                                               
#define STATIC
#define __NO_VERSION__
#include <linux/config.h>
#include <linux/version.h>

#ifdef CONFIG_MODVERSIONS
#define MODVERSIONS	1
#endif

#ifdef MODVERSIONS
#include <linux/modversions.h>
#endif /* MODVERSIONS */

#include <linux/module.h>
#define EQNX_VERSION_CODE LINUX_VERSION_CODE
#include <linux/errno.h>

#ifdef  MODULE_LICENSE
MODULE_LICENSE("GPL");
#endif  /* MODULE_LICENSE */                                                

/* Correct compile problems on Mandrake 6.? and SuSE 7.0 */
#ifdef __SMP__
#ifndef CONFIG_X86_LOCAL_APIC
#define CONFIG_X86_LOCAL_APIC
#endif
#endif
                
#include <linux/sched.h> 
#include <linux/timer.h>
#include <linux/wait.h>
#include <linux/interrupt.h>
#include <linux/fcntl.h>
#include <linux/tty.h>
#include <linux/tty_flip.h>
#include <linux/serial.h>
#include <linux/string.h>
#include <linux/delay.h>
#include <linux/major.h>
#include <linux/mm.h>
#include <linux/ioport.h>
#include <linux/slab.h>
#include <linux/vmalloc.h>

#if	(LINUX_VERSION_CODE >= 132096)
/* 2.4 kernels and after */
#include <linux/list.h>
#include <linux/isapnp.h>
#endif

#include <asm/io.h>
#if	(LINUX_VERSION_CODE >= 132096)
#include <linux/spinlock.h>
#else
#include <asm/semaphore.h>
#endif

#if (EQNX_VERSION_CODE > 131327)
#include <asm/uaccess.h>  /* 2.1.x+ kernel changes */
#endif

#if (EQNX_VERSION_CODE < 66304)
#include <asm/system.h>
#include <asm/segment.h>
#include <asm/bitops.h>
#include <asm/page.h>
#include <asm/pgtable.h>
extern char kernel_version[];
#endif

static void eqn_print_trace(void);
extern int maxbrd;
extern int din_num[], dout_num[], diag_num;

/*
 * Define hardware specific data types
 * needed by this driver.
 */
#include "version.h"
#include "icp.h"
#include "ticp.h"
#include "sst.h"
#include "eqnx.h"
#include "ist.h"
#include "eqnx_pci.h"
/* The following is for Star Technologies */
#include "mra.h"
/* End Star Technologies */
#include "eqn.h"

#if (EQNX_VERSION_CODE < 131328)  /* kernel 2.0.x and below */
#define ioremap vremap
#define iounmap vfree
#define eqn_to_user memcpy_tofs
#define eqn_from_user memcpy_fromfs
#define eqn_fatal_signal (current->signal & ~current->blocked)

#else				/* kernel 2.1.x and above */
#define put_fs_int put_user
#define eqn_to_user copy_to_user
#define eqn_from_user copy_from_user
#define eqn_fatal_signal (signal_pending(current))
#endif

/* RAMP START --------------------------------------------- */
#define MAX_MESS_LEN 80

extern int eqnx_max_mess;
extern int eqnx_max_mess_len;
extern char eqnx_err_messages[][MAX_MESS_LEN];

void eqnx_err_save( char *module, unsigned int retv, unsigned int board, 
		unsigned int lmx, unsigned int port);

int eqnx_err_message_count = 0;
int eqnx_err_message_wrap = 0;
static char htoa[16] = {'0','1','2','3','4','5','6','7','8','9','a','b','c','d','e','f'};
static char dtoa[10] = {'0','1','2','3','4','5','6','7','8','9'};
#define END_STRING(index) \
eqnx_err_messages[eqnx_err_message_count][index++] = '\0';

#define SPACE(index) \
eqnx_err_messages[eqnx_err_message_count][index++] = ' ';

#define DTOC(index, d) { \
unsigned char d0[8]; \
int dtoc_j, dtoc_i; \
for(dtoc_j = 0, dtoc_i = 10000000; dtoc_i ; dtoc_j++, dtoc_i /= 10) { \
   if (d < dtoc_i) \
      d0[dtoc_j] = 0; \
   else { \
      d0[dtoc_j] = d / dtoc_i; \
      d /= 10; \
   } \
} \
dtoc_j = 0; \
while(d0[dtoc_j] == 0) dtoc_j++; \
for( ;dtoc_j < 8; dtoc_j++) \
   eqnx_err_messages[eqnx_err_message_count][index++] = dtoa[d0[dtoc_j]]; \
}
   
#define START_HEX(index) \
SPACE(index); \
eqnx_err_messages[eqnx_err_message_count][index++] = '0'; \
eqnx_err_messages[eqnx_err_message_count][index++] = 'x';

#define HTOC(index, h) \
eqnx_err_messages[eqnx_err_message_count][index++] = htoa[(h & 0xF0) >> 4]; \
eqnx_err_messages[eqnx_err_message_count][index++] = htoa[(h & 0x0F)];

#define WTOC(index, w) \
HTOC(index,(w & 0xFF00) >> 8); \
HTOC(index,(w & 0xFF));

#define LTOC(index,l) \
WTOC(index,(l & 0xFFFF0000) >> 16); \
WTOC(index,(l & 0xFFFF));

#ifdef	IA64
/*
** required due to problems with memcpy on IA64
*/
void memcpy_IA64(void *, void *, int);
#define	memcpy(a,b,c)	memcpy_IA64(a,b,c)
#endif	/* IA64 */

void eqnx_err_save( char *module, unsigned int retv, unsigned int board, 
		unsigned int lmx, unsigned int port)
{
	unsigned int i = 0;
	char *s0;

	if (eqnx_err_message_count >= eqnx_max_mess) {
		eqnx_err_message_wrap = 1;
		eqnx_err_message_count = 0;
	}
	s0 = &(eqnx_err_messages[eqnx_err_message_count][0]);
	LTOC(i,CURRENT_TIME);
	SPACE(i);
	strcpy(s0 + i, module);
	i = strlen(eqnx_err_messages[eqnx_err_message_count]);
	START_HEX(i);
	WTOC(i, retv);

	strcpy(s0 + i, " board(");
	i = strlen(eqnx_err_messages[eqnx_err_message_count]);
	DTOC(i,board);
	strcpy(s0 + i, ")");
	i = strlen(eqnx_err_messages[eqnx_err_message_count]);
	
	strcpy(s0 + i, " lmx(");
	i = strlen(eqnx_err_messages[eqnx_err_message_count]);
	DTOC(i,lmx);
	strcpy(s0 + i, ")");
	i = strlen(eqnx_err_messages[eqnx_err_message_count]);
	
	strcpy(s0 + i, " port(");
	i = strlen(eqnx_err_messages[eqnx_err_message_count]);
	DTOC(i,port);
	strcpy(s0 + i, ")");
	i = strlen(eqnx_err_messages[eqnx_err_message_count]);

	eqnx_err_message_count += 1;
}

/* End of code to save non-displayed messages */

/* Tests if the channel is on a RAMP */
#define ISRAMP(mpc) \
( mpc->mpc_icp->lmx[mpc->mpc_lmxno].lmx_id == 0x08 || \
mpc->mpc_icp->lmx[mpc->mpc_lmxno].lmx_id == 0x09 || \
mpc->mpc_icp->lmx[mpc->mpc_lmxno].lmx_id == 0x0B )

/* Returns the slot structure given the mpc */
#define GET_SLOT(mpc) \
(&(slot_dev[mpc - meg_chan]))

/* Returns modem type. Valid only if slot_state = 0xFF. 00 = ISA.  01 = PNP. */
#define MODEM_TYPE(mpc) \
( GET_SLOT(mpc)->type )

/* Returns modem slot state. */
#define SLOT_STATE(mpc) \
( GET_SLOT(mpc)->slot_state )

/* Returns 1 if power on, 0 if power off. */
#define POWER_STATE(mpc) \
((mpc->mpc_icp->slot_ctrl_blk + ((mpc->mpc_chan) % 64))->power_state == 0xFF)

/* Tests if any call back flags are set in the state parameter */
#define MPA_CALL_BACK(state)  \
   ((state &  MPA_INITG_UART) | \
   (state & MPA_INITG_SET_LOOP_BACK) | \
   (state & MPA_INITG_CLR_LOOP_BACK) | \
   (state & MPA_INITG_START_BREAK) | \
   (state & MPA_INITG_STOP_BREAK) | \
   (state & MPA_INITG_INIT_MODEM) | \
   (state & MPA_INITG_MODIFY_SETTINGS) | \
   (state & MPA_INITG_HARD_RESET))

/* Sets up mpd, slot_chan, port, lmx, icp, curmarb given the mpc */
#define PARAM_SETUP(mpc, mpd, slot_chan, port, lmx, icp, curmarb) \
   mpd = mpc->mpc_mpd; \
   slot_chan = (mpc->mpc_chan) % 64; /*0 - 63, channel relative to SSP*/ \
   port = slot_chan % 16; /* 0 - 15, channel relative to LMX */ \
   lmx = slot_chan / 16; /* 0 - 3, lmx relative to SSP */ \
   icp = mpc->mpc_icp; \
   curmarb = GET_MARB(mpc);

/* Returns the address of the MARB given the mpc */
#define GET_MARB(mpc) \
(&(((mpc->mpc_icp)->slot_ctrl_blk + ((mpc->mpc_chan) % 64))->marb))

/* returns the saved control signals given the mpc */
#define GET_SAVED_CTRL_SIGS(mpc) \
(((GET_SLOT(mpc))->sv_cout_cntrl_sig & 0x02BB) | 0x0004)

/* Sets parameter val to control signals given mpc */
#define GET_CTRL_SIGS(mpc,val) \
val = mpc->mpc_cout->ssp.cout_ctrl_sigs; \
if (ISRAMP(mpc)) { \
   unsigned int state = (MPA_INITG_SET_LOOP_BACK | MPA_INITG_CLR_LOOP_BACK); \
   if (MPA_CALL_BACK(mpc->mpc_mpa_stat & ~(state))) { \
      if (mpc->mpc_mpa_cout_ctrl_sigs) \
         val = mpc->mpc_mpa_cout_ctrl_sigs; \
      else \
         val = GET_SAVED_CTRL_SIGS(mpc); \
   } \
}

/* sets control signals to val parameter given mpc */
#define SET_CTRL_SIGS(mpc, val) \
if (ISRAMP(mpc)) { \
   unsigned int set_sigs_state = (MPA_INITG_SET_LOOP_BACK | MPA_INITG_CLR_LOOP_BACK); \
   if (MPA_CALL_BACK(mpc->mpc_mpa_stat & ~set_sigs_state)) \
      mpc->mpc_mpa_cout_ctrl_sigs = val; \
   else \
      (mpc->mpc_icpo)->ssp.cout_ctrl_sigs = val; \
} else \
   (mpc->mpc_icpo)->ssp.cout_ctrl_sigs = val;

/* Prints or saves formatted RAMP message */
#define MESSAGE(module, retv, mpc) { \
unsigned int m_slot_chan = (mpc->mpc_chan) % 64; \
unsigned int m_port = (m_slot_chan % 16) + 1; \
unsigned int m_lmx = (m_slot_chan / 16) + 1; \
unsigned int m_board = mpc->mpc_brdno + 1; \
if (eqn_ramp_trace /*&& !eqn_quiet_mode*/) \
{ \
printk("%s 0x%.4X board(%d) port(%d) lmx(%d)\n",module,retv,m_board,m_port,m_lmx); \
}\
else \
eqnx_err_save(module,retv,m_board,m_lmx,m_port); \
}
/*printk("%s 0x%.4X board(%d) port(%d) lmx(%d)\n",module,retv,m_board,m_port,m_lmx); \*/

/* 
 * Prints RAMP formatted message. Also saves it if verbose messages turned 
 * off. 
 */
 /*eqn_ramp_trace = 1; */
#define PRINT(module, retv, mpc) { \
int p_save = eqn_ramp_trace; \
MESSAGE(module, retv, mpc); \
eqn_ramp_trace = p_save; \
/* If saving other messages, save these too. */ \
if (!eqn_ramp_trace /*|| eqn_quiet_mode*/) \
MESSAGE(module, retv, mpc); \
}
/* RAMP END ----------------------------------------------- */

struct sys_functs_struct ramp_import_blk = {0};

struct ist_struct ist; /* Installation Status Table */

int register_eqnx_dev(void);
static int SSTMINOR(dev_t dev);
static int megastty( struct mpchan *mpc, struct tty_struct *tp, 
		int arg, int dev);
#if (EQNX_VERSION_CODE > 131327)
static ssize_t eqnx_diagread(struct file *fp, char *buf, size_t count, loff_t *fpos);
#else
static int eqnx_diagread(struct inode *ip, struct file *fp, char *buf, 
		int count);
#endif
STATIC int eqnx_diagopen(struct inode *ip, struct file *fp);
STATIC int eqnx_diagclose(struct inode *ip, struct file *fp);
STATIC int eqnx_diagioctl(struct inode *ip, struct file *fp, unsigned int cmd,
            unsigned long arg);
int move_brd_addr( unsigned long int addr);
#ifndef MODULE
static void eqnx_meminit(long base);
static long eqnx_memhalt(void);
#endif
static void *eqnx_memalloc(int len);
static int eqnx_setserial(struct mpchan *mpc, struct serial_struct *sp);
static int set_modem_info(struct mpchan *mpc, unsigned int cmd,
		unsigned int *value);
static void eqnx_getserial(struct mpchan *mpc, struct serial_struct *sp);
static int eqnx_setserial(struct mpchan *mpc, struct serial_struct *sp);
static uchar_t eqn_cs_get( struct mpchan *mpc);
static void megatxint(struct mpchan *mpc);
static void megainput( register struct mpchan *mpc);
static void megamint(register struct mpchan *mpc);
static void megasint(register struct mpchan *mpc);
STATIC int eqnx_ioctl(struct tty_struct *tty, struct file * file,
            unsigned int cmd, unsigned long arg);
static void inline sst_write( struct mpchan *mpc, 
		unsigned char *buf, int count);
static void inline sst_write1(struct mpchan *mpc, int func_type);
static void eqnx_flush_chars(struct tty_struct *tty);
static int get_modem_info(struct mpchan *mpc, unsigned int *value);
static int mega_push_winicp( struct mpdev *mpd, int icp);
static void mega_rdv_wait( unsigned long arg);
static void mega_ldv_hangup( icpaddr_t icp);
static void mega_rdv_delta( icpaddr_t icp, struct mpchan *mpc, int rng_state);
static int mega_rng_delta( icpaddr_t icp, int reason, int nchan);
static void eqnx_dohangup(void *arg);
static void eqnx_flush_buffer(struct tty_struct *tty);
static void eqnx_delay(int len);
static int megajam( register struct mpchan *mpc, char c);
static int frame_wait( register struct mpchan *mpc, int count);
static int frame_ctr_reliable( icpgaddr_t icpg, register struct mpchan *mpc);
static int mega_pop_win(int instance);
static int mega_push_win( struct mpchan *mpc, int typ);
static int megaparam( int d);
static int megamodem(int d, int cmd);
static  ushort_t icpbaud( int val, struct mpchan *mpc);
static int chanon( struct mpchan *mpc);
static int chanoff( struct mpchan *mpc);
static int cur_chnl_sync( register struct mpchan *mpc);
#ifdef	MCA_ENAB
static uint_t mca_mem_strat( uchar_t *regs );
static uint_t mca_mem_size( uchar_t *regs, int req );
static paddr_t mca_base_paddr( uchar_t *regs );
static int mca_icp_count( uchar_t id );
static int mca_brd_found( int slot, uchar_t *regs);
#endif	/* MCA_ENAB */
static uint_t pci_mem_size( unsigned short id );
static unsigned long int ramp_block_int(void);
static void ramp_unblock_int(unsigned long int flags);
void ramp_fsm( struct mpchan *mpc);
void *ramp_map_fn( void *mpc);
void ramp_unmap_fn(void *mpc);
void ramp_hard_reset( struct mpchan *mpc);
void ramp_hard_reset_error( struct mpchan *mpc, int retv);
void ramp_modem_cleanup( struct mpchan *mpc);
void ramp_init_modem_error( struct mpchan *mpc, int retv);
void ramp_set_loop_back_error( struct mpchan *mpc, int retv);
void ramp_clr_loop_back_error( struct mpchan *mpc, int retv);
int ramp_clr_loop_back( struct mpchan *mpc);
int ramp_set_loop_back( struct mpchan *mpc);
int ramp_start_break( struct mpchan *mpc);
void ramp_start_break_error( struct mpchan *mpc, int retv);
void ramp_stop_break_error( struct mpchan *mpc, int retv);
void ramp_stop_break( struct mpchan *mpc);
void ramp_reg_modem_error( struct mpchan *mpc, int retv);
void ramp_dereg_modem_cleanup( struct mpchan *mpc);
void ramp_dereg_modem( struct mpchan *mpc);
int ramp_get_delay( struct mpchan *mpc);
static void eqnrflush( struct mpchan *mpc);
void ramp_reg_modem( struct mpchan *mpc);
void ramp_check_error( struct mpchan *mpc, int retv);
static int ramp_id_functs(void);
static void ramparam( struct mpchan *mpc);
int ramp_get_index( struct mpchan *mpc);
void ramp_set_index( struct mpchan *mpc);
void ramp_set_initg( struct mpchan *mpc, int index);
void ramp_init_modem( struct mpchan *mpc);
static ushort_t uart_baud( int val, struct mpchan *mpc);
int ramp_lmx_check(struct mpchan *mpc);

void brd_mem_cfg( struct mpdev *mpd );
int eqx_pci_buspresent(struct pci_csh *csh);

static int eisa_icp_count( ushort_t id );
static int mem_zero( struct mpdev *mpd, unsigned short *ramw, 
		int testlen, int icp);
static int mem_test(struct mpdev *mpd, int icp);
int mem_test_dram( struct mpdev *mpd, ushort_t *ramw, int testlen, int icp );
int mem_test_pram( struct mpdev *mpd, ushort_t *ramw, int testlen, int icp );
int mem_test_tag(struct mpdev *mpd, uchar_t *ramb, int testlen, int icp);
int test_8bit_hole(struct mpdev *mpd, int k, paddr_t addr );
#ifdef	ISA_ENAB
static int eisa_brd_found( int slot, ushort_t *regs );
static paddr_t eisa_base_paddr( ushort_t *regs );
static uint_t eisa_mem_size( ushort_t *regs, int req );
static uint_t eisa_mem_strat( ushort_t *regs );
int eisa_brdtyp_ok( ushort_t bbbbbrrr, ushort_t cfg );
#endif	/* ISA_ENAB */
static uchar_t fifo_mask [] = { 0x0, 0x3, 0xf, 0x3f, 0xff, 0x3, 0xf, 0x3f, 0xff};

uint_t icpbaud_tbl[] = { 0, 50, 75, 110, 134, 150, 200, 300, 
                           600, 1200, 1800, 2400, 4800, 9600, 19200, 38400,
				57600, 115200,230400, 460800, 921600};

#define NCHAN		64		/* usable channels per SSP64 */
#define NCHAN_BRD	128		/* max channels per board */
#ifndef MIN
#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
#endif
#ifndef MAX
#define MAX(a,b)	(((a) > (b)) ? (a) : (b))
#endif
#define MPDIALIN	0x1000		/* dial-in line */
#define MPSHARED	0x2000		/* uses channel pairing */
#define MPNOMODEM	0x4000		/* modem lines not used */
#define MPSCRN		0x8000		/* multi screen */

#define EQNX_RAW	0
#define EQNX_COOKED	1
#define EQNX_TXINT	2

#define TURNOFF		0		/* turn this channel off */
#define TURNON		1		/* turn this channel on */

#define LOCK_A		1		/* Bank A lock bit */
#define LOCK_B		2		/* Bank B lock bit */

#define MPTIMEO		((HZ*10)/1000)	/* 10ms. polling interval */
/*
#define CLSTIMEO	(HZ*20)		 20sec. close (drain) timeout */

#define CLSTIMEO	0		/* no timeout */
#define EQNX_CLOSEDELAY ((HZ*500)/1000)	/* 1/2 second */

#define false		0
#define true		(!false)	
#define	HWREGSLEN	0x4000
#define HWRXQWRAP	3
#define HWTXQWRAP	0x0b

#define EISA_EQX        0x3816
#define EISA_REGS_LEN	9
#define	EISA_MEM_LEN	0x200000
#define DSQSIZE		4096 	/*0x1000*/
#define DSQMASK		4096-1  /*0x0fff*/
/* Note: chan is set and cleared via the EQNCHMONSET/CLR ioctls */

static struct {
	int open;		/* true/false: Is the the data monitor open */
	int chan;		/* port number being scoped, meg_chan index*/
	int status;		/* status (for WRAP) */
#define DSQWRAP	0x01
	queue_t q;		/* Manage circular q buffer */
	int next;		/* queue structure does not have this */
	char buffer[DSQSIZE];	/* Actual circular buffer */

#if	(LINUX_VERSION_CODE < 132096)
	/* kernels before 2.4 */
	struct wait_queue *scope_wait;
#else
	/* 2.4 kernels and after */
	wait_queue_head_t scope_wait;
#endif

} dscope[2];

/*****************************************************************************/

#if	(LINUX_VERSION_CODE < 132096)
	/* kernels before 2.4 */
static struct file_operations	eqnx_fdiag = {
	NULL,
	eqnx_diagread,
	NULL,
	NULL,
	NULL,
	eqnx_diagioctl,
	NULL,
	eqnx_diagopen,
#if (EQNX_VERSION_CODE > 131327)
	NULL,		/* Pat added for > 2.1.x kernel */
#endif	
	eqnx_diagclose,
	NULL
};
#else
	/* 2.4 kernels and after */
static struct file_operations	eqnx_fdiag = {
	read:	eqnx_diagread,
	ioctl:	eqnx_diagioctl,
	open:	eqnx_diagopen,
	release: eqnx_diagclose,
};
#endif

/*****************************************************************************/

#ifdef	MCA_ENAB
#define MCA_REGS_LEN	12		/* max bytes of info */
#define MCA_EQX_64	0x6388
#define MCA_EQX_128	0x6389
#define MCA_EQX_8	0x638a
static
ushort_t mca_iobase[] = { 0x3000, 0x3080, 0x3400, 0x3480,
                          0x3800, 0x3880, 0x3c00, 0x3c80,
                          0x7000, 0x7080, 0x7400, 0x7480,
                          0x7800, 0x7880, 0x7c00, 0x7c80,
                          0xb000, 0xb080, 0xb400, 0xb480,
                          0xb800, 0xb880, 0xbc00, 0xbc80,
                          0xf000, 0xf080, 0xf400, 0xf480,
                          0xf800, 0xf880, 0xfc00, 0xfc80 };
#endif	/* MCA_ENAB */

#define FLAT128_MEM_LEN	0x200000
#define FLAT64_MEM_LEN	0x100000
#define FLAT8_MEM_LEN	0x10000
#define FLAT16K_MEM_LEN	0x4000
#define FLAT32K_MEM_LEN	0x8000
#define FLAT64K_MEM_LEN	0x10000
#define	WIN16_MEM_LEN	0x4000


#define HA_FLAT 0
#define HA_WIN16	1

#define HWCMDQSIZE	0
#define CIN_DEF_ENABLES		0x3800
#define COUT_DEF_ENABLES	0x8000
#define VTIMEO		192
#ifndef uint_t
#define uint_t unsigned int
#endif

#define RNG_BAD         0
#define RNG_GOOD        1
#define RNG_CHK		2
#define RNG_WAIT	3
/* NOTE: RNG_FAIL (value 4) is also used in mega_rng_delta() */

#define DEV_VIRGIN      0xff            /* device id is unknown */
#define DEV_BAD         0               /* device was known but now offline */
#define DEV_GOOD        1               /* device known and online */
#define DEV_INSANE      2               /* hardware error (unexpected) */
#define DEV_WAITING	3
#define MAXLDVMPC	16
#define WINDO_DEPTH	128

#define LINK_OFF	0x80

#define MM_COUNTRY_CODE_REG 	0x8800

static struct hwq_struct sst_hwq[] =
{
  { HWQ4SIZE, HWQ4BITS, HWQ4HIWAT, HWQ4LOWAT,
    HWQ4RXWRAP, HWQ4TXWRAP, HWQ4CMDSIZE,HWQ4MINCHAR },
  { HWQ2SIZE, HWQ2BITS, HWQ2HIWAT, HWQ2LOWAT,
    HWQ2RXWRAP, HWQ2TXWRAP, HWQ2CMDSIZE,HWQ2MINCHAR }
#ifdef SSM
  ,{ HWQ1SIZE, HWQ1BITS, HWQ1HIWAT, HWQ1LOWAT,
     HWQ1RXWRAP, HWQ1TXWRAP, HWQ1CMDSIZE,HWQ1MINCHAR },
  {  HWQ_5SIZE, HWQ_5BITS, HWQ_5HIWAT, HWQ_5LOWAT,
     HWQ_5RXWRAP, HWQ_5TXWRAP, HWQ_5CMDSIZE,HWQ_5MINCHAR }

#endif

};

/* It is possible to sleep in copyin/copyout due to 
   swapping and/or paging.  In order to ensure that
   multiple processes don't sleep and wake up in a
   different order and then use the incorrect window info
   from the stack, it is necessary to lock the window
   when doing the copyin/out calls.  In this way, only one
   process (context) can sleep with information on the 
   stack.  Other uses of the window stack (e.g., megapoll) 
   are ok.  They will push their window usage and pop them 
   correctly (even if interrupted by megapoll). By design,
   window information is not left on the stack when sleeping.
   However, the window must be active when doing copyin/out
   in bypass mode.  This locking mechanism ensures that only
   one process can be doing a copyin/out at a time. NOTE: this
   only applies to copyin/out requests that are moving data
   from the board.  Copies of data structures (between 
   kernel memory and user memory) are not affected.
   Monty 9/14/94  */

struct windo_stk
{
	ushort	pgctrl;
	ushort	pgdata;
        struct mpdev *ws_mpd;
};



/* SSM, testing found ... */
unsigned int mptimeo = MPTIMEO;
unsigned int ramp_admin_poll = RAMP_FAST_TIME;
int eqn_num_ramps = 0;

static int wtos = 0;			/* window stack index */
int wtos_max = 0;               /* to debug window code */
int last_pop = 1;		/* to debug window code */
int last_pop1 = 0;		/* to debug window code */
int last_pop2 = 0;		/* to debug window code */
int last_pop3 = 0;		/* to debug window code */
int last_pop4 = 0;		/* to debug window code */
int last_pop5 = 0;		/* to debug window code */
int last_pop6 = 0;		/* to debug window code */
int last_pop7 = 0;		/* to debug window code */

unsigned int poll_cnt = 0;	/* used to delay input processing in megapoll */
/* The following tables assign the transmit Q low water mark based on
   baud rate. It is based on (#chars per 40 milliseconds/64) plus some slop. 

   lowat baud rates = 0, 50, 75, 110, 134, 150, 200, 300, 
                           600, 1200, 1800, 2400, 4800, 9600, 19200, 38400 
				57600, 115200,230400, 460800, 921600

   The table contains the result. 1.07 */

uint_t lowat[] = { 0, 2, 2, 2, 2, 2, 2, 2, 
                           2, 2, 2, 2, 2, 2, 2, 4, 
				8, 14, 20, 40, 80};

/* The following table assigns the transmit Q low water mark based on
   baud rate. It is based on (#chars per 10 milliseconds/64) plus some 
   slop. */
uint_t ssp4_lowat[] = { 0, 2, 2, 2, 2, 2, 2, 2, 
                           2, 2, 2, 2, 2, 2, 2, 2, 
				2, 3, 5, 8, 12};

int eqn_ramp_trace = 0;	/* RAMP verbose messages flag */
int eqn_quiet_mode = 0;	/* suppress ring config messages when true */
int eqn_save_trace = 0; /* Remember ramp_trace flag when in quiet_mode */
struct cst_struct brd_cst[MAXBRDCHNL]; /* used in brdstatus ioctl */
static int nextmin;
static	int nmegaport = 0;			/* number of boards found */
static	int nicps;			/* number of icps found */
static int to_eqn_ramp_admin = -1;
	/*No Space.c for LINUX for now*/

#define IOBUSTYPE 1  			/* I/O bus type: 1=ISA,2=EISA,3=MCA */

#define MAX_BACKDOOR_PORTS 4

#ifdef	ISA_ENAB
extern unsigned long int MegAddr;
#endif	/* ISA_ENAB */


int dtr_down = 9; /* Drop DTR for 900 milliseconds when HUPCL */
int io_bus_type = IOBUSTYPE;
int EQNft = 0; /* For Star Technologies, make 1 for sentinel/mra */

struct mpdev meg_dev[NMEGAPORT]; /* Adapter struct */
struct ssp_struct ssp_dev[NMEGAPORT * NSSP4S]; /* sspcb struct */
struct slot_struct slot_dev[MAXCHNL];
struct mpchan *meg_chan;

static ushort_t portn[] = {0x200, 0x220, 0x240, 0x260, 0x280, 0x2a0, 0x2c0, 
		0x2e0, 0x300, 0x320, 0x340, 0x360, 0x380, 0x3a0, 0x3c0, 0x3e0};

struct windo_stk wstk[WINDO_DEPTH];
#if (EQNX_VERSION_CODE < 66304)
#define	PAGE_IOMEM	__pgprot(_PAGE_PRESENT | _PAGE_RW | _PAGE_DIRTY | _PAGE_ACCESSED | _PAGE_PCD)
#endif

#ifdef	ISA_ENAB
static uchar_t cmn_irq[] = {0xff, 0x3, 0x4, 0x5, 0x7, 0xa, 0xb, 0xc};
static int isa_icp_count( ushort id );
#endif	/* ISA_ENAB */

uint_t brd_nchan_cfg( int bid );

#include "ifns.c" /* Source file for finding isa holes etc. */
#include "pci.c" 


/* serial subtype definitions */
#define SERIAL_TYPE_NORMAL	1
#define SERIAL_TYPE_CALLOUT	2
static int eqnx_refcount;
#define pcisize  0x44			/* PCI data structure size */
char *eqnPCIcsh;
struct tty_driver *eqnx_driver; 
#if (LINUX_VERSION_CODE < 132096)
struct tty_driver *eqnx_callout_driver;
#endif
struct tty_struct **eqnx_ttys;
struct termios **eqnx_termios;
struct termios **eqnx_termioslocked;
static struct timer_list lmx_wait_timer;
static struct timer_list eqnx_timer, eqnx_ramp_timer;
void sstpoll(unsigned long arg);
void eqn_ramp_admin(unsigned long arg);

/*	Create temp write buffer for copying from user space to avoid
 	page fault problems.  */

static char	*eqnx_tmpwritebuf = (char *) NULL;

#if	(LINUX_VERSION_CODE < 132096)
	/* kernels before 2.4 */
static struct semaphore  eqnx_tmpwritesem = MUTEX;
#else
	/* 2.4 kernels and after */
static struct semaphore  eqnx_tmpwritesem;
#endif

/*	Local buffer for copying output characters. Used in eqnx_put_char */
static char	*eqnx_txcookbuf = (char *) NULL;
static int	eqnx_txcooksize = 0;
static int	eqnx_txcookrealsize = 0;
static struct tty_struct	*eqnx_txcooktty = (struct tty_struct *) NULL;

/*	Default termios structure.  */

static struct termios		eqnx_deftermios = {
	0,
	0,
	/*(B9600 | CS8 | CREAD | HUPCL),*/
	(B9600 | CS8 | CREAD | HUPCL | CLOCAL),
	0,
	0,
	INIT_C_CC
};

#define	XMIT_BUF_SIZE		4096


/*
 *	Memory allocation vars. These keep track of what memory allocation
 *	we can currently use. They help deal with memory in a consistent
 *	way, whether during init or run-time.
 */
static int	eqnx_meminited = 0;
static long	eqnx_memend;
#ifdef MODULE
int init_module(void);
void cleanup_module(void);
#endif

#if	(LINUX_VERSION_CODE >= 132096)
#ifdef	ISA_ENAB
/*
** ISA plug-and-play in linux kernels 2.4+
*/
#define	MAX_PNP_DEVS	4
struct	pci_dev	*pnp_found_devs[MAX_PNP_DEVS];
int	pnp_found = 0;
#endif
#endif

/*
 * Channel open - called whenever any sst channel
 * is opened by a process.  
 */

int eqnx_open(struct tty_struct *tty, struct file * filp)
{
	register struct mpchan *mpc;
	register struct tty_struct *tp;
	int d;
	struct mpdev *mpd;
	int win16;
	int nchan;
	int rc = 0;
	int flags = 0;

	MOD_INC_USE_COUNT;
	save_flags(flags);
	cli();
	d = SSTMINOR(tty->device);
#ifdef DEBUG
	printk("eqnx_open:for device %d\n",d);
#endif
	if((d > nmegaport*NCHAN_BRD) || (nmegaport == 0)
		|| (d < 0)) {
/*		MOD_DEC_USE_COUNT;*/
		restore_flags(flags); 

		return(-ENODEV);
	}

#ifdef SSM /* found testing SSM */
        if ( meg_dev[MPBOARD(d)].mpd_alive == 0){
		restore_flags(flags); 

		return(-ENODEV);
	}
#endif
        mpc = &meg_chan[d];
	if ((mpd = mpc->mpc_mpd) == (struct mpdev *) NULL){
		restore_flags(flags);

		return(-ENODEV);
	}

/* Do not allow ports to open on SST-16 boards without a panel
     0x17F8 == SSP4 SST-16I with no panel 
     0x80C8 == SSP4 SST-16P with no panel */
	if (((mpd->mpd_id & 0xFFF8) == 0x17F8) || 
	    ((mpd->mpd_id & 0xFFF8) == 0x80C8)){
		restore_flags(flags);

		return(-ENODEV);
	}

	nchan = MPCHAN(d);
	if (mpd >= &meg_dev[nmegaport] || mpd->mpd_alive == 0 ||
		nchan >= (int)mpd->mpd_nchan){
		restore_flags(flags); 

		return(-ENODEV);
	}
/* Verify that the hardware for this port is in a good state */
	if (mpc->mpc_icp->icp_rng_state != RNG_GOOD){
		restore_flags(flags);

		return(-ENODEV);
	}
	if (mpc->mpc_icp->lmx[mpc->mpc_lmxno].lmx_active != DEV_GOOD){
		restore_flags(flags); 

		return(-ENODEV);
	}
/* if a bridge, check the mux */
	if (mpc->mpc_icp->lmx[mpc->mpc_lmxno].lmx_type & 1) 
		if (mpc->mpc_icp->lmx[mpc->mpc_lmxno].lmx_rmt_active != DEV_GOOD){
			restore_flags(flags);

			return(-ENODEV);
	}

#ifdef SSM
      /* check for channel outside valid range, ie
           PM8      (0-7)
           ssm-12X  (0-11)
           ssm-24X  (0-11, 16-27)
     */
	if ((mpd->mpd_id & 0xF8) < SSP2_ASIC) /* only applies to SSP64 */ 
            if ( ( MPCHAN(d) % 16) >= mpc->mpc_icp->lmx[mpc->mpc_lmxno].lmx_chan)
            {
#if 0
                 printk("eqnx_open: mpchan(%d) A(%d) lmx_no(%d) lmx_chan(%d)\n",
                   MPCHAN(d),MPCHAN(d)%16,mpc->mpc_lmxno,
                      mpc->mpc_icp->lmx[mpc->mpc_lmxno].lmx_chan);
#endif
		restore_flags(flags);

		return(-ENODEV);
            }
#endif

	win16 = mpd->mpd_nwin;
/* RAMP START --------------------------------------------- */
	/* If RAMP config not completed, don't let any sends in */
	if (ISRAMP(mpc)) {
		icpiaddr_t icpi;
		icpbaddr_t icpb;
      /* check break bit */
      /* If not receiving a break, the slot is occupied by a UART */
      
		if ( !(mpc->mpc_mpa_stat & MPA_UART_CFG_DONE)){
      			icpi = mpc->mpc_icpi;
			if(win16) mega_push_win(mpc, 0);
      			icpb=(rx_locks & LOCK_A) ? &icpi->ssp.cin_bank_b 
				: &icpi->ssp.cin_bank_a;
  			if (!(icpb->ssp.bank_events & EV_REG_UPDT)) 
  				frame_wait(mpc,2); /* make sure regs are valid*/ 
         		if (!(icpb->ssp.bank_sigs & 0x01 )) {
				if(win16) 
					mega_pop_win(80);
            			/* Modem present in slot, sleep */
modem_wait:
				restore_flags(flags); 
				interruptible_sleep_on(&mpc->mpc_mpa_call_back);
				save_flags(flags);
				cli();
				if eqn_fatal_signal {
					restore_flags(flags);

					return(-ERESTARTSYS);
				}
				if(mpc->mpc_mpa_stat & MPA_CALL_BACK_ERROR){
					restore_flags(flags);

                        		return(-ENXIO);
				}
				else if ( !(mpc->mpc_mpa_stat & MPA_UART_CFG_DONE)){
					goto modem_wait;
				}
         		} else {
				if(win16) 
					mega_pop_win(81);
            			/* No modem, return error */
				restore_flags(flags); 
#ifdef DEBUG
printk("eqnx_open: No modem at device %d\n", d);
#endif

				return(-ENODEV);
         		}
		}
	}
/* RAMP END ----------------------------------------------- */

	mpc->refcount++;

	if(win16)
		mega_push_win(mpc, 0);

	if (!(mpc->flags & ASYNC_INITIALIZED)){
		mpc->mpc_tty = tty;
		tty->driver_data = mpc;
		if (tty->driver.subtype == SERIAL_TYPE_NORMAL)
			*tty->termios = *mpc->normaltermios;
#if (LINUX_VERSION_CODE < 132096)
		else
			*tty->termios = *mpc->callouttermios;
#endif
#ifdef RS422
        /*    force CLOCAL on for RS422 ports */
          if (mpc->mpc_icp->lmx[mpc->mpc_lmxno].lmx_id == LMX_8E_422 ||
               mpc->mpc_icp->lmx[mpc->mpc_lmxno].lmx_id == LMX_PM16_422)
          {
		tp = mpc->mpc_tty;
		tp->termios->c_cflag |= CLOCAL;
          }
#endif

		if((mpc->mpc_flags & MPC_OPEN) == 0) {
#ifdef DEBUG
		printk("eqnx_open:for device %d and flags = %x, calling megaparam with flags != ASYNC_INITIALIZED\n", d, (unsigned int) filp->f_flags);
#endif
		   chanon(mpc);
		   megaparam(d);
		}
		mpc->carr_state = megamodem(d, TURNON); /*shashi 02/02/98*/
		mpc->flags |= ASYNC_INITIALIZED;
		clear_bit(TTY_IO_ERROR, &tty->flags);
	}
	tp = mpc->mpc_tty;
/*
 *	If port is in the middle of closing, then wait. Get error status
 *	from flag settings.
 */
	if (mpc->flags & ASYNC_CLOSING) {
#ifdef DEBUG
	printk("eqnx_open:sleeping up on close_wait for device %d\n",d);
#endif
		if(win16) 
			mega_pop_win(81);
		restore_flags(flags); /* shashi 01/30/98 */
		interruptible_sleep_on(&mpc->close_wait);
		save_flags(flags); /* shashi 01/30/98 */
		cli();
		if (mpc->flags & ASYNC_HUP_NOTIFY){
			restore_flags(flags);

			return(-EAGAIN);
		}
		restore_flags(flags);

		return(-ERESTARTSYS);
	}


	if (tp->driver.subtype == SERIAL_TYPE_CALLOUT){
#ifdef DEBUG
	printk("opening dialout device %d with flags %x\n", d, (unsigned int) filp->f_flags);
#endif
		if (mpc->flags & ASYNC_NORMAL_ACTIVE){
			if(win16) 
				mega_pop_win(81);
			restore_flags(flags);

			return(-EBUSY);
		}
		if (mpc->flags & ASYNC_CALLOUT_ACTIVE){
			if((mpc->flags & ASYNC_SESSION_LOCKOUT) &&
				(mpc->session != current->session)){
				if(win16) 
					mega_pop_win(81);
				restore_flags(flags);

				return(-EBUSY);
			}
			if((mpc->flags & ASYNC_PGRP_LOCKOUT) &&
				(mpc->pgrp != current->pgrp)){
				if(win16) 
					mega_pop_win(81);
				restore_flags(flags);

				return(-EBUSY);
			}
		}
		mpc->flags |= ASYNC_CALLOUT_ACTIVE;
	}
	else{
#ifdef DEBUG
	printk("opening dialin device %d with flags %o\n", d, (unsigned int) filp->f_flags);
#endif
		if (filp->f_flags & O_NONBLOCK) {
			if (mpc->flags & ASYNC_CALLOUT_ACTIVE){
				if(win16) 
					mega_pop_win(81);
				restore_flags(flags);

				return(-EBUSY);
			}
#ifdef DEBUG
	printk("opening dialin device %d non_blocking with c_flags = %o\n", d,
		tp->termios->c_cflag);
#endif
		}
		else{
			mpc->openwaitcnt++;
			if (mpc->refcount)
				mpc->refcount--;
			while (1) {
				if (!(mpc->flags & ASYNC_CALLOUT_ACTIVE)){
					mpc->carr_state = megamodem(d, TURNON); /*shashi 02/02/98*/
				}
				if (tty_hung_up_p(filp) || ((mpc->flags & ASYNC_INITIALIZED) == 0)) {
					if (mpc->flags & ASYNC_HUP_NOTIFY)
						rc = -EBUSY;
					else
						rc = -ERESTARTSYS;
					break;
				}
				if (((mpc->flags & ASYNC_CALLOUT_ACTIVE) == 0) &&
						((mpc->flags & ASYNC_CLOSING) == 0) &&
						((tp->termios->c_cflag & CLOCAL) || (mpc->carr_state))){
#ifdef DEBUG
	printk("device %d open returning with c_cflag = %o, carr_state = %d\n", d, tp->termios->c_cflag, mpc->carr_state);
#endif
					break;
				}
				if eqn_fatal_signal {

/* Get compiler error here, if DEBUG, about
 * using bit masked array (current->signal)
 * - refered to as 'aggregate value' -
 * where integer is expected. 		

#ifdef DEBUG
	printk("device %d got a signal %lx\n", d, (unsigned long) current->signal);
#endif
*/
					rc = -ERESTARTSYS;
					break;
				}
#ifdef DEBUG
	printk("device %d going to sleep on open_wait\n", d);
#endif
				/*
				 * Wait if we're a dial-in line and DCD is low
				 */
				restore_flags(flags); /* shashi 01/30/98 */
				interruptible_sleep_on(&mpc->open_wait);
				save_flags(flags); /* shashi 01/30/98 */
				cli();
				if eqn_fatal_signal {

/* Get compiler error here, if DEBUG, about
 * using bit masked array (current->signal)
 * - refered to as 'aggregate value' -
 * where integer is expected. 		

#ifdef DEBUG
	printk("device %d got a signal %x\n", d, (unsigned int) current->signal);
#endif
*/

					rc = -ERESTARTSYS;
					break;
				}
				if (mpc->carr_state)
					break;
			}
			if(!tty_hung_up_p(filp))
				mpc->refcount++;
			mpc->openwaitcnt--;
			if (rc < 0){
				if(win16)
	   				mega_pop_win(1);
				restore_flags(flags);
#ifdef DEBUG
	printk("device %d returning with mpc_flags = %x, c_cflag = %o, carr_state = %d, rc = %d\n", d, mpc->flags, tp->termios->c_cflag, mpc->carr_state, rc);
#endif

				return(rc);
			}
		}
#ifdef DEBUG
	printk("device %d returning from open with mpc_flags = %x, c_cflag = %o, carr_state = %d, rc = %d\n", d, mpc->flags, tp->termios->c_cflag, mpc->carr_state, rc);
#endif
		mpc->flags |= ASYNC_NORMAL_ACTIVE;
	}
	if ((mpc->refcount == 1) && (mpc->flags & ASYNC_SPLIT_TERMIOS)) {
		if (tp->driver.subtype == SERIAL_TYPE_NORMAL)
			*tp->termios = *mpc->normaltermios;
#if (LINUX_VERSION_CODE < 132096)
		else
			*tp->termios = *mpc->callouttermios;
#endif
#ifdef DEBUG
	printk("SPLIT_TERMIOS: Calling megaparam for device %d\n", d);
#endif
		/*printk("eqnx_open:SPLIT_TERMIOS for device %d and flags = %x, calling megaparam \n", d, (unsigned int) filp->f_flags);*/
		/*megaparam(d);*/ /*shashi 3/23/98 */
	}
	if ((mpd->mpd_id & 0xF8) >= SSP2_ASIC)
		if (!mpc->xmit_buf){
			unsigned long page;
			page = get_free_page(GFP_KERNEL);
			if (!page){
				if(win16) 
					mega_pop_win(81);
				restore_flags(flags);

				return(-ENOMEM);
			}
			/*printk("allocated a free page for device %d\n", d);*/
			mpc->xmit_buf = (unsigned char *)page;
		}

	mpc->session = current->session;
	mpc->pgrp = current->pgrp;
	if(win16)
	   mega_pop_win(2);
      
	restore_flags(flags);
	
	return(0);
}
/*
 * Channel close called on last close of any megaport device.
 * Release resources, reset device, and wakeup any sleepers
 * waiting for channel to become idle.
 */

static void eqnx_close(struct tty_struct * tty, struct file * filp)
{
	register struct mpchan *mpc;
	struct mpdev *mpd;
	int nchan;
#ifndef LINUX
	register struct tty_struct *tp;
	struct tty_struct  *tp1;
	uchar_t scrn;
#endif
	int flags = 0;
	int d;
	int win16;

	d = SSTMINOR(tty->device);
	if(d > nmegaport*NCHAN_BRD)
		return;
        mpc = &meg_chan[d];
#ifdef DEBUG
	printk("close:device %d being closed with flags = %x\n", d, mpc->flags);
#endif
	if ((mpd = mpc->mpc_mpd) == (struct mpdev *) NULL)
		return;
	nchan = MPCHAN(d);
	if (mpd >= &meg_dev[nmegaport] || mpd->mpd_alive == 0 ||
		nchan >= (int)mpd->mpd_nchan)
		return;
	win16 = mpc->mpc_mpd->mpd_nwin;
	if(mpc->mpc_param & IOCTLLP) 
		return;
#ifdef DEBUG
printk("device %d being closedwith refcount = %d\n", d, mpc->refcount);
#endif
	save_flags(flags);
	cli();
	if (tty_hung_up_p(filp)){
		restore_flags(flags);
		MOD_DEC_USE_COUNT;
		return;
	}
	if (mpc->refcount)
		mpc->refcount -= 1;
	if (mpc->refcount) {
		restore_flags(flags);
#ifdef DEBUG
	printk("device %d being closed with refcount %d \n", d, mpc->refcount);
	printk("TTY_FLIPBUF_SIZE = %d and flip.count = %d for device %d with refcount\n",
		TTY_FLIPBUF_SIZE, tty->flip.count, d);
#endif
		MOD_DEC_USE_COUNT;
		return;
	}

	mpc->flags |= ASYNC_CLOSING;
#ifdef DEBUG
	printk("device %d being closed with flags = %x\n", d, mpc->flags);
#endif

	if (mpc->flags & ASYNC_NORMAL_ACTIVE)
		*mpc->normaltermios = *tty->termios;
#if (LINUX_VERSION_CODE < 132096)
	if (mpc->flags & ASYNC_CALLOUT_ACTIVE)
		*mpc->callouttermios = *tty->termios;
#endif
	if (tty == eqnx_txcooktty)
		eqnx_flush_chars(tty);
	tty->closing = 1;
#ifdef SCO
	while (mpc->mpc_flags & MPC_BUSY){
		struct lmx_struct *lmx;
		/* 
		   If this is an unoccupied modem slot,
		   or if the LMX is not present just 
		   get on with closing. 
		*/
		if (ISRAMP(mpc)) {
			icpiaddr_t icpi;
			icpbaddr_t icpb;

			/* check break bit */
			/* If receiving break, the slot is empty */
			icpi = mpc->mpc_icpi;
	   		if(win16) 
	      			mega_push_win(mpc, 0);
			icpb = (rx_locks & LOCK_A) ? 
				&icpi->ssp.cin_bank_b : &icpi->ssp.cin_bank_a;
      
			if (icpb->ssp.bank_sigs & 0x01) {
	   			if(win16) 
	           			mega_pop_win(63);
				/* No modem, just get out. */
				mpc->mpc_flags &= ~MPC_BUSY;
				break;
			}
	   		if(win16) 
	           		mega_pop_win(64);
		}
		lmx = &(mpc->mpc_icp->lmx[mpc->mpc_chan/16]);
		if (lmx->lmx_active != DEV_GOOD) {
			/* No LMX, just get out. */
			mpc->mpc_flags &= ~MPC_BUSY;
			break;
		}
		if (mpc->closing_wait != ASYNC_CLOSING_WAIT_NONE)
			tty_wait_until_sent(tty, mpc->closing_wait);
	   }
#endif
	if (mpc->mpc_flags & MPC_BUSY) 
		if (mpc->closing_wait != ASYNC_CLOSING_WAIT_NONE){
			restore_flags(flags); /* shashi 01/30/98 */
			tty_wait_until_sent(tty, 
					(unsigned long) mpc->closing_wait);
			save_flags(flags); /* shashi 01/30/98 */
			cli();
		}

	/*printk("eqnx_close:for device %d and flags = %x, setting flags to !ASYNC_INITIALIZED\n", d, (unsigned int) filp->f_flags);*/
	mpc->flags &= ~ASYNC_INITIALIZED;
#ifdef DEBUG
	printk("TTY_FLIPBUF_SIZE = %d and flip.count = %d for device %d\n",
		TTY_FLIPBUF_SIZE, tty->flip.count, d);
#endif
	if(win16) 
	      mega_push_win(mpc, 0);
	if (tty->termios->c_cflag & HUPCL ){
		/*if (!(tty->termios->c_cflag & CLOCAL)){*/
	mpc->flags &= ~ASYNC_INITIALIZED;
			(void) megamodem(d, TURNOFF);
		/*}
	   	if(win16) 
	           mega_pop_win(3);
		eqnx_delay((HZ*dtr_down)/10);*/ /* Hold DTR down for dtr_down*100 msecs */
/*#ifdef DEBUG
printk("after eqnx_delay for device %d\n",d);
#endif*/
	        /*if(win16) 
		   mega_push_win(mpc, 0);*/
         }
#ifdef DEBUG
printk("after HUPCL for device %d\n",d);
#endif
	 mpc->mpc_flags &= ~(MPC_SOFTCAR|MPC_DIALOUT|MPC_MODEM|MPC_DIALIN|MPC_CTS);
	 mpc->mpc_scrn_no = 0;
	 mpc->mpc_icpi->ssp.cin_attn_ena &= ~ENA_DCD_CNG;
	 mpc->mpc_icpo->ssp.cout_cpu_req &= ~TX_SUSP;
	 set_bit(TTY_IO_ERROR, &tty->flags);
	 if (tty->ldisc.flush_buffer)
		(tty->ldisc.flush_buffer)(tty);
#ifdef DEBUG
printk("after ldisc.flush_buffer for device %d\n",d);
#endif
	 chanoff(mpc);
#ifdef DEBUG
	printk("chanoff for device %d\n", d);
#endif
	restore_flags(flags); /* shashi 01/30/98 */
	 eqnx_flush_buffer(tty);
	save_flags(flags); /* shashi 01/30/98 */
	cli();
	 if(win16)
	      mega_pop_win(4);
	 tty->closing = 0;
	tty->driver_data = (void *) NULL;
	mpc->mpc_tty = (struct tty_struct *) NULL;
	if (mpc->openwaitcnt){
		if (mpc->close_delay){
			restore_flags(flags); /* shashi 01/30/98 */
			eqnx_delay(mpc->close_delay);
			save_flags(flags); /* shashi 01/30/98 */
			cli();
		}
#ifdef DEBUG
	printk("eqnx_close:waking up on open_wait for device %d\n",d);
#endif
		wake_up_interruptible(&mpc->open_wait);
	}
	if ((mpd->mpd_id & 0xF8) >= SSP2_ASIC)
		if (mpc->xmit_buf){
			free_page((unsigned long) mpc->xmit_buf);
			mpc->xmit_buf = 0;
		}
	mpc->flags &= ~(ASYNC_CALLOUT_ACTIVE | ASYNC_NORMAL_ACTIVE | ASYNC_CLOSING);
#ifdef DEBUG
	printk("eqnx_close:waking up on close_wait for device %d\n",d);
#endif
	wake_up_interruptible(&mpc->close_wait);
/*	MOD_DEC_USE_COUNT;*/
	restore_flags(flags);
	MOD_DEC_USE_COUNT;
}
/*****************************************************************************/

/*
 *	Write routine. Take the data from the user and put it in the sst 
 *	queue. 
 */

#if (EQNX_VERSION_CODE > 66304)
static int eqnx_write(struct tty_struct * tty, int from_user,
           const unsigned char *buf, int count)
#else
static int eqnx_write(struct tty_struct * tty, int from_user,
           unsigned char *buf, int count)
#endif
{
	struct mpchan *mpc;
	int d, flags = 0, written = 0, c = 0;
	unsigned char  *b;
	int space, nx, win16, datascope = 0, ret;
	volatile icpoaddr_t icpo;
	volatile icpgaddr_t icpg;
	volatile icpqaddr_t icpq;
	unsigned char oldreg;
	int qsize;
#ifdef MIDNIGHT
        unsigned char *dst_addr;
        int align, bytes;
#endif
#ifdef DEBUG
	printk("eqnx_write(tty=%d,from_user=%d,count=%d)\n", 
		(unsigned int) SSTMINOR(tty->device), from_user, count);
#endif

	if ((tty == (struct tty_struct *) NULL) || (eqnx_tmpwritebuf == (char *) NULL))
		return(0);
	if (tty == eqnx_txcooktty)
		eqnx_flush_chars(tty);
	mpc = tty->driver_data;
	if (mpc == (struct mpchan *) NULL)
		return(-ENODEV);
	d = SSTMINOR(tty->device);
	if(d > nmegaport*NCHAN_BRD)
		return(-ENODEV);
	save_flags(flags);
	cli();
	icpo = mpc->mpc_icpo;
	icpg = (icpgaddr_t)icpo;
	icpq = &icpo->ssp.cout_q0;
	win16 = mpc->mpc_mpd->mpd_nwin;
	if(win16)
		mega_push_win(mpc, 0);
	qsize = mpc->mpc_txq.q_size;
	space = qsize - (mpc->mpc_count + Q_data_count) -1 ;
	if (!tty->stopped && !tty->hw_stopped && space >= (qsize >> 4)){
	
	if (((mpc->mpc_mpd->mpd_id & 0xF8) >= SSP2_ASIC) && (mpc->xmit_cnt))
		goto ssp_end;
#ifdef DEBUG
	printk("eqnx_write: device = %d, q_size = %d, space = %d \n",
		SSTMINOR(tty->device),mpc->mpc_txq.q_size,space);
#endif
		c = MIN(space, count);
		b = (unsigned char *) buf;
		if (from_user){
			down(&eqnx_tmpwritesem);
			ret = eqn_from_user(eqnx_tmpwritebuf, buf, c);
			b = eqnx_tmpwritebuf;
			up(&eqnx_tmpwritesem);
			/* In case we got pre-empted */
			c = MIN(c, space);
			if (ret) {
				restore_flags(flags);
				return(-EFAULT);
			}
		}
		mpc->mpc_flags |= MPC_BUSY;
		if ((mpc->mpc_flags & MPC_DSCOPEW)
			&& (dscope[1].chan == mpc - meg_chan)) datascope=1;
		/*
 		* Blindingly fast block copy direct from 
 		* driver buffer to on-board buffers.
 		*/

		if(win16)
			mega_push_win(mpc, 2);
		nx = MIN(c, (mpc->mpc_txq.q_end - mpc->mpc_txq.q_ptr +1));
#ifdef MIDNIGHT
                dst_addr = mpc->mpc_txq.q_addr + mpc->mpc_txq.q_ptr;
                align = ((unsigned long)dst_addr) % 4;
                if (align) {
                  bytes = MIN(nx,(4-align));
                  memcpy(dst_addr, b, bytes);
                }
                else {
                  bytes = 0;
                }
                if (nx > bytes) {
                  memcpy(dst_addr+bytes, b+bytes, nx-bytes);
                }
#else
		memcpy((mpc->mpc_txq.q_addr + mpc->mpc_txq.q_ptr), 
			b, nx);
#endif
		mpc->mpc_txq.q_ptr += nx;
		if(mpc->mpc_txq.q_ptr > mpc->mpc_txq.q_end)
			mpc->mpc_txq.q_ptr = mpc->mpc_txq.q_begin;
		if (datascope) {
			int room,move_cnt;
			int d=1;
			unsigned char *ubase = b;
			room = DSQSIZE - ((dscope[d].next - dscope[d].q.q_ptr) 
				& (DSQMASK));
			if (nx > room)
				dscope[d].status |= DSQWRAP;
                	else
                        	room = nx;
			while (room > 0) {
				move_cnt = MIN(room,
				dscope[d].q.q_end - dscope[d].next + 1);
				memcpy(dscope[d].q.q_addr + dscope[d].next, 
					ubase, move_cnt);
                        	dscope[d].next += move_cnt;
                        	if (dscope[d].next > dscope[d].q.q_end)
                                	dscope[d].next = dscope[d].q.q_begin;
                                ubase += move_cnt;
                                room -= move_cnt;
			
	
			} /* while room */ 
			wake_up_interruptible(&dscope[d].scope_wait);
		} /* if MPC_DSCOPE */
		c -= nx;
		count -= nx;
		written += nx;
		buf += nx;
		b += nx;
		if( c ) {
#ifdef MIDNIGHT
                  dst_addr = mpc->mpc_txq.q_addr + mpc->mpc_txq.q_ptr;
                  align = ((unsigned long)dst_addr) % 4;
                  if (align) {
                        bytes = MIN(c,(4-align));
                        memcpy(dst_addr, b, bytes);
                  }
                  else {
                        bytes = 0;
                  }
                  if (c > bytes) {
                        memcpy(dst_addr+bytes, b+bytes, c-bytes);
                  }
#else
			memcpy(mpc->mpc_txq.q_addr + mpc->mpc_txq.q_ptr, 
				b, c);
#endif
			mpc->mpc_txq.q_ptr += c;
			if(mpc->mpc_txq.q_ptr > mpc->mpc_txq.q_end)
				mpc->mpc_txq.q_ptr = mpc->mpc_txq.q_begin;
			if (datascope) {
				int room,move_cnt;
				int d=1;
				unsigned char *ubase = b;
				room = DSQSIZE - ((dscope[d].next - 
					dscope[d].q.q_ptr) & (DSQMASK));
				if (c > room)
					dscope[d].status |= DSQWRAP;
                		else
                        		room = c;
				while (room > 0) {
					move_cnt = MIN(room,dscope[d].q.q_end - 
						dscope[d].next + 1);
					memcpy(dscope[d].q.q_addr + 
						dscope[d].next, ubase,move_cnt);
                        		dscope[d].next += move_cnt;
                        		if (dscope[d].next > dscope[d].q.q_end)
                                		dscope[d].next = 
							dscope[d].q.q_begin;
                                	ubase += move_cnt;
                                	room -= move_cnt;
				} /* while room */
				wake_up_interruptible(&dscope[d].scope_wait);
			} /* if MPC_DSCOPE */
			count -= c;
			written += c;
			b += c;
			buf += c;
		}
		if(win16)
			mega_pop_win(24);
		oldreg = tx_cpu_reg;
		tx_cpu_reg |= TX_SUSP;
		cur_chnl_sync(mpc);
		if(Q_data_count < 9) {
			if ((icpo->ssp.cout_int_save_togl & 0x4) == 
				(icpo->ssp.cout_cpu_req & 0x4)){
				tx_cpu_reg ^= (CPU_SND_REQ);
			mpc->mpc_cout_events &= ~EV_CPU_REQ_DN;
			}
		}
		Q_data_count += (c + nx);
		space -= (c + nx);
#ifdef DEBUG
printk("eqnx_write: wrote %d chars for device %d\n", (c + nx), 
	SSTMINOR(tty->device));
#endif
#ifdef DEBUG
printk("q data count %d, space %d for device %d\n", 
	Q_data_count,space, SSTMINOR(tty->device));
#endif
		if (!(oldreg & TX_SUSP))
			tx_cpu_reg &= ~TX_SUSP;
		mpc->mpc_output += (c + nx);
		tx_cie |= (ENA_TX_EMPTY_Q0| ENA_TX_LOW_Q0);
	}
ssp_end:
	if(win16)
		mega_pop_win(25);
	if ((mpc->mpc_mpd->mpd_id & 0xF8) < SSP2_ASIC){
		restore_flags(flags);
		return(written);
	}
	if (!count)
		goto end;
	while(1){
#ifdef DEBUG
printk("trying to fill xmit_buf with %d\n", count);
#endif
		c = MIN(count, MIN(XMIT_BUF_SIZE - mpc->xmit_cnt - 1,
				XMIT_BUF_SIZE - mpc->xmit_head));
		if (c <= 0)
			break;
		b = (unsigned char *) buf;
		if (from_user){
			down(&eqnx_tmpwritesem);
			ret = eqn_from_user(eqnx_tmpwritebuf, b, c);
			b = eqnx_tmpwritebuf;
			up(&eqnx_tmpwritesem);
			/* In case we got pre-empted */
			c = MIN(c, MIN(XMIT_BUF_SIZE - mpc->xmit_cnt - 1,
				XMIT_BUF_SIZE - mpc->xmit_head));
			if (ret) {
				restore_flags(flags);
				return(-EFAULT);
			}
		}
		memcpy(mpc->xmit_buf + mpc->xmit_head, b, c);
		mpc->xmit_head = (mpc->xmit_head + c) & (XMIT_BUF_SIZE-1);
		mpc->xmit_cnt += c;
		buf += c;
		count -= c;
		written += c;
#ifdef DEBUG
printk("eqnx_write: wrote %d chars for device %d in xmit_buf\n", c, d);
#endif
	}	
end:
	restore_flags(flags);
#ifdef DEBUG
printk("eqnx_write: return value = %d chars and space = %d for device %d\n", 
written, space, d);
#endif
	return(written);
}

/*
 *	Copy individual output characters to a temporary local buffer.
 *	This buffer will be copied to the board with memcpy to save time.
 */
static void eqnx_put_char(struct tty_struct *tty, unsigned char ch)
{
	register struct mpchan *mpc;
#ifdef DEBUG
	printk("eqnx_put_char(tty=%x,ch=%x)\n", 
		(unsigned int) SSTMINOR(tty->device), (int) ch);
#endif

	if (tty == (struct tty_struct *) NULL)
		return;
	if (tty != eqnx_txcooktty) {
		if (eqnx_txcooktty != (struct tty_struct *) NULL)
			eqnx_flush_chars(eqnx_txcooktty);
		eqnx_txcooktty = tty;
	}
	mpc = tty->driver_data;
	if ((mpc->mpc_mpd->mpd_id & 0xF8) >= SSP2_ASIC){
		int flags = 0;
		save_flags(flags);
		cli();
		/* Shashi 03/05/98 */
		if (mpc->xmit_head >= XMIT_BUF_SIZE - 1){
			eqnx_flush_chars(tty);
			eqnx_txcooktty = tty;
		}
		mpc->xmit_buf[mpc->xmit_head++] = ch;
		mpc->xmit_head &= XMIT_BUF_SIZE-1;
		mpc->xmit_cnt++;
		restore_flags(flags);
	} else{
		if (eqnx_txcooksize >= XMIT_BUF_SIZE -1){
			eqnx_flush_chars(eqnx_txcooktty);
			eqnx_txcooktty = tty;
		}
		eqnx_txcookbuf[eqnx_txcooksize++] = ch;
	}
}
/*****************************************************************************/

/*	Move temp buffer to the board, and clear for next port.  */

static void eqnx_flush_chars(struct tty_struct *tty)
{
	register struct mpchan *mpc;
	unsigned int		d;
	int flags = 0;
#ifdef DEBUG
	printk("eqnx_flush_chars(tty=%x)\n", 
		(unsigned int) SSTMINOR(tty->device));
#endif

	if (tty == (struct tty_struct *) NULL)
		return;
#ifdef	NOTNEEDED
	/* removed - is there a good reason for this test ? */
	/* it breaks echo - mikes */
	if (!O_OPOST(tty)){
#ifdef DEBUG
	printk("opost:eqnx_flush_chars(tty=%x)\n", 
		(unsigned int) SSTMINOR(tty->device));
#endif
		return;
	}
#endif	/* NOTNEEDED */

	/* Shashi: 03/05/98, lets check tty before we get to mpc */
	d = SSTMINOR(tty->device);
	if(d > nmegaport*NCHAN_BRD)
		return;
	/* Shashi 03/05/98, lets check mpc for eqnx_txcooktty */
	if (eqnx_txcooktty == NULL)
		return;
	else{
		mpc = eqnx_txcooktty->driver_data;
		if (mpc == (struct mpchan *) NULL)
			return;
		if ((mpc->mpc_mpd->mpd_id & 0xF8) < SSP2_ASIC){
			unsigned int cooksize = eqnx_txcooksize;
			eqnx_txcooksize = 0;
			eqnx_txcookrealsize = 0;
			eqnx_txcooktty = (struct tty_struct *) NULL;
			if (cooksize == 0)
				return;
#ifdef DEBUG
	printk("eqnx_flush_chars: calling sst_write\n");
#endif
			save_flags(flags);
			cli();
			sst_write(mpc, eqnx_txcookbuf, cooksize);
			restore_flags(flags);
		}
		else{
			if (mpc->xmit_cnt == 0)
				return;
#ifdef DEBUG
	printk("eqnx_flush_chars: calling sst_write1\n");
#endif
			save_flags(flags);
			cli();
			sst_write1(mpc,EQNX_COOKED);
			restore_flags(flags);
		}
	}

}
static int eqnx_write_room(struct tty_struct *tty)
{
	register struct mpchan *mpc;
	icpoaddr_t icpo;
	icpqaddr_t icpq;
	unsigned int space;
	int win16, d;
	/*int flags;*/
	if (tty == (struct tty_struct *) NULL)
		return(0);
	mpc = tty->driver_data;
	if (mpc == (struct mpchan *) NULL)
		return(-ENODEV);
	d = SSTMINOR(tty->device);
	if(d > nmegaport*NCHAN_BRD)
		return(-ENODEV);
#ifdef DEBUG
printk("eqnx_write_room: device = %d\n", d);
#endif
	if ((mpc->mpc_mpd->mpd_id & 0xF8) >= SSP2_ASIC){
		int ret;
		ret = (XMIT_BUF_SIZE - mpc->xmit_cnt -1);
		if (ret < 0)
			return(0);
		return(ret);
	}
	if (tty == eqnx_txcooktty){
		if (eqnx_txcookrealsize != 0){
			space = (eqnx_txcookrealsize - eqnx_txcooksize);
			return(space);
		}
	}
	win16 = mpc->mpc_mpd->mpd_nwin;
	if(win16)
		mega_push_win(mpc, 0);
	icpo = mpc->mpc_icpo;
	icpq = &icpo->ssp.cout_q0;
	space = mpc->mpc_txq.q_size - (mpc->mpc_count + Q_data_count) -1 ;
	if (tty == eqnx_txcooktty){
		eqnx_txcookrealsize = space;
		space -= eqnx_txcooksize;
	}
	if(win16)
		mega_pop_win(25);
	return(space);
}
static int eqnx_chars_in_buffer(struct tty_struct *tty)
{
	struct mpchan *mpc;
	int flags = 0;
	icpoaddr_t icpo;
	icpqaddr_t icpq;
	unsigned int count;
	int win16, d;
	if (tty == (struct tty_struct *) NULL)
		return(-ENODEV);
	mpc = tty->driver_data;
	if (mpc == (struct mpchan *) NULL)
		return(-ENODEV);
	d = SSTMINOR(tty->device);
	if(d > nmegaport*NCHAN_BRD)
		return(-ENODEV);
#ifdef DEBUG
printk("eqnx_chars_in_buffer: count for device %d = %d\n", d,mpc->xmit_cnt);
#endif
	/*if ((mpc->mpc_mpd->mpd_id & 0xF8) >= SSP2_ASIC){
printk("eqnx_chars_in_buffer: count for device %d = %d\n", d,mpc->xmit_cnt);
		return(mpc->xmit_cnt);
	}*/
	save_flags(flags);
	cli();
	win16 = mpc->mpc_mpd->mpd_nwin;
	if(win16)
		mega_push_win(mpc, 0);
	icpo = mpc->mpc_icpo;
	icpq = &icpo->ssp.cout_q0;
	count = Q_data_count;
	if ((mpc->mpc_mpd->mpd_id & 0xF8) >= SSP2_ASIC)
		count += mpc->xmit_cnt;
	if(win16)
		mega_pop_win(25);
	restore_flags(flags);
#ifdef DEBUG
printk("eqnx_chars_in_buffer: count for device %d = %d\n", d,count);
#endif
	return(count);
}
static void eqnx_flush_buffer(struct tty_struct *tty)
{
	register struct mpchan *mpc;
	register icpiaddr_t icpi;
	register icpoaddr_t icpo;
	register icpqaddr_t icpq;
	icpbaddr_t icpb;
	int win16;
	uchar_t cin;
	int flags = 0;
	int d;
	if (tty == (struct tty_struct *) NULL)
		return;
	mpc = tty->driver_data;
	if (mpc == (struct mpchan *) NULL)
		return;
	icpi = mpc->mpc_icpi;
	icpo = mpc->mpc_icpo;
	icpq = &icpo->ssp.cout_q0;
	d = SSTMINOR(tty->device);
	if(d > nmegaport*NCHAN_BRD)
		return;
#ifdef DEBUG
	printk("eqnx_flush_buffer: device = %d\n", d);
#endif
	win16 = mpc->mpc_mpd->mpd_nwin;

	save_flags(flags);
	cli();
	if (tty == eqnx_txcooktty){
		eqnx_txcooktty = (struct tty_struct *) NULL;
		eqnx_txcooksize = 0;
		eqnx_txcookrealsize = 0;
	}
	if(win16) 
	      mega_push_win(mpc, 0);
	/* flush transmit buffers */
	tx_lck_ctrl |= LCK_Q_ACT;
        cur_chnl_sync(mpc);
        tx_dma_stat = 0;
        icpo->ssp.cout_int_fifo_ptr = 0;  
        Q_data_count = 0;
        mpc->mpc_count = 0;
        Q_data_ptr = mpc->mpc_txq.q_begin + mpc->mpc_txbase;
        mpc->mpc_txq.q_ptr = mpc->mpc_txq.q_begin;
	mpc->xmit_cnt = mpc->xmit_head = mpc->xmit_tail = 0;
        icpo->ssp.cout_attn_ena = 0x8000;
        tx_lck_ctrl &= ~LCK_Q_ACT;
        mpc->mpc_flags &= ~MPC_BUSY;
	
	/* flush receiver queue */
	/* set rx ptr = tail */
	icpb = (rx_locks & LOCK_A) ? &icpi->ssp.cin_bank_b : &icpi->ssp.cin_bank_a;
  	if (!(icpb->ssp.bank_events & EV_REG_UPDT)) 
  		frame_wait(mpc,2); /* make sure regs are valid */ 
  	cin = icpi->ssp.cin_locks;
	icpi->ssp.cin_locks = cin | 0x03;
	cur_chnl_sync( mpc );
  	icpi->ssp.cin_bank_a.ssp.bank_fifo_lvl &= ~0x8f;
  	icpi->ssp.cin_bank_b.ssp.bank_fifo_lvl &= ~0x8f;
	icpi->ssp.cin_tail_ptr_a = 
	icpi->ssp.cin_tail_ptr_b =  
	mpc->mpc_rxq.q_ptr = rx_next;
        icpi->ssp.cin_locks = cin;
	if(win16)
	 mpc->mpc_rxq.q_ptr &= 0x3fff;
	mpc->mpc_flags |= MPC_RXFLUSH;
	mpc->mpc_block = 0;
	if(win16) 
	        mega_pop_win(5);
	restore_flags(flags);
	wake_up_interruptible(&tty->write_wait);
	if ((tty->flags & (1 << TTY_DO_WRITE_WAKEUP)) &&
		tty->ldisc.write_wakeup)
		tty->ldisc.write_wakeup(tty);
}
static void eqnx_throttle(struct tty_struct * tty)
{
	register struct mpchan *mpc;
	int flags = 0;
	
#if DEBUG
	printk("eqnx_throttle: device = %d\n", SSTMINOR(tty->device));
#endif

	if (tty == (struct tty_struct *) NULL)
		return;
	mpc = tty->driver_data;
	if (mpc == (struct mpchan *) NULL)
		return;

	save_flags(flags);
	cli();
#ifdef DEBUG
printk("eqnx_throttle: setting mpc_block for device %d\n", 
	SSTMINOR(tty->device));
#endif
	mpc->mpc_block = 1;
	restore_flags(flags);
}
static void eqnx_unthrottle(struct tty_struct * tty)
{
	register struct mpchan *mpc;
	int flags = 0;
	
#if DEBUG
	printk("eqnx_unthrottle: device = %d\n", SSTMINOR(tty->device));
#endif

	if (tty == (struct tty_struct *) NULL)
		return;
	mpc = tty->driver_data;
	if (mpc == (struct mpchan *) NULL)
		return;
	save_flags(flags);
	cli();
	mpc->mpc_block = 0;
	restore_flags(flags);
}
static void eqnx_set_termios(struct tty_struct *tty, 
		struct termios * old)
{
	register struct mpchan *mpc;
	int d;
	struct termios *tiosp;
	int win16;
	int flags = 0;

	if (tty == (struct tty_struct *) NULL)
		return;
	if ((mpc = tty->driver_data) == (struct mpchan *) NULL)
		return;
	d = SSTMINOR(tty->device);
	if(d > nmegaport*NCHAN_BRD)
		return;
#if DEBUG
	printk("eqnx_set_termios: trying setting termios for %d\n", d);
#endif
	tiosp = tty->termios;
	if ((tiosp->c_cflag == old->c_cflag) && 
		(tiosp->c_iflag == old->c_iflag) &&
		(tiosp->c_cc[VSTOP] == old->c_cc[VSTOP]) &&
		(tiosp->c_cc[VSTART] == old->c_cc[VSTART]))
		return;
#if DEBUG
	printk("eqnx_set_termios: new termios settings for %d\n", d);
#endif
	win16 = mpc->mpc_mpd->mpd_nwin;
	save_flags(flags);
	cli();
	if(win16)
		mega_push_win(mpc, 0);
/* RAMP START --------------------------------------------- */
	/* If RAMP config not completed, don't let any ioctl's in */
	if (ISRAMP(mpc)) {
      /* check break bit */
      /* If not receiving a break, the slot is occupied by a UART */
      
		if ( !(mpc->mpc_mpa_stat & MPA_UART_CFG_DONE) ||
                     (mpc->mpc_mpa_stat & (MPA_INITG_INIT_MODEM | MPA_INITG_MODIFY_SETTINGS))) {
			register icpiaddr_t icpi;
			register icpbaddr_t icpb;
      			icpi = mpc->mpc_icpi;
      			icpb=(rx_locks & LOCK_A) ? &icpi->ssp.cin_bank_b 
				: &icpi->ssp.cin_bank_a;
			/* make sure regs are valid */ 
  			if (!(icpb->ssp.bank_events & EV_REG_UPDT)) 
  				frame_wait(mpc,2); 
         		if (!(icpb->ssp.bank_sigs & 0x01 )) {
				if(win16) mega_pop_win(74);
            			/* Modem present in slot, sleep */
termios_modem_wait:
				restore_flags(flags); 
				interruptible_sleep_on(&mpc->mpc_mpa_call_back);
				save_flags(flags);
				cli();
				if eqn_fatal_signal {
					restore_flags(flags); 
					return;
				}
				if(mpc->mpc_mpa_stat & MPA_CALL_BACK_ERROR){
					restore_flags(flags); 
                        		return;
				}
				else if ( !(mpc->mpc_mpa_stat & MPA_UART_CFG_DONE) ||
                     (mpc->mpc_mpa_stat & (MPA_INITG_INIT_MODEM | MPA_INITG_MODIFY_SETTINGS))) {
					goto termios_modem_wait;
				}
				if(win16)
					mega_push_win(mpc, 0);
         		} else {
				if(win16) mega_pop_win(75);
            			/* No modem, return error */
				restore_flags(flags); 
				return;
         		}
		}
	}
/* RAMP END ----------------------------------------------- */
#if DEBUG
	printk("set_termios: Calling megaparam for device %d\n", d);
#endif
	megaparam(d);
	if(win16)
	   mega_pop_win(2);
	if ((old->c_cflag & CRTSCTS) && ((tiosp->c_cflag & CRTSCTS) == 0))
		tty->hw_stopped = 0;
	if (((old->c_cflag & CLOCAL) == 0) && (tiosp->c_cflag & CLOCAL)){
		mpc->carr_state = 1;
#if DEBUG
	printk("eqnx_set_termios:waking up on open_wait for device %d\n",d);
#endif
		wake_up_interruptible(&mpc->open_wait);
	}
	restore_flags(flags);
}

/*
 * ------------------------------------------------------------
 * eqnx_stop() and eqnx_start()
 *
 * This routines are called before setting or resetting tty->stopped.
 * They suspend or resume the tx cpu register, as necessary.
 * ------------------------------------------------------------
 */
static void eqnx_stop(struct tty_struct *tty)
{
	register struct mpchan *mpc;
	register icpoaddr_t icpo;
	int flags = 0;
	int win16;
	
#if DEBUG
	printk("eqnx_stop(tty=%d)\n", SSTMINOR(tty->device));
#endif

	if (tty == (struct tty_struct *) NULL)
		return;
	mpc = tty->driver_data;
	if (mpc == (struct mpchan *) NULL)
		return;
	win16 = mpc->mpc_mpd->mpd_nwin;
	icpo = mpc->mpc_icpo;
	save_flags(flags);
	cli();
	if(win16)
		mega_push_win(mpc,0);
	/* suspend output */
	tx_cpu_reg |= 0x80;
	if(win16)
		mega_pop_win(6);
	restore_flags(flags);
}
static void eqnx_start(struct tty_struct *tty)
{
	register struct mpchan *mpc;
	register icpoaddr_t icpo;
	int flags = 0;
	int win16;
	
#if DEBUG
	printk("eqnx_start(tty=%d)\n", SSTMINOR(tty->device));
#endif

	if (tty == (struct tty_struct *) NULL)
		return;
	mpc = tty->driver_data;
	if (mpc == (struct mpchan *) NULL)
		return;
	win16 = mpc->mpc_mpd->mpd_nwin;
	icpo = mpc->mpc_icpo;
	save_flags(flags);
	cli();
	if(win16)
		mega_push_win(mpc,0);
	/* resume output */
	tx_cpu_reg &= ~0x80;
	if(win16)
		mega_pop_win(7);
	restore_flags(flags);
}
/*****************************************************************************/

/*
 * 	eqnx_hangup() --- called by tty_hangup() when a hangup is signaled.
 *	Drop control signals, flush buffer, and close port.
 */

static void eqnx_hangup(struct tty_struct *tty)
{
	register struct mpchan *mpc;
	unsigned long	flags = 0;
	int d;
	int win16;

#ifdef DEBUG
	printk("eqnx_hangup: device = %d\n", (unsigned int) SSTMINOR(tty->device));
#endif

	if (tty == (struct tty_struct *) NULL)
		return;
	mpc = tty->driver_data;
	if (mpc == (struct mpchan *) NULL)
		return;
	d = SSTMINOR(tty->device);
	if(d > nmegaport*NCHAN_BRD)
		return;


	save_flags(flags);
	cli();
	mpc->flags &= ~ASYNC_INITIALIZED;
	win16 = mpc->mpc_mpd->mpd_nwin;
	if(win16) 
	      mega_push_win(mpc, 0);
	if (mpc->flags & ASYNC_NORMAL_ACTIVE)
		*mpc->normaltermios = *tty->termios;
#if (LINUX_VERSION_CODE < 132096)
	if (mpc->flags & ASYNC_CALLOUT_ACTIVE)
		*mpc->callouttermios = *tty->termios;
#endif
	if (tty == eqnx_txcooktty)
		eqnx_flush_chars(tty);
	if (tty->termios->c_cflag & HUPCL ){
		/*if (!(tty->termios->c_cflag & CLOCAL)){*/
			megamodem(d, TURNOFF);
		/*}*/
	}
	 mpc->mpc_flags &= ~(MPC_SOFTCAR|MPC_DIALOUT|MPC_MODEM|MPC_DIALIN|MPC_CTS);
	 mpc->mpc_scrn_no = 0;
	 mpc->mpc_icpi->ssp.cin_attn_ena &= ~ENA_DCD_CNG;
	 chanoff(mpc);
	 if(win16)
	      mega_pop_win(8);
	eqnx_flush_buffer(tty);
	set_bit(TTY_IO_ERROR, &tty->flags);
	/*tty->driver_data = (void *) NULL;*/
	mpc->flags &= ~(ASYNC_NORMAL_ACTIVE | ASYNC_CALLOUT_ACTIVE);
	mpc->mpc_tty = (struct tty_struct *) NULL;
	mpc->refcount = 0;
#ifdef DEBUG
	printk("eqnx_hangup: eqnx_close:waking up on open_wait for device %d\n",d);
#endif
	if (((mpc->mpc_mpd->mpd_id & 0xF8) >= SSP2_ASIC) && (mpc->xmit_buf)){
		free_page((unsigned long) mpc->xmit_buf);
		mpc->xmit_buf = 0;
		mpc->xmit_cnt = 0;
		mpc->xmit_head = 0;
		mpc->xmit_tail = 0;
	}
	wake_up_interruptible(&mpc->open_wait); /*Shashi 02/09/98 */
	restore_flags(flags);
}

/*****************************************************************************/
long eqnx_init(long kmem_start)
{
	register struct mpdev *mpd;
	register struct mpchan *mpc;
	volatile register icpiaddr_t icpi;
	volatile icpoaddr_t icpo;
	volatile icpgaddr_t icpg;
	volatile icpqaddr_t icpq;
	icpaddr_t icp;
	volatile struct hwq_struct *hwq;
#if (EQNX_VERSION_CODE < 66304)
	int rc;
#endif
	mpaddr_t mp;
	int i, j, k, lmx;
	uchar_t cchnl;
	volatile unsigned char *chnl_ptr;
	ushort_t no_cache;
	ushort_t no_icp;
	int ii, jj;
	int addr;
	struct pci_csh *cshp;
	unsigned short numboards = 0;
	unsigned char  config;
	unsigned char  rev_id;
	unsigned short dev_id;
	paddr_t	base_addr;
	unsigned char  slot;
	unsigned char c_code;  /* PatH - Multimodem country code */
	int duplicate, di;

#if	defined(ISA_ENAB) || defined(MCA_ENAB)
	register int b;
	uchar_t bid, cbid, lbid;
	struct isa_cnfg_parms isa_parms;
	int b0 = 0;
	int eisa_or_isa = 0;
	int mca_bus = 0;
	ushort_t eisa_cfg[EISA_REGS_LEN];
	uchar_t mca_cfg[ MCA_REGS_LEN ];
#endif	/* ISA_ENAB || MCA_ENAB */

        int ssp_channels = 0;
        volatile uchar_t *bus_ctrl_p;
	int lmx_factor;
	printk("Loading %s Version %s\n", Version, VERSNUM);
	printk("%s\n", Copyright);

	ist.struct_rev = IST_REV;
	strncpy(ist.drvr_ver, VERSNUM, 16);
	ist.drvr_type = 1 ; /* Character Driver */
	if (kmem_start)
		ist.drvr_bind_type = 1 ; /* Driver Linked with kernel */
	else
		ist.drvr_bind_type = 2 ; /* Loadable driver */
	ist.drvr_obj_type = 0x0C; /* LINUX */
	ist.ctlr_max_num = maxbrd;
	ist.ctlr_found = 0;

/* Make sure all boards start with "dead" status - Monty 6/12/96 */
	for (ii=0; ii < maxbrd; ii++)
		meg_dev[ii].mpd_alive = 0;

#if	(LINUX_VERSION_CODE >= 132096)
	/* 2.4 kernels and after */
	init_MUTEX(&eqnx_tmpwritesem);
#endif

	nmegaport = 0;
	nicps = 0;
	nextmin = 0;
	/* Start with Space.c info on bus_type */  

#if	defined(ISA_ENAB) || defined(MCA_ENAB)
	if (io_bus_type == EISA_BUS || io_bus_type == ISA_BUS) 
		eisa_or_isa = 1;
#endif	/* ISA_ENAB || MCA_ENAB */

	if (((char *) eqnPCIcsh = (char *)vmalloc( pcisize * maxbrd)) 
		== (char *) NULL){
		printk("EQUINOX: Failed to allocate virtual address space of size %d for eqnPCIcsh\n", pcisize * maxbrd);
		return(-1);
		}
	memset(eqnPCIcsh, 0, pcisize * maxbrd);
	cshp = (struct pci_csh *)&eqnPCIcsh[0];
	numboards = eqx_pci_buspresent(cshp);

#if	defined(ISA_ENAB) || defined(MCA_ENAB)
	if (io_bus_type == MCA_BUS) 
		mca_bus = 1;

	oper_input.sys_bus_type = io_bus_type; /* From space.c */ 
	oper_input.num_cntrls = maxbrd; /* From space.c */ 
	oper_input.quiet_mode = 0xff;  /* No messages for now  */
	oper_input.isa_def_specd = 1; /* Dynamic always for now */ 
#endif	/* ISA_ENAB || MCA_ENAB */

	if (numboards) {
		printk("PCI boards found\n");
		for(i = 0; i < numboards; i++, cshp++) {
			/* 
		 	 * See if board configured by checking the
		 	 * "Memory Space Control" bit in the Command
		 	 * register (bit 1 @ offset 0x04)
		 	 */
			dev_id = cshp->cs.csh00.dev_id;

			printk("Board %d id : %x\n",i, dev_id);

	  		mpd = &meg_dev[nmegaport];
			/* Save model number only */
	  		mpd->mpd_id = dev_id;
			ist.ctlr_array[nmegaport].ctlr_model = 
				mpd->mpd_id;
			ist.ctlr_array[nmegaport].ctlr_model_ext = 
				((mpd->mpd_id & 0xFF00) >> 8);

			dev_id &= 0xff; /* low byte only */
			config = cshp->cs.csh00.command;
			rev_id = cshp->cs.csh00.rev_id;
			base_addr=cshp->cs.csh00.base_addr_reg0 &
				PCI_BASE_ADDR_MASK;
			/*
			 * This code checks for duplicate boards.
			 * This happens because some brain-dead
			 * PCI controllers "see" multiple busses
			 * when there aren't multiple busses.
			 * So, they see the same bus multiple times,
			 * reporting our board on all the busses.
			 */
			duplicate = 0;
			for(di = 0; di < i; di++) {
				struct mpdev *mpd = &meg_dev[di];

				if (base_addr == mpd->mpd_pmem) {

					duplicate = 1;
					break;
				}
			}
			if (duplicate)
				continue;
			slot = cshp->slot;
			/* Make sure base address was really 
			   configured */
			if (!base_addr){
    				printk("SST:PCI Base Addr not configured\n");
				continue;
			}

			/* 
		 	 * Look at the Memory Space Control bit to
		 	 * see if this has been configured. 
		 	 */
			if (!(config & PCI_MSC)) {
				continue;
			}

			/* Make sure we don't find more boards than 
			 * our max. */
			if (++ist.ctlr_found > maxbrd) {
				ist.ctlr_array[nmegaport].cnfg_state=2; 
				/* maxbrd exceeded */
				ist.ctlr_array[nmegaport].cnfg_fail = 5; 
				continue;
			}
			else
				ist.ctlr_array[nmegaport].cnfg_state=1; 

			/* Mark board as being alive */
	  		mpd->mpd_alive = true;

			/* revision */
			ist.ctlr_array[nmegaport].ctlr_rev = rev_id;

			/*  PCI */
			mpd->mpd_bus =  PCI_BUS;
			ist.ctlr_array[nmegaport].ctlr_bus_type =  
				PCI_BUS; 
	  		mpd->mpd_slot = slot;
			ist.ctlr_array[nmegaport].ctlr_loc = 
				mpd->mpd_slot;
			ist.ctlr_array[nmegaport].ctlr_mca_port = 0;

			/*  PCI CONFIG */
			ist.ctlr_array[nmegaport].cnfg_init = 1; 

			/* eisa_icp_count has the right algorithm 
			 * for PCI */
			mpd->mpd_nicps = 
				(uchar_t)eisa_icp_count(mpd->mpd_id);
			nicps += mpd->mpd_nicps;

			/* Save the physical address */
			mpd->mpd_pmem = base_addr;
			ist.ctlr_array[nmegaport].cnfg_physc_base = 
				mpd->mpd_pmem;

			/* Get the memory size */
                       	mpd->mpd_addrsz = pci_mem_size(dev_id);
                       	ist.ctlr_array[nmegaport].cnfg_addr_size = 
				mpd->mpd_addrsz;

			/* PCI is always Flat */;
                       	mpd->mpd_nwin = HA_FLAT;
			ist.ctlr_array[nmegaport].cnfg_mode = 2;

  			if((dev_id == 0xC8) ||  /* SSP4 SST-16P with no panel */
     			(dev_id == 0xC4) || /* SSP4 SST-16P with 16RJ */
     			(dev_id == 0xC0) || /* SSP4 SST-16P with 16DB */
     			(dev_id == 0xD0))   /* SSP4 SST-16P with 16DB9 */
				mpd->mpd_memsz = mpd->mpd_addrsz;
			else /* DRAM memory size is one-half memory size */
				mpd->mpd_memsz = mpd->mpd_addrsz/2;

			/* Not used in HA_FLAT mode */
                       	mpd->mpd_pgctrl = mpd->mpd_slot + 2;

			/* PCI memory width is always 32 */
			mpd->mpd_mem_width = 32;

			/* 16 bit mode */
			ist.ctlr_array[nmegaport].cnfg_isa_bus_sz=0x10; 

			/* assume 128 ports adapter */
			mpd->mpd_nchan = brd_nchan_cfg(mpd->mpd_id);
			mpd->mpd_minor = nextmin;
			nextmin += NCHAN_BRD;

			nmegaport++;
		} /* End i < numboards */
	} /* End if numboards */

#if	defined(ISA_ENAB) || defined(MCA_ENAB)
	if (eisa_or_isa) {
		j = 8;
		/* loop thru EISA slots */
		for (b = 0; b < j ; b++) {
		  switch(eisa_brd_found(b, eisa_cfg))
		  {
		  case true:
			if (++ist.ctlr_found > maxbrd) {
				ist.ctlr_array[nmegaport].cnfg_state = 2; 
				ist.ctlr_array[nmegaport].cnfg_fail = 5; /* maxbrd exceeded */
	   		continue;
			}
			else
				ist.ctlr_array[nmegaport].cnfg_state = 1; 
		  	mpd = &meg_dev[nmegaport];
		  	mpd->mpd_alive = true;
		  	mpd->mpd_id = eisa_cfg[7] &0xFFF8 ; /* Save model number only */
			ist.ctlr_array[nmegaport].ctlr_model = mpd->mpd_id;
			ist.ctlr_array[nmegaport].ctlr_model_ext = 
				((mpd->mpd_id & 0xFF00) >> 8);
			ist.ctlr_array[nmegaport].ctlr_rev = eisa_cfg[7] & 0x7; /* revision */
			mpd->mpd_bus = EISA_BUS;
			ist.ctlr_array[nmegaport].ctlr_bus_type = EISA_BUS; /* EISA */
		  	mpd->mpd_slot = b;
			ist.ctlr_array[nmegaport].ctlr_loc = mpd->mpd_slot;
			ist.ctlr_array[nmegaport].ctlr_mca_port = 0;
			ist.ctlr_array[nmegaport].cnfg_init = 1; /* EISA CONFIG */
			mpd->mpd_nicps = (uchar_t)eisa_icp_count(mpd->mpd_id);
			nicps += mpd->mpd_nicps;
			mpd->mpd_pmem = eisa_base_paddr(eisa_cfg);
			ist.ctlr_array[nmegaport].cnfg_physc_base = 
				mpd->mpd_pmem;
                 /* SSM testing, found testing SSM , check for Zero size */
#ifdef SSM
                        if ((mpd->mpd_addrsz = eisa_mem_size(eisa_cfg, 0))== 0)
                        {
                            ist.ctlr_array[nmegaport].cnfg_state = 2;
                            ist.ctlr_array[nmegaport].cnfg_fail = 6; /* fail bad address */
                            nextmin += NCHAN_BRD;
                            nmegaport++;
                            mpd->mpd_alive = false;
			    printk("SSM: failed bad address\n");
                            continue;
                        }
#else
                        mpd->mpd_addrsz = eisa_mem_size(eisa_cfg, 0);
#endif

                        ist.ctlr_array[nmegaport].cnfg_addr_size = mpd->mpd_addrsz;
                        if ((mpd->mpd_nwin = eisa_mem_strat(eisa_cfg)) == HA_WIN16)
				ist.ctlr_array[nmegaport].cnfg_mode = 1 /* Paged */;
			else
				ist.ctlr_array[nmegaport].cnfg_mode = 2 /* Flat */;
			mpd->mpd_memsz = eisa_mem_size(eisa_cfg, 1);

			if ((mpd->mpd_id & 0xF8) < SSP2_ASIC)
                        	mpd->mpd_pgctrl = mpd->mpd_slot * 0x1000 + 0xC80 + 2;
			else
                        	mpd->mpd_pgctrl = mpd->mpd_slot * 0x1000 + 0xC80 + 8;
			mpd->mpd_mem_width = 16;
			ist.ctlr_array[nmegaport].cnfg_isa_bus_sz = 0x10; /* 16 bit mode */
/* assume 128 ports adapter */
				mpd->mpd_nchan = brd_nchan_cfg(mpd->mpd_id);
				mpd->mpd_minor = nextmin;
				nextmin += NCHAN_BRD;
				nmegaport++;
		  	break;
		  case -1:
#ifdef UNIX
			cmn_err(CE_CONT, 
			"Run EISA configuration utility for Equinox board in slot %d.\n", b);
#else
			printk("Run EISA configuration utility for Equinox board in slot %d.\n", b);
#endif
			break;
		  default:
			break;
		  }
		}	
	}
#endif	/* ISA_ENAB || MCA_ENAB */

#ifdef	ISA_ENAB
#if	(LINUX_VERSION_CODE >= 132096)
#ifdef	CONFIG_ISAPNP
	{
	/*
	** ISA plug-and-play in linux kernels 2.4+
	*/
	extern struct list_head isapnp_devices;
	struct list_head *list;
	struct pci_dev *dev;

	list = isapnp_devices.next;
	while (list != &isapnp_devices) {
		dev = pci_dev_g(list);	
		if ((dev->vendor == ISAPNP_VENDOR('E','Q','X')) &&
			(pnp_found < MAX_PNP_DEVS)) {
			pnp_found_devs[pnp_found++] = dev;
			dev->prepare(dev);
			dev->activate(dev);
		}
		list = list->next;
	}
	}
#endif	/* CONFIG_ISAPNP */
#endif

/*
** Memory returned from PnP configured boards is unreliable
** do not use MegAddr		
**			Mikes 12/19/01
*/

#ifdef	RELIABLE_PNP
/* Search for ISA Controllers */
/* First scan for any Plug and Play (PnP) configured boards 
   and use that ISA hole (if MegAddr not already configured) */
	if (eisa_or_isa && !MegAddr) {
		for(k=0; k < 16; k++) {
			bid = inb(portn[k]);
			cbid = ~inb(portn[k] + 1);
			lbid = ~inb(portn[k] + 1);
			if((bid == cbid || bid == lbid) && (cbid != lbid)) {
				unsigned long int SavAddr = 0;
				if ((bid & 0xf8) == 0xF8) { /* SSP4 */
				if (inb(portn[k] + 4) & 0x80) { 
					unsigned char port7,port9,porta;
					port7 = inb(portn[k] + 0x7);
					port9 = inb(portn[k] + 0x9);
					porta = inb(portn[k] + 0xA);
					SavAddr = (porta << 16) | ((port9 & 0xc0) << 8) ;
					if (SavAddr && ((port7 & 0xC0) == 0xC0))
						MegAddr = SavAddr;
					printk("MegAddr = %x\n", (unsigned int) MegAddr);
					break;
				}
			} else { /* check for 64 or 128 port board, rev 1 or greater */
				if ((((bid & 0xf8) == 0x10) || ((bid & 0xf8) == 0x08)) && 
				    ((bid & 0x7) > 0) && ((inb(portn[k] + 2) & 0xC0) == 0xC0)) { 
					unsigned char port4,port5,port7;
					port4 = inb(portn[k] + 0x4);
					port5 = inb(portn[k] + 0x5);
					port7 = inb(portn[k] + 0x7);
					SavAddr = (port5 << 16) | ((port4 & 0xc0) << 8) ;
					if (SavAddr && ((port7 & 0xC0) == 0xC0))
						MegAddr = SavAddr;
					printk("MegAddr = %x\n", (unsigned int) MegAddr);
					break;
				}
			} /* end if else */ 
			} /* end if (bid) */
		} /*for*/
	} /*if eisa or isa*/
#endif	/* RELIABLE_PNP */
#endif	/* ISA_ENAB */

#if	defined(ISA_ENAB) || defined(MCA_ENAB)
	if (eisa_or_isa) {
		int pnp = 0; /* default */
		for(k=0; k < 16; k++) {
			if(check_region(portn[k], 8))
				continue;
			bid = inb(portn[k]);
			cbid = ~inb(portn[k] + 1);
			lbid = ~inb(portn[k] + 1);
 			/* SSM found testing , SSM , need to validate toggle 
			   and brd type, coult get false board ...  */
                        if((bid == cbid || bid == lbid) && (cbid != lbid)) {
				if (++ist.ctlr_found > maxbrd) {
					ist.ctlr_array[nmegaport].cnfg_state=2; 
					/* maxbrd exceeded */
					ist.ctlr_array[nmegaport].cnfg_fail = 5;
		   			continue;
				}
#ifdef SSM /* note found testing SSM */
					switch (bid & 0xf8)
					{
						case 0x10:  /* 128 port board */
						case 0x08:  /* 64 port baord */
						case 0x30:  /* 12 port board */
						case 0x38:  /* 24 port board */
						case 0x18:  /* 8 port board */
						case 0x20:  /* 4 port board */
						case 0xF0:  /* SSP2 board */
						case 0xF8:  /* SSP4 board */
							ist.ctlr_array[nmegaport].cnfg_state = 1;
							break;
						default:
							ist.ctlr_array[nmegaport].cnfg_state = 2;
							/* maxbrd exceeded */
							ist.ctlr_array[nmegaport].cnfg_fail = 5;
							continue;
							break;
					} /* End Switch (bid) */
#endif /* SSM */
				mpd = &meg_dev[nmegaport];
				if ((bid & 0xF8) >= SSP2_ASIC) {
					uchar_t u1;
					ushort_t v1, v2;
					uint_t pp;
					/* Get qualifier bits */
     					pp = portn[k];
     					u1 = inb( (pp + 8));  /* port 8/9 */
     					outb( (u1 | 0x80),(pp + 8)); 
     					v1 = inb( (pp));  /* port 0 */
     					v1 = (v1 & 0x07);/*  set lower 3 bits */
					/* toggle rmod */
     					outb( ( u1 & ~0x80 ),(pp + 8));
     					v2 = inb( (pp));  /* port 0 */
     					v2 = ((v2 & 0x07) << 3); /* set upper bits */
					v1 |= v2;
					mpd->mpd_id = ((bid | (v1 << 8)) 
								& 0xfff8);
					/* revision does not live in same place 
						on SSP4 */
					ist.ctlr_array[nmegaport].ctlr_rev = 0; 
				}
				/* Save model number only */
				else{
					mpd->mpd_id = bid & 0xF8;
					ist.ctlr_array[nmegaport].ctlr_rev = 
						bid & 0x7; /* revision */
				}
				ist.ctlr_array[nmegaport].ctlr_model = 
					mpd->mpd_id;
				ist.ctlr_array[nmegaport].ctlr_model_ext = 
					((mpd->mpd_id & 0xFF00) >> 8);
				mpd->mpd_bus = ISA_BUS;
				ist.ctlr_array[nmegaport].ctlr_bus_type = 
					ISA_BUS; /* ISA */
				mpd->mpd_io = k;
				if ((mpd->mpd_id & 0xF8) < SSP2_ASIC)
					mpd->mpd_pgctrl = portn[k] + 2;
				else
					mpd->mpd_pgctrl = portn[k] + 8;
				ist.ctlr_array[nmegaport].ctlr_loc = portn[k];
				request_region(portn[k], 8, "eqnx");
				ist.ctlr_array[nmegaport].ctlr_mca_port = 0;
				pnp = 0; /* default */
				if (((bid & 0xf8) == 0xF8) && 
					(inb(portn[k] + 4) & 0x80)) { 
#ifdef	RELIABLE_PNP
					unsigned char port7,port9,porta;
#endif
					struct cntrl_cfig *idcp ;
					pnp = 1;
					/* use Plug and Play settings */ 
					idcp = &oper_input.isa_def_config;
					idcp->base_port = portn[k]; /* I/O port used */ 
	/*
	** Memory returned from PnP is unreliable - we'll find an ISA hole
	** ourselves.  The port number allocated is ok.
	**				Mikes 12/19/01
	*/
#ifdef	RELIABLE_PNP
					oper_input.isa_def_specd = 0xff; 
					idcp->cntrlr_mode = 0 ; /* Paged! */ 
					idcp->cntrlr_bus_inface = 0x20 ; /* 16 bits! */ 
					/* Address */ 
					port7 = inb(portn[k] + 0x7);
					port9 = inb(portn[k] + 0x9);
					porta = inb(portn[k] + 0xA);
					idcp->cntrlr_base_addr = (porta << 16) | ((port9 & 0xc0) << 8) ;
					/* Make sure Plug and Play gave us an address */
					if (!(idcp->cntrlr_base_addr 
						&& ((port7 & 0xC0) == 0xC0))) {
						ist.ctlr_array[nmegaport].cnfg_state=2; 
						ist.ctlr_array[nmegaport].cnfg_fail = 6;
						mpd->mpd_alive = false;
						nmegaport++;
		   			continue;
					}
#endif	/* RELIABLE_PNP */
				} /* end SSP4 PnP */ 
				if ((((bid & 0xf8) == 0x10) || ((bid & 0xf8) == 0x08)) && 
			    	((bid & 0x7) > 0) && ((inb(portn[k] + 2) & 0xC0) == 0xC0)) { 
#ifdef	RELIABLE_PNP
					unsigned char port4,port5,port7;
#endif
					struct cntrl_cfig *idcp ;
					pnp = 1;
					/* use Plug and Play settings */ 
					idcp = &oper_input.isa_def_config;
					/* I/O port used */ 
					idcp->base_port = portn[k]; 
	/*
	** Memory returned from PnP is unreliable - we'll find an ISA hole
	** ourselves.  The port number allocated is ok.
	**				Mikes 12/19/01
	*/
#ifdef	RELIABLE_PNP
					oper_input.isa_def_specd = 0xff; 
					idcp->cntrlr_mode = 0 ; /* Paged! */ 
					/* 16 bits! */ 
					idcp->cntrlr_bus_inface = 0x20 ; 
					/* Address */ 
					port4 = inb(portn[k] + 0x4);
					port5 = inb(portn[k] + 0x5);
					port7 = inb(portn[k] + 0x7);
					idcp->cntrlr_base_addr = 
					(port5 << 16) | ((port4 & 0xc0) << 8) ;
					if (!(idcp->cntrlr_base_addr && 
					((port7 & 0xC0) == 0xC0))) {
						ist.ctlr_array[nmegaport].
							cnfg_state=2; 
						ist.ctlr_array[nmegaport].
							cnfg_fail = 6;
						mpd->mpd_alive = false;
						nmegaport++;
						continue;
					}
#endif	/* RELIABLE_PNP */
				} /* end SSP64 PnP */ 
			if(!pnp) {
				if (MegAddr) {
					struct cntrl_cfig *idcp ;
					/* Default settings specified! */ 
					oper_input.isa_def_specd = 
						0xff; 
					idcp = &oper_input.isa_def_config;
					idcp->base_port = 
						portn[k]; /* I/O port used */ 
					/* Address */ 
					idcp->cntrlr_base_addr = MegAddr;
					idcp->cntrlr_mode = 0 ; /* Paged! */ 
					/* 16 bits! */ 
					idcp->cntrlr_bus_inface = 0x20 ;
				}
				else
					/* Dynamic for this board */ 
					oper_input.isa_def_specd = 1;

			}

#ifdef DEBUG
printk("before determ_isa_cnfg for address %x\n",(unsigned int) portn[k]);
#endif
/* Pat */
				determ_isa_cnfg(portn[k],&isa_parms,
					((uchar_t)(((mpd->mpd_id & 0xF8) < SSP2_ASIC) ? IFNS_SSP :
					IFNS_SSP4)));
#ifdef DEBUG
printk("finished determ_isa_cnfg for address %x\n",(unsigned int) portn[k]);
#endif
/* Pat */
				ist.ctlr_array[nmegaport].cnfg_init = 
					isa_parms.cntrlr_parms.cntrlr_mode;

				if (isa_parms.cntrlr_parms.cntrlr_mode == 0x40) 
				{
					printk("Board in flat mode\n");
					/*  flat mode */
					mpd->mpd_nwin = 0;
					switch(bid & 0xF8)
					{
						case 0x10:
							mpd->mpd_addrsz = 
								FLAT128_MEM_LEN;
							break;
						case 0x08:
							mpd->mpd_addrsz = 
								FLAT64_MEM_LEN;
							break;
					
#ifdef SSM
                                        	case 0x30:
                                        	case 0x38:
#endif
                                        	case 0x18:
						case 0x20:
						case 0x28:
							mpd->mpd_addrsz = 
								FLAT8_MEM_LEN;
							break;
						case 0xF0:
						case 0xF8:
							switch((bid & 0xFF00)>>8) {
							   case 0x24:
							   case 0x09:
							   case 0x36:
							      mpd->mpd_addrsz =
							      FLAT16K_MEM_LEN;
							      break;
							   case 0x1B:
							      mpd->mpd_addrsz =
							      FLAT32K_MEM_LEN;
							      break;
							   default:
							      mpd->mpd_addrsz =
							      FLAT16K_MEM_LEN;
							      break;
							}
							break;
						default:
							/* when flat mode is 
							 * allowed move below 
							 * code to here and 
							 * remove the break 
							 */
							break; 
					}
					/* FOR NOW FLAT MODE IS NOT ALLOWD FOR 
					 * ISA (default error case code) */
					ist.ctlr_array[nmegaport].cnfg_state=2; 
					/* invalid setting*/
					ist.ctlr_array[nmegaport].cnfg_fail = 6;
					/* SSM found testing SSM */
					mpd->mpd_alive = false;

					nmegaport++;
		   			continue; /* in the for loop */
				} else if (isa_parms.cntrlr_parms.cntrlr_mode ==
					0x0) {
					/*  use 16k window */
					mpd->mpd_nwin = HA_WIN16;
					/*  and 16K size */
					mpd->mpd_addrsz = 0x4000;
				} else {
					printk("Invalid card\n");
					ist.ctlr_array[nmegaport].cnfg_state=2; 
					/* invalid setting*/
					ist.ctlr_array[nmegaport].cnfg_fail = 6;
					/* SSM found testing SSM */
					mpd->mpd_alive = false;
					nmegaport++;
			   		continue;
				}

                                if (isa_parms.cntrlr_parms.cntrlr_bus_inface ==
					0x20)
					mpd->mpd_mem_width = 16;
				else {


#ifdef UNIX
					cmn_err(CE_CONT, 
					"EQNX: No 16 bit holes found\n");
#else
					printk("EQNX: No 16 bit holes found\n");
#endif
  					/* walk all 8-bit holes run pram test */
                                        for (i=0; i< HOLE_TABLE_SIZE; i++)
                                        {
                                                if (ist.sys_isa_holes[i] == 0)
                                                       continue;
                                                if (ist.sys_isa_hole_status[i]==
							0xf1 ||
						   ist.sys_isa_hole_status[i] ==
							0xfe)
                                                {
							b0 = test_8bit_hole(mpd,k,
							ist.sys_isa_holes[i]);
							if (b0)
								{
#ifdef UNIX
                                                              cmn_err(CE_CONT, "EQNX: Found a hole at %d \n",i);
#else
                                                              printk("EQNX: Found a hole at %d \n",i);
#endif
                                                              break;
                                                           }
                                                 }
                                        } /* End for */
                                        if (b0)
                                        { 
						mpd->mpd_mem_width = 16;
                                                isa_parms.cntrlr_parms.cntrlr_base_addr =
                                                     ist.sys_isa_holes[i];
#ifdef UNIX
                                                cmn_err(CE_CONT,
                                                "EQNX: ISA 8bit Address %x as 16 bit wide hole.\n",
                                                     ist.sys_isa_holes[i]);
#else
                                                printk("EQNX: ISA 8bit Address %x as 16 bit wide hole.\n",
                                                     (unsigned int) ist.sys_isa_holes[i]);
#endif

					} /* End if(b0) */

#if  0  /* re-done in new version see above*/
/* Try 0xb0000 video hole trick -- empirical data tells us that
   we can use that hole even if the video appears to be 8 bit.
   Algorithm: if 0xb0000 and 0xb4000 are available at 8 bits then check if 
   0xa0000, 0xa4000, 0xa8000, 0xac000, 0xb8000 and 0xbc000 are all ram. 
	If they are, use 0xb0000 as a 16 bit hole and print message. */ 
					int b0 = 1;
					for (i=0; i< HOLE_TABLE_SIZE; i++) {
						if (ist.sys_isa_holes[i] == 0) break;
						if ((ist.sys_isa_holes[i] == 0xa0000) &&
							 (ist.sys_isa_hole_status[i] != 2)) { b0 = 0; break; }
						if ((ist.sys_isa_holes[i] == 0xa4000) &&
							 (ist.sys_isa_hole_status[i] != 2)) { b0 = 0; break; }
						if ((ist.sys_isa_holes[i] == 0xa8000) &&
							 (ist.sys_isa_hole_status[i] != 2)) { b0 = 0; break; }
						if ((ist.sys_isa_holes[i] == 0xac000) &&
							 (ist.sys_isa_hole_status[i] != 2)) { b0 = 0; break; }
						if ((ist.sys_isa_holes[i] == 0xb8000) &&
							 (ist.sys_isa_hole_status[i] != 2)) { b0 = 0; break; }
						if ((ist.sys_isa_holes[i] == 0xbc000) &&
							 (ist.sys_isa_hole_status[i] != 2)) { b0 = 0; break; }
						if ((ist.sys_isa_holes[i] == 0xb0000) &&
							 (!(ist.sys_isa_hole_status[i] == 0xf0 || 
							  ist.sys_isa_hole_status[i] == 0xf1 ||
							  ist.sys_isa_hole_status[i] == 0xfe))) { b0 = 0; break; }
						if ((ist.sys_isa_holes[i] == 0xb4000) &&
							 (!(ist.sys_isa_hole_status[i] == 0xf0 || 
							  ist.sys_isa_hole_status[i] == 0xf1 ||
							  ist.sys_isa_hole_status[i] == 0xfe))) { b0 = 0; break; }
					}
					if (b0) { 
						mpd->mpd_mem_width = 16;
						isa_parms.cntrlr_parms.cntrlr_base_addr = 0xb0000;
						printk("EQNX:ISA: 0xb0000 used as 16 bit wide hole.\n");
                                        }

#endif /* 0 */
					else {
						/* 
						 * End 0xb0000 8-bit kludge code
						 */
						mpd->mpd_mem_width = 8;
						/* FOR NOW 8 bit MODE IS NOT 
						 * ALLOWD FOR ISA (default 
						 * error case code) */
						printk("EQNX:ISA: No 16 bit wide holes found.\n");
						ist.ctlr_array[nmegaport].cnfg_state = 2; 
						ist.ctlr_array[nmegaport].cnfg_fail = 6; /* invalid setting*/
                                                /* SSM found testing SSM */
                                                mpd->mpd_alive = false;
                                                nmegaport++;
		   				continue; /* in the for loop */
					}
				} /* else */

				mpd->mpd_pmem = 
					isa_parms.cntrlr_parms.cntrlr_base_addr;
#ifdef DEBUG
printk("ISA board mem addrress is %x\n", (unsigned int) mpd->mpd_pmem);
#endif
				mpd->mpd_nicps = 
					(uchar_t)isa_icp_count(mpd->mpd_id);
				nicps += mpd->mpd_nicps;
				mpd->mpd_nchan = brd_nchan_cfg(mpd->mpd_id);
                                /*SSM, found while testinf SSM, no memsz of ISA
                                    brds, thus no memory tests .....
                                */
#ifdef SSM
                                switch (mpd->mpd_id & 0xF8)
                                {
                                      case 0x10:   /* 128 port board */
                                         mpd->mpd_memsz= 0x100000;  /* 1 meg */
                                         break;
                                      case 0x08:   /* 64 port board */
                                         mpd->mpd_memsz= 0x80000;  /* 0.5 meg */
                                         break;
                                      case 0x30:   /* SSM 12 */
                                      case 0x38:   /* SSM 24 */
                                      case 0x18:  /* 8 port */
                                      case 0x20: /* 4 port board */
                                         mpd->mpd_memsz= 0x8000;  /* 32k */
                                         break;
				      case 0xF0:
				      case 0xF8:
				        switch ((mpd->mpd_id & 0xFF00) >> 8) {
				          case 0x24:
					  case 0x08: /* SST-4I,PWR */
				          case 0x09:
					  case 0x36: /* MPM-4 */
					  case 0x3F: /* SST-4I+ */
				            mpd->mpd_memsz = 0x4000; /* 16K */
				            break;
				          case 0x2D: /* SSM-8I */
					  case 0x12: /* SST-8I+ */
					  case 0x18: /* SST-8I,PWR */
					  case 0x1B: /* SST-8I */
				            mpd->mpd_memsz = 0x8000; /* 32K */
				            break;
					  case 0x17: /* SST-16I no panel */
					  case 0x0F: /* SST-16I with 16RJ */
					  case 0x07: /* SST-16I with 16DB */
						/* 64K */
						mpd->mpd_memsz = 0x10000; 
						break;
				          default:
				            mpd->mpd_memsz = 0;
				            break;
				        }
				        break;
                                      default:
                                         mpd->mpd_memsz= 0;
                                         break;
                                }
#endif
                                mpd->mpd_minor = nextmin;
				nextmin += NCHAN_BRD;
				mpd->mpd_alive = true;
				mpd->mpd_slot = 0;
				nmegaport++;

    				if ((mpd->mpd_id & 0xF8) < SSP2_ASIC) { /* SSP64 board */
       					/* Set the address, 16-bit mode, MAX2, 
					 * and flat off */ 
       					outb(( (mpd->mpd_pmem >> 14) & 0xff),
						portn[k]);
       					outb(((mpd->mpd_pmem >> 22) & 3)| 0x20,
						portn[k] + 1);
       					/* page register - deselect ram */  
				
       					outb(0,portn[k] + 2);
       					outb(0,portn[k] + 3);

       					/* disable interrupts */ 
       					outb(0, portn[k] + 6);
    				}
    				else { /* SSP4 board */
      					unsigned char b;
    				 
      					/* reg 8 (page reg) */
      					if(mpd->mpd_nwin == HA_FLAT)
         					b = 0x20;
      					else
         					b = 0;

      					outb( b | 0x00, portn[k] + 0x08);

      					b = ((mpd->mpd_pmem & 0x0000c000) >> 8);
      					/* no ints */
      					outb(b, portn[k] + 0x09);
      					b =((mpd->mpd_pmem & 0xFFFF0000) >> 16);
      					outb(b,portn[k] + 0x0a);
      					/* reg b*/
      					b=0;
      					outb(b,portn[k] + 0x0b);
    				} /* SSP4 */
	
			} /* board id found */
		} /* ISA for loop */
	}
#endif	/* ISA_ENAB || MCA_ENAB */

#ifdef	MCA_ENAB
	if (mca_bus) {
 		/* loop thru MCA slots to board(s) */
		for ( k = 0; k < 8; k++ )
		{
			switch ( mca_brd_found( k, mca_cfg ) )
		    {
		    case true:
				if (++ist.ctlr_found > maxbrd) {
					ist.ctlr_array[nmegaport].cnfg_state = 2; 
					ist.ctlr_array[nmegaport].cnfg_fail = 5; /* maxbrd exceeded */
		   		continue;
				}
				else
					ist.ctlr_array[nmegaport].cnfg_state = 1; 
				mpd = &meg_dev[nmegaport];
				mpd->mpd_alive = true;
				mpd->mpd_slot = k;
				ist.ctlr_array[nmegaport].ctlr_loc = mpd->mpd_slot;
				mpd->mpd_id = mca_cfg[8] & 0xf8; /* Save model number only */
				ist.ctlr_array[nmegaport].ctlr_model = mpd->mpd_id;
				ist.ctlr_array[nmegaport].ctlr_model_ext = 
					((mpd->mpd_id & 0xFF00) >> 8);
				ist.ctlr_array[nmegaport].ctlr_rev = mca_cfg[8] & 0x7;/* revision */
				mpd->mpd_bus = MCA_BUS;
				ist.ctlr_array[nmegaport].ctlr_bus_type = MCA_BUS; 
				ist.ctlr_array[nmegaport].ctlr_mca_port = mca_cfg[10];
				ist.ctlr_array[nmegaport].cnfg_init = 1; /* MCA is autoconfig */
				mpd->mpd_nchan = brd_nchan_cfg(mpd->mpd_id);
				mpd->mpd_minor = nextmin;
				nextmin += NCHAN_BRD;
				mpd->mpd_nicps = (uchar_t)mca_icp_count(mpd->mpd_id);
				nicps += mpd->mpd_nicps;
				mpd->mpd_pmem = mca_base_paddr( mca_cfg );
				ist.ctlr_array[nmegaport].cnfg_physc_base = mpd->mpd_pmem;
                         /* SSM testing, found testing SSM , check for Zero size */
#ifdef SSM
                                if ((mpd->mpd_addrsz = mca_mem_size( mca_cfg, 0 ))==0)
                                {
                                    ist.ctlr_array[nmegaport].cnfg_state = 2;
                                    ist.ctlr_array[nmegaport].cnfg_fail = 6; /* fail bad address */
                                    nextmin += NCHAN_BRD;
                                    nmegaport++;
                                    mpd->mpd_alive = false;
                                    continue;
                                }
#else
                                mpd->mpd_addrsz = mca_mem_size( mca_cfg, 0 );
#endif
                                ist.ctlr_array[nmegaport].cnfg_addr_size = mpd->mpd_addrsz;
				if ((mpd->mpd_nwin = mca_mem_strat(mca_cfg)) == HA_WIN16)
					ist.ctlr_array[nmegaport].cnfg_mode = 1 /* Paged */;
				else
					ist.ctlr_array[nmegaport].cnfg_mode = 2 /* Flat */;
				mpd->mpd_mem_width = 16;
				ist.ctlr_array[nmegaport].cnfg_isa_bus_sz = 0x10; /* 16 bit mode */
				mpd->mpd_memsz = mca_mem_size( mca_cfg, 1 );
				mpd->mpd_pgctrl = (mca_cfg[11] << 8) | (mca_cfg[10] + 2);
				nmegaport++;
					break;
		    case -1:
#ifdef UNIX
		    	cmn_err( CE_CONT, 
		"Run Microchannel configuration utility for Equinox board in slot %d.\n", 
		     	k );
#else
		printk("Run Microchannel configuration utility for Equinox board in slot %d.\n", 
		     	k );
#endif
					break;
				default:
					break;
    		}
			} /* for */
		} /* mca_bus */
#endif	/* MCA_ENAB */

	if(!nmegaport) {
#ifdef UNIX
	   cmn_err(CE_NOTE, "No Equinox boards found.\n");
#else
	   printk("NOTICE: No Equinox boards found.\n");
#endif
	   return 1;
	}
	if(((struct mpchan *) meg_chan = 
		(struct mpchan *)vmalloc(sizeof(struct mpchan) * 
			nmegaport * NCHAN_BRD)) == (struct mpchan *) NULL){
		printk("EQUINOX: Failed to allocate virtual address space of size for meg_chan %ld\n", (unsigned long)(sizeof(struct mpchan) * nmegaport * NCHAN_BRD));
		return(-1);
		}
	memset(meg_chan, 0, (sizeof(struct mpchan) 
			* nmegaport * NCHAN_BRD));
	mpc = meg_chan;
	for(k=0; k < nmegaport; k++) {
		/* map Adapter memory */
		mpd = &meg_dev[k];
		if (mpd->mpd_alive == 0) 
			continue; /* If board is dead skip it */
		mpd->mpd_mpc = (struct mpchan *)&meg_chan[k * NCHAN_BRD];
/* Pat */	
#if (EQNX_VERSION_CODE < 131328)
	if (mpd->mpd_pmem > 0x100000){
#endif
/* Pat */
#if (EQNX_VERSION_CODE > 66304)
		if(((void *) mpd->mpd_mem = ioremap(mpd->mpd_pmem,mpd->mpd_addrsz)) == (char *) NULL)
		{
			printk("EQUINOX: Failed to allocate virtual address space of size %d\n",
				mpd->mpd_addrsz);
		}
#else
		if(((void *) mpd->mpd_mem = vmalloc(mpd->mpd_addrsz)) == (char *) NULL)
		{
			printk("EQUINOX: Failed to allocate virtual address space of size %d\n",
				mpd->mpd_addrsz);
		}
		if ((rc = remap_page_range((TASK_SIZE + 
			((unsigned long) mpd->mpd_mem)), mpd->mpd_pmem, 
				mpd->mpd_addrsz, PAGE_IOMEM))) {
#ifdef	IX86
			printk("EQUINOX: Failed to map physical address space = %x, errno = %d for board %d\n",
#endif	/* IX86 */
#ifdef	IA64
			printk("EQUINOX: Failed to map physical address space = %lx, errno = %d for board %d\n",
#endif	/* IA64 */
				(unsigned long) mpd->mpd_pmem, rc,k + 1); 
		}
#endif
/* Pat */	
#if (EQNX_VERSION_CODE < 131328)
	}
	else
		mpd->mpd_mem =  mpd->mpd_pmem;
#endif
/* Pat */  
		if(!mpd->mpd_mem) {
	   		printk("EQNX: Memory mapping failed for board %d\n", 
				k +1);
			/* Driver init failed */
			ist.ctlr_array[k].cnfg_state = 2; 
			ist.ctlr_array[k].cnfg_fail = 1; /* sptalloc failed */
                	/* SSM found testing SSM */
                	mpd->mpd_alive = false;
        	   	continue;
		}
        	brd_mem_cfg(mpd);
		hwq = mpd->mpd_hwq;
		/* enable board memory - for eisa, memory is switched off at 
		 * boot */
		/* However, for PCI, memory is switched on already. */
		if((mpd->mpd_nwin == HA_FLAT) && !(mpd->mpd_bus == PCI_BUS)) {
                  if((mpd->mpd_id & 0xF8) < SSP2_ASIC)
	   	    outw(0x100, mpd->mpd_pgctrl); /* enable all in flat mode */
                  else
                     /*SSP2 enables all for HA_FLAT */
                     outb( 0x20, mpd->mpd_pgctrl);  
                } /* HA_FLAT */

		if((mpd->mpd_id & 0xF8) < SSP2_ASIC)
			mpd->mpd_sspchan = NCHAN; /* SSP64 */
		else
			mpd->mpd_sspchan = 4; /* SSP4 */ 

		/* loop n icps */
		for(j=0; j < (int)mpd->mpd_nicps; j++) {
			int mux;

			/* select correct page if necessary */
			if(mpd->mpd_nwin == HA_WIN16) {
                          if((mpd->mpd_id & 0xF8) < SSP2_ASIC)
			    outw(0x100 | j ,mpd->mpd_pgctrl);
                          else
                            outb(1 << j,  mpd->mpd_pgctrl); 
                        } /* HA_WIN16 */

			/* reset icp to known state */
			icp = &mpd->icp[j];
			icp->icp_minor_start = mpd->mpd_minor + (j * mpd->mpd_sspchan);
                        if ((mpd->mpd_id & 0xF8) < SSP2_ASIC) {  /* SSP64 */
			  icpg = (icpgaddr_t)((unsigned long)icp->icp_regs_start+0x2000);
			  i_gicp_attn = 0;
			  Gicp_initiate = 0;
                          bus_ctrl_p = &(icpg->ssp.gicp_bus_ctrl);
			/*
	 		 * If on pci_bus, then we must set up the Global Bus 
			 * Control Register with the CPU bus width (32 bit bus)
			 * and the 40 Bit DRAM bit. The Global Bus Control 
			 * Register is at offset 0x18.
	 		 */
	   		if(!j) {
	      			if ( mpd->mpd_bus ==  PCI_BUS)
					*bus_ctrl_p = 
						(STERNG | DRAM_40 | CPU_BUS_32);
	      			else
					/* 16-bits|dram-steer */
	         			*bus_ctrl_p = 0x9;
	   		}
	   		else {
	      			if ( mpd->mpd_bus ==  PCI_BUS)
	         			*bus_ctrl_p = (DRAM_40 | CPU_BUS_32);
	      			else
	         			*bus_ctrl_p = 0x1;		/* 16-bits */
	   		}
                        }
                        else { /* SSP2/4 */
                          /* VIRTUAL- ptr to global regs */
                          union global_regs_u *icp_glo =
                            (union global_regs_u *)((unsigned long)icp->icp_regs_start + 0x400);
                          /* VIRTUAL ptr to output regs */
                          union output_regs_u *icp_cout =
                            (union output_regs_u *)((unsigned long)icp->icp_regs_start + 0x200);
                          /* VIRTUAL ptr to input regs */
                          union input_regs_u *icp_cin =
                            (union input_regs_u *)(icp->icp_regs_start);

	                  icp_glo->ssp2.on_line = 0;   /* disable SSP2 */
	                  icp_cin->ssp2.locks = 0xFF;/* input locks SSP2 */
                          /* output locks SSP2 */
	                  icp_cout->ssp2.locks = 0x77;
                          /* Set Bus Control = 16 bits */
                          bus_ctrl_p = &(icp_glo->ssp2.bus_cntrl);
                          *bus_ctrl_p = 0x05;
			  if (mpd->mpd_bus == PCI_BUS) {
			     switch(mpd->mpd_id & 0xFF) {
			        case 0x90:
			        case 0x94:
			        case 0x88:
			        case 0xB8:
			        case 0x8C:
			        case 0x98:
			        case 0x9C:
			        case 0xFC:
			           ssp_channels = 4;
			           break;
			        case 0xA8:
			           ssp_channels = 2;
			           break;
  			        default:
			           ssp_channels = 4;
			           break;
			     }
			  } else {
                             switch((unsigned int)(mpd->mpd_id & 0xFF00) >> 8){
                                case 0x24:
                                    ssp_channels = 2;
                                    break;
                                case 0x09:
                                    ssp_channels = 4;
                                    break;
                                case 0x1B:
                                    ssp_channels = 4;
                                    break;
                                case 0x2E: /* SST-16I */
                                case 0x2D: /* SSM-8 */
                                case 0x12: /* SSM-8I+ (or SST-8I/RJ) */
                                case 0x3F: /* SSM-4I+ (or SST-4I/RJ) */
                                case 0x08: /* SSM-4I,PWR */
                                case 0x18: /* SSM-8I,PWR */
                                    ssp_channels = 4;
                                    break;
                                case 0x36: /* MPM-4 */
                                    ssp_channels = 4;
                                    break;
                                default:
                                    ssp_channels = 4;
                                    break;
                             } 
			  }

                          for(ii = 0; ii < ssp_channels; ii++) /* set all locks */
                          {
                              ((union input_regs_u *)((char *)icp_cin + ii))->ssp2.locks = 0xFF;
                              ((union output_regs_u *)((char *)icp_cout + ii))->ssp2.locks = 0x77;
                              ((union output_regs_u *)((char *)icp_cout + ii))->ssp2.cntrl_sigs = 0x0F;
                          }
			  icp_glo->ssp2.bus_cntrl = 0x01;
			  icp_glo->ssp2.on_line = 0;
                        }
	
			/* Adjust the number of icps on the board  */
			/*
			printk("no of icps found = %d\n",mpd->mpd_nicps);
			*/

  			if( (ii = mem_test(mpd,j))) {
#ifdef UNIX
				cmn_err(CE_CONT,
				"EQNX: Memory test failed for board %d SSP %d\n",
				k+1,j+1);
#else
				printk("EQNX: Memory test failed for board %d SSP %d\n",
				k+1,j+1);
#endif
				ist.ctlr_array[k].cnfg_state = 2; /* Driver init failed */
				ist.ctlr_array[k].cnfg_fail = 2; /* mem_test failed */
                		/* SSM found testing SSM */
                		mpd->mpd_alive = false;
                		continue;
  			}

  			/* verify that pram is not cached */
    			if ((mpd->mpd_id & 0xF8) < SSP2_ASIC) { /* SSP64 board */
			  	icpg = (icpgaddr_t)((unsigned long)icp->icp_regs_start+0x2000);
       				/* verify that pram is not cached */
      				cchnl = Gicp_chnl;
      				chnl_ptr = &(Gicp_chnl);
    			}
    			else { /* SSP4 board */
      				icpaddr_t icp;
      				union global_regs_u *icp_glo ;
      				icp = &mpd->icp[j];
      				icp_glo = (union global_regs_u *)((unsigned long)icp->icp_regs_start + 0x400);
      				cchnl = icp_glo->ssp2.chan_ctr;
      				chnl_ptr = &(icp_glo->ssp2.chan_ctr);
    			}
      			no_cache = false;
      			for ( ii = 0; ii < 0x100000; ii++ )
        			if ( *chnl_ptr != cchnl )
        			{
          				no_cache = true;
          				break;
        			}
  			if ( !no_cache)
  			{
    				if (mpd->mpd_bus == ISA_BUS)
     				/*SSM, found testing SSM wrong error messages */
          			printk("EQNX: PRAM memory appears to be cached %lx for I/O address 0x%x.\n",
          			(paddr_t) mpd->mpd_mem,portn[mpd->mpd_io]);
    				else
          				printk("EQNX: PRAM memory appears to be cached %lx for board in Slot %d.\n",
          				(paddr_t) mpd->mpd_mem,mpd->mpd_slot);
				/* Driver init failed */
				ist.ctlr_array[k].cnfg_state = 2; 
				/* PRAM cached */
				ist.ctlr_array[k].cnfg_fail = 3; 
                		/* SSM found testing SSM */
                		mpd->mpd_alive = false;
                		continue;
  			} /* End if(!no_cache) */
    			lmx = 0;
    			mp = (mpaddr_t)mpd->icp[j].icp_regs_start;

      			if((mpd->mpd_id & 0xF8) < SSP2_ASIC) {
				switch((mpd->mpd_id & 0xF8)) {
					case 0x10:
						/* Number per ICP */
						ssp_channels = 64;
						break;
					case 0x08:
						ssp_channels = 64;
						break;
					case 0x38:
						ssp_channels = 64;
						break;
					case 0x30:
						ssp_channels = 64;
						break;
					case 0x18:
						ssp_channels = 64;
						break;
					case 0x20:
						ssp_channels = 64;
						break;
					case 0x28:
						ssp_channels = 64;
						break;
					default:
						ssp_channels = 0;
						break;
				}
      			}
#ifdef DEBUG
printk("eqnx_init:queue size for board %d = %d\n\n", k, mpd->mpd_hwq->hwq_size);
#endif

/*NEW 3.30*/
    			if ((mpd->mpd_id & 0xF8) >= SSP2_ASIC) {/* SSP4 board */
      				icpaddr_t icp;
      				union global_regs_u *icp_glo ;
      				icp = &mpd->icp[j];
      				icp_glo = (union global_regs_u *)((unsigned long)icp->icp_regs_start + 0x400);
      				cchnl = icp_glo->ssp2.chan_ctr;
      				chnl_ptr = &(icp_glo->ssp2.chan_ctr);
						for ( ii = 0; ii < 0x10000; ii++ )
							if ( *chnl_ptr != cchnl ) break;
#ifdef v334g
						if (((mpd->mpd_id & 0xFFF8) == 0x2DF8) ||  /* SSM-8I */
						    ((mpd->mpd_id & 0xFFF8) == 0x3FF8) ||  /* SSM-4I/RJ */
						    ((mpd->mpd_id & 0xFFF8) == 0x12F8) ||     /* SSM-8I/RJ */
						    ((mpd->mpd_id & 0xFC) == 0x94)     || /* SST-8P/RJ */
						    ((mpd->mpd_id & 0xFC) == 0x8C) /* SST-4P/RJ */)
							
							icp_glo->ssp2.bus_cntrl = 0x8D; 
						else
#endif
							icp_glo->ssp2.bus_cntrl = 0xCD; 
						cchnl = icp_glo->ssp2.chan_ctr;
						for ( ii = 0; ii < 0x10000; ii++ )
							if ( *chnl_ptr != cchnl ) break;
    			}
/*NEW 3.30*/
			mux = 0; /* if SST-16I or SST-16P must mux ctrl sigs */
      	if ((mpd->mpd_id & 0xF8) >= SSP2_ASIC) { 
				int id; 
				if (mpd->mpd_bus == PCI_BUS) 
					id = (mpd->mpd_id & 0xFF);
				else
					id = (mpd->mpd_id & 0xFF00) >> 8;
				if( (id == 0xC0) || (id == 0xC4) || 
				  (id == 0xC8) || (id == 0xD0) ||
   				 (id == 0x07) || (id == 0x0F) || (id == 0x17) )   
					mux = 1;
			}

  			/*loop thru each channel - all bytes have been zero'd */
  			for ( i = 0; i <  ssp_channels; i++ )
			{
    				/* setup mpc virtual addresses for cpu access 
				 * of icp */
				jj = mpd->mpd_minor + i + 
					(j * mpd->mpd_sspchan);
				mpc = &meg_chan[jj];
				mpc->mpc_brdno = k;
				/* chan # on this icp */
				mpc->mpc_chan = i;
				mpc->mpc_icpi = (icpiaddr_t)&mp->mp_icpi[i];
      				if((mpd->mpd_id & 0xF8) < SSP2_ASIC)
					mpc->mpc_icpo = (icpoaddr_t)&mp->mp_icpo[i];
				else
					mpc->mpc_icpo = (icpoaddr_t)((unsigned char *)(mpc->mpc_icpi) + 0x200);
				mpc->mpc_mpd = mpd;
				mpc->mpc_icp = (icpaddr_t)&mpd->icp[j];
				/* icp # on this board */
				mpc->mpc_icpno = j;
				mpc->mpc_ptty = mpc->mpc_tty;


				if(((struct termios *) mpc->normaltermios = 
				(struct termios *)vmalloc(sizeof(
				struct termios))) == (struct termios *) NULL){
					printk("EQUINOX: Failed to allocate virtual address space of size %d for normaltermios\n", (unsigned int)sizeof(struct termios));
					return(-1);
				}
				memset(mpc->normaltermios, 0, 
					sizeof(struct termios));
				*mpc->normaltermios = eqnx_deftermios;
#if	(LINUX_VERSION_CODE < 132096)
				if(((struct termios *) mpc->callouttermios = 
				(struct termios *)vmalloc(sizeof(
				struct termios))) == (struct termios *) NULL){
					printk("EQUINOX: Failed to allocate virtual address space of size %d for callouttermios\n", (unsigned int)sizeof(struct termios));
					return(-1);
				}
				memset(mpc->callouttermios, 0, 
					sizeof(struct termios));
				*mpc->callouttermios = eqnx_deftermios;
#endif
				mpc->closing_wait = CLSTIMEO;
				mpc->close_delay = EQNX_CLOSEDELAY;
				mpc->tqhangup.routine = eqnx_dohangup;
				mpc->tqhangup.data = mpc;

				/*
				** initialize each of the wait queues
				*/
#if	(LINUX_VERSION_CODE < 132096)
				/* kernels before 2.4 */
				init_waitqueue(&mpc->open_wait);
				init_waitqueue(&mpc->close_wait);
				init_waitqueue(&mpc->raw_wait);
				init_waitqueue(&mpc->mpc_mpa_slb);
				init_waitqueue(&mpc->mpc_mpa_clb);
				init_waitqueue(&mpc->mpc_mpa_rst);
				init_waitqueue(&mpc->mpc_mpa_call_back);
#else
				/* 2.4 kernels and after */
				init_waitqueue_head(&mpc->open_wait);
				init_waitqueue_head(&mpc->close_wait);
				init_waitqueue_head(&mpc->raw_wait);
				init_waitqueue_head(&mpc->mpc_mpa_slb);
				init_waitqueue_head(&mpc->mpc_mpa_clb);
				init_waitqueue_head(&mpc->mpc_mpa_rst);
				init_waitqueue_head(&mpc->mpc_mpa_call_back);
#endif

				mpc->mpc_last_esc = 0xffff;
				mpc->mpc_esc_timo = 0;
				mpc->mpc_scrn_no = 0;
				mpc->mpc_input = 0;
				mpc->mpc_output = 0;
				mpc->mpc_scrn_count = 0;
				mpc->mpc_cin_events = 0;
				mpc->mpc_cout_events = 0;
				mpc->mpc_cin_ena = 0;
				mpc->mpc_cout_ena = 0;
				mpc->mpc_mpa_stat = 0;
         			mpc->mpc_mpa_init_retry = 0;
         			mpc->mpc_mpa_reset_retry = 0;
        			icpi = mpc->mpc_icpi;
				icpo = mpc->mpc_icpo;
      				if((mpd->mpd_id & 0xF8) < SSP2_ASIC) {
					icpg = (icpgaddr_t)icpo;
					lmx_factor = 16; 
				}
				else {
					icpg = (icpgaddr_t)((unsigned long)icpi + 0x400);
					lmx_factor = 4; 
				}
				icpq = &icpo->ssp.cout_q0;
				if(i == 0 * lmx_factor) {
	  				lmx = 0; 
	  				mpc->mpc_icp->lmx[lmx].lmx_mpc = mpc;
	  				mpc->mpc_icp->lmx[lmx].lmx_wait = -1;
				}

       				if(i == 1 * lmx_factor) {
         				lmx = 1;
         				mpc->mpc_icp->lmx[lmx].lmx_mpc = mpc;
         				mpc->mpc_icp->lmx[lmx].lmx_wait = -1;
      				}

        			if(i == 2 * lmx_factor) {
	   				lmx = 2;
	   				mpc->mpc_icp->lmx[lmx].lmx_mpc = mpc;
	   				mpc->mpc_icp->lmx[lmx].lmx_wait = -1;
				}
				if(i == 3 * lmx_factor) {
	   				lmx = 3; 
	   				mpc->mpc_icp->lmx[lmx].lmx_mpc = mpc;
	   				mpc->mpc_icp->lmx[lmx].lmx_wait = -1;
				}
				mpc->mpc_lmxno = lmx;


    				/* icp input - assign queues, etc. */
    				/* input buffer base pointer also sets tag base pointer! */
    				/* setup input hardware registers */
/*NEW*/
				icpi->ssp.cin_locks = 0xff; /* set locks */
				icpo->ssp.cout_lock_ctrl = 0xff;
				cur_chnl_sync(mpc); /* mpc is sufficiently initialized */
/*NEW*/
				if((mpd->mpd_id & 0xF8) < SSP2_ASIC) {
    					addr = (i + j * NCHAN) * 2 * hwq->hwq_size;
    					icpi->ssp.cin_dma_base = (addr >> 16);
    					icpi->ssp.cin_bank_a.ssp.bank_dma_ptr
          				= icpi->ssp.cin_bank_b.ssp.bank_dma_ptr
          				= icpi->ssp.cin_tail_ptr_a 
          				= icpi->ssp.cin_tail_ptr_b 
          				= addr & 0xffff;
        			} else {

           				addr = i * 0x400;  
           				icpi->ssp2.bank_a.nxt_dma
              				= icpi->ssp2.bank_b.nxt_dma
              				= icpi->ssp.cin_tail_ptr_a 
              				= icpi->ssp.cin_tail_ptr_b  
              				= addr & 0xffff;
        			}

    				/* setup mpc values for input hardware registers */
   				if(mpd->mpd_nwin == HA_FLAT) {
     					mpc->mpc_rxq.q_begin = addr & 0xffff;
     					mpc->mpc_rxq.q_ptr = addr & 0xffff;
     					mpc->mpc_rxbase = 0;
     					mpc->mpc_rxpg = 0;
     					mpc->mpc_tgpg = 0;
     					mpc->mpc_rxq.q_end =  (addr & 0xffff) + (hwq->hwq_size - 1); 
     					mpc->mpc_rxq.q_size = hwq->hwq_size;
					if ((mpd->mpd_id & 0xF8) < SSP2_ASIC) {
     						mpc->mpc_tags = addr >> 4;
     						mpc->mpc_rxq.q_addr = 
						(char *)(icp->icp_dram_start + 
						((i * 2 * hwq->hwq_size) & 
						0xffff0000));
					}
					else {
     						mpc->mpc_tags = 0;
     						mpc->mpc_rxq.q_addr = 
						(char *)(icp->icp_dram_start + 
						((i * hwq->hwq_size) & 
						0xffff0000));
					}
   				} else { 
     					mpc->mpc_rxq.q_begin = addr % 0x4000;
     					mpc->mpc_rxq.q_ptr = addr % 0x4000;
     					mpc->mpc_rxpg = addr / 0x4000;
     					mpc->mpc_rxbase = addr & 0xc000;
     					mpc->mpc_tags = (addr >> 4) / 0x4000;
     					mpc->mpc_tgpg = (addr >> 2) / 0x4000;
     					mpc->mpc_rxq.q_end =  (addr % 0x4000) + (hwq->hwq_size - 1); 
     					mpc->mpc_rxq.q_size = hwq->hwq_size;
     					mpc->mpc_rxq.q_addr = (char *)(icp->icp_dram_start);
   				}
    				/* setup additional icp input registers */
    				icpi->ssp.cin_overload_lvl = hwq->hwq_hiwat;
    				icpi->ssp.cin_susp_output_lmx = 0x41; 
				/* susp on lmx problems
				     	- TEMP was 0x63
                                     	- leave sigs masked */
    				icpi->ssp.cin_susp_output_sig = 0x00;
    				icpi->ssp.cin_q_ctrl = 0x20 | hwq->hwq_rxwrap;
				/* input q wrap size, tail ptr a, 
				 * overload enabled */
    				icpi->ssp.cin_min_char_lvl = 1;
    				icpi->ssp.cin_inband_flow_ctrl = 0;
    				/* unlock input */
				/* Start with Bank B locked */
    				icpi->ssp.cin_locks = 0x10 | LOCK_B; 
	
    				/* icp output - assign data queues and 
				 * command queues */
				/* setup queue 0 only for each channel */
    				/* use circular output data queue 
				 * - no session registers */

				if ((mpd->mpd_id & 0xF8) < SSP2_ASIC) {
					addr =((i + j * NCHAN) * 2 + 1) * 
					hwq->hwq_size;
    				Q_data_base = (addr >> 16);
    				Q_data_ptr = (addr & 0xffff);
    				q_type = 
			       		EN_CIRC_Q|EN_TX_LOW|EN_TX_EMPTY | 
					hwq->hwq_txwrap; 
    					/* permanent send data state */
    					Q_block_count = hwq->hwq_lowat / 64;  
					/* lowat mark in 64-byte blocks */ 
    					/* assign session reg. as first within 
			 	 	* this channel block */
    					icpo->ssp.cout_status |= (i << 10);
    					/* those fields need to be defined */
				} else {
					addr = i * 0x400;
					icpo->ssp2.q0.data_ptr = addr & 0xFFFF;
					icpo->ssp2.q0.buff_type_sz =
			       		   EN_CIRC_Q|EN_TX_LOW|EN_TX_EMPTY | 
					   hwq->hwq_txwrap; 
					icpo->ssp2.q0.low_threshold = 
					   hwq->hwq_lowat / 64;
					icpo->ssp2.intern_status |=
					   ((i * 4) << 8);
				}

				if((mpd->mpd_id & 0xF8) < SSP2_ASIC ) {
					/* circular,size, cause empty & lowat 
					 * events */
    					Q_out_state = CMDQ_CONT_SND | 
						hwq->hwq_cmdsize; 
    					icpo->ssp.cout_ses_ctrl_a = 1;
        			} else {
           				icpo->ssp2.q0.send_data_state = 0x10;  
           				icpo->ssp2.ses_a.expnd_ena = 0x01;
        			}

   				if(mpd->mpd_nwin == HA_FLAT) {
					int addr1;
					if ((mpd->mpd_id & 0xF8) >= SSP2_ASIC)
						addr1 = addr + 0x1000;
					else
						addr1 = addr;
     			 		mpc->mpc_txq.q_begin = addr1 & 0xffff;
     			 		mpc->mpc_txq.q_ptr = addr1 & 0xffff;
     					mpc->mpc_txbase = 0;
     					mpc->mpc_txpg = 0;
     					mpc->mpc_txq.q_end =  
						(addr1 & 0xffff) + 
						(hwq->hwq_size - 1); 
     					mpc->mpc_txq.q_size = hwq->hwq_size;
					if ((mpd->mpd_id & 0xF8) < SSP2_ASIC) 
     						mpc->mpc_txq.q_addr = 
						(char *)(icp->icp_dram_start + 
						(((i * 2 + 1) * hwq->hwq_size) &
						0xffff0000));
					else
     						mpc->mpc_txq.q_addr = 
						(char *)((unsigned long)icp->icp_dram_start + 
						((i * hwq->hwq_size) & 0xFFFF0000));
   				} else {
     					mpc->mpc_txq.q_begin = addr % 0x4000;
     					mpc->mpc_txq.q_ptr = addr % 0x4000;
     					mpc->mpc_txbase = addr & 0xc000;
     					mpc->mpc_txpg = addr / 0x4000;
     					mpc->mpc_txq.q_end =  
						(addr % 0x4000) + 
						(hwq->hwq_size - 1); 
     					mpc->mpc_txq.q_size = hwq->hwq_size;
					if ((mpd->mpd_id & 0xF8) < SSP2_ASIC) 
     						mpc->mpc_txq.q_addr = 
						(char *)(icp->icp_dram_start);
					else
     						mpc->mpc_txq.q_addr = 
						(char *)(icp->icp_dram_start +
						0x1000);
   				}
    				/* cmd queue */
				if ((mpd->mpd_id & 0xF8) < SSP2_ASIC)
    					addr = (MAXICPCHNL + i + 
					(j * NCHAN)) * hwq->hwq_size / 4;
				else
					addr = i * 0x400;
    				Q_cmd_base = (addr >> 16);
    				Q_cmd_ptr = (addr & 0xffff);

    				/* output timer - setup for 10 ms prescale */
#ifdef NEVER
    				icpo->ssp.cout_tmr_size = 20;   /* never change */
#endif /* NEVER */
    				icpo->ssp.cout_tmr_size = 2;   /* never change */
				/* place ms count here */
    				icpo->ssp.cout_tmr_count = 0;  

    				/* misc. output registers */
    				icpo->ssp.cout_ctrl_sigs &= ~0xff;  /* all off */
					if (mux)
	    				icpo->ssp.cout_ctrl_sigs |= 0x44;  /* mux em */
    				/* unlock output */
    				icpo->ssp.cout_lock_ctrl = 0x02;
    				icpo->ssp.cout_cpu_req ^= 0x04;  
				/* force send data state */
			}
  			/* check sanity of icp */
  			/* "ring clock failure" should be on since ring clock 
			 * is off */
			if ((mpd->mpd_id & 0xF8) < SSP2_ASIC) {
  				no_icp = !( i_gicp_attn & 0x04 );
  				if ( !no_icp || EQNft )
  				{
     					mpc = &meg_chan[icp->icp_minor_start];
    					/* setup mpc structures for each 
					 * channel */
    					for ( ii = 0; ii < ssp_channels; ii++,
						mpc++ )
    					{
      						/* don't enable lmx_cond_chng 
						 * until ring found */
      						icpi = mpc->mpc_icpi;
      						icpo = mpc->mpc_icpo;
						/* allow software attn */
      						icpi->ssp.cin_attn_ena = 0x8000;
						if (!(ii % 16))
							icpi->ssp.cin_attn_ena |= ENA_LMX_CNG;
						/* start with Bank B locked */
      						icpi->ssp.cin_locks = LOCK_B;
						/* allow software attn */
      						icpo->ssp.cout_attn_ena=0x8000;
      						icpo->ssp.cout_lock_ctrl^=0x03;
    					}

    					/* setup global interval timer for a 1 
					 * second pulse */
					/* 0.1 seconds before decrement */
    					Gicp_tmr_size = 192;   
					/* 10 decrements before pulse */
    					Gicp_tmr_count = 10;   
					gicp_bus |= 0x40;   /* enable it */

	icpg->ssp.gicp_watchdog = 0; /* Disable watchdog until megapoll runs */
       	 
    					/* enable global pram writes, dma, and 
				 	 * ring clock */
    					/* remaining ring/lmx managment 
					 * performed by megapoll() */
    					if(EQNft) {
	    					/* On fault tolerant system, 
					 	 * don't start up the ring clock
               				 	 * until program gives proper 
						 * IOCTL to turn it on. */
	    					Gicp_initiate = 
							0x1e - RNG_CLK_ON;
    					} else {
    						Gicp_initiate = 0x1e;
					}
    					mpd->icp[j].icp_rng_state = RNG_BAD;
					/* set ring bad */
    					mpd->icp[j].icp_rng_last = 0x4; 
  				} /* End if(!no_icp | EQNft) */
  				else
  				{
#ifdef UNIX
                			cmn_err( CE_CONT,"EQNX: SSP %d not detected for board with I/O address%d.\n",j+1,k+1);
#else
                			printk("EQNX: SSP %d not detected for board with I/O address%d.\n",j+1,k+1);
#endif
					/* Driver init failed */
					ist.ctlr_array[k].cnfg_state = 2; 
					/* ICP failed */
					ist.ctlr_array[k].cnfg_fail = 4; 
                			/* SSM found testing SSM */
                			mpd->mpd_alive = false;
                			continue;
  				}
/* RAMP START ---------------------------------------------- */
        /* If it is an expandable board 64 or 128 port allocate memory
	   and  register it with the RAMP services software  */

	  	if((((mpd->mpd_id & 0xF8) == 0x08) || ((mpd->mpd_id & 0xF8) == 0x10))
   	    && (mpd->mpd_alive == DEV_GOOD)) 
		{
                /*icpgaddr_t icpg;*/
		int retv;
                int index = (k * NSSPS) + j;
     		mpc = &meg_chan[icp->icp_minor_start];
		icp->sspcb = &ssp_dev[index];
    		icp->slot_ctrl_blk = &slot_dev[index * MAXICPCHNL];
                if ((retv = ramp_id_functs())) {
                   MESSAGE("ERROR id_functs:", retv, mpd->mpd_mpc);
    		   mpd->mpd_alive = DEV_BAD;
		   break;
                }
		    icp->sspcb->in_use = 0;
		    icp->sspcb->signature = NULL;

                    icpg = (icpgaddr_t)((unsigned long)icp->icp_regs_start+0x2000);
    	      	    if( mpd->mpd_nwin == HA_WIN16 ) {
                        mega_push_win(mpc, 0); 
		       /* register it */
	       	       retv = mpa_srvc_reg_ssp(icp->sspcb, 
		   	   (struct mpa_global_s *)icpg,
                           ramp_map_fn, 
                           mpc, 
                           ramp_unmap_fn);
                        mega_pop_win(43); 
                    }  else
                       retv = mpa_srvc_reg_ssp(icp->sspcb,
                           (struct mpa_global_s *)icpg,
                           NULL,NULL,NULL);
 		    if(retv)  {
               MESSAGE("ERROR reg_ssp:", retv, mpd->mpd_mpc);
    		   	mpd->mpd_alive = DEV_BAD;
			break;
                    }
                    if ((retv = mpa_srvc_pnp_probe_cntrl(icp->sspcb, 0xFF)))
                       MESSAGE("ERROR pnp_probe:", retv, mpd->mpd_mpc);
        }
/* RAMP END ------------------------------------------------ */
			} else { /* SSP2 board */
      				union global_regs_u *icp_glo ;
      				icp_glo = (union global_regs_u *)((unsigned long)icp->icp_regs_start + 0x400);
     				mpc = &meg_chan[icp->icp_minor_start];
				for (jj = 0; jj < ssp_channels; jj++,mpc++) {
					/* same for both FLAT and PAGED */
      					icpi = mpc->mpc_icpi;
      					icpo = mpc->mpc_icpo;
					icpi->ssp.cin_q_ctrl |= 0x80;
					/* allow software attention */
					icpi->ssp2.attn_ena = 0x8000;
					icpi->ssp2.locks = LOCK_B;
					/* 0.1 seconds before decrement */
					icpi->ssp2.tmr_preset = 0x00;
					/* 10 decrements before pulse */
					icpi->ssp2.tmr_count = 0x00;
					/* allow software attention */
					icpo->ssp2.attn_ena = 0x8000;
					icpo->ssp2.locks = LOCK_B;
				}
				/* setup global interval timer for 
				   a 1 second pulse */
				/* enable it */
				icp_glo->ssp2.bus_cntrl |= 0x0D; 

  				mpc = &meg_chan[icp->icp_minor_start];
        		/* fake ldv record */
        		icp->lmx[0].lmx_active = DEV_GOOD;
        		icp->lmx[0].lmx_mpc = mpc;
        		icp->lmx[0].lmx_id = mpd->mpd_id;
        		icp->lmx[1].lmx_active = DEV_BAD;
        		icp->lmx[2].lmx_active = DEV_BAD;
        		icp->lmx[3].lmx_active = DEV_BAD;

				if (mpd->mpd_bus == PCI_BUS) {
					if ((mpd->mpd_id & 0xF8) == 0xA8) {
              					icp->lmx[0].lmx_chan = 2;
              					icp->lmx[0].lmx_speed = 3;
					} else {
              					icp->lmx[0].lmx_chan = 4;
              					icp->lmx[0].lmx_speed = 3;
					}
				} else {
          				if((mpd->mpd_id & 0xF8) == SSP4_ID) {
              					icp->lmx[0].lmx_chan = 4;
#ifdef v334g
				            if (((mpd->mpd_id & 0xFFF8) == 0x2DF8) || /* SSM-8I */
						    		 ((mpd->mpd_id & 0xFFF8) == 0x3FF8) || /* SSM-4I/RJ */
						    		 ((mpd->mpd_id & 0xFFF8) == 0x12F8))  /* SSM-8I+ */
              					icp->lmx[0].lmx_speed = 2; /* Max 460 */
          					else 
#endif
              					icp->lmx[0].lmx_speed = 3; /* Max 920 */
          				} else  {
              					icp->lmx[0].lmx_chan = 2;
              					icp->lmx[0].lmx_speed = 3;
          				}
				}
	
          	icp->lmx[0].lmx_type = 0;
          	icp->lmx[0].lmx_rmt_active = 0;
          	icp->lmx[0].lmx_good_count = 0;
          	icp->lmx[0].lmx_wait = -1;
				/* fake ring state flags */
    				icp->icp_rng_state = RNG_GOOD;
				/* set "ring bad" last pass */
				icp->icp_rng_last = 0x04;
				icp_glo->ssp2.on_line = 0x01;
			}
			/* deselect page and leave board off */
			if(mpd->mpd_nwin == HA_WIN16) {
                          if((mpd->mpd_id & 0xF8) < SSP2_ASIC)
			    outw(0x00, mpd->mpd_pgctrl );
			  else
			    outb(0x00,mpd->mpd_pgctrl);
			}
 		}		/* loop nicps */
		/* deselect page and leave board off */
		if(mpd->mpd_nwin == HA_WIN16) {
                  if((mpd->mpd_id & 0xF8) < SSP2_ASIC)
		    outw(0x00, mpd->mpd_pgctrl );
		  else
		    outb(0x00,mpd->mpd_pgctrl);
		}

		/* PatH Store Multimodem country code and print to log */
		if ((mpd->mpd_id & 0xFF) == 0x98 || (mpd->mpd_id & 0xFF) == 0x9C)
		{
			c_code = *(((char *) mpd->mpd_mem ) + MM_COUNTRY_CODE_REG) 
							& 0x7F;
			mpd->mpd_ccode = c_code;
			printk ("EQUINOX: Multimodem board country code ID is %X\n",
						   	c_code);
		}
	}		/* loop Adapters */

	mpd = meg_dev;

	printk("EQNX: Driver Enabled for %d board(s).\n",nmegaport);



#ifndef MODULE
	eqnx_meminit(kmem_start);
#endif

/*
 *	Allocate a temporary write buffer.
 */
	eqnx_tmpwritebuf = (char *) eqnx_memalloc(XMIT_BUF_SIZE);
	if (eqnx_tmpwritebuf == (char *) NULL)
		printk("EQUINOX: failed to allocate memory (size=%d)\n", 
			XMIT_BUF_SIZE);
	eqnx_txcookbuf = (char *) eqnx_memalloc(XMIT_BUF_SIZE);
	if (eqnx_txcookbuf == (char *) NULL)
		printk("EQUINOX: failed to allocate memory (size=%d)\n", 
			XMIT_BUF_SIZE);

/*	Set up a character driver for ssdiag and sstty.
 */

	if ((diag_num = register_chrdev( 0, "eqnxdiag", &eqnx_fdiag)) <= 0)
		printk("EQUINOX: failed to register eqnx diag device\n");

	/* Initialize the tty_driver structure */
    
	if (register_eqnx_dev() != 0)
		return(-1);

	init_timer(&eqnx_timer);
	eqnx_timer.expires = jiffies + MPTIMEO;
	eqnx_timer.data = 0;
	eqnx_timer.function = &sstpoll;
	add_timer(&eqnx_timer);

#ifndef MODULE
	kmem_start = eqnx_memhalt();
#endif
#ifdef DEBUG
	printk("EQNX: registered EQNX driver\n");
#endif
	return kmem_start;
}

/*****************************************************************************/

/*
 *	Local memory allocation routines. These are used so we can deal with
 *	memory allocation at init time and during run-time in a consistent
 *	way. Everbody just calls the eqnx_memalloc routine to allocate
 *	memory and it will do the right thing. There is no common memory
 *	deallocation code - since this is only done is special cases, all of
 *	which are tightly controlled.
 */

#ifndef MODULE

static void eqnx_meminit(long base)
{
	eqnx_memend = base;
	eqnx_meminited = 1;
}

static long eqnx_memhalt(void)
{
	eqnx_meminited = 0;
	return(eqnx_memend);
}

#endif

static void *eqnx_memalloc(int len)
{
	void	*mem;

	if (eqnx_meminited) {
		mem = (void *) eqnx_memend;
		eqnx_memend += len;
	} else {
		mem = (void *) kmalloc(len, GFP_KERNEL);
	}
	return(mem);
}

/*****************************************************************************/

#ifdef MODULE

/*
 *	Include kernel version number for modules.
 */
int init_module()
{
	printk("init_module()\n");
	return((int) eqnx_init(0));
}

/*****************************************************************************/

void cleanup_module(void)
{
	struct mpdev *mpd;
	struct mpchan *mpc;
	icpgaddr_t icpg;
	icpaddr_t icp;
	unsigned long   flags = 0;
	int i, j, jj, m = 0, n = 0, k, win16, ssp_channels;

	printk("Unloading %s Version %s\n", Version, VERSNUM);
	printk("%s\n", Copyright);

	save_flags(flags);
	cli();
#ifdef DEBUG
printk("cleanup_module: trying to unregister eqnx_driver\n");
printk("cleanup_module:eqnx_refcount = %d\n", eqnx_refcount);
#endif
	for (i =0, j = 0; i < nmegaport; i += 2, j++){
		n = tty_unregister_driver(&eqnx_driver[j]);
#if (LINUX_VERSION_CODE < 132096)
		m = tty_unregister_driver(&eqnx_callout_driver[j]);
#endif /* 2.2 kernels */
	}
	if ( m || n){
	   printk("eqnx: Failed to unregister the tty driver, errno = %d %d\n",
		-m, -n);
		restore_flags(flags);
		return;
	}
	if (timer_pending(&eqnx_timer))
		del_timer(&eqnx_timer);

  	if ( to_eqn_ramp_admin != -1 ) {
		if (timer_pending(&eqnx_ramp_timer))
			del_timer(&eqnx_ramp_timer);
		to_eqn_ramp_admin = -1;
	}

  /* free the normaltermios and callouttermios pointers */
       for( k = 0; k < nmegaport; k++) {    /* for each board */
	 /* map Adapter memory */
	 mpd = &meg_dev[k];
	 for( j = 0; j < (int)mpd->mpd_nicps; j++) {  /* for each icp */
            if((mpd->mpd_id & 0xF8) < SSP2_ASIC) {
               switch((mpd->mpd_id & 0xF8)) {   /* get the channel count */
                   case 0x10: case 0x08: case 0x38: 
                   case 0x18: case 0x20: case 0x28:
                      /* Number per ICP */
                      ssp_channels = 64;
                      break;
                   default:
                      ssp_channels = 0;
                   break;
               }
            }
            else /* SSP4 board */ 
            {
               if (mpd->mpd_bus == PCI_BUS) {
                   switch(mpd->mpd_id & 0xFF) {
                       case 0x90: case 0x94: case 0x88: case 0x8C:
		       case 0xB8: case 0x98: case 0x9C: case 0xFC:
                             ssp_channels = 4;
                             break;
                       case 0xA8:
                             ssp_channels = 2;
                             break;
                       default:
                             ssp_channels = 4;
                             break;
                     }
                 } else {
                    switch((unsigned int)(mpd->mpd_id & 0xFF00) >> 8){
                        case 0x24:
                            ssp_channels = 2;
                            break;
                        case 0x09:
                            ssp_channels = 4;
                            break;
                        case 0x1B:
                            ssp_channels = 4;
                            break;
                        case 0x2E: /* SST-16I */
                        case 0x2D: /* SSM-8 */
                        case 0x12: /* SSM-8I+ (or SST-8I/RJ) */
                        case 0x3F: /* SSM-4I+ (or SST-4I/RJ) */
                            ssp_channels = 4;
                            break;
                        case 0x36: /* MPM-4 */
                            ssp_channels = 4;
                            break;
                         default:
                            ssp_channels = 4;
                            break;
                    } 
		}
            }
				restore_flags(flags); /* MONTY */
            /* go free the per channel termios structures */
            for ( i = 0; i <  ssp_channels; i++ ) {
               /* setup mpc virtual addresses for cpu access 
                  of icp */
               jj = mpd->mpd_minor + i + 
               (j * mpd->mpd_sspchan);
               mpc = &meg_chan[jj];

               if( mpc->normaltermios != NULL ) 
                  vfree( (void *)mpc->normaltermios ); 
#if (LINUX_VERSION_CODE < 132096)
               if( mpc->callouttermios != NULL ) 
                  vfree( (void *)mpc->callouttermios ); 
#endif
            }
				save_flags(flags); /* MONTY */
				cli(); /* MONTY */
         }              /* for nicps */
       }                /* for nmegaports */
  /* end free the normaltermios and callouttermios pointers */



	restore_flags(flags); /* MONTY */


	if (eqnx_ttys != (struct tty_struct **) NULL)
		vfree((void *)eqnx_ttys);
	if (eqnx_termios != (struct termios **) NULL)
		vfree((void *) eqnx_termios);
	if (eqnx_termioslocked != (struct termios **) NULL)
		vfree((void *)eqnx_termioslocked);
	if (eqnx_driver != (struct tty_driver *) NULL)
		vfree((void *)eqnx_driver);
#if (LINUX_VERSION_CODE < 132096)
	if (eqnx_callout_driver != (struct tty_driver *) NULL)
		vfree((void *)eqnx_callout_driver);
#endif
#ifdef DEBUG
printk("cleanup_module: trying to unregister eqnx_diag\n");
#endif
	if ((i = unregister_chrdev(diag_num, "eqnxdiag")))
		printk("EQUINOX: failed to unregister diag device, errno=%d\n",
			-i);
#ifdef DEBUG
printk("cleanup_module: trying to free meg_chan\n");
#endif
	if (meg_chan != (struct mpchan *) NULL)
		vfree((void * )meg_chan);
	if (eqnPCIcsh != (char *) NULL)
		vfree((void * )eqnPCIcsh);
	if (eqnx_tmpwritebuf != (char *) NULL)
		kfree(eqnx_tmpwritebuf);
	if (eqnx_txcookbuf != (char *) NULL)
		kfree(eqnx_txcookbuf);
	save_flags(flags); /* MONTY */
	cli(); /* MONTY */
	for(k=0; k < nmegaport; k++){
		mpd = &meg_dev[k];
		if (mpd->mpd_alive == 0)
			continue;
		else
			mpd->mpd_alive = 0;
		if ((mpd->mpd_id & 0xF8) < SSP2_ASIC) {
			/* SSP64 */
			win16 = mpd->mpd_nwin;
			for(i = 0; i < (int)mpd->mpd_nicps;i++){
				if(win16)
					mega_push_winicp(mpd, i);
				icp = &mpd->icp[i];
				icpg = (icpgaddr_t) ((unsigned long)icp->icp_regs_start
					+0x2000);
         			/* turn off ring clock, PRAM and DMA */
         			printk("turn off ring clock, PRAM and DMA\n");
				Gicp_initiate &= ~0x1e;
         			/* reset icp to known state */
				i_gicp_attn = 0;
				Gicp_initiate = 0;
				icpg->ssp.gicp_watchdog = 0;
				if(win16)
					mega_pop_win(9);
			}
		}
		restore_flags(flags);
		if (mpd->mpd_bus == ISA_BUS)
    {
			release_region(portn[mpd->mpd_io],8);
    }
		else
    {
/* Pat */
#if (EQNX_VERSION_CODE < 131328)
      if (mpd->mpd_mem > 0x100000)
#endif
/* Pat */      
			    iounmap((void *) mpd->mpd_mem);
    }

      restore_flags(flags);

#if	(LINUX_VERSION_CODE >= 132096)
#ifdef	ISA_ENAB
#ifdef	CONFIG_ISAPNP
	/*
	** ISA plug-and-play in linux kernels 2.4+
	*/
	for (i=0; i<pnp_found; i++) {
		pnp_found_devs[i]->deactivate(pnp_found_devs[i]);
		pnp_found_devs[i] = NULL;
	}

	pnp_found = 0;
#endif	/* CONFIG_ISAPNP */
#endif
#endif

	}
}
#endif
/*****************************************************************************/
int test_8bit_hole(struct mpdev *mpd, int k, paddr_t addr )
{
int ret=true;
int ram_index;
ushort_t *ramw;
int ii, jj,j;
paddr_t mpd_mem;
uchar_t cchnl;
ushort_t no_cache;
icpgaddr_t icpg;
icpaddr_t icp;
volatile unsigned char *chnl_ptr;
int testlen;
int ssp_channels = 0;
uchar_t *bus_ctrl_p;

#ifdef UNIX
           cmn_err(CE_CONT, "EQNX: test_8bit port (%x) addr (%x)\n",
                addr,portn[k]);

#else
           printk("EQNX: test_8bit port (%x) addr (%x)\n",
                (unsigned int) addr,(unsigned int) portn[k]);

#endif
  if ((mpd->mpd_id & 0xF8) < SSP2_ASIC) { /* SSP64 board */
    /* Set the address, 16-bit mode, MAX2, and flat off */ 
    outb((addr >> 14) & 0xff,portn[k]);
    outb(((addr >> 22) & 3)| 0x20, portn[k] + 1);
    /* page register - deselect ram */  
    outb(0, portn[k] + 2);
    outb(0, portn[k] + 3);
    /* disable interrupts */ 
    outb(0, portn[k] + 6);
  }
  else { /* SSP4 board */
      unsigned char b;
     
      /* reg 8 (page reg) */
      if(mpd->mpd_nwin == HA_FLAT)
         b = 0x20;
      else
         b = 0;

      outb( b, portn[mpd->mpd_io] + 0x08);

      b = ((addr & 0x0000c000) >> 8);
      /* no ints */
      outb(b, portn[mpd->mpd_io] + 0x09);
      b = ((addr & 0xFFFF0000) >> 16);
      outb(b, portn[mpd->mpd_io] + 0x0a);
      /* reg b*/
      b=0;
      outb(b, portn[mpd->mpd_io] + 0x0b);
  }
#ifdef LINUX
	mpd_mem = addr;
#else
#ifdef UNIX
                mpd_mem = (paddr_t) sptalloc( btoc(0x4000), (PG_P), btoc((caddr_t)addr),1);
#else
                mpd_mem = (paddr_t)mapphys((caddr_t)addr, 0x4000);
#endif
#endif /* Not LINUX */
        if(!mpd_mem)
        {
#ifdef UNIX
	   cmn_err(CE_CONT, "EQNX: Memory mapping failed for board %d\n", k +1);
#else
	   printk("EQNX: Memory mapping failed for board %d\n", k +1);
#endif
           ret= false;
           return(ret);
        }

        /*  1 icps */
         j=0;

	/* select correct page if necessary */
        if ( mpd->mpd_nwin == HA_WIN16 ){
          if((mpd->mpd_id & 0xF8) < SSP2_ASIC)
            outw( 0x0100 | j, portn[k] + 2 );
          else
            outb( 1 << j,portn[k] + 8); 
        }

	/* reset icp to known state */
        icp = &mpd->icp[j];
        if ((mpd->mpd_id & 0xF8) < SSP2_ASIC) {  /* SSP64 */
          icpg = (icpgaddr_t)((unsigned long)mpd_mem+0x2000);
          i_gicp_attn = 0;
          Gicp_initiate = 0;
	  /* Adjust the number of icps on the board  */
          icpg->ssp.gicp_bus_ctrl = 0x09;
	  bus_ctrl_p = &(icpg->ssp.gicp_bus_ctrl);
        }
        else { /* SSP2/4 */
          /* VIRTUAL- ptr to global regs */
          union global_regs_u *icp_glo =
            (union global_regs_u *)((unsigned long)mpd_mem + 0x400);
          /* VIRTUAL ptr to output regs */
          union output_regs_u *icp_cout =
            (union output_regs_u *)((unsigned long)mpd_mem + 0x200);
          /* VIRTUAL ptr to input regs */
          union input_regs_u *icp_cin =
            (union input_regs_u *)mpd_mem;

	  icp_glo->ssp2.on_line = 0;   /* disable SSP2 */
	  icp_cin->ssp2.locks = 0xFF;/* input locks SSP2 */
          /* output locks SSP2 */
	  icp_cout->ssp2.locks = 0x77;
          /* Set Bus Control = 16 bits */
          icp_glo->ssp2.bus_cntrl = 0x01;
	  bus_ctrl_p = &(icp_glo->ssp2.bus_cntrl);
          if (mpd->mpd_bus == PCI_BUS) {
             switch(mpd->mpd_id & 0xFF) {
		case 0xC8: /* SST-16P no panel */
		case 0xC4: /* SST-16P with 16RJ */
		case 0xC0: /* SST-16P with 16DB */
		case 0xD0: /* SST-16P with 16DB9 */
                	ssp_channels = 16;
                	break;
                case 0x90:
                case 0x94:	/* SSP4-8P/RJ */
                case 0xF4:	/* SSP4-8P,PWR */
                case 0x98:	/* SSP8-MM8P */
                   ssp_channels = 8;
                   break;
                case 0x88:
                case 0xB8:	/* SSP4-4P/LP */
                case 0x8C:	/* SSP4-4P/RJ */
                case 0xEC:	/* SSP4-4P,PWR */
                case 0x9C:	/* SSP4-MM4P */
                case 0xFC:	/* SSP4-4P/ULP */
                   ssp_channels = 4;
                   break;
                case 0xA8:
                   ssp_channels = 2;
                   break;
                default:
                   ssp_channels = 0;
                   break;
              }
           } else {
              switch((unsigned int)(mpd->mpd_id & 0xFF00) >> 8){
                 case 0x24:
                    ssp_channels = 2;
                    break;
                 case 0x09:
                    ssp_channels = 4;
                    break;
                 case 0x1B:
                    ssp_channels = 4;
                    break;
		case 0x2D: /* SSM-8 */
		case 0x08: /* SST-4I,PWR */
		case 0x18: /* SST-8I,PWR */
		case 0x12: /* SST-8I+ or SST-8I/RJ */
		case 0x3F: /* SST-4I+ or SST-4I/RJ */
		case 0x36: /* MPM-4 */
		    ssp_channels = 4;
		    break;
                 default:
                    ssp_channels = 4;
                    break;
              } 
           }

          for(ii = 0; ii < ssp_channels; ii++) /* set all locks */
          {
              (icp_cin + ii)->ssp2.locks = 0xFF;
              (icp_cout + ii)->ssp2.locks = 0x77;
              (icp_cout + ii)->ssp2.cntrl_sigs = 0x0F;
          }
        }
	

  /* test pram  - 16-bit test */
  ramw = (ushort_t *) &(mpd->icp[j].icp_regs_start);
  if ((mpd->mpd_id & 0xF8) < SSP2_ASIC) { /* SSP64 board */
    testlen = HWREGSLEN;
    ram_index = mem_test_pram(mpd, ramw, testlen, j);
  }
  else { /* SSP2/4 board */
    if ( mpd->mpd_nwin == HA_FLAT )
       outb( mpd->mpd_pgctrl, 0x20 );  /*SSP2 enables all for HA_FLAT */

    if (mpd->mpd_bus == PCI_BUS)
      testlen = ((mpd->mpd_id & 0xFF) == 0xA8) ? 0x100 : 0x200;
    else
      testlen = ((mpd->mpd_id & 0xFF) == SSP2_ID) ? 0x100 : 0x200;
    ram_index = mem_test_pram(mpd, ramw, testlen, j);
    if (!ram_index) {
      ramw = (ushort_t *)((unsigned long)(mpd->icp[j].icp_regs_start) + 0x200);
      ram_index = mem_test_pram(mpd, ramw, testlen, j);
    }
  }
  if ( ram_index )
  {
#ifdef UNIX
       cmn_err( CE_CONT,"EQNX: PRAM memory test failure %d for board with I/O address 0x%x.\n",ram_index,portn[k]);
#else
      printk("EQNX: PRAM memory test failure %d for board with I/O address 0x%x.\n",ram_index,portn[k]);
#endif
     ret= false;   /*need to de-map */
  }
  

  /* zero pram */
  ramw = (ushort_t *) mpd_mem;
/*
printk("clear pram icp=%d  ramw=%x\n",icp,ramw);
*/
  if ((mpd->mpd_id & 0xF8) < SSP2_ASIC) { /* SSP64 board */
    for ( ii = 0; ii < HWREGSLEN; ii += 2, ramw++ )
    {
      jj = ii & 0x7f;
      if ( ii >= HWREGSLEN/2  /* upper 8k of range includes global regs */
           && ((jj >= 0x18 && jj < 0x20)
                || (jj >= 0x38 && jj < 0x40)
                || (jj >= 0x58 && jj < 0x60)
                || (jj >= 0x78 && jj < 0x80)) )
        continue;
      *ramw = 0;
    }
  }
  else { /* SSP2/4 board */
    for ( ii = 0; ii < 0x400; ii += 2, ramw++ )
    {
      jj = ii & 0x7f;
      if (jj == 0x02 || 
          jj == 0x64 ||
          jj == 0x72  )  /* protect sensitive regs  */
        continue; 
      *ramw = 0;
    }
  }
  /* verify that pram is not cached */

    if ((mpd->mpd_id & 0xF8) < SSP2_ASIC) { /* SSP64 board */
        icpg = (icpgaddr_t)((unsigned long)mpd_mem + 0x2000);
       /* verify that pram is not cached */

      cchnl = Gicp_chnl;
      chnl_ptr = &(Gicp_chnl);
    }
    else { /* SSP4 board */
      icpaddr_t icp;
      union global_regs_u *icp_glo ;
      icp = &mpd->icp[j];
      icp_glo = (union global_regs_u *)((unsigned long)icp->icp_regs_start + 0x400);
      cchnl = icp_glo->ssp2.chan_ctr;
      chnl_ptr = &(icp_glo->ssp2.chan_ctr);
    }
      no_cache = false;
      for ( ii = 0; ii < 0x10000; ii++ )
        if ( *chnl_ptr != cchnl )
        {
          no_cache = true;
          break;
        }
  if ( !no_cache )
  {
     /* SSM, found testing SSM wrong error messages */
#ifdef UNIX
		 cmn_err(CE_CONT,
          "EQNX: PRAM memory appears to be cached %d for board with I/O address 0x%x.\n",
          ii,portn[k]);
#else
          printk("EQNX: PRAM memory appears to be cached %d for with I/O address 0x%x.\n",
          ii,portn[k]);
#endif
         ret=false;
  }
    *bus_ctrl_p = 0x0;           /* turn off */

    if ((mpd->mpd_id & 0xF8) < SSP2_ASIC) { /* SSP64 board */
    outw( 0x0000, portn[k] + 2) ;              /*Disable Controller's addressing                       */
    outw( 0x0000, portn[k]) ;
    }
    else { /* SSP4 board */
      unsigned char b = inb(portn[k] + 8);
      outb(b & 0xF0, portn[k] + 8);
    }
#ifndef LINUX
#ifdef UNIX
    sptfree( (caddr_t) mpd_mem, btoc(0x4000),   0 );
#else
        unmapphys( (caddr_t) mpd_mem, 0x4000 );
#endif
#endif /* Not Linux */

  return(ret);
}

#ifdef	ISA_ENAB
/******************************************************************************

	ISA_ICP_COUNT

	Return the number of ICP's present on the board described by
	registers "regs".

	As additional board types are added, this function must be updated.

******************************************************************************/

static int isa_icp_count( ushort id )
{
	if ((id & 0xf8) == 0xF8) { 
		switch ((id & 0xFF00) >> 8) {
			case 0x17: /* SST-16I no panel  */
			case 0x0F: /* SST-16I with 16RJ */
			case 0x07: /* SST-16I with 16DB */
				return 4;
			case 0x2D: /* 0xF82D == SSP4 SSM-8I  */
			case 0x12: /* 0xF812 == SSP4 SST-8I+ */
			case 0x18: /* SST-8I,PWR */
			case 0x1B: /* 0xF81B == SSP4 SST-8I  */
				return 2;
			default:
				return 1;  /*  All other SSP4 ISA boards have a single SSP */
		}
	}
  if((id & 0xf8) == 0x10) /* 128 port board */
    return 2;
  else
    return 1;  /*  All other ISA boards have a single SSP */
}  /* isa_icp_count */
#endif	/* ISA_ENAB */

/****************************************************************************/
uint_t brd_nchan_cfg( int bid )
{
	switch(bid & 0xFC){
	  /* SSP64 ISA EISA PCI MCA Cases */
	  case 0x10:		/* SS-128 */
		return 128;	
	  case 0x08: 		/* SS-64 */
		return 64;
	  case 0x18:		/* SS-8 */
		return 8;
	  case 0x20:		/* SS-4 */
		return 4;
	  case 0x28:		/* SS-4 */
		return 2;
#ifdef SSM
          case 0x30:            /* SSM-12 */
                return 12;
          case 0x38:            /* SSM-24 */
                return 32;
#endif

	  /* SSP4 ISA/EISA Cases */
	  case 0xF0:
	  case 0xF8:
	    switch((bid & 0xFF00)>>8) {
	      case 0x00:
                return 1;
	      case 0x10:
	      case 0x24:
	        return 2;
/* OOPS! WHY!
	      case 0x12:
*/
	      case 0x08: /* SST-4I,PWR */
	      case 0x36: /* MPM-4 */
	      case 0x3F: /* SSP4 SST-4I+ (or SST-4I/RJ) */
	        return 4;
/* Old SSM-8I ID
	      case 0x19:
*/		
	      case 0x18: /* SST-8I,PWR */
	      case 0x2D: /* SSP4 based SSM-8I */
	      case 0x12: /* SSP4 SST-8I+ (or SST-8I/RJ) */
	      case 0x1A:
	      case 0x1B:
	      case 0x20:
	        return 8;
	      case 0x17: /* SST-16I no panel */
	      case 0x0F: /* SST-16I with 16RJ */
	      case 0x07: /* SST-16I with 16DB */
		return 16;
	    }
	  /* SSP4 PCI Cases */
	case 0xC8: /* SST-16P no panel */
	case 0xC4: /* SST-16P with 16RJ */
	case 0xC0: /* SST-16P with 16DB */
	case 0xD0: /* SST-16P with 16DB9 */
		return 16;
	  case 0x90:     /* SST-8P(F) */
	  case 0x94:	/* SST-8P (RJ) */
	  case 0xF4:	/* SST-8P,PWR */
	  case 0x98:	/* SST-MM8P */
		return(8);
	  case 0x88:     /* SST-4P */
	  case 0xB8:	/* SST-4P/LP  */
	  case 0x8C:	/* SST-4P (RJ) */
	  case 0xEC:	/* SST-4P ,PWR */
	  case 0x9C:	/* SST-MM4P */
	  case 0xFC:	/* SST-4P/ULP  */
	    return 4;
	  case 0xA8:     /* SST-2P */
	    return 2;
	  default:
		return 0;
	}
}

/******************************************************************************

	EISA_MEM_CFG

	Place pointers to buffer, tag and cmd memory in the icp records
	pointed to be "mpd".  Also place the corresponding lengths in 
	appropriate fields.

	As additional board types are added, this function must be updated.

******************************************************************************/

void brd_mem_cfg( struct mpdev *mpd )
{
int i;

  for ( i = 0; i < (int)mpd->mpd_nicps; i++ )
  {
    if(mpd->mpd_nwin == HA_FLAT) {
      if ((mpd->mpd_id & 0xF8) < SSP2_ASIC) { /* SSP64 board */
	/* eisa memory config icp */
	mpd->icp[i].icp_regs_start = (mpaddr_t)((unsigned long)mpd->mpd_mem + (i * 0x4000));
	/* Special case PCI 64 port boards */
	if ((mpd->mpd_bus == PCI_BUS) && 
		((mpd->mpd_id & 0xF8) == 0x08))
	mpd->icp[i].icp_dram_start = ((unsigned long)mpd->mpd_mem + 
		mpd->mpd_memsz/2);
	else
	mpd->icp[i].icp_dram_start = ((unsigned long)mpd->mpd_mem + 
				mpd->mpd_memsz + (i * mpd->mpd_memsz / 2));
        if((mpd->mpd_id & 0xF8) == 0x18 || (mpd->mpd_id & 0xF8) == 0x20 ||
           (mpd->mpd_id & 0xF8) == 0x30 || (mpd->mpd_id & 0xF8) == 0x38 ) {

            if ((mpd->mpd_id & 0xF8) == 0x30 )
              mpd->mpd_hwq = &sst_hwq[2]; /* 1k */
            else
               if ((mpd->mpd_id & 0xF8) == 0x38 )
                 mpd->mpd_hwq = &sst_hwq[3]; /* .5 k */
               else
                  mpd->mpd_hwq = &sst_hwq[1]; /* 2k */
           mpd->icp[i].icp_tags_start = ((unsigned long)mpd->mpd_mem + 0x4000);
	   mpd->icp[i].icp_cmds_start = ((unsigned long)mpd->mpd_mem + 0x4000);
	} else {
	   mpd->icp[i].icp_tags_start = ((unsigned long)mpd->mpd_mem + 0x40000 + (i * 0x20000));
	   mpd->icp[i].icp_cmds_start = ((unsigned long)mpd->mpd_mem + 0x40000 + (i * 0x20000));
	   mpd->mpd_hwq = &sst_hwq[0]; /* 4k for SS64 & SS128 */
	} /* mpd_id */
      } /* SSP64 */
      else { /* SSP4 */
	mpd->icp[i].icp_regs_start = (mpaddr_t)((unsigned long)mpd->mpd_mem + 
          (i * sizeof(struct ssp2_addr_space_s)));
	mpd->icp[i].icp_dram_start = ((unsigned long)mpd->mpd_mem + 
	  (i * sizeof(struct ssp2_addr_space_s)) + 0x1000);
        mpd->icp[i].icp_tags_start = ((unsigned long)mpd->mpd_mem + 0x3000 + 
          (i * sizeof(struct ssp2_addr_space_s)));
        /* Note: cmds doesn't exist on SSP2, so point to tags */
        mpd->icp[i].icp_cmds_start = mpd->icp[i].icp_tags_start;
        mpd->mpd_hwq = &sst_hwq[2];
      } /* SSP4 */
    } /* HA_FLAT */
    else {  /* HA_WIN */
      if ((mpd->mpd_id & 0xF8) < SSP2_ASIC) { /* SSP64 board */
	mpd->icp[i].icp_regs_start = (mpaddr_t)((unsigned long)mpd->mpd_mem);
	mpd->icp[i].icp_dram_start = (unsigned long)mpd->mpd_mem;
	mpd->icp[i].icp_tags_start = (unsigned long)mpd->mpd_mem;
	mpd->icp[i].icp_cmds_start = (unsigned long)mpd->mpd_mem;
#ifdef SSM
         if ((mpd->mpd_id & 0xF8) == 0x30 )
           mpd->mpd_hwq = &sst_hwq[2]; /* 1k */
         else
            if ((mpd->mpd_id & 0xF8) == 0x38)
              mpd->mpd_hwq = &sst_hwq[3]; /* .5 k */
          else
#endif
            if((mpd->mpd_id & 0xF8) == 0x18 || (mpd->mpd_id & 0xF8) == 0x20)
                mpd->mpd_hwq = &sst_hwq[1]; /* 2k */
	else
		mpd->mpd_hwq = &sst_hwq[0]; /* 4k for SS64 & SS128 */
      } /* SSP64 board */
      else { /* SSP4 board */
	mpd->icp[i].icp_regs_start = (mpaddr_t)((unsigned long)mpd->mpd_mem);
	mpd->icp[i].icp_dram_start = (unsigned long)mpd->mpd_mem + 0x1000;
	mpd->icp[i].icp_tags_start = (unsigned long)mpd->mpd_mem + 0x3000;
	mpd->icp[i].icp_cmds_start = (unsigned long)mpd->mpd_mem + 0x3000;
	mpd->mpd_hwq = &sst_hwq[2]; /* 1k */
      }
    } /* HA_WIN */	
  } /* for */
}  /* eisa_mem_cfg */
/****************************************************************************/
int mem_test_pram( struct mpdev *mpd, ushort_t *ramw, int testlen, int icp )
{
  int ram_ok = 0;
  int ii, jj;

  for ( ii = 0; ii < testlen; ii += 2, ramw++ )  /* include input & output */
  {
    /* select correct page if necessary */
    if(mpd->mpd_nwin == HA_WIN16) {
      if((mpd->mpd_id & 0xF8) < SSP2_ASIC)
         outw(0x100 | icp, mpd->mpd_pgctrl);
      else
        outb( 1 << icp, mpd->mpd_pgctrl);
    }
    if ((mpd->mpd_id & 0xF8) < SSP2_ASIC) {
      jj = ii & 0x7f;
      if ( ii >= HWREGSLEN/2  /* upper 8k of range includes global regs */
           && ((jj >= 0x18 && jj < 0x20)
                || (jj >= 0x38 && jj < 0x40)
                || (jj >= 0x58 && jj < 0x60)
                || (jj >= 0x78 && jj < 0x80) ))
        continue;
    }
    *ramw = 0x55aa;
    ram_ok = true;
    if ( *ramw != 0x55aa )
    {
      ram_ok = false;
      break;
    }
    *ramw = 0xaa55;
    if ( *ramw != 0xaa55 )
    {
      ram_ok = false;
      break;
    }
  }
  if (ram_ok)
    ii = 0;
  return(ii);
} 

int mem_test_dram( struct mpdev *mpd, ushort_t *ramw, int testlen, int icp )
{
  int ram_ok;
  uchar_t ram_pg;
  int ii;

  ram_ok = true;
  ram_pg = 0;
  for ( ii = 0; ii < testlen; ii += 2, ramw++ )
  {
    /* select correct page if necessary */
    if(mpd->mpd_nwin == HA_WIN16 && !(ii % 0x4000)) {
      if((mpd->mpd_id & 0xF8) < SSP2_ASIC)
         outw(0x300 | ram_pg++, mpd->mpd_pgctrl);
      else
        outb( 1 << icp, mpd->mpd_pgctrl);

       if(ii)
    	 ramw = (ushort_t *) ((uchar_t *)ramw - 0x4000);
      }
    *ramw = 0x55aa;
    if ( *ramw != 0x55aa )
    {
      ram_ok = false;
      break;
    }
    *ramw = 0xaa55;
    if ( *ramw != 0xaa55 )
    {
      ram_ok = false;
      break;
    }
  }
  if (ram_ok)
    ii = 0;
  return(ii);
}

int mem_test_tag(struct mpdev *mpd, uchar_t *ramb, int testlen, int icp)
{
  int ram_ok;
  uchar_t ram_pg;
  volatile uchar_t *ramb2;
  int ii;

  ramb2 = ramb + 1;
  ram_ok = true;
  ram_pg = 0;
  for ( ii = 0; ii < testlen; ii += 2, ramb += 2, ramb2 += 2 )
  {
    /* select correct page if necessary */
    if(mpd->mpd_nwin == HA_WIN16 && !(ii % 0x4000)) {
      if((mpd->mpd_id & 0xF8) < SSP2_ASIC)
         outw(0x300 | ram_pg++, mpd->mpd_pgctrl);
      else
        outb( 1 << icp,mpd->mpd_pgctrl ); 

        if(ii) {
	  ramb -= 0x4000;
	  ramb2 -= 0x4000;
	}
      }
    *ramb = 0x55;
    *ramb2 = 0xaa;
    if ( *ramb != 0x55
	   || *ramb2 != 0xaa 
	   || !ram_ok )
    {
      ram_ok = false;
      break;
    }
  }
  if (ram_ok)
    ii = 0;
  return(ii);
}

static int mem_zero( struct mpdev *mpd, unsigned short *ramw, 
		int testlen, int icp)
{
  int ii, jj;

  for ( ii = 0; ii < testlen; ii += 2, ramw++ )
  {
    /* select correct page if necessary */
    if ( mpd->mpd_nwin == HA_WIN16 ){
      if((mpd->mpd_id & 0xF8) < SSP2_ASIC)
        outw( 0x0100 | icp,mpd->mpd_pgctrl );
      else
        outb( 1 << icp, mpd->mpd_pgctrl ); 
    }
    if ((mpd->mpd_id & 0xF8) < SSP2_ASIC) { /* SSP64 board */
      jj = ii & 0x7f;
      if ( ii >= HWREGSLEN/2  /* upper 8k of range includes global regs */
           && ((jj >= 0x18 && jj < 0x20)
                || (jj >= 0x38 && jj < 0x40)
                || (jj >= 0x58 && jj < 0x60)
                || (jj >= 0x78 && jj < 0x80) ))
        continue;
    } else {
           jj = ii & 0x7f;
           if ( (jj == 0x02) || 
                (jj == 0x64) || 
                (jj == 0x72 ))  /* protect sensitive regs  */
             continue; 
    }
    *ramw = 0;
  }
  return(0);
}

static int mem_test(struct mpdev *mpd, int icp)
{
int ram_index;
uchar_t *ramb;
ushort_t *ramw;
int err = 0;
int testlen;

  /* test pram  - 16-bit test */
  ramw = (ushort_t *) (mpd->icp[icp].icp_regs_start);
  if ((mpd->mpd_id & 0xF8) < SSP2_ASIC) { /* SSP64 board */
    testlen = HWREGSLEN;
    ram_index = mem_test_pram(mpd, ramw, testlen, icp);
  }
  else { /* SSP2/4 board */
    if ( mpd->mpd_nwin == HA_FLAT )
       outb( 0x20, mpd->mpd_pgctrl );  /*SSP2 enables all for HA_FLAT */

    if (mpd->mpd_bus == PCI_BUS)
      testlen = ((mpd->mpd_id & 0xFF) == 0xA8) ? 0x100 : 0x200;
    else
      testlen = ((mpd->mpd_id & 0xF8) == SSP2_ID) ? 0x100 : 0x200;
    ram_index = mem_test_pram(mpd, ramw, testlen, icp);
    if (!ram_index) {
      ramw = (ushort_t *)((unsigned char *)(mpd->icp[icp].icp_regs_start) + 0x200);
      ram_index = mem_test_pram(mpd, ramw, testlen, icp);
    }
  }
  if ( ram_index )
  {
    if (mpd->mpd_bus == ISA_BUS)
#ifdef UNIX
       cmn_err( CE_CONT,"PRAM memory test failure %d for board with I/O address 0x%x.\n",ram_index,portn[mpd->mpd_io]);
#else
       printk("EQNX: PRAM memory test failure %d for board with I/O address 0x%x.\n",(unsigned int) ram_index,(unsigned int) portn[mpd->mpd_io]);
#endif
    else
#ifdef UNIX
    cmn_err( CE_CONT,"EQNX: PRAM memory test failure %d for board in Slot %d.\n",ram_index,mpd->mpd_slot);
#else
    printk("EQNX: PRAM memory test failure %d for board in Slot %d.\n",ram_index,mpd->mpd_slot);
#endif
    err = 1;
  }
  
  /* test dram - word test */
  if ((mpd->mpd_id & 0xF8) < SSP2_ASIC) { /* SSP64 board */
    ramw = (ushort_t *) (mpd->icp[0].icp_dram_start);  /* dram */
    testlen = mpd->mpd_memsz;
    ram_index = mem_test_dram(mpd, ramw, testlen, icp);
  }
  else { /* SSP2/4 board */
    /* SSP2/4 enable each icp to test dram */
         
    if ( mpd->mpd_nwin == HA_FLAT )
       outb( 0x20, mpd->mpd_pgctrl );  /*SSP2 enables all for HA_FLAT */

    if (mpd->mpd_bus == PCI_BUS)
      testlen = ((mpd->mpd_id & 0xFF) == 0xA8) ? 0x800 : 0x1000;/*2 or4*/
    else
      testlen = ((mpd->mpd_id & 0xf8) == SSP2_ID) ? 0x800 : 0x1000;/*2 or4*/

    /* test input buff */
    ramw = (ushort_t *) (mpd->icp[icp].icp_dram_start);
    ram_index = mem_test_dram( mpd, ramw, testlen, icp); 

    if(!ram_index){
      /* test output buff */
      ramw = (ushort_t *) ((unsigned long)mpd->icp[icp].icp_dram_start + 0x1000);
      ram_index = mem_test_dram( mpd, ramw, testlen, icp); 
    }
  }
  if ( ram_index )
  {
    if (mpd->mpd_bus == ISA_BUS)
#ifdef UNIX
       cmn_err( CE_CONT, "EQNX: DRAM word memory test failure %d for board with I/O address 0x%x.\n",ram_index,portn[mpd->mpd_io]);
#else
       printk("EQNX: DRAM word memory test failure %d for board with I/O address 0x%x.\n",ram_index,portn[mpd->mpd_io]);
#endif
    else
#ifdef UNIX
       cmn_err( CE_CONT, "EQNX: DRAM word memory test failure %d for board in Slot %d.\n",ram_index,mpd->mpd_slot);
#else
       printk("EQNX: DRAM word memory test failure %d for board in Slot %d.\n",ram_index,mpd->mpd_slot);
#endif
    err |= 2;
  }

  /* test tag dram - requires BYTE accesses! */
  /* Same values OK for SSP64 and SSP4 */
  ramb = (uchar_t *) (mpd->icp[0].icp_tags_start);  /* tags ram */
  if ((mpd->mpd_id & 0xF8) < SSP2_ASIC)  /* SSP64 board */
    testlen = mpd->mpd_memsz/4;
  else {
    if (mpd->mpd_bus == PCI_BUS)
      testlen = ((mpd->mpd_id & 0xFF) == 0xA8) ? 0x200 : 0x400;/*2 or4*/
    else
      testlen = ((mpd->mpd_id & 0xf8) == SSP2_ID) ? 0x200 : 0x400;/*2 or4*/
  }
  ram_index = mem_test_tag(mpd, ramb, testlen, icp);
  if ( ram_index )
  {
    if (mpd->mpd_bus == ISA_BUS)
#ifdef UNIX
       cmn_err( CE_CONT, "EQNX: DRAM tags memory test failure %d for board with I/O address 0x%x.\n",ram_index,portn[mpd->mpd_io]);
#else
       printk("EQNX: DRAM tags memory test failure %d for board with I/O address 0x%x.\n",ram_index,portn[mpd->mpd_io]);
#endif
    else
#ifdef UNIX
       cmn_err( CE_CONT, "EQNX: DRAM tags memory test failure %d for board in Slot %d.\n",ram_index,mpd->mpd_slot);
#else
       printk("EQNX: DRAM tags memory test failure %d for board in Slot %d.\n",ram_index,mpd->mpd_slot);
#endif
    err |= 4;
  }
  /* zero pram */
  ramw = (ushort_t *) (mpd->icp[icp].icp_regs_start);
/*
printk("clear pram icp=%d  ramw=%x\n",icp,(unsigned int) ramw);
*/
  if ((mpd->mpd_id & 0xF8) < SSP2_ASIC)  /* SSP64 board */
    testlen = HWREGSLEN;
  else  /* SSP4 board */
    testlen = 0x400;
  mem_zero(mpd, ramw, testlen, icp);
  return(err);
}

/******************************************************************************

	EISA_BRDTYP_OK

	Return true for supported boards, else false.

******************************************************************************/

int eisa_brdtyp_ok( ushort_t bbbbbrrr, ushort_t cfg )
{
  bbbbbrrr &= 0xf8;
  if(bbbbbrrr < SSP2_ID)
  {
     if ( bbbbbrrr == 0x08
	  || bbbbbrrr == 0x10
	  || bbbbbrrr == 0x18
	  || bbbbbrrr == 0x20
	  || bbbbbrrr == 0x30
	  || bbbbbrrr == 0x38 )
       return true;
   }
   else
   {
      if( bbbbbrrr == SSP2_ID || bbbbbrrr == SSP4_ID ) 
      {
         if(cfg == 0x1b   /* SSP2/4 valid configurations */
            || cfg == 0x09
            || cfg == 0x24 )
          return true;
       }
   }
   return false;
}  /* eisa_brdtyp_ok */

#ifdef	ISA_ENAB
/******************************************************************************

	EISA_BRD_FOUND

	Examine EISA "slot" for an Equinox board.  If not found return false.
	If found, fill "regs" from the eisa registers.  Test that the board is
	enabled.  If so, return true, else -1.

	As additional board types are added, this function MAY have to be 
	updated.

******************************************************************************/

static int eisa_brd_found( int slot, ushort_t *regs )
{
ushort_t port, bid, port_org;
volatile ushort_t *regs_org;
volatile ushort_t *regs_enb;

  regs_org = regs;
  regs_enb = regs + 2;
  port = slot * 0x1000 + 0xC80;
  port_org = port;
  *regs = inw( port );        /* 0x?000 */
  if ( *regs != EISA_EQX )
    return false;

  regs++;
  port += 2;
  *regs++ = inw( port );    /* 0x?002 */
  port += 2;
  *regs = inw( port );    /* 0x?004 */

  port += 2;
  regs++;
  bid = regs_org[2] & (SSP4_MASK << 8);
  

  if((bid & 0xF800) < (SSP2_ASIC << 8)){	/* SSP64 */
     while ( (*regs = inb( port )) & 0xc0 );  /* 0x?006 - align w/zero */
     regs++;
     *regs++ = inb( port );    /* 0x?008  (regs[4])*/
     *regs++ = inb( port );    /* 0x?00A */
     *regs++ = inb( port );    /* 0x?00C */

  /* create a "standard" board id/rev byte and store at array index 7 64/128 */
     *regs++ = ((regs_org[2] >> 8) & 0xf8) | ((regs_org[3] >> 11) & 0x07);

     *regs = cmn_irq[ regs_org[3] & 0x07 ];  /* array index 8 */

     if ( !(*regs_enb & 0x01) )  /* enabled? */
        return -1;

     if ( *regs == 0xff )
     {
        if ( eisa_brdtyp_ok( regs_org[7] , regs_org[9] )) {
           return true;
	}
	else
           return -3;
      }

  } else { 	/* SSP2/4 */	
     ushort u1, v1, pp;

     port += 2;                /* Skip 2 unused ports */
     *regs++ = inw( port );    /* Device Select,MA15/MA14 (regs[3]) */	
     outw(regs_org[3] | 0x1000,port);
     port += 2;
     *regs++ = inw( port );    /* A23-A16, MA31-A24 (regs[4]) */ 
     *regs++ = 0;
     *regs++ = 0;
     *regs++ = 0;
     *regs++ = 0;


     /* create a standard board byte at array index 10 */
     /* must toggle rmod to both states */
     pp = slot * 0x1000 + 0x0c80;
     u1 = inw( (pp + 8));  /* port 8/9 */
     outw( (u1 | 0x1000), (pp + 8));  /* Set special duty don't hang bit */
     u1 = inw( (pp + 8));  /* port 8/9 */
     outw( (u1 | 0x0080), (pp + 8)); 
     v1 = inw( (pp + 6));  /* port 6/7 */
     regs_org[10] = ((ushort_t) (v1 & 0x3800) >> 11);	/*  set lower 3 bits */
     outw( ( u1 & ~0x0080 ), (pp + 8));	/* toggle rmod */
     v1 = inw( (pp + 6));  /* port 6/7 */
     regs_org[10] |= ((ushort_t) (v1 & 0x3800) >> 8); /* or in upper bits */

  /* board id/rev byte and store at array index 9 SSP2/4 */
     *regs++ = ((regs_org[2] >> 8) & 0xff) | ((regs_org[1] >> 8) & 0x07);
/*      *regs = ((regs_org[2] >> 8) & 0xff);
     *regs++ |= ((regs_org[1] >> 8) & 0x07);
 same as above
*/
     /* 
      * Set array index 7 to qualifier(high byte) and board ID(low byte).
      */
     regs_org[7] = (regs_org[10] << 8) | regs_org[9];

     if ( !(*regs_enb & 0x01) ) {  /* enabled? */
        return -1;
     }

     if ( eisa_brdtyp_ok( (regs_org[2] >>8) & 0xF8  , regs_org[10])) {
        return true;
     }
     else {
        return -3;
     }

   }
    return -2;
}  /* eisa_brd_found */
#endif	/* ISA_ENAB */

/******************************************************************************

	EISA_ICP_COUNT

	Return the number of ICP's present on the board described by
	registers "regs".

	As additional board types are added, this function must be updated.

******************************************************************************/

static int eisa_icp_count( ushort_t id )
{
  if(((id & 0xFC) == 0xC8) ||  /* SSP4 SST-16P with no panel */
     ((id & 0xFC) == 0xC4) ||  /* SSP4 SST-16P with 16RJ */
     ((id & 0xFC) == 0xC0) ||  /* SSP4 SST-16P with 16DB */
     ((id & 0xFC) == 0xD0))    /* SSP4 SST-16P with 16DB9 */
    return 4;

  /* If SST128 or SSPC8 */
  if(((id & 0xfC) == 0x10) ||  /* SSP64 SST-128x */
     ((id & 0xFC) == 0x90) ||  /* SSP4 PCI SST-8P(F) */
     ((id & 0xFC) == 0x94) ||  /* SSP4 PCI SST-8P(RJ) */
     ((id & 0xFC) == 0xF4) ||  /* SSP4 PCI SST-8P(PWR) */
     ((id & 0xFC) == 0x98) ||  /* SSP4 PCI SST-MM8P */
/* 0xF82D == SSP4 SSM-8I*/
     (((id & 0xfC) == 0xF8) && (((id & 0xFF00) >> 8) == 0x2D)) || 
/* 0xF812 == SSP4 SST-8I+ (or SST-8I/RJ) */
     (((id & 0xfC) == 0xF8) && (((id & 0xFF00) >> 8) == 0x12)) || 
/* SSP4 8 port ISA/EISA */ 
     (((id & 0xfC) == 0xF8) && (((id & 0xFF00) >> 8) == 0x1B)))
    /* 128 port board */
    return 2;
  else
    return 1;			/* 64 or 8 port board */
}  /* eisa_icp_count */

#ifdef	ISA_ENAB
/******************************************************************************

	EISA_BASE_PADDR

	Return the mapped physical address for the board.  "regs" contains
	the array of Equinox registers.

	As additional board types are added, this function must be updated.

******************************************************************************/

static paddr_t eisa_base_paddr( ushort_t *regs )
{
ushort_t bid;
paddr_t p = 0;
uint_t w;


  bid = regs[2] & (SSP4_MASK << 8);

  if((bid & 0xF800) < (SSP2_ASIC << 8) ){	/* SSP64 */
     w = *(regs + 4);
     w = (w & 0x3f) << 14;
     p |= (paddr_t) w;
     w = *(regs + 5);
     w = (w & 0x3f) << 20;
     p |= (paddr_t) w;
     w = *(regs + 6);
     w = (w & 0x3f) << 26;
     p |= (paddr_t) w;
     return p;
  } else {	/* SSP2/4 */
     w = *(regs + 3);
     w = (w & 0xC0) ;
     p |= (paddr_t) w;
     w = *(regs + 4);
     w = (w & 0xff) << 16;
     p |= (paddr_t) w;
     w = *(regs + 4);
     w = (w & 0xff00) << 16;
     p |= (paddr_t) w;
  }
  return p;
}  /* eisa_base_paddr */

/******************************************************************************

	EISA_MEM_SIZE

	Return the size of the memory map required by the board
	described by "regs".
	
	req == 0 return the size of memory to map
	req == 1 return the amount of buffer dram

	As additional board types are added, this function must be updated.

	This name is a public symbol in Unisys v1.2.

******************************************************************************/

static uint_t eisa_mem_size( ushort_t *regs, int req )
{
ushort_t w;
ushort_t id, bid;

  if ( !req )
  {
    /* return size of footprint */
  bid = regs[2] & (SSP4_MASK << 8);

  if((bid & 0xF800) < (SSP2_ASIC << 8) ){	/* SSP64 */
    w = *(regs + 3);
    id = regs[7] & SSP4_MASK;
  } else {				/* SSP2/4 */
    w = *(regs + 3);
    id = regs[2] & (SSP4_MASK << 8);
    id = (id >> 8);
  }

    switch ( id )
    {
      case 0x10:
        if ( w & 0x0010 )
          return FLAT128_MEM_LEN;
        else
          return WIN16_MEM_LEN;
      case 0x08:
        if ( w & 0x0010 )
          return FLAT64_MEM_LEN;
        else
          return WIN16_MEM_LEN;
      case 0x18:
      case 0x20:
        if ( w & 0x0010 )
          return FLAT8_MEM_LEN;
        else
          return WIN16_MEM_LEN;
       case SSP2_ID:
       case SSP4_ID:
	if ( w & 0x20 ) {		/* Flat mode SSP2/4; 16K * ASIC */
		switch(regs[10] & 0x3F) {  /* Switch on qualifier ID */
			case 0:
			case 0x24:
			case 0x09:
			case 0x36: /* MPM-4 */
				return(WIN16_MEM_LEN);
				break;
			case 0x10:
			case 0x1B:
			case 0x1A:
				return(WIN16_MEM_LEN * 2);
				break;
			case 0x18:
			case 0x20:
			case 0x08:
				return(WIN16_MEM_LEN * 4);
				break;
			default:
				return(WIN16_MEM_LEN * 4);
				break;
		}
	}
	else
          return WIN16_MEM_LEN;  /* Overlayed ASICS 16K  */
      case 0x30:
      case 0x38:
        if ( w & 0x0010 ) 
	   return 0x10000; 
	else
	   return 0x8000;
      default:
	return 0;
    }
  }
  else  /* req == 1 */
  {
    bid = regs[2] & (SSP4_MASK << 8);
    if((bid & 0xF8) < (SSP2_ASIC << 8) ){	/* SSP64 */
       w = *(regs + 3);
       id = regs[7] & SSP4_MASK;
    } else {				/* SSP2/4 */
       w = *(regs + 3);
       id = regs[2] & (SSP4_MASK << 8);
       id = (id >> 8);
    }
    /* return size of RAM */
    switch ( id  )
    {
      case 0x10:
        return 0x100000;  /* 1 meg */
      case 0x08:
        return 0x80000;   /* 1/2 meg */
      case 0x18:
      case 0x20:
      case 0x30:
      case 0x38:
	return 0x8000;    /* 32k */
      case SSP2_ID:
      case SSP4_ID:
	if ( w & 0x20 )		/* Flat mode SSP2/4; 16K num ICP */
		switch(regs[10] & 0x3F) {  /* Switch on qualifier ID */
			case 0:
			case 0x24:
			case 0x09:
			case 0x36: /* MPM-4 */
				return(WIN16_MEM_LEN);
				break;
			case 0x10:
			case 0x1B:
			case 0x1A:
				return(WIN16_MEM_LEN * 2);
				break;
			case 0x18:
			case 0x20:
			case 0x08:
				return(WIN16_MEM_LEN * 4);
				break;
			default:
				return(WIN16_MEM_LEN * 4);
				break;
		}
	else
          return 0x4000;  /* Overlayed ASICS 16K  */
      default:
	return 0;
    }
  }
}  /* eisa_mem_size */
#endif	/* ISA_ENAB */

static uint_t pci_mem_size( unsigned short id )
{
	/* return size of ram */
	switch(id & 0xFF) {
	case 0xC8: /* SST-16P no panel */
	case 0xC4: /* SST-16P with 16RJ */
	case 0xC0: /* SST-16P with 16DB */
	case 0xD0: /* SST-16P with 16DB9 */
                return FLAT64K_MEM_LEN;
	  case 0x10:
	  case 0x08:
		return FLAT128_MEM_LEN;
          case 0x90:
          case 0x94:
          case 0x88:
          case 0xB8:
          case 0x8C:
          case 0xA8:
          case 0xEC:
          case 0xF4:
          case 0x98:
          case 0x9C:
          case 0xFC:
                return FLAT64K_MEM_LEN;
	  default:
		return 0;
	}
}  /* pci_mem_size */

#ifdef	ISA_ENAB
/******************************************************************************

	EISA_MEM_STRAT

	Return the memory access strategy required by the board
	described by "regs".

	As additional board types are added, this function must be updated.

******************************************************************************/

static uint_t eisa_mem_strat( ushort_t *regs )
{
ushort_t bid;
ushort_t w;

  bid = regs[2] & (SSP4_MASK << 8);
  if((bid & 0xF800) < (SSP2_ASIC << 8) ){	/* SSP64 */
    w = *(regs + 3);
    if ( w & 0x0010 )
       return HA_FLAT;
    else
       return HA_WIN16;
  } else {				/* SSP2/4 */
    w = *(regs + 3);
    if ( w & 0x20 )		/* Flat mode SSP2/4; 16K * ASIC */
       return HA_FLAT;
    else
       return HA_WIN16;  /* Overlayed ASICS 16K  */
  }
}  /* eisa_mem_strat */
#endif	/* ISA_ENAB */

#ifdef	MCA_ENAB
/******************************************************************************

	MCA_BRD_FOUND

	Examine MCA "slot" for an Equinox board.  If not found return false.
	If found, fill "regs" from the POS registers.  Test that the board is
	enabled.  If so, return true, else -1.  If an IRQ is enabled, 
	return -2.

	As additional board types are added, this function MAY have to be 
	updated.

	The type of "regs" is expected to be uchar_t.
	The length of "regs" is expected to be MCA_REGS_LEN.
	The contents of "regs" is defined as follows:

	regs[0]		POS0
	regs[1]		POS1
	regs[2]		POS2
	regs[3]		POS3
	regs[4]		POS4
	regs[5]		POS5
	regs[6]		unused
	regs[7]		base i/o + 0	board status
	regs[8]		base i/o + 1	software id/rev (BBBBBRRR)
	regs[9]		software created IRQ level
	regs[10]	lsb of 16-bit i/o base address
	regs[11]	msb of 16-bit i/o base address

******************************************************************************/

static int mca_brd_found( int slot, uchar_t *regs)
{
ushort_t mca_id;
uchar_t b;
ushort_t w;
uchar_t *regs_org;

  outb(0xff,0x94);  			/* enable pos access */
  outb(0x8 | slot,0x96);               /* select slot */
  mca_id = (inb(0x100) | (inb(0x101) << 8));
  if ( (mca_id != MCA_EQX_128)
          && (mca_id != MCA_EQX_64)
          && (mca_id != MCA_EQX_8) )
  {
    outb(0x0, 0x96);             /* de-select slot */
    return false;
  }

  /* fill in remainder of regs */
  regs_org = regs;
  *regs++ = mca_id & 0x00ff;
  *regs++ = (unsigned)(mca_id & 0xff00) >> 8;
  *regs++ = inb(0x102);
  *regs++ = inb(0x103);
  *regs++ = inb(0x104);
  *regs++ = inb(0x105);
  outb(0x0,0x96 );             /* de-select slot */

  if ( !(regs_org[2] & 0x01) )  /* enabled? */
    return -1;

  /* compute base i/o address */
  w = mca_iobase[ (regs_org[5] >> 3) & 0x0f ];
  * (ushort_t *) &regs_org[10] = w;

  regs_org[7] = inb( w );
  regs_org[8] = inb( w + 1 );

  /* compute irq level */
  b = cmn_irq[ regs_org[5] & 0x07 ];
  regs_org[9] = b;

  if ( regs_org[9] == 0xff )
    return true;
  else
    return -2;
}  /* mca_brd_found */

/******************************************************************************

	MCA_ICP_COUNT

	Return the number of ICP's present on the board described by
	registers "regs".

	As additional board types are added, this function must be updated.

******************************************************************************/

static int mca_icp_count( uchar_t id )
{
  if ( (id & 0xf8) == 0x10 )  /* 128 port board */
    return 2;
  else
    return 1;  /* 64 or 8 port board */
}  /* mca_icp_count */

/******************************************************************************

	MCA_BASE_PADDR

	Return the mapped physical address for the board.  "regs" contains
	the array of Equinox registers.

	As additional board types are added, this function must be updated.

******************************************************************************/

static paddr_t mca_base_paddr( uchar_t *regs )
{
paddr_t p = (paddr_t)0;
uint_t w;

  w = regs[4] << 24;
  w |= regs[3] << 16;
  w |= (regs[2] & 0xc0) << 8;
  p = (paddr_t) w;
  return p;
}  /* mca_base_paddr */

/******************************************************************************

	MCA_MEM_SIZE

	Based on "req", return either the amount of ram on the card
	OR the size of the memory map required by the board (regs and
	ram) described by "regs".

		req == 0	return the size of the memory map footprint
		req == 1	return the amount of ram on the card

	As additional board types are added, this function must be updated.

	Assumes "MAX1" bit is on when appropriate.

******************************************************************************/

static uint_t mca_mem_size( uchar_t *regs, int req )
{
uchar_t b;

  if ( !req )
  {
    /* return size of footprint */
    b = *(regs + 2);
    switch ( regs[8] & 0xf8 )
    {
      case 0x10:
        if ( b & 0x02 )
          return FLAT128_MEM_LEN;
        else
          return WIN16_MEM_LEN;
      case 0x08:
        if ( b & 0x02 )
          return FLAT64_MEM_LEN;
        else
          return WIN16_MEM_LEN;
#ifdef SSM
      case 0x30:
      case 0x38:
#endif
      case 0x18:
      case 0x20:
        if ( b & 0x02 )
          return FLAT8_MEM_LEN;
        else
          return WIN16_MEM_LEN;
      default:
	return 0;
    }
  }
  else  /* req == 1 */
  {
    /* return size of RAM */
    switch ( regs[8] & 0xf8 )
    {
      case 0x10:
        return 0x100000;  /* 1 meg */
      case 0x08:
        return 0x80000;   /* 1/2 meg */
#ifdef SSM
      case 0x30:
      case 0x38:
#endif
      case 0x18:
      case 0x20:
	return 0x8000;    /* 32k */
      default:
	return 0;
    }
  }
}  /* mca_mem_size */

/******************************************************************************

	MCA_MEM_STRAT

	Return the memory access strategy required by the board
	described by "regs".

	As additional board types are added, this function must be updated.

******************************************************************************/

static uint_t mca_mem_strat( uchar_t *regs )
{
uchar_t b;

  b = *(regs + 2);
  if ( b & 0x02 )
    return HA_FLAT;
  else
    return HA_WIN16;
}  /* mca_mem_strat */

#endif /* MCA_ENAB */

/****************************************************************************/        
static int megamodem(int d, int cmd)
{
	register struct mpchan *mpc = &meg_chan[d];
	register icpiaddr_t icpi = mpc->mpc_icpi;
	icpbaddr_t icpb;
	ushort_t cur, mux;
	unsigned short id = mpc->mpc_mpd->mpd_id;

/*
	printk("megamodem enter \n");
*/
	icpb = (rx_locks & LOCK_A) ? &icpi->ssp.cin_bank_b : &icpi->ssp.cin_bank_a;
  	if (!(icpb->ssp.bank_events & EV_REG_UPDT)) 
  		frame_wait(mpc,2); /* make sure regs are valid */ 
        GET_CTRL_SIGS(mpc, cur);
	mux = 0x8000;
  switch ( cmd )
  {
    case TURNON:
      /* raise specific outbound signals */
      mpc->mpc_flags |= MPC_MODEM;
      /* mikes */
      /* default sigs for HW flow control off */
      cur |= (TX_DTR | TX_RTS | TX_HFC_DTR | TX_HFC_RTS);

      /* now check for HW flow control */
      if (mpc->mpc_param & IOCTRTS)
	 cur &= ~TX_HFC_RTS;
      if (mpc->mpc_tty) {
	 if (mpc->mpc_tty->termios)
	    if (mpc->mpc_tty->termios->c_cflag & CRTSCTS) 
	       cur &= ~TX_HFC_RTS;
      }

      if((rx_ctrl_sig & 0x2020) == 0x2020)
         cur |= mux;
      else
         cur &= ~mux;
/* Set the control signals "mux" bits for the SST-16I and SST-16P
     0xC8 == SST-16P with no panel 
     0xC4 == SST-16P with 16RJ 
     0xC0 == SST-16P with 16DB 

     0x17F8 == SST-16I with no panel  
     0x0FF8 == SST-16I with 16RJ 
     0x07F8 == SST-16I with 16DB */

	if (mpc->mpc_mpd->mpd_bus == PCI_BUS) {
	    if (((id & 0xFF) == 0xC0) || 
	        ((id & 0xFF) == 0xC4) || 
	        ((id & 0xFF) == 0xC8) ||  
	        ((id & 0xFF) == 0xD0))  
		cur |= 0x44;
	}
	else if (((id & 0xFFF8) == 0x17F8) || 
	    ((id & 0xFFF8) == 0x07F8) || 
	    ((id & 0xFFF8) == 0x0FF8)) 
		cur |= 0x44;
      cur ^= TX_SND_CTRL_TG;
      SET_CTRL_SIGS(mpc, cur);
      break;

    case TURNOFF:
      /* lower specific outbound signals */
      mpc->mpc_flags &= ~MPC_MODEM;
      cur &= ~(TX_HFC_DTR | TX_HFC_RTS | TX_DTR|TX_RTS);
      cur ^= TX_SND_CTRL_TG;
      SET_CTRL_SIGS(mpc, cur);
      break;

    default:
      break;
  }
/*
	printk("megamodem exit\n");
*/
  /* return current state of DCD */
  return ((icpb->ssp.bank_sigs & (unsigned) CIN_DCD) >> 1);
} 


/*
 * MEGAPARAM
 * Map unix termio parameters into megaport
 * icp channel control register parameters.
 */

/******************************************************************************/
static int megaparam( int chan)
{
register struct mpchan *mpc;
volatile register struct termios *tiosp;
volatile icpiaddr_t icpi;
volatile icpoaddr_t icpo;
unsigned speed;
unsigned char oldreg;
int ii, rslt = 0;
int flags = 0;
ushort d, e;

      save_flags(flags);
      cli();
  /* setup pointers for general use */
  mpc = &meg_chan[chan];
  tiosp = mpc->mpc_tty->termios;

   if(mpc->mpc_param & IOCTLCK){
      restore_flags(flags);
      return(0);
   }
/* RAMP START --------------------------------------------- */
   if (ISRAMP(mpc)){
	ramparam(mpc);
	restore_flags(flags);
	return(0);
   }
/* RAMP END ----------------------------------------------- */

  icpi = mpc->mpc_icpi;
  icpo = mpc->mpc_icpo;
  /* CLOCAL and carrier detect parameters */
  if (tiosp->c_cflag & CLOCAL){
    mpc->mpc_icpi->ssp.cin_attn_ena &= ~ENA_DCD_CNG;
		mpc->carr_state = 1;
  }
  else{  /* CLOCAL clear */
    /* set DCD processing */
/*printk("enabling DCD_CNG for device %d\n",chan);*/
    mpc->mpc_icpi->ssp.cin_attn_ena |= ENA_DCD_CNG;
	}

  /* outbound control signals */
  if ((tiosp->c_cflag & CBAUD) == 0)  /* B0 */
  {
	/*if (!(tiosp->c_cflag & CLOCAL)){
char prtstr[60];
sprintf(prtstr,"megaparam:turning off RTS/DTR for device %d\n",chan);
console_print(prtstr);*/
    (void) megamodem(chan, TURNOFF);
	/*}*/
      restore_flags(flags);
    return 0;
  }
else{
	mpc->carr_state = megamodem(chan, TURNON); /*shashi 02/04/98*/
}

  /* set output control signal flow control */
  if(mpc->mpc_param & IOCTCTS) 
     tiosp->c_cflag |= CRTSCTS;
  
/* set input control signal flow control */
  if(mpc->mpc_param & IOCTRTS) 
     tiosp->c_cflag |= CRTSCTS;

#ifdef RS422
   /* ignore flow control setting on rs-422 type of ports */

   if (mpc->mpc_icp->lmx[mpc->mpc_lmxno].lmx_id != LMX_8E_422 &&
       mpc->mpc_icp->lmx[mpc->mpc_lmxno].lmx_id != LMX_PM16_422)
   {
#endif

#ifdef SSM
       /* IF board type SSM, then re-map, ie change flow control
           to be DTR/DCD instead of RTS/CTS
           ASYNC_NORMAL_ACTIVE(ignore the hardware flow control setting)
           mpd_id == 0x2df8 is the SSP4 based SSM-8I
           mpd_id == 0x12f8 is the SSP4 based SST-8I/RJ
           mpd_id == 0x3ff8 is the SSP4 based SST-4I/RJ
           mpd_id == 0x94 is the SSP4 based SST-8P/RJ
           mpd_id == 0x8C is the SSP4 based SST-4P/RJ
       */

         if (((mpc->mpc_mpd->mpd_id & 0xF8) == 0x30)  || 
            ((mpc->mpc_mpd->mpd_id & 0xFFF8) == 0x2DF8) ||     
            ((mpc->mpc_mpd->mpd_id & 0xFFF8) == 0x12F8) ||     
            ((mpc->mpc_mpd->mpd_id & 0xFFF8) == 0x3FF8) ||     
            ((mpc->mpc_mpd->mpd_id & 0xF8) == 0x38) ||
            ((mpc->mpc_mpd->mpd_id & 0xF8) == 0x94) ||
            ((mpc->mpc_mpd->mpd_id & 0xF8) == 0x8C))
         {
	   if (mpc->flags & ASYNC_CALLOUT_ACTIVE){
               if(tiosp->c_cflag & CRTSCTS)  /* only need to change the mask */
                 icpi->ssp.cin_susp_output_lmx |= DCD_OFF;
               else
                  icpi->ssp.cin_susp_output_lmx &= ~DCD_OFF;
            }
         }
         else
         {
#endif
            if(tiosp->c_cflag & CRTSCTS)  /* only need to change the mask */
               icpi->ssp.cin_susp_output_lmx |= CTS_OFF;
            else
               icpi->ssp.cin_susp_output_lmx &= ~CTS_OFF;
#ifdef SSM
         }
#endif

#ifdef RS422
    }
#endif


#ifdef RS422
   /* ignore flow control setting on rs-422 type of ports */

   if (mpc->mpc_icp->lmx[mpc->mpc_lmxno].lmx_id != LMX_8E_422 &&
       mpc->mpc_icp->lmx[mpc->mpc_lmxno].lmx_id != LMX_PM16_422)
   {
#endif
#ifdef SSM
       /* IF board type SSM, then re-map, ie change flow control
           to be DTR/DCD instead of RTS/CTS
           ASYNC_NORMAL_ACTIVE(ignore the hardware flow control setting)
           mpd_id == 0x2df8 is the SSP4 based SSM-8I
       */

         if ((mpc->mpc_mpd->mpd_id & 0xF8) == 0x30  || 
            ((mpc->mpc_mpd->mpd_id & 0xFFF8) == 0x2DF8) ||     
            (mpc->mpc_mpd->mpd_id & 0xF8) == 0x38)
         {
	    if (mpc->flags & ASYNC_CALLOUT_ACTIVE){
                 if (tiosp->c_cflag & CRTSCTS)
                        tx_ctrl_sig &= ~TX_HFC_DTR;
                 else
                      tx_ctrl_sig |= TX_HFC_DTR;
            }
        }
        else
        {
#endif
              if (tiosp->c_cflag & CRTSCTS)
                    tx_ctrl_sig &= ~TX_HFC_RTS;
              else
                    tx_ctrl_sig |= TX_HFC_RTS;
#ifdef SSM
        }
#endif

#ifdef RS422
    }
#endif
#ifdef XX
  /* set input control signal flow control */
  if (tiosp->c_cflag & CRTSCTS)
    megamodem( d, TURNOFF_OVLD, TIOCM_RTS );  /* unmask it */
  else
    megamodem( d, TURNON_OVLD, TIOCM_RTS ); /* mask it */
#endif

  /* set output inband flow control */
  if(tiosp->c_iflag & IXON || mpc->mpc_param & IXONSET) {
    unsigned short char_ctrl = icpi->ssp.cin_char_ctrl;
    if(mpc->mpc_param & IOCTXON) { 
      char_ctrl &= ~EN_DNS_FLW;  /* do not discard xon/xoff */
      char_ctrl |= EN_XON|EN_XOFF;
    }
    else /* discard xon/xoff */
      char_ctrl |= EN_XON|EN_XOFF|EN_DNS_FLW;
    char_ctrl &= ~EN_DBL_FLW; /* disable double flow */
    if (( tiosp->c_iflag & IXANY ) && !(mpc->mpc_param & IXANYIG)) 
      char_ctrl |= EN_IXANY;
    else
      char_ctrl &= ~EN_IXANY;
    cur_chnl_sync( mpc );
    mpc->mpc_stop = tiosp->c_cc[VSTOP];
    mpc->mpc_start = tiosp->c_cc[VSTART];
    icpi->ssp.cin_xoff_1 = mpc->mpc_stop;
    icpi->ssp.cin_xon_1 = mpc->mpc_start;
    icpi->ssp.cin_char_ctrl = char_ctrl;
    icpi->ssp.cin_locks &= ~DIS_IBAND_FLW;  /* clear lock bit - enable inband */
  }
  else  /* cancel action */
  {
    icpi->ssp.cin_char_ctrl &= ~EN_DNS_FLW;  /* discard xon/xoff */
    icpi->ssp.cin_locks |= DIS_IBAND_FLW;     /* set lock bit - disable inband */
    cur_chnl_sync( mpc );
    icpi->ssp.cin_inband_flow_ctrl = 0;  /* calibrate */
  }
  
  /* set input inband flow control */
  if ( tiosp->c_iflag & IXOFF ){
    mpc->mpc_stop = tiosp->c_cc[VSTOP];
    mpc->mpc_start = tiosp->c_cc[VSTART];
    icpo->ssp.cout_xoff_1 = mpc->mpc_stop;
    icpo->ssp.cout_xon_1 = mpc->mpc_start;
    icpo->ssp.cout_flow_cfg &= ~TX_XON_DBL;
    icpo->ssp.cout_flow_cfg |= TX_XON_XOFF_EN;
  }
  else  /* cancel enable, send XON if we previously sent an XOFF */
  {
	 if ((icpo->ssp.cout_flow_cfg & TX_XON_XOFF_EN) && (icpi->ssp.cin_int_flags & 0x40)){
		/*printk("megaparam: cin_int_flags = %x before megajam\n",
			icpi->ssp.cin_int_flags);*/
		megajam(mpc, mpc->mpc_start);
	}
    icpo->ssp.cout_flow_cfg &= ~(TX_XON_XOFF_EN|TX_XON_DBL);
  }
  
  /* baudrate */
  speed = tiosp->c_cflag & CBAUD;
#ifdef DEBUG
printk("speed for device %d is %d\n", chan, speed);
#endif
#ifdef SPD
  ispeed = (tiosp->c_cflag & CIBAUD) >> IBSHIFT;
  if ( !ispeed )
  {
    ispeed = speed;
    /* set CIBAUD to match CBAUD since we support split speeds */
    tiosp->c_cflag |= (ispeed << IBSHIFT);
  }
#endif
  if (speed & CBAUDEX){
	speed &= ~CBAUDEX;
	if ((speed < 1) || (speed > 4))
		tiosp->c_cflag &= ~CBAUDEX;
	else 
		speed += 15;
  }
  if ( speed >= 0 && speed <= (B38400 + 4)) {
	icpqaddr_t icpq; /* 1.07 */
    d = icpbaud(speed, mpc);
    if (d != icpi->ssp.cin_baud)
       icpi->ssp.cin_baud = d;
    if (d != icpo->ssp.cout_baud) {
      icpo->ssp.cout_baud = d;
	  icpq = &icpo->ssp.cout_q0; /* 1.07 */
	
      cur_chnl_sync(mpc);
		if ((mpc->mpc_mpd->mpd_id & 0xF8) < SSP2_ASIC)
	    			icpq->ssp.q_block_count = lowat[speed];
		else
			icpq->ssp.q_block_count = ssp4_lowat[speed];
#ifdef DEBUG
printk("speed for for device %d is %d\n", chan, speed);
#endif
	if (speed == B38400){
		if ((mpc->flags & ASYNC_SPD_MASK) == ASYNC_SPD_HI){
			if ((mpc->mpc_mpd->mpd_id & 0xF8) < SSP2_ASIC)
	    			icpq->ssp.q_block_count = lowat[speed + 1];
			else
				icpq->ssp.q_block_count = ssp4_lowat[speed + 1];
		}
		else if ((mpc->flags & ASYNC_SPD_MASK) == ASYNC_SPD_VHI){
			if ((mpc->mpc_mpd->mpd_id & 0xF8) < SSP2_ASIC)
	    			icpq->ssp.q_block_count = lowat[speed + 2];
			else
				icpq->ssp.q_block_count = ssp4_lowat[speed + 2];
		}
	}
      icpo->ssp.cout_int_baud_ctr = 0;
      cur_chnl_sync(mpc);
      if (icpo->ssp.cout_int_baud_ctr)
        icpo->ssp.cout_int_baud_ctr = 0;
    }
    if ( d >= 0x7ffe )
      icpo->ssp.cout_flow_cfg |= TX_XTRA_DMA;
    else
      icpo->ssp.cout_flow_cfg &= ~TX_XTRA_DMA;
  } else
    	rslt = 1;  /* no valid speed to set */

  /* databits */
  e = icpi->ssp.cin_char_ctrl &= ~0x2000;  /* default to no discard bit7 */
  switch (tiosp->c_cflag & CSIZE)
  {                                    /* set character size */
    case CS5:
      d = 0x00;
      break;

    case CS6:
      d = 0x01;
      break;

    case CS7:
      d = 0x02;
      break;

    default:  /* case CS8 - no other value should occur */
      d = 0x03;
      if(tiosp->c_iflag & ISTRIP)
        e |= 0x2000;  /* discard bit 7 */
  }
  if(tiosp->c_cflag & PARENB)
  {
    d |= 0x04;
    if( !(tiosp->c_cflag & PARODD) )
      d |= 0x08;
  }
  e &= ~0x001f;
  oldreg = tx_cpu_reg;
  tx_cpu_reg |= TX_SUSP;
  cur_chnl_sync(mpc);
  icpi->ssp.cin_char_ctrl = (d | e);
  e = icpo->ssp.cout_char_fmt;
  e &= ~0x001f;
  icpo->ssp.cout_char_fmt = (d | e);
  if (!(oldreg & TX_SUSP))
    tx_cpu_reg &= ~TX_SUSP;

  /* prepare for input break/parity processing - cancel special features */
  icpi->ssp.cin_char_ctrl &= ~0x03e0;
  mpc->mpc_cin_stat_mask &= ~0x03a0;

  /* prepare for input break processing */
  if ( tiosp->c_iflag & IGNBRK ) 
  {
    /* hardware can ignore break nulls */
    icpi->ssp.cin_char_ctrl |= IGN_BRK_NULL;
  }
  else
  {
    /* always ignore break, break event handled in megasint */	  
    icpi->ssp.cin_char_ctrl |= IGN_BRK_NULL;
    mpc->mpc_cin_stat_mask |= 0x0120;   /* watch for framing errs w/nulls */
  }
  rx_cie |= mpc->mpc_cin_stat_mask;
  /* clear lookup table - processing flags already clear */
  for ( ii = 0; ii < 32; ii++ )
    icpi->ssp.cin_lookup_tbl[ii] = 0;

  /* prepare for input parity processing */
  if ( tiosp->c_cflag & PARENB
	&& tiosp->c_iflag & INPCK )
  {
    if ( tiosp->c_iflag & IGNPAR )
    {
      /* discard chars with parity/framing errors */
      icpi->ssp.cin_char_ctrl |= IGN_BAD_CHAR;
    }
    else
    {
      /* hardware must maintain and tag err'd chars */
      mpc->mpc_cin_stat_mask |= 0x0180; /* watch for parity and framing */
      if ( tiosp->c_iflag & PARMRK 
              && !(tiosp->c_iflag & ISTRIP) )
      {
	/* put 0xff in lookup table */
	icpi->ssp.cin_lookup_tbl[0x1f] |= 0x80;
	icpi->ssp.cin_char_ctrl |= EN_CHAR_LOOKUP;  /* enable lookup */
        mpc->mpc_cin_stat_mask |= 0x0200;     /* watch for lookup event */
     }
    }
  }
#ifndef LINUX
    if(mpc->mpc_flags & MPC_SCRN) {
	icpi->ssp.cin_lookup_tbl[mpc->mpc_scrn.hotkey[0] >> 3] = 
				1 << (mpc->mpc_scrn.hotkey[0] & 7);
	icpi->ssp.cin_char_ctrl |= EN_CHAR_LOOKUP;
	icpi->ssp.cin_attn_ena |= ENA_CHAR_LOOKUP;
    }
#endif /** Not LINUX **/

  /* output stop bits */
  if (tiosp->c_cflag & CSTOPB)
    icpo->ssp.cout_char_fmt |= 0x20;
  else
    icpo->ssp.cout_char_fmt &= ~0x20;
#ifndef LINUX
  setrwmode(mpc);
  /* output session control */
  if (mpc->mpc_wmode && (tiosp->t_oflag & OPOST))
  {
    icpo->ssp.cout_ses_ctrl_a = MOPOST;  /* enable opost */
    if ( tiosp->t_oflag & ONLCR )
      icpo->ssp.cout_ses_ctrl_a |= MONLCR;
    else
      icpo->ssp.cout_ses_ctrl_a &= ~MONLCR;

    if ( tiosp->t_oflag & OCRNL )
      icpo->ssp.cout_ses_ctrl_a |= MOCRNL;
    else
      icpo->ssp.cout_ses_ctrl_a &= ~MOCRNL;

    if ( (tiosp->t_oflag & TAB3) == TAB3 )
    {
      icpo->ssp.cout_ses_ctrl_a |= MTABX;
      icpo->ssp.cout_ses_col_a = 0;
    }
    else
      icpo->ssp.cout_ses_ctrl_a &= ~MTABX;

    if ( tiosp->t_oflag & ONOCR )
      icpo->ssp.cout_ses_ctrl_a |= MONOCR;
    else
      icpo->ssp.cout_ses_ctrl_a &= ~MONOCR;

    if ( tiosp->t_oflag & ONLRET )
      icpo->ssp.cout_ses_ctrl_a |= MONLRET;
    else
      icpo->ssp.cout_ses_ctrl_a &= ~MONLRET;
  }
  else
#endif
    icpo->ssp.cout_ses_ctrl_a = 0;  /* default */

  if ( tiosp->c_cflag & CREAD )
    /* make sure dma's to dram are enabled */
    icpi->ssp.cin_locks &= ~0x80;
  else
    /* disable unnecessary dma's to dram */
    icpi->ssp.cin_locks |= 0x80;
      restore_flags(flags);
  return rslt;
}   
/******************************************************************************
        
        ICPBAUD
  
        Compute the 15-bit baud value for "cin_baud" or "cout_baud".  The
        input is the the current strtty "t_cflag & CBAUD".  The 16th bit
        (activate autobaud) is always returned false.

        There are certain "unitcode" restrictions which are not reflected
        in this code.

******************************************************************************/

static  ushort_t icpbaud( int val, struct mpchan *mpc)
{
int baud, maxbaud;
#ifdef DEBUG
	printk("icpbaud: device %d, val = %d\n", mpc->mpc_chan, val);
#endif
   switch(mpc->mpc_icp->lmx[mpc->mpc_lmxno].lmx_speed)
   {
   case 0:
	maxbaud = 115200;
	break;
   case 1:
	maxbaud = 230400;
	break;
   case 2:
	maxbaud = 460800;
	break;
   case 3:
	maxbaud = 921600;
	break;
   default:
	maxbaud = 115200;
	break;
   }

    baud = icpbaud_tbl[val];
/*printk("baud rate for channel %d is %d\n", mpc->mpc_chan, baud);*/
  if (baud == 38400){
  	if((mpc->flags & ASYNC_SPD_MASK) == ASYNC_SPD_HI)
		baud = 57600;
  	else if((mpc->flags & ASYNC_SPD_MASK) == ASYNC_SPD_VHI)
		baud = 115200;
  }
  if(baud == 2 * maxbaud / 3)
	return(0x7ffe);
  
  return (~(2*maxbaud/baud - 2) & 0x7fff) | 1;
 
}  /* icpbaud */

/******************************************************************************
  
        CHANON
        
        Turn on a channel.

******************************************************************************/

static int chanon( struct mpchan *mpc)
{
  volatile register icpiaddr_t icpi = mpc->mpc_icpi;
  volatile register icpoaddr_t icpo = mpc->mpc_icpo;
  volatile register icpgaddr_t icpg = (icpgaddr_t)icpo;
  volatile icpqaddr_t icpq = &icpo->ssp.cout_q0;
  volatile icpbaddr_t icpb;
  unsigned short cur;

uchar_t cin;

	if ((mpc->mpc_mpd->mpd_id & 0xF8) >= SSP2_ASIC)
		icpg = (icpgaddr_t)((unsigned long)(mpc->mpc_icpi) + 0x400);
	icpb = (rx_locks & LOCK_A) ? &icpi->ssp.cin_bank_b : &icpi->ssp.cin_bank_a;
  if (!(icpb->ssp.bank_events & EV_REG_UPDT)) 
  	frame_wait(mpc,2); /* make sure regs are valid */ 
  if((rx_ctrl_sig & 0x20a0) == 0x20a0) {	/* if AMI CFG */
    icpi->ssp.cin_susp_output_lmx |= 0x20; /* susp on mux disconnect */ 
    tx_lmx_cmd |= 0x80;		/* in case of a mux clear loopback */
  }
  else {
    if (!(ISRAMP(mpc)))
          /* local panel */
    	icpi->ssp.cin_susp_output_lmx &= ~0x20;
    tx_lmx_cmd &= ~0x80;
  }
  tx_lck_ctrl |= LCK_Q_ACT;
  cin = icpi->ssp.cin_locks;
  icpi->ssp.cin_locks = cin | 0x03;
  cur_chnl_sync( mpc );

  icpi->ssp.cin_bank_a.ssp.bank_fifo_lvl &= ~0x8f;
  icpi->ssp.cin_bank_b.ssp.bank_fifo_lvl &= ~0x8f;
  icpi->ssp.cin_tail_ptr_a = 
  icpi->ssp.cin_tail_ptr_b =  
  mpc->mpc_rxq.q_ptr = rx_next;
  if(mpc->mpc_mpd->mpd_nwin)
     mpc->mpc_rxq.q_ptr &= 0x3fff;
  rx_vmin = 1;
  /* unlock input */
  icpi->ssp.cin_attn_ena |= CIN_DEF_ENABLES;
  icpi->ssp.cin_locks = cin;
  tx_dma_stat = 0;
  icpo->ssp.cout_int_fifo_ptr = 0;  /* clr internal fifo ptr */
  Q_data_count = 0;
  Q_data_ptr = mpc->mpc_txbase + mpc->mpc_txq.q_begin;
  icpi->ssp.cin_inband_flow_ctrl = 0;  /* calibrate */
  mpc->mpc_txq.q_ptr = mpc->mpc_txq.q_begin;
  mpc->mpc_count = 0;
  icpo->ssp.cout_status &= ~0x02;  /* clear sending break state */
  GET_CTRL_SIGS(mpc, cur);
  cur |= TX_HFC_DTR|TX_HFC_RTS;
  SET_CTRL_SIGS(mpc, cur);

  /* unlock output */
  icpo->ssp.cout_attn_ena |= COUT_DEF_ENABLES;  /* includes 0x8000 */
  tx_lck_ctrl &= ~LCK_Q_ACT;

  /* calibrate invent char count */
  mpc->mpc_tx_last_invent = icpo->ssp.cout_ses_invent_a;

  /* Calibrate events and control signals in both banks */
  FREEZ_BANK(mpc);
  FREEZ_BANK(mpc);
  mpc->mpc_cin_events = 0;

  mpc->mpc_flags |= MPC_OPEN;
  return(0);
}   /* chanon */

/******************************************************************************
        
        CHANOFF
  
        Process channel off.
	
******************************************************************************/

static int chanoff( struct mpchan *mpc)
{
  volatile register icpiaddr_t icpi;
  volatile register icpoaddr_t icpo;
  icpqaddr_t icpq;
  volatile icpbaddr_t icpb;

int loop;
int ok;
uchar_t cin;
ushort_t events;


  icpi = mpc->mpc_icpi;
  icpo = mpc->mpc_icpo;
  icpq = &icpo->ssp.cout_q0;
	icpb = (rx_locks & LOCK_A) ? &icpi->ssp.cin_bank_b : &icpi->ssp.cin_bank_a;
  if (!(icpb->ssp.bank_events & EV_REG_UPDT)) 
  	frame_wait(mpc,2); /* make sure regs are valid */ 

  mpc->mpc_param &= (IOCTLLP|IOCTLLI|IOCTLLO|IOCTLLB|IOCTLMC|IOCTCTS|IOMSCRN|IOCTLCK|IOCTRTS|IOCTLHS);
  /* disable events */
  /*icpi->ssp.cin_attn_ena &= ENA_LMX_CNG;*/ /* don't clr lmx_cond_chng */
  icpi->ssp.cin_attn_ena &= (0x1e);
  if (!(mpc->mpc_chan % 16)) /* Shashi 04/01/98 */
  icpi->ssp.cin_attn_ena |= ENA_LMX_CNG;

  tx_lck_ctrl |= LCK_Q_ACT;
  cur_chnl_sync( mpc );
  icpo->ssp.cout_int_fifo_ptr = 0;  /* clr internal fifo ptr */
  tx_dma_stat = 0;
  Q_data_count = 0;
  Q_data_ptr = mpc->mpc_txbase + mpc->mpc_txq.q_begin;
  mpc->mpc_txq.q_ptr = mpc->mpc_txq.q_begin;
  mpc->mpc_count = 0;
  mpc->carr_state = 0;
  tx_lck_ctrl &= ~LCK_Q_ACT;
  cur_chnl_sync( mpc );
  if ((icpo->ssp.cout_int_save_togl & 0x4) == (icpo->ssp.cout_cpu_req & 0x4)) 
    tx_cpu_reg ^= CPU_SND_REQ;

  /* wait for ack in chanoff */
  loop = 0;
  ok = false;
  while ( ++loop < 100000 )
  {
    if ( icpo->ssp.cout_status & 0x10 )
      events = icpo->ssp.cout_events_b;
    else
      events = icpo->ssp.cout_events_a;
    if ( events & 0x01 )
    {
      ok = true;
      break;
    }
  }
  if ( !ok )
#ifdef UNIX
    cmn_err( CE_WARN, "SST: cpu_req ack failed (chanoff)." );
#else
    printk("WARNING: SST: cpu_req ack failed (chanoff).\n" );
#endif

  cin = icpi->ssp.cin_locks;
  icpi->ssp.cin_locks = cin | 0x03;
  cur_chnl_sync( mpc );
  icpi->ssp.cin_bank_a.ssp.bank_fifo_lvl &= ~0x8f;
  icpi->ssp.cin_bank_b.ssp.bank_fifo_lvl &= ~0x8f;
  icpi->ssp.cin_tail_ptr_a = 
  icpi->ssp.cin_tail_ptr_b =  
  mpc->mpc_rxq.q_ptr = rx_next;
  if(mpc->mpc_mpd->mpd_nwin)
     mpc->mpc_rxq.q_ptr &= 0x3fff;
  rx_vmin = 1;
  /* unlock input */
  icpi->ssp.cin_attn_ena |= /*CIN_DEF_ENABLES*/ 0x2800; /* To handle the overrun 
							 when closed
							*/
  icpi->ssp.cin_locks = cin;
  if ( icpi->ssp.cin_inband_flow_ctrl & 0x01 )
  {
    icpi->ssp.cin_locks |= 0x08;
    cur_chnl_sync( mpc );
    icpi->ssp.cin_inband_flow_ctrl &= ~0x01;
    icpi->ssp.cin_locks &= ~0x08;
  }
  icpo->ssp.cout_attn_ena = 0x8000;  /* allow update enabled */
  mpc->mpc_flags &= ~MPC_OPEN;
mpc->mpc_cin_events = 0;
mpc->mpc_cout_events = 0;
  return(0);
}   /* chanoff */


static int cur_chnl_sync( register struct mpchan *mpc)
{
register icpgaddr_t icpg = (icpgaddr_t)mpc->mpc_icpo;
register int i = 0;
volatile unsigned char *chan_ptr;
	
	if ((mpc->mpc_mpd->mpd_id & 0xF8) < SSP2_ASIC)  /* SSP64 board */
		chan_ptr = &(Gicp_chnl);
	else { /* SSP4 board */
      		icpaddr_t icp;
      		union global_regs_u *icp_glo ;
      		icp = mpc->mpc_icp;
      		icp_glo = (union global_regs_u *)((unsigned long)icp->icp_regs_start + 0x400);
      		chan_ptr = &(icp_glo->ssp2.chan_ctr);
	}
	while(*chan_ptr == mpc->mpc_chan) { if( ++i > 9000) break; }
	return(0);
}
/****************************************************************************/        
static int mega_push_winicp( struct mpdev *mpd, int icp)
{
	int  flags = 0;
	save_flags(flags);
	cli();
        /* SSM changes, found while testing SSM */
	if(wtos) {
          /* deselect current before push, if necessary */
          if((wstk[wtos].ws_mpd->mpd_id & 0xF8) < SSP2_ASIC) 
	    outw(0,wstk[wtos].pgctrl);
           else 
             outb(0, wstk[wtos].pgctrl);
        }
        /* end changes, found SSM */

	if(++wtos >= WINDO_DEPTH) {
	   printk("win16: stk overflow - %d\n",wtos);
	   wtos--;
	   restore_flags(flags);
	   return(1);
	}
        wstk[wtos].ws_mpd = mpd;
        if ((mpd->mpd_id & 0xF8) < SSP2_ASIC) 
		outw((wstk[wtos].pgdata = (icp | 0x100)),
			(wstk[wtos].pgctrl = mpd->mpd_pgctrl));
        else
                outb((wstk[wtos].pgdata = (1 << icp)),
                	(wstk[wtos].pgctrl = mpd->mpd_pgctrl));
	restore_flags(flags);
	return(0);
}
static int mega_push_win( struct mpchan *mpc, int typ)
{
	struct mpdev *mpd = mpc->mpc_mpd;
	int  flags = 0;
	save_flags(flags);
	cli();
	if(wtos) {
          /* deselect current before push, if necessary */
          if((wstk[wtos].ws_mpd->mpd_id & 0xF8) < SSP2_ASIC) 
	    outw(0,wstk[wtos].pgctrl);
           else 
             outb(0, wstk[wtos].pgctrl);
        }
	if(++wtos >= WINDO_DEPTH) { 
	   printk("win16: stk overflow - %d\n",wtos);
	   wtos--;
	   restore_flags(flags);
	   return(1);
	}

        wstk[wtos].ws_mpd = mpd;
        if ((mpd->mpd_id & 0xF8) < SSP2_ASIC) { /* SSP64 board */
	  switch(typ) {
	    case 0:
		outw((wstk[wtos].pgdata = mpc->mpc_icpno | 0x100),
		(wstk[wtos].pgctrl = mpc->mpc_mpd->mpd_pgctrl));
		break;
	    case 1:
		outw((wstk[wtos].pgdata = mpc->mpc_rxpg | 0x300),
		(wstk[wtos].pgctrl = mpc->mpc_mpd->mpd_pgctrl));
		break;
	    case 2:
		outw((wstk[wtos].pgdata = mpc->mpc_txpg | 0x300),
		(wstk[wtos].pgctrl = mpc->mpc_mpd->mpd_pgctrl));
		break;
	    case 3:
		outw((wstk[wtos].pgdata = mpc->mpc_tgpg | 0x200),
		(wstk[wtos].pgctrl = mpc->mpc_mpd->mpd_pgctrl));
		break;
	  }
        }
        else { /* SSP4 board */
		outw((wstk[wtos].pgdata = (1 << mpc->mpc_icpno)),
		(wstk[wtos].pgctrl = mpc->mpc_mpd->mpd_pgctrl));
        }
	restore_flags(flags);
	return(0);
}

static int mega_pop_win(int instance)
{
	int  flags = 0;
if ((wtos == 0) && (last_pop)) { 
last_pop = 0;
/*
printk("wtos_max %d EXTRA POP on %d last was %d %d %d %d %d %d %d\n",wtos_max,instance,last_pop1,last_pop2,last_pop3,last_pop4,last_pop5,last_pop6,last_pop7);
*/
}
save_flags(flags);
cli();
last_pop7 = last_pop6;
last_pop6 = last_pop5;
last_pop5 = last_pop4;
last_pop4 = last_pop3;
last_pop3 = last_pop2;
last_pop2 = last_pop1;
last_pop1 = instance;
	if(wtos) {
          /* deselect current before pop, if necessary */
          if((wstk[wtos].ws_mpd->mpd_id & 0xF8) < SSP2_ASIC) 
	    outw(0,wstk[wtos].pgctrl);
           else 
             outb(0, wstk[wtos].pgctrl);
        }
	if(!(--wtos)) {
	   restore_flags(flags);
	   return(1);
	}
	if(wtos < 0) {
           /*
	   printf("win16: stk underflow - %d \n",wtos);
           printf("wtos_max %d EXTRA POP on %d last was %d %d %d %d %d %d %d\n",wtos_max,instance,last_pop1,last_pop2,last_pop3,last_pop4,last_pop5,last_pop6,last_pop7);
           */
	   wtos = 0;
	   restore_flags(flags);
	   return(1);
	}
        if ((wstk[wtos].ws_mpd->mpd_id & 0xF8) < SSP2_ASIC) 
	  outw(wstk[wtos].pgdata, wstk[wtos].pgctrl);
        else
	  outb((unsigned char)wstk[wtos].pgdata, wstk[wtos].pgctrl);
	restore_flags(flags);
	return(0);
}
	
/******************************************************************************
        
        FRAME_WAIT
        
        On "icp" wait at least "count" frames to elapse.
  
******************************************************************************/
static int frame_wait( register struct mpchan *mpc, int count)
{
icpgaddr_t icpg = (icpgaddr_t)(mpc->mpc_icpo);
/* ssp2_output_s pointer */
icpoaddr_t icpo = (icpoaddr_t)((unsigned long)(mpc->mpc_icp->icp_regs_start) + 0x200);
ushort_t final,original,current1;
volatile ushort_t *frame_ptr;
int x = 0;

  if ((mpc->mpc_mpd->mpd_id & 0xF8) < SSP2_ASIC)
    frame_ptr = &Gicp_fram_ctr;
  else {
    icpg = (icpgaddr_t)((unsigned long)(mpc->mpc_icpi) + 0x400);
    frame_ptr = &(icpo->ssp2.frame_ctr);
  }
  original = *frame_ptr;
  final = original + count;
  if ( final < (unsigned) count ) {  /* wrap */
    while (1) {
      if ( (current1 = *frame_ptr) < original && current1 > final )
        break;
      if ( ++x > 0x100000 )
      {
#ifdef UNIX
        cmn_err( CE_WARN,
  	       "SST: frame_wait error\n",
  	       0 );
#else
  	       printk("WARNING: SST: frame_wait error\n");
#endif
        break;
      }
    }
  } /* Wrap loop */
  else { /* Non-wrap */
    while (1) {
      if ( (current1 = *frame_ptr) > final || current1 < original )
        break;
      if ( ++x > 0x100000 )
      {
#ifdef UNIX
        cmn_err( CE_WARN,
  	       "SST: frame_wait error\n",
  	       0 );
#else
  	       printk("WARNING: SST: frame_wait error\n");
#endif
        break;
      }
    }
  } /* Non-wrap loop */
  return(0);
}  /* frame_wait */

/******************************************************************************
        
        FRAME_CTR_RELIABLE
  
******************************************************************************/
static int frame_ctr_reliable( icpgaddr_t icpg, register struct mpchan *mpc)
{
int ii = 0;
uchar_t w;
ushort_t x, b; 
volatile ushort_t *frame_ptr;

  if ((mpc->mpc_mpd->mpd_id & 0xF8) < SSP2_ASIC) { /* SSP64 board */
    w = Gicp_chnl;
    x = Gicp_fram_ctr;
    frame_ptr = &(Gicp_fram_ctr);
  }
  else { /* SSP4 board */
    icpgaddr_t icpg = (icpgaddr_t)((unsigned long)(mpc->mpc_icp->icp_regs_start) + 0x400);
    icpoaddr_t icpo = (icpoaddr_t)((unsigned long)(mpc->mpc_icp->icp_regs_start) + 0x200);
    w = icpg->ssp2.chan_ctr;
    x = icpo->ssp2.frame_ctr;
    frame_ptr = &(icpo->ssp2.frame_ctr);
  }
  if ( w & 0xc0 )  /* impossible channel number */
    return false;

  while (( b = *frame_ptr) == x )
    if ( ++ii > 0x10000 )
      return false;
  return true;
}  /* frame_ctr_reliable */

/*
 * Jam a character into the output queue.  If the transmitter
 * is idle, it's easy: just place the character in the output
 * queue and call megastart().  In the more difficult case,
 * we must stop the transmitter, push the character into the
 * "output next byte" register, and then restart normal output.
 */

static int megajam( register struct mpchan *mpc, char c)
{
	register icpoaddr_t icpo = mpc->mpc_icpo;
	volatile icpiaddr_t icpi = mpc->mpc_icpi;
        int ii=0;
/*printk("calling megajam for device %d\n", mpc->mpc_chan);*/
  if ( !(icpo->ssp.cout_flow_cfg & 0x02) )
    return(0);  /* may hang on second while */
	 if (!(icpi->ssp.cin_int_flags & 0x40))
		return(0);

/* SSM found testing SSM, while should not hang system ..... */
  while (( (icpo->ssp.cout_flow_cfg & 0x10) != (icpo->ssp.cout_int_flow_ctrl & 0x10)) &&
        ++ii < 100000);

  ii = 0;
  while (icpo->ssp.cout_int_flow_ctrl & 0x08 && ++ii < 100000 );  /* wait here */
  if(icpo->ssp.cout_int_flow_ctrl & 0x08)
  {
#ifdef UNIX
    cmn_err( CE_WARN,
             "SST: send flow char ack missing - char is %d",
             c );
#else
             printk("WARNING: SST: send flow char ack missing - char is %d\n", c );
#endif
    return(0);
  }
  if (!(ISRAMP(mpc))) {
  if(c == icpo->ssp.cout_xoff_1)
    icpo->ssp.cout_flow_cfg &= ~0x08;
  else
    if(c == icpo->ssp.cout_xon_1)
      icpo->ssp.cout_flow_cfg |= 0x08;
    else
      return(0);
  icpo->ssp.cout_flow_cfg ^= 0x10;
  }
    return(0);
}  

/*****************************************************************************/

/*  Routine to schedule a delay. Does not busy out CPU. */

static void eqnx_delay(int len)
{
#if DEBUG
	printk("eqnx_delay(len=%d)\n", len);
#endif
	if (len > 0) {
#if LINUX_VERSION_CODE < 0x2017f 
		current->state = TASK_INTERRUPTIBLE;
		current->timeout = jiffies + len;
		schedule();
#else
		current->state = TASK_INTERRUPTIBLE;
		schedule_timeout(len);
		current->state = TASK_RUNNING;
#endif
	}
}

/*****************************************************************************/
/*
 * This routine is called from the scheduler tqueue when the interrupt
 * routine has signalled that a hangup has occurred.  The path of
 * hangup processing is:
 *
 * 	poll routine -> (scheduler tqueue) ->
 * 	eqnx_dohangup() -> tty->hangup() -> eqnx_hangup()
 * 
 */
static void eqnx_dohangup(void *arg)
{
	struct mpchan *mpc;


	mpc = (struct mpchan *) arg;
	if (mpc == (struct mpchan *) NULL)
		return;
	if (mpc->mpc_tty == (struct tty_struct *) NULL)
		return;
#ifdef DEBUG
	printk("eqnx_dohangup: device =%x\n", 
		(unsigned int) SSTMINOR(mpc->mpc_tty->device));
#endif
	tty_hangup(mpc->mpc_tty);
}
/*
 * MEGAPOLL
 * Board polling routine.  Called once every MEGATIMEO
 * clock ticks.  Other than input control line transitions,
 * all megaport events along with input buffer not empty,
 * and output buffer empty conditions are handled here.
 */

#define WATCHDOGID 0xAA /* Timeout in 16 seconds, KEY of 0x0A */

void sstpoll(unsigned long arg)
{

	register struct mpchan *mpc;
	register icpgaddr_t icpg;
	register struct mpdev *mpd;
	icpoaddr_t icpo;
	icpiaddr_t icpi;
	icpbaddr_t icpb;
	icpaddr_t  icp;
	int i, j, ii, port, nports, win16;
	uchar_t g_attn;
	uint_t rbits, xbits, rbits0, xbits0, rbits1, xbits1;
	uchar_t rxtx_work;
	unsigned char read_poll;
	volatile icpqaddr_t icpq;

	poll_cnt++;
	if ((poll_cnt & 7) == 0)
		read_poll = 1; /* Do read side for SSP64 boards at 80 mills */
	else
		read_poll = 0;

	for (i=0; i < (int)nmegaport; i++) {	/* adapters loop */
		mpd = &meg_dev[i];
		if (mpd->mpd_alive == 0) 
			continue; /* If board is dead skip it */

		/* Process certain SSP64 boards every 40 mills */
		if (((mpd->mpd_id & 0xF8) < SSP2_ASIC) && (poll_cnt & 3)) {
			switch (mpd->mpd_id & 0xf8) {
				case 0x08:
				case 0x10:
				case 0x18:
				case 0x38:
					continue;
			} 
		}

		win16 = mpd->mpd_nwin;

		/* icps loop */
		for(j=0; j < (int)mpd->mpd_nicps; j++) {
			mpc = mpd->mpd_mpc + j * mpd->mpd_sspchan;
			if(win16)
				mega_push_winicp(mpd, j);
			icp = &mpd->icp[j];

			/* SSP64 board */
			if ((mpd->mpd_id & 0xF8) < SSP2_ASIC) {
				icpg = 
				   (icpgaddr_t)((unsigned long)mpd->icp[j].icp_regs_start
				   + 0x2000);
				g_attn = i_gicp_attn;
#ifdef DEBUG1
	printk("frame counter = %d, poll_cnt = %d\n", poll_cnt, Gicp_fram_ctr);
#endif
#ifdef WDOG
				icpg->ssp.gicp_watchdog = WATCHDOGID;
#endif

				/* ATMEL: Give the ring time to settle */
				if (icp->icp_rng_state == RNG_WAIT) {  /* ATMEL */
				    /* if we lose ring clock - start over */
				    if (g_attn & RNG_FAIL) { 
					icp->icp_rng_wait = 0;
					icp->icp_rng_last = 0;
				    }
				    /* else wait 800 milliseconds */
				    else if (icp->icp_rng_wait++ < 20) {
					if (win16)
					    mega_pop_win(28);
					continue;
				    }
				    /* call mega_rng_delta with RNG_GOOD below */
				    icp->icp_rng_wait = 0; 
				}
				/* End ATMEL: Give the ring time to settle */
				/* ATMEL: Check the ring sync for 800 mills */
				if (icp->icp_rng_state == RNG_CHK) { /* ATMEL */
				    /* if we lose ring clock, start over */
				    if (g_attn & RNG_FAIL) { 
					icp->icp_rng_wait = 0;
					icp->icp_rng_last = 0;
				    } else {
					icp->icp_rng_svcreq = mega_rng_delta(icp,
						RNG_CHK, mpd->mpd_nchan);
					if (win16)
					    mega_pop_win(28);
					continue;
				    }
				}
				/* End ATMEL: Verify ring sync for 800 mills */                      
				if((g_attn & RNG_FAIL) && !icp->icp_rng_last) {
		   			/* bit changed to on - 
					 * ring changed to "bad" */
					icp->icp_rng_svcreq = 
					   mega_rng_delta(icp, RNG_BAD, 
					   mpd->mpd_nchan);
					/* save last status */ 
					icp->icp_rng_last = g_attn & RNG_FAIL; 
				}
				else if(!(g_attn & RNG_FAIL) && 
					icp->icp_rng_last) {
					/* bit changed to off - 
					 * ring changed to "good" */

					/* ATMEL: Let the ring clock stabilize */
					if (icp->icp_rng_state == RNG_BAD) {
						icp->icp_rng_state = RNG_WAIT;
						if (win16)
							mega_pop_win(28);
						continue;
					}
					/* END ATMEL */
				
					/* save last status */
					icp->icp_rng_last = g_attn & RNG_FAIL;
					if(Gicp_initiate & RNG_CLK_ON) {
						icp->icp_rng_svcreq = 
						   mega_rng_delta(icp, RNG_GOOD,
						   mpd->mpd_nchan);
					}
					if (icp->icp_rng_state == RNG_BAD) {
						icp->icp_rng_state = RNG_WAIT;
						icp->icp_rng_wait = 0;
					}
					if (win16)
						mega_pop_win(28);
					continue;
					} else {
						/* check software reason */
          					if(!(mpc->mpc_mpa_stat & MPA_INITG_UART)) {
	   						if(icp->icp_rng_svcreq){
							  icp->icp_rng_svcreq = 
						   	   mega_rng_delta(icp, 
						   	   icp->icp_rng_svcreq, 
						   	   mpd->mpd_nchan);
							}
						} else {
          /* RAMP: When initializing a modem via ramp services, false 
             lmx condition change and control signal events can and will
             occur. They will be ignored for now. Monitor severity. */

             						MESSAGE("WARNING: Ignoring ring delta on modem", mpc->mpc_mpa_stat,mpc);
#ifdef DEBUG_RAMP
             						printk("WARNING: Ignoring ring delta on modem %d with stat = %x", mpc->mpc_chan, mpc->mpc_mpa_stat);
#endif
             						icp->icp_rng_svcreq = 0;
             						mpc->mpc_cin_events &= 
								~EV_LMX_CNG;
						 }
					}
#ifdef WDOG
					if(g_attn & WDOG_EXP) {
#ifdef UNIX
						cmn_err(CE_WARN, 
						"SST watchdog timer expired, expansion bus disabled on devs %d to %d",
							icp->icp_minor_start,icp->icp_minor_start+MAXICPCHNL-1);
#else
       printk("WARNING: SST watchdog timer expired, expansion bus disabled on devs %d to %d\n",
							icp->icp_minor_start,icp->icp_minor_start+MAXICPCHNL-1);
#endif
						/* Disable watchdog until next megapoll */
						icpg->ssp.gicp_watchdog = 0; 
	    				Gicp_initiate = 0x1e - RNG_CLK_ON;
						if(win16)
							mega_pop_win(28);
						continue; 
					}
#endif /** if WDOG **/

					if(!(g_attn & (GATTN_RX|GATTN_TX))) {
						if(win16)
							mega_pop_win(28);
						continue; 
					}

					/* Note: if brd is ssm based 12/24 don't do odd read */
#ifdef SSM
					if ((mpd->mpd_id & 0xF8) != 0x30 && 
					   (mpd->mpd_id & 0xF8) != 0x38 )
					{
#endif
						/* Skip read side processing cycles */
						/*if (!read_poll)
							g_attn &= ~GATTN_RX;*/
#ifdef SSM
					}
#endif

					/* allow clear when read */
					Gicp_initiate &= ~0x10;	
					if(g_attn & GATTN_RX) {
						rbits0 = * (uint_t *) &gicp_rx_attn[0];
					rbits1 = * (uint_t *) &gicp_rx_attn[4];
				}
				else 
					rbits0 = rbits1 = 0;

				/* ATMEL */
				if (icp->icp_rng_state != RNG_GOOD) { 
				    /* block clear when read */
				    Gicp_initiate |= 0x10; 
				    if (win16) 
					    mega_pop_win(28);
				    continue;
				}
				/* ATMEL */

				if(g_attn & GATTN_TX) {
					xbits0 = * (uint_t *) &gicp_tx_attn[0];
					xbits1 = * (uint_t *) &gicp_tx_attn[4];
				}
				else 
					xbits0 = xbits1 = 0;

				Gicp_initiate |= 0x10;	/* block clear when read */

				xbits = xbits0; 
				rbits = rbits0; 
				if (mpd->mpd_nchan < NCHAN) 
					nports = mpd->mpd_nchan; 
				else
					nports = NCHAN; 

				for(port = 0; port < nports; port++, mpc++) { /* ports loop */
					if(xbits & 1) {
						icpo = mpc->mpc_icpo;
						icpq = &icpo->ssp.cout_q0;
						TX_EVENTS(mpc->mpc_cout_events, mpc);
						if(mpc->mpc_cout_events & 
							EV_TX_EMPTY_Q0) {
	/*printk("SSP64:EMPTY_Q0 event for device %d with frame counter %d\n", 
		mpc->mpc_chan + (i * NCHAN_BRD) + (j * NCHAN), Gicp_fram_ctr);*/
#ifdef DEBUG1
	printk("SSP64:EMPTY_Q0 event for device %d\n", 
			mpc->mpc_chan + (i * NCHAN_BRD) + (j * NCHAN));
	icpq = &icpo->ssp.cout_q0;
	printk("Q data count %d\n", Q_data_count);
	printk("frame counter %d\n", Gicp_fram_ctr);
#endif
							if (Q_data_count)
							tx_cie |= (ENA_TX_EMPTY_Q0);
							else{
								mpc->mpc_flags &= ~MPC_BUSY;
								megatxint(mpc);
							}
							mpc->mpc_cout_events &= 
						   		~EV_TX_EMPTY_Q0;
						}
						else if(mpc->mpc_cout_events & 
							EV_TX_LOW_Q0) {
#ifdef DEBUG
	printk("LOW_Q0 event for device %d\n", 
			mpc->mpc_chan + (i * NCHAN_BRD) + (j * NCHAN));
	icpq = &icpo->ssp.cout_q0;
	printk("Q data count for device %d\n", Q_data_count);
	printk("frame counter for device %d\n", Gicp_fram_ctr);
	printk("q block count %d\n", icpq->ssp.q_block_count);
#endif
							if (Q_data_count >=
						(icpq->ssp.q_block_count * 64))
							tx_cie |= (ENA_TX_LOW_Q0);
							else
								megatxint(mpc);
							mpc->mpc_cout_events &= 
						   	~EV_TX_LOW_Q0;
						}
						if(mpc->mpc_cout_events & 
							EV_OUT_TMR_EXP) {
							mpc->mpc_cout_events &= 
						   	~EV_OUT_TMR_EXP;
						}
					}		
if ((mpc->mpc_mpa_stat & (MPA_INITG_INIT_MODEM | MPA_INITG_MODIFY_SETTINGS |
MPA_INITG_UART ))
&& (rbits & 1))
{
        icpi = mpc->mpc_icpi;
        icpb = (rx_locks & LOCK_A) ? &icpi->ssp.cin_bank_b : &icpi->ssp.cin_bank_a;
        cur_chnl_sync(mpc);
        rx_events &= ~CHAN_ATTN_SET;
        cur_chnl_sync(mpc);
        rx_events &= ~CHAN_ATTN_SET;
} else
					if(rbits & 1) {
						icpi = mpc->mpc_icpi;
						icpb = (rx_locks & LOCK_A) ? &icpi->ssp.cin_bank_b : &icpi->ssp.cin_bank_a;
						FREEZ_BANK(mpc);
#ifdef DEBUG_RAMP
printk("channel %d:%d -  bank sigs = %x mpa_stat = %x cin_events = %x cin_attn_ena = %x\n", j,mpc->mpc_chan,icpb->ssp.bank_sigs,mpc->mpc_mpa_stat,mpc->mpc_cin_events,icpi->ssp.cin_attn_ena);
#endif
					   if (!(Gicp_initiate & RNG_CLK_ON)) 
							mpc->mpc_cin_events &= ~EV_LMX_CNG;
					if ((!(icpb->ssp.bank_sigs & 0x80)) &&
					  (icp->lmx[mpc->mpc_lmxno].lmx_active != DEV_GOOD))
							mpc->mpc_cin_events &= ~EV_LMX_CNG;
					if (icpb->ssp.bank_sigs & 0x20)
							mpc->mpc_cin_events &= ~EV_LMX_CNG;
						/*if((!(port % 16) && 
					   	(mpc->mpc_cin_events & EV_LMX_CNG) &&
					   	(icpi->ssp.cin_attn_ena & 
					   	EV_LMX_CNG))  &&
((!ISRAMP(mpc)) || (((mpc->mpc_mpa_stat & MPA_INITG_UART) && 
(!(icpb->ssp.bank_sigs & 0x80))) || 
((!(mpc->mpc_mpa_stat & MPA_INITG_UART)) && (icp->sspcb->pnp_probe_ena) && (!(icpb->ssp.bank_sigs & 0x80)))  || ((!(mpc->mpc_mpa_stat & MPA_INITG_UART)) && (!(icp->sspcb->pnp_probe_ena))))))*/  /** 3.30 changes **/
if(!(port % 16) &&
/*!(ISRAMP(mpc)) &&*/ (mpc->mpc_cin_events & EV_LMX_CNG) &&
                                                (icpi->ssp.cin_attn_ena &
                                                EV_LMX_CNG)){
							int lmx_flag = 1;
							mpc->mpc_cin_events &= 
						   	~EV_LMX_CNG;
							/*if (ISRAMP(mpc)) {
								lmx_flag = ramp_lmx_check(mpc);
							}*/
							if (lmx_flag) {
#ifdef DEBUG
printk("ev_lmx_chg  rng_last=%x  port=%d\n",icp->icp_rng_last,port);
#endif
							if (!(icp->lmx[mpc->mpc_lmxno].
								lmx_type & 1)) {
								icpo = mpc->mpc_icpo;
								icpo->ssp.cout_lmx_cmd =
							   	0x40; 
								frame_wait( mpc, 6 );
								icpo->ssp.cout_lmx_cmd =
							   	0;
								mega_rng_delta(icp, 
								RNG_FAIL, 
								mpd->mpd_nchan);
							} else { /* not a panel */
								if((rx_ctrl_sig & 0xe0)
							   	== 0x80) { 
									/* bridge 
								 	* offline */
									/* ensure active
								 	* bank valid */
									frame_wait(mpc,
									2); 
									mega_rng_delta(
									icp, 
									RNG_FAIL, 
									mpd->mpd_nchan);
								} else {  
									if(icp->
									icp_rng_last & 
									RNG_FAIL)
										mega_rdv_delta(icp, mpc, RNG_BAD);
									else
										mega_rdv_delta(icp, mpc, RNG_GOOD);
								}  
							}	/* not a panel */
							}
					
						}
						else
							mpc->mpc_cin_events &= 
						   	~EV_LMX_CNG;
						if(mpc->mpc_cin_events & EV_REG_UPDT) {
							mpc->mpc_cin_events &= 
						   	~EV_REG_UPDT;
							rx_cie &= ~ENA_REG_UPDT;
						}
						if(mpc->mpc_cin_events & 
							(EV_BREAK_CNG)) {
							megasint(mpc);
							mpc->mpc_cin_events &= 
						   	~EV_BREAK_CNG;
						}
						if(mpc->mpc_cin_events & EV_OVERRUN) {
	     						/*printk("overrun for device %d\n", mpc->mpc_chan + (i * NCHAN_BRD) + (j * NCHAN));*/
							mpc->mpc_cin_events &= ~EV_OVERRUN;
						}
						if(mpc->mpc_cin_events & EV_VMIN) {
	     						/*if((mpc->mpc_chan + (i * NCHAN_BRD) + (j * NCHAN)) == 16)
	printk("frame counter on input with poll_cnt = %d, %d\n", poll_cnt, Gicp_fram_ctr);*/
	     						megainput(mpc);
							mpc->mpc_cin_events &= ~EV_VMIN;
						}
						if(mpc->mpc_cin_events & EV_VTIME) {
	/*printk("frame counter on EV_TIME, poll_cnt = %d, %d\n", poll_cnt, Gicp_fram_ctr);*/
							/*megatime(mpc);*/
							mpc->mpc_cin_events &= 
						   	~EV_VTIME;
						}
						if(mpc->mpc_cin_events & EV_DCD_CNG) {
							megamint(mpc);
							mpc->mpc_cin_events &= 
						   	~EV_DCD_CNG;
						}
/*
rx_events = mpc->mpc_cin_events;
*/
					}	/* rx event per port */
					if (!rbits && !xbits ) {
						if (port <= 31) {
							mpc += (31 - port);
							port = 31;
						}
						else
							break;
					}
					if(port == 31) {
						rbits = rbits1;
						xbits = xbits1;
						if (!rbits && !xbits )
							break;
					} else {
						rbits = rbits >> 1;
						xbits = xbits >> 1;
					}		
				}		/* ports loop */
			} else {

				/* SSP4 board */
				union global_regs_u *icp_glo =
				   (union global_regs_u *)((unsigned long)mpd->icp[j].icp_regs_start + 0x400);
				g_attn = icp_glo->ssp2.attn_pend;
				/*icpo = mpc->mpc_icpo;
	printk("frame counter on input before FREEZ with poll_cnt = %d, %d\n", poll_cnt, icpo->ssp2.frame_ctr);*/
				if(g_attn & 0x01 || icp->icp_poll_defer) { /* check global attn bit */
					icp->icp_poll_defer = 0;
        			icp_glo->ssp2.bus_cntrl &= ~0x10;/* allow clear when read */
        			rxtx_work = icp_glo->ssp2.chan_attn;

        			icp_glo->ssp2.bus_cntrl |= 0x10; /* block clear when read */
        
        			for ( ii = 0; ii < 4; ii++,mpc++ )  /* DEBUG: testing 4 chnls for 2 and 4 */
        			{

            		if ( rxtx_work & 0x01 )
            		{    /* TX WORK */
				icpo = mpc->mpc_icpo;
				icpq = &icpo->ssp.cout_q0;
				TX_EVENTS(mpc->mpc_cout_events, mpc);
				if(mpc->mpc_cout_events & EV_TX_EMPTY_Q0) {
#ifdef DEBUG1
	printk("EMPTY_Q0 event for device %d\n", 
	     		mpc->mpc_chan + (i * NCHAN_BRD) + (j * NCHAN));
	/*icpq = &icpo->ssp.cout_q0;
	printk("Q data count %d\n", Q_data_count);
	printk("frame counter %d\n", icpo->ssp2.frame_ctr);
	printk("q block count %d\n", icpq->ssp.q_block_count);
	printk("SSP4:EMPTY_Q0 event for device %d with frame counter %d\n", 
		mpc->mpc_chan + (i * NCHAN_BRD) + (j * NCHAN), icpo->ssp2.frame_ctr);*/
#endif
					if (Q_data_count)
						tx_cie |= (ENA_TX_EMPTY_Q0);
					else{
						if (!mpc->xmit_cnt)
							mpc->mpc_flags 
								&= ~MPC_BUSY;
						megatxint(mpc);
					}
					mpc->mpc_cout_events &= 
				   		~EV_TX_EMPTY_Q0;
				}
				else if(mpc->mpc_cout_events & EV_TX_LOW_Q0) {
#ifdef DEBUG
	printk("LOW_Q0 event for device %d\n", 
	     		mpc->mpc_chan + (i * NCHAN_BRD) + (j * NCHAN));
	icpq = &icpo->ssp.cout_q0;
	printk("Q data count for device %d\n", Q_data_count);
	printk("frame counter for device %d\n", icpo->ssp2.frame_ctr);
	printk("q block count %d\n", icpq->ssp.q_block_count);
#endif
					if (Q_data_count >=
						(icpq->ssp.q_block_count * 64))
						tx_cie |= (ENA_TX_LOW_Q0);
					else
						megatxint(mpc);
					mpc->mpc_cout_events &= 
					   	~EV_TX_LOW_Q0;
				}
				if(mpc->mpc_cout_events & EV_OUT_TMR_EXP) {
	   				mpc->mpc_cout_events &= ~EV_OUT_TMR_EXP;
				}
				icpo->ssp.cout_attn_ena |= 0x8000;	
            		}  /* TX WORK */

            		rxtx_work = rxtx_work >> 1;   /* prepare next channel */

			if ( (rxtx_work & 0x08) || (mpc->mpc_flags & MPC_DEFER) ) { 
/* Defer processing to every 40 mills (<= 115Kb)*/
/* Defer processing to every 20 mills (== 230Kb)*/
if (mpc->mpc_tty != (struct tty_struct *) NULL){
  	unsigned int baud = mpc->mpc_tty->termios->c_cflag & CBAUD;
	if (baud == B38400){
		if ((mpc->flags & ASYNC_SPD_MASK) == ASYNC_SPD_HI)
			baud = B57600;
		else if ((mpc->flags & ASYNC_SPD_MASK) == ASYNC_SPD_VHI)
			baud = B115200;
	}
	if ((baud < B230400) && (poll_cnt & 3)) {
		icp->icp_poll_defer = 1;
		mpc->mpc_flags |= MPC_DEFER; 
		continue;
	}
	if ((baud == B230400) && (poll_cnt & 1)) {
		icp->icp_poll_defer = 1;
		mpc->mpc_flags |= MPC_DEFER; 
		continue;
	}
}
else
	if (poll_cnt & 3) { 
		icp->icp_poll_defer = 1;
		mpc->mpc_flags |= MPC_DEFER; 
		continue;
	}
mpc->mpc_flags &= ~MPC_DEFER; 
/*TEMP DEBUG */
				/*icpo = mpc->mpc_icpo;
	printk("frame counter on input before FREEZ with poll_cnt = %d, %d\n", poll_cnt, icpo->ssp2.frame_ctr);*/
	  				icpi = mpc->mpc_icpi;
               				FREEZ_BANK(mpc);
 	     				mpc->mpc_cin_events &= ~EV_LMX_CNG;
	  				if(mpc->mpc_cin_events & EV_REG_UPDT) {
	     					mpc->mpc_cin_events &= ~EV_REG_UPDT;
	     					rx_cie &= ~ENA_REG_UPDT;
	  				}
	  				if(mpc->mpc_cin_events & EV_VMIN) {
#ifdef DEBUG
				icpo = mpc->mpc_icpo;
	printk("frame counter on input with poll_cnt = %d, %d\n", poll_cnt, icpo->ssp2.frame_ctr);
#endif
	     					megainput(mpc);
	     					mpc->mpc_cin_events &= ~EV_VMIN;
	  				}
	  				if(mpc->mpc_cin_events & EV_VTIME) {
	     					/*megatime(mpc);*/
	     					mpc->mpc_cin_events &= ~EV_VTIME;
	  				}
	  				if(mpc->mpc_cin_events & (EV_BREAK_CNG|EV_CHAR_LOOKUP)) {
	     					megasint(mpc);
	     					mpc->mpc_cin_events &= ~EV_BREAK_CNG;
	  				}
	  				if(mpc->mpc_cin_events & EV_DCD_CNG) {
	     					megamint(mpc);
	     					mpc->mpc_cin_events &= ~EV_DCD_CNG;
	  				}
				icpi->ssp.cin_attn_ena |= 0x8000;	
            		}      /* RX WORK */
        			} /* for(ii = 0; ii < 4; ii++) */
				} /* Global attention bit set */
  			} /* SSP2/4 rx/tx work */
			if(win16)
	   			mega_pop_win(29);
		}		/* icps loop */
	}			/* adapters loop */

	/*restore_flags(flags);*/
	eqnx_timer.expires = jiffies + MPTIMEO;
	add_timer(&eqnx_timer);
}
 /************************************************************************** 
        Handle a change in the ring pointed to by "icp".  "reason" may be
        
                RNG_BAD         status just went to bad
                RNG_GOOD        status just went to good (includes init time)
                RNG_WAIT        debounce of RNG_CLK in progress
		RNG_CHK         service of a transistion to RNG_GOOD in progress
                                
        Initialization code relies on this state machine to establish the
        ring as well as ring failure detection and re-initialization.
        This routine is responsible for "local" lmx's only!!!
        
        Return the new value for "rng_reqsvc" to indicate if this routine
        should be run next entry of megapoll() without any change in the
        ring status.

        An inherent assumption in ring processing is that ring processing
        has precedence over lmx processing.  This implies:
                1) changes in local lmx's will be prompted by loss of ring.
                2) lmx condition change events will be due to mux
                  changes (not local lmx changes)

******************************************************************************/

static int mega_rng_delta( icpaddr_t icp, int reason, int nchan)
{
register icpbaddr_t icpb;
register icpiaddr_t icpi;
register icpoaddr_t icpo;
icpgaddr_t icpg;
int chnl, lx;
int frames_ok = 1;
struct mpchan *mpc;
struct lmx_struct *lmx;
struct slot_struct *slot_ptr;
int ldv_chng = false;
unsigned int addr;
unsigned int bit_test;

  switch ( reason )
  {
    case RNG_FAIL:
	icp->icp_rng_last = RNG_FAIL;

    case RNG_BAD:
	if(icp->icp_rng_state == RNG_BAD)
	   break;	
	if( !eqn_quiet_mode && !EQNft )

#ifdef UNIX
      cmn_err( CE_WARN, 
               "SST expansion bus failure detected for minor devices %d to %d.",
               icp->icp_minor_start, icp->icp_minor_start + MAXICPCHNL - 1 );
#else
               printk("WARNING: SST expansion bus failure detected for minor devices %d to %d\n",
               icp->icp_minor_start, icp->icp_minor_start + MAXICPCHNL - 1 );
#endif
/* RAMP START -------------------------------------------------- */
      for(lx = 0; lx < 4; lx++)
      {
	  lmx = &icp->lmx[lx];
          if((lmx->lmx_id == 0x08) ||
               (lmx->lmx_id == 0x09) ||
               (lmx->lmx_id == 0x0B) )
          {
           int i, fret;
           mpc = lmx->lmx_mpc;
	      if((fret = mpa_srvc_dereg_mpa(icp->sspcb,(unsigned char)(lx * 16),
                 &slot_ptr)))
	       {
                  MESSAGE("ERROR: dereg_mpa", fret, mpc);
	       }
	       else	/* success: place engines online */	
	       { 	
                  MESSAGE("SUCCESS: dereg_mpa", 
                     icp->sspcb->mpa[lx].mpa_num_engs, mpc);
	       }
            if (eqn_num_ramps > 0)
               eqn_num_ramps--;
            if (eqn_num_ramps == 0) {
               if ( to_eqn_ramp_admin != -1 ) {
		       if (timer_pending(&eqnx_ramp_timer))
				del_timer(&eqnx_ramp_timer);
	       }

               to_eqn_ramp_admin = -1;
               /* DEBUG
               printf("mega_rng_delta: No modem pool adapters left. Untimeout ramp_admin\n");
               */
            }
            for(i = 0; i < lmx->lmx_chan; i++, mpc++) {
               ramp_check_error(mpc, fret);
               ramp_dereg_modem_cleanup(mpc);
            }
          }
      }
/* RAMP END ---------------------------------------------------- */
      /* the icp should have flow controlled channel data */
      /* set state of ldev's and any rdev's to bad */
      icp->icp_rng_fail_count++;
      icp->icp_rng_state = RNG_BAD;
      icp->lmx[0].lmx_active = DEV_BAD;
      icp->lmx[1].lmx_active = DEV_BAD;
      icp->lmx[2].lmx_active = DEV_BAD;
      icp->lmx[3].lmx_active = DEV_BAD;
      /* allow LMX_CNG events in RNG_BAD state */
      /*icp->lmx[0].lmx_mpc->mpc_icpi->ssp.cin_attn_ena &= ~0x40;
      icp->lmx[1].lmx_mpc->mpc_icpi->ssp.cin_attn_ena &= ~0x40;
      icp->lmx[2].lmx_mpc->mpc_icpi->ssp.cin_attn_ena &= ~0x40;
      icp->lmx[3].lmx_mpc->mpc_icpi->ssp.cin_attn_ena &= ~0x40;*/
      icp->lmx[0].lmx_rmt_active = DEV_BAD;
      icp->lmx[1].lmx_rmt_active = DEV_BAD;
      icp->lmx[2].lmx_rmt_active = DEV_BAD;
      icp->lmx[3].lmx_rmt_active = DEV_BAD;
      break;

    case RNG_GOOD:
	if(icp->icp_rng_state == RNG_GOOD)  {
	  break;	
	}
      /* this occurs as a normal part of initialization */
      /* loop thru all possible lmx's - check chnls 0, 16, 24, 48 */
      /* hold processor here while local lmx's are init'd */
      /* setup appropriate state flags so that remote lmx configuration
        takes place during "lmx_cond_changed" processing */

	if(icp->icp_rng_good_count && !eqn_quiet_mode )
        /* no message on initial find */
#ifdef UNIX
        cmn_err( CE_NOTE, 
                 "SST expansion bus established for minor devices %d to %d.",
                 icp->icp_minor_start, icp->icp_minor_start + MAXICPCHNL - 1 );
#else
                 printk("NOTICE: SST expansion bus established for minor devices %d to %d\n",
                 icp->icp_minor_start, icp->icp_minor_start + MAXICPCHNL - 1 );
#endif
	icp->icp_rng_state = RNG_CHK;
	lmx = &icp->lmx[0];
        mpc = lmx->lmx_mpc;
	icpo = mpc->mpc_icpo;
	/* tested in "for" loop */
      frames_ok = frame_ctr_reliable( (icpgaddr_t ) icpo, mpc );  
      for ( chnl = 0, lx = 0; chnl < MIN(MAXICPCHNL, nchan); chnl += 16, lx++ )
      {
      int prod_id;
      int retv, ramp, ramp_id, lmx_speed;
      int ami_interface;
      int no_lmx;
      int mario; /* ATMEL FIX */

	lmx = &icp->lmx[lx];
        mpc = lmx->lmx_mpc;
	icpi = mpc->mpc_icpi;
	icpo = mpc->mpc_icpo;

          icpo->ssp.cout_cpu_req |= 0x80; /* Pause output */  
          frame_wait( mpc, 3 );
          icpo->ssp.cout_lmx_cmd = 0xc0; /* make sure we are offline */  
          frame_wait( mpc, 6 );
          icpo->ssp.cout_lmx_cmd = 0x40; /* make sure we are offline */  
          frame_wait( mpc, 6 );
          icpo->ssp.cout_lmx_cmd &= ~0x40;
          icpo->ssp.cout_cpu_req &= ~0x80; /* Resume output */  
          frame_wait( mpc, 3 );

	icpb = (rx_locks & LOCK_A) ? &icpi->ssp.cin_bank_b : &icpi->ssp.cin_bank_a;
	if ((mpc->mpc_mpd->mpd_id & 0xF8) < SSP2_ASIC)
		icpg = (icpgaddr_t) icpo;
	else
		icpg = (icpgaddr_t)((unsigned long)(mpc->mpc_icpi) + 0x400);
/* Fix ATMEL State Machine Problem 4.01 */
for (mario=0; mario < 16; mario++)
{
	bit_test = (0x5a5a ^ ((16*lx) + mario)) & ~0x3;
	(mpc + mario)->mpc_icpi->ssp.cin_locks |= LOCK_A;
	(mpc + mario)->mpc_icpi->ssp.cin_locks &= ~LOCK_B;
	cur_chnl_sync(mpc); /* mpc is sufficiently initialized */
	(mpc + mario)->mpc_icpi->ssp.cin_bank_a.ssp.bank_fifo_lvl &= ~0x8f;
	(mpc + mario)->mpc_icpi->ssp.cin_bank_a.ssp.bank_dma_ptr = bit_test;
	(mpc + mario)->mpc_icpi->ssp.cin_locks |= LOCK_B;
	(mpc + mario)->mpc_icpi->ssp.cin_locks &= ~LOCK_A;
	cur_chnl_sync(mpc); /* mpc is sufficiently initialized */
	(mpc + mario)->mpc_icpi->ssp.cin_bank_b.ssp.bank_fifo_lvl &= ~0x8f;
	(mpc + mario)->mpc_icpi->ssp.cin_bank_b.ssp.bank_dma_ptr = bit_test;
	(mpc + mario)->mpc_icpi->ssp.cin_tail_ptr_a = bit_test;
	(mpc + mario)->mpc_icpi->ssp.cin_tail_ptr_b = bit_test;
}
frame_wait( mpc, 1 );
for (mario=0; mario < 16; mario++)
{
   bit_test = (0x5a5a ^ ((16*lx) + mario)) & ~0x3;
   if (((mpc+mario)->mpc_icpi->ssp.cin_tail_ptr_a == bit_test) &&
       ((mpc+mario)->mpc_icpi->ssp.cin_tail_ptr_b == bit_test) &&
       ((mpc+mario)->mpc_icpi->ssp.cin_bank_a.ssp.bank_dma_ptr == bit_test) &&
       ((mpc+mario)->mpc_icpi->ssp.cin_bank_b.ssp.bank_dma_ptr == bit_test))
   {
       continue;
   }
   else
   {
      Gicp_initiate = 0x1c;
      frame_wait( mpc, 1 );
      Gicp_initiate = 0x1e;
      frame_wait( mpc, 1 );
      icp->icp_rng_last = RNG_FAIL;
      icp->icp_rng_state = RNG_BAD;
      return 0;
   }
}
/* End ATMEL fix part 1 */

/* Fix ATMEL State Machine Problem 4.02 */
for (mario=0; mario < 16; mario++)
{
   int addr;
   struct mpchan *mpc1;
   icpiaddr_t icpi;
   int j; /* 3.49 */
   unsigned char *ptr; /* 3.49 */
   mpc1 = mpc + mario;
   icpi = mpc1->mpc_icpi;

   Gicp_initiate = 0x1A; 
   frame_wait( mpc1, 1 );
   /* 3.49 */
   ptr = (unsigned char *) icpi;
   for (j=0x40; j<0x80; j++)
	ptr[j] = 0;
   /* 3.49 */
   icpi->ssp.cin_bank_a.ssp.bank_fifo_lvl &= ~0x8f;
   icpi->ssp.cin_bank_b.ssp.bank_fifo_lvl &= ~0x8f;
/**** New 3.49 ATMEL fix for both Flat and Paged Mode ****/
   addr = (mpc1->mpc_rxpg * 0x4000) + mpc1->mpc_rxq.q_begin;
   mpc1->mpc_rxq.q_ptr = mpc1->mpc_rxq.q_begin;
   icpi->ssp.cin_bank_a.ssp.bank_dma_ptr
      = icpi->ssp.cin_bank_b.ssp.bank_dma_ptr
      = icpi->ssp.cin_tail_ptr_a 
      = icpi->ssp.cin_tail_ptr_b 
      = addr & 0xffff;
/**** End New 3.49 ****/
   icpi->ssp.cin_inband_flow_ctrl = 0;  /* calibrate */
   cur_chnl_sync(mpc1);
   Gicp_initiate = 0x1e;
   frame_wait( mpc1, 1 );
   FREEZ_BANK(mpc1);
   FREEZ_BANK(mpc1);
   mpc1->mpc_cin_events = 0;
}
/* End ATMEL fix part 2 */

frame_wait( mpc, 1 );
if(i_gicp_attn & RNG_FAIL) {
   icp->icp_rng_last = RNG_FAIL;
   icp->icp_rng_state = RNG_BAD;
   return 0;
}
/* End ATMEL fix part 1 */

icpb = (rx_locks & LOCK_A) ? &icpi->ssp.cin_bank_b : &icpi->ssp.cin_bank_a;
	/* new 2.03a */
	if(rx_ctrl_sig & 0x20)  { /* Not offline, try again next megapoll */
		icp->icp_rng_last = RNG_FAIL;
		icp->icp_rng_state = RNG_BAD;
		break;
	}
	/* end 2.03a */
	if((rx_ctrl_sig & 0x80) && frames_ok ) {
          /* lmx present at this position */
	  /* GDC: Changed to and with 0x3f instead of 0x0f */
          prod_id = (unsigned)((mpc +1)->mpc_icpi->ssp.cin_tdm_early & 0x3f) >> 1;
/* RAMP START ---------------------------------------------- */
	  ramp_id =
           (unsigned)((mpc+1)->mpc_icpi->ssp.cin_tdm_early & 0xffff); 
	  ramp_id = ((ramp_id  >> 1) & 0x3f);

	  if(ramp_id == 0x08 || ramp_id == 0x09 || ramp_id == 0x0B)
	      ramp = true;
	  else
	      ramp = false;
	 
	  if(ramp)
	  {    /* clear in_use field */
	      int ii;
              for(ii = 0; ii < 16; ii++)
	         (icp->slot_ctrl_blk + chnl + ii)->in_use = 0x00; 

	         if((retv = mpa_srvc_reg_mpa(icp->sspcb, 
				  (unsigned char)chnl,
				  (icp->slot_ctrl_blk + chnl),
				  (struct mpa_input_s *)(mpc->mpc_icpi))) )
	          {

#ifdef DEBUG_RAMP
                printk("ERROR reg_mpa: ret = %d, chan = %d\n", 
			retv, mpc->mpc_chan);
#endif
               MESSAGE("ERROR reg_mpa:",retv,mpc);
	          }
	          else	/* success: place engines online */	
	          { 	
#ifdef DEBUG_RAMP
                printk("SUCCESS reg_mpa: ret = %d, chan = %d\n", 
			retv, mpc->mpc_chan);
#endif
                MESSAGE("SUCCESS reg_mpa:", retv, mpc);
  			if ( to_eqn_ramp_admin != -1 ) {
				if (timer_pending(&eqnx_ramp_timer))
					del_timer(&eqnx_ramp_timer);
			}
			to_eqn_ramp_admin = 1;
			
			/*
	 		* Set up the timer channel.  
	 		*/
			init_timer(&eqnx_ramp_timer);
			eqnx_ramp_timer.expires = jiffies + 210;
			eqnx_ramp_timer.data = 0;
			eqnx_ramp_timer.function = &eqn_ramp_admin;
			add_timer(&eqnx_ramp_timer);
                        retv = 0;
	          }
                  eqn_num_ramps++;
                  ramp_admin_poll = RAMP_FAST_TIME;
	  }		  	   
/* RAMP END ------------------------------------------------ */
          if(lmx->lmx_active == DEV_BAD && prod_id != lmx->lmx_id)
          {
            /* ring configuration has changed */
	    ldv_chng = true;
            mega_ldv_hangup( icp );
	    icp->icp_active = DEV_VIRGIN;
            lmx->lmx_id = prod_id;
            icpo->ssp.cout_lmx_cmd &= ~0x90;
          }
            
          /* clear/set scan speed */
          lmx_speed = (unsigned)(icpi->ssp.cin_tdm_early & 0xc0) >> 6;
	  if (!ramp) {
          Gicp_scan_spd &= ~(0x03 << (lx * 2));
          Gicp_scan_spd |= lmx_speed << (lx * 2);
	  }

          /* put local lmx online
               - set loopback in case external device is streaming data 
               - guarantees supervisory frames */
/* online, internal loopback, lock lmx, cmd enable */
		/* lock disabled 04/01/98 */
          icpo->ssp.cout_lmx_cmd = 0x49; 
          frame_wait( mpc, 6 );
          /* use bank_sigs - can't use cin_tdm_early here! */
          ami_interface = rx_ctrl_sig & 0x2000;
          no_lmx = !( rx_ctrl_sig & 0x20 );  /* lmx_online */
          /* clear cmd enable and loopback */
          icpo->ssp.cout_lmx_cmd &= ~0x41;
          if ( !no_lmx )
          {
            /* lmx went online as expected */
            lmx->lmx_speed = lmx_speed;
	    if(!ramp)
            lmx->lmx_id = prod_id;
	    else
		lmx->lmx_id = ramp_id;

/* if we are the secondary icp, tell the icp to say so */
            if ( rx_ctrl_sig & 0x200 )  
               Gicp_initiate |= 1;
            else
               Gicp_initiate &= ~1;
		
/* Fix ATMEL State Machine Problem 3.48 */
for (mario=0; mario < 16; mario++)
{
   struct mpchan *mpc1;
   icpiaddr_t icpi;
   int	j;		/* 4.02 */
   unsigned char *ptr;	/* 4.02 */
   mpc1 = mpc + mario;
   /* Restore Loopback if enabled */
   if ((mpc1->mpc_flags & MPC_LOOPBACK) && (!ramp)) {
      /* int/ext loopback, command enable */
      mpc1->mpc_cout->ssp.cout_lmx_cmd |= 0x43;
      frame_wait( mpc1, 6 );
      /* clear cmd enable */
      mpc1->mpc_cout->ssp.cout_lmx_cmd &= ~0x40;
   }
   icpi = mpc1->mpc_icpi;
   Gicp_initiate = 0x1A;
   frame_wait( mpc1, 1 );
   /* 4.02 */
   ptr = (unsigned char *) icpi;
   for (j=0x40; j<=0x80; j++)
	   ptr[j] = 0;
   /* 4.02 */
   
   icpi->ssp.cin_bank_a.ssp.bank_fifo_lvl &= ~0x8f;
   icpi->ssp.cin_bank_b.ssp.bank_fifo_lvl &= ~0x8f;

   addr = (mpc1->mpc_rxpg * 0x4000) + mpc1->mpc_rxq.q_begin;
   mpc1->mpc_rxq.q_ptr = mpc1->mpc_rxq.q_begin;
   icpi->ssp.cin_bank_a.ssp.bank_dma_ptr
         = icpi->ssp.cin_bank_b.ssp.bank_dma_ptr
         = icpi->ssp.cin_tail_ptr_a 
         = icpi->ssp.cin_tail_ptr_b 
         = (addr & 0xffff);

   icpi->ssp.cin_inband_flow_ctrl = 0;  /* calibrate */
   cur_chnl_sync(mpc1);
   Gicp_initiate = 0x1e;
   frame_wait( mpc1, 1 );
   FREEZ_BANK(mpc1);
   FREEZ_BANK(mpc1);
   mpc1->mpc_cin_events = 0;
}
/* End ATMEL fix part 2 */

            if ( ami_interface )
            {
              lmx->lmx_active = DEV_GOOD;
              lmx->lmx_type |= 1; /* indicate a bridge */
              /* set local lmx transparent */
              icpo->ssp.cout_lmx_cmd = 0x78;  /* online, transparent, lock lmx,
                                                               cmd enable */
              frame_wait( mpc, 6 );
              /* clear cmd enable */
              icpo->ssp.cout_lmx_cmd &= ~0x40;
		lmx->lmx_chan = 16; /* remotes are all 16 at this time */
	      lmx->lmx_rmt_active = DEV_VIRGIN;
              /* remainder of processing takes place under "lmx cond changed"
                 processing in mega_rdv_delta() */
                 
              /* unmask "lmx condition changed" so processing will continue */
              icpi->ssp.cin_attn_ena |= 0x40;
            }  /* ami_interface */
            else
            {
              /* no ami_interface - panel present */
              lmx->lmx_type &= ~1; /* indicate not a bridge */
              /* configure ldv record for panel */

	      switch ( lmx->lmx_id )
	      {
		case 3:  /* software selectable speed multiplier */
		  switch( mpc->mpc_mpd->mpd_id & 0xf8 )
		  {
		    case 0x18:
                      lmx->lmx_chan = 8;
		      break;
		    case 0x20:
                      lmx->lmx_chan = 4;
		      break;
		    default:  /* shouldn't happen */
                      lmx->lmx_chan = MAXLMXMPC;
		      break;
		  }
		  break;
		case 4:
                  lmx->lmx_chan = 8;
		  break;
		case 5:
                  lmx->lmx_chan = 12;
		  break;
		case 6:
                  lmx->lmx_chan = 8;
		  break;
		case 7:
                  lmx->lmx_chan = 16;
		  break;
		case 8:		/* RAMP found */
		  lmx->lmx_chan = 16;
		  break;
		case 9:
		  lmx->lmx_chan = 8;
		  break;
		case 0xB:
		  lmx->lmx_chan = 4;
		  break;
/* GDC */
		case 0x10:  /* GDC */
		  lmx->lmx_chan = 16;
		  break;
		default:
                  lmx->lmx_chan = MAXLMXMPC;
		  break;
	      }

	      if ( (lmx->lmx_chan == 8 
	           || lmx->lmx_chan == 4)
                   && (!ramp ))
	      {
		unsigned char cin_lck[16];
		unsigned char cout_lck[16];
		volatile unsigned char *chan_ptr = &(Gicp_chnl),cur_chan;
		int i=0;
                  int kk = 0;
                  /* change lmx speed for this device */
                  while (true) {
                    (icpo + kk)->ssp.cout_ctrl_sigs |= 0x100;
                    (icpo + kk + 1)->ssp.cout_ctrl_sigs &= ~0x100;
                    kk += 2;
                    if(kk >= 16) break;
                  }
                  frame_wait(mpc, 6);
		for (kk=0; kk<16; kk++) {
		  cin_lck[kk] = (mpc + kk)->mpc_icpi->ssp.cin_locks;
		  (mpc + kk)->mpc_icpi->ssp.cin_locks = 0xff;
		  cout_lck[kk] = (mpc + kk)->mpc_icpo->ssp.cout_lock_ctrl;
		  (mpc + kk)->mpc_icpo->ssp.cout_lock_ctrl = 0xff;
		}
		cur_chan = *chan_ptr;
		while(*chan_ptr == cur_chan) { if( ++i > 9000) break; }
                Gicp_scan_spd &= ~(0x03 << (lx * 2));
                Gicp_scan_spd |= (1 << (lx*2));
                lmx->lmx_speed = 1;
		i=0;
		cur_chan = *chan_ptr;
		while(*chan_ptr == cur_chan) { if( ++i > 9000) break; }
		for (kk=0; kk<16; kk++) {
			(mpc + kk)->mpc_icpi->ssp.cin_locks = cin_lck[kk];
			(mpc + kk)->mpc_icpo->ssp.cout_lock_ctrl = cout_lck[kk];
		}
#ifndef LINUX
		if ( ldv_chng )
		{
		  /* reset baudrate */
		  for ( jj = 0; jj < lmx->lmx_chan; jj++ )
                    megaparam(mpc-meg_chan);
		}
#endif
	      }
#ifndef LINUX
	      else
	      {
		if(!ramp) {
		    if ( ldv_chng )
		    {
		      /* reset baudrate */
		      for ( jj = 0; jj < 16; jj++ )
                        megaparam(mpc - meg_chan);
		    }
                }
/* RAMP START --------------------------------------------- */
		else
		   if(ldv_chng)   /* RAMP */
		    {
		      /* reset baudrate */
		      for ( jj = 0; jj < lmx->lmx_chan; jj++ )
                      {
                        (mpc + jj)->mpc_mpa_stat = 0;
/* wait until ramp_admin runs
		        ramparam(mpc);
*/
                      }
		    }
/* RAMP END ----------------------------------------------- */
			
	      }
#endif


              /* set alive */
              lmx->lmx_active = DEV_GOOD;
              /* panel ready for use */
              if ( icp->icp_rng_good_count && !eqn_quiet_mode )
	      /* no message on initial find */
#ifdef UNIX
                cmn_err( CE_NOTE,
                         "SST: minor devices %d to %d available",
                         icp->icp_minor_start + lx * MAXLDVMPC, 
                         icp->icp_minor_start + (lx + 1) * MAXLDVMPC - 1 );
#else
                         printk("NOTICE: SST: minor devices %d to %d available\n",
                         icp->icp_minor_start + lx * MAXLMXMPC, 
                         icp->icp_minor_start + (lx + 1) * MAXLMXMPC - 1 );
#endif
	      FREEZ_BANK(mpc); /* flush lmx events */
 	      mpc->mpc_cin_events = 0;
#ifdef DEBUG_RAMP
		printk("setting lmx change attention for channel %d\n", mpc->mpc_chan);
#endif
              icpi->ssp.cin_attn_ena |= 0x40;
            }
          }  /* local lmx online as expected */
          else
          {
            /* local lmx not put online as expected */
#ifdef DEBUG_RAMP
		printk("lmx for channel %d did not go online\n", mpc->mpc_chan);
#endif
            icpo->ssp.cout_lmx_cmd = 0;
            lmx->lmx_id = -1;
            if(lmx->lmx_active == DEV_BAD)
            {
              mega_ldv_hangup( icp );
              lmx->lmx_active = DEV_VIRGIN;
              lmx->lmx_id = -1;
            }
/* ATMEL */
		icp->icp_rng_last = RNG_FAIL;
		icp->icp_rng_state = RNG_BAD;
		return 0;
/* ATMEL */
          }  /* lmx not online as expected */
        }  /* local lmx present 
             and inbound early/late tdm's match 
             and frames_ok */
        else
        {
#ifdef DEBUG_RAMP
               printk("mega_rng_delta: no lmx present at this position %d\n", lx);
#endif
          /* no local lmx present
            or inbound early/late tdm's do not match 
            or !frames ok */
          if(lmx->lmx_active == DEV_BAD )
            mega_ldv_hangup( icp ); 
          lmx->lmx_active = DEV_VIRGIN;
          lmx->lmx_id = -1;
        }
      }  /* for loop - case RNG_GOOD */
	icp->icp_rng_good_count++;
      break;

      /* ATMEL - 4.02 */
      case RNG_CHK:
          {
	  int mario; /* ATMEL FIX */
	  int badptr = 0;

	  if (icp->icp_rng_wait++ > 20) { /* Do this for 800 milliseconds */
		icp->icp_rng_state = RNG_GOOD;
		icp->icp_rng_wait = 0; 
	  }

	  lmx = &icp->lmx[0];
	  mpc = lmx->lmx_mpc;
	  icpo = mpc->mpc_icpo;

	  for (mario=0; mario < 64; mario++) {
		int baddr,eaddr;
		struct mpchan *mpc1;
		icpiaddr_t icpi;
		mpc1 = mpc + mario;
		if (mpc1->mpc_icp->lmx[mpc1->mpc_lmxno].lmx_active != DEV_GOOD)
			continue;
		icpi = mpc1->mpc_icpi;
		baddr = (mpc1->mpc_rxpg * 0x4000) + mpc1->mpc_rxq.q_begin;
		eaddr = (mpc1->mpc_rxpg * 0x4000) + mpc1->mpc_rxq.q_end;
		if ((icpi->ssp.cin_bank_a.ssp.bank_dma_ptr < (baddr & 0xffff)) ||
		    (icpi->ssp.cin_bank_a.ssp.bank_dma_ptr > (eaddr & 0xffff))) {
			badptr = 1;
			break;
		}
		if ((icpi->ssp.cin_bank_b.ssp.bank_dma_ptr < (baddr & 0xffff)) ||
		    (icpi->ssp.cin_bank_b.ssp.bank_dma_ptr > (eaddr & 0xffff))) {
			badptr = 1;
			break;
		}
		if ((icpi->ssp.cin_tail_ptr_a < (baddr & 0xffff)) ||
		    (icpi->ssp.cin_tail_ptr_a > (eaddr & 0xffff))) {
			badptr = 1;
			break;
		}
		if ((icpi->ssp.cin_tail_ptr_b < (baddr & 0xffff)) ||
		    (icpi->ssp.cin_tail_ptr_b > (eaddr & 0xffff))) {
			badptr = 1;
			break;
		}
	  }

	  if (badptr == 1) { /* Start Over */
	  	if ((mpc->mpc_mpd->mpd_id & 0xF8) < SSP2_ASIC)
			icpg = (icpgaddr_t) icpo;
	  	else
			icpg = (icpgaddr_t)((unsigned long)(mpc->mpc_icpi) + 
					0x400);
		Gicp_initiate = 0x1c;
		frame_wait( mpc, 1 );
		Gicp_initiate = 0x1e;
		frame_wait( mpc, 1 );
		icp->icp_rng_last = RNG_FAIL;
		icp->icp_rng_state = RNG_BAD;
		return 0;
	  }
	  }
	break;
	/* ATMEL - 4.02 */
      
    default:
      break;
  }  /* switch reason */
  return 0;
}  /* eqn_ring_delta */

static void mega_rdv_wait( unsigned long arg)
{
	struct mpchan *mpc = (struct mpchan *) arg;
	register icpiaddr_t icpi = mpc->mpc_icpi;
	register icpaddr_t icp = mpc->mpc_icp;

	icp->lmx[mpc->mpc_lmxno].lmx_wait = -1;
	mpc->mpc_cin_events |= EV_LMX_CNG;
	if(mpc->mpc_mpd->mpd_nwin) {
	   mega_push_win(mpc, 0);
           rx_cie |= 0xc000;
	   mega_pop_win(30);
	} else {
          rx_cie |= 0xc000;
	}
}

/******************************************************************************
        
        MEGA_RDV_DELTA
  
        Handle a change in the remote lmx pointed to by channel "mp".
        This change was indicated by "lmx condition changed" bit
        in input events register, now stored in "mpc->mpc_rx_bank".  

        Only channels 0, 16, 32 and 48 arrive here!

        Remote lmx init/failure/recovery is handled by this routine.
        Local lmx (ldev) init/failure/recovery is handled by 
        "mega_ring_delta" by detecting changes in the ring.  
        
        The channel lock for "mp" is assumed held on entry.

******************************************************************************/

static void mega_rdv_delta( icpaddr_t icp, struct mpchan *mpc, int rng_state)
{
register icpiaddr_t icpi = mpc->mpc_icpi;
register icpoaddr_t icpo = mpc->mpc_icpo;
icpgaddr_t icpg = (icpgaddr_t)icpo;
register icpbaddr_t icpb;
struct lmx_struct *lmx;
int rmt_lmx_found, lmx_online;
int chnl;

  if ((mpc->mpc_mpd->mpd_id & 0xF8) >= SSP2_ASIC)
	icpg = (icpgaddr_t)((unsigned long)(mpc->mpc_icpi) + 0x400);

  if ( !(rx_tdm_early & 0x100)) 
       return;  /* not a bridge  */

  chnl = mpc->mpc_chan;
  lmx = &icp->lmx[mpc->mpc_lmxno];
  /* make sure the current bank is valid */
  	frame_wait(mpc,2); /* make sure regs are valid */ 
  icpb = (rx_locks & LOCK_A) ? &icpi->ssp.cin_bank_b : &icpi->ssp.cin_bank_a;
/*  if (!(icpb->ssp.bank_events & EV_REG_UPDT)) 
  	frame_wait(mpc,2); */ /* make sure regs are valid */ 

  if((rx_ctrl_sig & 0x60) == 0x00) {	/* bridge not on line */
      icp->icp_rng_svcreq = RNG_FAIL;
      return;
  }

  switch ( lmx->lmx_rmt_active )  /* current state */
  {
    case DEV_VIRGIN:
    case DEV_BAD:
dev_bad:
      /* "tdm's from mux" should be on in "bank_sigs" if remote present */
/* TEMP
      rmt_lmx_found = (rx_ctrl_sig & 0x00c0);
*/
      rmt_lmx_found = ((rx_tdm_early & 0x300) == 0x300); 
                                            /* tdm's from mux */
      if ( rng_state == RNG_GOOD )
	if ( rmt_lmx_found )
	{
          lmx->lmx_active = DEV_GOOD;
/*
	  if((rx_ctrl_sig & 0x60) == 0x60) {
		goto dev_good;
	  }
*/
	  /* wait for link to settle down */
          lmx->lmx_rmt_active = DEV_WAITING;
/*
          mpc->mpc_cin_events &= ~EV_REG_UPDT;
          mpc->mpc_cin_events |= EV_LMX_CNG;
          rx_cie |= 0xc000;
          to_rdv_wait = -1;
*/
          if ( lmx->lmx_wait == -1 ){
		init_timer(&lmx_wait_timer);
		lmx_wait_timer.expires = 100;
		lmx_wait_timer.data = (unsigned long) mpc;
		lmx_wait_timer.function = &mega_rdv_wait;
		add_timer(&lmx_wait_timer);
	  }
	}  
      break;

    case DEV_WAITING:
      if ( lmx->lmx_wait != -1 )
      {
	/* event occured while waiting */
	del_timer(&lmx_wait_timer);
	lmx->lmx_wait = -1;
	lmx->lmx_rmt_active = DEV_BAD;
/* TEMP */
	rmt_lmx_found = ((rx_tdm_early & 0x300) == 0x300); 
	lmx_online = rx_ctrl_sig & 0x20; 
	if (lmx_online && !rmt_lmx_found) 
		goto dev_bad;
	else
		icp->icp_rng_svcreq = RNG_FAIL;
      break;
      }
      /* wait for mux link to settle is complete */
      /* get mux id from odd channel */
        lmx->lmx_rmt_id = 
	     (unsigned)((mpc +1)->mpc_icpi->ssp.cin_tdm_early & 0x1fe) >> 1;
	lmx->lmx_rmt_type = 0;
      /* put remote lmx online */
      icpo->ssp.cout_lmx_cmd = 0xe8;  /* target mux, lock lmx,
                                                       lmx online, 
                                                       cmd enable */
      frame_wait( mpc, 6 );
      /* clear cmd enable */
      icpo->ssp.cout_lmx_cmd &= ~0x40;

      /* did it work? */
      rmt_lmx_found = rx_ctrl_sig & 0x60;  /* tdm from mux/lmx_online */
      if ( rmt_lmx_found != 0x60 )
        rmt_lmx_found = false;
      if ( rmt_lmx_found )
      {
        lmx->lmx_rmt_active = DEV_GOOD;
        /* no equalization - mux ready for use */

	/* Restore Loopback if enabled */
	if (mpc->mpc_flags & MPC_LOOPBACK) {
		/* int/ext loopback, command enable */
		mpc->mpc_cout->ssp.cout_lmx_cmd |= 0x43;
		frame_wait( mpc, 6 );
		/* clear cmd enable */
		mpc->mpc_cout->ssp.cout_lmx_cmd &= ~0x40;
	}

        if ( lmx->lmx_good_count++ && !eqn_quiet_mode ) /* initial find? */
#ifdef UNIX
          cmn_err( CE_NOTE,
                   "SST: minor devices %d to %d available on remote mux",
                   icp->icp_minor_start + chnl, 
                   icp->icp_minor_start + chnl + MAXLDVMPC - 1 );
#else
                   printk("NOTICE: SST: minor devices %d to %d available on remote mux\n",
                   icp->icp_minor_start + chnl, 
                   icp->icp_minor_start + chnl + MAXLMXMPC - 1 );
#endif
	  FREEZ_BANK(mpc); /* flush lmx events */
 	  mpc->mpc_cin_events = 0;
          icpi->ssp.cin_attn_ena |= 0x40;
      }
      else
      {
	/* wait again for link to return */
        lmx->lmx_rmt_active = DEV_BAD;
	goto dev_bad;
      }
      break;
      
    case DEV_GOOD:
      rmt_lmx_found = rx_ctrl_sig & 0x40;
      lmx_online = rx_ctrl_sig & 0x20;
                                            /* tdm's from mux and lmx online */
      if( rng_state == DEV_GOOD )
      {
	if((!rmt_lmx_found && lmx_online) || (rmt_lmx_found && !lmx_online))
	{
	  /* remote lmx has disappeared */
	  /* wait for RNG_GOOD and rmt_lmx_found */
          lmx->lmx_rmt_active = DEV_BAD;
	       if( !eqn_quiet_mode )
#ifdef UNIX
          cmn_err( CE_WARN,
                "SST: remote mux failure detected for minor devices %d to %d.",
                 icp->icp_minor_start + chnl, 
                 icp->icp_minor_start + chnl + MAXLDVMPC - 1 );
#else
                printk("WARNING: SST: remote mux failure detected for minor devices %d to %d\n",
                 icp->icp_minor_start + chnl, 
                 icp->icp_minor_start + chnl + MAXLMXMPC - 1 );
#endif
	  /* tdm's from mux and lmx offline - no second event will occur */
	  if(rmt_lmx_found && !lmx_online)
	     goto dev_bad;
	}
        else 
	{
	  /* ring failed, tear down */
	  icp->icp_rng_svcreq = RNG_FAIL;
/*TEMP icp->icp_rng_last = RNG_FAIL; */
          icpo->ssp.cout_lmx_cmd = 0xc0;
          frame_wait( mpc, 6 );
          icpo->ssp.cout_lmx_cmd = 0;
          lmx->lmx_active = DEV_BAD;
          icpo->ssp.cout_lmx_cmd = 0x40;
          frame_wait( mpc, 6 );
          icpo->ssp.cout_lmx_cmd = 0;
	}
      }
      else
      {
	/* ring is now bad - wait for ring processing */
      }
      break;

    default:
      /* ignore delta -
         wait for ring processing to set DEV_BAD or DEV_VIRGIN */
      break;
  }
}  /* mega_rdv_delta */


/******************************************************************************
        
        MEGA_LDV_HANGUP
  
        Hangup any open sessions on the passed "ldv".  If "ldv" points to
        an "rdv", hangup those sessions.
        
        This routine is called when the ring has been restored and the"ldv" 
        is no longer present or the device id has changed.
        
******************************************************************************/
static void mega_ldv_hangup( icpaddr_t icp)
{
	return;
}  /* mega_ldv_hangup */

/******************************************************************************/
static void megatxint(struct mpchan *mpc)
{
	struct tty_struct *tty = mpc->mpc_tty;

	if (tty == (struct tty_struct *) NULL)
		return;
#ifdef DEBUG
	printk("megatxint: device = %d\n", SSTMINOR(tty->device));
#endif
	if ((mpc->mpc_mpd->mpd_id & 0xF8) >= SSP2_ASIC)
		if (mpc->xmit_cnt){
			sst_write1(mpc, EQNX_TXINT);
			return;
		}

	/*mpc->mpc_flags &= ~MPC_BUSY;*/
#ifdef DEBUG
	printk("megatxint: calling write_wakeup for device = %d\n", 
		SSTMINOR(tty->device));
#endif
		if ((tty->flags & (1 << TTY_DO_WRITE_WAKEUP)) 
			&& tty->ldisc.write_wakeup)
			(tty->ldisc.write_wakeup)(tty);
		wake_up_interruptible(&tty->write_wait);
}
/******************************************************************************/
/*
 * Low speed input routine.
 */


static void megainput( register struct mpchan *mpc)
{
	register struct tty_struct  *tp = mpc->mpc_tty;
	register icpiaddr_t icpi = mpc->mpc_icpi;
	register caddr_t bp;
	uchar_t curtags,tags_l,tags_h;
	caddr_t tgp;
	icpbaddr_t icpb = mpc->mpc_icpb;
	int win16 = mpc->mpc_mpd->mpd_nwin;
	unsigned int rcv_cnt;
	int fifo_cnt, i;
	ushort dma_ptr;
	unsigned int tagp;
	char fifo[0x10];
	unsigned short err;
	struct termios *tiosp;
	icpoaddr_t icpo;
	unsigned char *cbuf, *fbuf;
	int ldisc_count = 0;
#ifdef MIDNIGHT
        unsigned char *src_addr;
        int align, bytes;
#endif
	if (mpc->mpc_tty == (struct tty_struct *) NULL)
		return;
	icpo = mpc->mpc_icpo;
#ifdef DEBUG
	printk("megainput: device %d with flip_count = %d\n", SSTMINOR(tp->device),
			tp->flip.count);
#endif
	tiosp = tp->termios;
	mpc->mpc_flags &= ~MPC_RXFLUSH;

	/*
	 * Just return if input queue is in danger of overflowing and
	 * an input delimiter has been received, or lack of system
	 * resources prevent us from copying data from board to clists.
	 */
	if (mpc->mpc_block){
#ifdef DEBUG
	printk("megainput: device %d with flip_count = %d blocked\n", 
		SSTMINOR(tp->device), tp->flip.count);
#endif
		return;
	}
	if((mpc->mpc_flags & MPC_OPEN) == 0){
		return;
	}

	rcv_cnt = (rx_next + (rx_fifo_cnt & 0xf) - mpc->mpc_rxq.q_ptr) 
			& (mpc->mpc_rxq.q_size -1);

#ifdef DEBUG
	printk("megainput: device %d with flip_count = %d rcv_cnt = %d\n", 
		SSTMINOR(tp->device), tp->flip.count, rcv_cnt);
#endif
	rcv_cnt = MIN(rcv_cnt, (2 * TTY_FLIPBUF_SIZE));

	rcv_cnt = MIN(rcv_cnt, tp->ldisc.receive_room(tp));
	/*rcv_cnt = MIN(rcv_cnt, MIN((N_TTY_BUF_SIZE - tp->read_cnt),
					(N_TTY_BUF_SIZE - tp->read_head)));*/
	if(rcv_cnt == 0) {
	/*printk("megainput: frame counter = %d, device %d with flip_count = %d rcv_cnt = %d return\n", 
		icpo->ssp2.frame_ctr, SSTMINOR(tp->device), tp->flip.count, tmp_rcv_cnt);*/
	/*printk("megainput: device %d with flip_count = %d rcv_cnt = %d return\n", 
		SSTMINOR(tp->device), tp->flip.count, tmp_rcv_cnt);*/
	   return;
	}
	cbuf = tp->flip.char_buf;
	fbuf = tp->flip.flag_buf;
	/* copy fifo's to memory */
	fifo_cnt = rx_fifo_cnt & 0xf; 
	if(fifo_cnt) {
		for(i=0; i< fifo_cnt; i++)
			fifo[i] = rx_fifo[i];
	}
	dma_ptr = rx_next;

	/* check if there was an error char */
	if ((err = (mpc->mpc_cin_events & EV_PAR_ERR))) {
		if (tiosp->c_iflag & IGNPAR) {  
			err=0;
		}
		if (!(tiosp->c_iflag & INPCK) )
			err=0;
	}
	err |= mpc->mpc_cin_events & EV_FRM_ERR; 
	mpc->mpc_cin_events &= ~(EV_PAR_ERR | EV_FRM_ERR);

	/* Only need to copy tags when there is an error */ 
	if(err && fifo_cnt) {
  		uchar_t *tagptr;
		if ((mpc->mpc_mpd->mpd_id & 0xF8) < SSP2_ASIC) 
  			tagptr = (uchar_t *) 
				mpc->mpc_mpd->icp[0].icp_tags_start;
		else
  			tagptr = (uchar_t *) 
			mpc->mpc_mpd->icp[mpc->mpc_icpno].icp_tags_start;
		tags_l = icpb->ssp.bank_tags_l;
		tags_h = icpb->ssp.bank_tags_h;
		if(win16) { 
			mega_push_win(mpc, 3);
			dma_ptr &= 0x3fff;
		}
		if ((mpc->mpc_mpd->mpd_id & 0xF8) < SSP2_ASIC) {
			tagp = rx_base << 16;
			tagp |= dma_ptr;
		}
		else
			tagp = dma_ptr;
		if(fifo_cnt > 4) {
			int h_tagp = tagp + 4;
			if(h_tagp > (ushort)(mpc->mpc_rxq.q_end -1))
				h_tagp = (ushort)mpc->mpc_rxq.q_begin;
			tagptr[h_tagp >> 2] &= ~fifo_mask[fifo_cnt];
			tagptr[h_tagp >> 2] |= tags_h & fifo_mask[fifo_cnt];
			tagptr[tagp >> 2] &= ~fifo_mask[4];
			tagptr[tagp >> 2] |= tags_l & fifo_mask[4];
		} else {
			tagptr[tagp >> 2] &= ~fifo_mask[fifo_cnt];
			tagptr[tagp >> 2] |= tags_l & fifo_mask[fifo_cnt];
		}
		if(win16) 
			mega_pop_win(53);
	}
	if(win16) {
	   mega_push_win(mpc, 1);
	   dma_ptr &= 0x3fff;
	}
	if(fifo_cnt) { 
		for(i=0; i< fifo_cnt; i++) {
			mpc->mpc_rxq.q_addr[dma_ptr] = fifo[i];
			if(dma_ptr++ > (ushort)(mpc->mpc_rxq.q_end -1)) 
				dma_ptr = (ushort)mpc->mpc_rxq.q_begin;
		}
	}	


	bp = mpc->mpc_rxq.q_addr + mpc->mpc_rxq.q_ptr;
	while (rcv_cnt) {
		int val_cnt = 0;
		int code = 0;
		int qptrCnt = mpc->mpc_rxq.q_ptr;
		/* If error, look for first invalid character based on tag bits */
		if (err) {
  			uchar_t *tagptr;
			if ((mpc->mpc_mpd->mpd_id & 0xF8) < SSP2_ASIC) 
  				tagptr = (uchar_t *) 
					mpc->mpc_mpd->icp[0].icp_tags_start;
			else
  				tagptr = (uchar_t *) 
				mpc->mpc_mpd->icp[mpc->mpc_icpno].icp_tags_start;
			if(win16)  
				mega_push_win(mpc, 3);
			if ((mpc->mpc_mpd->mpd_id & 0xF8) < SSP2_ASIC) {
				tagp = rx_base << 16;
				tagp |= mpc->mpc_rxq.q_ptr;
			}
			else
				tagp = mpc->mpc_rxq.q_ptr;
			while (val_cnt < rcv_cnt) {
				tgp = tagptr + (tagp >> 2);
				curtags = *tgp >> ((tagp % 4) * 2);
				if ((curtags & 3) == 1) { 
					code = 1;
					break;
				}
				else if ((curtags & 3) == 2) {
					code = 2;
					break;
				}
				else{
					code = 0;
				}
				if(qptrCnt++ > (ushort)(mpc->mpc_rxq.q_end -1)){
               				qptrCnt = (ushort)mpc->mpc_rxq.q_begin;
					if ((mpc->mpc_mpd->mpd_id & 0xF8) 
						< SSP2_ASIC){
						tagp = rx_base << 16;
						tagp |= (ushort)mpc->mpc_rxq.q_begin;
					}
					else
						tagp = 
						(ushort)mpc->mpc_rxq.q_begin;
				}
				else
					tagp++;
				val_cnt++;
			}
			if(win16) 
				mega_pop_win(32);
			rcv_cnt -= val_cnt;
			/* Move as much data as the ld will take while 
				checking for wrap */
			while (val_cnt){
				int count;
				count = MIN(val_cnt, mpc->mpc_rxq.q_end - 
					mpc->mpc_rxq.q_ptr + 1);
#ifdef MIDNIGHT
                                src_addr = mpc->mpc_rxq.q_addr + 
					mpc->mpc_rxq.q_ptr;
                                align = ((unsigned long)src_addr) % 4;
                                if (align) {
                                  bytes = MIN(count,(4-align));
                                  memcpy(cbuf, src_addr, bytes);
                                }
                                else {
                                  bytes = 0;
                                }

                                if (count > bytes) {
                                  memcpy(cbuf+bytes, (mpc->mpc_rxq.q_addr + 
                                     mpc->mpc_rxq.q_ptr + bytes), count-bytes);
                                }
#else
				memcpy(cbuf, 
					(mpc->mpc_rxq.q_addr + 
					mpc->mpc_rxq.q_ptr), count);
#endif
				if ((mpc->mpc_flags & MPC_DSCOPER) 
					&& (dscope[0].chan == mpc - meg_chan)) {
					int d=0;
					unsigned char *ubase = 
						(mpc->mpc_rxq.q_addr 
							+ mpc->mpc_rxq.q_ptr);
					int room,move_cnt;
					room = DSQSIZE - 
					((dscope[d].next - dscope[d].q.q_ptr) 
						& (DSQMASK));
					if (count > room)
						dscope[d].status |= DSQWRAP;
                			else
                        			room = count;
					while (room > 0) {
					move_cnt = MIN(room,
					dscope[d].q.q_end - dscope[d].next + 1);
					memcpy(dscope[d].q.q_addr + 
						dscope[d].next,ubase, move_cnt);
                        		dscope[d].next += move_cnt;
                        		if (dscope[d].next > dscope[d].q.q_end)
                                		dscope[d].next = 
							dscope[d].q.q_begin;
                                	ubase += move_cnt;
                                	room -= move_cnt;
					} /* while room */ 
				wake_up_interruptible(&dscope[d].scope_wait);
				} /* if MPC_DSCOPE */

				val_cnt -= count;
				memset(fbuf, 0, count);
				cbuf += count;
				fbuf += count;
				ldisc_count += count;
				mpc->mpc_input += count;
				mpc->mpc_rxq.q_ptr += count;
				if (mpc->mpc_rxq.q_ptr > mpc->mpc_rxq.q_end)
					mpc->mpc_rxq.q_ptr 
						= mpc->mpc_rxq.q_begin;
			}
			if (code){
				unsigned char c = 
				*(mpc->mpc_rxq.q_addr + mpc->mpc_rxq.q_ptr);
				/* c is an error character */
				if (code == 1){
					ldisc_count++;
					*fbuf++ = TTY_PARITY;
					*cbuf++ = c;
				}
				else if (code == 2){
					ldisc_count++;
					*fbuf++ = TTY_FRAME;
					*cbuf++ = c;
				}
				else{
					ldisc_count++;
					*fbuf++ = TTY_NORMAL;
					*cbuf++ = c;
				}
				rcv_cnt--;
				mpc->mpc_input++;
				if (++mpc->mpc_rxq.q_ptr > mpc->mpc_rxq.q_end) {
					mpc->mpc_rxq.q_ptr = mpc->mpc_rxq.q_begin;
					bp = mpc->mpc_rxq.q_addr + mpc->mpc_rxq.q_ptr;
				}
				if ((mpc->mpc_flags & MPC_DSCOPER) 
					&& (dscope[0].chan == mpc - meg_chan)) {
	
					int d=0;
					dscope[d].buffer[dscope[d].next] = c;
					if (dscope[d].next == dscope[d].q.q_ptr)
				  		wake_up_interruptible(&dscope[d].scope_wait);
					if (dscope[d].next++ == dscope[d].q.q_ptr) 
						dscope[d].status |= DSQWRAP;
					if (dscope[d].next > dscope[d].q.q_end)
						dscope[d].next = dscope[d].q.q_begin;
				} /* if MPC_DSCOPE */
			}
			else if (rcv_cnt)
			{
				printk("Unknown error code value %d", code);
				break; /* Perhaps unnecessary... */
			}
		} 
		else {
			val_cnt = rcv_cnt;
			rcv_cnt = 0;
			while (val_cnt){
				int count;
				count = MIN(val_cnt, mpc->mpc_rxq.q_end - 
					mpc->mpc_rxq.q_ptr + 1);
#ifdef MIDNIGHT
                                src_addr = mpc->mpc_rxq.q_addr + 
					mpc->mpc_rxq.q_ptr;
                                align = ((unsigned long)src_addr) % 4;
                                if (align) {
                                  bytes = MIN(count,(4-align));
                                  memcpy(cbuf, src_addr, bytes);
                                }
                                else {
                                  bytes = 0;
                                }

                                if (count > bytes) {
                                  memcpy(cbuf+bytes, (mpc->mpc_rxq.q_addr + 
                                     mpc->mpc_rxq.q_ptr + bytes), count-bytes);
                                }
#else
				memcpy(cbuf, 
					(mpc->mpc_rxq.q_addr + 
					mpc->mpc_rxq.q_ptr), count);
#endif
				if ((mpc->mpc_flags & MPC_DSCOPER) 
					&& (dscope[0].chan == mpc - meg_chan)) {
					int d=0;
					unsigned char *ubase = 
						(mpc->mpc_rxq.q_addr 
							+ mpc->mpc_rxq.q_ptr);
					int room,move_cnt;
					room = DSQSIZE - 
					((dscope[d].next - dscope[d].q.q_ptr) 
						& (DSQMASK));
					if (count > room)
						dscope[d].status |= DSQWRAP;
                			else
                        			room = count;
					while (room > 0) {
					move_cnt = MIN(room,
					dscope[d].q.q_end - dscope[d].next + 1);
					memcpy(dscope[d].q.q_addr + 
						dscope[d].next,ubase, move_cnt);
                        		dscope[d].next += move_cnt;
                        		if (dscope[d].next > dscope[d].q.q_end)
                                		dscope[d].next = 
							dscope[d].q.q_begin;
                                	ubase += move_cnt;
                                	room -= move_cnt;
					} /* while room */ 
				wake_up_interruptible(&dscope[d].scope_wait);
				} /* if MPC_DSCOPE */

				val_cnt -= count;
				memset(fbuf, 0, count);
				cbuf += count;
				fbuf += count;
				ldisc_count += count;
				mpc->mpc_input += count;
				mpc->mpc_rxq.q_ptr += count;
				if (mpc->mpc_rxq.q_ptr > mpc->mpc_rxq.q_end)
					mpc->mpc_rxq.q_ptr 
						= mpc->mpc_rxq.q_begin;
			}
		} /* end  if err */
	}      /*  while rcv_cnt */
	if(win16) 
	   mega_push_win(mpc, 0);
	rx_tail(mpc->mpc_rxbase + mpc->mpc_rxq.q_ptr);
	if(win16) {
		mega_pop_win(33);
		mega_pop_win(34);
	}
#ifdef DEBUG
	printk("calling tty_schedule_flip with %d charsfor device %d\n",
		tp->flip.count, SSTMINOR(tp->device));
#endif
	/*tty_schedule_flip(tp);*/
	/*queue_task_irq_off(&tp->flip.tqueue, &tq_timer);*/
#ifdef DEBUG
	printk("calling n_tty_receive_buf with %d chars for device %d\n",
		ldisc_count, SSTMINOR(tp->device));
#endif
	tp->ldisc.receive_buf(tp, tp->flip.char_buf, 
		tp->flip.flag_buf,ldisc_count);
}
/*
 * Process an input control signal transition.  In the usual
 * case, a transition indicates a change in the data carrier
 * detect.  Hardware flow control is not supported in hardware
 * and must be done here if "ctsflow" is enabled.
 */

static void megamint(register struct mpchan *mpc)
{
	register icpbaddr_t icpb = mpc->mpc_icpb;
	register struct tty_struct  *tp;
	register ushort dcd;
	int old_carr_state;
	/*char prtstr[70];*/

	if (mpc->mpc_tty == (struct tty_struct *) NULL)
		return;
    if (!(mpc->mpc_icpi->ssp.cin_attn_ena & ENA_DCD_CNG)){
		/*printk("DCD_CNG event: bank sigs for device %d is %x\n", SSTMINOR(mpc->mpc_tty->device),(unsigned int) icpb->ssp.bank_sigs);*/
		return;
	}
	tp = mpc->mpc_tty;
	dcd = icpb->ssp.bank_sigs & CIN_DCD;
	dcd = dcd >> 1;

	old_carr_state = mpc->carr_state;
	if(dcd && (!mpc->carr_state)){
		mpc->carr_state = 1;
		wake_up_interruptible(&mpc->open_wait);
#ifdef DEBUG
		printk("waking up device %d\n", SSTMINOR(mpc->mpc_tty->device));
#endif
	}
	if (!dcd)
		mpc->carr_state = 0;
	if ((!dcd) && (!((mpc->flags & ASYNC_CALLOUT_ACTIVE) &&
			(mpc->flags & ASYNC_CALLOUT_NOHUP))))
#ifdef DEBUG
		{
		printk("megamint: hanging up device %d\n", SSTMINOR(mpc->mpc_tty->device));
    if (mpc->mpc_icpi->ssp.cin_attn_ena & ENA_DCD_CNG)
		printk("DCD_CNG enabled for device %d\n", SSTMINOR(mpc->mpc_tty->device));
#endif
		{
		/*char prtstr[70];
		sprintf(prtstr,"megamint: hanging up device %d\n", SSTMINOR(mpc->mpc_tty->device));
		console_print(prtstr);
		sprintf(prtstr, "bank sigs for device %d is %x\n", SSTMINOR(mpc->mpc_tty->device),(unsigned int) icpb->ssp.bank_sigs);
		console_print(prtstr);*/
#ifdef DEBUG
		printk("megamint: calling tty_hangup for device %d\n", SSTMINOR(mpc->mpc_tty->device));
#endif
		tty_hangup(tp);
		}
#ifdef DEBUG
		}
#endif
}

/*
 * Megaport channel interrupt status register not empty.
 * Look for events that are of interest and then clear
 * the register.
 */

static void megasint(register struct mpchan *mpc)
{
	register struct termios *tiosp;
	struct tty_struct *tty = mpc->mpc_tty;
	char	fbuf, cbuf;

	if (mpc->mpc_tty == (struct tty_struct *) NULL)
		return;
        tiosp = mpc->mpc_tty->termios;
	/*printk("megasint: device = %d\n", SSTMINOR(tty->device));*/
	if(mpc->mpc_cin_events & EV_BREAK_CNG){	
		if (((tiosp->c_iflag & IGNBRK) == 0) &&
      			(tiosp->c_iflag & BRKINT )){
			if (ISRAMP(mpc)){
      				mpc->mpc_mpa_stat |= MPA_CLR_LOOP_BACK_ERROR;
				mpc->mpc_mpa_stat |= MPA_SET_LOOP_BACK_ERROR;
				mpc->mpc_mpa_stat |= MPA_HARD_RESET_ERROR;
				mpc->mpc_mpa_stat |= MPA_CALL_BACK_ERROR;
      				wake_up_interruptible(&mpc->mpc_mpa_slb);
      				wake_up_interruptible(&mpc->mpc_mpa_clb);
      				wake_up_interruptible(&mpc->mpc_mpa_rst);
      				wake_up_interruptible(&mpc->mpc_mpa_call_back);
			}
		}
		if (tty->ldisc.receive_room(tty)) {
			fbuf = TTY_BREAK;
			cbuf = 0;
#ifndef MODULE
			if(mpc->flags & ASYNC_SAK)
				do_SAK(tty);
#endif
			/*tty_schedule_flip(tty);*/ /* Shashi 03/05/98 */
#ifdef DEBUG
	printk("megasint: calling n_tty_receive_buf for device %d\n",
		SSTMINOR(tty->device));
#endif
			tty->ldisc.receive_buf(tty, &cbuf, &fbuf, 1);
		}
	}
}
/*****************************************************************************/

/*
 *	Generate the serial struct info.
 */

static void eqnx_getserial(struct mpchan *mpc, struct serial_struct *sp)
{
	struct serial_struct	sio;

#if DEBUG
#ifdef	IX86
	printk("eqnx_getserial(mpc=%lx,sp=%lx)\n", (unsigned long) mpc, (unsigned long) sp);
#endif	/* IX86 */
#ifdef	IA64
	printk("eqnx_getserial(mpc=%lx,sp=%lx)\n", (unsigned long) mpc, (unsigned long) sp);
#endif	/* IA64 */

#endif

	memset(&sio, 0, sizeof(struct serial_struct));
	sio.type = PORT_UNKNOWN;
	sio.line = SSTMINOR(mpc->mpc_tty->device);
	if (mpc->mpc_mpd->mpd_bus !=  ISA_BUS)
		sio.port = mpc->mpc_mpd->mpd_slot;
	else
		sio.port = ist.ctlr_array[MPBOARD(sio.line)].ctlr_loc;
	sio.irq = 0;
	sio.flags = mpc->flags;
	sio.baud_base = mpc->baud_base;
	sio.close_delay = mpc->close_delay;
	sio.closing_wait = mpc->closing_wait;
	sio.custom_divisor = mpc->custom_divisor;
	sio.xmit_fifo_size = 0;
	sio.hub6 = 0;
	eqn_to_user(sp, &sio, sizeof(struct serial_struct));
}

/*****************************************************************************/

static int eqnx_setserial(struct mpchan *mpc, struct serial_struct *sp)
{
	struct serial_struct	sio;
	int win16;

#if DEBUG
#ifdef	IX86
	printk("eqnx_setserial(mpc=%lx,sp=%lx)\n", (unsigned long) mpc, (unsigned long) sp);
#endif	/* IX86 */
#ifdef	IA64
	printk("eqnx_setserial(mpc=%lx,sp=%lx)\n", (unsigned long) mpc, (unsigned long) sp);
#endif	/* IA64 */
#endif

	if (eqn_from_user(&sio, sp, sizeof(struct serial_struct)))
		return(-EFAULT);
	if (!suser()) {
		if ((sio.baud_base != mpc->baud_base) ||
				(sio.close_delay != mpc->close_delay) ||
				((sio.flags & ~ASYNC_USR_MASK) != (mpc->flags & ~ASYNC_USR_MASK)))
			return(-EPERM);
	} 

	mpc->flags = (mpc->flags & ~ASYNC_USR_MASK) | (sio.flags & ASYNC_USR_MASK);
	mpc->baud_base = sio.baud_base;
	mpc->close_delay = sio.close_delay;
	mpc->closing_wait = sio.closing_wait;
	mpc->custom_divisor = sio.custom_divisor;

	if (mpc->mpc_tty != (struct tty_struct *) NULL) {
		win16 = mpc->mpc_mpd->mpd_nwin;
		if(win16)
			mega_push_win(mpc, 0);
	/*printk("set_serial: Calling megaparam for device %d\n", SSTMINOR(mpc->mpc_tty->device));*/
		megaparam(SSTMINOR(mpc->mpc_tty->device));
		if(win16)
	   		mega_pop_win(2);
	}
	return(0);
}

/*****************************************************************************/
STATIC int eqnx_ioctl(struct tty_struct *tty, struct file * file,
            unsigned int cmd, unsigned long arg)
{
	register struct mpchan *mpc;
	register icpiaddr_t icpi;
	register icpbaddr_t icpb;
	register icpoaddr_t icpo;
	register icpqaddr_t icpq;
	int win16;
	unsigned int arg_int;
	int d, rc = 0;
	int flags = 0;

	if (!tty->driver_data)
		return(-ENXIO);                                           

	if ((cmd == MEGASTTY || cmd == MEGADTR) && !suser())
			return(-EPERM);

	d = SSTMINOR(tty->device);
	if((d > nmegaport*NCHAN_BRD) || (nmegaport == 0)
		|| (d < 0)) 
		return(-ENODEV);

#ifdef SSM /* found testing SSM */
        if ( meg_dev[MPBOARD(d)].mpd_alive == 0)
		return(-ENODEV);
#endif
	save_flags(flags);
	cli();
        mpc = tty->driver_data;
	win16 = mpc->mpc_mpd->mpd_nwin;
	if(win16)
		mega_push_win(mpc, 0);
/* RAMP START --------------------------------------------- */
	/* If RAMP config not completed, don't let any ioctl's in */
	if (ISRAMP(mpc)) {
      /* check break bit */
      /* If not receiving a break, the slot is occupied by a UART */
      
		if ( !(mpc->mpc_mpa_stat & MPA_UART_CFG_DONE) ||
                     (mpc->mpc_mpa_stat & (MPA_INITG_INIT_MODEM | MPA_INITG_MODIFY_SETTINGS))) {
      			icpi = mpc->mpc_icpi;
      			icpb=(rx_locks & LOCK_A) ? &icpi->ssp.cin_bank_b 
				: &icpi->ssp.cin_bank_a;
			/* make sure regs are valid */ 
  			if (!(icpb->ssp.bank_events & EV_REG_UPDT)) 
  				frame_wait(mpc,2); 
         		if (!(icpb->ssp.bank_sigs & 0x01 )) {
				if(win16) mega_pop_win(74);
            			/* Modem present in slot, sleep */
ioctl_modem_wait:
				restore_flags(flags); 
				interruptible_sleep_on(&mpc->mpc_mpa_call_back);
				save_flags(flags);
				cli();
				if eqn_fatal_signal {
					restore_flags(flags); 
					return(-ERESTARTSYS);
				}
				if(mpc->mpc_mpa_stat & MPA_CALL_BACK_ERROR){
					restore_flags(flags); 
                        		return(-ENXIO);
				}
				else  if ( !(mpc->mpc_mpa_stat & MPA_UART_CFG_DONE) ||
                     (mpc->mpc_mpa_stat & (MPA_INITG_INIT_MODEM | MPA_INITG_MODIFY_SETTINGS))) {
					goto ioctl_modem_wait;
				}
				if(win16)
					mega_push_win(mpc, 0);
         		} else {
				if(win16) mega_pop_win(75);
				restore_flags(flags); 
            			/* No modem, return error */
				return(-ENXIO);
         		}
		}
	}
/* RAMP END ----------------------------------------------- */
	icpi = mpc->mpc_icpi;
	icpo = mpc->mpc_icpo;
	icpq = &icpo->ssp.cout_q0;
#ifdef DEBUG
	printk("eqnx_ioctl: cmd %x for device %d\n", (unsigned int) cmd, SSTMINOR(tty->device));
#endif
	switch (cmd) {
	case TCSBRK:		/* SVID version: non-zero arg --> no break */
		if ((rc = tty_check_change(tty)) == 0) {
			restore_flags(flags);
			tty_wait_until_sent(tty, 0);
			if (! arg){
				/* send break */
				if(ISRAMP(mpc))
            				ramp_start_break(mpc);
				else
					icpo->ssp.cout_flow_cfg |= 0x20; 
				eqnx_delay(HZ/4);	/* 1/4 second */
				/* stop break */
				if(ISRAMP(mpc))
            				ramp_stop_break(mpc);
				else
					icpo->ssp.cout_flow_cfg &= ~0x20; 
			}
			save_flags(flags);
			cli();
		}
		break;
	case TCSBRKP:
		if ((rc = tty_check_change(tty)) == 0) {
			restore_flags(flags);
			tty_wait_until_sent(tty, 0);
			/* send break */
			if(ISRAMP(mpc))
            			ramp_start_break(mpc);
			else
				icpo->ssp.cout_flow_cfg |= 0x20; 
			eqnx_delay( arg ? arg*(HZ/10) : HZ/4);
			/* stop break */
			if(ISRAMP(mpc))
            			ramp_stop_break(mpc);
			else
				icpo->ssp.cout_flow_cfg &= ~0x20; 
			save_flags(flags);
			cli();
		}
		break;
	case TIOCGSOFTCAR:
		if ((rc = verify_area(VERIFY_WRITE, (void *) arg, sizeof(int))) == 0)
			put_fs_int(((tty->termios->c_cflag & CLOCAL) ? 1 : 0), (unsigned int *) arg);
		break;
	case TIOCSSOFTCAR:
		if ((rc = verify_area(VERIFY_READ, (void *) arg, sizeof(int))) == 0) {
#if (EQNX_VERSION_CODE < 131328)  
			arg_int = get_fs_long((unsigned long *) arg);
#else
			get_user(arg_int, (unsigned int *) arg);
#endif
			tty->termios->c_cflag = (tty->termios->c_cflag & ~CLOCAL) | (arg_int ? CLOCAL : 0);
		}
		break;
	case TIOCMGET:
		if ((rc = verify_area(VERIFY_WRITE, (void *) arg, sizeof(unsigned int))) == 0)
			rc = get_modem_info(mpc, (unsigned int *) arg);
		break;
	case TIOCMBIS:
	case TIOCMBIC:
	case TIOCMSET:
		rc = set_modem_info(mpc, cmd, (unsigned int *) arg);
		break;
	case TIOCGSERIAL:
		if ((rc = verify_area(VERIFY_WRITE, (void *) arg, sizeof(struct serial_struct))) == 0)
			eqnx_getserial(mpc, (struct serial_struct *) arg);
		break;
	case TIOCSSERIAL:
		if ((rc = verify_area(VERIFY_READ, (void *) arg, sizeof(struct serial_struct))) == 0)
			rc = eqnx_setserial(mpc, (struct serial_struct *) arg);
		break;
	case MEGASTTY:		/* change parameters */
		rc = megastty(mpc, tty, arg, d);
		break;
	case MEGADTR: {
		icpoaddr_t icpo; 
   		uchar_t cur;
		mpc = &meg_chan[d];
		win16 = mpc->mpc_mpd->mpd_nwin;
		if(win16)
		   mega_push_win(mpc, 0);
		icpo = mpc->mpc_icpo;
		GET_CTRL_SIGS(mpc, cur);
      		if (arg == 1) /* unconditionally raise dtr */ 
			cur |= (TX_DTR | TX_HFC_DTR);
		else /* unconditionally drop dtr */ 
			cur &= ~(TX_DTR | TX_HFC_DTR);
		if ((icpo->ssp.cout_int_opost & 0x8) && (cur & TX_SND_CTRL_TG))
			cur ^= TX_SND_CTRL_TG;
		if (!(icpo->ssp.cout_int_opost & 0x8) && !(cur & TX_SND_CTRL_TG))  
			cur ^= TX_SND_CTRL_TG;
		SET_CTRL_SIGS(mpc, cur);
		if(win16)
		   mega_pop_win(26);
		}
		break;
	default:
		rc = -ENOIOCTLCMD;
		break;
	}
	if(win16)
		mega_pop_win(35);
	restore_flags(flags);
	return(rc);
}
/******************************************************************************
        
        EQN_CS_GET
  
        Return the current set of input and output control signals.

        The value returned is "native" icp bit positions regardless of the 
        origin of the signals.  Namely,

                b0      dcd
                b1      cts
                b2      dsr
                b3      ri
                -----------
                b4      dtr
                b5      rts
                b6      0
                b7      0
  
        The returned bits are POSITIVE logic... 1 means signal is on.

******************************************************************************/
static uchar_t eqn_cs_get( struct mpchan *mpc)
{
	uchar_t ret_val = 0; /* default */
	register icpiaddr_t icpi;
	union bank_regs_u *bank;
	ushort_t rawcs;


  	icpi = mpc->mpc_icpi;
	bank = (rx_locks & LOCK_A) ? &icpi->ssp.cin_bank_b : &icpi->ssp.cin_bank_a;
        if (!(bank->ssp.bank_events & EV_REG_UPDT)) 
        	frame_wait(mpc,2); /* make sure regs are valid */ 

  /* get INPUT control signals */
  rawcs = (bank->ssp.bank_sigs >> 1) & 0x0f;
  ret_val = rawcs & 0xff;

  /* get OUTPUT control signals */
  if ( mpc->mpc_cout->ssp.cout_int_flow_ctrl & 0x04 ){
    /* input overload signals sent to lmx */
    GET_CTRL_SIGS(mpc, rawcs);
    rawcs = rawcs >> 4;
  }
  else
    /* normal signals sent to lmx */
    GET_CTRL_SIGS(mpc, rawcs);

  /* combine in signals and out signals */
  ret_val |= ((rawcs & 0x0f) << 4 );
  return ret_val;
}  /* eqn_cs_get */

static int get_modem_info(struct mpchan *mpc, unsigned int *value)
{
	unsigned char status;
	unsigned int result = 0;

	status = eqn_cs_get(mpc);

	result = (((status & 0x20) ? TIOCM_RTS : 0)
		| ((status & 0x10) ? TIOCM_DTR : 0)
		| ((status & 0x1) ? TIOCM_CAR : 0)
		| ((status & 0x8) ? TIOCM_RI : 0)
		| ((status & 0x4) ? TIOCM_DSR : 0)
		| ((status & 0x2) ? TIOCM_CTS : 0));

	put_fs_int(result, (unsigned int *) value);
	return(0);
}

static int set_modem_info(struct mpchan *mpc, unsigned int cmd,
		unsigned int *value)
{
	int error;
	unsigned int arg, temp;
	icpoaddr_t icpo = mpc->mpc_icpo;

	error = verify_area(VERIFY_READ, value, sizeof(int));
	if (error)
		return error;
#if (EQNX_VERSION_CODE < 131328)  
	arg = get_fs_long((unsigned long *) value);
#else
	get_user(arg, (unsigned int *) value);
#endif
	GET_CTRL_SIGS(mpc, temp);
	switch (cmd) {
	case TIOCMBIS: 
		if (arg & TIOCM_RTS)
			temp |= TX_RTS | TX_HFC_RTS;
		if (arg & TIOCM_DTR)
			temp |= TX_DTR | TX_HFC_DTR;
		break;
	case TIOCMBIC: 
		if (arg & TIOCM_RTS)
			temp &= ~(TX_RTS | TX_HFC_RTS);
		if (arg & TIOCM_DTR)
			temp &= ~(TX_DTR | TX_HFC_DTR);
		break;
	case TIOCMSET: 
		temp = ((temp & ~(TX_RTS | TX_HFC_RTS 
				| TX_DTR | TX_HFC_DTR))
			| ((arg & TIOCM_RTS) ? TX_RTS | TX_HFC_RTS : 0)
			| ((arg & TIOCM_DTR) ? TX_DTR | TX_HFC_DTR : 0));

		break;
	}
	if ( ((icpo->ssp.cout_int_opost & 0x8) && (temp & TX_SND_CTRL_TG)) ||
	   (!(icpo->ssp.cout_int_opost & 0x8) && !(temp & TX_SND_CTRL_TG)) )  
			temp ^= TX_SND_CTRL_TG;
	SET_CTRL_SIGS(mpc, temp);
	return(0);
}

/* overwrite stty setup */
static int megastty( struct mpchan *mpc, struct tty_struct *tty,

		int arg, int dev)
{
	int flags = 0, win16;
	int rc = 0;
	struct termios *tp = tty->termios;
	struct marb_struct *curmarb;
	struct lmx_struct *lmx;
	int slot_chan, port, ldv;

  switch( arg ) {
  case 1:
	mpc->mpc_param |= IXONSET;	/* force ixon */
	tp->c_iflag |= IXON;
	megaparam(dev);
	break;

  case 2:
	mpc->mpc_param &= ~IXONSET;	/* cancel forcing ixon */
	tp->c_iflag &= ~IXON;
	megaparam(dev);
	break;

  case 3:
	mpc->mpc_param |= IXANYIG;	/* force ignore ixany */
	megaparam(dev);
	break;

  case 4:
	mpc->mpc_param &= ~IXANYIG;	/* cancel ignore ixany */
	megaparam(dev);
	break;

  case 5:
	mpc->mpc_param |= IOCTLFB;	/* force  TCSETA -> TCSETAW */
	break;

  case 6:
	mpc->mpc_param &= ~IOCTLFB;	/* cancel forcing IOCTLFB */
	break;

  case 7:
	mpc->mpc_param |= IOCTLLP;	/* force  spoller sync */
	break;

  case 8:
	mpc->mpc_param &= ~IOCTLLP;	/* cancel forcing IOCTLLP */
	break;

  case 9:
	break;

  case 10:
	break;
	
#ifndef LINUX
  case 11:
	mpc->mpc_param |= IOCTLLI;	/* force  input ld */
	mpc->mpc_rmode = LINEDISCIPLINE;
	break;

  case 12:
	mpc->mpc_param &= ~IOCTLLI;	/* cancel forcing IOCTLLI */
	setrwmode(mpc);
	break;

  case 13:
	mpc->mpc_param |= IOCTLLO;	/* force  output ld */
	mpc->mpc_wmode = LINEDISCIPLINE;
	break;

  case 14:
	mpc->mpc_param &= ~IOCTLLO;	/* cancel forcing IOCTLLO */
	setrwmode(mpc);
	break;
#endif
  case 15:
	tp->c_cflag |= CRTSCTS;
	megaparam(dev);
	if((tp->c_cflag & CRTSCTS) == 0) 
	   rc = -EINVAL;
	else
	   mpc->mpc_param |= IOCTCTS;
	break;

  case 16:
	tp->c_cflag &= ~CRTSCTS;
	mpc->mpc_param &= ~IOCTCTS;
	megaparam(dev);
	break;

  case 17:
	tp->c_cflag |= CRTSCTS;
	if((mpc->mpc_tty->termios->c_cflag & RTSFLOW) == 0) {
		if((tp->c_cflag & CRTSCTS) == 0)
		   rc = -EINVAL;
		else
		   mpc->mpc_param |= IOCTRTS;
	}
	megaparam(dev);
	break;

  case 18:
	tp->c_cflag &= ~CRTSCTS;
	mpc->mpc_param &= ~IOCTRTS;
	megaparam(dev);
	break;

  case 19:
	if(tty == mpc->mpc_tty) {
	   mpc->mpc_start = 101;
	   mpc->mpc_stop = 103;
	   mpc->mpc_param |= (IXONSET|IOCTLPC);	/* force ixon */
	   megaparam(dev);
	}
	break;

  case 20:
	if(tty == mpc->mpc_tty) {
	   mpc->mpc_start = 17;
	   mpc->mpc_stop = 19;
	   mpc->mpc_param &= ~(IXONSET|IOCTLPC);
	   megaparam(dev);
	}
	break;

  case 21:
	save_flags(flags);
	cli();
	   mpc->mpc_param |= IOCTLLB;
		/* int/ext loopback, command enable */
/* RAMP START --------------------------------------------- */
	   if (ISRAMP(mpc)) {
      	   	lmx = &(mpc->mpc_icp->lmx[mpc->mpc_lmxno]);
	   	win16 = mpc->mpc_mpd->mpd_nwin;
	   	if(win16)
		   mega_push_win(mpc, 0);
			/* 64 ports per ICP, 16 ports per lmx */
 		ldv = ((mpc - meg_chan) % 64) / 16; 
 		port = (mpc - meg_chan) % MAXLMXMPC;
 		slot_chan = port + (ldv * 16);
         	curmarb = &((mpc->mpc_icp->slot_ctrl_blk + slot_chan)->marb);
#ifdef DEBUG
printk("calling ramp_set_loop_back for device %d\n", mpc->mpc_chan);
#endif
         	if (ramp_set_loop_back(mpc)) {
            		if(win16)
               			mega_pop_win(51);
	   		rc = -ENXIO;
			restore_flags(flags); 
			break;
         	}
	        if(win16)
		         mega_pop_win(52);
loopback_21:
		restore_flags(flags); 
		interruptible_sleep_on(&mpc->mpc_mpa_slb);
		save_flags(flags);
		cli();
		if eqn_fatal_signal {
			rc = -ERESTARTSYS;
			restore_flags(flags); 
			break;
		}
		if (mpc->mpc_mpa_stat & MPA_SET_LOOP_BACK_ERROR){
	   		rc = -ENXIO;
			restore_flags(flags); 
		}
		else if (mpc->mpc_mpa_stat == MPA_INITG_SET_LOOP_BACK)
			goto loopback_21;
		else
			restore_flags(flags);
	} else {
/* RAMP END ----------------------------------------------- */
		/* int/ext loopback, command enable */
		mpc->mpc_cout->ssp.cout_lmx_cmd |= 0x43;
		frame_wait( mpc, 6 );
		/* clear cmd enable */
		mpc->mpc_cout->ssp.cout_lmx_cmd &= ~0x40;
		mpc->mpc_flags |= MPC_LOOPBACK;
		restore_flags(flags);
	}
	   break;


  case 22:
	save_flags(flags);
	cli();
	   mpc->mpc_param &= ~IOCTLLB;
/* RAMP START --------------------------------------------- */
	if (ISRAMP(mpc)) {
		win16 = mpc->mpc_mpd->mpd_nwin;
      		lmx = &(mpc->mpc_icp->lmx[mpc->mpc_lmxno]);
		if(win16)
	   		mega_push_win(mpc, 0);
		/* 64 ports per ICP, 16 ports per lmx */
 		ldv = ((mpc - meg_chan) % 64) / 16; 
 		port = (mpc - meg_chan) % MAXLMXMPC;
 		slot_chan = port + (ldv * 16);
         	curmarb = &((mpc->mpc_icp->slot_ctrl_blk + slot_chan)->marb);
         	if (ramp_clr_loop_back(mpc)) {
			if(win16)
				mega_pop_win(6);
	   		rc = -ENXIO;
			restore_flags(flags); 
			break;
		}
		if(win16)
			mega_pop_win(6);
loopback_22:
		restore_flags(flags); 
		interruptible_sleep_on(&mpc->mpc_mpa_clb);
		save_flags(flags);
		cli();
		if eqn_fatal_signal {
			rc = -ERESTARTSYS;
			restore_flags(flags); 
			break;
		}
		if (mpc->mpc_mpa_stat & MPA_CLR_LOOP_BACK_ERROR){
	   		rc = -ENXIO;
			restore_flags(flags); 
		}
		else if (mpc->mpc_mpa_stat == MPA_INITG_CLR_LOOP_BACK)
			goto loopback_22;
		else
			restore_flags(flags);
	} else {
/* RAMP END ----------------------------------------------- */
		mpc->mpc_cout->ssp.cout_lmx_cmd &= ~0x03;       /* int/ext loopback */
		mpc->mpc_cout->ssp.cout_lmx_cmd |= 0x40;        /* command enable */
		frame_wait( mpc, 6 );
		/* clear cmd enable */
		mpc->mpc_cout->ssp.cout_lmx_cmd &= ~0x40;
		mpc->mpc_flags &= ~MPC_LOOPBACK;
		restore_flags(flags);
	}
	break;

  case 23:
	   mpc->mpc_param |= IOCTXON;
	   megaparam(dev);
	   break;

  case 24:
	   mpc->mpc_param &= ~IOCTXON;
	   megaparam(dev);
	   break;

  case 25:
	   mpc->mpc_param |= IOCTLCK;
	   break;

  case 26:
	   mpc->mpc_param &= ~IOCTLCK;
	   break;
  default: ;
  }
  return(rc);
}
int move_brd_addr( unsigned long int addr)
{
register struct mpdev *mpd;
register struct mpchan *mpc;
volatile unsigned char *chnl_ptr;

int ret=true;
int ram_ok=false;
int ii, jj,j, i;
paddr_t mpd_mem = 0;
int z = 0,y = 0;
uchar_t cchnl;
ushort_t no_cache;
icpgaddr_t icpg;
int win16;
mpaddr_t mp;
int k;
int first_time=0;
int rerror=0;
#if (EQNX_VERSION_CODE < 66304)
int rc;
#endif
int flags = 0;

   if (io_bus_type < 1 || io_bus_type > 2)
   {
#ifdef UNIX
       cmn_err(CE_CONT,"Eqnx: Invalid Bus type (%x)\n",io_bus_type);
#else
       printk("Eqnx: Invalid Bus type (%x)\n",io_bus_type);
#endif
       return(-EFAULT);
   }
    /* all ISA brd that are alive */
   for (k=0 ; k < nmegaport; k++)
   {

         mpd = &meg_dev[k];
         win16 = mpd->mpd_nwin;
         /* check if brd is page and ISA
            otherwise an error
         */

         if (!win16   || mpd->mpd_bus != ISA_BUS  || mpd->mpd_alive== false)
           continue;
         if (mpd->mpd_pmem == addr)
		return(rerror);

       /*
          Check if memory address were in an 0xFF if so error
          check if given addr is valid in table
          0xfe or 0xf1  fir only the first isa brd , since all are map to
          same window
       */

       if (!first_time)
       {
            for (z=0; z< HOLE_TABLE_SIZE; z++)
            {
                    if (ist.sys_isa_holes[z] == 0)
                           continue;
                    if (ist.sys_isa_holes[z] == mpd->mpd_pmem)
                    {


                        if (ist.sys_isa_hole_status[z] == 0xff)
                        {
#ifdef UNIX
                              cmn_err(CE_CONT,
                                 "Eqnx: Warning move brd at hole %d from 16 bit addr %x\n",z,addr);
#else
                              printk("Eqnx: Warning move brd at hole %d to the 16 bit addr %x\n",z, (unsigned int) addr);
#endif
                              ram_ok=true;
                              break;
                        }
                        else
                         ram_ok=true;
                      break;
                  }
            }
            if (ram_ok )
            {
                ram_ok=false;
                for (y=0; y< HOLE_TABLE_SIZE; y++)
                {
                        if (ist.sys_isa_holes[y] == 0)
                               continue;
                        if (ist.sys_isa_holes[y] == addr)
                        {

                            if (ist.sys_isa_hole_status[y] == 0xf1 ||
                                   ist.sys_isa_hole_status[y] == 0xf2 ||
                                   ist.sys_isa_hole_status[y] == 0xf3) 
                            {

                                  ram_ok=true;
                                  break;
                            }
                          break;
                      }
                }

           }
           if (!ram_ok)
           {
#ifdef UNIX
               cmn_err(CE_CONT,
               "EQNX: Invalid Memory Address (%x)\n", addr);
#else
               printk("EQNX: Invalid Memory Address (%x)\n",
			(unsigned int) addr);
#endif

               ret=-EFAULT;
               return(ret);
            }
    }
#ifdef LINUX
/* Pat */
#if (EQNX_VERSION_CODE < 131328)
	if (addr > 0x100000){
#endif
/* Pat */       
#if (EQNX_VERSION_CODE > 66304)
		if(((void *) mpd_mem = ioremap(addr,0x4000)) == (char *) NULL){
			printk("EQUINOX: Failed to allocate virtual address space of size %d\n",
				0x4000);
		}
#else
		if(((void *) mpd_mem = vmalloc(0x4000)) == (char *) NULL){
			printk("EQUINOX: Failed to allocate virtual address space of size %d\n",
				0x4000);
		}
		if ((rc = remap_page_range((TASK_SIZE + 
			((unsigned long) mpd_mem)), addr, 
				0x4000, PAGE_IOMEM))) {
			printk("EQUINOX: Failed to map physical address space = %x, errno = %d in move_brd_addr",
				(int) addr, rc);
		}
#endif
/* Pat */
#if (EQNX_VERSION_CODE < 131328)
	}
	else
		mpd_mem = addr;
#endif
/* Pat */		
#else
#ifdef UNIX
          mpd_mem = (paddr_t) sptalloc( btoc(0x4000), (PG_P), btoc((caddr_t)addr),1);
#else
          mpd_mem = (paddr_t)mapphys((caddr_t)addr, 0x4000);
#endif
#endif /** Not Linux **/

        if(!mpd_mem)
        {

#ifdef UNIX
           cmn_err(CE_CONT,
           "EQNX: Memory mapping failed for board %d\n", k +1);
#else
	   printk("EQNX: Memory mapping failed for board %d\n", k +1);
#endif
           ret= ENOMEM;
           return(ret);
        }
	save_flags(flags);
	cli();
         if(win16)
           mega_push_winicp(mpd, 0);

    	if ((mpd->mpd_id & 0xF8) < SSP2_ASIC) { /* SSP64 board */
       		/* Set the address, 16-bit mode, MAX2, 
		 * and flat off 
		 */ 
       		outb(((addr >> 14) & 0xff), portn[mpd->mpd_io]);
       		outb((((addr >> 22) & 3)| 0x20),portn[mpd->mpd_io] + 1);
       		/* page register - deselect ram */  
		
       		outb(0, portn[mpd->mpd_io] + 2);
       		outb(0, portn[mpd->mpd_io] + 3);

       		/* disable interrupts */ 
       		outb(0, portn[mpd->mpd_io] + 6);
    	}
    	else { /* SSP4 board */
      		unsigned char b;
    		
      		/* reg 8 (page reg) */
      		if(mpd->mpd_nwin == HA_FLAT)
         		b = 0x20;
      		else
         		b = 0;

      		outb(b, portn[mpd->mpd_io] + 0x08);

      		b = ((addr & 0x0000c000) >> 8);
      		/* no ints */
      		outb(b,portn[mpd->mpd_io] + 0x09);
      		b =((addr & 0xFFFF0000) >> 16);
      		outb(b,portn[mpd->mpd_io] + 0x0a);
      		/* reg b*/
      		b=0;
      		outb(b, portn[mpd->mpd_io] + 0x0b);

    	} /* SSP4 */
    
            /*  1 icps */
             j=0;
    
    /* select correct page if necessary */
    if ( mpd->mpd_nwin == HA_WIN16 ){
      if((mpd->mpd_id & 0xF8) < SSP2_ASIC)
        outw( 0x0100 | j, portn[mpd->mpd_io] + 2);
      else
        outb( 1 << j, portn[mpd->mpd_io] + 8);
    }

    if ((mpd->mpd_id & 0xF8) < SSP2_ASIC) { /* SSP64 board */
        icpg = (icpgaddr_t)((unsigned long)mpd_mem + 0x2000);
       /* verify that pram is not cached */

      cchnl = Gicp_chnl;
      chnl_ptr = &(Gicp_chnl);
    }
    else { /* SSP4 board */
      icpaddr_t icp;
      union global_regs_u *icp_glo ;
      icp = &mpd->icp[j];
      icp_glo = (union global_regs_u *)((unsigned long)mpd_mem + 0x400);
      cchnl = icp_glo->ssp2.chan_ctr;
      chnl_ptr = &(icp_glo->ssp2.chan_ctr);
    }
      no_cache = false;
      ret=true;
      for ( ii = 0; ii < 0x10000; ii++ )
        if ( *chnl_ptr != cchnl )
        {
          no_cache = true;
          break;
        }
      if ( !no_cache )
      {
         /* SSM, found testing SSM wrong error messages */
#ifdef UNIX
		 cmn_err(CE_CONT,
          "EQNX: PRAM memory appears to be cached %d for board with I/O address 0x%x.\n",
          ii,portn[mpd->mpd_io]);
#else
          printk("EQNX: PRAM memory appears to be cached %d for with I/O address 0x%x.\n",
          ii,portn[mpd->mpd_io]);
#endif
         ret=false;
         rerror=-EFAULT;
      }
      else
        rerror=0;

      if (ret == false )
      {
        /* if failure map back to org addr */
        if ((mpd->mpd_id & 0xF8) < SSP2_ASIC) { /* SSP64 board */

        /*Disable Controller's addressing */
        outw( 0x0000, portn[mpd->mpd_io] + 2);
        outw( 0x0000, portn[mpd->mpd_io]);

        /* Set the address, 16-bit mode, MAX2, and flat off */
        outb((mpd->mpd_pmem >> 14) & 0xff, portn[mpd->mpd_io]);
        outb((((mpd->mpd_pmem >> 22) & 3)| 0x20), portn[mpd->mpd_io] + 1);

        /* page register - deselect ram */
        outb(0, portn[mpd->mpd_io] + 2);
        outb(0, portn[mpd->mpd_io] + 3);

        /* disable interrupts */
        outb(0, portn[mpd->mpd_io] + 6);
        }
        else { /* SSP4 board */
          unsigned char b;
     
          /* reg 8 (page reg) */
          if(mpd->mpd_nwin == HA_FLAT)
             b = 0x20;
          else
             b = 0;

          outb( b, portn[mpd->mpd_io] + 0x08);

          b = ((mpd->mpd_pmem & 0x0000c000) >> 8);
          /* no ints */
          outb(b, portn[mpd->mpd_io] + 0x09);
          b = ((mpd->mpd_pmem & 0xFFFF0000) >> 16);
          outb(b, portn[mpd->mpd_io] + 0x0a);
          /* reg b*/
          b=0;
          outb(b,portn[mpd->mpd_io] + 0x0b);
        }

#ifdef LINUX
/* Pat */
#if (EQNX_VERSION_CODE < 131328)
	if (mpd_mem > 0x100000)
#endif
/* Pat */
		iounmap((void *) mpd_mem);
#else
#ifdef UNIX
         sptfree( (caddr_t) mpd_mem, btoc(0x4000),   0 );
#else
         unmapphys( (caddr_t) mpd_mem, 0x4000 );
#endif
#endif /** Not Linux **/

      }
      else
      {
       /* free old alloc */
#ifdef LINUX
/* Pat */
#if (EQNX_VERSION_CODE < 131328)
	if (mpd->mpd_mem > 0x100000)
#endif
/* Pat */	
		iounmap((void *) mpd->mpd_mem);
#else
#ifdef UNIX
          sptfree( (caddr_t) mpd->mpd_mem, btoc(0x4000),   0 );
#else
          unmapphys( (caddr_t) mpd->mpd_mem, 0x4000 );
#endif
#endif /** Not Linux **/

         /* change status setting in hold table
            for first success
         */

        if (!first_time)
        {
           switch (ist.sys_isa_hole_status[z])
           {
              case 0xfd:
                ist.sys_isa_hole_status[z] = 0xf1;    /* old */
                break;

              case 0xff:
                 ist.sys_isa_hole_status[z] = 0xf3;    /* old */
                break;

              case 0xfe:
              default:
                ist.sys_isa_hole_status[z] = 0xf1;    /* old */
                break;
           }

           /* set new status */
           switch (ist.sys_isa_hole_status[y])
           {

              case 0xf2:
                ist.sys_isa_hole_status[y] = 0xfd;    /* new */
                break;
              case 0xf3:
                ist.sys_isa_hole_status[y] = 0xff;    /* new */
                break;

              case 0xf1:
              default:
                ist.sys_isa_hole_status[y] = 0xfe;    /* new */
                break;
           }

           first_time=1;
        }
           printk("EQNX: Moved board(%d) from addr(%lx) to addr(%lx) ok \n",
                    k,(unsigned long) mpd->mpd_pmem,(unsigned long) addr);

	   mpd->mpd_mem= mpd_mem;
	   mpd->mpd_pmem= addr;

/* This should be unneccesary but playing it safe... */
	if((mpd->mpd_id & 0xF8) < SSP2_ASIC)
		mpd->mpd_sspchan = NCHAN; /* SSP64 */
	else
		mpd->mpd_sspchan = 4; /* SSP4 */ 
/* end playing it safe */
	brd_mem_cfg(mpd);

          for(j=0; j < (int)mpd->mpd_nicps; j++)
          {
              mp = (mpaddr_t)mpd->icp[j].icp_regs_start;

            /* loop thru each channel - all bytes have been zero'd */
            for ( i = 0; i <  MIN(MAXICPCHNL, mpd->mpd_sspchan); i++ )
            {
                jj = mpd->mpd_minor + i + (j * mpd->mpd_sspchan);
                mpc = &meg_chan[jj];

                mpc->mpc_icpi = (icpiaddr_t)&mp->mp_icpi[i];
            	if ((mpd->mpd_id & 0xF8) < SSP2_ASIC)  /* SSP64 board */
                	mpc->mpc_icpo = (icpoaddr_t)&mp->mp_icpo[i];
		else
                	mpc->mpc_icpo = (icpoaddr_t)((unsigned char *)
				(mpc->mpc_icpi) + 0x200);
                mpc->mpc_icp = (icpaddr_t)&mpd->icp[j];
     		mpc->mpc_rxq.q_addr = (char *)(mpc->mpc_icp->icp_dram_start);
		if ((mpd->mpd_id & 0xF8) < SSP2_ASIC) 
     			mpc->mpc_txq.q_addr = (char *)(mpc->mpc_icp->icp_dram_start);
		else
     			mpc->mpc_txq.q_addr = (char *)(mpc->mpc_icp->icp_dram_start +
						0x1000);
            }
         }
    }

    if (win16)
      mega_pop_win(48);

    restore_flags(flags);
  }

  return(rerror);
}

STATIC int eqnx_diagioctl(struct inode *ip, struct file *fp, unsigned int cmd,
            unsigned long arg)
{
	int nchan,d;
	struct mpchan *mpc;
	struct mpdev *mpd;
	register icpiaddr_t icpi;
	struct termios *tp;
	int rc = 0, minor;
	unsigned int length;
	int win16, flags = 0;
	ushort_t cur;

	save_flags(flags);
	cli();
#ifdef DEBUG
	printk("eqnx_diagioctl: cmd %x for device %d\n", (unsigned int) cmd, 
		MINOR(ip->i_rdev));
#endif
	if ((cmd == STATUS1) && !suser()) {
			restore_flags(flags);
			return(-EPERM);
	}
	minor = MINOR(ip->i_rdev);
	switch (minor){
	case 0:
	switch (cmd){
      	case EQNTRACEON:
		{
			eqn_ramp_trace = true;
			eqn_print_trace();
			break;
		}

      	case EQNTRACEOFF:
			eqn_ramp_trace = false;
			break;

	case EQNQUIETOFF:
		eqn_quiet_mode = false;
		eqn_save_trace = true;
		break;
	case EQNQUIETON:
		eqn_quiet_mode = true;
		if (eqn_ramp_trace || eqn_save_trace){
			eqn_print_trace();
		}
		eqn_save_trace = false;
		break;
	case MEGARESUME:		/* resume output */
					/* min number passed in as arg */
		{
		if (eqn_from_user(&length, (unsigned int *)arg, 
			sizeof(unsigned int))) {
			restore_flags(flags);
			return(-EFAULT);
		}

		d = SSTMINOR(length);
		if((d > nmegaport*NCHAN_BRD) || (nmegaport == 0)
			|| (d < 0)) {
			restore_flags(flags);
			return(-ENODEV);
		}
        	mpc = &meg_chan[d];
		if ((mpd = mpc->mpc_mpd) == (struct mpdev *) NULL){
			restore_flags(flags);
			return(-ENODEV);
		}
		nchan = MPCHAN(d);
		if (mpd >= &meg_dev[nmegaport] || mpd->mpd_alive == 0 ||
			nchan >= (int)mpd->mpd_nchan){
				restore_flags(flags);
				return(-ENODEV);
		}
		win16 = mpc->mpc_mpd->mpd_nwin;
		if(win16)
			mega_push_win(mpc, 0);
		eqnx_flush_buffer(mpc->mpc_tty);
		icpi = mpc->mpc_icpi;
		icpi->ssp.cin_inband_flow_ctrl &= ~0x01;
		if(win16)
			mega_pop_win(35);
		}
		break;
	case EQNISTCFG:
		if (eqn_to_user((struct ist_struct *) arg, &ist, sizeof(ist))) {
			restore_flags(flags);
			return(-EFAULT); 
		}
		
		break;
	case STATUS2:{
		struct SttyStatus sttystatus;
		if (eqn_from_user(&sttystatus, (struct SttyStatus *)arg, 
			sizeof(struct SttyStatus))) {
			restore_flags(flags);
			return(-EFAULT); 
		}

		d = SSTMINOR(sttystatus.mp_dev);
		if((d > nmegaport*NCHAN_BRD) || (nmegaport == 0)
			|| (d < 0)) {
			restore_flags(flags);
			return(-ENODEV);
		}
        	mpc = &meg_chan[d];
		if ((mpd = mpc->mpc_mpd) == (struct mpdev *) NULL){
			restore_flags(flags);
			return(-ENODEV);
		}
		nchan = MPCHAN(d);
		if (mpd >= &meg_dev[nmegaport] || mpd->mpd_alive == 0 ||
			nchan >= (int)mpd->mpd_nchan){
			restore_flags(flags);
			return(-ENODEV);
		}

		sttystatus.mpc_param = mpc->mpc_param;
		if (eqn_to_user((int *) arg, &sttystatus, 
			sizeof(sttystatus))) {
			restore_flags(flags);
			 return(-EFAULT);
		}
		}
		break;
	case STATUS3: { 	/* status3 */
		struct ModemStatus mdstatus;
		if (eqn_from_user(&mdstatus, (struct ModemStatus *)arg, 
			sizeof(struct ModemStatus))) {
			restore_flags(flags);
			return(-EFAULT);
		}

		d = SSTMINOR(mdstatus.mp_dev);
		if((d > nmegaport*NCHAN_BRD) || (nmegaport == 0)
			|| (d < 0)) {
			restore_flags(flags);
			return(-ENODEV);
		}
        	mpc = &meg_chan[d];
		if ((mpd = mpc->mpc_mpd) == (struct mpdev *) NULL){
			restore_flags(flags);
			return(-ENODEV);
		}
		nchan = MPCHAN(d);
		if (mpd >= &meg_dev[nmegaport] || mpd->mpd_alive == 0 ||
			nchan >= (int)mpd->mpd_nchan){
			restore_flags(flags);
			return(-ENODEV);
		}

		mdstatus.mpstatus = eqn_cs_get(mpc);
		if (eqn_to_user((struct ModemStatus *) arg, &mdstatus, 
			sizeof(mdstatus))) {
			restore_flags(flags);
			return(-EFAULT); 
		}
		
		break;
	}
	case STATUS1: { 	/* status1 */
		struct disp dp;
		icpaddr_t  icp;
		icpiaddr_t icpi;
		icpoaddr_t icpo;
		icpgaddr_t icpg;
		icpqaddr_t icpq;
		icpbaddr_t icpb;
		if (eqn_from_user(&dp, (unsigned char *)arg, sizeof(dp))) {
			restore_flags(flags);
			return(-EFAULT);
		}
			
		d = SSTMINOR(dp.mp_dev);
		if((d > nmegaport*NCHAN_BRD) || (nmegaport == 0)
			|| (d < 0)) {
			restore_flags(flags);
			return(-ENODEV);
		}
        	mpc = &meg_chan[d];
		if ((mpd = mpc->mpc_mpd) == (struct mpdev *) NULL){
			restore_flags(flags);
			return(-ENODEV);
		}
		nchan = MPCHAN(d);
		if (mpd >= &meg_dev[nmegaport] || mpd->mpd_alive == 0 ||
			nchan >= (int)mpd->mpd_nchan){
				restore_flags(flags);
				return(-ENODEV);
		}
		win16 = mpc->mpc_mpd->mpd_nwin;
		if(win16)
			mega_push_win(mpc, 0);

		dp.mp_state = mpc->flags;
#ifndef LINUX
		dp.mp_xstate = 0;
#endif
		if (mpc->flags & ASYNC_INITIALIZED){
			tp = mpc->mpc_tty->termios;
			dp.mp_iflag = tp->c_iflag;
			dp.mp_oflag = tp->c_oflag;
			dp.mp_lflag = tp->c_lflag;
			dp.mp_cflag = tp->c_cflag;
			dp.mp_ldisp = tp->c_line;
		}
#ifndef LINUX
		dp.mp_xoutqcc = 0;
		/*dp.mp_delct = tp->t_delct;*/
#endif
		dp.mp_param = mpc->mpc_param;
		dp.mp_flags = mpc->mpc_flags;
		dp.mp_mpa_stat = mpc->mpc_mpa_stat;
		icp = mpc->mpc_icp;
		icpi = mpc->mpc_icpi;
		icpo = mpc->mpc_icpo;
		if ((mpc->mpc_mpd->mpd_id & 0xF8) < SSP2_ASIC)
			icpg = (icpgaddr_t)icpo;
		else
			icpg = (icpgaddr_t)((unsigned long)icpi + 0x400);
		icpq = &icpo->ssp.cout_q0;
		icpb = (rx_locks & LOCK_A) 
			? &icpi->ssp.cin_bank_b : &icpi->ssp.cin_bank_a;
		dp.mp_tx_count = mpc->mpc_count;
		dp.mp_tx_base = Q_data_base;
		dp.mp_tx_q_ptr = Q_data_ptr;
		dp.mp_rawqcc = dp.mp_tx_q_cnt = Q_data_count;
		dp.mp_outqcc = eqnx_write_room(mpc->mpc_tty);
		dp.mp_rx_base = rx_base;
		dp.mp_rx_next = rx_next;
		dp.mp_rx_tail = mpc->mpc_rxq.q_ptr;
		dp.mp_rx_fmt = rx_param;
		dp.mp_rxbaud = rx_baud;
		dp.mp_cin_ctrl = rx_ctrl_sig;
		dp.mp_lmx_cmd = icpo->ssp.cout_lmx_cmd;
		dp.mp_tx_fmt = tx_char_fmt;
		dp.mp_txbaud = tx_baud;
		dp.mp_cout_ctrl = tx_ctrl_sig;
		dp.mp_rx_csr = rx_csr;
		dp.mp_tx_csr = tx_csr;
		dp.mp_tx_flow_cfg = tx_flow_config;
		dp.mp_susp_lmx = rx_suspo_lmx;
		dp.mp_susp_sig = rx_suspo_sig;
		dp.mp_rx_locks = rx_locks;
		dp.mp_cie = rx_cie;
		dp.mp_cis = rx_events | mpc->mpc_cin_events;
		dp.mp_cieo = tx_cie;
/*printk("cie = %x, cout_ctrl = %x\n", rx_cie, tx_ctrl_sig); 
printk("dp.cie = %x, dp.cout_ctrl = %x\n", dp.mp_cie, dp.mp_cout_ctrl); */
		if(tx_csr & TXSR_EV_B_ACT) 
		   dp.mp_tx_events = icpo->ssp.cout_events_b; 
		else
		   dp.mp_tx_events = icpo->ssp.cout_events_a; 
		dp.mp_rxtdm = rx_tdm_early;
		dp.mp_txtdm = 0;
		dp.mp_int_flgs = icpi->ssp.cin_int_flags;
		dp.mp_vmin = rx_vmin;
		dp.mp_vtime = rx_vtime;
		dp.mp_fifo_cnt = rx_fifo_cnt;
		dp.mp_rx_rcv_cnt = rx_rcv_cnt;
		dp.mp_tx_cpu_reg = tx_cpu_reg;
#ifndef LINUX
		dp.mp_rmode = mpc->mpc_rmode;
		dp.mp_wmode = mpc->mpc_wmode;
#endif
		dp.mp_start = mpc->mpc_start;
		dp.mp_stop = mpc->mpc_stop;
		dp.mp_input = mpc->mpc_input;
		dp.mp_output = mpc->mpc_output;
#ifndef LINUX
		dp.mp_esctim = mpc->mpc_scrn.esc_time;
		dp.mp_htky1 = mpc->mpc_scrn.hotkey[0];
		dp.mp_htky2 = mpc->mpc_scrn.hotkey[1];
#endif
		dp.mp_bus = mpc->mpc_mpd->mpd_bus;
		dp.mp_brdno = mpc->mpc_brdno;
		dp.mp_icpno = mpc->mpc_icpno;
		dp.mp_lmxno = mpc->mpc_lmxno;
		dp.mp_chan = mpc->mpc_chan;
		if ((mpc->mpc_mpd->mpd_id & 0xF8) < SSP2_ASIC) {
			dp.mp_g_attn= i_gicp_attn;
			dp.mp_g_init = Gicp_initiate;
		}
		else {
			dp.mp_g_attn= icpg->ssp2.attn_pend;
			dp.mp_g_init = 0;
		}
		dp.mp_g_rx_attn0 = gicp_rx_attn[0];
		if(win16)
		   mega_pop_win(8);
		if (eqn_to_user((unsigned char *) arg, &dp, sizeof(dp)))
			rc = -EFAULT;
			
		break;
	} /* End STATUS1 */
	case EQNMOVEBRD:{
		struct movebrd mb;
		if (eqn_from_user(&mb, (struct movebrd *) arg, 
			sizeof(struct movebrd))) {
			rc = -EFAULT;
			break;
		}

		if (!mb.addr)
			rc = -EINVAL;
		else
			rc = move_brd_addr(mb.addr);
		}
		break;
      case EQNBRDSTATLEN:
			length = sizeof(struct eqnbrdstatus); 
			if (eqn_to_user((unsigned int *) arg, &length, 
				sizeof(length)))
				rc = -EFAULT; 

			break;
      case EQNBRDSTATUS: {
			struct	eqnbrdstatus	brdstat;
			int	index;

			if (eqn_from_user(&index, (unsigned int *) arg, 
				sizeof(unsigned int))) {
				rc = -EFAULT;
				break;
			}

			/* Board number is expected to start at 1 (not zero) */
			if(index > nmegaport || index < 1) {
				rc = -EINVAL;
				break;
			}

			/* build eqnbrdstatus structure */

			brdstat.brd_nbr = index;
			index--; 	/* bias board for internal indices */

			sprintf(brdstat.gbl_version_str, 
					"%s Version %s", Version, VERSNUM);

			strncpy(brdstat.gbl_copyright_str,
					Copyright, BRDSTAT_STRLEN);
		
			brdstat.gbl_neqnbrds = nmegaport;
			brdstat.gbl_neqnicps = nicps;

			memcpy(&brdstat.brd_dev, &meg_dev[index], 
					sizeof(struct mpdev));

			/* update brd_cst */
			mpc = &meg_chan[index*NCHAN_BRD];
			for (index=0; index < MAXBRDCHNL; index++,mpc++) {
				brd_cst[index].cst_in = mpc->mpc_input;
				brd_cst[index].cst_out = mpc->mpc_output;
			}

			memcpy(&brdstat.brd_cst, brd_cst, sizeof(brd_cst));

			/* copy out the return structure */
			if (eqn_to_user((unsigned char *) arg, &brdstat,
				sizeof(struct eqnbrdstatus))) {
				rc = -EFAULT;
				break;
			}

			break;
			}

      case EQNCHSTATLEN: 
			length = sizeof(struct eqnchstatus); 
			if (eqn_to_user((unsigned int *) arg, &length,
				sizeof(unsigned int))) 
				rc = -EFAULT;

			break;  
      case EQNBRDSTATCLR:
			if (eqn_from_user(&length, (unsigned int *)arg, 
				sizeof(unsigned int))) {
				rc = -EFAULT; 
				break;
			}

			/* Board number is expected to start at 1 (not zero) */
			if(length > nmegaport || length < 1)
				rc = -EINVAL;
			length--; /* make board biased to zero for internal 
					indices */
			mpc = &meg_chan[length*NCHAN_BRD];
			for (length=0; length < MAXBRDCHNL; length++,mpc++) {
				mpc->mpc_input = 0;
				mpc->mpc_output = 0;
			}
			break;

      case EQNCHSTATUS: {
			struct	eqnchstatus	chstat;
			int	index;

			if (eqn_from_user(&index, (unsigned int *)arg, 
				sizeof(unsigned int))) {
				restore_flags(flags);
				return(-EFAULT);
			}

			d = SSTMINOR(index);
			if((d > nmegaport*NCHAN_BRD) || (nmegaport == 0)
				|| (d < 0)) {
				restore_flags(flags);
				return(-ENODEV);
			}
        		mpc = &meg_chan[d];
			if ((mpd = mpc->mpc_mpd) == (struct mpdev *) NULL){
				restore_flags(flags);
				return(-ENODEV);
			}
			nchan = MPCHAN(d);
			if (mpd >= &meg_dev[nmegaport] || mpd->mpd_alive == 0 ||
				nchan >= (int)mpd->mpd_nchan){
				restore_flags(flags);
				return(-ENODEV);
			}

			/* build eqnchstatus structure */

			chstat.ec_nbr = index;

			if (mpc->flags & ASYNC_INITIALIZED){
				chstat.c_iflag = 
					mpc->mpc_tty->termios->c_iflag;
				chstat.c_oflag = 
					mpc->mpc_tty->termios->c_oflag;
				chstat.c_cflag = 
					mpc->mpc_tty->termios->c_cflag;
				chstat.c_lflag = 
					mpc->mpc_tty->termios->c_lflag;
			}
			else {
				chstat.c_iflag = 0;
				chstat.c_oflag = 0; 
				chstat.c_cflag = 0;
				chstat.c_lflag = 0; 
			}

			memcpy(&chstat.ec_mpc, mpc, sizeof(struct mpchan));

			win16 = mpc->mpc_mpd->mpd_nwin;
			if(win16)
			   mega_push_win(mpc, 0);

			memcpy(&chstat.ec_cin, mpc->mpc_icpi,
				sizeof(union input_regs_u));
			memcpy(&chstat.ec_cout, mpc->mpc_icpo,
				sizeof(union output_regs_u));

			if(win16)
				mega_pop_win(3);

			if(ISRAMP(mpc)){
				memcpy(&chstat.ec_mpa, 
					&mpc->mpc_icp->sspcb->mpa[mpc->mpc_lmxno],
                    			sizeof(struct mpa_struct));

				memcpy(&chstat.ec_slot,
					(mpc->mpc_icp->slot_ctrl_blk + 
					((mpc->mpc_chan) % 64)),
                    				sizeof(struct slot_struct));
			} else {
				memset(&chstat.ec_mpa, 0, 
						sizeof(struct mpa_struct));
				memset(&chstat.ec_slot, 0, 
						sizeof(struct slot_struct));
			}

			memset(&chstat.ec_lpslave, 0, 
						sizeof(struct megaslave));

			/* copy out the return structure */
			if (eqn_to_user((unsigned char *) arg, &chstat,
				sizeof(struct eqnchstatus))) {
				rc = -EFAULT;
				break;
			}

			break;
			}

      case EQNCHBUFSTAT: {
			struct eqnbufstatus chb,*chbfst = &chb;
			icpiaddr_t icpi;
			icpqaddr_t icpq;
			icpbaddr_t icpb;
			uchar_t c;
			if (eqn_from_user(&chb, (unsigned char *)arg, 
				sizeof(chb))) {
				restore_flags(flags);
				return(-ENODEV);
			}
			d = SSTMINOR(chb.ebf_nbr);
			if((d > nmegaport*NCHAN_BRD) || (nmegaport == 0)
				|| (d < 0)) {
				restore_flags(flags);
				return(-ENODEV);
			}
        		mpc = &meg_chan[d];
			if ((mpd = mpc->mpc_mpd) == (struct mpdev *) NULL){
				restore_flags(flags);
				return(-ENODEV);
			}
			nchan = MPCHAN(d);
			if (mpd >= &meg_dev[nmegaport] || mpd->mpd_alive == 0 ||
				nchan >= (int)mpd->mpd_nchan){
				restore_flags(flags);
				return(-ENODEV);
			}
			win16 = mpc->mpc_mpd->mpd_nwin;
			if(win16)
			   mega_push_win(mpc, 0);

			icpi = mpc->mpc_icpi;

			icpb = (rx_locks & LOCK_A) ? &icpi->ssp.cin_bank_b : &icpi->ssp.cin_bank_a;
  			if (!(icpb->ssp.bank_events & EV_REG_UPDT)) 
  				frame_wait(mpc,2); /* make sure regs are valid */ 
			chbfst->ebf_incount = rx_rcv_cnt;
			icpq = (icpqaddr_t)&mpc->mpc_icpo->ssp.cout_q0;
			chbfst->ebf_outcount = Q_data_count; 
/* above lines mimic this code from streams: (Monty)
			chbfst->ebf_incount = mpc->mpc_rx_bank.ssp.ssp.bank_num_chars; 
			chbfst->ebf_outcount = mpc->mpc_icpo->ssp.cout_q0.q_data_count; 
*/
			if (( mpc->mpc_block)
 				|| (mpc->mpc_cin->ssp.cin_int_flags & 0x40 ))
				chbfst->ebf_inflow_state = 0;
			else
				chbfst->ebf_inflow_state = 1;
			chbfst->ebf_outflow_state = !(mpc->mpc_cin->ssp.cin_int_flags & 0x20);
			c = eqn_cs_get( mpc );
			chbfst->ebf_insigs = c & 0x0f;
			chbfst->ebf_outsigs = (unsigned) (c & 0x30) >> 4;
			if(win16)
				mega_pop_win(4);
			if (eqn_to_user((unsigned char *) arg, &chb,
				sizeof(chb))) {
				restore_flags(flags);
				return(-EFAULT);
			}
			} /* end of EQNCHBUFSTAT */
			break;

      case EQNCHIXON:
      case EQNCHXIXON:
			if (eqn_from_user(&length, (unsigned int *)arg, 
				sizeof(unsigned int))) {
				restore_flags(flags);
				return(-EFAULT);
			}
			
			d = SSTMINOR(length);
			if((d > nmegaport*NCHAN_BRD) || (nmegaport == 0)
				|| (d < 0)) {
				restore_flags(flags);
				return(-ENODEV);
			}
        		mpc = &meg_chan[d];
			if ((mpd = mpc->mpc_mpd) == (struct mpdev *) NULL){
				restore_flags(flags);
				return(-ENODEV);
			}
			nchan = MPCHAN(d);
			if (mpd >= &meg_dev[nmegaport] || mpd->mpd_alive == 0 ||
				nchan >= (int)mpd->mpd_nchan){
				restore_flags(flags);
				return(-ENODEV);
			}

			win16 = mpc->mpc_mpd->mpd_nwin;
			if(win16)
			   mega_push_win(mpc, 0);
			if ( cmd == EQNCHIXON ) {
				mpc->mpc_param |= IXONSET;	/* force ixon */
				megaparam(d);
			}
			else {  /* EQNCHXIXON */
				mpc->mpc_param &= ~IXONSET;	/* cancel forcing ixon */
				megaparam(d);
			}
			if(win16)
				mega_pop_win(5);
			break;

      case EQNCHLOOPON:
      case EQNCHLOOPOFF:

			if (eqn_from_user(&length, (unsigned int *)arg, 
				sizeof(unsigned int))) {
				restore_flags(flags);
				return(-EFAULT);
			}

			d = SSTMINOR(length);
			if((d > nmegaport*NCHAN_BRD) || (nmegaport == 0)
				|| (d < 0)) {
				restore_flags(flags);
				return(-ENODEV);
			}
        		mpc = &meg_chan[d];
			if ((mpd = mpc->mpc_mpd) == (struct mpdev *) NULL){
				restore_flags(flags);
				return(-ENODEV);
			}
			nchan = MPCHAN(d);
			if (mpd >= &meg_dev[nmegaport] || mpd->mpd_alive == 0 ||
				nchan >= (int)mpd->mpd_nchan){
				restore_flags(flags);
				return(-ENODEV);
			}

			win16 = mpc->mpc_mpd->mpd_nwin;
			if(win16)
			   mega_push_win(mpc, 0);
  	      		if ( cmd == EQNCHLOOPON ) {
/* RAMP START --------------------------------------------- */
				if (ISRAMP(mpc)) {
              				if (ramp_set_loop_back(mpc)) {
		            			if(win16)
		               				mega_pop_win(46);
						restore_flags(flags);
						return(-ENXIO);
              				}
		            		if(win16)
		               			mega_pop_win(47);
loopon_wait:
					restore_flags(flags); 
					interruptible_sleep_on(
						&mpc->mpc_mpa_slb);
					save_flags(flags);
					cli();
					if eqn_fatal_signal {
						restore_flags(flags); 
						return(-ERESTARTSYS);
					}
					if (mpc->mpc_mpa_stat 
						& MPA_SET_LOOP_BACK_ERROR){
						restore_flags(flags);
						return(-ENXIO);
					}
					else if (mpc->mpc_mpa_stat 
						== MPA_INITG_SET_LOOP_BACK)
						goto loopon_wait;
					if(win16)
			   			mega_push_win(mpc, 0);
				}
				else {
/* RAMP END ----------------------------------------------- */
					/* int/ext loopback, command enable */
					mpc->mpc_cout->ssp.cout_lmx_cmd |= 0x43;
					frame_wait( mpc, 6 );
					/* clear cmd enable */
					mpc->mpc_cout->ssp.cout_lmx_cmd &= 
						~0x40;
				}
				mpc->mpc_flags |= MPC_LOOPBACK;
			}
			else {  /* EQNCHLOOPOFF */
/* RAMP START --------------------------------------------- */
				if (ISRAMP(mpc)) {
              				if (ramp_clr_loop_back(mpc)) {
				     		if(win16)
					     		mega_pop_win(6);
						restore_flags(flags);
						return(-ENXIO);
			     		}
		        		if(win16)
		           			mega_pop_win(49);
loopoff_wait:
					restore_flags(flags); 
					interruptible_sleep_on(
						&mpc->mpc_mpa_clb);
					save_flags(flags);
					cli();
					if eqn_fatal_signal {
						restore_flags(flags); 
						return(-ERESTARTSYS);
					}
					if (mpc->mpc_mpa_stat & 
						MPA_CLR_LOOP_BACK_ERROR){
						restore_flags(flags);
						return(-ENXIO);
              				}
					else if (mpc->mpc_mpa_stat 
						== MPA_INITG_SET_LOOP_BACK)
						goto loopoff_wait;
					if(win16)
			   			mega_push_win(mpc, 0);
				} else {
/* RAMP END ----------------------------------------------- */
					mpc->mpc_cout->ssp.cout_lmx_cmd &= 
						~0x03;   /* int/ext loopback */
					mpc->mpc_cout->ssp.cout_lmx_cmd 
						|= 0x40;   /* command enable */
					frame_wait( mpc, 6 );
					/* clear cmd enable */
					mpc->mpc_cout->ssp.cout_lmx_cmd 
						&= ~0x40;
				}
				mpc->mpc_flags &= ~MPC_LOOPBACK;
			}
			if(win16)
				mega_pop_win(6);
			break;

      case EQNCHRESUME:

			if (eqn_from_user(&length, (unsigned int *)arg, 
				sizeof(unsigned int))) {
				restore_flags(flags);
				return(-EFAULT);
			}

			d = SSTMINOR(length);
			if((d > nmegaport*NCHAN_BRD) || (nmegaport == 0)
				|| (d < 0)) {
				restore_flags(flags);
				return(-ENODEV);
			}
        		mpc = &meg_chan[d];
			if ((mpd = mpc->mpc_mpd) == (struct mpdev *) NULL){
				restore_flags(flags);
				return(-ENODEV);
			}
			nchan = MPCHAN(d);
			if (mpd >= &meg_dev[nmegaport] || mpd->mpd_alive == 0 ||
				nchan >= (int)mpd->mpd_nchan){
				restore_flags(flags);
				return(-ENODEV);
			}

     	   		eqnx_start( mpc->mpc_tty);
			icpi = mpc->mpc_icpi;
  			icpi->ssp.cin_inband_flow_ctrl &= ~0x01;
			break;
	case EQNMRASTATUS:
	case EQNMRARINGON:
	case EQNMRARINGOFF:

	 {

		icpgaddr_t icpg;
		icpiaddr_t icpi;
		icpbaddr_t icpb;
		int	tmpint;

			if (eqn_from_user(&length, (unsigned int *)arg, 
				sizeof(unsigned int))) {
				restore_flags(flags);
				return(-EFAULT);
			}

			d = SSTMINOR(length);
			if((d > nmegaport*NCHAN_BRD) || (nmegaport == 0)
				|| (d < 0)) {
				restore_flags(flags);
				return(-ENODEV);
			}
        		mpc = &meg_chan[d];
			if ((mpd = mpc->mpc_mpd) == (struct mpdev *) NULL){
				restore_flags(flags);
				return(-ENODEV);
			}
			nchan = MPCHAN(d);
			if (mpd >= &meg_dev[nmegaport] || mpd->mpd_alive == 0 ||
				nchan >= (int)mpd->mpd_nchan){
				restore_flags(flags);
				return(-ENODEV);
			}

			/* Always use first port on first LMX for status */
		d = d & 0xffffffc0; 

		mpc = &meg_chan[d];

		if ((mpc->mpc_mpd->mpd_id & 0xF8) < SSP2_ASIC)
			icpg = (icpgaddr_t)mpc->mpc_icpo;
		else
			icpg = (icpgaddr_t)((unsigned long)(mpc->mpc_icpi) + 0x400);

  		icpi = mpc->mpc_icpi;
		icpb = (rx_locks & LOCK_A) ? 
			&icpi->ssp.cin_bank_b : &icpi->ssp.cin_bank_a;

		arg += sizeof(unsigned int); /* skip over major/minor num */ 

		win16 = mpc->mpc_mpd->mpd_nwin;

		if(win16)
		   mega_push_winicp(mpc->mpc_mpd, mpc->mpc_icpno);

#if TRACKICPG
		printf("\nMRASTAT: mpd=%x icpno=%x icp=%x icpg=%x\n",
			mpc->mpc_mpd, mpc->mpc_icpno, mpc->mpc_icp, icpg);
		printf("Gicp_initiate = %x\n", Gicp_initiate, tmpint);
#endif

     	if (!(icpb->ssp.bank_events & EV_REG_UPDT)) 
     		frame_wait(mpc,2); /* make sure regs are valid */ 
		if ((mpc->mpc_mpd->mpd_id & 0xF8) < SSP2_ASIC)
			tmpint = Gicp_initiate | (i_gicp_attn << 8) | (icpb->ssp.bank_sigs << 16);
		else  
			tmpint = Gicp_initiate | (icpb->ssp.bank_sigs << 16);

		switch(cmd) {
			case EQNMRARINGON:
				Gicp_initiate |= RNG_CLK_ON;
				break;
			case EQNMRARINGOFF:
				Gicp_initiate &= ~RNG_CLK_ON;
				break;
		}

		if(win16)
		      mega_pop_win(43);

		if (eqn_to_user((int *) arg, &tmpint, sizeof(int))) {
			 restore_flags(flags); 
			 return(-EFAULT); 
		}

		}
		break;
	case EQNCHRESET:
	{
		icpiaddr_t icpi;
		icpbaddr_t icpb;

		if (eqn_from_user(&length, (unsigned int *)arg, 
			sizeof(unsigned int))) {
			 restore_flags(flags); 
			 return(-EFAULT); 
		}
			
		d = SSTMINOR(length);
		if((d > nmegaport*NCHAN_BRD) || (nmegaport == 0)
			|| (d < 0)) {
			restore_flags(flags);
			return(-ENODEV);
		}
#ifdef DEBUG
		printk("EQNCHRESET: d = %d", d);
#endif
        	mpc = &meg_chan[d];
		if ((mpd = mpc->mpc_mpd) == (struct mpdev *) NULL){
			restore_flags(flags);
			return(-ENODEV);
		}
		nchan = MPCHAN(d);
		if (mpd >= &meg_dev[nmegaport] || mpd->mpd_alive == 0 ||
			nchan >= (int)mpd->mpd_nchan){
			restore_flags(flags);
			return(-ENODEV);
		}
		/* PatH - Multimodem  reset */
		if ((mpd->mpd_id & 0xFC) == 0x98 || (mpd->mpd_id & 0xFC) == 0x9C )
		{
  			icpi = mpc->mpc_icpi;
			icpb = (rx_locks & LOCK_A) ? 
				&icpi->ssp.cin_bank_b : &icpi->ssp.cin_bank_a;
			GET_CTRL_SIGS(mpc, cur);
			/* Turn on reset signal */
			cur |= TX_HFC_2 | TX_CNT_2;
			if((rx_ctrl_sig & 0x2020) == 0x2020)
				cur |= 0x8000;
	  		else
			 	cur &= ~0x8000;
	  		cur ^= TX_SND_CTRL_TG;
			SET_CTRL_SIGS(mpc, cur);
			/* Need to pause for 150 ms */
			eqnx_delay( HZ*15 / 100);
			/* Turn off reset signal */
	  		cur &= ~(TX_HFC_2 | TX_CNT_2);
	  		cur ^= TX_SND_CTRL_TG;
			SET_CTRL_SIGS(mpc, cur);
		}	

/* RAMP START --------------------------------------------- */
		if (!(ISRAMP(mpc))){
			restore_flags(flags);
			return(0);
		}
		/* If modem not registered, don't call reset */
		if (!(mpc->mpc_mpa_stat & MPA_UART_CFG_DONE)){
			restore_flags(flags);
			return(0);
		}
		win16 = mpc->mpc_mpd->mpd_nwin;
		if(win16)
		   mega_push_win(mpc, 0);
		ramp_hard_reset(mpc);
		if(win16)
			mega_pop_win(61);
reset_wait:
		restore_flags(flags); 
		interruptible_sleep_on(&mpc->mpc_mpa_rst);
		save_flags(flags);
		cli();
		if eqn_fatal_signal {
			restore_flags(flags); 
			return(-ERESTARTSYS);
		}
		if (mpc->mpc_mpa_stat & MPA_HARD_RESET_ERROR){
			restore_flags(flags);
#ifdef DEBUG
printk("hard_reset_error for device %d\n", d);
#endif
			return(-ENXIO);
		}
		else if (mpc->mpc_mpa_stat 
				== MPA_INITG_HARD_RESET)
			goto reset_wait;
/* RAMP END ----------------------------------------------- */
	}
		break;
	default:
		rc = -ENOIOCTLCMD;
		break;
	}
	break;
	case 1:
	case 2:
	switch (cmd){
	case EQNCHMONSET:
	case EQNCHMONCLR:{
		int d;
		if (eqn_from_user(&length, (unsigned int *)arg, 
				sizeof(unsigned int))) {
			 restore_flags(flags); 
			 return(-EFAULT); 
		}
			
		d = SSTMINOR(length);
		if((d > nmegaport*NCHAN_BRD) || (nmegaport == 0)
			|| (d < 0)) {
			restore_flags(flags);
			return(-ENODEV);
		}
        	mpc = &meg_chan[d];
		if ((mpd = mpc->mpc_mpd) == (struct mpdev *) NULL){
			restore_flags(flags);
			return(-ENODEV);
		}
		nchan = MPCHAN(d);
		if (mpd >= &meg_dev[nmegaport] || mpd->mpd_alive == 0 ||
			nchan >= (int)mpd->mpd_nchan){
				restore_flags(flags);
				return(-ENODEV);
		}
		if (cmd == EQNCHMONSET){
			dscope[minor -1].chan = d;
			if (minor & 1)
				mpc->mpc_flags |= MPC_DSCOPER;
			else
				mpc->mpc_flags |= MPC_DSCOPEW;
		}
		else{
			dscope[minor - 1].chan = -1;
			if (minor & 1)
				mpc->mpc_flags &= ~MPC_DSCOPER;
			else
				mpc->mpc_flags &= ~MPC_DSCOPEW;
		}
		}
		break;
	default:
		rc = -ENOIOCTLCMD;
		break;
	}
	}
#ifdef DEBUG
	printk("eqnx_diagioctl: ret val %d for cmd %x for device %d\n", 
		rc, (unsigned int) cmd, MINOR(ip->i_rdev));
#endif
	restore_flags(flags);
	return(rc);
}
/* Pat */
#if (EQNX_VERSION_CODE > 131327)
static ssize_t eqnx_diagread(struct file *fp, char *buf, size_t count, loff_t *fpos)
#else		
static int eqnx_diagread(struct inode *ip, struct file *fp, char *buf, 
		int count)
#endif
{
	int d, cc,n, ccm,rc;
#if (EQNX_VERSION_CODE > 131327)
	struct dentry *dent;
	struct inode *ip;
	
	dent = fp->f_dentry;
	ip = dent->d_inode;
#endif
	d = MINOR(ip->i_rdev) - 1;
	if ((d != 0) && (d != 1))
		return(-ENODEV);
	cc = (dscope[d].next - dscope[d].q.q_ptr) & (DSQMASK);
	if (!(cc)){
		if (fp->f_flags & O_NONBLOCK) {
			return(0);
		}
		interruptible_sleep_on(&dscope[d].scope_wait);
		cc = (dscope[d].next - dscope[d].q.q_ptr) & (DSQMASK);
	}
	rc = ccm = MIN(cc, count);
	while (ccm > 0) {
		n= MIN(ccm,dscope[d].q.q_end - dscope[d].q.q_ptr + 1);
                if (eqn_to_user(buf, (unsigned char *) (dscope[d].q.q_addr 
			+ dscope[d].q.q_ptr), (sizeof(char) * n)))
			return (-EFAULT);
                buf += n;
                ccm -= n;
                dscope[d].q.q_ptr += n;
                if (dscope[d].q.q_ptr > dscope[d].q.q_end)
                        dscope[d].q.q_ptr = dscope[d].q.q_begin;
	}
	return(rc);
}
STATIC
int eqnx_diagopen(struct inode *ip, struct file *fp)
{
	int d;

	d = MINOR(ip->i_rdev);
	if (!d)
	{
		MOD_INC_USE_COUNT;
		return(0);
	}
	d--;
	if ((d != 0) && (d != 1))
		return(-ENODEV);
	if (dscope[d].open)
		return(-EBUSY);

	dscope[d].q.q_addr = dscope[d].buffer;
	dscope[d].q.q_begin = 0;
	dscope[d].q.q_end = DSQMASK;
	dscope[d].q.q_ptr = dscope[d].q.q_begin;
	dscope[d].next = dscope[d].q.q_begin;
	dscope[d].open = true;
	dscope[d].chan = -1;

#if	(LINUX_VERSION_CODE < 132096)
	/* kernels before 2.4 */
	init_waitqueue(&dscope[d].scope_wait);
#else
	/* 2.4 kernels and after */
	init_waitqueue_head(&dscope[d].scope_wait);
#endif

	MOD_INC_USE_COUNT;
	return(0);
}
STATIC
int eqnx_diagclose(struct inode *ip, struct file *fp)
{
	int d;

	MOD_DEC_USE_COUNT;
	d = MINOR(ip->i_rdev);
	if (!d)
		return(0);
	d--;
	if ((d != 0) && (d != 1))
		return(0);
	dscope[d].chan = -1;
	dscope[d].open = false;

	return(0);
}
static int SSTMINOR(dev_t dev)
{ 
	int i, major = MAJOR(dev); 
	if ((major < din_num[0]) || (major > diag_num))
		return(-1);
#if (LINUX_VERSION_CODE < 132096)
	if (major < dout_num[0]){ 
#endif
		for ( i=0; i < (maxbrd /2); i++)
			if (major == din_num[i])
				break;
#if (LINUX_VERSION_CODE < 132096)
	}else {
		for ( i=0; i < (maxbrd /2); i++)
			if (major == dout_num[i])
				break;
	}
#endif
	return((i * 256) + MINOR(dev)); 
}
static void inline sst_write( struct mpchan *mpc, unsigned char *buf, int count)
{
	int c, nx, win16, datascope = 0;
	volatile icpoaddr_t icpo;
	volatile icpgaddr_t icpg;
	volatile icpqaddr_t icpq;
	unsigned char oldreg;
#ifdef MIDNIGHT
        unsigned char *dst_addr;
        int align, bytes;
#endif


	icpo = mpc->mpc_icpo;
	icpg = (icpgaddr_t)icpo;
	icpq = &icpo->ssp.cout_q0;
	win16 = mpc->mpc_mpd->mpd_nwin;

	/*if(win16)
		mega_push_win(mpc, 0);*/
	/*block_count = icpq->ssp.q_block_count * 64;
	space = mpc->mpc_txq.q_size - 
		(mpc->mpc_count + Q_data_count) -1 ;*/
	/*while(1){*/
		/*c = MIN(space, count);
		if (c <= 0)
			break;*/
		c = count;
		mpc->mpc_flags |= MPC_BUSY;
		if ((mpc->mpc_flags & MPC_DSCOPEW)
			&& (dscope[1].chan == mpc - meg_chan)) datascope=1;
		/*
	 	* Blindingly fast block copy direct from 
	 	* driver buffer to on-board buffers.
	 	*/

		if(win16)
			mega_push_win(mpc, 2);
		nx = MIN(c, (mpc->mpc_txq.q_end - mpc->mpc_txq.q_ptr +1));
#ifdef MIDNIGHT
                dst_addr = mpc->mpc_txq.q_addr + mpc->mpc_txq.q_ptr;
                align = ((unsigned long)dst_addr) % 4;
                if (align) {
                  bytes = MIN(nx,(4-align));
                  memcpy(dst_addr, buf, bytes);
                }
                else {
                  bytes = 0;
                }
                if (nx > bytes) {
                  memcpy(dst_addr+bytes, buf+bytes, nx-bytes);
                }
#else
		memcpy((mpc->mpc_txq.q_addr + mpc->mpc_txq.q_ptr), 
			buf, nx);
#endif
		mpc->mpc_txq.q_ptr += nx;
		if(mpc->mpc_txq.q_ptr > mpc->mpc_txq.q_end)
			mpc->mpc_txq.q_ptr = mpc->mpc_txq.q_begin;
		if (datascope) {
			int room,move_cnt;
			int d=1;
			char *ubase = buf;
			room = DSQSIZE - ((dscope[d].next - dscope[d].q.q_ptr) 
				& (DSQMASK));
			if (nx > room)
				dscope[d].status |= DSQWRAP;
                	else
                        	room = nx;
			while (room > 0) {
				move_cnt = MIN(room,
					dscope[d].q.q_end - dscope[d].next + 1);
				memcpy(dscope[d].q.q_addr + dscope[d].next, 
					ubase, move_cnt);
                        	dscope[d].next += move_cnt;
                        	if (dscope[d].next > dscope[d].q.q_end)
                                	dscope[d].next = dscope[d].q.q_begin;
                                ubase += move_cnt;
                                room -= move_cnt;
			} /* while room */
			wake_up_interruptible(&dscope[d].scope_wait);
		} /* if MPC_DSCOPE */
		buf += nx;
		count -= nx;
		c -= nx;
		if( c ) {
#ifdef MIDNIGHT
                  dst_addr = mpc->mpc_txq.q_addr + mpc->mpc_txq.q_ptr;
                  align = ((unsigned long)dst_addr) % 4;
                  if (align) {
                        bytes = MIN(c,(4-align));
                        memcpy(dst_addr, buf, bytes);
                  }
                  else {
                        bytes = 0;
                  }
                  if (c > bytes) {
                        memcpy(dst_addr+bytes, buf+bytes, c-bytes);
                  }
#else
			memcpy(mpc->mpc_txq.q_addr + mpc->mpc_txq.q_ptr, 
				buf, c);
#endif
			mpc->mpc_txq.q_ptr += c;
			if(mpc->mpc_txq.q_ptr > mpc->mpc_txq.q_end)
				mpc->mpc_txq.q_ptr = mpc->mpc_txq.q_begin;
			if (datascope) {
				int room,move_cnt;
				int d=1;
				char *ubase = buf;
				room = DSQSIZE - ((dscope[d].next - 
					dscope[d].q.q_ptr) & (DSQMASK));
				if (c > room)
					dscope[d].status |= DSQWRAP;
	                	else
	                        	room = c;
				while (room > 0) {
					move_cnt = MIN(room,dscope[d].q.q_end - 
						dscope[d].next + 1);
					memcpy(dscope[d].q.q_addr + 
						dscope[d].next, ubase,move_cnt);
	                        	dscope[d].next += move_cnt;
	                        	if (dscope[d].next > dscope[d].q.q_end)
	                                	dscope[d].next = 
							dscope[d].q.q_begin;
	                                ubase += move_cnt;
	                                room -= move_cnt;
				} /* while room */
				wake_up_interruptible(&dscope[d].scope_wait);
			} /* if MPC_DSCOPE */
			buf += c;
			count -= c;
		}
		if(win16)
			mega_pop_win(24);
	if(win16)
		mega_push_win(mpc, 0);
		oldreg = tx_cpu_reg;
		tx_cpu_reg |= TX_SUSP;
		cur_chnl_sync(mpc);
		if(Q_data_count < 9) {
			if ((icpo->ssp.cout_int_save_togl & 0x4) == 
				(icpo->ssp.cout_cpu_req & 0x4)){
				tx_cpu_reg ^= (CPU_SND_REQ);
				mpc->mpc_cout_events &= ~EV_CPU_REQ_DN;
			}
		}
		Q_data_count += (c + nx);
		/*space -= (c + nx);*/
#ifdef DEBUG
printk("sst_write: wrote %d chars for device %d\n", (c + nx), 
		SSTMINOR(mpc->mpc_tty->device));
#endif
		if (!(oldreg & TX_SUSP))
			tx_cpu_reg &= ~TX_SUSP;
		mpc->mpc_output += (c + nx);
		tx_cie |= (ENA_TX_EMPTY_Q0| ENA_TX_LOW_Q0);
	/*}*/
	if(win16)
		mega_pop_win(25);
	/*if (count < block_count){
#ifdef DEBUG
	printk("sst_write: calling write_wakeup for device = %d\n", 
		SSTMINOR(mpc->mpc_tty->device));
#endif
		if ((tty->flags & (1 << TTY_DO_WRITE_WAKEUP)) && tty->ldisc.write_wakeup)
			(tty->ldisc.write_wakeup)(tty);
		wake_up_interruptible(&tty->write_wait);
	}*/
}
int register_eqnx_dev(void)
{
	int i,j;
	int num_drivers;

	num_drivers = MAX(1, (((nmegaport - 1)/2) + 1));

	/* Initialize the tty_driver structure */
#ifdef DEBUG
	printk("EQNX: registering the driver\n");
#endif
	if(((struct tty_driver *) eqnx_driver = 
		(struct tty_driver *)vmalloc(sizeof(struct tty_driver) 
			* num_drivers)) 
			== (struct tty_driver *) NULL){
		printk("EQUINOX: Failed to allocate virtual address space of size %d for eqnx_driver\n", (unsigned int)(sizeof(struct tty_driver) * num_drivers));
		return(-1);
	}
	memset(eqnx_driver, 0, (sizeof(struct tty_driver) 
			* num_drivers));

#if (LINUX_VERSION_CODE < 132096)
	if(((struct tty_driver *) eqnx_callout_driver = 
		(struct tty_driver *)vmalloc(sizeof(struct tty_driver) 
			* num_drivers)) 
			== (struct tty_driver *) NULL){
		printk("EQUINOX: Failed to allocate virtual address space of size %d for eqnx_callout_driver\n", (unsigned int)(sizeof(struct tty_driver) * num_drivers));
		return(-1);
	}
	memset(eqnx_callout_driver, 0, (sizeof(struct tty_driver) 
			* num_drivers));
#endif	

	if(((struct tty_struct **) eqnx_ttys = 
		(struct tty_struct **)vmalloc(sizeof(struct tty_struct *) 
			* nmegaport * NCHAN_BRD)) 
			== (struct tty_struct **) NULL){
		printk("EQUINOX: Failed to allocate virtual address space of size %d for eqnx_ttys\n", (unsigned int)(sizeof(struct tty_struct *) * nmegaport * NCHAN_BRD));
		return(-1);
	}
	memset(eqnx_ttys, 0, (sizeof(struct tty_struct *) 
			* nmegaport * NCHAN_BRD));
	if(((struct termios **) eqnx_termios = 
		(struct termios **)vmalloc(sizeof(struct termios *) 
			* nmegaport * NCHAN_BRD)) == (struct termios **) NULL){
		printk("EQUINOX: Failed to allocate virtual address space of size %d for eqnx_termios\n", (unsigned int)(sizeof(struct termios *) * nmegaport * NCHAN_BRD));
		return(-1);
	}

	memset(eqnx_termios, 0, (sizeof(struct termios *) 
			* nmegaport * NCHAN_BRD));
	
	if(((struct termios **) eqnx_termioslocked = 
		(struct termios **)vmalloc(sizeof(struct termios *) 
			* nmegaport * NCHAN_BRD)) == (struct termios **) NULL){
		printk("EQUINOX: Failed to allocate virtual address space of size %d for eqnx_termioslocked\n", 
		(unsigned int)(sizeof(struct termios *) * nmegaport * NCHAN_BRD));
		return(-1);
	}
	memset(eqnx_termioslocked, 0, (sizeof(struct termios *) 
			* nmegaport * NCHAN_BRD));
	for (i =0, j = 0; i < nmegaport; i += 2, j++){
		memset(&eqnx_driver[j], 0, sizeof(struct tty_driver));
#if (LINUX_VERSION_CODE < 132096)
		memset(&eqnx_callout_driver[j], 0, sizeof(struct tty_driver));
		eqnx_callout_driver[j].magic = 
#endif
			eqnx_driver[j].magic 
			= TTY_DRIVER_MAGIC;
		eqnx_driver[j].name = "Eqnx tty";
		eqnx_driver[j].major = 0;
#if (LINUX_VERSION_CODE < 132096)
		eqnx_callout_driver[j].minor_start = 
#endif
			eqnx_driver[j].minor_start = 0;
		if (nmegaport >= (i + 2)){
#if (LINUX_VERSION_CODE < 132096)
			eqnx_callout_driver[j].num = 
#endif
				eqnx_driver[j].num = 256;
		}else{
#if (LINUX_VERSION_CODE < 132096)
			eqnx_callout_driver[j].num = 
#endif
				eqnx_driver[j].num = 128;
		}
#if (LINUX_VERSION_CODE < 132096)
		eqnx_callout_driver[j].type = 
#endif
			eqnx_driver[j].type 
			= TTY_DRIVER_TYPE_SERIAL;
		eqnx_driver[j].subtype = SERIAL_TYPE_NORMAL;
#if (LINUX_VERSION_CODE < 132096)
		eqnx_callout_driver[j].init_termios = 
#endif
			eqnx_driver[j].init_termios = eqnx_deftermios;
#if (LINUX_VERSION_CODE < 132096)
		eqnx_callout_driver[j].flags = 
#endif
			eqnx_driver[j].flags = TTY_DRIVER_REAL_RAW;
#if (LINUX_VERSION_CODE < 132096)
		eqnx_callout_driver[j].refcount = 
#endif
			eqnx_driver[j].refcount = 
			&eqnx_refcount;
#if (LINUX_VERSION_CODE < 132096)
		eqnx_callout_driver[j].table = 
#endif
			eqnx_driver[j].table = 
			&eqnx_ttys[j * 256];
#if (LINUX_VERSION_CODE < 132096)
		eqnx_callout_driver[j].termios = 
#endif
			eqnx_driver[j].termios = 
			&eqnx_termios[j * 256];
#if (LINUX_VERSION_CODE < 132096)
		eqnx_callout_driver[j].termios_locked = 
#endif
			eqnx_driver[j].termios_locked = 
				&eqnx_termioslocked[j * 256];
#if (LINUX_VERSION_CODE < 132096)
		eqnx_callout_driver[j].open = 
#endif
			eqnx_driver[j].open = eqnx_open;
#if (LINUX_VERSION_CODE < 132096)
		eqnx_callout_driver[j].close = 
#endif
			eqnx_driver[j].close = eqnx_close;
#if (LINUX_VERSION_CODE < 132096)
		eqnx_callout_driver[j].write = 
#endif
			eqnx_driver[j].write = eqnx_write;
#if (LINUX_VERSION_CODE < 132096)
		eqnx_callout_driver[j].put_char = 
#endif
			eqnx_driver[j].put_char = eqnx_put_char;
#if (LINUX_VERSION_CODE < 132096)
		eqnx_callout_driver[j].flush_chars = 
#endif
			eqnx_driver[j].flush_chars = eqnx_flush_chars;
#if (LINUX_VERSION_CODE < 132096)
		eqnx_callout_driver[j].write_room = 
#endif
			eqnx_driver[j].write_room = eqnx_write_room;
#if (LINUX_VERSION_CODE < 132096)
		eqnx_callout_driver[j].chars_in_buffer = 
#endif
			eqnx_driver[j].chars_in_buffer = eqnx_chars_in_buffer;
#if (LINUX_VERSION_CODE < 132096)
		eqnx_callout_driver[j].flush_buffer = 
#endif
			eqnx_driver[j].flush_buffer = eqnx_flush_buffer;
#if (LINUX_VERSION_CODE < 132096)
		eqnx_callout_driver[j].ioctl = 
#endif
			eqnx_driver[j].ioctl = eqnx_ioctl;
#if (LINUX_VERSION_CODE < 132096)
		eqnx_callout_driver[j].throttle = 
#endif
			eqnx_driver[j].throttle = eqnx_throttle;
#if (LINUX_VERSION_CODE < 132096)
		eqnx_callout_driver[j].unthrottle = 
#endif
			eqnx_driver[j].unthrottle = eqnx_unthrottle;
#if (LINUX_VERSION_CODE < 132096)
		eqnx_callout_driver[j].set_termios= 
#endif
			eqnx_driver[j].set_termios = eqnx_set_termios;
#if (LINUX_VERSION_CODE < 132096)
		eqnx_callout_driver[j].stop = 
#endif
			eqnx_driver[j].stop = eqnx_stop;
#if (LINUX_VERSION_CODE < 132096)
		eqnx_callout_driver[j].start= 
#endif
			eqnx_driver[j].start = eqnx_start;
#if (LINUX_VERSION_CODE < 132096)
		eqnx_callout_driver[j].hangup = 
#endif
			eqnx_driver[j].hangup = eqnx_hangup;
	
#if (LINUX_VERSION_CODE < 132096)
		/*
		 * The callout device is just like normal device except for
		 * major number and the subtype code.
		 */
		eqnx_callout_driver[j].name = "Eqnx cu";
		eqnx_callout_driver[j].major = 0;
		eqnx_callout_driver[j].subtype = SERIAL_TYPE_CALLOUT;
#endif
	
	}
	/* Register ALL callout devices before registering normal devices */
	/* Order is reversed; kernel assigns major nums "top down" */
	/* and we end on first board so that din_num, dout_num is valid */
	j--;
#if (LINUX_VERSION_CODE < 132096)
	for ( i=j ; i >= 0 ; i--){
		if ((dout_num[i] = tty_register_driver(&eqnx_callout_driver[i])) <= 0)
		   printk("Couldn't register eqnx_callout_driver for driver %d, error = %d\n", i, dout_num[i]);
	}
#endif
	/* Now register normal devices */
	for ( i=j ; i >= 0 ; i--){
		if ((din_num[i] = tty_register_driver(&eqnx_driver[i])) <= 0)
			printk("Couldn't register eqnx_driver for driver %d, error = %d\n", i,din_num[i]);

#ifdef LINUX
		else 
			ist.ctlr_array[2*i].ctlr_major = ist.ctlr_array[2*i + 1].ctlr_major = din_num[i];
#endif
		
	}
	return(0);
}
static void inline sst_write1(struct mpchan *mpc, int func_type)
{
	struct tty_struct *tty = mpc->mpc_tty;
	int space, c, nx, win16, datascope = 0;
	volatile icpoaddr_t icpo;
	volatile icpgaddr_t icpg;
	volatile icpqaddr_t icpq;
	unsigned char oldreg;
	int block_count;
#ifdef MIDNIGHT
        unsigned char *dst_addr;
        int align, bytes;
#endif

	icpo = mpc->mpc_icpo;
	icpg = (icpgaddr_t)icpo;
	icpq = &icpo->ssp.cout_q0;
	win16 = mpc->mpc_mpd->mpd_nwin;

	if(win16)
		mega_push_win(mpc, 0);
	block_count = icpq->ssp.q_block_count * 64;
	space = mpc->mpc_txq.q_size - 
		(mpc->mpc_count + Q_data_count) -1;
#ifdef DEBUG
printk("sst_write1: device = %d, q_size = %d, space = %d, xmit_cnt = %d, xmit_tail = %d, xmit_head = %d\n",
	SSTMINOR(tty->device),mpc->mpc_txq.q_size,space, mpc->xmit_cnt, mpc->xmit_tail, mpc->xmit_head);
#endif
	if ((func_type != EQNX_TXINT) && 
		((space < block_count) && (mpc->xmit_cnt >= block_count))){
			if(win16)
			mega_pop_win(25);
		return;
	}
	while(1){
		c = MIN(space, MIN(mpc->xmit_cnt,
				XMIT_BUF_SIZE - mpc->xmit_tail));
		if ( c <= 0 )
			break;
#ifdef DEBUG
	printk("sst_write1: device = %d, xmit_cnt = %d\n", 
		SSTMINOR(tty->device), mpc->xmit_cnt);
#endif
		mpc->mpc_flags |= MPC_BUSY;
		if ((mpc->mpc_flags & MPC_DSCOPEW)
			&& (dscope[1].chan == mpc - meg_chan)) datascope=1;
		/*
	 	* Blindingly fast block copy direct from 
	 	* driver buffer to on-board buffers.
	 	*/

		if(win16)
			mega_push_win(mpc, 2);
		nx = MIN(c, (mpc->mpc_txq.q_end - mpc->mpc_txq.q_ptr +1));
#ifdef MIDNIGHT
                dst_addr = mpc->mpc_txq.q_addr + mpc->mpc_txq.q_ptr;
                align = ((unsigned long)dst_addr) % 4;
                if (align) {
                  bytes = MIN(nx,(4-align));
                  memcpy(dst_addr, mpc->xmit_buf + mpc->xmit_tail , bytes);
                }
                else {
                  bytes = 0;
                }
                if (nx > bytes) {
                  memcpy(dst_addr+bytes, 
			mpc->xmit_buf + mpc->xmit_tail + bytes, nx-bytes);
                }
#else
		memcpy((mpc->mpc_txq.q_addr + mpc->mpc_txq.q_ptr), 
			mpc->xmit_buf + mpc->xmit_tail, nx);
#endif
		mpc->mpc_txq.q_ptr += nx;
		if(mpc->mpc_txq.q_ptr > mpc->mpc_txq.q_end)
			mpc->mpc_txq.q_ptr = mpc->mpc_txq.q_begin;
		if (datascope) {
			int room,move_cnt;
			int d=1;
			char *ubase = mpc->xmit_buf + mpc->xmit_tail;
			room = DSQSIZE - ((dscope[d].next - dscope[d].q.q_ptr) 
				& (DSQMASK));
			if (nx > room)
				dscope[d].status |= DSQWRAP;
                	else
                        	room = nx;
			while (room > 0) {
				move_cnt = MIN(room,
					dscope[d].q.q_end - dscope[d].next + 1);
				memcpy(dscope[d].q.q_addr + dscope[d].next, 
					ubase, move_cnt);
                        	dscope[d].next += move_cnt;
                        	if (dscope[d].next > dscope[d].q.q_end)
                                	dscope[d].next = dscope[d].q.q_begin;
                                ubase += move_cnt;
                                room -= move_cnt;
			} /* while room */
			wake_up_interruptible(&dscope[d].scope_wait);
		} /* if MPC_DSCOPE */
		mpc->xmit_tail += nx;
		mpc->xmit_tail &= XMIT_BUF_SIZE - 1;
		mpc->xmit_cnt -= nx;
		c -= nx;
		if( c ) {
#ifdef MIDNIGHT
                  dst_addr = mpc->mpc_txq.q_addr + mpc->mpc_txq.q_ptr;
                  align = ((unsigned long)dst_addr) % 4;
                  if (align) {
                        bytes = MIN(c,(4-align));
                        memcpy(dst_addr, mpc->xmit_buf + mpc->xmit_tail, bytes);
                  }
                  else {
                        bytes = 0;
                  }
                  if (c > bytes) {
                        memcpy(dst_addr+bytes, 
				mpc->xmit_buf + mpc->xmit_tail +bytes, c-bytes);
                  }
#else
			memcpy(mpc->mpc_txq.q_addr + mpc->mpc_txq.q_ptr, 
				mpc->xmit_buf + mpc->xmit_tail, c);
#endif
			mpc->mpc_txq.q_ptr += c;
			if(mpc->mpc_txq.q_ptr > mpc->mpc_txq.q_end)
				mpc->mpc_txq.q_ptr = mpc->mpc_txq.q_begin;
			if (datascope) {
				int room,move_cnt;
				int d=1;
				char *ubase = mpc->xmit_buf + mpc->xmit_tail;
				room = DSQSIZE - ((dscope[d].next - 
					dscope[d].q.q_ptr) & (DSQMASK));
				if (c > room)
					dscope[d].status |= DSQWRAP;
	                	else
	                        	room = c;
				while (room > 0) {
					move_cnt = MIN(room,dscope[d].q.q_end - 
						dscope[d].next + 1);
					memcpy(dscope[d].q.q_addr + 
						dscope[d].next, ubase,move_cnt);
	                        	dscope[d].next += move_cnt;
	                        	if (dscope[d].next > dscope[d].q.q_end)
	                                	dscope[d].next = 
							dscope[d].q.q_begin;
	                                ubase += move_cnt;
	                                room -= move_cnt;
				} /* while room */
				wake_up_interruptible(&dscope[d].scope_wait);
			} /* if MPC_DSCOPE */
			mpc->xmit_tail += c;
			mpc->xmit_tail &= XMIT_BUF_SIZE - 1;
			mpc->xmit_cnt -= c;
		}
		if(win16)
			mega_pop_win(24);
		oldreg = tx_cpu_reg;
		tx_cpu_reg |= TX_SUSP;
		cur_chnl_sync(mpc);
		if(Q_data_count < 9) {
			if ((icpo->ssp.cout_int_save_togl & 0x4) == 
				(icpo->ssp.cout_cpu_req & 0x4)){
				tx_cpu_reg ^= (CPU_SND_REQ);
				mpc->mpc_cout_events &= ~EV_CPU_REQ_DN;
			}
		}
		Q_data_count += (c + nx);
		space -= (c + nx);
#ifdef DEBUG
printk("sst_write1: wrote %d chars for device %d\n", (c + nx), 
		SSTMINOR(tty->device));
#endif
#ifdef DEBUG
printk("q data count %d, space %d for device %d\n", 
		Q_data_count,space, SSTMINOR(tty->device));
printk("xmit_cnt = %d, xmit_tail = %d, xmit_head = %d\n",
	mpc->xmit_cnt, mpc->xmit_tail, mpc->xmit_head);
#endif
		if (!(oldreg & TX_SUSP))
			tx_cpu_reg &= ~TX_SUSP;
		mpc->mpc_output += (c + nx);
		tx_cie |= (ENA_TX_EMPTY_Q0| ENA_TX_LOW_Q0);
	}
	if ((mpc->xmit_cnt < block_count) && (func_type == EQNX_TXINT)){
#ifdef DEBUG
	printk("sst_write1: calling write_wakeup for device = %d\n", 
		SSTMINOR(tty->device));
#endif
		if ((tty->flags & (1 << TTY_DO_WRITE_WAKEUP)) && tty->ldisc.write_wakeup)
			(tty->ldisc.write_wakeup)(tty);
		wake_up_interruptible(&tty->write_wait);
	}
	if(win16)
		mega_pop_win(25);
}
void *ramp_map_fn( void *mpc)
{
   mega_push_win((struct mpchan *)mpc,0);
   return(mpc);
}

void ramp_unmap_fn(void *mpc)
{
   mega_pop_win(57); 
}
static void eqn_print_trace(void)
{
	int first, last, i;
	if (eqnx_err_message_wrap) {
		first = eqnx_err_message_count;
	} else {
		first = 0;
	}
	last =  eqnx_err_message_count;
	for(i = first; i < last; i++)
		printk("%s\n",eqnx_err_messages[i]);
	eqnx_err_message_count = 0;
	eqnx_err_message_wrap  = 0;
	return;
}

/*
 * RAMP_ID_FUNCTS
 *
 * Provide the addresses of functions to block interrupts, and to
 * unblock interrupts for use by the ramp services code.
 * The driver does not use locking, so the addresses of the
 * locking routines are initialized to NULL.
 *
 */
static int ramp_id_functs(void)
{
   int ret = 0;
   int retv = 0;

   ramp_import_blk.blk_ints_fn = ramp_block_int;
   ramp_import_blk.rstr_ints_fn = ramp_unblock_int;
   retv = mpa_srvc_id_functs(&ramp_import_blk);
   if (retv == 0x00FC) {
      struct mpchan *mpc;
      int i;

      for(i = 0, mpc = meg_chan; i < nmegaport * NCHAN_BRD; i++, mpc++)
         mpc->mpc_mpa_stat = MPA_INIT_ERROR;
      ret = retv;
   } else
      ret = 0;
   return(ret); 
}

/*
 * RAMP_BLOCK_INT
 *
 * blocks interrupt.
 *
 */
static unsigned long int ramp_block_int(void)
{
	static int flags;
	save_flags(flags);
	cli();
	return(flags);
}

/*
 * RAMP_UNBLOCK_INT
 *
 * unblocks interrupt
 *
 */
static void ramp_unblock_int(unsigned long int flags)
{
   restore_flags(flags);
}

void ramp_check_error( struct mpchan *mpc, int retv)
{
#ifdef DEBUG_RAMP
printk("ramp_check_error:chan %d with stat = %d\n", mpc->mpc_chan,
			mpc->mpc_mpa_stat);
#endif
   if (mpc->mpc_mpa_stat & MPA_INITG_UART) {
      ramp_reg_modem_error(mpc, retv);
   }
   if (mpc->mpc_mpa_stat & MPA_INITG_SET_LOOP_BACK) {
      ramp_set_loop_back_error(mpc, retv);
   }
   if (mpc->mpc_mpa_stat & MPA_INITG_CLR_LOOP_BACK) {
      ramp_clr_loop_back_error(mpc, retv);
   }
   if (mpc->mpc_mpa_stat & MPA_INITG_START_BREAK) {
      ramp_start_break_error(mpc, retv);
   }
   if (mpc->mpc_mpa_stat & MPA_INITG_STOP_BREAK) {
      ramp_stop_break_error(mpc, retv);
   }
   if (mpc->mpc_mpa_stat & MPA_INITG_HARD_RESET) {
      ramp_hard_reset_error(mpc, retv);
   }
   if ((mpc->mpc_mpa_stat & MPA_INITG_INIT_MODEM) ||
      (mpc->mpc_mpa_stat & MPA_INITG_MODIFY_SETTINGS)) {
      ramp_init_modem_error(mpc, retv);
   }
}

/*
 * RAMP_REG_MODEM_ERROR
 *
 * Sets mpa_stat flag to indicate error during register modem,
 * prints error message on console, and wakes up any sleepers.
 */
void ramp_reg_modem_error( struct mpchan *mpc, int retv)
{
   mpc->mpc_mpa_stat &= ~MPA_INITG_UART; 
   mpc->mpc_mpa_stat |= MPA_INIT_ERROR;  
   if (mpc->mpc_mpa_cout_ctrl_sigs) {
      SET_CTRL_SIGS(mpc, mpc->mpc_mpa_cout_ctrl_sigs);
      mpc->mpc_mpa_cout_ctrl_sigs = 0;
   }
MESSAGE("ERROR: reg_modem", retv, mpc);
      mpc->mpc_mpa_stat |= MPA_SET_LOOP_BACK_ERROR;
      mpc->mpc_mpa_stat |= MPA_CLR_LOOP_BACK_ERROR;
      mpc->mpc_mpa_stat |= MPA_HARD_RESET_ERROR;
      mpc->mpc_mpa_stat |= MPA_CALL_BACK_ERROR;
   wake_up_interruptible(&mpc->mpc_mpa_slb);
   wake_up_interruptible(&mpc->mpc_mpa_clb);
   wake_up_interruptible(&mpc->mpc_mpa_rst);
   wake_up_interruptible(&mpc->mpc_mpa_call_back);
}

void ramp_set_loop_back_error( struct mpchan *mpc, int retv)
{
   mpc->mpc_mpa_stat &= ~MPA_INITG_SET_LOOP_BACK;
   mpc->mpc_mpa_stat |= MPA_SET_LOOP_BACK_ERROR;
      mpc->mpc_mpa_stat |= MPA_CLR_LOOP_BACK_ERROR;
      mpc->mpc_mpa_stat |= MPA_HARD_RESET_ERROR;
      mpc->mpc_mpa_stat |= MPA_CALL_BACK_ERROR;
   wake_up_interruptible(&mpc->mpc_mpa_slb);
   wake_up_interruptible(&mpc->mpc_mpa_clb);
   wake_up_interruptible(&mpc->mpc_mpa_rst);
   wake_up_interruptible(&mpc->mpc_mpa_call_back);
   MESSAGE("ERROR: set_loop_back", retv, mpc);
}

void ramp_clr_loop_back_error( struct mpchan *mpc, int retv)
{
   mpc->mpc_mpa_stat &= ~MPA_INITG_CLR_LOOP_BACK;
   mpc->mpc_mpa_stat &= ~MPA_SET_LOOP_BACK_DONE;
   mpc->mpc_mpa_stat |= MPA_CLR_LOOP_BACK_ERROR;
      mpc->mpc_mpa_stat |= MPA_SET_LOOP_BACK_ERROR;
      mpc->mpc_mpa_stat |= MPA_HARD_RESET_ERROR;
	mpc->mpc_mpa_stat |= MPA_CALL_BACK_ERROR;
   wake_up_interruptible(&mpc->mpc_mpa_clb);
   wake_up_interruptible(&mpc->mpc_mpa_slb);
   wake_up_interruptible(&mpc->mpc_mpa_rst);
   wake_up_interruptible(&mpc->mpc_mpa_call_back);
   MESSAGE("ERROR: clr_loop_back", retv, mpc);
}

int ramp_clr_loop_back( struct mpchan *mpc)
{
   struct mpdev *mpd;
   struct marb_struct *curmarb;
   struct icp_struct *icp;
   int slot_chan, port, lmx, retv, ret = 0;
   unsigned int state = mpc->mpc_mpa_stat;

   /* If doing non-waited call, try again later. */
   if (MPA_CALL_BACK(mpc->mpc_mpa_stat & ~MPA_INITG_CLR_LOOP_BACK)) {
      if (mpc->mpc_mpa_delay[CLR_LOOP_BACK_INDEX] == 0)
         mpc->mpc_mpa_delay[CLR_LOOP_BACK_INDEX] = ramp_get_delay(mpc);
      return(ret);
   }

   mpc->mpc_mpa_delay[CLR_LOOP_BACK_INDEX] = 0;

   if (!(state & MPA_UART_CFG_DONE)) {
      return(0xFF);
   }

   if (!(state & MPA_SET_LOOP_BACK_DONE)) {
      return(0xFF);
   }

   PARAM_SETUP(mpc, mpd, slot_chan, port, lmx, icp, curmarb);

   mpc->mpc_mpa_stat |= MPA_INITG_CLR_LOOP_BACK;
   curmarb->sspcb = icp->sspcb; 
   curmarb->slot_chan = slot_chan;
   curmarb->scratch_area = NULL;
   curmarb->req_type = 0xFF;    /* NON-WAITED */

   retv = mpa_srvc_clr_loop_back( curmarb );

   if (retv == 0) {
      MESSAGE("SUCCESS: clr_loop_back", retv, mpc);
      mpc->mpc_mpa_stat &= ~MPA_INITG_CLR_LOOP_BACK;
      mpc->mpc_mpa_stat &= ~MPA_SET_LOOP_BACK_DONE;
      mpc->mpc_mpa_stat &= ~MPA_CLR_LOOP_BACK_ERROR;
      wake_up_interruptible(&mpc->mpc_mpa_clb);
   } else if(retv == 0xFF00)  {
      /* waited call schedule more work */
   } else    {
      ramp_clr_loop_back_error(mpc, retv);
      ret = retv;
   }
   return(ret);
}

void ramp_start_break_error( struct mpchan *mpc, int retv)
{
   mpc->mpc_mpa_stat &= ~MPA_INITG_START_BREAK;
   if (mpc->mpc_mpa_cout_ctrl_sigs) {
      SET_CTRL_SIGS(mpc, mpc->mpc_mpa_cout_ctrl_sigs);
      mpc->mpc_mpa_cout_ctrl_sigs = 0;
   }
   MESSAGE("ERROR: start_break", retv, mpc);
}

/*
 * RAMP_STOP_BREAK_ERROR
 *
 * Resets mpa_stat flag and displays error message.
 */
void ramp_stop_break_error( struct mpchan *mpc, int retv)
{
   mpc->mpc_mpa_stat &= ~MPA_INITG_STOP_BREAK;
   if (mpc->mpc_mpa_cout_ctrl_sigs) {
      SET_CTRL_SIGS(mpc, mpc->mpc_mpa_cout_ctrl_sigs);
      mpc->mpc_mpa_cout_ctrl_sigs = 0;
   }
   MESSAGE("ERROR: stop_break", retv, mpc);
}

void ramp_hard_reset_error( struct mpchan *mpc, int retv)
{
   mpc->mpc_mpa_stat &= ~MPA_INITG_HARD_RESET;
   mpc->mpc_mpa_stat |= MPA_HARD_RESET_ERROR;  
   if (mpc->mpc_mpa_cout_ctrl_sigs) {
      SET_CTRL_SIGS(mpc, mpc->mpc_mpa_cout_ctrl_sigs);
      mpc->mpc_mpa_cout_ctrl_sigs = 0;
   }
   MESSAGE("ERROR: hard_reset", retv, mpc);
      mpc->mpc_mpa_stat |= MPA_SET_LOOP_BACK_ERROR;
      mpc->mpc_mpa_stat |= MPA_CLR_LOOP_BACK_ERROR;
	mpc->mpc_mpa_stat |= MPA_CALL_BACK_ERROR;
   wake_up_interruptible(&mpc->mpc_mpa_slb);
   wake_up_interruptible(&mpc->mpc_mpa_clb);
   wake_up_interruptible(&mpc->mpc_mpa_rst);
   wake_up_interruptible(&mpc->mpc_mpa_call_back);
}

void ramp_init_modem_error( struct mpchan *mpc, int retv)
{
   PRINT("ERROR: init_modem", retv, mpc);
   mpc->mpc_mpa_stat |= MPA_INIT_MODEM_ERROR;
   mpc->mpc_mpa_stat &= ~MPA_INITG_INIT_MODEM;
   mpc->mpc_mpa_stat &= ~MPA_INITG_MODIFY_SETTINGS;
   if (mpc->mpc_mpa_cout_ctrl_sigs) {
      SET_CTRL_SIGS(mpc, mpc->mpc_mpa_cout_ctrl_sigs);
      mpc->mpc_mpa_cout_ctrl_sigs = 0;
   }
      mpc->mpc_mpa_stat |= MPA_SET_LOOP_BACK_ERROR;
      mpc->mpc_mpa_stat |= MPA_CLR_LOOP_BACK_ERROR;
      mpc->mpc_mpa_stat |= MPA_HARD_RESET_ERROR;
	mpc->mpc_mpa_stat |= MPA_CALL_BACK_ERROR;
   wake_up_interruptible(&mpc->mpc_mpa_slb);
   wake_up_interruptible(&mpc->mpc_mpa_clb);
   wake_up_interruptible(&mpc->mpc_mpa_rst);
   wake_up_interruptible(&mpc->mpc_mpa_call_back);
}

/*
 * RAMP_DEREG_MODEM_CLEANUP
 *
 * Called to clean up after a deregister modem.
 *
 * This routine will perform all the necessary wakeups and signals to
 * ensure that no process is left sleeping.
 */
void ramp_dereg_modem_cleanup( struct mpchan *mpc)
{

   register struct tty_struct  *tp;
   int i;


   /* Make sure everyone wakes up */
   if (mpc->mpc_mpa_cout_ctrl_sigs) {
      SET_CTRL_SIGS(mpc, mpc->mpc_mpa_cout_ctrl_sigs);
      mpc->mpc_mpa_cout_ctrl_sigs = 0;
   }
   mpc->mpc_flags &= ~MPC_BUSY;
      mpc->mpc_mpa_stat &= ~MPA_SET_LOOP_BACK_ERROR;
      mpc->mpc_mpa_stat &= ~MPA_CLR_LOOP_BACK_ERROR;
   mpc->mpc_mpa_stat |= MPA_HARD_RESET_ERROR;
   mpc->mpc_mpa_stat |= MPA_CALL_BACK_ERROR;
   wake_up_interruptible(&mpc->mpc_mpa_slb);
   wake_up_interruptible(&mpc->mpc_mpa_rst);
   wake_up_interruptible(&mpc->mpc_mpa_clb);
   wake_up_interruptible(&mpc->mpc_mpa_call_back);

   /* Clean up status flag */
   mpc->mpc_mpa_stat = 0;
   mpc->mpc_mpa_init_retry = 0;
   mpc->mpc_mpa_reset_retry = 0;

   /* Clean up pending commands */
   for(i = 0; i < MAX_RAMP_INDEX; i++)
      mpc->mpc_mpa_delay[i] = 0;

	mpc->carr_state = 0;
	tp = mpc->mpc_tty;
	if (tp == (struct tty_struct *) NULL)
		return;
   /* Send SIGHUP if open */
   if (!((mpc->flags & ASYNC_CALLOUT_ACTIVE) &&
	(mpc->flags & ASYNC_CALLOUT_NOHUP))){
#ifdef DEBUG
		printk("ramp_dereg_cleanup: calling tty_hangup for device %d\n", SSTMINOR(tp->device));
#endif
		tty_hangup(tp);
	}
}

/*
 * RAMP_FSM
 *
 * run from eqn_ramp_admin.
 *
 * If break is off, modem in slot, so it will be registered.
 * If break is on, modem was removed, so it will be deregistered.
 *
 * If neither needs to be done, see if anything is doing call backs.
 * If so, go call back. If not, look at the array of functions which
 * are pending waiting on something else to be done. If one is pending,
 * run it.
 *
 * The pending function array is an array of characters. The value of the
 * array element is the order to run the function, with the lowest value
 * being the first to run. The index of the array represents the function
 * to run. See eqnx.h for the values of the array indices. This represents
 * only a one level memory of functions to run, but no single function
 * should be called again before it has run. If there is an attempt to
 * do so, the function will be called only once, the subsequent calls
 * will be silently "forgotten". 
 */
void ramp_fsm( struct mpchan *mpc)
{
   struct mpdev *mpd;
   struct marb_struct *curmarb;
   struct icp_struct *icp;
   int slot_chan, port, lmx;
   icpiaddr_t icpi;
   icpbaddr_t icpb;
   int win16, index;

   PARAM_SETUP(mpc, mpd, slot_chan, port, lmx, icp, curmarb);
   win16 = mpc->mpc_mpd->mpd_nwin;
   if(win16)
      mega_push_win(mpc, 0);

   /* check break bit */
   /* If not receiving a break, the slot is occupied by a UART */
   icpi = mpc->mpc_icpi;
   icpb = (rx_locks & LOCK_A) ? &icpi->ssp.cin_bank_b : &icpi->ssp.cin_bank_a;
      
   if (mpc->mpc_mpa_stat == 0) {
      /* Initial state */
      if (!(icpb->ssp.bank_sigs & 0x01 )) {
#ifdef DEBUG_RAMP
printk("calling ramp_reg_modem for %d\n", mpc->mpc_chan);
#endif
         ramp_reg_modem(mpc);
      }
   } else if ((mpc->mpc_mpa_stat & MPA_UART_CFG_DONE)
      && (icpb->ssp.bank_sigs & 0x01 )
      /* Resetting some modems causes break bit to come on. Ignore it. */
      && (!(mpc->mpc_mpa_stat & MPA_INITG_HARD_RESET))) {
      /* modem configured and break detected
        (ie. modem unplugged) deregister it */
#ifdef DEBUG_RAMP
printk("calling dereg_modem for %d \n", mpc->mpc_chan);
#endif
      ramp_dereg_modem(mpc);
   } else {
      /* If nothing pending, see if anything waiting to be done. */
      if (!(MPA_CALL_BACK(mpc->mpc_mpa_stat))) {
         if ((index = ramp_get_index(mpc)) && (index < MAX_RAMP_INDEX)) {
            mpc->mpc_mpa_delay[index] = 0;
            ramp_set_initg(mpc, index);
            ramp_set_index(mpc);
         }
      } else {
         /*printf("%s-%d: mpc_stat(%x) \n",__FILE__,__LINE__,mpc->mpc_mpa_stat)*/;
      }

      if (mpc->mpc_mpa_stat & MPA_INITG_UART) {
         ramp_reg_modem(mpc);
      } else if (mpc->mpc_mpa_stat & MPA_INITG_HARD_RESET) {
         ramp_hard_reset(mpc);
      } else if (mpc->mpc_mpa_stat & MPA_INITG_SET_LOOP_BACK) {
         ramp_set_loop_back(mpc);
      } else if (mpc->mpc_mpa_stat & MPA_INITG_CLR_LOOP_BACK) {
         ramp_clr_loop_back(mpc);
      } else if (mpc->mpc_mpa_stat & MPA_INITG_START_BREAK) {
         ramp_start_break(mpc);
      } else if (mpc->mpc_mpa_stat & MPA_INITG_STOP_BREAK) {
         ramp_stop_break(mpc);
      } else if (mpc->mpc_mpa_stat & MPA_INITG_INIT_MODEM) {
         ramp_init_modem(mpc);
      } else if (mpc->mpc_mpa_stat & MPA_INITG_MODIFY_SETTINGS) {
         ramp_init_modem(mpc);
      }
   }
   if(win16)
      mega_pop_win(50);
   if ((index = ramp_get_index(mpc)))
      ;
}

/******************************************************************************

        EQN_RAMP_ADMIN

        Scheduled RAMP administration routine.
        
        If any channels are referenced, eqn_push_win()/eqn_pop_win() 
	must be called.

	This is scheduled initially from mega_rng_delta upon successful
	completion of init_mpa. It is first run 2.1 seconds after the
	init mpa completes. This delay gives modems which emulate a UART
	time to initialize. After this, it is scheduled every 200 ms.
	when no call back is active. It is scheduled every 10 ms. when
	a call back is active.
******************************************************************************/
        
void eqn_ramp_admin(unsigned long arg)
{
struct mpdev *mpd;
struct mpchan *mpc;
struct icp_struct *icp;
struct lmx_struct *lmxp;
int slot_chan, port, nicp, lmx;
int i;
#ifdef DEBUG
	icpgaddr_t icpg;
#endif

  to_eqn_ramp_admin = -1;

  ramp_admin_poll = MPRAMPTIME0;

  /* check each icp on each mpd */
   for (i=0; i < (int)nmegaport; i++) {	/* adapters loop */
      mpd = &meg_dev[i];
      if (mpd->mpd_alive != DEV_GOOD) 
         continue; /* If board is dead skip it */

    /* if not 64 or 128 port board skip it  */
    if((mpd->mpd_id & 0xF8) != 0x08 && (mpd->mpd_id & 0xF8) != 0x10)
    {
       continue;
    }
    for ( nicp = 0; nicp < mpd->mpd_nicps; nicp++ )
    {
#ifdef DEBUG
icpg = (icpgaddr_t)((unsigned long)mpd->icp[nicp].icp_regs_start + 0x2000);
	printk("ramp_admin_poll = %d, frame counter = %d\n", ramp_admin_poll,
Gicp_fram_ctr);
#endif
      icp = mpd->mpd_mpc->mpc_icp + nicp;
      for(lmx = 0; lmx < 4; lmx++)
      {
         lmxp = &(icp->lmx[lmx]);
         switch(lmxp->lmx_id)
         {
             case 8:
             case 9:
             case 0xB:	/* fall thru */
                  break;
             default:  /* not a RAMP module */
                  continue;
         } 

         for ( port = 0; port < lmxp->lmx_chan; port++ )
         {
             
            slot_chan = port + (lmx * 16);
            /* process this channel */
            mpc = mpd->mpd_mpc + (nicp * MAXICPCHNL) + (lmx * MAXLMXMPC) + port;

            if(lmxp->lmx_active != DEV_GOOD)
            {
                /* ldv went away: clear status word */
                mpc->mpc_mpa_stat = 0;
                continue;
            }

            if(mpc->mpc_mpa_stat & MPA_INIT_ERROR) {
               if (mpc->mpc_mpa_init_retry < MPA_MAX_RETRY) {
                  unsigned int win16;
                  mpc->mpc_mpa_init_retry++;

                  win16 = mpc->mpc_mpd->mpd_nwin;
                  if(win16)
                     mega_push_win(mpc, 0);
                  ramp_dereg_modem(mpc);
                  if(win16)
                     mega_pop_win(59);
                  mpc->mpc_mpa_stat = 0;
               } else if (mpc->mpc_mpa_reset_retry < MPA_MAX_RETRY) {
                  unsigned int win16;
                  mpc->mpc_mpa_reset_retry++;

                  win16 = mpc->mpc_mpd->mpd_nwin;
                  if(win16)
                     mega_push_win(mpc, 0);
                  ramp_hard_reset(mpc);
                  if(win16)
                     mega_pop_win(60);
               } else {
                  continue;
               }
            } else if ((mpc->mpc_mpa_stat & MPA_LOOP_ERROR) ||
                       (mpc->mpc_mpa_stat & MPA_BREAK_ERROR) ||
                       (mpc->mpc_mpa_stat & MPA_INIT_MODEM_ERROR) ||
                       (mpc->mpc_mpa_stat & MPA_HARD_RESET_ERROR)) {
                  continue;
            }
            /* Handle MPA states */
            ramp_fsm(mpc);

         }   /* for all RAMP ports */
      }  /* for each lmx */
    } /* for each ICP */
   

  }  /* adapter's loop */

/*-------------------------------------------------------*/

   mpa_srvc_diag_poll();

  /* reschedule */
	eqnx_ramp_timer.expires = jiffies + ramp_admin_poll;
	add_timer(&eqnx_ramp_timer);
	to_eqn_ramp_admin = 1;
}  /* eqn_ramp_admin */

/*
 * RAMP_DEREG_MODEM
 *
 * deregisters modem unconditionally. Following completion of the
 * deregister, the cleanup routine will ensure that any sleepers
 * are revived.
 */
void ramp_dereg_modem( struct mpchan *mpc)
{
   struct mpdev *mpd;
   struct marb_struct *curmarb;
   struct icp_struct *icp;
   int chars;
   int slot_chan, port, lmx, retv;
   icpiaddr_t icpi;
   icpbaddr_t icpb;

   PARAM_SETUP(mpc, mpd, slot_chan, port, lmx, icp, curmarb);
   curmarb->sspcb = icp->sspcb; 
   curmarb->req_type = 0x00;    /* WAITED: unconditionly */
   curmarb->slot_chan = slot_chan;
   curmarb->scratch_area = NULL;
   /* flush the input buffer first: this is necessary */
   icpi = mpc->mpc_icpi;
   icpb = (rx_locks & LOCK_A) ? &icpi->ssp.cin_bank_b : &icpi->ssp.cin_bank_a;
   chars = icpb->ssp.bank_num_chars;
   if(chars) {
      eqnrflush(mpc);
      return;
   }
   retv = mpa_srvc_dereg_modem( curmarb );


   if(retv == 0) {
      MESSAGE("SUCCESS: dereg_modem", retv, mpc);
      if (!POWER_STATE(mpc)) {
         mpc->mpc_mpa_stat = 0;
         mpc->mpc_mpa_init_retry = 0;
         mpc->mpc_mpa_reset_retry = 0;
      }
   } else {
      MESSAGE("ERROR: dereg_modem", retv, mpc);
      ramp_check_error(mpc, retv);
   }
   ramp_dereg_modem_cleanup(mpc);
}

static void eqnrflush( struct mpchan *mpc)
{
	icpbaddr_t icpb;
	icpiaddr_t icpi = mpc->mpc_icpi;

	if ((mpc->mpc_icpi->ssp.cin_locks & 0x01) == 1)
		icpb = &icpi->ssp.cin_bank_b;
	else
		icpb = &icpi->ssp.cin_bank_a;
	mpc->mpc_rxq.q_ptr = rx_next + (rx_fifo_cnt & 0xf);
	if(mpc->mpc_mpd->mpd_nwin)
		mpc->mpc_rxq.q_ptr &= 0x3fff;
	if (mpc->mpc_rxq.q_ptr > mpc->mpc_rxq.q_end)
		mpc->mpc_rxq.q_ptr = mpc->mpc_rxq.q_begin;
	if(mpc->mpc_mpd->mpd_nwin) {
		rx_tail(mpc->mpc_rxbase + mpc->mpc_rxq.q_ptr);
	}
	else {
		rx_tail(mpc->mpc_rxq.q_ptr);
	}
	mpc->mpc_flags |= MPC_RXFLUSH;
}   /* eqnrflush */

/*
 * These next few routines implement a one level queue of
 * commands to execute. Each command to execute is "remembered"
 * by placing a value in an array. The array index is the command
 * to "remember". The value is the order to execute the commands. 
 * The lowest value is executed first. This is how we keep from
 * calling non-waited calls when a previous non-waited call has
 * not yet finished. 
 */

/* When ramp_fsm finds no more commands to 
 * process, it will call the ramp_set_initg() routine to set the
 * state flag to the next command to process. 
 */
void ramp_set_initg( struct mpchan *mpc, int index)
{
#ifdef DEBUG_RAMP
printk("ramp_set_initg:chan %d with index= %d\n", mpc->mpc_chan,index);
#endif
   switch(index) {
      case REG_MODEM_INDEX:
         mpc->mpc_mpa_stat |= MPA_INITG_UART;
         break;
      case SET_LOOP_BACK_INDEX:
         mpc->mpc_mpa_stat |= MPA_INITG_SET_LOOP_BACK;
         break;
      case CLR_LOOP_BACK_INDEX:
         mpc->mpc_mpa_stat |= MPA_INITG_CLR_LOOP_BACK;
         break;
      case START_BREAK_INDEX:
         mpc->mpc_mpa_stat |= MPA_INITG_START_BREAK;
         break;
      case STOP_BREAK_INDEX:
         mpc->mpc_mpa_stat |= MPA_INITG_STOP_BREAK;
         break;
      case INIT_MODEM_INDEX:
         mpc->mpc_mpa_stat |= MPA_INITG_INIT_MODEM;
         break;
      case MODIFY_SETTINGS_INDEX:
         mpc->mpc_mpa_stat |= MPA_INITG_MODIFY_SETTINGS;
         break;
      case HARD_RESET_INDEX:
         mpc->mpc_mpa_stat |= MPA_INITG_HARD_RESET;
         break;
      default:
         mpc->mpc_mpa_stat |= MPA_INIT_ERROR;
         break;
   }
}

/*
 * ramp_get_index() is called to find the index of the command with
 * the lowest value. This is the command to execute next.
 */
int ramp_get_index( struct mpchan *mpc)
{
   int i;
   int min_index = 0;

   for (i = 0; i < MAX_RAMP_INDEX; i++) {
      if (mpc->mpc_mpa_delay[i]) {
         if (mpc->mpc_mpa_delay[min_index] == 0)
            min_index = i;
         else
            min_index=(mpc->mpc_mpa_delay[i] < mpc->mpc_mpa_delay[min_index]) ? 
               i : min_index;
      }
   }
   return(min_index);
}

/*
 * ramp_set_index() reduces the values of the members of the
 * array of commands to be remembered, so we don't have to worry 
 * about wrap around. These values should always be in the 
 * range (1 to MAX_RAMP_INDEX).
 */
void ramp_set_index( struct mpchan *mpc)
{
   int i;

   for(i = 0; i < MAX_RAMP_INDEX; i++) {
      if (mpc->mpc_mpa_delay[i]) {
         mpc->mpc_mpa_delay[i]--;
      }
   }
}

/*
 * ramp_get_delay () finds the largest current value in the commands
 * array. One more than this is returned as the value to put in the
 * array for the next command to remember.
 */
int ramp_get_delay( struct mpchan *mpc)
{
   int i;
   int max_index = 0;

   for (i = 0; i < MAX_RAMP_INDEX; i++) {
      max_index = (mpc->mpc_mpa_delay[i] > max_index) ? mpc->mpc_mpa_delay[i] : max_index;
   }
   return(max_index + 1);
}

void ramp_init_modem( struct mpchan *mpc)
{

   struct mpdev *mpd;
   int rslt = 0;  /* default */
   struct icp_struct *icp;
   unsigned speed;
   ushort_t d;
   struct marb_struct oldmarb;
   struct marb_struct *curmarb;
   volatile register struct termios *tiosp;
   icpiaddr_t icpi;
   icpoaddr_t icpo;
   int slot_chan, ii, lmx, port, retv;
   uchar_t cur;
   int updated = 0;

   if (!(mpc->mpc_mpa_stat & MPA_UART_CFG_DONE)) {
      return;
   }

   PARAM_SETUP(mpc, mpd, slot_chan, port, lmx, icp, curmarb);
   memset(&oldmarb, 0, sizeof( struct marb_struct));

   /* First time (not on call backs) set up signals, etc. */
   if (!(mpc->mpc_mpa_stat & (MPA_INITG_INIT_MODEM | 
	MPA_INITG_MODIFY_SETTINGS))) {
ramp_termios:
        updated = 1;
     if (mpc->mpc_tty != (struct tty_struct *) NULL){ /* shashi: 03/22/98 */
      /* setup pointers for general use */
        tiosp = mpc->mpc_tty->termios;
  	icpi = mpc->mpc_icpi;
  	icpo = mpc->mpc_icpo;
  	/* CLOCAL and carrier detect parameters */
  	if (tiosp->c_cflag & CLOCAL){
    		mpc->mpc_icpi->ssp.cin_attn_ena &= ~ENA_DCD_CNG;
		mpc->carr_state = 1;
  	}
  	else{  /* CLOCAL clear */
    		/* set DCD processing */
		/*printk("enabling DCD_CNG for device %d\n",chan);*/
    		mpc->mpc_icpi->ssp.cin_attn_ena |= ENA_DCD_CNG;
	}

  	/* outbound control signals */
  	if ((tiosp->c_cflag & CBAUD) == 0)  /* B0 */
  	{
		/*if (!(tiosp->c_cflag & CLOCAL)){*/
    			(void) megamodem(SSTMINOR(mpc->mpc_tty->device), 
					TURNOFF);
		/*}*/
    		return;
  	}
	else{
		mpc->carr_state = megamodem(
			SSTMINOR(mpc->mpc_tty->device), TURNON); /*shashi 02/04/98*/
	}

	/* set output control signal flow control */
	if(mpc->mpc_param & IOCTCTS) 
		tiosp->c_cflag |= CRTSCTS;

	/* set input control signal flow control */
	if(mpc->mpc_param & IOCTRTS) 
		tiosp->c_cflag |= CRTSCTS;
#if 0 /* Hardware flow done below for modem pool */
	if(tiosp->c_cflag & CRTSCTS)  /* only need to change the mask */
		icpi->ssp.cin_susp_output_lmx |= CTS_OFF;
	else
		icpi->ssp.cin_susp_output_lmx &= ~CTS_OFF;


	if (tiosp->c_cflag & CRTSCTS)
		tx_ctrl_sig &= ~TX_HFC_RTS;
	else
        	tx_ctrl_sig |= TX_HFC_RTS;
  
#endif

      /* set output inband flow control */
      if(tiosp->c_iflag & IXON || mpc->mpc_param & IXONSET) {
         icpi->ssp.cin_xoff_1 = mpc->mpc_stop;
         icpi->ssp.cin_xon_1 = mpc->mpc_start;
         icpi->ssp.cin_char_ctrl |= EN_XON|EN_XOFF;
         icpi->ssp.cin_locks &= ~DIS_IBAND_FLW;  /* clear lock bit - enable inband */
         cur_chnl_sync( mpc );
         icpi->ssp.cin_inband_flow_ctrl = 0; /* calibrate */
         if(mpc->mpc_param & IOCTXON)
            icpi->ssp.cin_char_ctrl &= ~EN_DNS_FLW;  /* do not discard xon/xoff */
         else
            icpi->ssp.cin_char_ctrl |= EN_DNS_FLW;  /* discard xon/xoff */
         icpi->ssp.cin_char_ctrl &= ~EN_DBL_FLW; /* disable double flow */
         if (( tiosp->c_iflag & IXANY ) && !(mpc->mpc_param & IXANYIG)) 
            icpi->ssp.cin_char_ctrl |= EN_IXANY;
         else
            icpi->ssp.cin_char_ctrl &= ~EN_IXANY;
      } else  /* cancel action */ {
         icpi->ssp.cin_char_ctrl &= ~EN_DNS_FLW;  /* discard xon/xoff */
         icpi->ssp.cin_locks |= DIS_IBAND_FLW;     /* set lock bit - disable inband */
         cur_chnl_sync( mpc );
         icpi->ssp.cin_inband_flow_ctrl = 0;  /* calibrate */
      }
      icpo->ssp.cout_xoff_1 = mpc->mpc_stop;
      icpo->ssp.cout_xon_1 = mpc->mpc_start;
      /* set input inband flow control */
      if ( tiosp->c_iflag & IXOFF ) {
         icpo->ssp.cout_flow_cfg &= ~TX_XON_DBL;
         icpo->ssp.cout_flow_cfg |= TX_XON_XOFF_EN;
      } else  /* cancel enable, send XON if we previously sent an XOFF */ {
         if ((icpo->ssp.cout_flow_cfg & TX_XON_XOFF_EN) && (icpi->ssp.cin_int_flags & 0x40)) {
            megajam(mpc, mpc->mpc_start);
         }
         icpo->ssp.cout_flow_cfg &= ~(TX_XON_XOFF_EN|TX_XON_DBL);
      }

      /* DEBUG: this fixes data errors */
      cur = mpc->mpc_cout->ssp.cout_lmx_cmd & 0x03; /* preserve RAMP bits */
      cur |= 0x68;
      cur &= ~0x40;
      mpc->mpc_cout->ssp.cout_lmx_cmd = cur;
   
      /* get a copy of the old marb values */
      memcpy(&oldmarb, curmarb, sizeof( struct marb_struct));

      curmarb->sspcb = mpc->mpc_icp->sspcb; 
      curmarb->slot_chan = slot_chan;
      curmarb->scratch_area = NULL;
      curmarb->req_type = 0xFF;    /* NON-WAITED */
      curmarb->sup_uchar0 = 0x00;

/* Tell the MPA to handle HW flow control */
      if(tiosp->c_cflag & CRTSCTS)  /*  Bit 3     IN      CTS	*/
      	curmarb->sup_uchar0 |= 0x08;
      if (tiosp->c_cflag & CRTSCTS)  /*  Bit 1     OUT     RTS	*/
      	curmarb->sup_uchar0 |= 0x02;

      /* baudrate */
      speed = tiosp->c_cflag & CBAUD;
      if (speed & CBAUDEX){
	speed &= ~CBAUDEX;
	if ((speed < 1) || (speed > 4))
		tiosp->c_cflag &= ~CBAUDEX;
	else 
		speed += 15;
      }
      if ( speed > 0 && speed <= (B38400 + 4) ) {

#if 0 /* This does nothing except possibly glitch characters. */
         while( mpc->mpc_cout->ssp.cout_int_baud_ctr != (ushort_t) 0
            && (++ctr < 10000) )
            mpc->mpc_cout->ssp.cout_int_baud_ctr = (ushort_t) 0;
#endif

         /* RAMP */
         curmarb->sup_ushort = uart_baud(speed, mpc); 
   
      } else
         rslt = 1;  /* no valid speed to set */

      /* databits */
      switch (tiosp->c_cflag & CSIZE){  /* set character size */
         case CS5:
            d = 0x00;
            break;

         case CS6:
            d = 0x01;
            break;

         case CS7:
            d = 0x02;
            break;

         default:  /* case CS8 - no other value should occur */
            d = 0x03;
      }
      /* "d" has the correct value for RAMP also */
      curmarb->sup_uchar1 = d;
   
      if(tiosp->c_cflag & PARENB){
            d |= 0x04;
    	    if( !(tiosp->c_cflag & PARODD) ) /* even parity */ {
               d |= 0x08;
               curmarb->sup_uchar3 = 0x18;    /* RAMP: even parity generation */
            } else		
               curmarb->sup_uchar3 = 0x08;    /* RAMP: odd parity generation */
      } else
         curmarb->sup_uchar3 = 0x00;    /* RAMP: no parity generation */

#if 0
      mpc->mpc_cin->ssp.cin_char_ctrl |= d;
      mpc->mpc_cout->ssp.cout_char_fmt |= d;
#endif
      /* DEBUG LAST CHANGE */

      /* prepare for input break/parity processing - cancel special features */
#if 0
      mpc->mpc_cin->ssp.cin_char_ctrl &= ~0x03e0;
#endif
      mpc->mpc_cin_stat_mask &= ~0x03a0;

      /* prepare for input break processing */
      if ( tiosp->c_iflag & IGNBRK ){
         rslt = 0; /* get rid of empty "if" */
      } else {
         /* hardware must maintain break nulls */
         mpc->mpc_cin_stat_mask |= 0x0120;   /* watch for framing errs w/nulls */
      }

      /* clear lookup table - processing flags already clear */
      for ( ii = 0; ii < 32; ii++ )
         mpc->mpc_cin->ssp.cin_lookup_tbl[ii] = 0;

      /* prepare for input parity processing */
  if ( tiosp->c_cflag & PARENB && tiosp->c_iflag & INPCK ) {
    	if ( tiosp->c_iflag & IGNPAR ) {
            rslt = 0; /* get rid of empty "if" */
    	}
    	else{
            /* hardware must maintain and tag err'd chars */
            mpc->mpc_cin_stat_mask |= 0x0180; /* watch for parity and framing */
      	    if ( tiosp->c_iflag & PARMRK && !(tiosp->c_iflag & ISTRIP) ){
               /* put 0xff in lookup table */
               mpc->mpc_cin->ssp.cin_lookup_tbl[0x1f] |= 0x80;
               mpc->mpc_cin_stat_mask |= 0x0200;     /* watch for lookup event */
            }
         }
      }

      /* output stop bits */
      if (tiosp->c_cflag & CSTOPB){
         /* RAMP: 2 stop bits */
         curmarb->sup_uchar2 = 0x04;
      } else {
         /* RAMP: 1 stop bits */
         curmarb->sup_uchar2 = 0x00;
      }
     } /* port tty valid */
     else{
      	wake_up_interruptible(&mpc->mpc_mpa_call_back);
     	return;
     }
   } /* First time */

   /* RAMP */
   /* If no values have actualy been modified don't call init_modem. */
   /* ksh needlessly issues ioctls when the user hits the enter key. */
   /* This almost guarantees that data will be in transit when we    */
   /* call this service. Which breaks the init_modem call            */
         
   if( (mpc->mpc_mpa_stat & MPA_INIT_MODEM_DONE)
      && curmarb->sup_uchar0 == oldmarb.sup_uchar0
      && curmarb->sup_uchar1 == oldmarb.sup_uchar1
      && curmarb->sup_uchar2 == oldmarb.sup_uchar2
      && curmarb->sup_uchar3 == oldmarb.sup_uchar3
      && curmarb->sup_ushort == oldmarb.sup_ushort ) {
         retv = 0;
      	wake_up_interruptible(&mpc->mpc_mpa_call_back);
	 return;
   } else if ((mpc->mpc_mpa_stat & MPA_INITG_MODIFY_SETTINGS) ||
             (mpc->mpc_mpa_stat & MPA_INIT_MODEM_DONE)) {
      mpc->mpc_mpa_stat &= ~MPA_INIT_MODEM_DONE;
      mpc->mpc_mpa_stat |= MPA_INITG_MODIFY_SETTINGS;
      retv = mpa_srvc_modify_settings( curmarb );
   } else {
      mpc->mpc_mpa_stat |= MPA_INITG_INIT_MODEM;
      retv = mpa_srvc_init_modem( curmarb );
   }
   if(retv == 0xFF00) {
      ;
   } else if (retv) {
      ramp_check_error(mpc, retv);
   } else {
      MESSAGE("SUCCESS: init_modem", retv, mpc);
      ramp_modem_cleanup(mpc);
      mpc->mpc_mpa_stat |= MPA_INIT_MODEM_DONE;
      mpc->mpc_mpa_stat &= ~MPA_INITG_INIT_MODEM;
      mpc->mpc_mpa_stat &= ~MPA_INITG_MODIFY_SETTINGS;
      if (mpc->mpc_mpa_cout_ctrl_sigs) {
         SET_CTRL_SIGS(mpc, mpc->mpc_mpa_cout_ctrl_sigs);
         mpc->mpc_mpa_cout_ctrl_sigs = 0;
      }
	mpc->mpc_mpa_stat &= ~MPA_CALL_BACK_ERROR;
      if (!updated)
	goto ramp_termios;
      else
      	wake_up_interruptible(&mpc->mpc_mpa_call_back);
   }

} /* ramp_init_modem */

void ramp_modem_cleanup( struct mpchan *mpc)
{
#ifndef LINUX
   register tty_struct *tp;
#endif
   icpiaddr_t icpi;
   icpoaddr_t icpo;

   icpi = mpc->mpc_icpi;
   icpo = mpc->mpc_icpo;

#ifndef LINUX
   setrwmode(mpc);
   tp = mpc->mpc_tty;
   /* output session control */
  if (mpc->mpc_wmode && (tp->t_oflag & OPOST))
  {
		icpo->ssp.cout_ses_ctrl_a = MOPOST;  /* enable opost */
      if ( tp->t_oflag & ONLCR )
         mpc->mpc_cout->ssp.cout_ses_ctrl_a |= MONLCR;
      else
         mpc->mpc_cout->ssp.cout_ses_ctrl_a &= ~MONLCR;

      if ( tp->t_oflag & OCRNL )
         icpo->ssp.cout_ses_ctrl_a |= MOCRNL;
      else
         icpo->ssp.cout_ses_ctrl_a &= ~MOCRNL;

      if ( (tp->t_oflag & TAB3) == TAB3 ) {
         icpo->ssp.cout_ses_ctrl_a |= MTABX;
         icpo->ssp.cout_ses_col_a = 0;
      } else
         icpo->ssp.cout_ses_ctrl_a &= ~MTABX;

      if ( tp->t_oflag & ONOCR )
         icpo->ssp.cout_ses_ctrl_a |= MONOCR;
      else
         icpo->ssp.cout_ses_ctrl_a &= ~MONOCR;

      if ( tp->t_oflag & ONLRET )
         icpo->ssp.cout_ses_ctrl_a |= MONLRET;
      else
         icpo->ssp.cout_ses_ctrl_a &= ~MONLRET;
   } else
      mpc->mpc_cout->ssp.cout_ses_ctrl_a = 0x00;  /* default */

   if ( tp->t_cflag & CREAD )
      /* make sure dma's to dram are enabled */
      mpc->mpc_cin->ssp.cin_locks &= ~0x80;
   else
      /* disable unnecessary dma's to dram */
      mpc->mpc_cin->ssp.cin_locks |= 0x80;
#endif

   /* set cin_tmr_preset_count and enable */
   mpc->mpc_cin->ssp.cin_locks |= 0x10;
   mpc->mpc_cin->ssp.cin_tmr_preset_count = 1;
   mpc->mpc_cin->ssp.cin_char_tmr_remain = 0;
   mpc->mpc_cin->ssp.cin_tmr_size = 0;
   mpc->mpc_cin->ssp.cin_locks &= ~0x10;

}

void ramp_hard_reset( struct mpchan *mpc)
{
   struct mpdev *mpd;
   struct marb_struct *curmarb;
   struct icp_struct *icp;
   int chars;
   int slot_chan, port, lmx, retv;
   icpiaddr_t icpi;
   icpbaddr_t icpb;

   PARAM_SETUP(mpc, mpd, slot_chan, port, lmx, icp, curmarb);
#ifdef DEBUG_RAMP
printk("hard_reset: for channel %d\n", mpc->mpc_chan);
#endif

   /* If doing non-waited call, try again later. */
   if (MPA_CALL_BACK(mpc->mpc_mpa_stat & ~MPA_INITG_HARD_RESET)) {
      if (mpc->mpc_mpa_delay[HARD_RESET_INDEX] == 0) {
         mpc->mpc_mpa_delay[HARD_RESET_INDEX] = ramp_get_delay(mpc);
      }
      return;
   }

   icpi = mpc->mpc_icpi;
   icpb=(rx_locks & LOCK_A) ? &icpi->ssp.cin_bank_b : &icpi->ssp.cin_bank_a;
   if (!(mpc->mpc_mpa_stat & MPA_INITG_HARD_RESET)) {

      /* flush the input buffer first: this is necessary */
      chars = icpb->ssp.bank_num_chars;

      if(chars)  {
         mpc->mpc_mpa_stat |= MPA_INITG_HARD_RESET;
         eqnrflush(mpc);
         return;
      }
   }

   mpc->mpc_mpa_stat |= MPA_INITG_HARD_RESET;

   if (mpc->mpc_flags & MPC_BUSY)
      return;

   curmarb->sspcb = icp->sspcb; 
   curmarb->slot_chan = slot_chan;
   curmarb->scratch_area = NULL;
   curmarb->req_type = 0xFF;    /* NON-WAITED */

   retv = mpa_srvc_hard_reset( curmarb );

   if (retv == 0) {
      MESSAGE("SUCCESS: hard_reset", retv, mpc);
      /* Clear false LMX change */
      FREEZ_BANK(mpc);
      FREEZ_BANK(mpc);
      mpc->mpc_cin_events = 0;
      mpc->mpc_mpa_stat &= ~MPA_INITG_HARD_RESET;
      mpc->mpc_mpa_init_retry = 0;
      mpc->mpc_mpa_reset_retry = 0;
      if (mpc->mpc_mpa_cout_ctrl_sigs) {
         SET_CTRL_SIGS(mpc, mpc->mpc_mpa_cout_ctrl_sigs);
         mpc->mpc_mpa_cout_ctrl_sigs = 0;
      }
      wake_up_interruptible(&mpc->mpc_mpa_rst);
   } else if(retv == 0xFF00)  {
      /* waited call schedule more work */
   } else    {
      ramp_hard_reset_error(mpc, retv);
   }
}

int ramp_set_loop_back( struct mpchan *mpc)
{
   struct mpdev *mpd;
   struct marb_struct *curmarb;
   struct icp_struct *icp;
   int slot_chan, port, lmx, retv, ret = 0;
   unsigned int state;

   /* If doing non-waited call, try again later. */
   if (MPA_CALL_BACK(mpc->mpc_mpa_stat & ~MPA_INITG_SET_LOOP_BACK)) {
      if (mpc->mpc_mpa_delay[SET_LOOP_BACK_INDEX] == 0) {
         mpc->mpc_mpa_delay[SET_LOOP_BACK_INDEX] = ramp_get_delay(mpc);
      }
      return(ret);
   }

   mpc->mpc_mpa_delay[SET_LOOP_BACK_INDEX] = 0;

   state = mpc->mpc_mpa_stat;

   if (!(state & MPA_UART_CFG_DONE)) {
      return(0xFF);
   }


   if (state & MPA_SET_LOOP_BACK_DONE) {
      return(0xFF);
   }

   PARAM_SETUP(mpc, mpd, slot_chan, port, lmx, icp, curmarb);

   curmarb->sspcb = icp->sspcb; 
   curmarb->slot_chan = slot_chan;
   curmarb->scratch_area = NULL;
   curmarb->req_type = 0xFF;    /* NON-WAITED */
   mpc->mpc_mpa_stat |= MPA_INITG_SET_LOOP_BACK;

   retv = mpa_srvc_set_loop_back( curmarb );

   if (retv == 0) {
      MESSAGE("SUCCESS: set_loop_back", retv, mpc);
      mpc->mpc_mpa_stat &= ~MPA_INITG_SET_LOOP_BACK;
      mpc->mpc_mpa_stat |= MPA_SET_LOOP_BACK_DONE;
      mpc->mpc_mpa_stat &= ~MPA_SET_LOOP_BACK_ERROR;
      wake_up_interruptible(&mpc->mpc_mpa_slb);
   } else if(retv == 0xFF00)  {
      /* waited call schedule more work */
      ;
   } else    {
      ramp_set_loop_back_error(mpc, retv);
      ret = retv;
   }
   return(ret);
}

int ramp_start_break( struct mpchan *mpc)
{
   struct mpdev *mpd;
   struct marb_struct *curmarb;
   struct icp_struct *icp;
   int slot_chan, port, lmx, retv, ret = 0;
   unsigned int state = mpc->mpc_mpa_stat;

   /* If doing non-waited call, try again later. */
   if (MPA_CALL_BACK(mpc->mpc_mpa_stat & ~MPA_INITG_START_BREAK)) {
      if (mpc->mpc_mpa_delay[START_BREAK_INDEX] == 0)
         mpc->mpc_mpa_delay[START_BREAK_INDEX] = ramp_get_delay(mpc);
      return(ret);
   }

   mpc->mpc_mpa_delay[START_BREAK_INDEX] = 0;

   if (!(state & MPA_UART_CFG_DONE))
      return(0xFF);

   if ((state & MPA_START_BREAK_DONE))
      return(ret);

   PARAM_SETUP(mpc, mpd, slot_chan, port, lmx, icp, curmarb);

   mpc->mpc_mpa_stat |= MPA_INITG_START_BREAK;
   curmarb->sspcb = icp->sspcb; 
   curmarb->slot_chan = slot_chan;
   curmarb->req_type = 0xFF; /* NON-WAITED */

   retv = mpa_srvc_start_break( curmarb );

   if (retv == 0) {
      MESSAGE("SUCCESS: start_break", retv, mpc);
      mpc->mpc_mpa_stat &= ~MPA_INITG_START_BREAK;
      mpc->mpc_mpa_stat |= MPA_START_BREAK_DONE;
      if (mpc->mpc_mpa_cout_ctrl_sigs) {
         SET_CTRL_SIGS(mpc, mpc->mpc_mpa_cout_ctrl_sigs);
         mpc->mpc_mpa_cout_ctrl_sigs = 0;
      }
   } else if(retv == 0xFF00)  {
      /* waited call schedule more work */
      ;
   } else    {
      ramp_start_break_error(mpc, retv);
      ret = retv;
   }
   return(ret);
}

void ramp_stop_break( struct mpchan *mpc)
{
   struct mpdev *mpd;
   struct marb_struct *curmarb;
   struct icp_struct *icp;
   int slot_chan, port, lmx, retv;
   unsigned int state = mpc->mpc_mpa_stat;

   if (!(state & MPA_UART_CFG_DONE))
      return;

   /* If doing non-waited call, try again later. */
   if (MPA_CALL_BACK(mpc->mpc_mpa_stat & ~MPA_INITG_STOP_BREAK)) {
      if (mpc->mpc_mpa_delay[STOP_BREAK_INDEX] == 0)
         mpc->mpc_mpa_delay[STOP_BREAK_INDEX] = ramp_get_delay(mpc);
      return;
   }

   mpc->mpc_mpa_delay[STOP_BREAK_INDEX] = 0;

   if (!(state & MPA_START_BREAK_DONE))
      return;

   PARAM_SETUP(mpc, mpd, slot_chan, port, lmx, icp, curmarb);

   mpc->mpc_mpa_stat |= MPA_INITG_STOP_BREAK;
   curmarb->sspcb = icp->sspcb; 
   curmarb->slot_chan = slot_chan;
   curmarb->req_type = 0xFF; /* NON-WAITED */

   retv = mpa_srvc_stop_break( curmarb );

   if (retv == 0) {
      MESSAGE("SUCCESS: stop_break", retv, mpc);
      mpc->mpc_mpa_stat &= ~MPA_INITG_STOP_BREAK;
      mpc->mpc_mpa_stat &= ~MPA_START_BREAK_DONE;
      if (mpc->mpc_mpa_cout_ctrl_sigs) {
         SET_CTRL_SIGS(mpc, mpc->mpc_mpa_cout_ctrl_sigs);
         mpc->mpc_mpa_cout_ctrl_sigs = 0;
      }
   } else if(retv == 0xFF00) {
      /* waited call schedule more work */
      ;
   } else {
      ramp_stop_break_error(mpc, retv);
   }
   return;
}

/*
 * RAMP_REG_MODEM
 *
 * Registers modem and wakes up sleepers when done.
 *
 */
void ramp_reg_modem( struct mpchan *mpc)
{
   struct mpdev *mpd;
   struct marb_struct *curmarb;
   struct icp_struct *icp;
   int chars;
   int slot_chan, port, lmx, retv = 0;
   icpiaddr_t icpi;
   icpbaddr_t icpb;
#ifdef DEBUG_RAMP
printk("register modem for channel %d\n", mpc->mpc_chan);
#endif
   PARAM_SETUP(mpc, mpd, slot_chan, port, lmx, icp, curmarb);

   if (!(mpc->mpc_mpa_stat & MPA_INITG_UART)) {

      /* flush the input buffer first: this is necessary */
      icpi = mpc->mpc_icpi;
      icpb=(rx_locks & LOCK_A) ? &icpi->ssp.cin_bank_b : &icpi->ssp.cin_bank_a;
      chars = icpb->ssp.bank_num_chars;

      if(chars)  {
         eqnrflush(mpc);
#ifdef DEBUG_RAMP
printk("returning from register modem for channel %d because of chars\n", mpc->mpc_chan);
#endif
         return;
      }
   }

#ifdef DEBUG_RAMP
printk("setting MPA_INITG_UART for channel %d\n", mpc->mpc_chan);
#endif
   mpc->mpc_mpa_stat = MPA_INITG_UART;
   curmarb->sspcb = icp->sspcb; 
   curmarb->slot_chan = slot_chan;
   curmarb->scratch_area = NULL;
   curmarb->req_type = 0xFF;    /* NON-WAITED */
   curmarb->sup_uchar0 = 0x00; /* no MPA to UART/Modem flow control */
   curmarb->sup_uchar1 = 3; /* 8 bits */
   curmarb->sup_uchar2 = curmarb->sup_uchar3 = 0; /* 1 stop, No parity */
   curmarb->sup_ushort = (115200/9600);

   retv = mpa_srvc_reg_modem( curmarb );
  
   if(retv == 0) {
      icpiaddr_t icpi;
      icpi = mpc->mpc_icpi;
#ifdef DEBUG_RAMP
      printk("SUCCESS reg_modem of type %x, ret = %d for channel %d\n", MODEM_TYPE(mpc), retv, mpc->mpc_chan);
#endif
      MESSAGE("SUCCESS reg_modem", retv, mpc);
      mpc->mpc_mpa_init_retry = 0;
      mpc->mpc_mpa_reset_retry = 0;
      mpc->mpc_cout->ssp.cout_cpu_req &= ~0x80;
      mpc->mpc_mpa_stat &= ~MPA_INITG_UART; 
      mpc->mpc_mpa_stat = MPA_UART_CFG_DONE; 
      if (mpc->mpc_mpa_cout_ctrl_sigs) {
         SET_CTRL_SIGS(mpc, mpc->mpc_mpa_cout_ctrl_sigs);
         mpc->mpc_mpa_cout_ctrl_sigs = 0;
      }
      /* Clear out flase LMX change */
      FREEZ_BANK(mpc);
      mpc->mpc_cin_events &= ~EV_LMX_CNG;

	mpc->mpc_mpa_stat &= ~MPA_CALL_BACK_ERROR;
      wake_up_interruptible(&mpc->mpc_mpa_call_back);
   } else if(retv ==0xFF00)  {
      /* waited call schedule more work */
#ifdef DEBUG_RAMP
printk("waited call in reg_modem for channel %d\n", mpc->mpc_chan);
#endif
      ;

   } else {
         /* error */
      ramp_check_error(mpc, retv);
   }
}

/******************************************************************************
        
       RAMPARAM

	This routine configures a ramp channel as eqnparam does for non
	ramp devices. This must be performed using the ramp services.
	
	
	 
******************************************************************************/
static void ramparam( struct mpchan *mpc)
{

   /* If doing non-waited call, try again later. */
   if (MPA_CALL_BACK(mpc->mpc_mpa_stat)) {
      if ((mpc->mpc_mpa_stat & MPA_INIT_MODEM_DONE) ||
         (mpc->mpc_mpa_stat & MPA_INITG_MODIFY_SETTINGS)) {
         if (mpc->mpc_mpa_delay[MODIFY_SETTINGS_INDEX] == 0)
            mpc->mpc_mpa_delay[MODIFY_SETTINGS_INDEX] = ramp_get_delay(mpc);
      } else {
         if (mpc->mpc_mpa_delay[INIT_MODEM_INDEX] == 0)
            mpc->mpc_mpa_delay[INIT_MODEM_INDEX] = ramp_get_delay(mpc);
      }
      return;
   }

   ramp_init_modem(mpc);
}


/*
 * RAMP_LMX_CHECK
 *
 * Checks to see if LMX change on a modem pool slot is
 * flase or true. False LMX changes occur just after registering
 * modem, or when slot is empty and Plug-n-play check occurs
 * during diagnostic poll call.
 *
 * Return values:
 *   0  False LMX change
 *   1  True  LMX change
 */
int ramp_lmx_check(struct mpchan *mpc)
{
	int i;

	for(i = 0; i < 16; i++, mpc++) {
   	icpiaddr_t icpi;
   	icpbaddr_t icpb;

   	icpi = mpc->mpc_icpi;
   	icpb = (rx_locks & LOCK_A) ? 
			&icpi->ssp.cin_bank_b : &icpi->ssp.cin_bank_a;
      
		/* Find occupied slot */
		if (!(icpb->ssp.bank_sigs & 0x01 ) &&
			/* not in the middle of initializing */
			(mpc->mpc_mpa_stat & MPA_UART_CFG_DONE)) {

			/* We've already looked at first channel's events */
			if (!(mpc->mpc_chan % 16) ||
			   /* This is the same check as in megapoll */
			   (icpb->ssp.bank_events & EV_LMX_CNG)) {
				/* true LMX change */
				return(1);
			} else {
				/* false LMX change */
				return(0);
			}
		}
	}
	return(0); /* No modems in modem pool, must be false LMX change */
}
/******************************************************************************
   UART_BAUD Computes the uart divisor value for the RAMP services.

******************************************************************************/

static ushort_t uart_baud( int speed, struct mpchan *mpc)
{
int baud;
int val = icpbaud_tbl[speed];

  if (val == 38400){
  	if((mpc->flags & ASYNC_SPD_MASK) == ASYNC_SPD_HI)
		val = 57600;
  	else if((mpc->flags & ASYNC_SPD_MASK) == ASYNC_SPD_VHI)
		val = 115200;
  }
  switch(val)
  {
     case 0:		baud = 0;
     case 50:		baud = 115200/50 ; break;
     case 75:		baud = 115200/75 ; break;
     case 110:		baud = 115200/110 ; break;
     case 134:		baud = 856 ; break;
     case 150:		baud = 115200/150 ; break;
     case 200:		baud = 115200/200 ; break;
     case 300:		baud = 115200/300 ; break;
     case 600:		baud = 115200/600 ; break;
     case 1200:		baud = 115200/1200 ; break;
     case 1800:		baud = 115200/1800 ; break;
     case 2400:		baud = 115200/2400 ; break;
     case 4800:		baud = 115200/4800 ; break;
     case 9600:		baud = 115200/9600 ; break;
     case 19200:	baud = 115200/19200 ; break;
     case 38400:	baud = 115200/38400 ; break;
     case 57600:	baud = 115200/57600 ; break;
     case 115200:	baud = 115200/115200 ; break;
     case 230400:	baud = 115200/115200 ; break;
     case 460800:	baud = 115200/115200 ; break;
     default:		baud = 0; break; /* can't be here */
 	
  }

  return(baud);
#if (LINUX_VERSION_CODE <= 66304)
#undef UTS_VERSION
#undef LINUX_COMPILE_TIME
#undef LINUX_COMPILE_BY
#undef LINUX_COMPILE_HOST
#undef LINUX_COMPILE_DOMAIN
#undef LINUX_COMPILER
#endif
#undef LINUX_VERSION_CODE
#undef UTS_RELEASE
}  /* uart_baud */

#ifdef	IA64
/*
** simple byte copy.
** used because of problems with memcpy on IA64
*/
void memcpy_IA64(void * dest, void * src, int len)
{
	unsigned char	*destp, *srcp;
	int	i;

	destp = (unsigned char *) dest;
	srcp = (unsigned char *) src;
	for (i=0; i<len; i++)
		*(destp+i) = *(srcp+i);
}
#endif	/* IA64 */
